#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>

#include <cmdline.h>
#include <logf.h>
#include <dboxpath.h>
#include <oldtools.h>
#include <zconnect.h>
#include <account.h>
#include <spoolp.h>

void seek(FILE *infile, dword entry_number)
 {
  buchungt transaction;

  rewind(infile);
  
  while(entry_number!=0)
   {
    transaction.read(infile);
    entry_number--;
   }
 }

void sum(buchungt &transaction1, buchungt transaction2)
 {
  signed long sum_value;

  if(transaction1.amount.scale>transaction2.amount.scale)
    transaction2.amount=rescale(transaction2.amount, transaction1.amount.scale);
  else if(transaction2.amount.scale>transaction1.amount.scale)
    transaction1.amount=rescale(transaction1.amount, transaction2.amount.scale);

  sum_value=(transaction1.amount.amount*(transaction1.haben?1:-1))+
            (transaction2.amount.amount*(transaction2.haben?1:-1));

  transaction1.amount.amount=labs(sum_value);
  transaction1.haben=(sum_value>=0);
 }

void get_balance(FILE *infile, dword entry_number, buchungt &balance)
 {
  buchungt transaction;

  rewind(infile);
  
  balance.amount.amount=0;
  balance.amount.scale=config.currency_scale;
  balance.haben=TRUE;
 
  while(entry_number!=0)
   {
    transaction.read(infile);
    entry_number--;
    sum(balance, transaction);
   }
 }

bool pattern_accumulate_compare(const buchungt &pattern, const buchungt &transaction)
 {
  if(pattern.bearbeiter[0]!=0)
    if(!checkjoker(pattern.bearbeiter, transaction.bearbeiter))
      return TRUE;

  if(pattern.buchungstyp[0]!=0)
    if(!checkjoker(pattern.buchungstyp, transaction.buchungstyp))
      return TRUE;

  if(pattern.mwstsatz!=0)
    if(pattern.mwstsatz!=transaction.mwstsatz)
      return TRUE;

  if(pattern.haben!=transaction.haben)
      return TRUE;

  return FALSE;
 }

class billt
 {
 public:
  dword midcounter;

  struct patternt
   {
    buchungt sum;
    time_t date_min, date_max;
    unsigned number;
   };

  vector<patternt> patterns;

  cmdlinet &cmdline;
  char format;
  charsett charset;
  userpt up;
  string description, counter, pid;
  string mailoutfilename;
  amountt end_balance;

  billt(cmdlinet &_cmdline);

  void read_patterns();
  bool accumulate(const buchungt &transaction);
  void table_header(FILE *outfile);
  void table_footer(FILE *outfile);
  void table_separator(FILE *outfile);
  void bill_user();
  void dobill();
  void show_vat(FILE *outfile, vector<buchungt> &vat);
  void bill(FILE *infile, FILE *outfile,
            dword start_number, dword end_number,
            bool payments, bool care_payments, 
            bool inverse,
            vector<buchungt> &vat,
            buchungt &transaction_sum);

  void bill(FILE *infile, FILE *outfile,
            dword start_number, dword end_number,
            amountt start_balance);

  void mail(const string &filename,
            const string &mailoutfilename);
 };

billt::billt(cmdlinet &_cmdline):cmdline(_cmdline)
 {
  midcounter=0;
  read_patterns();
  char pid_str[TEXTLEN];
  snprintf(pid_str, TEXTLEN, "%d", getpid());
  pid=pid_str;

  getdboxtempdir(mailoutfilename, config.dboxdir);
  mailoutfilename+="listgen.";
  mailoutfilename+=pid;
  mailoutfilename+=".spool.tmp";
 }

bool billt::accumulate(const buchungt &transaction)
 {
  dword i;

  for(i=0; i<patterns.size(); i++)
    if(!pattern_accumulate_compare(patterns[i].sum, transaction))
     {
      sum(patterns[i].sum, transaction);
      patterns[i].number++;
      if(transaction.datum!=0)
       {
        patterns[i].date_min=min(patterns[i].date_min, transaction.datum);
        patterns[i].date_max=max(patterns[i].date_max, transaction.datum);
       }
      return FALSE;
     }

  return TRUE;
 }

void billt::read_patterns()
 {
  if(cmdline.isset("accumulate"))
   {
    FILE *infile2;
    string filename=cmdline.getval("accumulate");

    if(filename!="" && filename[0]!=DIRCHAR[0])
     {
      filename=config.cwd;
      filename+=DIRCHAR;
      filename+=cmdline.getval("accumulate");
     }
  
    infile2=fopen(filename.c_str(), "rt");
    if(infile2!=NULL)
     {
      patternt pattern;
      while(!pattern.sum.read(infile2))
       {
        pattern.sum.amount.amount=0;
        pattern.sum.amount.scale=0;
        pattern.sum.number=(dword)-1;
        pattern.sum.datum=0;
        patterns.push_back(pattern);
       }
      fclose(infile2);
     }
    else
     {
      cerr << "file " << filename << "not found\n";
      return;
     }
   }
 }

void billt::table_header(FILE *outfile)
 {
  switch(format)
   {
   case 'a':
    fprintf(outfile, "%s\n", formatkonto_start1);
    fprintf(outfile, "%s\n", formatkonto_start2);
    break;

   case 'l':
    fprintf(outfile,
      "\\begin{tabular*}{\\textwidth}{c|c|r|r|l@{\\extracolsep{\\fill}}}\n"
      "Nr. & Datum & \\multicolumn{1}{c|}{Betrag} & "
      "\\multicolumn{1}{c|}{MwSt} & Verwendungszweck \\hfill\\\\"
      "\\hline\n");
    break;
   }
 }

void billt::table_separator(FILE *outfile)
 {
  switch(format)
   {
   case 'a':
    fprintf(outfile, "%s\n", formatkonto_start2);
    break;

   case 'l':
    fprintf(outfile, "\\hline\n");
    break;

   case 'e':
    break;
   }
 }

void billt::table_footer(FILE *outfile)
 {
  switch(format)
   {
   case 'a':
    fprintf(outfile, "\n");
    break;

   case 'l':
    fprintf(outfile, "\\end{tabular*}\n"
                     "\n");
    break;

   case 'e':
    break;
   }

 }

void billt::show_vat(FILE *outfile, vector<buchungt> &vat)
 {
  for(unsigned i=0; i<vat.size(); i++)
   {
    amountt vat_betrag;
    char vat_betrag_s[TEXTLEN];

    vat_betrag.amount=labs(vat[i].amount.amount)-
                (labs(vat[i].amount.amount)*1000/(vat[i].mwstsatz+1000));
    vat_betrag.scale=vat[i].amount.scale;

    vat_betrag=rescale(vat_betrag, 2);
    currency_string(vat_betrag_s, vat_betrag, TRUE);

    switch(format)
     {
     case 'a':
      fprintf(outfile, "In diesem Betrag sind %s "
                       "MwSt zu %ld,%ld%% enthalten.\n",
                       vat_betrag_s,
                       vat[i].mwstsatz/10, vat[i].mwstsatz%10);
      break;

     case 'l':
      fprintf(outfile, "In diesem Betrag sind %s "
                       "MwSt zu %ld,%ld\\%% enthalten.\\\\\n",
                       vat_betrag_s,
                       vat[i].mwstsatz/10, vat[i].mwstsatz%10);
      break;

     case 'e':
      break;
     }
   }
 }

void billt::bill(FILE *infile, FILE *outfile,
                 dword start_number, dword end_number,
                 bool payments, bool care_payments,
                 bool inverse,
                 vector<buchungt> &vat,
                 buchungt &transaction_sum)
 {
  unsigned i;

  buchungt transaction;

  for(i=0; i<patterns.size(); i++)
   {
    patterns[i].sum.amount.amount=0;
    patterns[i].sum.amount.scale=0;
    patterns[i].sum.haben^=1;
    patterns[i].date_min=time(NULL);
    patterns[i].date_max=0;
    patterns[i].number=0;
   }

  seek(infile, start_number);

  for(; !transaction.read(infile) && start_number<end_number; start_number++)
   {
    bool is_payment=(strnicmp(transaction.buchungstyp, "ZAHLUNG.", 8)==0);

    if(is_payment) transaction.mwstsatz=(dword)-1;

    if(care_payments && is_payment!=payments) continue;

    if(inverse)
      transaction.haben^=1;

    if(accumulate(transaction))
      transaction.format(outfile, format, FALSE, charset);

    sum(transaction_sum, transaction);

    if(transaction.mwstsatz!=0)
     {
      for(i=0; i<vat.size(); i++)
        if(vat[i].mwstsatz==transaction.mwstsatz)
          break;

      if(i==vat.size()) 
       {
        buchungt t;
        t.mwstsatz=transaction.mwstsatz;
        t.amount.amount=0;
        t.amount.scale=0;
        vat.push_back(t);
       }

      sum(vat[i], transaction);
     }
   }

  for(i=0; i<patterns.size(); i++)
    if(patterns[i].sum.amount.amount!=0)
     {
      if(patterns[i].number==1)
        patterns[i].sum.datum=patterns[i].date_min;
      else
       {
        string purpose=patterns[i].sum.verwendungszweck;
        char date[TEXTLEN];
        struct tm start=*localtime(&patterns[i].date_min);
        struct tm end=*localtime(&patterns[i].date_max);
        snprintf(date, TEXTLEN, " (%u.%u.%u-%u.%u.%u)",
          start.tm_mday, start.tm_mon+1, start.tm_year+1900,
          end.tm_mday, end.tm_mon+1, end.tm_year+1900);
        purpose+=date;
        strmaxcpy(patterns[i].sum.verwendungszweck, purpose.c_str(),
          TEXTLEN-1);
       }
      
      patterns[i].sum.format(outfile, format, FALSE, charset);
     }

 }

void billt::bill(FILE *infile, FILE *outfile,
                 dword start_number, dword end_number,
                 amountt start_balance)
 {
  int type=atoi(cmdline.getval("type"));

  if(type<1 || type>3)
   {
    cerr << "Unknown type\n";
    return;
   }

  if(cmdline.isset("format"))
   {
    format=cmdline.getval("format")[0];
    if(format=='l') charset=ISO;
   }
  else
   {
    format='a';
    charset=IBM;
   }

  unsigned nr=0;

  if(cmdline.isset("counter"))
   {
    counter=cmdline.getval("counter");
    char id[TEXTLEN];
    snprintf(id, TEXTLEN, "auszug.%s.nummer", counter.c_str());
    nr=atol(up.get(id))+1;
   }

  switch(type)
   {
   case 1: // account statement
    switch(format)
     {
     case 'a':
     case 'e':
      fprintf(outfile, "Kontoauszug: %s/%u (%lu-%lu)\n\n",
        up.name, nr, start_number, end_number-1);
      break;

     case 'l':
      fprintf(outfile, "Kontoauszug: {\\bf %s/%u} (%lu-%lu)\n\n",
        up.name, nr, start_number, end_number-1);
      break;
     }
    break;   

   case 2:
   case 3:
    switch(format)
     {
     case 'a':
     case 'e':
      fprintf(outfile, "Rechnungsnummer: %s/%u (%lu-%lu)\n\n",
        up.name, nr, start_number, end_number-1);
      break;

     case 'l':
      fprintf(outfile, "Rechnungsnummer: {\\bf %s/%u} (%lu-%lu)\n\n",
        up.name, nr, start_number, end_number-1);
      break;
     }
   }

  if(type==3)
   {
    switch(format)
     {
     case 'a':
     case 'e':
      fprintf(outfile, "A. LEISTUNGEN\n\n");
      break;

     case 'l':
      fprintf(outfile, "{\\bf A. LEISTUNGEN}\n\n");
      break;
     }
   }

  vector<buchungt> vat;
  buchungt transaction_sum;

  if(type==1)
   {
    table_header(outfile);

    if(start_number!=0)
     {
      get_balance(infile, start_number, transaction_sum);
      strmaxcpy(transaction_sum.verwendungszweck, "alter Kontostand", TEXTLEN-1);
      transaction_sum.number=(dword)-1;
      transaction_sum.mwstsatz=(dword)-1;
      transaction_sum.format(outfile, format, FALSE, charset);
      table_separator(outfile);  
     }

    bill(infile, outfile, start_number, end_number, FALSE, FALSE, FALSE,
         vat, transaction_sum);
  
    table_separator(outfile);  
    strmaxcpy(transaction_sum.verwendungszweck, "neuer Kontostand", TEXTLEN-1);
    transaction_sum.number=(dword)-1;
    transaction_sum.mwstsatz=(dword)-1;
    transaction_sum.format(outfile, format, FALSE, charset);
    table_footer(outfile);

    end_balance=transaction_sum.amount;
    if(!transaction_sum.haben) end_balance.amount*=-1;
   }
  else if(type==2)
   {
    table_header(outfile);
    bill(infile, outfile, start_number, end_number, FALSE, TRUE, TRUE,
         vat, transaction_sum);

    table_separator(outfile);  
    strmaxcpy(transaction_sum.verwendungszweck, "Rechnungsbetrag", TEXTLEN-1);
    transaction_sum.number=(dword)-1;
    transaction_sum.mwstsatz=(dword)-1;
    transaction_sum.amount=
      rescale(transaction_sum.amount, config.currency_scale);
    transaction_sum.format(outfile, format, FALSE, charset);
    table_footer(outfile);

    show_vat(outfile, vat);

    get_balance(infile, start_number, transaction_sum);
    end_balance=transaction_sum.amount;
    if(!transaction_sum.haben) end_balance.amount*=-1;
   }
  else if(type==3)
   {
    table_header(outfile);
    bill(infile, outfile, start_number, end_number, FALSE, TRUE, TRUE, 
         vat, transaction_sum);

    table_separator(outfile);  
    strmaxcpy(transaction_sum.verwendungszweck, "Rechnungsbetrag", TEXTLEN-1);
    transaction_sum.number=(dword)-1;
    transaction_sum.mwstsatz=(dword)-1;
    transaction_sum.amount=
      rescale(transaction_sum.amount, config.currency_scale);
    transaction_sum.format(outfile, format, FALSE, charset);
    table_footer(outfile);

    show_vat(outfile, vat);

    switch(format)
     {
     case 'a':
     case 'e':
      fprintf(outfile, "\nB. KONTOAUSZUG\n\n");
      break;

     case 'l':
      fprintf(outfile, "\n\\vspace*{0.5ex}\n");
      fprintf(outfile, "{\\bf B. KONTOAUSZUG}\n\n");
      break;
     }

    buchungt current_invoice=transaction_sum;

    transaction_sum.amount=start_balance;
    if(transaction_sum.amount.amount<0)
     {
      transaction_sum.amount.amount=-transaction_sum.amount.amount;
      transaction_sum.haben=FALSE;
     }
    else
      transaction_sum.haben=TRUE;

    //get_balance(infile, start_number, transaction_sum);
    //transaction_sum.amount=
    //  rescale(transaction_sum.amount, config.currency_scale);

    table_header(outfile);

    if(transaction_sum.amount.amount!=0)
     {
      strmaxcpy(transaction_sum.verwendungszweck, "bertrag", TEXTLEN-1);
      transaction_sum.haben^=1;
      transaction_sum.number=(dword)-1;
      transaction_sum.mwstsatz=(dword)-1;
      transaction_sum.format(outfile, format, FALSE, charset);
      table_separator(outfile);  
     }

    bill(infile, outfile, start_number, end_number, TRUE, TRUE, TRUE,
         vat, transaction_sum);

    strmaxcpy(current_invoice.verwendungszweck, "Leistungen dieser Rechnung", TEXTLEN-1);
    current_invoice.number=(dword)-1;
    current_invoice.mwstsatz=(dword)-1;
    current_invoice.format(outfile, format, FALSE, charset);
    sum(transaction_sum, current_invoice);

    table_separator(outfile);  
    if(transaction_sum.haben)
      strmaxcpy(transaction_sum.verwendungszweck, "zu zahlender Betrag", TEXTLEN-1);
    else
      strmaxcpy(transaction_sum.verwendungszweck, "Guthaben", TEXTLEN-1);

    transaction_sum.number=(dword)-1;
    transaction_sum.mwstsatz=(dword)-1;
    transaction_sum.format(outfile, format, FALSE, charset);
    table_footer(outfile);

    end_balance=transaction_sum.amount;
    transaction_sum.haben^=1;
    if(!transaction_sum.haben) end_balance.amount*=-1;
   }
 }

void billt::mail(const string &filename,
                 const string &mailoutfilename)
 {
  FILE *tmpfile, *infile, *outfile;
  dword len;
  string tmpfilename;
  char tempstr[TEXTLEN], line[1000], *tptr;

  getdboxtempdir(tmpfilename, config.dboxdir);
  tmpfilename+="listgen.";
  tmpfilename+=pid;
  tmpfilename+=".mail.tmp";

  outfile=fopen(mailoutfilename.c_str(), "ab");

  infile=fopen(filename.c_str(), "rt");
  if(infile==NULL) return;

  tmpfile=fopen(tmpfilename.c_str(), "wb");
  if(tmpfile==NULL)
    logf.internalerror("mail", "Temporaerdatei konnte nicht geoeffnet werden");

  while(fgets(line, 900, infile)!=NULL)
   {
    tptr=strchr(line, '\r'); if(tptr!=NULL) *tptr=0;
    tptr=strchr(line, '\n'); if(tptr!=NULL) *tptr=0;

    fprintf(tmpfile, "%s\r\n", line);
   }

  fflush(tmpfile);
  len=ftell(tmpfile);
  fclose(infile);
  fclose(tmpfile);

  fprintf(outfile, "ABS:\tSYSOP@%s.%s\r\n", config.boxname, config.domain);

  if(counter=="")
    fprintf(outfile, "BET:\tAuszge fr User %s\r\n", up.name);
  else
   {
    char id[TEXTLEN];
    snprintf(id, TEXTLEN, "auszug.%s.nummer", counter.c_str());
    up.get(id, tempstr);
    fprintf(outfile, "BET:\tAuszug %ld fr User %s\r\n", 
            atol(tempstr)+1, up.name);
   }

  makeeda(time(NULL), tempstr);
  fprintf(outfile, "EDA:\t%s\r\n", tempstr);

  fprintf(outfile, "EMP:\t%s@%s.%s\r\n",
    up.name, config.boxname, config.domain);

  fprintf(outfile, "LEN:\t%lu\r\n", len);

  fprintf(outfile, "MAILER:\tDBOX Kontoverwaltung\r\n");

  fprintf(outfile, "MID:\tlistgen.%lu.%lu@%s.%s\r\n",
    midcounter++, time(NULL), config.boxname, config.domain);

  if(config.organization[0]!=0)
    fprintf(outfile, "ORG:\t%s\r\n", config.organization);

  fprintf(outfile, "ROT:\r\n");

  fprintf(outfile, "\r\n");

  infile=fopen(tmpfilename.c_str(), "rb");
  filecopy(infile, outfile, len);
  fclose(infile);
  fclose(outfile);

  remove(tmpfilename.c_str());
  remove(filename.c_str());
 }

void billt::bill_user()
 {
  dword start_entry_number=0, end_entry_number=0;
  amountt start_balance;

  FILE *infile;

   {
    string filename=up.getpath();
    filename+="konto";  
    infile=fopen(filename.c_str(), "rt");
    if(infile==NULL) return;
   }

  if(cmdline.isset("upto"))
   {
    end_entry_number=max(atol(cmdline.getval("below")), 
                         atol(up|"buchungsnummer"));
   }
  else if(cmdline.isset("month") || cmdline.isset("year"))
   {
    rewind(infile);

    end_entry_number=0;

    int month, year;

    if(cmdline.isset("year"))
      year=atoi(cmdline.getval("year"));
    else
     {      
      time_t t;
      time(&t);
      struct tm *tm=localtime(&t);
      year=tm->tm_year+1900;
     }

    if(cmdline.isset("month"))
      month=atoi(cmdline.getval("month"))-1;
    else
      month=11;

    while(TRUE)
     {
      buchungt transaction; 

      if(transaction.read(infile)) break;

      struct tm *tm=localtime(&transaction.datum);

      if((tm->tm_year+1900)>year) break;
      if(((tm->tm_year+1900)==year) && (tm->tm_mon>month)) break;

      end_entry_number++;
     }
   }
  else
    end_entry_number=atol(up|"buchungsnummer");

  if(cmdline.isset("counter"))
   {
    counter=cmdline.getval("counter");
    char id[TEXTLEN];

    snprintf(id, TEXTLEN, "auszug.%s.buchungsnummer", counter.c_str());
    start_entry_number=atol(up.get(id));

    snprintf(id, TEXTLEN, "auszug.%s.uebertrag", counter.c_str());
    start_balance.amount=atol(up.get(id));

    snprintf(id, TEXTLEN, "auszug.%s.uebertrag.scale", counter.c_str());
    start_balance.scale=atol(up.get(id));
   }

  cerr << "billing " << up.name << " entries " << start_entry_number 
       << "-" << end_entry_number-1 << "\n";

  if(cmdline.isset("mail"))
   {
    if(start_entry_number!=end_entry_number)
     {
      string filename;
      getdboxtempdir(filename, config.dboxdir);
      filename+="listgen.";
      filename+=pid;
      filename+=".mail.tmp.2";

      FILE *outfile=fopen(filename.c_str(), "wt");
      if(outfile==NULL)
        logf.internalerror("makekonto", "Failed to open tempfile");

      bill(infile, outfile, start_entry_number, end_entry_number,
        start_balance);

      fclose(outfile);

      mail(filename, mailoutfilename);
 
      unlink(filename.c_str());
     }
   }
  else
    bill(infile, stdout, start_entry_number, end_entry_number,
      start_balance);

  fclose(infile);

  if(cmdline.isset("save") && counter!="" &&
     start_entry_number!=end_entry_number)
   {
    if(up.lock())
      cerr << "locking " << up.name << " failed\n";
    else
     {
      char id[TEXTLEN];
      snprintf(id, TEXTLEN, "auszug.%s.nummer", counter.c_str());
      up.set(id, atol(up.get(id))+1);

      sprintf(id, "auszug.%s.uebertrag", counter.c_str());
      up.set(id, end_balance.amount);

      sprintf(id, "auszug.%s.uebertrag.scale", counter.c_str());
      up.set(id, (int)end_balance.scale);

      sprintf(id, "auszug.%s.buchungsnummer", counter.c_str());
      up.set(id, end_entry_number);

      up.write();
      up.unlock();
     }
   }
 }

void billt::dobill()
 {
  char username[TEXTLEN];

  while(fgets(username, TEXTLEN-1, stdin)!=NULL)
    if(username[0]!=0 && username[0]!='.' && username[0]!='_')
     {
      char *tptr=strchr(username, '\r'); if(tptr!=NULL) *tptr=0;
            tptr=strchr(username, '\n'); if(tptr!=NULL) *tptr=0;

      username[TEXTLEN-1]=0;
      tptr=strrchr(username, '/');
      if(tptr!=NULL) *tptr=0;
      up.setname(username);
      if(up.read())
        fprintf(stderr, "Not found: %s\n", username);
      else
        bill_user();
     }

  if(cmdline.isset("mail"))
    spool_zc_puffer_mail(mailoutfilename.c_str(), config.boxname);
 }

void dobill(cmdlinet &cmdline)
 {
  billt bill(cmdline);
  bill.dobill();
 }
