#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>

#include <cmdline.h>
#include <logf.h>
#include <dboxpath.h>
#include <oldtools.h>
#include <zconnect.h>
#include <account.h>
#include <spoolp.h>

void seek(FILE *infile, dword entry_number)
 {
  buchungt transaction;

  rewind(infile);
  
  while(entry_number!=0)
   {
    transaction.read(infile);
    entry_number--;
   }
 }

bool pattern_accumulate_compare(const buchungt &pattern, const buchungt &transaction)
 {
  if(pattern.bearbeiter[0]!=0)
    if(!checkjoker(pattern.bearbeiter, transaction.bearbeiter))
      return TRUE;

  if(pattern.buchungstyp[0]!=0)
    if(!checkjoker(pattern.buchungstyp, transaction.buchungstyp))
      return TRUE;

  if(pattern.mwstsatz!=0)
    if(pattern.mwstsatz!=transaction.mwstsatz)
      return TRUE;

  if(pattern.haben!=transaction.haben)
      return TRUE;

  return FALSE;
 }

void sum(buchungt &transaction1, buchungt transaction2)
 {
  signed long sum_value;

  if(transaction1.amount.scale>transaction2.amount.scale)
    transaction2.amount=rescale(transaction2.amount, transaction1.amount.scale);
  else if(transaction2.amount.scale>transaction1.amount.scale)
    transaction1.amount=rescale(transaction1.amount, transaction2.amount.scale);

  sum_value=(transaction1.amount.amount*(transaction1.haben?1:-1))+
            (transaction2.amount.amount*(transaction2.haben?1:-1));

  transaction1.amount.amount=labs(sum_value);
  transaction1.haben=(sum_value>=0);
 }

class billt
 {
 public:
  dword midcounter;
  vector<buchungt> patterns;
  cmdlinet &cmdline;
  char format;
  charsett charset;
  userpt up;
  string description, counter, pid;
  string mailoutfilename;
  amountt end_balance;

  billt(cmdlinet &_cmdline);

  void read_patterns();
  bool accumulate(const buchungt &transaction);
  void table_header(FILE *outfile);
  void bill_user();
  void dobill();
  void bill(FILE *infile, FILE *outfile,
            dword start_number, dword end_number, bool payments);
  void bill(FILE *infile, FILE *outfile,
            dword start_number, dword end_number);
  void mail(const string &filename,
            const string &mailoutfilename);
 };

billt::billt(cmdlinet &_cmdline):cmdline(_cmdline)
 {
  midcounter=0;
  read_patterns();
  char pid_str[TEXTLEN];
  snprintf(pid_str, TEXTLEN, "%d", getpid());
  pid=pid_str;

  getdboxtempdir(mailoutfilename, config.dboxdir);
  mailoutfilename+="listgen.";
  mailoutfilename+=pid;
  mailoutfilename+=".spool.tmp";
 }

bool billt::accumulate(const buchungt &transaction)
 {
  dword i;

  for(i=0; i<patterns.size(); i++)
    if(!pattern_accumulate_compare(patterns[i], transaction))
     {
      sum(patterns[i], transaction);
      return FALSE;
     }

  return TRUE;
 }

void billt::read_patterns()
 {
  if(cmdline.isset("accumulate"))
   {
    FILE *infile2;
    string filename=cmdline.getval("accumulate");

    if(filename!="" && filename[0]!=DIRCHAR[0])
     {
      filename=config.cwd;
      filename+=DIRCHAR;
      filename+=cmdline.getval("accumulate");
     }
  
    infile2=fopen(filename.c_str(), "rt");
    if(infile2!=NULL)
     {
      buchungt pattern;
      while(!pattern.read(infile2))
       {
        pattern.amount.amount=0;
        pattern.amount.scale=0;
        pattern.buchungsnummer=(dword)-1;
        pattern.datum=0;
        patterns.push_back(pattern);
       }
      fclose(infile2);
     }
    else
     {
      cerr << "file " << filename << "not found\n";
      return;
     }
   }
 }

void billt::table_header(FILE *outfile)
 {
  switch(format)
   {
   case 'a':
    fprintf(outfile, "%s\n", formatkonto_start1);
    fprintf(outfile, "%s\n", formatkonto_start2);
    break;

   case 'l':
    fprintf(outfile,
      "\\begin{tabular*}{\\textwidth}{c|c|r|r|l@{\\extracolsep{\\fill}}}\n"
      "Nr. & Datum & \\multicolumn{1}{c|}{Betrag} & "
      "\\multicolumn{1}{c|}{MwSt} & Verwendungszweck \\hfill\\\\"
      "\\hline\n");
    break;
   }
 }

void billt::bill(FILE *infile, FILE *outfile,
                 dword start_number, dword end_number, bool payments)
 {
  unsigned i;

  table_header(outfile);

  buchungt transaction_sum, transaction;
  vector<buchungt> mwst;

  transaction_sum.amount.amount=0;
  transaction_sum.amount.scale=config.currency_scale;
  transaction_sum.haben=TRUE;

  for(i=0; i<patterns.size(); i++)
   {
    patterns[i].amount.amount=0;
    patterns[i].amount.scale=0;
    patterns[i].haben^=1;
   }

  seek(infile, start_number);

  for(; !transaction.read(infile) && start_number<end_number; start_number++)
   {
    bool is_payment=(strnicmp(transaction.buchungstyp, "ZAHLUNG.", 8)==0);

    if(is_payment!=payments) continue;

    transaction.haben^=1;

    if(accumulate(transaction))
      transaction.format(outfile, format, FALSE, charset);

    sum(transaction_sum, transaction);

    if(transaction.mwstsatz!=0)
     {
      for(i=0; i<mwst.size(); i++)
        if(mwst[i].mwstsatz==transaction.mwstsatz)
          break;

      if(i==mwst.size()) 
       {
        buchungt t;
        t.mwstsatz=transaction.mwstsatz;
        t.amount.amount=0;
        t.amount.scale=0;
        mwst.push_back(t);
       }

      sum(mwst[i], transaction);
     }
   }

  for(i=0; i<patterns.size(); i++)
    if(patterns[i].amount.amount!=0)
      patterns[i].format(outfile, format, FALSE, charset);

  switch(format)
   {
   case 'a':
    fprintf(outfile, "%s\n", formatkonto_start2);
    break;

   case 'l':
    fprintf(outfile, "\\hline\n");
    break;

   case 'e':
    break;
   }

  strcpy(transaction_sum.verwendungszweck, "Gesamt");
  transaction_sum.buchungsnummer=(dword)-1;
  transaction_sum.mwstsatz=(dword)-1;
  transaction_sum.format(outfile, format, FALSE, charset);

  switch(format)
   {
   case 'a':
    fprintf(outfile, "\n");
    if(mwst.size()!=0) fprintf(outfile, "\n");
    break;

   case 'l':
    fprintf(outfile, "\\end{tabular*}\n"
                     "\n");
    break;

   case 'e':
    break;
   }

  for(i=0; i<mwst.size(); i++)
   {
    amountt mwst_betrag;
    char mwst_betrag_s[TEXTLEN];

    mwst_betrag.amount=labs(mwst[i].amount.amount)-
                (labs(mwst[i].amount.amount)*1000/(mwst[i].mwstsatz+1000));
    mwst_betrag.scale=mwst[i].amount.scale;

    mwst_betrag=rescale(mwst_betrag, 2);
    currency_string(mwst_betrag_s, mwst_betrag, TRUE);

    switch(format)
     {
     case 'a':
      fprintf(outfile, "In diesem Betrag sind %s "
                       "MwSt zu %ld,%ld%% enthalten.\n",
                       mwst_betrag_s,
                       mwst[i].mwstsatz/10, mwst[i].mwstsatz%10);
      break;

     case 'l':
      fprintf(outfile, "In diesem Betrag sind %s "
                       "MwSt zu %ld,%ld\\%% enthalten.\\\\\n",
                       mwst_betrag_s,
                       mwst[i].mwstsatz/10, mwst[i].mwstsatz%10);
      break;

     case 'e':
      break;
     }
   }
 }

void billt::bill(FILE *infile, FILE *outfile,
                 dword start_number, dword end_number)
 {
  int type=atoi(cmdline.getval("type"));
  //bool invoice=(type>=2);
  //char format=cmdline.getval("format")[0];

  bill(infile, outfile, start_number, end_number, type==3);

  #if 0
  if(cmdline.isset("counter"))
   {
    char id[TEXTLEN];
    snprintf(id, TEXTLEN, "auszug.%c.nummer", what);
    dword invoice_nr=atol(up.get(id))+1;

    switch(format)
     {
     case 'e':
     case 'a':
      fprintf(outfile, "%s %ld (%s %lu-%lu)\n", 
        description, invoice_nr, up.name, start_number, end_number-1);
      fprintf(outfile, "\n");
      break;

     case 'l':
      fprintf(outfile, "%s {\\bf %ld} (%s %lu-%lu)\n", 
        description, invoice_nr, up.name, start_number, end_number-1);
      fprintf(outfile, "\n");
      break;
     }
   }

  if(type==3)
   {
    switch(format)
     {
     case 'e':
     case 'a':
      fprintf(outfile, "A. LEISTUNGEN\n");
      fprintf(outfile, "\n");
      break;

     case 'l':
      fprintf(outfile, "{\\bf A. LEISTUNGEN}\n");
      fprintf(outfile, "\n");
     }
   }
  #endif

  table_header(outfile);


    
    


  #if 0
  struct tm *tm;
  time_t t;
  dword pos, postennr;
  buchungt buchung, summenbuchung;
  signed long uebertrag;
  byte uebertrag_scale;
  int anzmwst, i;
  dword anzposten=0;
  charsett charset;
  buchungt mwst[100];
  #endif

  //if(format=='l') charset=ISO; else charset=IBM;

  #if 0
  if(rechnung)
   {
    dword first=0, last=0;
    bool isfirst=TRUE;
    dword rechnungsnummer;

    time(&t);
    tm=localtime(&t);
    anzmwst=0;
    pos=ftell(infile);

    summenbuchung.amount.amount=0;
    summenbuchung.amount.scale=2;
    summenbuchung.haben=TRUE;

    for(postennr=0; postennr<anzposten; postennr++)
      posten[postennr].haben^=1;

    while(!buchung.read(infile))
      if(strnicmp(buchung.buchungstyp, "ZAHLUNG.", 8)!=0 &&
                  buchung.buchungsnummer<(dword)atol(up|"buchungsnummer"))
       {
        buchung.haben^=1;

        if(akkumulieren(anzposten, posten, &buchung))
          buchung.format(outfile, format, FALSE, charset);

        sum_buchung(&summenbuchung, &buchung);

        if(buchung.mwstsatz!=0)
         {
          for(i=0; i<anzmwst; i++)
            if(mwst[i].mwstsatz==buchung.mwstsatz)
              break;

          if(i==anzmwst) 
           {
            mwst[i].mwstsatz=buchung.mwstsatz;
            mwst[i].amount.amount=0;
            mwst[i].amount.scale=0;
            anzmwst++;
           }

          sum_buchung(&mwst[i], &buchung);
         }
       }

    for(postennr=0; postennr<anzposten; postennr++)
      if(posten[postennr].amount.amount!=0)
        posten[postennr].format(outfile, format, FALSE, charset);

    switch(format)
     {
     case 'a':
      fprintf(outfile, "%s\n", formatkonto_start2);
      break;

     case 'l':
      fprintf(outfile, "\\hline\n");
      break;

     case 'e':
      break;
     }

    strcpy(buchung.verwendungszweck, "Gesamt");

    summenbuchung.buchungsnummer=(dword)-1;
    summenbuchung.mwstsatz=(dword)-1;
    summenbuchung.format(outfile, format, FALSE, charset);

    switch(format)
     {
     case 'a':
      fprintf(outfile, "\n");
      if(anzmwst!=0) fprintf(outfile, "\n");
      break;

     case 'l':
      fprintf(outfile, "\\end{tabular*}\n"
                       "\n");
      break;

     case 'e':
      break;
     }

    for(i=0; i<anzmwst; i++)
     {
      amountt mwst_betrag;
      char mwst_betrag_s[TEXTLEN];

      mwst_betrag.amount=labs(mwst[i].amount.amount)-
                  (labs(mwst[i].amount.amount)*1000/(mwst[i].mwstsatz+1000));
      mwst_betrag.scale=mwst[i].amount.scale;

      mwst_betrag=rescale(mwst_betrag, 2);
      currency_string(mwst_betrag_s, mwst_betrag, TRUE);

      switch(format)
       {
       case 'a':
        fprintf(outfile, "In diesem Betrag sind %s "
                         "MwSt zu %ld,%ld%% enthalten.\n",
                         mwst_betrag_s,
                         mwst[i].mwstsatz/10, mwst[i].mwstsatz%10);
        break;

       case 'l':
        fprintf(outfile, "In diesem Betrag sind %s "
                         "MwSt zu %ld,%ld\\%% enthalten.\\\\\n",
                         mwst_betrag_s,
                         mwst[i].mwstsatz/10, mwst[i].mwstsatz%10);
        break;

       case 'e':
        break;
       }
     }

    switch(format)
     {
     case 'a':
      fprintf(outfile, "\n");
      fprintf(outfile, "B. ZAHLUNGEN\n");
      fprintf(outfile, "\n");

      fseek(infile, pos, SEEK_SET);

      fprintf(outfile, "%s\r\n", formatkonto_start1);
      fprintf(outfile, "%s\r\n", formatkonto_start2);
      break;

     case 'l':
      fprintf(outfile, "\n");
      fprintf(outfile, "\\vspace*{0.5ex}\n");
      fprintf(outfile, "{\\bf B. ZAHLUNGEN}\n");
      fprintf(outfile, "\n");

      fseek(infile, pos, SEEK_SET);

      fprintf(outfile,
        "\\begin{tabular*}{\\textwidth}{c|c|r|r|l@{\\extracolsep{\\fill}}}\n"
        "Nr. & Datum & \\multicolumn{1}{c|}{Betrag} & "
        "\\multicolumn{1}{c|}{MwSt} & Verwendungszweck \\hfill\\\\"
        "\\hline\n");
      break;

     case 'e':
      fseek(infile, pos, SEEK_SET);
      break;
     }
 
    if(summenbuchung.amount.amount!=0)
     {
      summenbuchung.datum=t;
      snprintf(summenbuchung.verwendungszweck, TEXTLEN,
              "Leistungen Rechnung %ld", rechnungsnummer);
      summenbuchung.format(outfile, format, FALSE, charset);
     }

    if(what=='k')
     {
      uebertrag=0;
      uebertrag_scale=0;
     }
    else
     {
      snprintf(id, TEXTLEN, "auszug.%c.uebertrag", what);
      up.get(id, tempstr);
      uebertrag=atol(tempstr);

      snprintf(id, TEXTLEN, "auszug.%c.uebertrag.scale", what);
      up.get(id, tempstr);
      if(tempstr[0]==0)
        uebertrag_scale=2;
      else
        uebertrag_scale=atol(tempstr);
     }

    if(uebertrag!=0)
     { 
      buchung.clear();
      strcpy(buchung.verwendungszweck, "bertrag der letzten Rechnung");
      buchung.buchungsnummer=(dword)-1;
      buchung.mwstsatz=(dword)-1;
      buchung.amount.amount=labs(uebertrag);
      buchung.haben=(uebertrag<0);
      buchung.amount.scale=uebertrag_scale;
      buchung.format(outfile, format, FALSE, charset);
     }
       
    while(!buchung.read(infile))
      if(strnicmp(buchung.buchungstyp, "ZAHLUNG.", 8)==0 &&
                  buchung.buchungsnummer<(dword)atol(up|"buchungsnummer"))
       {
        buchung.haben^=1;
        buchung.mwstsatz=(dword)-1;
        buchung.format(outfile, format, FALSE, charset);
       }

    switch(format)
     {
     case 'a':
      fprintf(outfile, "%s\n", formatkonto_start2);
      break;

     case 'l':
      fprintf(outfile, "\\hline\n");
      break;
     }
       
    buchung.clear();
    buchung.buchungsnummer=(dword)-1;
    buchung.mwstsatz=(dword)-1;
    buchung.amount=abs(rescale(getbalance(up), 2));
    buchung.haben=(getbalance(up).amount<0);

    if(!buchung.haben)
      strmaxcpy(buchung.verwendungszweck, "Guthaben", TEXTLEN-1);
    else
      strmaxcpy(buchung.verwendungszweck, "Zu zahlender Betrag", TEXTLEN-1);

    buchung.format(outfile, format, FALSE, charset);

    switch(format)
     {
     case 'l':
      fprintf(outfile, "\\end{tabular*}\n\n");
      break;
     }
   }
  else
    switch(format)
     {
     case 'a':
      if(what=='k')
        fprintf(outfile, "Kontoauszge von......: %s\n", up.name);
      else
       {
        fprintf(outfile, "Kontoauszug von.......: %s\n", up.name);
        snprintf(id, TEXTLEN, "auszug.%c.nummer", what);
        up.get(id, tempstr);
        fprintf(outfile, "Nummer................: %ld\n", atol(tempstr)+1);
       }

      currency_string(tempstr, getbalance(up), TRUE);
      fprintf(outfile, "aktueller Kontostand..: %s\n", tempstr);
      //fprintf(outfile, "Kontoberzug erlaubt..: %s\n", noyes1(up.kontoueberzug));
      fprintf(outfile, "\n%s\n%s\n", formatkonto_start1, formatkonto_start2);

      while(!buchung.read(infile))
        if(buchung.buchungsnummer<(dword)atol(up|"buchungsnummer"))
          if(akkumulieren(anzposten, posten, &buchung))
            buchung.format(outfile, format, TRUE, IBM);

      for(postennr=0; postennr<anzposten; postennr++)
        if(posten[postennr].amount.amount!=0)
          posten[postennr].format(outfile, format, TRUE, IBM);

      fprintf(outfile, "%s\n", formatkonto_ende);
      break;

     case 'e':
      if(what=='k')
        fprintf(outfile, "Kontoauszge von:\t%s\n", up.name);
      else
       {
        fprintf(outfile, "Kontoauszug von:\t%s\n", up.name);
        snprintf(id, TEXTLEN, "auszug.%c.nummer", what);
        up.get(id, tempstr);
        fprintf(outfile, "Nummer:\t%ld\n", atol(tempstr)+1);
       }

      currency_string(tempstr, getbalance(up), TRUE);
      fprintf(outfile, "aktueller Kontostand:\t%s\n", tempstr);
      fprintf(outfile, "Kontoberzug:\t%s\n", noyes1(atoi(up|"kontoueberzug")));
      fprintf(outfile, "\n"
              "Nr.\tDatum\tBetrag\tMwSt\tVerwendungszweck\n");
          
      while(!buchung.read(infile))
        if(buchung.buchungsnummer<(dword)atol(up|"buchungsnummer"))
          if(akkumulieren(anzposten, posten, &buchung))
            buchung.format(outfile, format, FALSE, IBM);

      for(postennr=0; postennr<anzposten; postennr++)
        if(posten[postennr].amount.amount!=0)
          posten[postennr].format(outfile, format, FALSE, IBM);

      fprintf(outfile, "\n");
      break;

     case 'l':
      while(!buchung.read(infile))
        if(buchung.buchungsnummer<(dword)atol(up|"buchungsnummer"))
          if(akkumulieren(anzposten, posten, &buchung))
            buchung.format(outfile, format, FALSE, ISO);

      for(postennr=0; postennr<anzposten; postennr++)
        if(posten[postennr].amount.amount!=0)
          posten[postennr].format(outfile, format, FALSE, ISO);
      break;
     }
  #endif
 }

void billt::mail(const string &filename,
                 const string &mailoutfilename)
 {
  FILE *tmpfile, *infile, *outfile;
  dword len;
  string tmpfilename;
  char tempstr[TEXTLEN], line[1000], *tptr;

  getdboxtempdir(tmpfilename, config.dboxdir);
  tmpfilename+="listgen.";
  tmpfilename+=pid;
  tmpfilename+=".mail.tmp";

  outfile=fopen(mailoutfilename.c_str(), "ab");

  infile=fopen(filename.c_str(), "rt");
  if(infile==NULL) return;

  tmpfile=fopen(tmpfilename.c_str(), "wb");
  if(tmpfile==NULL)
    logf.internalerror("mail", "Temporaerdatei konnte nicht geoeffnet werden");

  while(fgets(line, 900, infile)!=NULL)
   {
    tptr=strchr(line, '\r'); if(tptr!=NULL) *tptr=0;
    tptr=strchr(line, '\n'); if(tptr!=NULL) *tptr=0;

    fprintf(tmpfile, "%s\r\n", line);
   }

  fflush(tmpfile);
  len=ftell(tmpfile);
  fclose(infile);
  fclose(tmpfile);

  fprintf(outfile, "ABS:\tSYSOP@%s.%s\r\n", config.boxname, config.domain);

  if(counter=="")
    fprintf(outfile, "BET:\tAuszge fr User %s\r\n", up.name);
  else
   {
    char id[TEXTLEN];
    snprintf(id, TEXTLEN, "auszug.%s.nummer", counter.c_str());
    up.get(id, tempstr);
    fprintf(outfile, "BET:\tAuszug %ld fr User %s\r\n", 
            atol(tempstr)+1, up.name);
   }

  #if 0
    if(what=='k')
      fprintf(outfile, "BET:\tGesamtrechnung fr User %s\r\n", up.name);
    else
     {
      snprintf(id, TEXTLEN, "auszug.%c.nummer", what);
      up.get(id, tempstr);
      fprintf(outfile, "BET:\tRechnung %ld fr User %s\r\n", 
              atol(tempstr)+1, up.name);
     }
    break;

   case 'e':
    fprintf(outfile, "BET:\tBuchungs-Export fr User %s\r\n", up.name);
    break;
  #endif

  makeeda(time(NULL), tempstr);
  fprintf(outfile, "EDA:\t%s\r\n", tempstr);

  fprintf(outfile, "EMP:\t%s@%s.%s\r\n",
    up.name, config.boxname, config.domain);

  fprintf(outfile, "LEN:\t%lu\r\n", len);

  fprintf(outfile, "MAILER:\tDBOX Kontoverwaltung\r\n");

  fprintf(outfile, "MID:\tlistgen.%lu.%lu@%s.%s\r\n",
    midcounter++, time(NULL), config.boxname, config.domain);

  if(config.organization[0]!=0)
    fprintf(outfile, "ORG:\t%s\r\n", config.organization);

  fprintf(outfile, "ROT:\r\n");

  fprintf(outfile, "\r\n");

  infile=fopen(tmpfilename.c_str(), "rb");
  filecopy(infile, outfile, len);
  fclose(infile);
  fclose(outfile);

  remove(tmpfilename.c_str());
  remove(filename.c_str());
 }

void billt::bill_user()
 {
  dword start_entry_number=0, end_entry_number=0;

  FILE *infile;

   {
    string filename=up.getpath();
    filename+="konto";  
    infile=fopen(filename.c_str(), "rt");
    if(infile==NULL) return;
   }

  if(cmdline.isset("upto"))
   {
    end_entry_number=max(atol(cmdline.getval("below")), 
                         atol(up|"buchungsnummer"));
   }
  else if(cmdline.isset("month") || cmdline.isset("year"))
   {
    // TODO
   }
  else
    end_entry_number=atol(up|"buchungsnummer");

  //amountt neu_uebertrag=getbalance(up);

  if(cmdline.isset("counter"))
   {
    counter=cmdline.getval("counter");
    char id[TEXTLEN];
    snprintf(id, TEXTLEN, "auszug.%s.buchungsnummer", counter.c_str());
    start_entry_number=atol(up.get(id));
   }

  cerr << "billing " << up.name << " entries " << start_entry_number 
       << "-" << end_entry_number-1 << "\n";

  seek(infile, start_entry_number);

  if(cmdline.isset("mail"))
   {
    if(start_entry_number!=end_entry_number)
     {
      string filename;
      getdboxtempdir(filename, config.dboxdir);
      filename+="listgen.";
      filename+=pid;
      filename+=".mail.tmp.2";

      FILE *outfile=fopen(filename.c_str(), "wt");
      if(outfile==NULL)
        logf.internalerror("makekonto", "Failed to open tempfile");

      bill(infile, outfile, start_entry_number, end_entry_number);

      fclose(outfile);

      mail(filename, mailoutfilename);
 
      unlink(filename.c_str());
     }
   }
  else
    bill(infile, stdout, start_entry_number, end_entry_number);

  fclose(infile);

  if(cmdline.isset("save") && counter!="" &&
     start_entry_number!=end_entry_number)
   {
    if(up.lock())
      cerr << "locking " << up.name << " failed\n";
    else
     {
      char id[TEXTLEN];
      snprintf(id, TEXTLEN, "auszug.%s.nummer", counter.c_str());
      up.set(id, atol(up.get(id))+1);

      sprintf(id, "auszug.%s.uebertrag", counter.c_str());
      up.set(id, end_balance.amount);

      sprintf(id, "auszug.%s.uebertrag.scale", counter.c_str());
      up.set(id, (int)end_balance.scale);

      sprintf(id, "auszug.%s.buchungsnummer", counter.c_str());
      up.set(id, end_entry_number);

      up.write();
      up.unlock();
     }
   }
 }

void billt::dobill()
 {
  char username[TEXTLEN];

  while(fgets(username, TEXTLEN-1, stdin)!=NULL)
    if(username[0]!=0 && username[0]!='.' && username[0]!='_')
     {
      char *tptr=strchr(username, '\r'); if(tptr!=NULL) *tptr=0;
            tptr=strchr(username, '\n'); if(tptr!=NULL) *tptr=0;

      username[TEXTLEN-1]=0;
      tptr=strrchr(username, '/');
      if(tptr!=NULL) *tptr=0;
      up.setname(username);
      if(up.read())
        fprintf(stderr, "Not found: %s\n", username);
      else
        bill_user();
     }

  if(cmdline.isset("mail"))
    spool_puffer_mail(mailoutfilename.c_str(), config.boxname);
 }

void dobill(cmdlinet &cmdline)
 {
  billt bill(cmdline);
  bill.dobill();
 }
