#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>

#include <vector.h>

#include <userdb.h>
#include <account.h>
#include <charsettools.h>
#include <libdboxversion.h>
#include <stringtools.h>
#include <dboxversion.h>
#include <cmdline.h>

bool accumulate_compare(const buchungt &trans1, const buchungt &trans2)
 {
  if(stringcasecmp(trans1.buchungstyp, trans2.buchungstyp)!=0)
    return TRUE;

  if(trans1.mwstsatz!=trans2.mwstsatz)
    return TRUE;

  return FALSE;
 }

void sum_transaction(buchungt &trans1, const buchungt &_trans2)
 {
  signed long summe;
  buchungt trans2=_trans2;

  if(trans1.amount.scale>trans2.amount.scale)
    trans2.amount=rescale(trans2.amount, trans1.amount.scale);
  else if(trans2.amount.scale>trans1.amount.scale)
    trans1.amount=rescale(trans1.amount, trans2.amount.scale);

  summe=(trans1.amount.amount*(trans1.haben?1:-1))+
        (trans2.amount.amount*(trans2.haben?1:-1));

  trans1.amount.amount=labs(summe);
  trans1.haben=(summe>=0);
 }

void accumulate(vector<buchungt> &transactions, const buchungt &transaction)
 {
  dword i;

  for(i=0; i<transactions.size(); i++)
    if(!accumulate_compare(transactions[i], transaction))
     {
      sum_transaction(transactions[i], transaction);
      return;
     }

  transactions.push_back(transaction);
 }

void displaysum(vector<buchungt> &transactions, bool format)
 {
  if(format)
   {
    amountt sum;
    char tempstr[TEXTLEN];

    sum.amount=0;
    sum.scale=0;
    cout << "Debit:\n\n";

    for(unsigned i=0; i<transactions.size(); i++)
      if(!transactions[i].haben)
       {
        printf("  %-30s ", transactions[i].buchungstyp);
        rescale(transactions[i].amount, config.currency_scale);
        currency_string(tempstr, transactions[i].amount, TRUE);
        printf("%15s", tempstr);

        printf(" %2lu,%lu%%",
          transactions[i].mwstsatz/10, transactions[i].mwstsatz%10);

        printf("\n");

        sum+=transactions[i].amount;
       }

    printf("  %-30s ", "total");
    rescale(sum, config.currency_scale);
    currency_string(tempstr, sum, TRUE);
    printf("%15s\n\n", tempstr);

    sum.amount=0;
    sum.scale=0;
    cout << "Credit:\n\n";

    for(unsigned i=0; i<transactions.size(); i++)
      if(transactions[i].haben)
       {
        printf("  %-30s ", transactions[i].buchungstyp);
        rescale(transactions[i].amount, config.currency_scale);
        currency_string(tempstr, transactions[i].amount, TRUE);
        printf("%15s", tempstr);

        printf(" %2lu,%lu%%",
          transactions[i].mwstsatz/10, transactions[i].mwstsatz%10);

        printf("\n");

        sum+=transactions[i].amount;
       }

    printf("  %-30s ", "total");
    rescale(sum, config.currency_scale);
    currency_string(tempstr, sum, TRUE);
    printf("%15s\n\n", tempstr);
   }
  else
    for(unsigned i=0; i<transactions.size(); i++)
     {
      cout << "H:" << transactions[i].haben
           << ";B:" << transactions[i].amount.amount 
           << ";A:" << int(transactions[i].amount.scale)
           << ";M:" << transactions[i].mwstsatz
           << ";T:" << transactions[i].buchungstyp
           << "\n";
     }
 }

void dosum(cmdlinet &cmdline)
 {
  userpt up;
  char username[TEXTLEN];
  char *tptr;
  vector<buchungt> transactions;

  while(fgets(username, TEXTLEN-1, stdin)!=NULL)
    if(username[0]!=0 && username[0]!='.' && username[0]!='_')
     {
      tptr=strchr(username, '\n'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(username, '\r'); if(tptr!=NULL) *tptr=0;
      username[TEXTLEN-1]=0;
      tptr=strrchr(username, '/');
      if(tptr!=NULL) *tptr=0;
      strcpy(up.name, username);
      if(up.read())
        fprintf(stderr, "Not found: %s\n", username);
      else
       {
        char filename[DIRLEN];
 
        snprintf(filename, DIRLEN, "%skonto", up.getpath());
        FILE *infile=fopen(filename, "rt");
        if(infile!=NULL)
         {
          buchungt trans;
          while(!trans.read(infile))
           {
            struct tm *tm;
            tm=localtime(&trans.datum);

            if(cmdline.isset("year"))
              if((tm->tm_year+1900)!=atol(cmdline.getval("year")))
                continue;

            if(cmdline.isset("month"))
              if((tm->tm_mon+1)!=atol(cmdline.getval("month")))
                continue;

            if(cmdline.isset("day"))
              if((tm->tm_mday)!=atol(cmdline.getval("day")))
                continue;

            accumulate(transactions, trans);
           }

          fclose(infile);
         }
       }
     }

  displaysum(transactions, cmdline.isset("formatted"));
 }

