/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: main file of check; tests ZCONNECT files

*/

#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/mman.h>

#include <config.h>
#include <mtoolzc.h>
#include <tools.h>
#include <dboxversion.h>
#include <libdboxversion.h>

void printmessage(zcmessaget *m)
 {
  char tempstr[TEXTLEN];

  m->copyheadertext(tempstr, m->mid, TEXTLEN-1);
  cout << "MID: " << tempstr << "\n";
 }

void checkfile(const char *filename)
 {
  int handle;
  zcmessaget m;
  dword laenge;
  char *messagefile=(char *)NULL;
  bool error=FALSE, mapped=FALSE;

  chdir(config.cwd);
  handle=open(filename, O_RDONLY|O_BINARY);
  config.dboxcd();

  if(handle<0) return;

  laenge=filelength(handle);

  cout << "File size: " << laenge << " Bytes\n\n";

  if(laenge==0) return;

  if(laenge>60000)
   {
    messagefile=(char *)mmap(NULL, laenge, PROT_READ, MAP_FILE|MAP_PRIVATE, handle, 0);
    mapped=TRUE;
    if(messagefile==NULL)
     {
      cerr << "mmap() failed!\n";
      return;
     }
   }

  if(messagefile==NULL)
   {
    messagefile=(char *)malloc(laenge);
    if(messagefile==NULL)
     {
      cerr << "malloc() failed!\n";
      return;
     }

    if(read(handle, messagefile, laenge)!=(signed long)laenge)
     {
      cerr << "read() failed!\n";
      return;
     }
   }

  m.data=messagefile;
  m.endoffile=messagefile+laenge;

  while(!m.eof() && !error)
   {
    if(m.parse() || m.mid==NULL)
      error=TRUE;
    else
     {
      printmessage(&m);
      m.data=m.endofmail;
     }
   }

  if(mapped)
    munmap(messagefile, laenge);
  else
    free(messagefile);

  if(error)
    cout << "Message file has errors!\n";
  else
    cout << "Message file ok!\n";

  close(handle);
 }

static bool parseoptions(int argc, char **argv)
 {
  if(argc<2) goto help;
  if(argc>2) goto aufruf;

  if(argv[1][0]=='-') goto help;

  checkfile(argv[1]);

  return FALSE;

 aufruf:
  cerr << "Falscher Aufruf!\n";

 help:
  cerr << 
    "\n"
    "* * CHECK - DBOX Module for message file checking - Version "
    DBOXVERSIONNR " * *\n"
    "\n"
    "Usage:                       Effect:\n"
    "\n"
    " check [-?] [-h]              display help\n"
    " check filename               check message file\n"
    "\n"
    "Further options:\n"
    "\n";

  return TRUE;
 }

int main(int argc, char **argv)
 {
  if(init_libdbox(LIBDBOXVERSION, LIBDBOXVERSIONNR)) return 1;

  if(config.read("")) return 1;

  parseoptions(argc, argv);

  return 0;
 }
