#include <unistd.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <netdb.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>

#include <string>

#include <dboxpath.h>
#include <config.h>
#include <logf.h>
#include <flags.h>
#include <userdb.h>
#include "../webonline/checkfilename.h"
#include "../online/userlog.h"
#include "../online/ostatus.h"
#include "../webonline/www_prof.h"
#include "../webquota/www_quota.h"
#include "dboxftpd.h"

class dboxftpdt
 {
 public:
  byte failcount;
  bool login, has_homedir;
  int data_fd;
  struct sockaddr_in sa;
  struct sockaddr_in port_sa;
  struct sockaddr_in peer;
  bool passive_socket, port_sa_ok;
  char cwd[DIRLEN], homedir[DIRLEN];
  string alias;

  bool dcon_open();
  void dcon_close();
  void mk_filename(const char *args, char *filename);
  void mk_homedir();
  void dostor(const char *args);
  void dolist(bool nlst, const char *args);
  void docwd(const char *args);
  bool docwd2(const char *args);
  void dolist_file(FILE *outfile, bool nlst, const char *filename);
  bool douser(const char *args);
  bool dopass(const char *args);
  bool dopasv();
  bool doport(const char *args);
  bool docommand(char *command);
  void domkd(const char *args);
  void dodele(const char *args);
  void doit();
 };

void dboxftpdt::mk_filename(const char *args, char *filename)
 {
  if(args[0]=='/')
    snprintf(filename, DIRLEN, "%s%s", homedir, args+1);
  else if(cwd[0]==0)
    snprintf(filename, DIRLEN, "%s%s", homedir, args);
  else
    snprintf(filename, DIRLEN, "%s%s%c%s", homedir, cwd, DIRCHAR[0], args);
 }

void dboxftpdt::mk_homedir()
 {
  char filename[DIRLEN];

  if(has_homedir) return;

  snprintf(filename, DIRLEN, "%s%s", config.homepagedir, alias.c_str());

  if(mkdir(filename, 0755)!=0)
    userlogprintf1(status, "mk_homedir", "Failed to create %s", filename);
  else
    userlogprintf1(status, "mk_homedir", "%s created", filename);

  has_homedir=TRUE;
 }

bool dboxftpdt::dcon_open()
 {
  socklen_t len;
  int s, on, trials;

  if(passive_socket)
   {
    len=sizeof(sa);
    s=accept(data_fd, (struct sockaddr *)&sa, &len);
    if(s<0 || ntohs(sa.sin_port)<IPPORT_RESERVED ||
       sa.sin_addr.s_addr!=peer.sin_addr.s_addr)
     {
      printf("425 accept failed.\r\n");
      return TRUE;
     }

    close(data_fd);
    data_fd=s;
   }
  else
   {
    if(!port_sa_ok)
     {
      userlogprintf1(status, "dcon_open", "Tried to make default active connection.\r\n");
      printf("500 not implemented.\r\n");
      return TRUE;
     }

    data_fd=socket(AF_INET, SOCK_STREAM, 0);
    if(data_fd<0) return TRUE;

    setsockopt(data_fd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));

    trials=3;
   schleife:
    if(connect(data_fd, (struct sockaddr *)&port_sa, sizeof(sa))<0)
     {
      if((errno==EAGAIN || errno==EINPROGRESS) && trials>0)
       {
        trials--;
        sleep(1);
        goto schleife;
       } 
      return TRUE;
     }

    fflush(stdout);
   }

  return FALSE;
 }

void dboxftpdt::dcon_close()
 {
  close(data_fd);
  passive_socket=FALSE;
 }

void dboxftpdt::domkd(const char *args)
 {
  char filename[DIRLEN], *file;

  if(strstr(args, "..")!=NULL)
   {
    printf("504 Not available for security reasons.\r\n");
    return;
   }

  if(!has_homedir)
    mk_homedir();

  mk_filename(args, filename);

  file=strrchr(filename, DIRCHAR[0]);
  if(file==NULL) 
    file=filename;
  else
    file++;

  if(checkfilename(file))
   {
    printf("550 Illegal file name\r\n");
    userlogprintf1(status, "domkd", "Tried to mkdir %s", filename);
    return;
   }

  if(mkdir(filename, 0755)!=0)
   {
    printf("550 Failed to create\r\n");
    userlogprintf1(status, "domkd", "Failed to mkdir %s", filename);
    return;
   }

  printf("257 MKD command successful.\r\n");

  userlogprintf1(status, "domkd", "%s", filename);
 }

void dboxftpdt::dodele(const char *args)
 {
  char filename[DIRLEN], *file;

  if(strstr(args, "..")!=NULL)
   {
    printf("504 Not available for security reasons.\r\n");
    return;
   }

  if(!has_homedir) 
   {
    printf("550 Failed to delete\r\n");
    return;
   }

  mk_filename(args, filename);

  file=strrchr(filename, DIRCHAR[0]);
  if(file==NULL) 
    file=filename;
  else
    file++;

  if(checkfilename(file))
   {
    printf("550 Illegal file name\r\n");
    userlogprintf1(status, "dodele", "Tried to delete %s", filename);
    return;
   }

  wwwpt wwwp;
  wwwp.setname(alias);
  if(!wwwp.lock())
   {
    www_quota(wwwp, filename, FALSE);
    wwwp.write();
    wwwp.unlock();
   }

  if(unlink(filename)!=0)
   {
    if(rmdir(filename)!=0)
     {
      printf("550 Failed to delete\r\n");
      userlogprintf1(status, "dodele", "Failed to delete %s", filename);
      return;
     }
   }

  printf("250 DELE command successful.\r\n");

  userlogprintf1(status, "dodele", "DELE: %s", filename);
 }

bool dboxftpdt::doport(const char *args)
 {
  char *a, *p;
  int h1, h2, h3, h4, p1, p2;
  socklen_t len;

  if(passive_socket)
   {
    close(data_fd);
    passive_socket=FALSE;
   }

  len=sizeof(port_sa);
  if(getsockname(0, (struct sockaddr *)&port_sa, &len)<0)
    return TRUE;

  a=(char *)&port_sa.sin_addr.s_addr;
  p=(char *)&port_sa.sin_port;

  sscanf(args, "%d,%d,%d,%d,%d,%d", &h1, &h2, &h3, &h4, &p1, &p2);

  a[0]=h1&0xff;
  a[1]=h2;
  a[2]=h3;
  a[3]=h4;
  p[0]=p1;
  p[1]=p2;

  if(port_sa.sin_addr.s_addr!=peer.sin_addr.s_addr ||
     ntohs(port_sa.sin_port)<IPPORT_RESERVED)
   {
    printf("500 Illegal PORT Command %s %d,%d,%d,%d,%d,%d\r\n",
      args, h1, h2, h3, h4, p1, p2);
    port_sa_ok=FALSE;
   }
  else
   {
    printf("200 PORT command successful.\r\n");
    port_sa_ok=TRUE;
   }

  return FALSE;
 }

bool dboxftpdt::docwd2(const char *args)
 {
  char *tptr, *file;

  if(args[0]==0 || strcmp(args, ".")==0)
   {
   }
  else if(strcmp(args, "/")==0)
    cwd[0]=0;
  else if(strcmp(args, "..")==0)
   {
    if(cwd[0]==0) return TRUE;

    tptr=strrchr(cwd, DIRCHAR[0]);
    if(tptr!=NULL)
      *tptr=0;
    else
      cwd[0]=0;
   }
  else if(args[0]=='/') 
   {
    cwd[0]=0;
    return docwd2(args+1);
   }
  else
   {
    tptr=strchr(args, '/');
    if(tptr!=NULL)
     {
      char tempstr[DIRLEN];

      strmaxcpy(tempstr, args, DIRLEN-1);
      tptr=strchr(tempstr, '/');
      if(tptr!=NULL)
       {
        *tptr=0;
        if(docwd2(tempstr)) return TRUE;
        return docwd2(tptr+1);
       }
     }
    else
     {
      struct stat st;
      char filename[DIRLEN];

      mk_filename(args, filename);

      file=strrchr(filename, DIRCHAR[0]);
      if(file==NULL) 
        file=filename;
      else
        file++;

      if(checkfilename(file))
        return TRUE;

      if(stat(filename, &st)<0 || !S_ISDIR(st.st_mode))
        return TRUE;

      if(cwd[0]==0)
        strmaxcpy(cwd, file, DIRLEN-1);
      else
        snprintf(cwd+strlen(cwd), DIRLEN-strlen(cwd), DIRCHAR "%s", file);
     }
   }

  return FALSE;
 }

void dboxftpdt::docwd(const char *args)
 {
  if(docwd2(args))
    printf("550 %s No such file or directory.\r\n", args);
  else
    printf("250 CWD command successfull.\r\n");
 }

void dboxftpdt::dostor(const char *args)
 {
  struct stat st;
  char filename[DIRLEN], *file;
  int fd, count;
  char buf[4096];
  dword size=0;

  if(strstr(args, "..")!=NULL)
   {
    printf("504 Not available for security reasons.\r\n");
    return;
   }

  if(!has_homedir)
    mk_homedir();

  mk_filename(args, filename);

  file=strrchr(filename, DIRCHAR[0]);
  if(file==NULL) 
    file=filename;
  else
    file++;

  if(checkfilename(file))
   {
    printf("550 Illegal file name\r\n");
    userlogprintf1(status, "dostor", "Tried to store %s", filename);
    return;
   }

  if(stat(filename, &st)>=0 && !S_ISREG(st.st_mode))
   {
    printf("550 Not a regular file\r\n");
    userlogprintf1(status, "dostor", "Tried to store %s", filename);
    return;
   }

  wwwpt wwwp;
  wwwp.setname(alias);
  if(access(filename, 00)==0)
    if(!wwwp.lock())
     {
      www_quota(wwwp, filename, FALSE);
      wwwp.write();
      wwwp.unlock();
     }

  fd=open(filename, O_WRONLY|O_CREAT|O_NOCTTY|O_TRUNC|O_BINARY, 0644);
  if(fd<0)
   {
    printf("550 Failed to open\r\n");
    userlogprintf1(status, "dostor", "Failed to store %s", filename);
    return;
   }

  printf("150 Opening BINARY mode data connection for %s.\r\n", args);
  fflush(stdout);

  if(dcon_open()) return;

  while((count=read(data_fd, buf, sizeof(buf)))>0)
   {
    if(size<config.homepagemaxfilesize)
     {
      write(fd, buf, count);
      if(count>0) size+=count;
     }
   }

  close(fd);

  if(count<0)
    printf("426 Data Connection\r\n");
  else
    printf("226 Transfer complete.\r\n");

  if(!wwwp.lock())
   {
    www_quota(wwwp, filename, TRUE);
    wwwp.write();
    wwwp.unlock();
   }

  userlogprintf1(status, "dostor", "Stored %s (%lu Bytes)", filename, size);

  dcon_close();
 }

void dboxftpdt::dolist_file(FILE *outfile, bool nlst, const char *filename)
 {
  const char *file; 
  struct stat st;
  struct tm *tm;
  char date[100];
  time_t t;
  int year;

  file=strrchr(filename, DIRCHAR[0]);
  if(file==NULL) 
    file=filename;
  else
    file++;

  if(strcmp(file, "profile")==0) return;

  if(!nlst)
   {
    if(stat(filename, &st)>=0 && 
       (S_ISREG(st.st_mode) || S_ISDIR(st.st_mode)))
     {
      if(S_ISDIR(st.st_mode))
        fprintf(outfile, "drwxr-xr-x");
      else
        fprintf(outfile, "-rw-r--r--");

      time(&t);
      tm=localtime(&t);
      year=tm->tm_year;
      tm=localtime(&st.st_mtime);
      strftime(date, TEXTLEN-1, "%b %d ", tm);
      strftime(date+strlen(date), TEXTLEN-1,  
        (tm->tm_year==year)?"%H:%M":" %Y", tm);

      fprintf(outfile, "%4d %-8s %-8s %8lu %12s %s\r\n", 
        st.st_nlink, status.username, "none", st.st_size, date, file);
     }
   }
  else
   {
    if(stat(filename, &st)>=0 && S_ISREG(st.st_mode))
      fprintf(outfile, "%s\r\n", file);
   }
 }

static int sort_string(const void *a, const void *b)
 {
  const char *aa, *bb;

  aa=*(const char **)a;
  bb=*(const char **)b;

  return stricmp(aa, bb);
 }

#define MAXFILES 10000

void dboxftpdt::dolist(bool nlst, const char *args)
 {
  struct stat st;
  DIR *dirp;
  struct dirent *dir;
  FILE *outfile;
  char filename[DIRLEN], tempstr[DIRLEN*10];
  char *files[MAXFILES];
  dword anzfiles=0, i;

  if(strstr(args, "..")!=NULL)
   {
    printf("504 Not available for security reasons.\r\n");
    return;
   }

  printf("150 Opening data connection.\r\n");
  fflush(stdout);

  if(dcon_open()) return;

  if(has_homedir)
   {
    outfile=fdopen(data_fd, "wt");
    if(outfile==NULL) return;

    if(args[0]=='-')
     {
      nlst=FALSE;
      args="";
     }

    mk_filename(args, filename);

    if(stat(filename, &st)>=0)
     {
      if(S_ISREG(st.st_mode))
       {
        dolist_file(outfile, nlst, filename);
       }
      else if(S_ISDIR(st.st_mode))
       {
        if((dirp=opendir(filename))!=NULL)
         {
          while((dir=readdir(dirp))!=NULL && anzfiles<MAXFILES)
           {
            snprintf(tempstr, DIRLEN, "%s%c%s", filename, DIRCHAR[0], dir->d_name);
            files[anzfiles++]=strdup(tempstr);
           }
	
          closedir(dirp);
         }
       }
     }
    else
     {

     }

    qsort(files, anzfiles, sizeof(char *), sort_string);

    for(i=0; i<anzfiles; i++)
     {
      dolist_file(outfile, nlst, files[i]);
      free(files[i]);
     }

    fclose(outfile);
   }

  printf("226 Transfer complete.\r\n");

  dcon_close();
 }

bool dboxftpdt::dopasv()
 {
  char *a, *p;
  socklen_t len;

  if(passive_socket)
   {
    close(data_fd);
    passive_socket=FALSE;
   }

  data_fd=socket(AF_INET, SOCK_STREAM, 0);
  if(data_fd<0) return TRUE;

  len=sizeof(sa);
  if(getsockname(0, (struct sockaddr *)&sa, &len)<0)
    return TRUE;

  sa.sin_port=htons(0);

  if(bind(data_fd, (struct sockaddr *)&sa, sizeof(sa))<0)
    return TRUE;

  if(getsockname(data_fd, (struct sockaddr *)&sa, &len)<0)
    return TRUE;

  if(listen(data_fd, 1)<0)
    return TRUE;

  a=(char *)&sa.sin_addr.s_addr;
  p=(char *)&sa.sin_port;

  printf("227 Entering Passive Mode (%d,%d,%d,%d,%d,%d)\r\n",
    a[0]&0xff, a[1]&0xff, a[2]&0xff, a[3]&0xff, 
    p[0]&0xff, p[1]&0xff);

  passive_socket=TRUE;

  return FALSE;
 }

bool dboxftpdt::douser(const char *args)
 {
  if(login)
    printf("530 Already logged in.\r\n");
  else
   {
    strmaxcpy(status.userprofile.name, args, TEXTLEN-1);
    dnstring(status.userprofile.name); 
    printf("331 Password required.\r\n");
   }

  return FALSE;
 }

bool dboxftpdt::dopass(const char *args)
 {
  if(login)
    printf("530 Already logged in.\r\n");
  else
   {
    if(status.userprofile.name[0]==0)
      printf("503 Please login first.\r\n");
    else if(stricmp(status.userprofile.name, "ftp")==0 || 
            stricmp(status.userprofile.name, "anonymous")==0)
     {
      printf("503 No anonymous access allowed.\r\n");
     }
    else
     {
      if(status.userprofile.read())
       {
        sleep(1);
        logf.printf("dboxftpd", "Login %s failed", status.userprofile.name);
        printf("503 Login incorrect.\r\n");
        login=FALSE;
        failcount--;
        if(failcount==0)
         {
          printf("221 Goodbye.\r\n");
          return TRUE;
         }
       }
      else
       {
        if((status.userprofile|"alias")[0]!=0)
         {
          alias=status.userprofile.name;
          strmaxcpy(status.userprofile.name, status.userprofile|"alias", TEXTLEN-1);

          if(status.userprofile.read())
           {
            sleep(1);
            logf.printf("dboxftpd", "Fatal: User %s for alias %s not found",
              status.userprofile.name, alias.c_str());

            printf("503 Login incorrect.\r\n");
            printf("221 Goodbye.\r\n");
            return TRUE;
           }
         }
        else
          alias=status.userprofile.name;

        strmaxcpy(status.username, status.userprofile.name, TEXTLEN-1);
        snprintf(status.userlogfile, DIRLEN, "%slogfile", status.userprofile.getpath());

        if(verifylogin(&status.userprofile, args))
         {
          sleep(1);
          userlogprintf1(status, "dboxftpd", "Login failed");
          printf("503 Login incorrect.\r\n");
          login=FALSE;
          failcount--;
          if(failcount==0)
           {
            printf("221 Goodbye.\r\n");
            return TRUE;
           }

          return FALSE;
         }

        strmaxcpy(status.gruppe.name, status.userprofile|"gruppe", TEXTLEN-1);

        status.gruppe.read();

        if(!getinternalflag(&status.userprofile, &status.gruppe, LOGIN_WEB))
         {
          userlogprintf1(status, "dboxftpd", "Permission denied");
          printf("211 You have no permission.\r\n");
          return TRUE;
         }
        else
         {
          char filename[DIRLEN];

          userlogprintf1(status, "dboxftpd", "Login successful, alias %s",
            alias.c_str());

          printf("230 User %s logged in.\r\n", status.userprofile.name);
          login=TRUE;
          
          snprintf(homedir, DIRLEN, "%s%s%c", config.homepagedir, 
            alias.c_str(), DIRCHAR[0]);

          snprintf(filename, DIRLEN, "%s.", homedir);
          has_homedir=(access(filename, 00)==0);
         }
       }
     }
   }

  return FALSE;
 }

bool dboxftpdt::docommand(char *command)
 {
  char *tptr, *args;

  tptr=strchr(command, ' ');
  if(tptr==NULL)
    args="";
  else
   {
    *tptr=0;
    args=tptr+1;
   }

  if(stricmp(command, "USER")==0)
   {
    if(douser(args)) return TRUE;
   }
  else if(stricmp(command, "PASS")==0)
   {
    if(dopass(args)) return TRUE;
   }
  else if(stricmp(command, "QUIT")==0)
   {
    printf("221 Goodbye.\r\n");
    if(login)
      userlogprintf1(status, "dboxftpd", "%s/%s: Logout", status.username,
        alias.c_str());
    else
      logf.printf("dboxftpd", "Logout");

    return TRUE;
   }
  else if(stricmp(command, "PORT")==0)
   {
    if(login)
     {
      if(doport(args))
       {
        printf("425 Internal error\r\n");
        return TRUE;
       }
     }
    else
      printf("530 Please login first.\r\n");
   }
  else if(stricmp(command, "PASV")==0)
   {
    if(login)
     {
      if(dopasv())
       {
        printf("425 Internal error\r\n");
        return TRUE;
       }
     }
    else
      printf("530 Please login first.\r\n");
   }
  else if(stricmp(command, "TYPE")==0)
   {
    printf("200 Type is ignored\r\n");
   }
  else if(stricmp(command, "STRU")==0)
   {
    printf("200 Structure file is assumed\r\n");
   }
  else if(stricmp(command, "MODE")==0)
   {
    if(toupper(args[0])=='S')
      printf("200 MODE S ok.\r\n");
    else
     {
      printf("502 Unimplemented MODE type.\r\n");
      logf.printf("dboxftpd", "MODE %s not implemented", args);
     }
   }
  else if(stricmp(command, "RETR")==0)
   {
    if(login)
     {
      userlogprintf1(status, "dboxftpd", "RETR not implemented");
      printf("502 not implemented\r\n");
     }
    else
      printf("530 Please login first.\r\n");
   }
  else if(stricmp(command, "STOR")==0)
   {
    if(login)
      dostor(args);
    else
      printf("530 Please login first.\r\n");
   }
  else if(stricmp(command, "APPE")==0)
   {
    if(login)
     {
      userlogprintf1(status, "dboxftpd", "APPE not implemented");
      printf("502 not implemented\r\n");
     }
    else
      printf("530 Please login first.\r\n");
   }
  else if(stricmp(command, "ALLO")==0)
   {
    printf("200 nooping...\r\n");
   }
  else if(stricmp(command, "REST")==0)
   {
    if(login)
     {
      userlogprintf1(status, "dboxftpd", "REST not implemented");
      printf("502 not implemented\r\n");
     }
    else
      printf("530 Please login first.\r\n");
   }
  else if(stricmp(command, "RNFR")==0)
   {
    if(login)
     {
      userlogprintf1(status, "dboxftpd", "RNFR not implemented");
      printf("502 not implemented\r\n");
     }
    else
      printf("530 Please login first.\r\n");
   }
  else if(stricmp(command, "RNTO")==0)
   {
    if(login)
     {
      userlogprintf1(status, "dboxftpd", "RNTO not implemented");
      printf("502 not implemented\r\n");
     }
    else
      printf("530 Please login first.\r\n");
   }
  else if(stricmp(command, "ABOR")==0)
   {
    if(login)
     {
      userlogprintf1(status, "dboxftpd", "ABOR not implemented");
      printf("502 not implemented\r\n");
     }
    else
      printf("530 Please login first.\r\n");
   }
  else if(stricmp(command, "DELE")==0)
   {
    if(login)
      dodele(args);
    else
      printf("530 Please login first.\r\n");
   }
  else if(stricmp(command, "RMD")==0)
   {
    if(login)
      dodele(args);
    else
      printf("530 Please login first.\r\n");
   }
  else if(stricmp(command, "MKD")==0)
   {
    if(login)
      domkd(args);
    else
      printf("530 Please login first.\r\n");
   }
  else if(stricmp(command, "PWD")==0)
   {
    if(login)
      printf("257 \"/%s\" is current directory\r\n", cwd);
    else
      printf("530 Please login first.\r\n");
   }
  else if(stricmp(command, "LIST")==0)
   {
    if(login)
      dolist(FALSE, args);
    else
      printf("530 Please login first.\r\n");
   }
  else if(stricmp(command, "NLST")==0)
   {
    if(login)
      dolist(TRUE, args);
    else
      printf("530 Please login first.\r\n");
   }
  else if(stricmp(command, "SITE")==0)
   {
    if(login)
     {
      userlogprintf1(status, "dboxftpd", "SITE not implemented");
      printf("502 not implemented\r\n");
     }
    else
      printf("530 Please login first.\r\n");
   }
  else if(stricmp(command, "SYST")==0)
   {
    printf("215 UNIX Type: L8\r\n");
   }
  else if(stricmp(command, "STAT")==0)
   {
    userlogprintf1(status, "dboxftpd", "STAT not implemented");
    printf("502 not implemented\r\n");
   }
  else if(stricmp(command, "HELP")==0)
   {
    printf("214 Sorry, no help\r\n");
   }
  else if(stricmp(command, "CDUP")==0)
   {
    if(login)
      docwd("..");
    else
      printf("530 Please login first.\r\n");
   }
  else if(stricmp(command, "CWD")==0)
   {
    if(login)
      docwd(args);
    else
      printf("530 Please login first.\r\n");
   }
  else if(stricmp(command, "NOOP")==0)
   {
    printf("200 nooping...\r\n");
   }
  else
   {
    printf("502 not implemented\r\n");
   }

  return FALSE;
 }

void ftpalarm(int signal)
 {
  printf("221 Timeout\r\n");
  exit(1);
 }

void dboxftpdt::doit()
 {
  char command[TEXTLEN], *tptr;

  if(config.homepagedir[0]==0)
   {
    printf("502 Service not available\r\n");
    return;
   }

  printf("220-All transfers are logged.\r\n");
  printf("220-\r\n");
  printf("220 %s.%s DBOX FTP Server ready\r\n", config.boxname, config.domain);

  status.userprofile.name[0]=0;
  status.username[0]=0;
  alias="";
  failcount=3;
  login=has_homedir=FALSE;
  passive_socket=FALSE;
  port_sa_ok=FALSE;
  strmaxcpy(cwd, "", DIRLEN-1);
   
   {
    socklen_t len;

    len=sizeof(peer);
    if(getpeername(0, (struct sockaddr *)&peer, &len)<0)
      return;
   }

  fflush(stdout);

  signal(SIGALRM, ftpalarm);

  while(TRUE)
   {
    alarm(10*60);
    if(fgets(command, 90, stdin)==NULL) break;
    alarm(0);

    tptr=strchr(command, '\r'); if(tptr!=NULL) *tptr=0;
    tptr=strchr(command, '\n'); if(tptr!=NULL) *tptr=0;

    if(docommand(command)) return;

    fflush(stdout);
   }
 }

void dboxftpd()
 {
  dboxftpdt dboxftp;

  dboxftp.doit();
 }
