/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>

#include <strstream.h>
#include <fstream.h>

#include <sfolderrfc.h>
#include <stringtools.h>
#include "dboximap4.h"

void dboximap4t::doappend2()
 {
  if(fp.lock())
   {
    cout << tag << " NO failed to lock folder\r\n";
    return; 
   }

  dword nr;

  if(spool_in_folder(rfcm, fp, FALSE, "", nr))
   {
    fp.unlock();
    cout << tag << " NO internal error\r\n";
    return;
   }

  folderindext fi;
  char filename[DIRLEN];

  snprintf(filename, DIRLEN, "%sindex", fp.getpath());
  fstream index(filename, ios::in|ios::out|ios::bin);

  if(!index)
   {
    fp.unlock();
    cout << tag << " NO internal error 1\r\n";
    return;
   }

  index.seekp(-(signed)sizeof(fi), ios::end);
  streampos pos=index.tellp();
  if(!index.read(&fi, sizeof(fi)))
   {
    fp.unlock();
    cout << tag << " NO internal error 2 " << pos << "\r\n";
    return;
   }

  unsigned argnr=1;
  if(arguments[argnr].begin_list) // Flags
   {
    for(argnr++; argnr<arguments.size() && 
        !arguments[argnr].end_list; argnr++)
     {
      const char *item=arguments[argnr].data.c_str();
      if(stricmp(item, "\\Answered")==0)
        fi.flags.imapflags|=DBOXIMAP4_FLAG_Answered;
      else if(stricmp(item, "\\Flagged")==0)
        fi.flags.imapflags|=DBOXIMAP4_FLAG_Flagged;
      else if(stricmp(item, "\\Deleted")==0)
        fi.flags.deleted=TRUE;
      else if(stricmp(item, "\\Seen")==0)
        fi.flags.seen=TRUE;
      else if(stricmp(item, "\\Draft")==0)
        fi.flags.imapflags|=DBOXIMAP4_FLAG_Draft;
     }
   }

  if(!arguments[argnr].literal) // internaldate
    fi.received=parse_date(arguments[argnr].data);

  index.seekp(pos, ios::beg);
  index.write(&fi, sizeof(fi));

  // APPENDUID: RFC2359
  dword uidvalidity=atol(fp|"imap.uidvalidity");
  if(uidvalidity==0) uidvalidity=1;

  fp.unlock();

  cout << tag << " OK [APPENDUID " << uidvalidity << " " 
       << fi.id << "] APPEND completed\r\n";
 }

void dboximap4t::doappend()
 {
  int args=arguments.size();

  command_ok=FALSE;

  if(args<2)
   {
    cout << tag << " NO give arguments\r\n";
    return; 
   }

  string mailbox_name=arguments[0].data, folder_owner, subfolder;

  if(process_mailbox_name(mailbox_name, folder_owner, subfolder))
    return;

  folder_grantst grants;
  ::get_folder_grants(folder_owner, subfolder, up, gp, grants);
  if(!grants.insert)
   {
    cout << tag << " NO permission denied\r\n";
    return;
   }

  string folder_name=subfolder+"@"+folder_owner;

  folderpt fp(folder_name.c_str());

   {
    size_t size=arguments[args-1].data.size();
    rfcm.data=new char[size];
    memcpy((char *)rfcm.data, arguments[args-1].data.data(), size);
    rfcm.endoffile=rfcm.data+size;
    rfcm.endofmail=rfcm.endoffile;

    if(rfcm.parse() || rfcm.headerlen==0)
     {
      cout << tag << " NO invalid message\r\n";
     }

    doappend2();

    delete rfcm.data;
   }
 }

