/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: COPY command

*/

#include <unistd.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

#include <fstream.h>

#include <stringtools.h>
#include <sfolderrfc.h>
#include "imap4copy.h"

bool copy_set_commandt::start()
 {
  string mailbox_name=dboximap4.arguments[argnr].data, 
         folder_owner, subfolder;

  if(dboximap4.process_mailbox_name(mailbox_name, folder_owner, subfolder))
    return TRUE;

  folder_grantst grants;
  get_folder_grants(folder_owner, subfolder, dboximap4.up, dboximap4.gp,
                    grants);
  if(!grants.insert)
   {
    cout << dboximap4.tag << " NO permission denied\r\n";
    dboximap4.command_ok=FALSE;
    return TRUE;
   }

  string folder_name=subfolder+"@"+folder_owner;

  new_fp.setname(folder_name);

  if(new_fp.lock())
   {
    cout << dboximap4.tag << " NO failed to lock folder\r\n";
    dboximap4.command_ok=FALSE;
    return TRUE; 
   }

  return FALSE;
 }

bool copy_set_commandt::msg()
 {
  if(dboximap4.get_body(nr-1))
   {
    cout << "* " << nr << " ERROR: failed to get message\r\n";
    return FALSE;
   }

  if(dboximap4.rfcm.parse() || 
     dboximap4.rfcm.headerlen==0)
   {
    cout << "* " << nr << " ERROR: invalid message\r\n";
    return FALSE;
   }

  if(spool_in_folder(dboximap4.rfcm, new_fp, FALSE, "", nr))
   {
    cout << dboximap4.tag << " NO internal error\r\n";
    dboximap4.command_ok=FALSE;
    return TRUE;
   }

  folderindext new_fi;

  string filename=new_fp.getpath();
  filename+="index";
  fstream index(filename.c_str(), ios::in|ios::out|ios::bin);

  if(!index)
   {
    cout << dboximap4.tag << " NO internal error 1\r\n";
    dboximap4.command_ok=FALSE;
    return TRUE;
   }

  index.seekp(-(signed)sizeof(new_fi), ios::end);
  streampos pos=index.tellp();
  if(!index.read(&new_fi, sizeof(new_fi)))
   {
    cout << dboximap4.tag << " NO internal error 2 " << pos << "\r\n";
    dboximap4.command_ok=FALSE;
    return TRUE;
   }

  new_fi.flags.imapflags=dboximap4.fi.flags.imapflags;
  new_fi.flags.deleted=dboximap4.fi.flags.deleted;
  new_fi.flags.seen=dboximap4.fi.flags.seen;
  new_fi.received=dboximap4.fi.received;

  index.seekp(pos, ios::beg);
  index.write(&new_fi, sizeof(new_fi));

  if(copyuid!="") copyuid+=",";
  copyuid+=i2string(unsigned(new_fi.id));

  return FALSE;
 }

void copy_set_commandt::end()
 {
  // COPYUID: RFC2359
  dword uidvalidity=atol(new_fp|"imap.uidvalidity");
  if(uidvalidity==0) uidvalidity=1;

  new_fp.unlock();

  cout << dboximap4.tag << " OK [COPYUID " << uidvalidity << " " 
       << copyuid << "] COPY completed\r\n";

  dboximap4.command_ok=FALSE;
 }
