/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: rename a folder

*/

#include <unistd.h>
#include <sys/stat.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

#include <fstream.h>

#include <stringtools.h>
#include <dboxpath.h>
#include <lock.h>
#include "mod-utf7.h"
#include "dboximap4.h"

void dboximap4t::dorename()
 {
  if(arguments.size()!=2)
   {
    cout << tag << " NO give two mailbox names\r\n";
    command_ok=FALSE;
    return;
   }

  string old_mailbox_name=decode_mod_utf7(arguments[0].data),
         new_mailbox_name=decode_mod_utf7(arguments[1].data);

  if(old_mailbox_name=="" ||
     new_mailbox_name=="")
   {
    cout << tag << " NO give a non-empty mailbox names\r\n";
    command_ok=FALSE;
    return;
   }
  
  if(stringcasecmp(old_mailbox_name, "INBOX")==0)
   {
    cout << tag << " NO I won't rename INBOX\r\n";
    command_ok=FALSE;
    return;
   }
  
  if(stringcasecmp(new_mailbox_name, "INBOX")==0)
   {
    cout << tag << " NO folder already exists\r\n";
    command_ok=FALSE;
    return;
   }
  
  if(check_mailbox_name(new_mailbox_name))
   {
    cout << tag << " NO new mailbox name is not valid\r\n";
    command_ok=FALSE;
    return;
   }

  if(lock("folders", up.name, config.boxname))
   {
    cout << tag << " NO folder list is locked\r\n";
    command_ok=FALSE;
    return;
   }

  get_folder_list();

  unsigned i;
  for(i=0; i<folders.size(); i++)
    if(new_mailbox_name==folders[i].name)
     {
      unlock("folders", up.name, config.boxname);
      cout << tag << " NO mailbox already exists\r\n";
      command_ok=FALSE;
      return;
     }

  for(i=0; i<folders.size(); i++)
    if(old_mailbox_name==folders[i].name)
     {
      folders[i].name=new_mailbox_name;
      break;
     }

  if(i==folders.size())
   {
    unlock("folders", up.name, config.boxname);
    cout << tag << " NO no such mailbox\r\n";
    command_ok=FALSE;
    return;
   }

   {
    char outfilename[DIRLEN];
    snprintf(outfilename, DIRLEN, "%sfolders", up.getpath());
    ofstream out(outfilename, ios::out);

    for(i=0; i<folders.size(); i++)
      out << folders[i].name << "\n";
   }

  unlock("folders", up.name, config.boxname);
 }
