/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: IMAP4 STATUS command

*/

#include <unistd.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <stringtools.h>
#include "dboximap4.h"

void dboximap4t::dostatus()
 {
  if(arguments.size()<1)
   {
    cout << tag << " NO give a mailbox name\r\n";
    command_ok=FALSE;
    return; 
   }

  string mailbox_name=arguments[0].data, folder_owner, subfolder;

  if(process_mailbox_name(mailbox_name, folder_owner, subfolder))
    return;

  folder_grantst grants;
  ::get_folder_grants(folder_owner, subfolder, up, gp, grants);
  if(!grants.read)
   {
    cout << tag << " NO permission denied\r\n";
    command_ok=FALSE;
    return;
   }

  string folder_name=subfolder+"@"+folder_owner;
  folderpt fp;
  fp.setname(folder_name);
  fp.read();

  string indexfilename=fp.getpath();
  indexfilename+="index";

  int index=open(indexfilename.c_str(), O_BINARY|O_RDONLY);
  dword messages_in_folder=0, recent=0, unseen=0;

  if(index>=0)
   {
    while(read(index, &fi, sizeof fi)==sizeof fi)
     {
      messages_in_folder++;
      if(!fi.flags.seen) unseen++;
     }

    ::close(index);
   }

  cout << "* STATUS ";
  send_string(arguments[0].data);
  cout << ' ';

  begin_list();

  for(unsigned argnr=1; argnr<arguments.size(); argnr++)
   {
    if(stringcasecmp(arguments[argnr].data, "MESSAGES")==0)
     {
      list_item(); cout << "MESSAGES";
      list_item(); cout << messages_in_folder;
     }
    else if(stringcasecmp(arguments[argnr].data, "RECENT")==0)
     {
      list_item(); cout << "RECENT";
      list_item(); cout << recent;
     }
    else if(stringcasecmp(arguments[argnr].data, "UIDNEXT")==0)
     {
      dword nextid=atol(fp|"nextid");
      if(nextid==0) nextid=1;
      fp.read();
      list_item(); cout << "UIDNEXT";
      list_item(); cout << nextid;
     }
    else if(stringcasecmp(arguments[argnr].data, "UIDVALIDITY")==0)
     {
      dword uidvalidity=atol(fp|"imap.uidvalidity");
      if(uidvalidity==0) uidvalidity=1;
      list_item(); cout << "UIDVALIDITY";
      list_item(); cout << uidvalidity;
     }
    else if(stringcasecmp(arguments[argnr].data, "UNSEEN")==0)
     {
      list_item(); cout << "UNSEEN";
      list_item(); cout << unseen;
     }
   }

  end_list();

  cout << "\r\n";
 }
