/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: base class for remote mailbox access

*/

#include <unistd.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <fcntl.h>
#include <signal.h>

#include <tools.h>
#include <logf.h>
#include <flags.h>
#include <dboxpath.h>
#include <getpeer.h>
#include "pop.h"

char peer[TEXTLEN];

dboxpopt::dboxpopt()
 {
  transfer=FALSE;
  protocol="unknown";
  timeout_minutes=10;
 }

bool dboxpopt::getcommand()
 {
  char *tptr;
  char line[1000];

  alarm(timeout_minutes*60);
  fflush(stdout);
  if(fgets(line, 900, stdin)==NULL) return TRUE; 
  alarm(0);

  tptr=strchr(line, '\r'); if(tptr!=NULL) *tptr=0;
  tptr=strchr(line, '\n'); if(tptr!=NULL) *tptr=0;

  commandline=line;
 
  return FALSE;
 }

bool dboxpopt::login(const string &password)
 {
  if(up.read())
   {
    logf.printf("login", "Invalid user name %s from %s", up.name, peer);
    return TRUE;
   }

  snprintf(userlogfile, DIRLEN-1, "%slogfile", up.getpath());

  if(verifylogin(&up, password.c_str()))
   {
    logf.fileprintf(userlogfile, "login", 
                    "wrong %s password from %s", protocol.c_str(), peer);
    logf.printf("login", "%s: wrong password from %s", up.name, peer);
    return TRUE;
   }

  return login2();
 }

bool dboxpopt::login2()
 {
  userpt up2;
  get_profiles(up, up2, gp);

  if(!get_internalflag(up, up2, gp, LOGIN_POP3))
   {
    logf.fileprintf(userlogfile, "login", "%s access denied", protocol.c_str());
    logf.printf("login", "%s: access denied", up.name);
    return TRUE;
   }

  logf.fileprintf(userlogfile, "login", "Successfull %s login from %s",
    protocol.c_str(), peer);

  logf.printf("login", "%s: Successfull %s login from %s", 
    up.name, protocol.c_str(), peer);

  transfer=TRUE;

  return FALSE;
 }

void dboxpopt::getpeer()
 {
  string peer_string;

  ::getpeer(peer_string);

  strmaxcpy(peer, peer_string.c_str(), TEXTLEN-1);
 }

void dboxpopt::docrunch(const char *username)
 {
  char filename[DIRLEN];
  FILE *outfile;

  snprintf(filename, DIRLEN, BOXTODODIR "crunch.%s", config.boxdir, username);
  outfile=fopen(filename, "wt");
  if(outfile!=NULL) fclose(outfile);
 }

