#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <fcntl.h>

#include <dbox.h>
#include <logins.h>
#include <flags.h>
#include <portinfo.h>
#include <libdboxversion.h>
#include <dboxpath.h>
#include <logf.h>
#include <account.h>

#undef DEBUG
#include "radius.h"

#include "dboxradius.h"
#include "dbox_misc.h"

extern "C" VALUE_PAIR *get_attribute(VALUE_PAIR *value_list, int attribute);

class dboxaccountingt
 {
 public:
  userpt up;
  clientpt cp;
  long int port, dauer, delay_time;
  bool stop, keepalive;
  struct userinfot userinfo;

  char protocol[TEXTLEN], ip[100], 
       charge_menu[TEXTLEN], userlogfile[DIRLEN],
       sessionid[TEXTLEN], from[TEXTLEN],
       called_station[TEXTLEN], boxname[TEXTLEN],
       errormsg[TEXTLEN], login[TEXTLEN];

  void dboxaccounting(const char *pm, VALUE_PAIR *pair);
  void getnasport(portinfot *portinfo);
  void startreq(portinfot *portinfo);
  void keepalivereq(portinfot *portinfo);
  void stopreq(portinfot *portinfo);
  void processaccounting();
 };

void dboxaccountingt::getnasport(portinfot *portinfo)
 {
  long int oldport;
  word first_free_port=MAXPORTS;

  oldport=port;

  port=0;
  sprintf(sessionid+strlen(sessionid), "%ld", oldport);

  do
   {
    port++;
    portinfo->read(port);
    if(portinfo->info.status==portinfot::infot::OFFLINE &&
       first_free_port==MAXPORTS)
       first_free_port=port;
   }
  while(stricmp(portinfo->info.sessionid, sessionid)!=0 && port<MAXPORTS);

  if(port==MAXPORTS)
   {
    if(first_free_port==MAXPORTS)
     {
      logf.printf("dboxradius::accounting", 
        "radius.ignore-NAS-Port: kein freier Port", oldport, port);
      port=0;
     }
    else
     {
      port=first_free_port;
     }
   }

  logf.printf("dboxradius::accounting", 
    "radius.ignore-NAS-Port: %d -> %d", oldport, port);
 }

void dboxaccountingt::stopreq(portinfot *portinfo)
 {
  char tempstr[TEXTLEN*2];

  if(portinfo->owninfo.status==portinfot::infot::SLIP ||
     portinfo->owninfo.status==portinfot::infot::NETCALL ||
     portinfo->owninfo.status==portinfot::infot::ONLINE)
   {
    portinfo->owninfo.status=portinfot::infot::OFFLINE;

    strmaxcpy(portinfo->owninfo.modemname, "", TEXTLEN-1);
    strmaxcpy(portinfo->owninfo.telefon, "", TEXTLEN-1);
    strmaxcpy(portinfo->owninfo.username, "", TEXTLEN-1);
    strmaxcpy(portinfo->owninfo.from, "", TEXTLEN-1);
    strmaxcpy(portinfo->owninfo.boxname, boxname, TEXTLEN-1);
    portinfo->owninfo.lastcall=time(NULL);
    portinfo->owninfo.slipaddr=0;

    portinfo->write(port);

    sprintf(tempstr, "%s-Logout, Verbindungsdauer: %ld Sekunden"
      " (%s Port %ld)", protocol, dauer, cp.name, port);
    logf.fileprintf(userlogfile, "dboxradius", tempstr);

    do_check_login2(cp, up, errormsg,
                    protocol, charge_menu, &userinfo);

    if(charge_menu[0]!=0)
     {
      char zweck[TEXTLEN];
      time_t t;

      time(&t);

      snprintf(zweck, TEXTLEN, "R-%s Zeitgebhr", protocol);
      chargemenu(&up, charge_menu, zweck, t-dauer, t);
     }
   }
  else
   {
    sprintf(tempstr, "%s-Logout ignoriert (%s, %ld, %s, %s)",
      protocol, up.name, port, cp.name, sessionid);
    logf.printf("dboxradius::accounting", tempstr);
    logf.fileprintf(userlogfile, "dboxradius", tempstr);
   }
 }

void dboxaccountingt::keepalivereq(portinfot *portinfo)
 {
  char tempstr[TEXTLEN*2];

  if(ip[0]!=0)
   {
    portinfo->owninfo.slipaddr=getinternetaddress(ip);
    portinfo->write(port);
   }

  sprintf(tempstr, "%s-Keepalive (%s Port %ld)", protocol, cp.name, port);

  if(ip[0]!=0)
    sprintf(tempstr+strlen(tempstr), ", IP %s", ip);

  logf.fileprintf(userlogfile, "dboxradius", tempstr);
 }

void dboxaccountingt::startreq(portinfot *portinfo)
 {
  char tempstr[TEXTLEN*4];
  VALUE_PAIR *get_pair;

  if(!up.lock())
   {
    up.set("letzteranruf", time(NULL));
    up.set("anrufe", atol(up|"anrufe")+1);

    up.write();
    up.unlock();
   }

  if(stricmp(protocol, "BBS")==0)
    portinfo->owninfo.status=portinfot::infot::ONLINE;
  else if(stricmp(protocol, "SLIP")==0 ||
          stricmp(protocol, "CSLIP")==0 ||
          stricmp(protocol, "PPP")==0)
    portinfo->owninfo.status=portinfot::infot::SLIP;
  else if(stricmp(protocol, "JANUS")==0 ||
          stricmp(protocol, "JANUS2")==0 ||
          stricmp(protocol, "ALPHABOX")==0 ||
          stricmp(protocol, "GSBOX")==0 ||
          stricmp(protocol, "UUCP")==0)
    portinfo->owninfo.status=portinfot::infot::NETCALL;
  else
   {
    logf.fileprintf(userlogfile, "dboxradius", "unknown protocol");
    return;
   }

  strmaxcpy(portinfo->owninfo.modemname, "Radius", TEXTLEN-1);
  strmaxcpy(portinfo->owninfo.telefon, called_station, TEXTLEN-1);
  strmaxcpy(portinfo->owninfo.username, up.name, TEXTLEN-1);
  strmaxcpy(portinfo->owninfo.from, from, TEXTLEN-1);
  strmaxcpy(portinfo->owninfo.boxname, boxname, TEXTLEN-1);
  strmaxcpy(portinfo->owninfo.sessionid, sessionid, TEXTLEN-1);

  if(ip[0]!=0)
    portinfo->owninfo.slipaddr=getinternetaddress(ip);
  else
    portinfo->owninfo.slipaddr=0;

  portinfo->owninfo.lastcall=time(NULL);
  portinfo->write(port);

  sprintf(tempstr, "%s-Login (%s Port %ld)", protocol, cp.name, port);
  if(ip[0]!=0) sprintf(tempstr+strlen(tempstr), ", IP %s", ip);

  get_pair=get_attribute(userinfo.pair, PW_CALLING);
  if(get_pair!=NULL)
    sprintf(tempstr+strlen(tempstr),  " from %s", 
      get_pair->strvalue);

  logf.fileprintf(userlogfile, "dboxradius", tempstr);
 }

void dboxaccountingt::processaccounting()
 {
  char tempstr[DIRLEN];
  portinfot portinfo;

  sprintf(userlogfile, "%slogfile", up.getpath());
  sprintf(portinfo.filename, DBOXCLIENTDIR "portinfo", config.dboxdir, cp.name);

  // fprintf(stderr, "DK: checkpoint accounting 5 dboxradius.C\r\n");

  cp.get("radius.ignore-NAS-Port", tempstr);
  if(atoi(tempstr)==1)
    getnasport(&portinfo);

  if(port==-1)
   {
    logf.printf("dboxradius::accounting", "no port number (user %s)",
      up.name);
    return;
   }

  portinfo.read(port);
  portinfo.owninfo=portinfo.info;

  // fprintf(stderr, "DK: checkpoint accounting 6 dboxradius.C\r\n");

  if(stop)
    stopreq(&portinfo);
  else if(keepalive)
    keepalivereq(&portinfo);
  else
    startreq(&portinfo);
 }

void dboxaccountingt::dboxaccounting(const char *pm, VALUE_PAIR *pair)
 {
  bool readok=FALSE;

  userinfo.pair=pair;
  stop=FALSE;
  keepalive=FALSE;
  port=-1;
  dauer=0;
  delay_time=0;

  protocol[0]=ip[0]=userlogfile[0]=charge_menu[0]=0;
  sessionid[0]=from[0]=called_station[0]=boxname[0]=0;
  errormsg[0]=login[0];

  strmaxcpy(cp.name, pm, TEXTLEN-1);
  dnstring(cp.name);
  cp.read();

  up.name[0]=0;

  // fprintf(stderr, "DK: checkpoint accounting start dboxradius.C\r\n");

  while(pair!=NULL)
   {
    if(stricmp(pair->name, "User-Name")==0)
     {
      if(!readok)
       {
        strmaxcpy(login, pair->strvalue, TEXTLEN-1);
	  
        dnstring(login);
        filterstring(login);

        // fprintf(stderr, "DK: checkpoint accounting 2 dboxradius.C\r\n");

        do_check_login1(cp, login, up.name, boxname, errormsg,
                        protocol, &userinfo);

        // fprintf(stderr, "DK: checkpoint accounting 3 dboxradius.C\r\n");

        if(boxname[0]==0)
          strmaxcpy(boxname, config.boxname, TEXTLEN-1);

        if(up.name[0]==0)
         {
         }
        else if(up.read())
          logf.printf("dboxradius::accounting", 
                      "fatal: unknown user: \"%s\"", up.name);
        else
          readok=TRUE;
       }
     }
    else if(stricmp(pair->name, "NAS-Port")==0)
     {
      port=getint(pair);
     }
    else if(stricmp(pair->name, "Acct-Status-Type")==0)
     {
      if(getint(pair)==2)      // Stop
        stop=TRUE;
      else if(getint(pair)==3) // Keepalive (Cisco)
        keepalive=TRUE;
     }
    else if(stricmp(pair->name, "Acct-Session-Time")==0)
     {
      dauer=getint(pair);
     }
    else if(stricmp(pair->name, "Acct-Delay-Time")==0)
     {
      delay_time=getint(pair);
     }
    else if(stricmp(pair->name, "Acct-Session-Id")==0)
     {
      strmaxcpy(sessionid, pair->strvalue, TEXTLEN-1);
     }
    else if(stricmp(pair->name, "Framed-IP-Address")==0)
     {
      ipaddr2str(ip, pair->lvalue);
     }
    else if(stricmp(pair->name, "Called-Station-Id")==0)
     {
      strmaxcpy(called_station, pair->strvalue, TEXTLEN-1);
     }
    else if(stricmp(pair->name, "Calling-Station-Id")==0)
     {
      strmaxcpy(from, pair->strvalue, TEXTLEN-1);
     }

    pair=pair->next;
   }

  // fprintf(stderr, "DK: checkpoint accounting 4 dboxradius.C %d\r\n", readok);

  if(readok)
    processaccounting();

  // fprintf(stderr, "DK: checkpoint accounting ende dboxradius.C\r\n");
 }

extern "C" int dboxaccounting(char *pm, VALUE_PAIR *pair)
 {
  dboxaccountingt dboxaccounting;

  dboxaccounting.dboxaccounting(pm, pair);

  return 0;
 }
