#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <fcntl.h>

#include <dbox.h>
#include <logins.h>
#include <flags.h>
#include <portinfo.h>
#include <libdboxversion.h>
#include <dboxpath.h>
#include <logf.h>
#include <account.h>

#undef DEBUG
#include "radius.h"

#include "dboxradius.h"
#include "dbox_misc.h"

extern "C" VALUE_PAIR *get_attribute(VALUE_PAIR *value_list, int attribute);

extern "C" int dboxuser(struct userinfot *userinfo, 
                        char **rejecterror, char *pm)
 {
  userpt up, up2;
  clientpt cp;
  grouppt gp;
  char userlogfile[DIRLEN], error[TEXTLEN],
       boxname[TEXTLEN], login[TEXTLEN], charge_menu[TEXTLEN]="",
       protocol[TEXTLEN]="";
  static char r_error[TEXTLEN];
  VALUE_PAIR *get_pair;
  configt pconfig;

  // fprintf(stderr, "DK: checkpoint user start dboxradius.C\r\n");

  strmaxcpy(cp.name, pm, TEXTLEN-1);
  dnstring(cp.name);
  cp.read();

  // fprintf(stderr, "DK: checkpoint user 2 dboxradius.C\r\n");

  strmaxcpy(login, userinfo->username, TEXTLEN-1);
  dnstring(login);

  *rejecterror=r_error;

  if(do_check_login1(cp, login, up.name, boxname, r_error, protocol,
                     userinfo))
    return 1;

  // fprintf(stderr, "DK: checkpoint user 3 dboxradius.C\r\n");

  if(up.name[0]==0) return 1;

  if(boxname[0]==0)
    strmaxcpy(boxname, config.boxname, DIRLEN-1);
  else if(stricmp(boxname, config.boxname)!=0)
   {
    pconfig.read(boxname);
    up.pconfig=&pconfig;
   }

  if(up.read())
   {
    snprintf(r_error, TEXTLEN-1, "account %s@%s not found", 
      up.name, boxname);
    return 1;
   }

  snprintf(userlogfile, DIRLEN-1, "%slogfile", up.getpath());

   {
    char logentry[1000];

    snprintf(logentry, 999, "%s-Auth-Request from %s", protocol, pm);
    get_pair=get_attribute(userinfo->pair, 32); // NAS-Identifier
    if(get_pair!=NULL)
      snprintf(logentry+strlen(logentry), 999-strlen(logentry),
        "/%s", get_pair->strvalue);

    logf.fileprintf(userlogfile, "dboxradius", logentry);

    strmaxcpy(logentry, "Call ", 999);

    get_pair=get_attribute(userinfo->pair, PW_CALLING);
    snprintf(logentry+strlen(logentry), 999-strlen(logentry),
      "%s", get_pair==NULL?"(unknown)":get_pair->strvalue);

    get_pair=get_attribute(userinfo->pair, PW_CALLED);
    if(get_pair!=NULL)
    snprintf(logentry+strlen(logentry), 999-strlen(logentry),
      "->%s", get_pair==NULL?"(unknown)":get_pair->strvalue);

    logf.fileprintf(userlogfile, "dboxradius", "%s", logentry);
   }

  getplaintextpassword(&up, userinfo->password);
  userinfo->message[0]=0;

  // fprintf(stderr, "DK: checkpoint user 4 dboxradius.C\r\n");

  if((up|"alias")[0]!=0 &&
     !atoi(up|"account"))
   {
    strmaxcpy(r_error, "permission denied", TEXTLEN-1);
    logf.fileprintf(userlogfile, "dboxradius", 
      "Permission denied: not an account");
    return 1;
   }

  get_profiles(up, up2, gp);

  if(!get_internalflag(up, up2, gp, LOGIN_SLIP))
   {
    strmaxcpy(r_error, "permission denied", TEXTLEN-1);
    logf.fileprintf(userlogfile, "dboxradius", 
      "Permission denied: no login_slip flag");
    return 1;
   }

  if(do_check_login2(cp, up2, r_error, protocol,
                     charge_menu, userinfo))
    return 1;

  if(userinfo->session_timeout[0]!=0)
    logf.fileprintf(userlogfile, "dboxradius", "session time limit: %s", 
      userinfo->session_timeout);

  // fprintf(stderr, "DK: checkpoint user 5 dboxradius.C\r\n");

  if(charge_menu[0]!=0 && 
     checkchargemenu(&up2, &gp, charge_menu, error))
   {
    sprintf(r_error, "%s", error);
    *rejecterror=r_error;
    logf.fileprintf(userlogfile, "dboxradius", "failed: %s", error);
    return 1;
   }

  // fprintf(stderr, "DK: checkpoint user 6 dboxradius.C\r\n");

  if(charge_menu[0]!=0)
   {
    char balance_s[TEXTLEN];
    amountt balance;

    balance=getbalance(&up2);
    currency_string(balance_s, balance, TRUE);

    sprintf(userinfo->message, "Ihr Kontostand: %s", balance_s);
   }
  else
    userinfo->message[0]=0;

  logf.fileprintf(userlogfile, "dboxradius", "Auth-Request: Ok");

  // fprintf(stderr, "DK: checkpoint user ende dboxradius.C\r\n");

  return 0;
 }
 