#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <fcntl.h>

#include <dbox.h>
#include <logins.h>
#include <flags.h>
#include <portinfo.h>
#include <libdboxversion.h>
#include <dboxpath.h>
#include <logf.h>
#include <account.h>

#undef DEBUG
#include "radius.h"

#include "dboxradius.h"
#include "dbox_misc.h"

extern "C" VALUE_PAIR *get_attribute(VALUE_PAIR *value_list, int attribute);

class radius_check_login1t:public check_login1t
 {
 public:
  virtual bool radius_check(const char *item, const char *mask);
  VALUE_PAIR *pair;
 };

class radius_check_login2t:public check_login2t
 {
 public:
  virtual bool radius_check(const char *item, const char *mask);
  VALUE_PAIR *pair;
 };

bool do_radius_check(VALUE_PAIR *pair, 
                     const char *item, const char *mask)
 {
  VALUE_PAIR *p=pair;

  while(p!=NULL)
   {
    if(stricmp(p->name, item)==0)
     {
      if(p->type==PW_TYPE_STRING)
       {
        if(checkjoker(mask, p->strvalue, TRUE))
          return TRUE;
       }
     }

    p=p->next;
   }

  return FALSE;
 }

bool radius_check_login1t::radius_check(const char *item, const char *mask)
 {
  return ::do_radius_check(pair, item, mask);
 }

bool radius_check_login2t::radius_check(const char *item, const char *mask)
 {
  return ::do_radius_check(pair, item, mask);
 }

long int getint(VALUE_PAIR *pair)
 {
  if(pair->type==PW_TYPE_STRING)
    return atol(pair->strvalue);

  return pair->lvalue;
 }

void filterstring(char *s)
 {
  while(*s!=0)
   {
    if(*s!=' ' && *s!='\t')
      s++;
    else
      *s=0;
   }
 }

bool do_check_login1(clientpt &cp, const char *login,
                     char *username, char *boxname,
                     char *errormsg, char *protocol,
                     userinfot *userinfo)
 {
  // fprintf(stderr, "DK: checkpoint dochecklogin start dboxradius.C\r\n");

  strmaxcpy(username, login, TEXTLEN-1);
  errormsg[0]=0;
  userinfo->framed_protocol[0]=0;
  userinfo->service_type[0]=0;
  userinfo->framed_compression[0]=0;
  userinfo->login_service[0]=0;

  FILE *infile;
   
   {
    char filename[DIRLEN];
    snprintf(filename, DIRLEN, "%sprotocols", cp.getpath());
    infile=fopen(filename, "rt");
   }

  // fprintf(stderr, "DK: checkpoint dochecklogin 2 dboxradius.C\r\n");

  if(infile==NULL)
   {
    char filename[DIRLEN];
    snprintf(filename, DIRLEN, BOXSYSTEMDIR "protocols", config.boxdir);
    infile=fopen(filename, "rt");
    if(infile==NULL)
     {
      snprintf(filename, DIRLEN, DBOXSYSTEMDIR "protocols", config.dboxdir);
      infile=fopen(filename, "rt");
     }
   }

  // fprintf(stderr, "DK: checkpoint dochecklogin 3 dboxradius.C\r\n");

  if(infile==NULL) return FALSE;

  // fprintf(stderr, "DK: checkpoint dochecklogin 4 dboxradius.C\r\n");

  radius_check_login1t check_login1;
  check_login1.pair=userinfo->pair;
  check_login1.protocol=protocol;
  check_login1.login=login;
  bool result=check_login1.check_login1(infile);
  
  // fprintf(stderr, "DK: checkpoint dochecklogin 5 dboxradius.C\r\n");

  fclose(infile);
  
  // fprintf(stderr, "DK: checkpoint dochecklogin ende dboxradius.C\r\n");

  strmaxcpy(protocol, check_login1.protocol_set, TEXTLEN-1);
  strmaxcpy(boxname, check_login1.boxname, TEXTLEN-1);
  strmaxcpy(errormsg, check_login1.errormsg, TEXTLEN-1);
  strmaxcpy(username, check_login1.username, TEXTLEN-1);

  userinfo->framed_protocol[0]=0;
  userinfo->login_service[0]=0;
  userinfo->framed_compression[0]=0;
  userinfo->login_ip_host[0]=0;

  if(stricmp(protocol, "ppp")==0)
   {
    strmaxcpy(userinfo->service_type, "Framed-User", TEXTLEN-1);
    strmaxcpy(userinfo->framed_protocol, "PPP", TEXTLEN-1);
   }
  else if(stricmp(protocol, "slip")==0)
   {
    strmaxcpy(userinfo->service_type, "Framed-User", TEXTLEN-1);
    strmaxcpy(userinfo->framed_protocol, "SLIP", TEXTLEN-1);
    strmaxcpy(userinfo->framed_compression, "None", TEXTLEN-1);
   }
  else if(stricmp(protocol, "cslip")==0)
   {
    strmaxcpy(userinfo->service_type, "Framed-User", TEXTLEN-1);
    strmaxcpy(userinfo->framed_protocol, "SLIP", TEXTLEN-1);
    strmaxcpy(userinfo->framed_compression, "Van-Jacobson-TCP-IP", TEXTLEN-1);
   }
  else if(stricmp(protocol, "janus")==0 ||
          stricmp(protocol, "janus2")==0 ||
          stricmp(protocol, "uucp")==0)
   {
    strmaxcpy(userinfo->service_type, "Login-User", TEXTLEN-1);
    strmaxcpy(userinfo->login_service, "TCP-Clear", TEXTLEN-1);
    strmaxcpy(userinfo->login_ip_host, check_login1.forwardto_host, TEXTLEN-1);
   }
  else if(protocol[0]==0)
    strmaxcpy(protocol, "None", TEXTLEN-1);

  return result;
 }

bool do_check_login2(clientpt &cp,
                     userpt &up,
                     char *errormsg,
                     const char *protocol,
                     char *charge_menu,
                     userinfot *userinfo)
 {
  // fprintf(stderr, "DK: checkpoint dochecklogin start dboxradius.C\r\n");

  errormsg[0]=0;

  FILE *infile;
   
   {
    char filename[DIRLEN];
    snprintf(filename, DIRLEN, "%scharge", cp.getpath());
    infile=fopen(filename, "rt");
   }

  // fprintf(stderr, "DK: checkpoint dochecklogin 2 dboxradius.C\r\n");

  if(infile==NULL)
   {
    char filename[DIRLEN];
    snprintf(filename, DIRLEN, BOXSYSTEMDIR "charge", config.boxdir);
    infile=fopen(filename, "rt");
    if(infile==NULL)
     {
      snprintf(filename, DIRLEN, DBOXSYSTEMDIR "charge", config.dboxdir);
      infile=fopen(filename, "rt");
     }
   }

  // fprintf(stderr, "DK: checkpoint dochecklogin 3 dboxradius.C\r\n");

  if(infile==NULL)
    return FALSE;

  // fprintf(stderr, "DK: checkpoint dochecklogin 4 dboxradius.C\r\n");

  radius_check_login2t check_login2;
  check_login2.pair=userinfo->pair;
  check_login2.up=&up;
  check_login2.protocol=protocol;
  bool result=check_login2.check_login2(infile);
  
  // fprintf(stderr, "DK: checkpoint dochecklogin 5 dboxradius.C\r\n");

  fclose(infile);
  
  // fprintf(stderr, "DK: checkpoint dochecklogin ende dboxradius.C\r\n");

  strmaxcpy(errormsg, check_login2.errormsg, TEXTLEN-1);
  strmaxcpy(charge_menu, check_login2.charge_menu, TEXTLEN-1);

  if(check_login2.session_timeout!=0)
    snprintf(userinfo->session_timeout, TEXTLEN,
      "%lu", check_login2.session_timeout);
  else
    userinfo->session_timeout[0]=0;

  return result;
 }

