#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <fcntl.h>

#include <dbox.h>
#include <logins.h>
#include <flags.h>
#include <portinfo.h>
#include <libdboxversion.h>
#include <dboxpath.h>
#include <logf.h>
#include <account.h>

#undef DEBUG
#include "radius.h"

#include "dboxradius.h"

#if 0
extern "C" VALUE_PAIR *get_attribute(VALUE_PAIR *value_list, int attribute);

class radiuschecklogint:public checklogint
 {
 public:
  virtual bool radius_check(const char *item, const char *mask);
  VALUE_PAIR *pair;
 };

bool radiuschecklogint::radius_check(const char *item, const char *mask)
 {
  VALUE_PAIR *p=pair;

  while(p!=NULL)
   {
    if(stricmp(p->name, item)==0)
     {
      if(p->type==PW_TYPE_STRING)
       {
        if(checkjoker(mask, p->strvalue, TRUE))
          return TRUE;
       }
     }

    p=p->next;
   }

  return FALSE;
 }

long int getint(VALUE_PAIR *pair)
 {
  if(pair->type==PW_TYPE_STRING)
    return atol(pair->strvalue);

  return pair->lvalue;
 }

void filterstring(char *s)
 {
  while(*s!=0)
   {
    if(*s!=' ' && *s!='\t')
      s++;
    else
      *s=0;
   }
 }

bool dochecklogin(clientpt *cp, const char *login,
                  char *username, char *boxname, char *errormsg,
                  char *protocol, char *charge_menu,
                  userinfot *userinfo)
 {
  radiuschecklogint checklogin;
  FILE *infile;
  char filename[DIRLEN];
  bool result;

  // fprintf(stderr, "DK: checkpoint dochecklogin start dboxradius.C\r\n");

  strmaxcpy(username, login, TEXTLEN-1);
  boxname[0]=errormsg[0]=0;
  result=FALSE;
  userinfo->framed_protocol[0]=0;
  userinfo->service_type[0]=0;
  userinfo->framed_compression[0]=0;
  userinfo->login_service[0]=0;
  checklogin.pair=userinfo->pair;

  sprintf(filename, "%slogins", cp->getpath());
  infile=fopen(filename, "rt");

  // fprintf(stderr, "DK: checkpoint dochecklogin 2 dboxradius.C\r\n");

  if(infile==NULL)
   {
    sprintf(filename, BOXSYSTEMDIR "logins", config.boxdir);
    infile=fopen(filename, "rt");
    if(infile==NULL)
     {
      sprintf(filename, DBOXSYSTEMDIR "logins", config.dboxdir);
      infile=fopen(filename, "rt");
     }
   }

  // fprintf(stderr, "DK: checkpoint dochecklogin 3 dboxradius.C\r\n");

  if(infile==NULL) return FALSE;

  // fprintf(stderr, "DK: checkpoint dochecklogin 4 dboxradius.C\r\n");

  result=checklogin.checklogin(infile, login, protocol);
  
  // fprintf(stderr, "DK: checkpoint dochecklogin 5 dboxradius.C\r\n");

  fclose(infile);
  
  // fprintf(stderr, "DK: checkpoint dochecklogin ende dboxradius.C\r\n");

  if(checklogin.protocol_set[0]!=0)
    strmaxcpy(protocol, checklogin.protocol_set, TEXTLEN-1);

  strmaxcpy(boxname, checklogin.boxname, TEXTLEN-1);
  strmaxcpy(errormsg, checklogin.errormsg, TEXTLEN-1);
  strmaxcpy(username, checklogin.username, TEXTLEN-1);
  strmaxcpy(charge_menu, checklogin.charge_menu, TEXTLEN-1);

  userinfo->framed_protocol[0]=0;
  userinfo->login_service[0]=0;
  userinfo->framed_compression[0]=0;
  userinfo->login_ip_host[0]=0;
  if(checklogin.session_timeout!=0)
    snprintf(userinfo->session_timeout, TEXTLEN, "%lu",
             checklogin.session_timeout);
  else
    userinfo->session_timeout[0]=0;

  if(stricmp(protocol, "ppp")==0)
   {
    strmaxcpy(userinfo->service_type, "Framed-User", TEXTLEN-1);
    strmaxcpy(userinfo->framed_protocol, "PPP", TEXTLEN-1);
   }
  else if(stricmp(protocol, "slip")==0)
   {
    strmaxcpy(userinfo->service_type, "Framed-User", TEXTLEN-1);
    strmaxcpy(userinfo->framed_protocol, "SLIP", TEXTLEN-1);
    strmaxcpy(userinfo->framed_compression, "None", TEXTLEN-1);
   }
  else if(stricmp(protocol, "cslip")==0)
   {
    strmaxcpy(userinfo->service_type, "Framed-User", TEXTLEN-1);
    strmaxcpy(userinfo->framed_protocol, "SLIP", TEXTLEN-1);
    strmaxcpy(userinfo->framed_compression, "Van-Jacobson-TCP-IP", TEXTLEN-1);
   }
  else if(stricmp(protocol, "janus")==0 ||
          stricmp(protocol, "janus2")==0 ||
          stricmp(protocol, "uucp")==0)
   {
    strmaxcpy(userinfo->service_type, "Login-User", TEXTLEN-1);
    strmaxcpy(userinfo->login_service, "TCP-Clear", TEXTLEN-1);
    strmaxcpy(userinfo->login_ip_host, checklogin.forwardto_host, TEXTLEN-1);
   }
  else if(protocol[0]==0)
    strmaxcpy(protocol, "BBS", TEXTLEN-1);

  return result;
 }

class dboxaccountingt
 {
 public:
  userpt up;
  clientpt cp;
  long int port, dauer, delay_time;
  bool stop, keepalive;
  struct userinfot userinfo;

  char protocol[TEXTLEN], ip[100], 
       charge_menu[TEXTLEN], userlogfile[DIRLEN],
       sessionid[TEXTLEN], from[TEXTLEN],
       called_station[TEXTLEN], boxname[TEXTLEN],
       errormsg[TEXTLEN], login[TEXTLEN];

  void dboxaccounting(const char *pm, VALUE_PAIR *pair);
  void getnasport(portinfot *portinfo);
  void startreq(portinfot *portinfo);
  void keepalivereq(portinfot *portinfo);
  void stopreq(portinfot *portinfo);
  void processaccounting();
 };

void dboxaccountingt::getnasport(portinfot *portinfo)
 {
  long int oldport;
  word first_free_port=MAXPORTS;

  oldport=port;

  port=0;
  sprintf(sessionid+strlen(sessionid), "%ld", oldport);

  do
   {
    port++;
    portinfo->read(port);
    if(portinfo->info.status==portinfot::infot::OFFLINE &&
       first_free_port==MAXPORTS)
       first_free_port=port;
   }
  while(stricmp(portinfo->info.sessionid, sessionid)!=0 && port<MAXPORTS);

  if(port==MAXPORTS)
   {
    if(first_free_port==MAXPORTS)
     {
      logf.printf("dboxradius::accounting", 
        "radius.ignore-NAS-Port: kein freier Port", oldport, port);
      port=0;
     }
    else
     {
      port=first_free_port;
     }
   }

  logf.printf("dboxradius::accounting", 
    "radius.ignore-NAS-Port: %d -> %d", oldport, port);
 }

void dboxaccountingt::stopreq(portinfot *portinfo)
 {
  char tempstr[TEXTLEN*2];

  if(portinfo->owninfo.status==portinfot::infot::SLIP ||
     portinfo->owninfo.status==portinfot::infot::NETCALL ||
     portinfo->owninfo.status==portinfot::infot::ONLINE)
   {
    portinfo->owninfo.status=portinfot::infot::OFFLINE;

    strmaxcpy(portinfo->owninfo.modemname, "", TEXTLEN-1);
    strmaxcpy(portinfo->owninfo.telefon, "", TEXTLEN-1);
    strmaxcpy(portinfo->owninfo.username, "", TEXTLEN-1);
    strmaxcpy(portinfo->owninfo.from, "", TEXTLEN-1);
    strmaxcpy(portinfo->owninfo.boxname, boxname, TEXTLEN-1);
    portinfo->owninfo.lastcall=time(NULL);
    portinfo->owninfo.slipaddr=0;

    portinfo->write(port);

    sprintf(tempstr, "%s-Logout, Verbindungsdauer: %ld Sekunden"
      " (%s Port %ld)", protocol, dauer, cp.name, port);
    logf.fileprintf(userlogfile, "dboxradius", tempstr);

    if(charge_menu[0]!=0)
     {
      char zweck[TEXTLEN];
      time_t t;

      time(&t);

      sprintf(zweck, "R-%s Zeitgebhr", protocol);
      chargemenu(&up, charge_menu, zweck, t-dauer, t);
     }
   }
  else
   {
    sprintf(tempstr, "%s-Logout ignoriert (%s, %ld, %s, %s)",
      protocol, up.name, port, cp.name, sessionid);
    logf.printf("dboxradius::accounting", tempstr);
    logf.fileprintf(userlogfile, "dboxradius", tempstr);
   }
 }

void dboxaccountingt::keepalivereq(portinfot *portinfo)
 {
  char tempstr[TEXTLEN*2];

  if(ip[0]!=0)
   {
    portinfo->owninfo.slipaddr=getinternetaddress(ip);
    portinfo->write(port);
   }

  sprintf(tempstr, "%s-Keepalive (%s Port %ld)", protocol, cp.name, port);

  if(ip[0]!=0)
    sprintf(tempstr+strlen(tempstr), ", IP %s", ip);

  logf.fileprintf(userlogfile, "dboxradius", tempstr);
 }

void dboxaccountingt::startreq(portinfot *portinfo)
 {
  char tempstr[TEXTLEN*4];
  VALUE_PAIR *get_pair;

  if(!up.lock())
   {
    up.set("letzteranruf", time(NULL));
    up.set("anrufe", atol(up|"anrufe")+1);

    up.write();
    up.unlock();
   }

  if(stricmp(protocol, "BBS")==0)
    portinfo->owninfo.status=portinfot::infot::ONLINE;
  else if(stricmp(protocol, "SLIP")==0 ||
          stricmp(protocol, "CSLIP")==0 ||
          stricmp(protocol, "PPP")==0)
    portinfo->owninfo.status=portinfot::infot::SLIP;
  else if(stricmp(protocol, "JANUS")==0 ||
          stricmp(protocol, "JANUS2")==0 ||
          stricmp(protocol, "ALPHABOX")==0 ||
          stricmp(protocol, "GSBOX")==0 ||
          stricmp(protocol, "UUCP")==0)
    portinfo->owninfo.status=portinfot::infot::NETCALL;
  else
   {
    logf.fileprintf(userlogfile, "dboxradius", "unknown protocol");
    return;
   }

  strmaxcpy(portinfo->owninfo.modemname, "Radius", TEXTLEN-1);
  strmaxcpy(portinfo->owninfo.telefon, called_station, TEXTLEN-1);
  strmaxcpy(portinfo->owninfo.username, up.name, TEXTLEN-1);
  strmaxcpy(portinfo->owninfo.from, from, TEXTLEN-1);
  strmaxcpy(portinfo->owninfo.boxname, boxname, TEXTLEN-1);
  strmaxcpy(portinfo->owninfo.sessionid, sessionid, TEXTLEN-1);

  if(ip[0]!=0)
    portinfo->owninfo.slipaddr=getinternetaddress(ip);
  else
    portinfo->owninfo.slipaddr=0;

  portinfo->owninfo.lastcall=time(NULL);
  portinfo->write(port);

  sprintf(tempstr, "%s-Login (%s Port %ld)", protocol, cp.name, port);
  if(ip[0]!=0) sprintf(tempstr+strlen(tempstr), ", IP %s", ip);

  get_pair=get_attribute(userinfo.pair, PW_CALLING);
  if(get_pair!=NULL)
    sprintf(tempstr+strlen(tempstr),  " from %s", 
      get_pair->strvalue);

  logf.fileprintf(userlogfile, "dboxradius", tempstr);
 }

void dboxaccountingt::processaccounting()
 {
  char tempstr[DIRLEN];
  portinfot portinfo;

  sprintf(userlogfile, "%slogfile", up.getpath());
  sprintf(portinfo.filename, DBOXCLIENTDIR "portinfo", config.dboxdir, cp.name);

  // fprintf(stderr, "DK: checkpoint accounting 5 dboxradius.C\r\n");

  cp.get("radius.ignore-NAS-Port", tempstr);
  if(atoi(tempstr)==1)
    getnasport(&portinfo);

  if(port==-1)
   {
    logf.printf("dboxradius::accounting", "No port number (user %s)",
      up.name);
    return;
   }

  portinfo.read(port);
  portinfo.owninfo=portinfo.info;

  // fprintf(stderr, "DK: checkpoint accounting 6 dboxradius.C\r\n");

  if(stop)
    stopreq(&portinfo);
  else if(keepalive)
    keepalivereq(&portinfo);
  else
    startreq(&portinfo);
 }

void dboxaccountingt::dboxaccounting(const char *pm, VALUE_PAIR *pair)
 {
  bool readok=FALSE;

  userinfo.pair=pair;
  stop=FALSE;
  keepalive=FALSE;
  port=-1;
  dauer=0;
  delay_time=0;

  protocol[0]=ip[0]=userlogfile[0]=charge_menu[0]=0;
  sessionid[0]=from[0]=called_station[0]=boxname[0]=0;
  errormsg[0]=login[0];

  strmaxcpy(cp.name, pm, TEXTLEN-1);
  dnstring(cp.name);
  cp.read();

  up.name[0]=0;

  // fprintf(stderr, "DK: checkpoint accounting start dboxradius.C\r\n");

  while(pair!=NULL)
   {
    if(stricmp(pair->name, "User-Name")==0)
     {
      if(!readok)
       {
        strmaxcpy(login, pair->strvalue, TEXTLEN-1);
	  
        dnstring(login);
        filterstring(login);

        // fprintf(stderr, "DK: checkpoint accounting 2 dboxradius.C\r\n");

        dochecklogin(&cp, login, up.name, boxname, errormsg,
                     protocol, charge_menu, &userinfo);

        // fprintf(stderr, "DK: checkpoint accounting 3 dboxradius.C\r\n");

        if(boxname[0]==0)
          strmaxcpy(boxname, config.boxname, TEXTLEN-1);

        if(up.name[0]==0)
         {
         }
        else if(up.read())
          logf.printf("dboxradius::accounting", 
                      "Fatal: unbekannter User: \"%s\"", up.name);
        else
          readok=TRUE;
       }
     }
    else if(stricmp(pair->name, "NAS-Port")==0)
     {
      port=getint(pair);
     }
    else if(stricmp(pair->name, "Acct-Status-Type")==0)
     {
      if(getint(pair)==2)      // Stop
        stop=TRUE;
      else if(getint(pair)==3) // Keepalive (Cisco)
        keepalive=TRUE;
     }
    else if(stricmp(pair->name, "Acct-Session-Time")==0)
     {
      dauer=getint(pair);
     }
    else if(stricmp(pair->name, "Acct-Delay-Time")==0)
     {
      delay_time=getint(pair);
     }
    else if(stricmp(pair->name, "Acct-Session-Id")==0)
     {
      strmaxcpy(sessionid, pair->strvalue, TEXTLEN-1);
     }
    else if(stricmp(pair->name, "Framed-IP-Address")==0)
     {
      ipaddr2str(ip, pair->lvalue);
     }
    else if(stricmp(pair->name, "Called-Station-Id")==0)
     {
      strmaxcpy(called_station, pair->strvalue, TEXTLEN-1);
     }
    else if(stricmp(pair->name, "Calling-Station-Id")==0)
     {
      strmaxcpy(from, pair->strvalue, TEXTLEN-1);
     }

    pair=pair->next;
   }

  // fprintf(stderr, "DK: checkpoint accounting 4 dboxradius.C %d\r\n", readok);

  if(readok)
    processaccounting();

  // fprintf(stderr, "DK: checkpoint accounting ende dboxradius.C\r\n");
 }

extern "C" int dboxaccounting(char *pm, VALUE_PAIR *pair)
 {
  dboxaccountingt dboxaccounting;

  dboxaccounting.dboxaccounting(pm, pair);

  return 0;
 }

extern "C" int dboxuser(struct userinfot *userinfo, 
                        char **rejecterror, char *pm)
 {
  userpt up, up2;
  clientpt cp;
  grouppt gp;
  char userlogfile[DIRLEN], error[TEXTLEN],
       boxname[TEXTLEN], login[TEXTLEN], charge_menu[TEXTLEN]="",
       protocol[TEXTLEN]="";
  static char r_error[TEXTLEN];
  VALUE_PAIR *get_pair;
  configt pconfig;

  // fprintf(stderr, "DK: checkpoint user start dboxradius.C\r\n");

  strmaxcpy(cp.name, pm, TEXTLEN-1);
  dnstring(cp.name);
  cp.read();

  // fprintf(stderr, "DK: checkpoint user 2 dboxradius.C\r\n");

  strmaxcpy(login, userinfo->username, TEXTLEN-1);
  dnstring(login);

  *rejecterror=r_error;
  if(dochecklogin(&cp, login, up.name, boxname, r_error, protocol,
                  charge_menu, userinfo))
    return 1;

  // fprintf(stderr, "DK: checkpoint user 3 dboxradius.C\r\n");

  if(up.name[0]==0) return 1;

  if(boxname[0]==0)
    strmaxcpy(boxname, config.boxname, DIRLEN-1);
  else if(stricmp(boxname, config.boxname)!=0)
   {
    pconfig.read(boxname);
    up.pconfig=&pconfig;
    up2.pconfig=&pconfig;
    gp.pconfig=&pconfig;
   }

  if(up.read())
   {
    sprintf(r_error, "account %s@%s not found", up.name, boxname);
    return 1;
   }

  snprintf(userlogfile, DIRLEN-1, "%slogfile", up.getpath());

  if(userinfo->session_timeout[0]!=0)
    logf.fileprintf(userlogfile, "dboxradius", "session time limit: %s", 
      userinfo->session_timeout);

   {
    char logentry[1000];

    snprintf(logentry, 999, "%s-Auth-Request from %s", protocol, pm);
    get_pair=get_attribute(userinfo->pair, 32); // NAS-Identifier
    if(get_pair!=NULL)
      snprintf(logentry+strlen(logentry), 999-strlen(logentry),
        "/%s", get_pair->strvalue);

    logf.fileprintf(userlogfile, "dboxradius", logentry);

    strmaxcpy(logentry, "Call ", 999);

    get_pair=get_attribute(userinfo->pair, PW_CALLING);
    snprintf(logentry+strlen(logentry), 999-strlen(logentry),
      "%s", get_pair==NULL?"(unknown)":get_pair->strvalue);

    get_pair=get_attribute(userinfo->pair, PW_CALLED);
    if(get_pair!=NULL)
    snprintf(logentry+strlen(logentry), 999-strlen(logentry),
      "->%s", get_pair==NULL?"(unknown)":get_pair->strvalue);

    logf.fileprintf(userlogfile, "dboxradius", "%s", logentry);
   }

  getplaintextpassword(&up, userinfo->password);
  userinfo->message[0]=0;

  // fprintf(stderr, "DK: checkpoint user 4 dboxradius.C\r\n");

  if((up|"alias")[0]!=0)
   {
    up2.setname(up|"alias");
    up2.read();
   }
  else
    up2.copy_from(&up);

  gp.setname(up2|"gruppe");
  gp.read();

  if(!getinternalflag(up, up2, gp, LOGIN_SLIP))
   {
    strmaxcpy(r_error, "permission denied", TEXTLEN-1);
    logf.fileprintf(userlogfile, "dboxradius", 
      "Permission denied: no login_slip flag");
    return 1;
   }

  // fprintf(stderr, "DK: checkpoint user 5 dboxradius.C\r\n");

  if(charge_menu[0]!=0 && 
     checkchargemenu(&up2, &gp, charge_menu, error))
   {
    sprintf(r_error, "%s", error);
    *rejecterror=r_error;
    logf.fileprintf(userlogfile, "dboxradius", "Failed: %s", error);
    return 1;
   }

  // fprintf(stderr, "DK: checkpoint user 6 dboxradius.C\r\n");

  if(charge_menu[0]!=0)
   {
    char balance_s[TEXTLEN];
    amountt balance;

    balance=getbalance(&up2);
    currency_string(balance_s, balance, TRUE);

    sprintf(userinfo->message, "Ihr Kontostand: %s", balance_s);
   }
  else
    userinfo->message[0]=0;

  logf.fileprintf(userlogfile, "dboxradius", "Auth-Request: Ok");

  // fprintf(stderr, "DK: checkpoint user ende dboxradius.C\r\n");

  return 0;
 }
#endif

int main(int argc, char **argv)
 {
  if(init_libdbox(LIBDBOXVERSION, LIBDBOXVERSIONNR) || config.read(""))
    return 1;

  main_radius(argc, argv);

  return 0;
 }
