/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: main file of setup program

*/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <signal.h>
#include <pwd.h>
#include <sys/types.h>

#include <dbox.h>
#include <dboxpath.h>
#include <tools.h>
#include <config.h>
#include <oldterm.h>
#include <cmdline.h>
#include <netdefs.h>
#include <oldtools.h>
#include <ui.h>
#include <libdboxversion.h>
#include <dboxversion.h>

bool showweiter=FALSE;
const char *distdir;

#ifdef OS_WIN32
#define SHELLSCRIPT ""
#else
#define SHELLSCRIPT "#!/bin/bash\n"
#endif

int ask(char **q, screent *s, byte y)
 {
  int id, i;
  menut menu(s);

  y+=11;

  for(i=0; q[i]!=NULL; i++)
   {
    buttonmet *me=new buttonmet;
    me->text=q[i];
    me->x=25;
    me->y=y+i*2;
    me->sizex=30;
    me->id=i;
    me->free_me=TRUE;
    menu.add(me);
   }

  do
    id=menu.menu();
  while(id==MENU_HELP);

  return id;
 }

char *q1[]=
 {
  "^Fortfahren",
  "^Programm beenden",
  (char *)NULL
 };

int ask_text(char *text, screent *s, byte y)
 {
  int i, id;
  menut menu(s);
  y+=11;

  editstrmet editme;
  editme.text=text;
  editme.x=25;
  editme.y=y;
  editme.sizex=30;
  editme.maxlen=TEXTLEN-1;
  menu.add(&editme);

  for(i=0; q1[i]!=NULL; i++)
   {
    buttonmet *me=new buttonmet;
    me->text=q1[i];
    me->x=25;
    me->id=i+1;
    me->y=y+i*2+2;
    me->sizex=30;
    me->free_me=TRUE;
    menu.add(me);
   }

  do
    id=menu.menu();
  while(id==MENU_HELP);

  return id;
 }

void config_box(FILE *outfile, 
  const char *boxname,
  const char *domain,
  const char *org,
  const char *sysop,
  const char *server,
  bool gast,
  bool maps,
  bool ticket,
  bool bretter)
 {
  fprintf(outfile,
          "# Anlegen der Verzeichnisse\n"
          "\n"
          "echo Anlegen der Verzeichnisse\n"
          "\n"
          "mkdir ~/%s\n"
          "mkdir ~/%s/bad\n"
          "mkdir ~/%s/gruppen\n"
          "mkdir ~/%s/gruppen/sysop\n"
          "mkdir ~/%s/gruppen/user\n"
          "mkdir ~/%s/gruppen/system\n"
          "mkdir ~/%s/gruppen/gast\n"
          "mkdir ~/%s/spool\n"
          "mkdir ~/%s/spool/zcmail\n"
          "mkdir ~/%s/spool/zcnews\n"
          "mkdir ~/%s/system\n"
          "mkdir ~/%s/log\n"
          "mkdir ~/%s/lock\n"
          "mkdir ~/%s/todo\n"
          "mkdir ~/%s/ticket\n"
          "mkdir ~/%s/ticket/tickets\n"
          "mkdir ~/%s/ticket/domain\n"
          "mkdir ~/%s/user\n", 
          boxname, boxname, boxname, boxname, boxname, boxname, boxname,
          boxname, boxname, boxname, boxname, boxname, boxname, boxname,
          boxname, boxname, boxname, boxname);

  if(bretter)
    fprintf(outfile,
            "mkdir ~/%s/bretter\n"
            "mkdir ~/%s/bretter/sys\n"
            "mkdir ~/%s/bretter/sys/unzustellbar\n"
            "mkdir ~/%s/bretter/sys/unzustellbar/prv\n"
            "mkdir ~/%s/bretter/sys/unzustellbar/brt\n",
            boxname, boxname, boxname, boxname, boxname);
  
  fprintf(outfile, "\n");

  fprintf(outfile, "# Erstellung der Konfigurationsdateien\n\n"
                   "echo Erstellung der Konfigurationsdateien\n\n");

  fprintf(outfile,
    "cat << EOF > ~/%s/gruppen/sysop/bretter\n"
    "/* RW\n"
    "EOF\n\n", boxname);
  
  fprintf(outfile,
    "cat << EOF > ~/%s/gruppen/sysop/profile\n"
    "userflag.VERWALTUNG=1\n"
    "internalflag.verwmenu_point=1\n"
    "internalflag.verwmenu_bretter=1\n"
    "internalflag.verwmenu_allgemein=1\n"
    "internalflag.verwmenu_netz=1\n"
    "internalflag.login_online=1\n"
    "internalflag.login_slip=1\n"
    "internalflag.login_netz=1\n"
    "internalflag.login_web=1\n"
    "internalflag.login_pop3=1\n"
    "security=9\n"
    "EOF\n\n", boxname);
  
  fprintf(outfile,
    "cat << EOF > ~/%s/gruppen/user/bretter\n"
    "/Z-NETZ/WICHTIG R\n"
    "/SYS/* -\n"
    "/* RW\n"
    "EOF\n\n", boxname);

  fprintf(outfile,
    "cat << EOF > ~/%s/gruppen/user/slip.chg\n"
    "displayunit=1\n"
    "unit=18\n"
    "unitprice=0.01\n"
    "description=Internet\n"
    "EOF\n\n", boxname);
  
  fprintf(outfile,
    "cat << EOF > ~/%s/gruppen/sysop/slip.chg\n"
    "displayunit=1\n"
    "unit=18\n"
    "unitprice=0.01\n"
    "description=Internet\n"
    "EOF\n\n", boxname);
  
  fprintf(outfile,
    "cat << EOF > ~/%s/gruppen/user/netcall.chg\n"
    "displayunit=1\n"
    "unit=1\n"
    "unitprice=0\n"
    "description=Netcalls\n"
    "EOF\n\n", boxname);
  
  fprintf(outfile,
    "cat << EOF > ~/%s/gruppen/system/netcall.chg\n"
    "displayunit=1\n"
    "unit=1\n"
    "unitprice=0\n"
    "description=Netcalls\n"
    "EOF\n\n", boxname);
  
  fprintf(outfile,
    "cat << EOF > ~/%s/gruppen/sysop/netcall.chg\n"
    "displayunit=1\n"
    "unit=1\n"
    "unitprice=0\n"
    "description=Netcalls\n"
    "EOF\n\n", boxname);
  
  fprintf(outfile,
    "cat << EOF > ~/%s/gruppen/user/profile\n"
    "timeout=180\n"
    "security=1\n"
    "internalflag.login_online=1\n"
    "internalflag.login_slip=1\n"
    "internalflag.login_netz=1\n"
    "internalflag.login_web=1\n"
    "internalflag.login_pop3=1\n"
    "EOF\n\n", boxname);
  
  fprintf(outfile,
    "cat << EOF > ~/%s/gruppen/gast/bretter\n"
    "/SYS/* -\n"
    "/* R\n"
    "EOF\n\n", boxname);
  
  fprintf(outfile,
    "cat << EOF > ~/%s/gruppen/gast/profile\n"
    "timeout=180\n"
    "security=1\n"
    "internalflag.login_online=1\n"
    "EOF\n\n", boxname);
  
  fprintf(outfile,
    "cat << EOF > ~/%s/gruppen/system/bretter\n"
    "/* RW\n"
    "EOF\n\n", boxname);
  
  fprintf(outfile,
    "cat << EOF > ~/%s/gruppen/system/profile\n"
    "security=1\n"
    "internalflag.login_netz=1\n"
    "EOF\n\n", boxname);

  #if 0  
  fprintf(outfile,
    "cat << EOF > ~/%s/system/oalias\n", boxname);

  if(maps)
    fprintf(outfile, "maps=maps@maps.zer\n");

  if(stricmp(sysop, "gast")!=0 && gast)
    fprintf(outfile, "gast=%s\n", sysop);

  fprintf(outfile, "EOF\n\n");
  #endif

  fprintf(outfile,
    "cat << EOF > ~/%s/system/profile\n"
    "boxname           = %s\n"
    "domain            = %s\n"
    "organization      = %s\n"
    "antrag.gruppe     = gast\n"
    "antrag.mail       = %s@%s.%s\n"
    "webonline.bgcolor = #ffffe0\n"
    "EOF\n\n", boxname, boxname, domain, org, sysop, boxname, domain);

  fprintf(outfile,
    "\ncat << EOF > ~/%s/system/login\n"
    "\n"
    "Willkommen in der %s-Mailbox!\n"
    "\n"
    "EOF\n\n", boxname, boxname);

  fprintf(outfile,
    "\ncat << EOF > ~/%s/system/logout\n"
    "\n"
    "Auf Wiedersehen!\n"
    "\n"
    "EOF\n\n", boxname);

  if(server[0]!=0)  
    fprintf(outfile,
      "cat << EOF > ~/%s/system/routing\n"
      "absmask: *\n"
      "\n"
      "Domainserver: %s\n"
      "Maxlen: 1000000\n"
      "Domain: .\n"
      "\n"
      "EOF\n\n", boxname, server);

  fprintf(outfile, "mkdir ~/%s/user/%s\n", boxname, boxname);
  fprintf(outfile, "mkdir ~/%s/user/%s/mail\n", boxname, boxname);

  fprintf(outfile,
    "\ncat << EOF > ~/%s/user/%s/profile\n"
    "Cryptedpassword = locked\n"
    "Domains         = %s;zer\n"
    "System          = 1\n"
    "Disablepop      = 1\n"
    "Gruppe          = system\n"
    "Packer          = Intern\n"
    "Eintragsdatum   = %ld\n"
    "EOF\n\n", boxname, boxname, domain, time((time_t *)NULL));

  if(bretter)
    fprintf(outfile,
      "\ncat << EOF > ~/%s/user/%s/feed\n"
      "/*\n"
      "EOF\n\n", boxname, boxname);

  fprintf(outfile,
    "\ncat << EOF > ~/%s/user/%s/maps\n"
    "EOF\n\n", boxname, boxname);

  if(ticket)
   {
    fprintf(outfile, "mkdir ~/%s/user/ticket\n", boxname);
    fprintf(outfile, "mkdir ~/%s/user/ticket/mail\n", boxname);

    fprintf(outfile,
      "\ncat << EOF > ~/%s/user/ticket/profile\n"
      "Cryptedpassword = locked\n"
      "Domains         = zer\n"
      "System          = 1\n"
      "Gruppe          = user\n"
      "Packer          = Intern\n"
      "Memo            = DBOX Ticketverwaltung\n"
      "Eintragsdatum   = %ld\n"
      "Disablepop      = 1\n"
      "EOF\n\n", boxname, time((time_t *)NULL));

    fprintf(outfile,
      "\ncat << EOF > ~/%s/user/ticket/maps\n"
      "EOF\n\n", boxname);
   }

  if(maps)
   {
    fprintf(outfile, "mkdir ~/%s/user/maps\n", boxname);
    fprintf(outfile, "mkdir ~/%s/user/maps/mail\n", boxname);

    fprintf(outfile,
      "\ncat << EOF > ~/%s/user/maps/maps\n"
      "EOF\n\n", boxname);

    fprintf(outfile,
      "\ncat << EOF > ~/%s/user/maps/profile\n"
      "Cryptedpassword = locked\n"
      "Domains         = zer\n"
      "System          = 0\n"
      "Gruppe          = system\n"
      "Memo            = DBOX Brettbestellsystem\n"
      "Packer          = Intern\n"
      "Eintragsdatum   = %ld\n"
      "Disablepop      = 1\n"
      "EOF\n\n", boxname, time((time_t *)NULL));
   }

  fprintf(outfile, "mkdir ~/%s/user/%s\n", boxname, sysop);
  fprintf(outfile, "mkdir ~/%s/user/%s/mail\n", boxname, sysop);

  fprintf(outfile,
    "\ncat << EOF > ~/%s/user/%s/profile\n"
    "Cryptedpassword = none\n"
    "Gruppe          = sysop\n"
    "Eintragsdatum   = %ld\n"
    "Usecolor        = 1\n"
    "EOF\n\n", boxname, sysop, time((time_t *)NULL));

  if(stricmp(sysop, "postmaster")!=0)
   {
    fprintf(outfile,
      "\ncat << EOF >> ~/%s/user/%s/alias\n"
      "postmaster\n"
      "EOF\n\n", boxname, sysop);

    fprintf(outfile, "mkdir ~/%s/user/postmaster\n", boxname);
    fprintf(outfile, "mkdir ~/%s/user/postmaster/mail\n", boxname);

    fprintf(outfile,
      "\ncat << EOF > ~/%s/user/postmaster/profile\n"
      "Alias           = %s\n"
      "Eintragsdatum   = %ld\n"
      "EOF\n\n", boxname, sysop, time((time_t *)NULL));
   }

  if(stricmp(sysop, "sysop")!=0)
   {
    fprintf(outfile,
      "\ncat << EOF >> ~/%s/user/%s/alias\n"
      "sysop\n"
      "EOF\n\n", boxname, sysop);

    fprintf(outfile, "mkdir ~/%s/user/sysop\n", boxname);
    fprintf(outfile, "mkdir ~/%s/user/sysop/mail\n", boxname);

    fprintf(outfile,
      "\ncat << EOF > ~/%s/user/sysop/profile\n"
      "Alias           = %s\n"
      "Eintragsdatum   = %ld\n"
      "EOF\n\n", boxname, sysop, time((time_t *)NULL));
   }

  if(server[0]!=0)
   {
    fprintf(outfile, "mkdir ~/%s/user/%s\n", boxname, server);
    fprintf(outfile, "mkdir ~/%s/user/%s/mail\n", boxname, server);

    fprintf(outfile,
      "\ncat << EOF > ~/%s/user/%s/profile\n"
      "Cryptedpassword = locked\n"
      "Domains         = %s\n"
      "System          = 1\n"
      "Allemps         = 1\n"
      "Disablepop      = 1\n"
      "Gruppe          = system\n"
      "Eintragsdatum   = %ld\n"
      "EOF\n\n", boxname, server, domain, time((time_t *)NULL));

    fprintf(outfile,
      "\ncat << EOF > ~/%s/user/%s/maps\n"
      "EOF\n\n", boxname, server);
   }
  
  if(gast)
   {
    fprintf(outfile, "mkdir ~/%s/user/gast\n", boxname);
    fprintf(outfile, "mkdir ~/%s/user/gast/mail\n", boxname);

    fprintf(outfile,
      "\ncat << EOF > ~/%s/user/gast/profile\n"
      "Cryptedpassword = none\n"
      "Gruppe          = gast\n"
      "Eintragsdatum   = %ld\n"
      "Dogastsettings  = 1\n"
      "EOF\n\n", boxname, time((time_t *)NULL));
   }

  if(server[0]!=0)  
    fprintf(outfile,
      "cat << EOF > ~/%s/system/routing\n"
      "absmask: *\n"
      "\n"
      "Domainserver: %s\n"
      "Maxlen: 1000000\n"
      "Domain: .\n"
      "\n"
      "EOF\n\n", boxname, server);

  if(bretter)
   {
    fprintf(outfile, "# Anlegen der Bretter\n\n");

    fprintf(outfile,
      "\ncat << EOF > ~/%s/bretter/sys/unzustellbar/brt/profile\n"
      "Forceextern   =0\n"
      "Type          =%d\n"
      "EOF\n\n", boxname, GROUPTYPE_ZCONNECT);

    fprintf(outfile,
      "\ncat << EOF > ~/%s/bretter/sys/unzustellbar/prv/profile\n"
      "Forceextern   =0\n"
      "Type          =%d\n"
      "EOF\n\n", boxname, GROUPTYPE_ZCONNECT);
   }
 }

void new_setup(consolet *console)
 {
  byte y=11; 
  int i;
  char tempstr[1000], boxname[TEXTLEN]="", server[TEXTLEN]="",
       sysop[TEXTLEN]="sysop", org[TEXTLEN]="", domain[TEXTLEN]="";
  bool gast;
  char *qjn[]={ "^Ja", "^Nein", (char *)NULL };
  FILE *outfile, *infile;

  clearworkscreen(console);

  console->setcursor(4, 11);
  console->printf("Dieses Programm installiert das DBOX Mailboxsystem.");
  console->setcursor(4, 12);
  console->printf("Wollen Sie fortfahren?");

  if(ask(q1, console, 3)!=0) goto quit;

  clearworkscreen(console);

  console->setattr(1, FALSE);
  console->setcursor(4, y++);
  console->printf("Teste dbsetup...");

  if(access("dbsetup", 00)!=0)
   {
    errorbox(console, "dbsetup nicht gefunden!");
    goto quit;
   }

  console->printf(" Ok");

  console->setcursor(4, y++);
  console->printf("Teste Permissions...");
  if(getuid()==0 || geteuid()==0)
   {
    errorbox(console, "Dieses Programm sollte nicht als ROOT gestartet werden!");
    goto quit;
   }
  console->printf(" Ok");

  sleep(1);

  clearworkscreen(console);
  console->setcursor(4, 11);
  console->printf("DBSetup wird jetzt einige Fragen zu Ihrer "
                 "Installation stellen.");
  console->setcursor(4, 12);
  console->printf("Wollen Sie fortfahren?");

  if(ask(q1, console, 3)!=0) goto quit;
  
  clearworkscreen(console);
  console->setcursor(4, 11);
  console->printf("Die lautet der Netzname Ihrer Mailbox?");
  console->setcursor(4, 13);
  console->printf("Der Netzname einer Mailbox sollte nicht mehr als "
                 "acht Zeichen haben");
  console->setcursor(4, 14);
  console->printf("und darf nur aus Buchstaben, Ziffern und dem Zeichen "
                 "\"-\" (Bindestrich)");
  console->setcursor(4, 15);
  console->printf("bestehen. Die Gross-/Kleinschreibung wird nicht beachtet.");
  console->setcursor(4, 17);
  console->printf("Netzname Ihrer Mailbox:");

 schleife:
  if(ask_text(boxname, console, 7)!=1) goto quit;
  if(boxname[0]==0) goto schleife;

  clearworkscreen(console);
  console->setcursor(4, 11);
  console->printf("Die lautet die Hauptdomain Ihrer Mailbox?");
  console->setcursor(4, 13);
  console->printf("Bitte geben Sie nur die Hauptdomain Ihrer Mailbox an.");
  console->setcursor(4, 14);
  console->printf("Bitte verwenden Sie keinen fuehrenden Punkt.");
  console->setcursor(4, 15);
  console->printf("Die Gross-/Kleinschreibung wird nicht beachtet.");
  console->setcursor(4, 17);
  console->printf("Hauptdomain Ihrer Mailbox:");

 schleife2:
  if(ask_text(domain, console, 7)!=1) goto quit;
  if(domain[0]==0) goto schleife2;

  clearworkscreen(console);
  console->setcursor(4, 11);
  console->printf("Die lautet der Netzname Ihrer Server-Mailbox?");
  console->setcursor(4, 13);
  console->printf("Die Server-Mailbox ist die Mailbox, die Ihre Mailbox mit "
                  "Daten aus");
  console->setcursor(4, 14);
  console->printf("Mailboxnetzwerken und E-Mail versorgt. Sie koennen dieses "
                  "Feld auch leer-");
  console->setcursor(4, 15);
  console->printf("lassen. Eine Routingtabelle wird dann nicht angelegt.");
  console->setcursor(4, 17);
  console->printf("Netzname Ihrer Server-Mailbox:");

  if(ask_text(server, console, 7)!=1) goto quit;
  
  clearworkscreen(console);
  console->setcursor(4, 11);
  console->printf("Die lautet der Username des Systembetreibers?");
  console->setcursor(4, 13);
  console->printf("Die Username des Systembetreibers sollte nicht mehr "
                 "als 30 Buchstaben um-");
  console->setcursor(4, 14);
  console->printf("fassen und darf nur aus Buchstaben und Ziffern bestehen.");
  console->setcursor(4, 15);
  console->printf("Die Gross-/Kleinschreibung wird nicht beachtet.");
  console->setcursor(4, 17);
  console->printf("Username des Systembetreibers (SYSOP):");

 schleife3:
  if(ask_text(sysop, console, 7)!=1) goto quit;
  if(sysop[0]==0) goto schleife3;

  clearworkscreen(console);
  console->setcursor(4, 11);
  console->printf("Die lautet der Name Ihrer Organisation/Firma?");
  if(ask_text(org, console, 2)!=1) goto quit;

  clearworkscreen(console);
  console->setcursor(4, 11);
  console->printf("Soll eine GAST-Kennung eingerichtet werden?");

  i=ask(qjn, console, 2);
  if(i==0)
    gast=TRUE;
  else if(i==1)
    gast=FALSE;
  else
    goto quit;

  clearworkscreen(console);
  console->setcursor(4, 11);
  console->printf("Das Programm legt nun eine Script-Datei an, die die "
                 "Installation ab-");
  console->setcursor(4, 12);
  console->printf("schliessen wird. Das Script wird folgendes durchfuehren:");
  console->setcursor(4, 13);
  console->printf(" - Anlegen der Verzeichnisstruktur");
  console->setcursor(4, 14);
  console->printf(" - Kopieren der Programmdateien");
  console->setcursor(4, 15);
  console->printf(" - Erstellung der Konfigurationsdateien");
  console->setcursor(4, 17);
  console->printf("Starten Sie nun dieses Script.");

  dnstring(boxname);
  dnstring(domain);
  dnstring(sysop);
  dnstring(server);

  sprintf(tempstr, "%s/dosetup.sh", getenv("HOME"));
  outfile=fopen(tempstr, "wt");
  fprintf(outfile, SHELLSCRIPT
          "\n"
          "#\n"
          "#  DBOX - Setup Script\n"
          "#\n"
          "\n");

  fprintf(outfile, "# Umgebungsvariablen\n"
          "\n"
          "export DIST=\"%s\"\n", distdir);
  if(gast)
    fprintf(outfile, "GUESTDIR=~/%s/user/gast/\n", boxname);

  fprintf(outfile, "\n");

  config_box(outfile,
    boxname,
    domain,
    org,
    sysop,
    server,
    TRUE,          // gast
    TRUE,          // maps
    FALSE,         // ticket
    TRUE);         // bretter

  fprintf(outfile, "# Anlegen weiterer Verzeichnisse\n"
          "\n"
          "echo Anlegen weiterer Verzeichnisse\n"
          "\n");

  fprintf(outfile, "mkdir ~/bin\n"
          "mkdir ~/chat\n"
          "mkdir ~/html\n"
          "mkdir ~/port\n"
          "mkdir ~/port/0\n"
          "mkdir ~/port/0/tmp\n"
          "mkdir ~/port/1\n"
          "mkdir ~/port/1/tmp\n"
          "mkdir ~/system\n"
          "mkdir ~/system/clients\n"
          "mkdir ~/system/clients/localhost.dbox\n"
          "mkdir ~/tmp\n"
          "mkdir ~/tools\n");

  fprintf(outfile,
    "mkdir ~/duucp\n"
    "mkdir ~/duucp/log\n"
    "mkdir ~/duucp/system\n");

  fprintf(outfile, "\n");

  infile=fopen("dbsetup/dosetup.sh", "rt");
  if(infile!=NULL)
   {
    filecopy(infile, outfile, filelength(fileno(infile)));
    fclose(infile);
   }

  fprintf(outfile, "\n");

  fprintf(outfile, "# Erstellung der Konfigurationsdateien\n\n"
                   "echo Erstellung der Konfigurationsdateien\n\n");

  fprintf(outfile,
    "\ncat << EOF > ~/system/username\n"
    "errormsg=Der Username ist zu kurz!\n"
    "deny: ?\n"
    "deny: ??\n"
    "deny: ???\n"
    "deny: ????\n"
    "\n"
    "errormsg=Der Username ist zu lang!\n"
    "deny: ??????????????????????????????*\n"
    "\n"
    "errormsg=Der Username ist nicht erwnscht!\n"
    "deny: *hitler*\n"
    "deny: *rambo*\n"
    "deny: *arschloch*\n"
    "deny: *sysop*\n"
    "deny: *killer*\n"
    "deny: *goebbels*\n"
    "deny: *zentrale*\n"
    "deny: *hacker*\n"
    "deny: *amiga*\n"
    "deny: *atari*\n"
    "deny: *mr.*\n"
    "deny: *zerberus*\n"
    "deny: *janus*\n"
    "deny: *zconnect*\n"
    "deny: *maps*\n"
    "deny: *test*\n"
    "deny: *keiner*\n"
    "deny: *niemand*\n"
    "deny: *nobody*\n"
    "deny: *webmaster*\n"
    "deny: *postmaster*\n"
    "deny: *kroening*\n"
    "deny: *info*\n"
    "deny: *anmeldung*\n"
    "deny: *email*\n"
    "deny: ppp-*\n"
    "deny: slip-*\n"
    "deny: cslip-*\n"
    "deny: .*\n"
    "deny: _*\n"
    "deny: -*\n"
    "EOF\n\n");

  fprintf(outfile,
    "cat << EOF > ~/system/dbsetup\n"
    "Version=" DBOXVERSION "\n"
    "Versionnr=" DBOXVERSIONNR "\n"
    "\n"
    "EOF\n\n");

  fprintf(outfile,
    "# DUUCP\n"
    "\n"
    "cat << EOF > ~/duucp/system/newsgrps\n"
    "EOF\n\n");

  fprintf(outfile,
    "\ncat << EOF > ~/port/1/profile\n"
    #ifdef OS_WIN32
    "Console=tty\n"
    #else
    "Console=9\n"
    "; Specify the console device of the port\n"
    #endif
    "\n"
    "Telefon=01234/56789\n"
    "; Specify the phone number of the port\n"
    "\n"
    "Comport=1\n"
    "; Specify the device the modem is connected to\n"
    "; 0 = /dev/cua0\n"
    "; 1 = /dev/cua1\n"
    "; usw.\n"
    "\n"
    "DTE=38400\n"
    "; Specify the communication bit rate\n"
    "\n"
    "EOF\n\n");

  fprintf(outfile,
    "\ncat << EOF > ~/port/1/device\n"
    "Modemname=Supermodem High-Speed!\n"
    "; Tragen Sie bitte hier den Namen des Modems ein\n"
    "\n"
    "Init=AT\n"
    "; Tragen Sie bitte hier den Initstring des Modems ein.\n"
    "; Vorsicht: Manche Modems \"vergessen\" bei ATZ oder AT&F die DTE!\n"
    "\n"
    "Abheben=ATA\n"
    "; Dieses Kommando sollte das Modem dazu bringen, einen Anruf anzunehmen\n"
    "\n"
    "Waehlen=ATDT\n"
    "; Benutzen Sie ATDP, wenn Sie Pulswahl benoetigen.\n"
    "\n"
    "EOF\n\n");

  fprintf(outfile,
    "\ncat << EOF > ~/system/flags\n"
    "VERWALTUNG\n"
    "18\n"
    "EOF\n\n");

  fprintf(outfile,
    "\ncat << EOF > ~/system/mclass\n"
    "Name: local\n"
    "Mask: %s.%s\n"
    "\n"
    "Name: national\n"
    "Mask: de\n"
    "Mask: sub.org\n"
    "Mask: sub.net\n"
    "Mask: sub.com\n"
    "Mask: xlink.net\n"
    "Mask: maz.net\n"
    "Mask: germany.net\n"
    "Mask: zer\n"
    "\n"
    "Name: international\n"
    "Mask: *\n"
    "EOF\n\n", boxname, domain);

  fprintf(outfile,
    "cat << EOF > ~/system/profile\n"
    "boxname       = %s\n"
    "domain        = %s\n"
    "organization  = %s\n"
    "mwstsatz      = 160\n"
    "EOF\n\n", boxname, domain, org);

  fprintf(outfile,
    "cat << EOF > ~/bin/callsysop\n"
    SHELLSCRIPT
    "#\n"
    "# Hier koennen Sie z.B. eine Audiodatei abspielen lassen.\n"
    "#\n"
    "EOF\n\n");

  fprintf(outfile, "chmod a+rx ~/bin/callsysop\n\n");

  fprintf(outfile,
    "cat << EOF > ~/bin/dospool\n"
    SHELLSCRIPT
    "#\n"
    "# DBOX-Spool Script\n"
    "#\n"
    "\n"
    "cd ~/bin\n"
    "\n"
    "# Erstmal gucken, ob noch genug Plattenplatz da ist.\n"
    "if df ~ | ~/bin/checkdf 80 ; then\n"
    "  echo \"------------------------\"\n"
    "  echo \"  Die Platte ist voll!  \"\n"
    "  echo \"------------------------\"\n"
    "  sleep 1000d\n"
    "fi\n"
    "\n"
    "# Puffer auspacken\n"
    "nice ~/bin/prearc -e\n"
    "\n"
    "# News spool verteilen\n"
    "nice ~/bin/spoolzcn\n"
    "\n"
    "# E-Mail verteilen\n"
    "nice ~/bin/spoolzcm\n"
    "\n"
    "# News in die Bretter einsortieren\n"
    "nice ~/bin/onews\n"
    "\n"
    "# Maps starten\n"
    "nice ~/bin/maps\n"
    "\n"
    "# Vorpacken\n"
    "nice ~/bin/prearc\n"
    "\n");

  fprintf(outfile,
    "# DUUCP\n"
    "#\n"
    "# nach Bedarf aktivieren:\n"
    "#\n"
    "# duucp 7 %s\n"
    "#\n"
    "#  ODER\n"
    "#\n"
    "# duucp 8 %s\n"
    "#\n"
    "\n", server, server);

  fprintf(outfile,
    "# cron: Postfcher crunchen\n"
    "if [ -e ~/%s/todo/cron.crunch.mail ] ; then\n"
    "  rm ~/%s/todo/cron.crunch.mail\n"
    "  omail -c\n"
    "fi\n\n", boxname, boxname);

  fprintf(outfile,
    "# Warten\n"
    "sleep 10m\n"
    "\n"
    "# und wieder von vorne\n"
    "cd ~/bin\n"
    "exec dospool\n"
    "EOF\n\n");

  fprintf(outfile, "chmod a+rx ~/bin/dospool\n\n");

   {
    struct passwd *pw;

    fprintf(outfile,
      "cat << EOF > ~/bin/duucp.cfg\n"
      "site=%s\n"
      "domain=%s\n"
      "gateid=UU\n"
      "maillogfile=%s/duucp/log/uucprout.log\n"
      "logfile=%s/duucp/log/duucp.log\n"
      "homedir=%s/duucp\n"
      "maxspool=5000\n",
         boxname, domain, getenv("HOME"), getenv("HOME"), getenv("HOME"));

    pw=getpwnam("uucp");
    if(pw==NULL)
      fprintf(outfile, "uucpuid=0\n");
    else
      fprintf(outfile, "uucpuid=%d\n", pw->pw_uid);

    fprintf(outfile, "dboxuid=%d\n"
      "EOF\n\n", getuid());
   }

  fprintf(outfile,
    "cat << EOF > ~/bin/ocon\n"
    "~/bin/oserv -i\n"
    "EOF\n\n");

  fprintf(outfile, "chmod a+rx ~/bin/ocon\n\n");

  fprintf(outfile,
    "cat << EOF > ~/bin/online_script\n"
    "~/bin/oserv $1 $2 $3 $4\n"
    "EOF\n\n");

  fprintf(outfile, "chmod a+rx ~/bin/online_script\n\n");

  fprintf(outfile, 
    "\n"
    "echo \"Fertig!\"\n"
    "echo\n"
    "echo \"Mit dem Befehl ~/bin/ocon knnen Sie nun einen\"\n"
    "echo \"Consolenport starten. Melden Sie sich als User\"\n"
    "echo \"%s an, und vergessen Sie nicht, fr diesen User\"\n"
    "echo \"ein Passwort einzurichten.\"\n"
    "echo\n", sysop);

  fclose(outfile);

  sprintf(tempstr, "chmod a+rx ~/dosetup.sh");
  system(tempstr);
  
   {
    char *qe[]={ "^Ende", (char *)NULL };
    ask(qe, console, 8);
   }

  showweiter=TRUE;

 quit:;
 }

void add_virtual_box(consolet *console)
 {
  char boxname[TEXTLEN]="", domain[TEXTLEN]="", tempstr[DIRLEN]="",
       sysop[TEXTLEN]="sysop";
  FILE *outfile;

  clearworkscreen(console);

  console->setcursor(4, 11);
  console->printf("Die lautet der Netzname der virtuellen Mailbox?");
  console->setcursor(4, 13);
  console->printf("Der Netzname einer Mailbox sollte nicht mehr als "
                 "acht Zeichen haben");
  console->setcursor(4, 14);
  console->printf("und darf nur aus Buchstaben, Ziffern und dem Zeichen "
                 "\"-\" (Bindestrich)");
  console->setcursor(4, 15);
  console->printf("bestehen. Die Gross-/Kleinschreibung wird nicht beachtet.");
  console->setcursor(4, 17);
  console->printf("Netzname der virtuellen Mailbox:");

 schleife:
  if(ask_text(boxname, console, 7)!=1) return;
  if(boxname[0]==0) goto schleife;

  clearworkscreen(console);
  console->setcursor(4, 11);
  console->printf("Die lautet die Hauptdomain der virtuellen Mailbox?");
  console->setcursor(4, 13);
  console->printf("Bitte geben Sie nur die Hauptdomain der Mailbox an.");
  console->setcursor(4, 14);
  console->printf("Bitte verwenden Sie keinen fuehrenden Punkt.");
  console->setcursor(4, 15);
  console->printf("Die Gross-/Kleinschreibung wird nicht beachtet.");
  console->setcursor(4, 17);
  console->printf("Hauptdomain der virtuellen Mailbox:");

 schleife2:
  if(ask_text(domain, console, 7)!=1) return;
  if(domain[0]==0) goto schleife2;

  clearworkscreen(console);
  console->setcursor(4, 11);
  console->printf("Das Programm legt nun eine Script-Datei an, die die "
                 "Installation ab-");
  console->setcursor(4, 12);
  console->printf("schliessen wird. Das Script wird folgendes durchfuehren:");
  console->setcursor(4, 13);
  console->printf(" - Anlegen der Verzeichnisstruktur");
  console->setcursor(4, 14);
  console->printf(" - Kopieren der Programmdateien");
  console->setcursor(4, 15);
  console->printf(" - Erstellung der Konfigurationsdateien");
  console->setcursor(4, 17);
  console->printf("Starten Sie nun dieses Script.");

  dnstring(boxname);
  dnstring(domain);
  dnstring(sysop);

  sprintf(tempstr, "%s/dosetup.sh", getenv("HOME"));
  outfile=fopen(tempstr, "wt");
  fprintf(outfile, SHELLSCRIPT
          "\n"
          "#\n"
          "#  DBOX - Setup Script\n"
          "#\n"
          "# Hinzufuegen einer virtuellen Box\n"
          "# Hauptbox: %s\n"
          "#\n"
          "\n", config.boxname);

  config_box(outfile,
    boxname,
    domain,
    "",             // org
    sysop,
    config.boxname, // server
    FALSE,          // gast
    FALSE,          // maps
    FALSE,          // ticket
    FALSE);         // bretter

  fprintf(outfile,
          "\n"
          "(cd ~/%s ; ln -s ~/%s/bretter )\n"
          "\n", boxname, config.boxname);

  fprintf(outfile, 
    "\n"
    "echo \"Fertig!\"\n"
    "echo\n"
    "echo \"Mit dem Befehl ~/bin/oserv -b%s knnen Sie nun einen\"\n"
    "echo \"Consolenport der virtuellen Box starten. Melden Sie sich\"\n"
    "echo \"als User %s an, und vergessen Sie nicht, fr diesen User\"\n"
    "echo \"ein Passwort einzurichten.\"\n"
    "echo\n", boxname, sysop);

  fclose(outfile);

  sprintf(tempstr, "chmod a+rx ~/dosetup.sh");
  system(tempstr);

   {
    char *qe[]={ "^Ende", (char *)NULL };
    ask(qe, console, 8);
   }

  showweiter=TRUE;
 }

int update_setup(consolet *console)
 {
  char filename[DIRLEN], *home;

  home=getenv("HOME");
  if(home==NULL)
    return TRUE;

  snprintf(filename, DIRLEN-1, "%s" DIRCHAR "system/profile", home);

  if(access(filename, 00)!=0)
    return TRUE;

  if(config.read(""))
   {
    chdir(config.cwd);
    return TRUE;
   }

  chdir(config.cwd);

  clearworkscreen(console);

  console->setcursor(4, 11);
  console->printf("Es wurde eine vorhandene DBOX-Installation entdeckt.");
  console->setcursor(4, 12);
  console->printf("Wie wollen Sie fortfahren?");

  char *q1[]=
   {
    "^Konfiguration der Installation",
    "^Neuinstallation",
    "^Abbruch",
    (char *)NULL
   };

  switch(ask(q1, console, 3))
   {
   case 0:
    break;
   
   case 1:
    return TRUE;

   default:
    return FALSE;
   }

  clearworkscreen(console);

  console->setcursor(4, 11);
  console->printf("Hauptmailbox: %s", config.boxname);
  console->setcursor(4, 13);
  console->printf("Bitte waehlen Sie eine Option:");

  char *q2[]=
   {
    "^Virtuelle Box hinzufuegen",
    "^Programm beenden",
    (char *)NULL
   };

  switch(ask(q2, console, 5))
   {
   case 0:
    add_virtual_box(console);
    break;
   }

  return FALSE;
 }

void setup()
 {
  consolet console;

  console.assign("");
  console.open();

  console.col1v=config.col1v;
  console.col2v=config.col2v;
  console.col3v=config.col3v;
  console.col1h=config.col1h;
  console.col2h=config.col2h;
  console.col3h=config.col3h;

  console.setattr(1, FALSE);
  drawscreen(&console, "Willkommen", 
    "Version " DBOXVERSIONNR " vom " DBOXVERSION);

  if(update_setup(&console))
   {
    if(distdir==NULL || distdir[0]==0)
      errorbox(&console, "Please use \"make install\"!");
    else
      new_setup(&console);
   }

  console.setcursor(1, 1);
  console.setattr(0xff, FALSE);
  console.clrscr();
  console.close();

  if(showweiter)
    printf(
      "So geht es weiter:\n\n"
      "In Ihrem Home-Verzeichnis finden Sie eine Datei \"dosetup.sh\".\n"
      "Ueberpruefen Sie den Inhalt dieser Datei und starten Sie sie dann.\n"
      "Dieses Script legt die Verzeichnisse und Konfigurationsdateien an\n"
      "und schliesst so die Installation ab.\n"
      "\n");
  else
    printf("Bye!\n\n");
 }

int main(int argc, char **argv)
 {
  if(init_libdbox(LIBDBOXVERSION, LIBDBOXVERSIONNR)) return 1;

  cmdlinet cmdline;
 
  if(cmdline.parse(argc, argv, "?hd:") || 
     cmdline.isset('?') ||
     cmdline.isset('h'))
   {
    fprintf(stderr,
      "\n"
      "* * DBSETUP - Installing DBOX - Version " DBOXVERSIONNR " * *\n"
      "\n"
      "Usage:\n"
      "\n"
      " dbsetup [-?] [-h]               show help\n"
      " dbsetup -d distrib. dir.        start installation\n"
      "\n"
      "Weitere Optionen:\n"
      "\n");
    return 0;
   }
  
  distdir=cmdline.getval('d');
  setup();
  
  return 0;
 }

