#include <sys/types.h>
#include <stdlib.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/signal.h>
#include <sys/file.h>
#include <sys/syslog.h>
#include <netdb.h>
#include <pwd.h>
#include <sys/stat.h>

#include <sys/termios.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <net/if.h>

#include <stdio.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>

#ifdef OS_UNIX

#ifndef N_SLIP
#define N_SLIP SLIPDISC /* slip line discipline */
#endif

/* Different names for the same beast. */
#ifndef TIOCMODG                        /* BSD 4.3 */
#  ifdef TIOCMGET
#    define TIOCMODG TIOCMGET           /* Posix */
#  else
#    ifdef MCGETA
#      define TIOCMODG MCGETA           /* HP/UX */
#    endif
#  endif
#endif

#define FALSE 0
#define TRUE 1

volatile int hup;

void hup_handler(int s)
 {
  hup=TRUE;
 }
 
int dcd(void)
 {
  int mcs;
  if(ioctl(0, TIOCMODG, &mcs)!=0) return 0;
  return(mcs&TIOCM_CAR?1:0);
 }

int doslip(const char *ip, int cslip, int checkcarrier)
 {
  struct termios tios, otios;
  int ldisc, odisc, slip_mode;
  char unit[100];
  char command[1000], hostname[1000];

  // struct sigaction sa;

  // Go to root
  (void) setuid(0);

  /*
  if(setsid()!=0) 
   {
    printf("setsid() failed\r\n");
    return 3;
   }*/

  /*if(ioctl(0, TIOCSCTTY, (caddr_t)0)!=0)
   {
    printf("Warning: can't set my controlling tty\r\n");
    return 4;
   }*/

  if(tcgetattr(0, &tios)<0) 
   {
    printf("tcgetattr failed!\r\n");
    return 5;
   }

  otios=tios;

  cfmakeraw(&tios);
  tios.c_iflag&=~IMAXBEL;

  if(tcsetattr(0, TCSAFLUSH, &tios)<0)
   {
    printf("tcsetattr() failed!\r\n");
    return 6;
   }

  /* find out what ldisc we started with */
  if(ioctl(0, TIOCGETD, (caddr_t)&odisc)<0)
   {
    printf("ioctl(0, TIOCGETD, ...) failed!\r\n");
    return 7;
   }

  printf("Welcome to DBOX-SLIP-Interface.\r\n"
         "Your IP is: %s\r\n"
         "\r\nStarting %s\r\n", ip, cslip?"CSLIP":"SLIP");

  fflush(stdout);
  tcdrain(0);
  sleep(1);

  ldisc = N_SLIP;

  if(ioctl(0, TIOCSETD, (caddr_t)&ldisc) < 0) 
   {
    printf("ioctl(0, TIOCSETD, ...) failed!\r\n");
    return 9;
   }
  
  /* find out what unit number we were assigned */

  if (ioctl(0, SIOCGIFNAME, (caddr_t)unit)<0)
   {
    printf("ioctl(0, SIOCGIFNAME, ...) failed!\r\n");
    return 10;
   }
    
  /* set slip mode */

  if(cslip)
    slip_mode=1;
  else
    slip_mode=0;

    // "normal",	0,	/* SL_MODE_SLIP */
    // "compressed",	1,	/* SL_MODE_CSLIP */

  if(ioctl(0, SIOCSIFENCAP, (caddr_t)&slip_mode)<0)
   {
    printf("ioctl(0, SLIOCSFLAGS, ...) failed!\r\n");
    return 11;
   }

  gethostname(hostname, 999);

  sprintf(command, 
    "/sbin/ifconfig %s %s pointopoint %s mtu 1500", unit, hostname, ip);

  system(command);

  sprintf(command, 
    "/sbin/route add %s %s", ip, unit);
  system(command);
 
  /* twiddle thumbs until we get a signal */
  if(checkcarrier)
   {
    while(dcd()) sleep(10); 
   }
  else
   {
    hup=FALSE;

    //if(setsid()>=0)
    // {     
    //  if(ioctl(0, TIOCSCTTY, 0)>=0)
    //   {     
          signal(SIGHUP, hup_handler);
          while(!hup) pause();
    //   }
    //  else
    //   { 
    //    printf("TIOCSCTTY failed!\r\n");
    //   }
    // }
    //else
    // { 
    //  printf("setsid failed!\r\n");
    // }
   }

  ioctl(0, TIOCSETD, (caddr_t)&odisc);
  tcsetattr(0, TCSAFLUSH, &otios);

  alarm(10);
  close(0);

  sprintf(command, 
    "/sbin/route del %s", ip);
  system(command);

  sprintf(command, 
    "/sbin/ifconfig %s down", unit);

  system(command);

  return 0;
 }

int doppp(const char *ip, int checkcarrier)
 {
  char command[1000], hostname[1000];

  gethostname(hostname, 999);

  sprintf(command, "/usr/sbin/pppd %s:%s auth"
                   " lock netmask 255.255.255.255"
                   " noipx", hostname, ip);

  if(checkcarrier)
    sprintf(command+strlen(command), " modem");
  else
    sprintf(command+strlen(command), " local");

  system(command);
 
  return 0;
 }

int main(int argc, char **argv)
 {
  int cslip=FALSE, checkcarrier=FALSE, ppp=FALSE;
  int erg, i;
  char *ip, *tptr;

  if(geteuid()!=0)
   {
    printf("Not suid root!\r\n");
    return 1;
   }

  if(argc<2)
   {
   call:
    printf("Call: doslip 1.2.3.4 [-c] [-v] [-p]\r\n"
           " -c: Carrier check\r\n"
           " -v: CSLIP\r\n"
           " -p: PPP\r\n");
    return 2;
   }

  ip=argv[1];

  for(tptr=ip; *tptr!=0; tptr++)
    if(!isdigit(*tptr) && *tptr!='.')
      goto call;

  for(i=2; i<argc; i++)
   {
    if(argv[i][0]!='-') goto call;
    switch(argv[i][1])
     {
     case 'c': checkcarrier=TRUE; break;
     case 'v': cslip=TRUE; break;
     case 'p': ppp=TRUE; break;
     default: goto call;
     }
   }

  if(checkcarrier)
   {
    if(!dcd())
     {
      printf("Warning: no carrier!\r\n");
      sleep(10);
     }
   }

  if(ppp)
    erg=doppp(ip, checkcarrier);
  else
    erg=doslip(ip, cslip, checkcarrier);

  return 0;  
 }

#else

int main()
 {
  return 0;
 }

#endif
