#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <time.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>

#include <minmax.h>

#include <dboxpath.h>
#include <config.h>
#include <logf.h>
#include <oldtools.h>
#include <account.h>
#include <parsetime.h>
#include <charsettools.h>
#include <tools.h>

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

bool operator<(amountt a, amountt b)
 {
  byte newscale=max(a.scale, b.scale);
  return rescale(a, newscale).amount<rescale(b, newscale).amount;  
 }

bool operator>(amountt a, amountt b)
 {
  byte newscale=max(a.scale, b.scale);
  return rescale(a, newscale).amount>rescale(b, newscale).amount;  
 }

bool operator<=(amountt a, amountt b)
 {
  byte newscale=max(a.scale, b.scale);
  return rescale(a, newscale).amount<=rescale(b, newscale).amount;  
 }

bool operator>=(amountt a, amountt b)
 {
  byte newscale=max(a.scale, b.scale);
  return rescale(a, newscale).amount>=rescale(b, newscale).amount;  
 }

bool operator==(amountt a, amountt b)
 {
  byte newscale=max(a.scale, b.scale);
  return rescale(a, newscale).amount==rescale(b, newscale).amount;  
 }

bool operator!=(amountt a, amountt b)
 {
  byte newscale=max(a.scale, b.scale);
  return rescale(a, newscale).amount!=rescale(b, newscale).amount;  
 }

amountt operator+(amountt a, amountt b)
 {
  amountt result;

  result=a;
  result+=b;

  return result;
 }

amountt operator-(amountt a, amountt b)
 {
  amountt result;

  result=a;
  result-=b;

  return result;
 }

amountt amountt::operator+=(amountt a)
 {
  byte newscale=max(a.scale, scale);

  *this=rescale(*this, newscale);
  a=rescale(a, newscale);

  amount+=a.amount;

  return *this;
 }

amountt amountt::operator-=(amountt a)
 {
  byte newscale=max(a.scale, scale);

  *this=rescale(*this, newscale);
  a=rescale(a, newscale);

  amount-=a.amount;

  return *this;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

void bankpt::setfilename()
 {
  snprintf(path, DIRLEN, BOXUSERDIR, pconfig->boxdir, name);
  strmaxcpy(locktype, "bankprofile", TEXTLEN-1);
  snprintf(filename, DIRLEN, "%seinzug", path);
 }
 
bool bankpt::add()
 {
  return TRUE;
 }

bool bankpt::del()
 {
  return TRUE;
 }



// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

amountt abs(amountt a)
 {
  a.amount=labs(a.amount);
  return a;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

void chargeinfopt::setfilename()
 {
  snprintf(path, DIRLEN, BOXGRUPPENDIR, pconfig->boxdir, group);
  snprintf(filename, DIRLEN, "%s%s.chg", path, name);
  strmaxcpy(locktype, "chargeinfo", TEXTLEN-1);
 }

bool chargeinfopt::add()
 {
  return TRUE;
 }

bool chargeinfopt::del()
 {
  return TRUE;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

amountt getbalance(userpt *up)
 {
  amountt balance;
  const char *konto_scale=*up|"konto.scale";

  if(konto_scale[0]==0)
    balance.scale=config.currency_scale;
  else
    balance.scale=atoi(konto_scale);

  balance.amount=atol(*up|"konto");

  return balance;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

bool checkcredit(userpt *up, grouppt *gp)
 {
  char tempstr[TEXTLEN];
  amountt balance=getbalance(up), credit;

  if(atoi(*up|"kontoueberzug")) return FALSE;
  
  // check balance in user profile

  if(balance.amount>0) return FALSE;

  // check credit in user profile

  up->get("credit.amount", tempstr);
  if(tempstr[0]!=0)
   {
    credit.amount=atol(tempstr);
    credit.scale=atoi(*up|"credit.scale");
    
    return abs(balance)>=credit;
   }

  // check credit in group profile

  if(gp!=NULL)
   {
    credit.amount=atol(*gp|"credit.amount");
    credit.scale=atoi(*gp|"credit.scale");
    
    return abs(balance)>=credit;
   }

  return TRUE;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

signed long power10(int n)
 {
  int result=1;

  for(int i=0; i<n; i++)
    result*=10;
    
  return result;
 }

amountt rescale(amountt amount, byte newscale)
 {
  if(newscale>amount.scale)
    amount.amount*=power10(newscale-amount.scale);
  else if(newscale<amount.scale)
    amount.amount/=power10(amount.scale-newscale);
    
  amount.scale=newscale;

  return amount;
 }

amountt parse_currency_string(const char *string)
 {
  amountt amount;
  const char *tptr, *tptr2;
  int sign=1;
  
  // Skip leading spaces

  while(string[0]==' ') string++;

  // Check for sign

  if(string[0]=='-') 
   {
    sign=-1;
    string++;
   }

  amount.scale=0;
  amount.amount=atol(string);
  tptr=string;
  while(*tptr!=0 && *tptr!='.' && *tptr!=',')
    tptr++;

  if(*tptr==0) { amount.amount*=sign; return amount; }
  tptr++;

  tptr2=tptr;
  while(isdigit(*tptr2))
   {
    tptr2++;
    amount.scale++;
   }

  amount.amount*=power10(amount.scale);

  amount.amount+=atol(tptr);
  amount.amount*=sign;
  
  return amount;
 }

void currency_string(char *string, 
                     amountt amount,
                     bool add_currency_symbol)
 {
  byte display_scale;
  unsigned long divisor;
  char format_string[TEXTLEN];
  int i;

  display_scale=max(amount.scale, config.currency_scale);

  divisor=power10(amount.scale);
  
  snprintf(string, TEXTLEN, "%s%lu", amount.amount<0?"-":"", 
    labs(amount.amount)/divisor);

  if(display_scale>0)
   {
    snprintf(string+strlen(string), TEXTLEN-strlen(string), ",");

    if(amount.scale!=0)
     {
      snprintf(format_string, TEXTLEN, "%%0%dlu", amount.scale);
      snprintf(string+strlen(string), TEXTLEN-strlen(string),
        format_string, labs(amount.amount)%divisor);
     }

    for(i=amount.scale; i<display_scale; i++)
      snprintf(string+strlen(string), TEXTLEN-strlen(string), "0");
   }

  if(add_currency_symbol)
    snprintf(string+strlen(string), TEXTLEN-strlen(string),
      " %s", config.currency_symbol);
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

bool checkchargemenu(userpt *up, grouppt *gp, 
                     const char *menu, char *error)
 {
  chargeinfopt ci;
  amountt unitprice;

  strmaxcpy(ci.name, menu, TEXTLEN-1);
  strmaxcpy(ci.group, *up|"gruppe", TEXTLEN-1);

  if(ci.read())
   {
    strcpy(error, "Access denied!");
    return TRUE;
   }

  if(getfreedata(up, "freetime", menu)>0)
    return FALSE;

  unitprice=parse_currency_string(ci|"unitprice");

  if(unitprice.amount==0)
    return FALSE;

  if(checkcredit(up, gp))
   {
    strcpy(error, "Konto leer!");
    return TRUE;
   }

  return FALSE;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

static time_t unit_round(time_t t, dword unit)
 {
  if((((dword)t)/unit*unit)==(dword)t)
    return t;

  return ((t/unit)+1)*unit;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

void getchargeinfo(chargeinfopt &ci, time_t start, time_t &end)
 {
  FILE *infile;
  bool done=FALSE;
  char id[TEXTLEN], text[TEXTLEN];
  long int time_check_min=-1, time_check_max=-1, second_of_day;
  struct tm *tm;

  tm=localtime(&start);
  second_of_day=tm->tm_hour*3600+tm->tm_min*60+tm->tm_sec;

  ci.clear();

  infile=fopen(ci.getfilename(), "rt");
  if(infile==NULL) return;

  while(!done)
   {
    switch(readconfigline(infile, id, text, TEXTLEN-1))
     {
     case 0:
      if(stricmp(id, "time.check.min")==0)
        time_check_min=parsetime(text);
      else if(stricmp(id, "time.check.max")==0)
        time_check_max=parsetime(text);
      else if(stricmp(id, "permit")==0)
       {
        if(!time_check(second_of_day, time_check_min, time_check_max))
         {
          time_t end_max=0;

          done=TRUE;

          tm=localtime(&start);
          tm->tm_hour=0;
          tm->tm_min=0;

          if(time_check_min==-1 && time_check_max==-1)
           {
            // nothing
           }
          else if(time_check_min==-1)
           {
            tm->tm_sec=time_check_max;
            end_max=mktime(tm);
           }
          else if(time_check_max==-1)
           {
            tm->tm_sec=0;
            tm->tm_mday++;
            end_max=mktime(tm);
           }
          else if(time_check_min<=time_check_max)
           {
            tm->tm_sec=time_check_max;
            end_max=mktime(tm);
           }
          else // min > max
           {
            tm->tm_sec=time_check_max;
            if(second_of_day>=time_check_min)
              tm->tm_mday++;
            end_max=mktime(tm);
           }

          if(end_max!=0 && end>end_max) end=end_max;
         }
       }
      else
        ci.set(id, text);
      break;

     case 1:
      done=TRUE;
      break;

     default:
      logf.printf("getchargeinfo", "error in %s", ci.getfilename());
      done=TRUE;
     }
   }

  fclose(infile);
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

#if 0
void chargemenu(userpt *up, const char *menu, const char *zweck, 
                time_t start, time_t ende)
 {
  time_t t_neu, t, t_freetime;
  dword number_of_units;
  amountt charge, unitprice;
  char zeit[TEXTLEN], start_str[TEXTLEN], ende_str[TEXTLEN];
  buchungt buchung;
  struct tm *tm;
  chargeinfopt ci;
  dword unit, displayunit;

  if(ende<=start)
    ende=start+1;

  t=ende-start;

  strmaxcpy(ci.name, menu, TEXTLEN-1);
  strmaxcpy(ci.group, *up|"gruppe", TEXTLEN-1);

  if(ci.read())
   {
    logf.printf("chargemenu", "Failed to charge %ld seconds to "
                "user %s for menu %s: no charge profile",
                t, up->name, menu);
    return;
   }

  unit=atoi(ci|"unit");
  if(unit==0) unit=1;

  displayunit=atoi(ci|"displayunit");
  if(displayunit==0) 
    displayunit=unit;
  else if(displayunit>unit) 
    unit=displayunit;

  t=unit_round(t, displayunit);
  t_freetime=chargefreedata(up, "freetime", menu, t);

  t_neu=unit_round(t_freetime, unit);

  if(t>0)
   {
    if(t_neu>0)
     {
      number_of_units=t_neu/unit; // Division sollte ohne Rest gehen
      unitprice=parse_currency_string(ci|"unitprice");
      charge.amount=unitprice.amount*number_of_units;
      charge.scale=unitprice.scale;
     }
    else
     {
      charge.amount=0;
      charge.scale=0;
     }

    if(charge.amount>0 || t!=t_neu)
     {
      if(displayunit==60 && unit==60)
       {
        if(t_freetime==t)
          snprintf(zeit, TEXTLEN, "%ldm", t/60);
        else   
          snprintf(zeit, TEXTLEN, "%ldm, davon %ld frei", 
            t/60, (t-t_neu)/60);
       }
      else
       {
        if(t_freetime==t)
          snprintf(zeit, TEXTLEN, "%lds", t);
        else   
          snprintf(zeit, TEXTLEN, "%lds, davon %ld frei", t, t-t_freetime);
       }

      tm=localtime(&start);
      snprintf(start_str, TEXTLEN, "%02d:%02d", tm->tm_hour, tm->tm_min);
       
      tm=localtime(&ende);
      snprintf(ende_str, TEXTLEN, "%02d:%02d", tm->tm_hour, tm->tm_min);

      snprintf(buchung.verwendungszweck, TEXTLEN, "%s-%s %s (%s)", 
              start_str, ende_str, zweck, zeit);

      buchung.haben=FALSE;
      buchung.amount.amount=charge.amount;
      buchung.amount.scale=charge.scale;
      strcpy(buchung.bearbeiter, "sysop");
      snprintf(buchung.buchungstyp, TEXTLEN, "MENU.%s", menu);
      buchung.mwstsatz=config.mwstsatz;
      dobuchung(up, &buchung);
     }
   }
 }
#else
void chargemenu(userpt *up, const char *menu, const char *zweck, 
                time_t start, time_t end)
 {
  dword number_of_units;
  struct tm *tm;
  chargeinfopt ci;
  amountt one_time_charge;

  if(end<=start)
    end=start+1;

  strmaxcpy(ci.name, menu, TEXTLEN-1);
  strmaxcpy(ci.group, *up|"gruppe", TEXTLEN-1);

  if(ci.read())
   {
    logf.printf("chargemenu", "Failed to charge %ld seconds to "
                "user %s for menu %s: no charge profile",
                end-start, up->name, menu);
    return;
   }

   {
    time_t new_start=end;
    getchargeinfo(ci, start, new_start);
    one_time_charge=parse_currency_string(ci|"one_time_charge");
   }

  while(start<=end)
   {
    time_t new_start=end, duration, t_freetime, duration2;
    amountt charge, unitprice;
    char zeit[TEXTLEN], start_str[TEXTLEN], ende_str[TEXTLEN];
    buchungt buchung;
    dword unit, displayunit;

    getchargeinfo(ci, start, new_start);

    unit=atoi(ci|"unit");
    if(unit==0) unit=1;

    displayunit=atoi(ci|"displayunit");
    if(displayunit==0)
      displayunit=unit;
    else if(displayunit>unit) 
      unit=displayunit;

    duration=unit_round(new_start-start+1, displayunit);
    t_freetime=chargefreedata(up, "freetime", menu, duration);
    duration2=unit_round(t_freetime, unit);

    if(duration>0 || one_time_charge.amount!=0)
     {
      if(duration2>0)
       {
        number_of_units=duration2/unit; // Division sollte ohne Rest gehen
        unitprice=parse_currency_string(ci|"unitprice");
        charge.amount=unitprice.amount*number_of_units;
        charge.scale=unitprice.scale;
       }
      else
       {
        charge.amount=0;
        charge.scale=0;
       }

      if(one_time_charge.amount!=0)
       {
        charge+=one_time_charge;
        one_time_charge.amount=0;
        one_time_charge.scale=0;
       }

      if(charge.amount>0 || duration!=duration2)
       {
        if(displayunit==60 && unit==60)
         {
          if(t_freetime==duration)
            snprintf(zeit, TEXTLEN, "%ldm", duration/60);
          else   
            snprintf(zeit, TEXTLEN, "%ldm, davon %ld frei", 
              duration/60, (duration-duration2)/60);
         }
        else
         {
          if(t_freetime==duration)
            snprintf(zeit, TEXTLEN, "%lds", duration);
          else   
            snprintf(zeit, TEXTLEN, "%lds, davon %ld frei", 
              duration, duration-t_freetime);
         }

        tm=localtime(&start);
        snprintf(start_str, TEXTLEN, "%02d:%02d", tm->tm_hour, tm->tm_min);
       
        tm=localtime(&new_start);
        snprintf(ende_str, TEXTLEN, "%02d:%02d", tm->tm_hour, tm->tm_min);

         {
          const char *description=ci|"description";
          if(description[0]!=0) zweck=description;
         }

        snprintf(buchung.verwendungszweck, TEXTLEN, "%s-%s %s (%s)", 
                 start_str, ende_str, zweck, zeit);

        buchung.haben=FALSE;
        buchung.amount.amount=charge.amount;
        buchung.amount.scale=charge.scale;
        strcpy(buchung.bearbeiter, "sysop");
        snprintf(buchung.buchungstyp, TEXTLEN, "MENU.%s", menu);
        buchung.mwstsatz=config.mwstsatz;
        dobuchung(up, &buchung);
       }
     }

    start=new_start+unit;
   }
 }
#endif

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

void getfreealias(const char *type, const char *name, char *alias)
 {
  FILE *infile;
  char filename[DIRLEN];
  char id[1000], text[1000];

  strcpy(alias, name);

  snprintf(filename, DIRLEN, BOXSYSTEMDIR "freealias", config.boxdir);
  infile=fopen(filename, "rt");
  if(infile==NULL)
   {
    snprintf(filename, DIRLEN, DBOXSYSTEMDIR "freealias", config.dboxdir);
    infile=fopen(filename, "rt");
   }
  
  if(infile!=NULL)
   {
   schleife:
    switch(readconfigline(infile, id, text, 900))
     {
     case 0:
      if(stricmp(id, name)==0)
       {
        strcpy(alias, text);
        break;
       }
      goto schleife;

     case 1:
      break;

     case 2:
      break;
     }

    fclose(infile);
   }
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

long int getfreedata(userpt *up, const char *type, const char *name)
 {
  FILE *infile;
  char filename[DIRLEN];
  char id[1000], text[1000], newname[TEXTLEN*2], aliasname[TEXTLEN];

  getfreealias(type, name, aliasname);

  snprintf(newname, TEXTLEN, "%s.%s", type, aliasname);

  up->setfilename();
  snprintf(filename, DIRLEN, "%sfree", up->getpath());
  infile=fopen(filename, "rt");

  if(infile!=NULL)
   {
   schleife:
    switch(readconfigline(infile, id, text, 900))
     {
     case 0:
      if(stricmp(id, newname)==0)
       {
        fclose(infile);
        return atol(text);
       }
      goto schleife;

     case 1:
      break;

     case 2:
      break;
     }

    fclose(infile);
   }

  return 0;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

// Rueckgabewert: Restzeit/Restvolumen (zu zahlen)
long int chargefreedata(userpt *up, const char *type, const char *name,
                        long int value)
 {
  FILE *infile, *outfile;
  char filename[DIRLEN], filename_tmp[DIRLEN];
  long int free_value, tmp_value;
  char id[1000], text[1000], newname[TEXTLEN*2], aliasname[TEXTLEN];

  if(value<=0) return 0;

  getfreealias(type, name, aliasname);

  snprintf(newname, DIRLEN, "%s.%s", type, aliasname);
  newname[TEXTLEN-1]=0;

  up->setfilename();
  snprintf(filename, DIRLEN, "%sfree", up->getpath());
  infile=fopen(filename, "rt");
  if(infile!=NULL)
   {
    free_value=0;

   schleife1:
    switch(readconfigline(infile, id, text, 900))
     {
     case 0:
      if(stricmp(id, newname)==0)
        free_value=atol(text);
      else 
        goto schleife1;
      break;

     case 1:
      break;

     case 2:
      break;
     }

    if(free_value>0) 
     {
      tmp_value=value;
      if(tmp_value>free_value) tmp_value=free_value;

      free_value-=tmp_value;
      value-=tmp_value;

      rewind(infile);

      snprintf(filename_tmp, DIRLEN, "%s%s.lck", up->getpath(), type);
      outfile=fopen(filename_tmp, "wt");
      if(outfile!=NULL)
       {
       schleife2:
        switch(readconfigline(infile, id, text, 900))
         {
         case 0:
          if(stricmp(id, newname)==0)
            fprintf(outfile, "%s=%ld\n", id, free_value);
          else 
            fprintf(outfile, "%s=%s\n", id, text);

          goto schleife2;

         case 1:
          break;

         case 2:
          break;
         }
       }  

      fclose(outfile);
     }

    fclose(infile);

    rename(filename_tmp, filename);
   }
  
  return value;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

bool dobuchung(userpt *up, buchungt *buchung)
 {
  FILE *outfile;
  char filename[DIRLEN];
  amountt balance;

  time(&buchung->datum);
  buchung->number=atol(*up|"buchungsnummer");

  if(up->lock())
   {
    logf.printf("dobuchung", "Fatal: lock() failed!");
    return TRUE;
   }

  if(buchung->verwendungszweck[0]==0)
    strmaxcpy(buchung->verwendungszweck, "none", TEXTLEN-1);

  snprintf(filename, TEXTLEN, "%skonto", up->getpath());
  outfile=fopen(filename, "at");
  if(outfile==NULL)
   {
    logf.printf("dobuchung", "fatal: failed to open %s", filename);
    up->unlock();
    return TRUE;
   }

  buchung->write(outfile);

  fclose(outfile);

  snprintf(filename, DIRLEN, "%sauszug", up->getpath());
  outfile=fopen(filename, "at");
  if(outfile==NULL)
   {
    logf.printf("dobuchung", "fatal: failed to open %s", filename);
    up->unlock();
    return TRUE;
   }

  buchung->write(outfile);

  fclose(outfile);

  up->set("buchungsnummer", buchung->number+1);

  balance=getbalance(up);

  if(buchung->haben)
    balance+=buchung->amount;
  else
    balance-=buchung->amount;

  up->set("konto", balance.amount);
  up->set("konto.scale", (int)balance.scale);

  if(up->write())
   {
    logf.printf("dobuchung", "fatal: write() failed");
    up->unlock();
    return TRUE;
   }

  up->unlock();

  return FALSE;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

void buchungt::clear()
 {
  number=0;
  amount.scale=2;
  amount.amount=0;
  haben=FALSE;
  datum=0;
  bearbeiter[0]=0;
  alias[0]=0;
  verwendungszweck[0]=0;
  mwstsatz=0;
  buchungstyp[0]=0;
 }

char buchungt::parse(char **b, char *s)
 {
  char *line=*b, ch, *tptr;

  ch=line[0];
  line++;
  if(line[0]!=':') return 0;
  line++;
  tptr=line;

  while(line[0]!=0 && line[0]!=';')
    line++;

  if(line[0]==0)
    strcpy(s, tptr);
  else
   {
    *line=0;
    strcpy(s, tptr);
    *line=';';
    line++;    
   }
  
  *b=line;
  return ch;
 }

#define MAXLEN 10000

bool buchungt::parse(char *line)
 {
  char *tptr, *tptr2;

  clear();

  tptr=strchr(line, '\r'); if(tptr!=NULL) *tptr=0;
  tptr=strchr(line, '\n'); if(tptr!=NULL) *tptr=0;

  if(line[0]==0) return TRUE;

  if(line[1]==':')
   {
    char *lineptr, *buffer;

    buffer=(char *)malloc(MAXLEN);
    if(buffer==NULL) return TRUE;

    lineptr=line;

    while(*lineptr!=0)
     {
      char ch=parse(&lineptr, buffer);

      switch(upcase(ch))
       {
       case 'Z':
        strmaxcpy(verwendungszweck, buffer, (TEXTLEN*2)-1);
        break;

       case 'N':
        number=atol(buffer);
        break;

       case 'H':
        haben=atoi(buffer);
        break;

       case 'B':
        amount.amount=atol(buffer);
        break;

       case 'A':
        amount.scale=atoi(buffer);
        break;

       case 'L':
        strmaxcpy(alias, buffer, TEXTLEN-1);
        break;

       case 'D':
        datum=atol(buffer);
        break;

       case 'S':
        strmaxcpy(bearbeiter, buffer, TEXTLEN-1);
        break;

       case 'M':
        mwstsatz=atol(buffer);
        break;

       case 'T':
        strmaxcpy(buchungstyp, buffer, TEXTLEN-1);
        break;
        
       default:
        return TRUE;
       }
     }

    free(buffer);
   }
  else
   {
    tptr=line;
    number=atol(tptr);
    tptr=strchr(tptr, ' ');
    if(tptr==NULL) return TRUE;
    tptr++;

    haben=atoi(tptr);
    tptr=strchr(tptr, ' ');
    if(tptr==NULL) return TRUE;
    tptr++;

    amount.amount=atol(tptr);
    tptr=strchr(tptr, ' ');
    if(tptr==NULL) return TRUE;
    tptr++;

    datum=atol(tptr);
    tptr=strchr(tptr, ' ');
    if(tptr==NULL) return TRUE;
    tptr++;

    tptr2=tptr;
    tptr=strchr(tptr, ' ');
    if(tptr==NULL) return TRUE;
    *tptr=0;
    tptr++;
    strmaxcpy(bearbeiter, tptr2, TEXTLEN-1);

    strmaxcpy(verwendungszweck, tptr, (TEXTLEN*2)-1);

    mwstsatz=0;
    buchungstyp[0]=0;
   }

  return FALSE;
 }

bool buchungt::read(FILE *infile)
 {
  char *line, *tptr;
  bool erg;

  line=(char *)malloc(MAXLEN);
  if(line==NULL) return TRUE;

  if(fgets(line, MAXLEN-1, infile)==NULL) return TRUE;

  tptr=strchr(line, '\r'); if(tptr!=NULL) *tptr=0;
  tptr=strchr(line, '\n'); if(tptr!=NULL) *tptr=0;

  if(line[0]==0) return TRUE;

  erg=parse(line);

  free(line);

  return erg;
 }

bool buchungt::write(FILE *outfile)
 {
  /*
  fprintf(outfile, "%lu %u %lu %lu %s %s\n",
    buchungsnummer, haben, betrag, datum, bearbeiter, verwendungszweck);
  */

  fprintf(outfile, "N:%lu;H:%d;B:%ld;A:%d;D:%lu;S:%s;M:%lu;T:%s;Z:%s",
    number, haben, amount.amount, amount.scale, datum, bearbeiter,
    mwstsatz, buchungstyp, verwendungszweck);

  if(alias[0]!=0)
    fprintf(outfile, "L:%s", alias);

  fprintf(outfile, "\n");

  return FALSE;
 }

void buchungt::format(FILE *outfile, 
                      char format, bool crlf, charsett charset)
 {
  char *tptr;
  char tempstr[TEXTLEN], str_number[TEXTLEN],
       str_date[TEXTLEN], str_amount[TEXTLEN], str_mwstsatz[TEXTLEN];

  if(number==(dword)-1)
    strmaxcpy(str_number, "", TEXTLEN-1);
  else
    snprintf(str_number, TEXTLEN, "%04lu", number%10000);

  if(datum==0)
    strcpy(str_date, "");
  else
   {
    struct tm *t;
    t=localtime(&datum);
    snprintf(str_date, TEXTLEN, "%02u.%02u.%04u", 
      t->tm_mday, t->tm_mon+1, t->tm_year+1900);
   }

  currency_string(tempstr, amount, FALSE);
  snprintf(str_amount, TEXTLEN, "%c%s", haben?'+':'-', tempstr);

  if(mwstsatz!=(dword)-1)
   {
    snprintf(str_mwstsatz, TEXTLEN, "%2lu,%lu", mwstsatz/10, mwstsatz%10);
    switch(format)
     {
     case 'a': strcat(str_mwstsatz, "%"); break;
     case 'l': strcat(str_mwstsatz, "\\%"); break;
     }
   }
  else
    strcpy(str_mwstsatz, "");

  switch(format)
   {
   case 'a':
    snprintf(tempstr, TEXTLEN, "%4s  %10s %8s  %5s  ",
            str_number,
            str_date,
            str_amount, 
            str_mwstsatz);
    break;

   case 'l':
    snprintf(tempstr, TEXTLEN, "%4s & %10s &%8s & %6s & ",
            str_number,
            str_date,
            str_amount, 
            str_mwstsatz);
    break;

   case 'e':
    snprintf(tempstr, TEXTLEN, "%s\t%s\t%s\t%s\t",
            str_number,
            str_date,
            str_amount, 
            str_mwstsatz);
    break;
   }

  if(charset==ISO)
    ibm2iso(tempstr, strlen(tempstr));
  else if(charset==ASCII)
   {
   }

  fprintf(outfile, "%s", tempstr);

  switch(format)
   {
   case 'a':
     {
      byte line=0;
      char *tptr2;

      tptr=verwendungszweck;

      while(*tptr!=0)
       {
        line++;

        if(strlen(tptr)>39)
         {
          strmaxcpy(tempstr, tptr, 41);
          tptr2=strrchr(tempstr, ' ');
          if(tptr2==NULL || (tptr2-tempstr)<10)
           {
            tptr+=min(strlen(tptr), (unsigned)41);
           }
          else
           {
            *tptr2=0;
            tptr=tptr+(tptr2-tempstr)+1;
           }
         }
        else
         {
          strcpy(tempstr, tptr);
          tptr="";
         }

        if(line!=1)
          fprintf(outfile, "                                  ");

        if(crlf)
          fprintf(outfile, "%s\r\n", tempstr);
        else
          fprintf(outfile, "%s\n", tempstr);
       }
     }
    break;

   case 'e':
    strmaxcpy(tempstr, verwendungszweck, TEXTLEN-1);

    if(charset==ISO)
      ibm2iso(tempstr, strlen(tempstr));
    else if(charset==ASCII)
     {
     }
    
    if(crlf)
      fprintf(outfile, "%s\r\n", tempstr);
    else
      fprintf(outfile, "%s\n", tempstr);

    break;

   case 'l':
    strmaxcpy(tempstr, verwendungszweck, TEXTLEN-1);

    if(charset==ISO)
      ibm2iso(tempstr, strlen(tempstr));
    else if(charset==ASCII)
     {
     }
    
    if(crlf)
      fprintf(outfile, "%s\\\\\r\n", tempstr);
    else
      fprintf(outfile, "%s\\\\\n", tempstr);

    break;
   }
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

long int gethostchargeinfo(grouppt *group, const char *host, dword size)
 {
  char *info;
  char filename[DIRLEN];
  long int charge=-1;
  char tempstr[DIRLEN], *tptr, *iptr;

  if(group->read()) return -1;

  snprintf(filename, DIRLEN, "%sch_host", group->getpath());

  info=readtextconfigfile(filename);
  if(info==NULL) return -1;
  iptr=info;

  while(*iptr!=0)
   {
    strmaxcpy(tempstr, iptr, TEXTLEN-1);

    tptr=tempstr;
    while(*tptr!=0 && *tptr!=' ' && *tptr!='\t') tptr++;

    if(*tptr==0)
      break;

    *(tptr++)=0;

    while(*tptr==' ' || *tptr=='\t') tptr++;

    if(*tptr==0)
      break;

    if(checkjoker(tempstr, host, TRUE))
     {
      long int i;
      i=atol(tptr);

      if(i<0)
        charge=-1;
      else if(i==0)
        charge=0;
      else
       {
        charge=size/i;
        if((size%i)!=0)
          charge++;

        if(charge==0) charge=1;
       }
      break;
     }

    iptr+=strlen(iptr)+1;
   }

  free(info);

  return charge;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

const char *formatkonto_start1=
  " Nr.    Datum      Betrag   MwSt              Verwendungszweck";

const char *formatkonto_start2=
  "";

const char *formatkonto_ende=
  "";

void formatkonto(FILE *infile, FILE *outfile, 
                 char format, bool crlf, charsett charset)
 {
  buchungt buchung;

  while(!buchung.read(infile))
    buchung.format(outfile, format, crlf, charset);
 }

