#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <fcntl.h>
#include <time.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>

#include <fstream.h>

#include <general.h>
#include <dbox.h>
#include <dboxpath.h>

#include <config.h>
#include <dboxdb.h>

// **********************************************************************
// *                                                                    *
// *                           stringarrayt                             *
// *                                                                    *
// **********************************************************************

int stringarrayt::index(const char *string)
 {
  int i;

  for(i=0; strings[i]!=NULL; i++)
    if(stricmp(strings[i], string)==0)
      return i;

  return -1;
 }

// **********************************************************************
// *                                                                    *
// *                           userflagdbt                              *
// *                                                                    *
// **********************************************************************

bool userflagdbt::read()
 {
  return read(&config);
 }

bool userflagdbt::read(configt *config)
 {
  char tempstr[DIRLEN], *tptr;
  ifstream infile;

  snprintf(tempstr, DIRLEN, BOXSYSTEMDIR "flags", config->boxdir);
  
  infile.open(tempstr);
  if(!infile)
   {
    snprintf(tempstr, DIRLEN, DBOXSYSTEMDIR "flags", config->dboxdir);
    infile.open(tempstr);
   }

  dword i=0;

  if(infile)
   {
    while(infile.get(tempstr, 900))
     {
      char ch;

      infile.get(ch); // get \n

      tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;

      if(tempstr[0]!=0)
        strings[i++]=strdup(tempstr);
     }
   }

  strings[i]=(char *)NULL;
  number_of_elements=i;

  return FALSE;
 }

userflagdbt::~userflagdbt()
 {
  dword i;

  for(i=0; strings[i]!=NULL; i++)
    free((void *)strings[i]);
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

void clear(userflagst *userflags)
 {
  int i;

  for(i=0; i<MAXUSERFLAGS; i++)
    (*userflags)[i]=FALSE;
 }

// **********************************************************************
// *                                                                    *
// *                           groupdbt                                 *
// *                                                                    *
// **********************************************************************

groupdbt groupdb;

static int cmpstring(const void *a, const void *b)
 {
  return stricmp(*(const char **)a, *(const char **)b);
 }

bool groupdbt::read()
 {
  return read(&config);
 }

bool groupdbt::read(configt *config)
 {
  DIR *dir;
  struct dirent *ent;
  char tempstr[DIRLEN];

  snprintf(tempstr, DIRLEN, BOXGRUPPENDIR ".", config->boxdir, ".");
  if((dir=opendir(tempstr))!=NULL)
   {
    dword i=0;
    while((ent=readdir(dir))!=NULL && i<MAXGRUPPEN)
     {
      if(ent->d_name[0]!='.' && ent->d_name[0]!='_')
        strings[i++]=strdup(ent->d_name);
     }
    closedir(dir);

    qsort(strings, i, sizeof(char *), cmpstring);

    number_of_elements=i;
   }

  return FALSE;
 }

groupdbt::~groupdbt()
 {
  dword i;

  for(i=0; strings[i]!=NULL; i++)
    free((void *)strings[i]);
 }

// **********************************************************************
// *                                                                    *
// *                           netpackerdbt                             *
// *                                                                    *
// **********************************************************************

netpackerdbt netpackerdb;

const char *netpackerlist[]=
 {
  "ZIP",
  "ZIP2",
  "LHA",
  "LHA2",
  "ZOO",
  "ARJ",
  "RAR",
  "Ungepackt",
  "Intern",
  (char *)NULL
 };

bool netpackerdbt::read()
 {
  return read(&config);
 }

bool netpackerdbt::read(configt *config)
 {
  int i;

  i=0;

  do
   {
    strings[i]=netpackerlist[i];
    i++;
   }
  while(netpackerlist[i]!=NULL);

  number_of_elements=i;

  return FALSE;
 }

// **********************************************************************
// *                                                                    *
// *                           nettypedbt                               *
// *                                                                    *
// **********************************************************************

nettypedbt nettypedb;

const char *nettypelist[]=
 {
  "Janus",
  "Janus2",
  "Z3.8",
  "ZConnect",
  "UUCP",
  "Alphabox",
  "GSBOX",
  "Fido",
  "POP3/IMAP4",
  (char *)NULL
 };

bool nettypedbt::read()
 {
  return read(&config);
 }

bool nettypedbt::read(configt *config)
 {
  int i;

  i=0;

  do
   {
    strings[i]=nettypelist[i];
    i++;
   }
  while(nettypelist[i]!=NULL);

  number_of_elements=i;

  return FALSE;
 }

// **********************************************************************
// *                                                                    *
// *                           netprotocoldbt                           *
// *                                                                    *
// **********************************************************************

netprotocoldbt netprotocoldb;

char *netprotocollist[]=
 {
  "Z-Modem",
  "Y-Modem",
  "X-Modem",
  "Hydra",
  (char *)NULL
 };

bool netprotocoldbt::read()
 {
  return read(&config);
 }

bool netprotocoldbt::read(configt *config)
 {
  int i;

  i=0;

  do
   {
    strings[i]=netprotocollist[i];
    i++;
   }
  while(netprotocollist[i]!=NULL);

  number_of_elements=i;

  return FALSE;
 }

// **********************************************************************
// *                                                                    *
// *                           grouptypedbt                             *
// *                                                                    *
// **********************************************************************

char *grouptypelist[]=
 {
  "",
  "ZConnect",
  "Lokales Newssystem",
  "NNTP",
  "Filesystem",
  (char *)NULL
 };

bool grouptypedbt::read()
 {
  return read(&config);
 }

bool grouptypedbt::read(configt *config)
 {
  int i;

  i=0;

  do
   {
    strings[i]=grouptypelist[i];
    i++;
   }
  while(grouptypelist[i]!=NULL);

  number_of_elements=i;

  return FALSE;
 }

