#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <time.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>

#include <string>

#ifdef OS_UNIX
#include <sched.h>
#endif

#include <dboxpath.h>
#include <lock.h>

//
// Locking Services
// 
// The types/name combinations used should be unique
//
// Todo: This locking does not work over NFS
//       This locking probably does not work with Win32
//       Automatic stale lock detection is missing
//

static void mklockname(string &filename,
  const char *type, const char *name, const char *boxname)
 {
  char tmpname[TEXTLEN], *tptr;
  char boxdir[DIRLEN];

  mkboxdir(boxdir, boxname);

  strmaxcpy(tmpname, name, TEXTLEN-1);
  
  for(tptr=tmpname; tptr[0]!=0; tptr++)
    if(tptr[0]==DIRCHAR[0])
      tptr[0]='_';

  getboxlockdir(filename, boxdir);
  filename+=type;
  filename+='.';
  filename+=tmpname;
 }

bool lock(const char *type, const char *name, const char *boxname)
 {
  string filename;
  char tempstr[TEXTLEN];
  int handle, attempt=0;

  mklockname(filename, type, name, boxname);

  while(TRUE)
   {
    attempt++;
    handle=open(filename.c_str(), O_WRONLY|O_CREAT|O_EXCL|O_BINARY, 0600);
    if(handle<0) 
     {
      #ifdef OS_WIN32
      sleep(1);
      #else
      if(attempt>=2) 
        sleep(1);
      else
        sched_yield();
      #endif

      if(attempt>20) return TRUE;
     }
    else
      break;
   }

  gethostname(tempstr, TEXTLEN-1);
  sprintf(tempstr+strlen(tempstr), ".%d\n", getpid());

  write(handle, tempstr, strlen(tempstr));

  close(handle);

  return FALSE;
 }

bool unlock(const char *type, const char *name, const char *boxname)
 {
  string filename;
  mklockname(filename, type, name, boxname);
  unlink(filename.c_str());

  return FALSE;
 }

