#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <dirent.h>
#include <time.h>

#include <dbox.h>
#include <userdb.h>
#include <dboxpath.h>
#include <logf.h>
#include <parsetime.h>
#include <logins.h>

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

void clientpt::setfilename()
 {
  snprintf(path, DIRLEN, DBOXCLIENTDIR, config.dboxdir, name);
  strmaxcpy(locktype, "client", TEXTLEN-1);
  snprintf(filename, DIRLEN, "%sprofile", path);
 }
 
bool clientpt::add()
 {
  return TRUE;
 }

bool clientpt::del()
 {
  return TRUE;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

bool check_login_baset::radius_check(const char *item, const char *mask)
 {
  return FALSE;
 }

void check_login_baset::init()
 {
  errormsg[0]=0;
  number_of_radius_checks=0;
  strmaxcpy(protocol_check, "*", TEXTLEN-1);
 }

bool check_login_baset::do_line(const char *id, const char *text)
 {
  if(stricmp(id, "errormsg")==0)
   {
    strmaxcpy(errormsg, text, TEXTLEN-1);
   }
  else if(stricmp(id, "protocol.check")==0)
   {
    strmaxcpy(protocol_check, text, TEXTLEN-1);
   }
  #define rad_check "radius.check."
  else if(strnicmp(id, rad_check, strlen(rad_check))==0)
   {
    if(number_of_radius_checks<MAX_CHECKS)
     {
      strmaxcpy(radius_checks[number_of_radius_checks].item, id+strlen(rad_check), TEXTLEN-1);
      strmaxcpy(radius_checks[number_of_radius_checks].mask, text, TEXTLEN-1);
      number_of_radius_checks++;
     }
   }
  else
    return TRUE;

  return FALSE;
 }

bool check_login_baset::do_protocol_check()
 {
  if(stricmp(protocol_check, "framed")==0)
   {
    return stricmp(protocol, "slip")!=0 &&
           stricmp(protocol, "cslip")!=0 &&
           stricmp(protocol, "ppp")!=0;
   }
  else if(stricmp(protocol_check, "netcall")==0)
   {
    return stricmp(protocol, "janus")!=0 &&
           stricmp(protocol, "janus2")!=0 &&
           stricmp(protocol, "uucp")!=0 &&
           stricmp(protocol, "alphabox")!=0 &&
           stricmp(protocol, "gsbox")!=0;
   }
  else
    return !checkjoker(protocol_check, protocol, TRUE);
 }

bool check_login_baset::do_radius_check()
 {
  unsigned i;

  for(i=0; i<number_of_radius_checks; i++) 
    if(radius_checks[i].item[0]!=0 &&
       radius_check(radius_checks[i].item, radius_checks[i].mask))
      return TRUE;

  return FALSE;
 }

bool check_login_baset::do_checks()
 {
  return do_protocol_check() || do_radius_check();
 }

bool check_login_baset::do_check(FILE *infile)
 {
  // fprintf(stderr, "DK: checkpoint start logins.C\r\n");

  if(infile==NULL)
   {
    init();
    return FALSE;
   }

  while(TRUE)
   {
    char id[500]="", text[500];

    switch(readconfigline(infile, id, text, 499))
     {
     case 0:
      if(stricmp(id, "permit")==0 || 
         stricmp(id, "deny")==0)
       {
        if(!do_checks())
          return stricmp(id, "deny")==0;

        init();
       }
      else if(do_line(id, text))
        logf.printf("checklogin", "unknown id \"%s\"", id);
      break;

     case 1:
      init();
      return TRUE;

     default:
      logf.printf("checklogin", "error in configuration file");
      init();
      return TRUE;
     }
   }
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

bool check_login1t::check_login1(FILE *infile)
 {
  return do_check(infile);
 }

void check_login1t::init()
 {
  check_login_baset::init();

  boxname[0]=0;
  protocol_set[0]=0;
  forwardto_host[0]=0;
  radius_auth_server[0]=0;
  radius_auth_secret[0]=0;
  radius_acct_server[0]=0;
  radius_acct_secret[0]=0;
  radius_autoadd=FALSE;

  strmaxcpy(username, login, TEXTLEN-1);
  strmaxcpy(protocol_set, protocol, TEXTLEN-1);
  strmaxcpy(login_check, "*", TEXTLEN-1);
 }

bool check_login1t::do_checks()
 {
  return check_login_baset::do_checks() || do_login_check();
 }

bool check_login1t::do_line(const char *id, const char *text)
 {
  if(stricmp(id, "protocol.set")==0)
   {
    strmaxcpy(protocol_set, text, TEXTLEN-1);
   }
  else if(stricmp(id, "login.check")==0)
   {
    strmaxcpy(login_check, text, TEXTLEN-1);
   }
  else if(stricmp(id, "username.strip.prefix")==0)
   {
    char tempstr[1000];
    unsigned i=strlen(text);
    unsigned j=strlen(username);
    if(i<=j)
     {
      if(strnicmp(username, text, i)==0)
       {
        strmaxcpy(tempstr, username+i, 999);
        strmaxcpy(username, tempstr, TEXTLEN-1);
       }
     }
   }
  else if(stricmp(id, "username.strip.suffix")==0)
   {
    unsigned i=strlen(text);
    unsigned j=strlen(username);
    if(i<=j)
     {
      if(stricmp(username+j-i, text)==0)
        username[j-i]=0;
     }
   }
  else if(stricmp(id, "username.add.prefix")==0)
   {
    char tempstr[TEXTLEN];
    snprintf(tempstr, TEXTLEN, "%s%s", username, text);
    strmaxcpy(username, tempstr, TEXTLEN-1);
   }
  else if(stricmp(id, "username.add.suffix")==0)
   {
    char tempstr[TEXTLEN];
    snprintf(tempstr, TEXTLEN, "%s%s", username, text);
    strmaxcpy(username, tempstr, TEXTLEN-1);
   }
  else if(stricmp(id, "boxname.set")==0)
    strmaxcpy(boxname, text, TEXTLEN-1);
  else if(stricmp(id, "radius.auth.server")==0)
    strmaxcpy(radius_auth_server, text, TEXTLEN-1);
  else if(stricmp(id, "radius.auth.secret")==0)
    strmaxcpy(radius_auth_secret, text, TEXTLEN-1);
  else if(stricmp(id, "radius.acct.server")==0)
    strmaxcpy(radius_acct_server, text, TEXTLEN-1);
  else if(stricmp(id, "radius.acct.secret")==0)
    strmaxcpy(radius_acct_secret, text, TEXTLEN-1);
  else if(stricmp(id, "forwardto.host")==0)
    strmaxcpy(forwardto_host, text, TEXTLEN-1);
  else
    return check_login_baset::do_line(id, text);

  return FALSE;
 }

bool check_login1t::do_login_check()
 {
  return !checkjoker(login_check, login, TRUE);
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

bool check_login2t::do_username_check()
 {
  return !checkjoker(username_check, up->name, TRUE);
 }

bool check_login2t::do_user_check()
 {
  unsigned i;

  for(i=0; i<number_of_user_checks; i++) 
    if(user_checks[i].item[0]!=0 &&
       user_check(user_checks[i].item, user_checks[i].mask))
      return TRUE;

  return FALSE;
 }

bool check_login2t::do_time_check()
 {
  //if(!time_check(second_of_day, time_check_min, time_check_max))

  return FALSE;
 }

bool check_login2t::do_checks()
 {
  return check_login_baset::do_checks() || do_username_check() ||
         do_user_check() || do_time_check();
 }

void check_login2t::init()
 {
  check_login_baset::init();

  number_of_user_checks=0;
  strmaxcpy(username_check, "*", TEXTLEN-1);
  time_check_min=time_check_max=time_logout=-1;
  session_timeout=0;
 }

bool check_login2t::check_login2(FILE *infile)
 {
  time(&now);
  tm=localtime(&now);
  second_of_day=tm->tm_hour*3600+tm->tm_min*60+tm->tm_sec;

  return do_check(infile);
 }

bool check_login2t::do_line(const char *id, const char *text)
 {
  if(stricmp(id, "charge.menu")==0)
   {
    strmaxcpy(charge_menu, text, TEXTLEN-1);
   }
  #define user_check_str "user.check."
  else if(strnicmp(id, user_check_str, strlen(user_check_str))==0)
   {
    if(number_of_user_checks<MAX_CHECKS)
     {
      strmaxcpy(user_checks[number_of_user_checks].item, id+strlen(user_check_str), TEXTLEN-1);
      strmaxcpy(user_checks[number_of_user_checks].mask, text, TEXTLEN-1);
      number_of_user_checks++;
     }
   }
  else if(stricmp(id, "time.check.min")==0)
    time_check_min=parsetime(text);
  else if(stricmp(id, "time.check.max")==0)
    time_check_max=parsetime(text);
  else if(stricmp(id, "time.logout")==0)
   {
    time_logout=parsetime(text);
    if(time_logout==-1)
      session_timeout=0;
    else if(time_logout==second_of_day)
      session_timeout=1;
    else if(time_logout<second_of_day)
      session_timeout=second_of_day-time_logout;
    else if(time_logout>second_of_day)
      session_timeout=time_logout-second_of_day;
   }
  else
    return check_login_baset::do_line(id, text);

  return FALSE;
 }

bool check_login2t::user_check(const char *item,
                               const char *mask)
 {
  if(checkjoker(mask, *up|item, TRUE))
    return FALSE;

  return TRUE;
 }

