#include <stddef.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <ctype.h>
#include <stdarg.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/time.h>

#include <minmax.h>

#include <dbox.h>
#include <dboxpath.h>

#include <interlib.h>
#include <config.h>
#include <oldterm.h>
#include <oldtools.h>
#include <oldui.h>
#include <logf.h>
#include <keys.h>
#include <profile.h>
#include <userdb.h>
#include <blzdb.h>

void stringlistt::parsestring(const char *s)
 {
  dword i;
  char *tptr, *stemp, *sptr;

  for(i=0; i<STRINGLIST_MAXSTRINGS; i++)
    strings[i]=(char *)NULL;

  stemp=strdup(s);
  if(stemp==NULL) 
    logf.internalerror("stringlistt::parsestring", "strdup failed!");
 
  sptr=stemp;
  i=0;

  while(sptr[0]!=0)
   {
    tptr=sptr;
    while(*tptr!=0 && *tptr!=';' && *tptr!=',' && *tptr!=' ')
      tptr++;

    if(*tptr!=0)
     {
      *tptr=0;
      tptr++;
      while(*tptr==';' || *tptr==',' || *tptr==' ')
        tptr++;
     }

    while(*sptr==' ') sptr++;

    if(i<STRINGLIST_MAXSTRINGS)
     {
      strings[i]=strdup(sptr);

      if(strings[i]==NULL)
        logf.internalerror("stringlistt::parsestring", "strdup failed!");
      i++;
     }

    sptr=tptr;
   }

  strings[i]=(char *)NULL;

  free(stemp);
 }

stringlistt::~stringlistt()
 {
  dword i;

  for(i=0; strings[i]!=(char *)NULL; i++)
    free(strings[i]);

  strings[0]=(char *)NULL;
 }

//*******************************************************************//
//*******************************************************************//
//*******************************************************************//

bool readline(screent *screen, char *line, bool pw, bool special,
              bool preserve_case, bool hide)
 {
  char *tptr;
  time_t starttime;
  word key;
  bool keypressed=FALSE;

  line[0]=0;
  time(&starttime);
  asyncio.flush_output();

  while(TRUE)
   {
    key=screen->doreadkey();

    if(key==0xffff)
      return TRUE;
    else if(key==Key_C_D)
      return TRUE;
    else if(key>' ' && key<126)
     {
      if(strlen(line)<38)
       {
        tptr=line+strlen(line);
        *(tptr++)=preserve_case?key:dncase(key);
        *tptr=0;
        key=preserve_case?key:upcase(key);
        if(!hide) screen->putch(pw?'*':key);
        keypressed=TRUE;
       }
     }
    else if(key=='\r' || key=='\n')
     {
      screen->printf("\r\n");
      asyncio.flush_output();

      if(special && strstr(line, "emsi_inq")!=NULL)
        strmaxcpy(line, "FIDO", TEXTLEN-1);
      return FALSE;
     }
    else if(key==8 || key==127)
     {
      if(strlen(line)!=0)
       {
        if(!hide)
         {
          screen->putch(8);
          screen->putch(' ');
          screen->putch(8);
         }
        line[strlen(line)-1]=0;
       }
     }
    else if(special && (byte)key==0xae)
     {
      strmaxcpy(line, "FIDO", TEXTLEN-1);
      return FALSE;
     }
    else if(special && (byte)key==0xf1)
     {
      strmaxcpy(line, "FIDO", TEXTLEN-1);
      return FALSE;
     }
    else if(special && (byte)key==0x7e)
     {
      strmaxcpy(line, "@PPP", TEXTLEN-1);
      return FALSE;
     }

    if((time((time_t *)NULL)>(starttime+10)) && !keypressed) return FALSE;

    if(time((time_t *)NULL)>(starttime+60))
     {
      screen->printf("\r\n--TIMEOUT--\r\n");
      asyncio.flush_output();

      return TRUE;
     }
   }
 }

void printtextfile(screent *screen, FILE *file)
 {
  char tempstr[100], *tptr;
  word key;

  while(fgets(tempstr, 90, file)!=NULL)
   {
    tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;
    tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;
    tempstr[78]=0;
    screen->printf("%s\r\n", tempstr);
    key=screen->readkey();
    if(key==Key_C_X || key==Key_C_C)
     {
      screen->printf("--BREAK--\r\n");
      fclose(file);
      return;
     }

    if(key==0xffff)
     {
      fclose(file);
      return;
     }
   }

  fclose(file);
 }

void printattrstr(screent *screen, const char *text, byte sizex, byte layer)
 {
  bool hi=FALSE, escaped=FALSE;

  while(sizex!=0 && *text!=0)
   {
    if(*text=='\\' && !escaped)
      escaped=TRUE;
    else if(*text=='~' && !escaped)
     {
      hi=!hi;
      screen->setattr(layer, hi);
     }
    else
     {
      escaped=FALSE;

      if(*text>=' ' || *text<=0)
       {
        screen->printf("%c", *text);
        sizex--;
       }
     }

    text++;
   }

  if(hi)
    screen->setattr(layer, FALSE);
 }

void drawtitle(screent *screen, const char *title)
 {
  int y, i, j;
  char tempstr[TEXTLEN*2];

  screen->setattr(10, FALSE);
  y=strlen(title);
  if(y>((TEXTLEN-1)*2)) y=(TEXTLEN-1)*2;
  if(y>screen->sizex-2) y=screen->sizex-2;
  strmaxcpy(tempstr, title, y);

  j=(screen->sizex-y)/2;
  screen->setcursor(1, 1);
  for(i=0; i<j; i++) screen->printf(" ");
  screen->printf("%s", tempstr);
  j=screen->sizex-j-y;
  for(i=0; i<j; i++) screen->printf(" ");
  screen->setattr(1, FALSE);
 }

class viewtextfilet
 {
 public:
  const char *title;
  FILE *file;
  screent *screen;
  bool useattr;
  charsett charset;

  void drawline(byte sizex);
  void drawstatus();
  void view(screent *s, FILE *f, const char *t, charsett cs, bool ua);
 };

void viewtextfilet::drawline(byte sizex)
 {
  int x=0;
  bool hi=FALSE, escaped=FALSE;
  char ch;

  while(TRUE)
   {
    ch=fgetc(file);

    if(ch=='\\' && !escaped)
      escaped=TRUE;
    else if(ch=='~' && !escaped && useattr)
     {
      hi=!hi;
      screen->setattr(1, hi);
     }
    else
     {
      escaped=FALSE;

      if(x==sizex)
       {
        while(ch!='\n' && ch!=(char)EOF) ch=fgetc(file);
        break;
       }
 
      if(ch=='\t') ch=' ';

      if((ch>=' ' || ch<=0) && ch!=(char)EOF)
       {
        screen->printf("%c", ch);
        x++;
       }
      else if(ch=='\n' || ch==(char)EOF)
        break;
     }
   }

  if(hi)
    screen->setattr(1, FALSE);
 }

void viewtextfilet::drawstatus()
 {
  int y, i, j;
  char status[]="-- Drcken Sie Return um fortzufahren --";

  screen->setattr(10, FALSE);
  y=strlen(status);
  j=(screen->sizex-y)/2;
  screen->setcursor(1, screen->sizey);
  for(i=0; i<j; i++) screen->putch(' ');
  screen->printf("%s", status);
  j=screen->sizex-j-y-1;
  for(i=0; i<j; i++) screen->printf(" ");
  screen->setattr(1, FALSE);
 }

void viewtextfilet::view(screent *s, FILE *f, const char *t,
  charsett cs, bool ua)
 {
  long y;

  title=t;
  file=f;
  screen=s;
  charset=cs;
  useattr=ua;

  if(title==NULL)
    title="";

  while(TRUE)
   {
   schleife:
    screen->setattr(1, FALSE);
    screen->clrscr();
    drawtitle(screen, title);
    drawstatus();

    if(charset!=IBM)
      screen->set_app_out_charset(charset);

    for(y=0; y<screen->sizey-3 && !feof(file); y++)
     {
      screen->setcursor(1, y+2);
      drawline(screen->sizex-2);
     }

    if(charset!=IBM)
      screen->set_app_out_charset(IBM);

    screen->setcursor(1, y+2);
    screen->setattr(1, TRUE);
    screen->printf(feof(file)?"--ENDE--":"--MORE--");
    screen->setattr(1, FALSE);

    while(TRUE)
     {
      switch(screen->doreadkey())
       {
       case Key_C_X:
       case Key_C_C:
       case Key_Esc:
       case 0xffff:
        //fclose(file);
        return;

       case Key_Spa:
       case 10:
       case 13:
        if(feof(file))
         {
          //fclose(file);
          return;
         }
        goto schleife;
       }
     }
   }
 }

void viewtextfile(screent *screen, FILE *file, const char *title,
                  charsett charset, bool useattr)
 {
  viewtextfilet vtf;
  vtf.view(screen, file, title, charset, useattr);
 }

bool viewarchive(char *infilename, char *outfilename)
 {
  FILE *infile;
  char tempstr[DIRLEN];

  infile=fopen(infilename, "rw");
  if(infile==NULL) return TRUE;

  remove(outfilename);
  fread(tempstr, 20, 1, infile);
  fclose(infile);

  if(tempstr[0]=='P' && tempstr[1]=='K' &&    // ZIP
     tempstr[2]>=0 && tempstr[2]<=' ' &&
     tempstr[3]>=0 && tempstr[2]<=' ')
   {
    snprintf(tempstr, DIRLEN, 
             "unzip -v \"%s\" > %s 2> /dev/null", infilename, outfilename);
    system(tempstr);
    return FALSE;
   }
  else if(tempstr[0]==0x1a && tempstr[1]>=0 && tempstr[1]<=9)
   {
    snprintf(tempstr, DIRLEN, 
      "unarc -v \"%s\" > %s 2> /dev/null", infilename, outfilename);
    system(tempstr);
    return FALSE;
   }
  else if(tempstr[2]=='-' && tempstr[3]=='l' &&
          (tempstr[4]=='z' || tempstr[4]=='h') &&
          tempstr[6]=='-')
   {
    snprintf(tempstr, DIRLEN, 
      "lha -l \"%s\" > %s 2> /dev/null", infilename, outfilename);
    system(tempstr);
    return FALSE;
   }
  else if((byte)tempstr[20]==0xdc && (byte)tempstr[21]==0xa7 &&
          (byte)tempstr[22]==0xc4 && (byte)tempstr[23]==0xfd)
   {
    snprintf(tempstr, DIRLEN,
      "zoo -v \"%s\" > %s 2> /dev/null", infilename, outfilename);
    system(tempstr);
    return FALSE;
   }
  else if((byte)tempstr[0]==0x52 && (byte)tempstr[1]==0x61 &&     // RAR
          (byte)tempstr[2]==0x72 && (byte)tempstr[3]==0x21)
   {
    snprintf(tempstr, DIRLEN,
      "unrar v \"%s\" > %s 2> /dev/null", infilename, outfilename);
    system(tempstr);
    return FALSE;
   }
  else if(tempstr[0]==0x1f)
   {
    snprintf(tempstr, DIRLEN,
      "tar tvfz \"%s\" > %s 2> /dev/null", infilename, outfilename);
    system(tempstr);
    return FALSE;
   }
  else if(tempstr[0]=='G' && tempstr[1]=='I' && tempstr[2]=='F')
   {
    snprintf(tempstr, DIRLEN,
      "echo Detected GIF > %s", outfilename);
    system(tempstr);
    return FALSE;
   }
  return TRUE;
 }

void drawscreen(screent *screen, const char *title, const char *version)
 {
  int i;

  screen->clrscr();
  screen->box(1, 1, screen->sizex, screen->sizey-1, BOXFRAMEDBL, title, FALSE);

  screen->setcursor(1, 3);

  if(screen->hw_out_charset==IBM)
    screen->printf(
      "             \r\n"
      "            \r\n"
      "              \r\n"
      "            \r\n"
      "             ");
  else
    screen->printf(
      "  ,------   ,----    ,----   ,-- ,--\r\n"
      "  ,--   ,-- ,-- ,-- ,--  ,-- ,-- ,--\r\n"
      "  ,--   ,-- ,----   ,--  ,--   ,--\r\n"
      "  ,--   ,-- ,-- ,-- ,--  ,-- ,-- ,--\r\n"
      "  ,------   ,----    ,----   ,-- ,--");

  screen->setcursor(48, 4);
  screen->printf("%s", version);
  screen->setcursor(52, 6);
  screen->printf("by Daniel Krning");

  for(i=3; i<8; i++)
   {
    screen->setcursor(41, i);
    screen->printf("");
   }

  screen->setcursor(3, 9);

  for(i=3; i<screen->sizex-1; i++)
    screen->printf("");
 }

void clearworkscreen(screent *screen)
 {
  int x, y;

  for(y=10; y<screen->sizey-1; y++)
   {
    screen->setcursor(2, y);
    for(x=2; x<screen->sizex; x++) 
      #ifdef __DEBUG__
      screen->printf("");
      #else
      screen->putch(' ');
      #endif
   }
 }

void formatdate(char *s, time_t t)
 {
  struct tm *tm;

  tm=localtime(&t);

  snprintf(s, TEXTLEN, "%02u.%02u.%04u, %02u:%02u Uhr",
    tm->tm_mday, tm->tm_mon+1, tm->tm_year+1900, tm->tm_hour, tm->tm_min);
 }

#if 0
void doselect(int fd1, int fd2)
 {
  fd_set readfds;
  struct timeval timeout={ WAIT, 0 };
  int numfds;

  numfds=fd1;
  if(fd2!=0 && fd2>numfds) numfds=fd2;

  FD_ZERO(&readfds);
  FD_SET(fd1, &readfds);
  if(fd2!=0) FD_SET(fd2, &readfds);
  select(numfds+1, &readfds, (fd_set *)NULL, (fd_set *)NULL, &timeout);
 }
#endif

void errorbox(screent *screen, const char *error)
 {
  met mes[2];
  int i;
  byte x;
  char tempstr[TEXTLEN];

  i=snprintf(tempstr, TEXTLEN, "Fehler: %s", error)+3;
  x=(screen->sizex-i)/2;
  screen->setattr(3, FALSE);
  screen->box(x, 10, x+i, 16, BOXFRAMEDBL, "Fehler", TRUE);
  screen->setcursor(x+2, 12);
  screen->printf("%s", tempstr);

  mes[0].type=met::COMMANDBUTTON;
  mes[0].x=(screen->sizex-9)/2;
  mes[0].y=14;
  mes[0].sizex=9;
  mes[0].text="^Abbruch";

  mes[1].type=met::END_OF_LIST;

 schleife:
  i=domenu(screen, mes, 0, TRUE);
  if(i==MENU_HELP) goto schleife;
  screen->setattr(1, FALSE);
 }

void infobox(screent *screen, const char *text)
 {
  met mes[2];
  int i;
  byte x;

  i=max(unsigned(strlen(text)+3), (unsigned)20);
  x=(screen->sizex-i)/2;
  screen->setattr(2, FALSE);
  screen->box(x, 10, x+i, 16, BOXFRAMEDBL, (char *)NULL, TRUE);
  screen->setcursor(x+2, 12);
  screen->printf("%s", text);

  mes[0].type=met::COMMANDBUTTON;
  mes[0].x=(screen->sizex-9)/2;
  mes[0].y=14;
  mes[0].sizex=10;
  mes[0].text="O^k";

  mes[1].type=met::END_OF_LIST;

 schleife:
  i=domenu(screen, mes, 0, TRUE);
  if(i==MENU_HELP) goto schleife;
  screen->setattr(1, FALSE);
 }

void deldir(const char *directory)
 {
  char tempstr[DIRLEN], tempstr2[DIRLEN];
  DIR *dir;
  struct dirent *ent;

  if(directory[0]==0) return;

  if(directory[strlen(directory)-1]==DIRCHAR[0])
   {
    strmaxcpy(tempstr, directory, DIRLEN-1);
    tempstr[strlen(tempstr)-1]=0;
   }
  else
    strmaxcpy(tempstr, directory, DIRLEN-1);

  if((dir=opendir(tempstr))==NULL) return;

  while((ent=readdir(dir))!=NULL)
   {
    if(ent->d_name[0]!='.')
     {
      snprintf(tempstr2, DIRLEN, "%s" DIRCHAR "%s", tempstr, ent->d_name);
      remove(tempstr2);
     }
   }

  closedir(dir);
 }

