#include <stddef.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <ctype.h>
#include <stdarg.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/time.h>

#include <minmax.h>

#include <dbox.h>
#include <dboxpath.h>

#include <config.h>
#include <oldterm.h>
#include <oldtools.h>
#include <oldui.h>
#include <logf.h>
#include <keys.h>
#include <profile.h>
#include <userdb.h>
#include <blzdb.h>

static char gethotkey(char *text)
 {
  text=strchr(text, '^');
  if(text==NULL) return 0;
  return *(text+1);
 }

static void seteditcursor(screent *screen, met *me)
 {
  screen->setcursor(me->x+me->posx-me->scrollx+1, me->y);
 }

static void doblz(screent *screen, met *me)
 {
  dword blz;
  char name[MAXBLZLEN];
  int i;

  if(me->blzx==0 || me->blzy==0) return;

  blz=atol(me->text);

  if(blz<10000000) blz=0;

  if(blz!=me->oldblz)
   {
    if(blz==0)
      name[0]=0;
    else
      blzdb.lookup(blz, name);

    screen->setcursor(me->blzx, me->blzy);
    screen->setattr(10, FALSE);
    screen->printf(" %s", name);
    i=MAXBLZLEN-strlen(name);
    while(i>0) { screen->putch(' '); i--; }
    screen->setattr(1, FALSE);

    me->oldblz=blz;
   }
 }

static void drawedit(screent *screen, met *me, bool selected)
 {
  char *tptr;
  int i;

  screen->setcursor(me->x, me->y);
  screen->setattr(me->disabled?1:(selected?11:10), FALSE);
  screen->putch(selected?'>':' ');
  tptr=me->text+me->scrollx;
  i=me->sizex;
  while(*tptr!=0 && i!=0)
   {
    if(*tptr>=' ' || *tptr<0)
      screen->printf("%c", *tptr);
    else
     {
      screen->setattr(selected?11:10, TRUE);
      screen->printf("*");
      screen->setattr(selected?11:10, FALSE);
     }
    i--;
    tptr++;
   }
  while(i>0)
   {
    screen->putch('_');
    i--;
   }
  screen->putch(selected?'<':' ');
  screen->setattr(1, FALSE);

  if(me->edittype==met::BLZ) doblz(screen, me);

  if(selected) seteditcursor(screen, me);
 }

void drawme(screent *screen, met *me, bool selected)
 {
  char *tptr;
  int i, j;

  switch(me->type)
   {
   case met::COMMANDBUTTON:
    if(me->center)
     {
      if(strchr(me->text, '^')==NULL) i=0; else i=1;
      j=((int)me->sizex-(int)strlen(me->text)+i)/2;
      if(j<0) j=0;
     }
    else
      j=0;

    screen->setcursor(me->x, me->y);
    screen->setattr(me->disabled?1:(selected?11:10), FALSE);
    screen->putch(selected?'>':' ');
    for(i=0; i<j; i++) screen->putch(' ');
    tptr=me->text;
    i=me->sizex-j;
    while(*tptr!=0)
     {
      if(*tptr=='^' && *(tptr+1)!=0)
       {
        tptr++;
        screen->setattr(me->disabled?1:(selected?11:10), TRUE);
        screen->printf("%c", *(tptr++));
        screen->setattr(me->disabled?1:(selected?11:10), FALSE);
       }
      else
        screen->printf("%c", *(tptr++));
      i--;
     }

    while(i>0)
     {
      screen->putch(' ');
      i--;
     }

    screen->putch(selected?'<':' ');
    screen->setattr(1, FALSE);

    if(selected) screen->setcursor(me->x, me->y);
    break;

   case met::EDIT:
    if(selected)
     {
      me->posx=strlen(me->text);
      if(me->posx>me->sizex-1)
        me->scrollx=me->posx-me->sizex+1;
      else
        me->scrollx=0;
     }
    else
     {
      me->posx=me->scrollx=0;
     }

    drawedit(screen, me, selected);
    break;

   case met::LISTBOX:

    break;

   case met::CHECKBOX:
    #if 0
    screen->setcursor(me->x, me->y);
    screen->setattr(FALSE, TRUE);
    screen->printf(" [%c] ", me->selected?'x':' ');
    screen->putch(' ');
    tptr=me->text;
    while(*tptr!=0)
     {
      if(*tptr=='^' && *(tptr+1)!=0)
       {
        tptr++;
        screen->setattr(TRUE, TRUE);
        screen->printf("%c", *(tptr++));
        screen->setattr(FALSE, TRUE);
       }
      else
        screen->printf("%c", *(tptr++));
     }
    #endif
    break;

   case met::KEY:
    break;

   case met::END_OF_LIST:
   case met::IGNORE:
   case met::SELECT:
    break;
   }
 }

static bool selectable(met *me)
 {
  switch(me->type)
   {
   case met::COMMANDBUTTON:
   case met::LISTBOX:
   case met::CHECKBOX:
   case met::EDIT:
    return !me->disabled;

   case met::IGNORE:
   case met::END_OF_LIST:
   case met::KEY:
   case met::SELECT:
    break;
   }
  return FALSE;
 }

/*

-1: ESC
-2: Bitte Ende
-3: Bitte Hilfe

*/

int domenu(screent *screen, met *mes, int start, bool redrawall)
 {
  met *me;
  word key;
  int i, sele, lastsele=-1, first_select=-1, last_select=-1;

  me=mes;
  sele=start;
  i=0;
  while(me->type!=met::END_OF_LIST)
   {
    if(mes[i].type==met::SELECT)
     {
      if(first_select==-1) first_select=i;
      last_select=i;
     }      

    if(selectable(&mes[i]))
     {
      if(!selectable(&mes[sele])) sele=i;
      lastsele=i;
     }
    if(redrawall || i==sele) drawme(screen, me, i==sele);
    i++;
    me++;
   }

  if(!selectable(&mes[sele]))
   {
    sele=-1;
    screen->setcursor(1, 1);
   }
  else if(mes[sele].type==met::EDIT)
   {
    me=&mes[sele];
    seteditcursor(screen, me);
   }
  else
    screen->setcursor(mes[sele].x, mes[sele].y);

  while(TRUE)
   {
    switch(key=screen->doreadkey())
     {
     case 0xffff:
      return -2;

     case Key_Esc:
      return -1;

     case Key_F1:
     case '?':
      return MENU_HELP;

     case Key_Up:
     case Key_Lft:
      if(sele==-1)
       {
        if(key==Key_Up) return first_select;
        goto dodefault;
       }

      if(key==Key_Lft)
       {
        if(mes[sele].type==met::EDIT)
         {
          me=&mes[sele];
          if(me->posx!=0)
           {
            me->posx--;
            if(me->posx<me->scrollx)
             {
              me->scrollx--;
              drawedit(screen, me, TRUE);
             }
            else
              seteditcursor(screen, me);
           }
         }
        else
          goto dodefault;
       }
      else
       {
        drawme(screen, &mes[sele], FALSE);

        do
         {
          if(sele==0)
            sele=lastsele;
          else
            sele--;

          if(mes[sele].type==met::SELECT)
            return sele;
         }
        while(!selectable(&mes[sele]));
        drawme(screen, &mes[sele], TRUE);
       }
      break;

     dodown:
     case Key_Tab:
     case Key_Dwn:
     case Key_Rgt:
     case Key_Spa:
      if(sele==-1)
       {
        if(key==Key_Dwn) return last_select;
        goto dodefault;
       }

      if(key==Key_Spa && mes[sele].type==met::EDIT)
        goto dodefault;

      if(key==Key_Rgt)
       {
        if(mes[sele].type==met::EDIT)
         {
          me=&mes[sele];
          if(me->posx!=strlen(me->text))
           {
            me->posx++;
            if(me->posx-me->scrollx>=me->sizex)
             {
              me->scrollx++;
              drawedit(screen, me, TRUE);
             }
            else
              seteditcursor(screen, me);
           }
         }
        else
          goto dodefault;
       }
      else
       {
        drawme(screen, &mes[sele], FALSE);
        do
         {  
          sele++;
          if(mes[sele].type==met::END_OF_LIST) 
            sele=0;

          if(mes[sele].type==met::SELECT)
            return sele; 
         }
        while(!selectable(&mes[sele]));
        drawme(screen, &mes[sele], TRUE);
       }
      break;

     case Key_Ent:
      if(sele==-1)
       {
        if(last_select!=-1) return last_select;
        goto dodefault;
       }

      if(mes[sele].type==met::EDIT)
        goto dodown;

      return sele;

     case Key_End:
      if(sele==-1) goto dodefault;

      if(mes[sele].type==met::EDIT)
       {
        me=&mes[sele];
        if(me->posx<strlen(me->text))
         {
          me->posx=strlen(me->text);
          if(me->posx-me->scrollx>=me->sizex-1)
           {
            me->scrollx=me->posx-me->sizex+1;
            drawedit(screen, me, TRUE);
           }
          else
            seteditcursor(screen, me);
         }
       }
      else
        goto dodefault;

      break;

     case Key_Home:
      if(sele==-1) goto dodefault;

      if(mes[sele].type==met::EDIT)
       {
        me=&mes[sele];
        if(me->posx!=0)
         {
          me->posx=0;
          if(me->scrollx!=0)
           {
            me->scrollx=0;
            drawedit(screen, me, TRUE);
           }
          else
            seteditcursor(screen, me);
         }
       }
      else
        goto dodefault;

      break;

     case Key_BS:
      if(sele==-1) goto dodefault;

      if(mes[sele].type==met::EDIT)
       {
        me=&mes[sele];
        if(me->posx!=0)
         {
          char tempstr[TEXTLEN*2];

          strmaxcpy(tempstr, me->text+me->posx, (TEXTLEN*2)-1);
          me->posx--;
          strmaxcpy(me->text+me->posx, tempstr, TEXTLEN-me->posx-1);
          if(me->posx<me->scrollx)
           {
            me->scrollx--;
            drawedit(screen, me, TRUE);
           }
          else
           {
            tempstr[me->sizex-(me->posx-me->scrollx)]=0;
            seteditcursor(screen, me);
            screen->setattr(11, FALSE);
            screen->printf("%s", tempstr);
            if(strlen(tempstr)+(me->posx-me->scrollx)<me->sizex)
              screen->printf("_");
            screen->setattr(1, FALSE);

            if(me->edittype==met::BLZ) doblz(screen, me);

            seteditcursor(screen, me);
           }
         }
       }
      else
        goto dodefault;
      break;

     default:
     dodefault:
      if(key!=0)
       {
        char ch=lo(key);
        if(mes[sele].type==met::EDIT && ((ch>=' ' && ch<127) || ch<0))
         {
          bool keyok=TRUE;

          me=&mes[sele];

          switch(me->edittype)
           {
           case met::NUMBER:
           case met::BLZ:
            if(!isdigit(ch)) keyok=FALSE;
            break;

           case met::USERNAME:
            if(!usercharisok(ch))
              keyok=FALSE;

            if(me->posx==0 && !(isalpha(ch) || isdigit(ch)))
              keyok=FALSE;

            break;

           default:;
           }

          if(keyok && strlen(me->text)<me->maxlen)
           {
            char tempstr[TEXTLEN*2];
            snprintf(tempstr, TEXTLEN, "%c%s", ch, me->text+me->posx);
            strmaxcpy(me->text+me->posx, tempstr, TEXTLEN-me->posx-1);
            me->posx++;
            if(me->posx-me->scrollx>=me->sizex)
             {
              me->scrollx++;
              drawedit(screen, me, TRUE);
             }
            else
             {
              tempstr[me->sizex-(me->posx-me->scrollx)+1]=0;
              screen->setattr(11, FALSE);
              screen->printf("%s", tempstr);
              screen->setattr(1, FALSE);

              if(me->edittype==met::BLZ) doblz(screen, me);

              if(strlen(tempstr)>1 || me->edittype==met::BLZ)
                seteditcursor(screen, me);
             }
           }
         }
        else
         {
          me=mes;
          i=0;
          while(me->type!=met::END_OF_LIST)
           {
            switch(me->type)
             {
             case met::COMMANDBUTTON:
              if(me->key==key ||
                 (lo(key)!=0 &&
                 (upcase(lo(key))==upcase(gethotkey(me->text)))))
                return i;
              break;

             case met::KEY:
             case met::SELECT:
              if(me->key==key) return i;
              break;

             case met::IGNORE:
             case met::EDIT:
             case met::CHECKBOX:
             case met::LISTBOX:
             case met::END_OF_LIST:
              break;
             }
            me++;
            i++;
           }
          if(upcase(lo(key))=='H') return MENU_HELP;
         }
       }
      break;
     }
   }
 }

