#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <time.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/time.h>
#include <errno.h>

#include <minmax.h>

#include <dbox.h>
#include <md5.h>
#include <config.h>
#include <radius.h>

#define AUTH_VECTOR_LEN		16
#define AUTH_PASS_LEN		16
//#define AUTH_ID_LEN		64
//#define AUTH_STRING_LEN		128	/* maximum of 253 */

//#define NAME_LENGTH		32
//#define	GETSTR_LENGTH		128	/* must be bigger than AUTH_PASS_LEN */

// /* codes for radius_buildreq, radius_getport, etc. */
//#define AUTH			0
//#define ACCT			1

typedef struct pw_auth_hdr
 {
  byte code;
  byte id;
  word length;
  byte vector[AUTH_VECTOR_LEN];
  byte data[2];
 } AUTH_HDR;

#define AUTH_HDR_LEN			20
#define MAX_SECRET_LENGTH		16
//#define CHAP_VALUE_LENGTH		16

void md5_calc(unsigned char *output, unsigned char *input, unsigned int inlen);

int radiust::authenticate(const char *password, char *message)
 {
  int result, i;

  anzattributes=0;
  code=1;
  message[0]=0;

  attributes[anzattributes].type=1;   // Username
  strmaxcpy(attributes[anzattributes].data.string, username, TEXTLEN-1);
  attributes[anzattributes].length=strlen(username);
  anzattributes++;
  
  attributes[anzattributes].type=2;   // Password
  strmaxcpy(attributes[anzattributes].data.string, password, TEXTLEN-1);
  attributes[anzattributes].length=strlen(password);
  anzattributes++;

  if(nas_ip_address!=0)
   {
    attributes[anzattributes].type=4;   // NAS-IP-Address
    attributes[anzattributes].data.integer=nas_ip_address;
    attributes[anzattributes].length=4;
    anzattributes++;
   }

  if(port!=0)
   {
    attributes[anzattributes].type=5;   // NAS-Port
    attributes[anzattributes].data.integer=htonl((dword)port);
    attributes[anzattributes].length=4;
    anzattributes++;
   }

  attributes[anzattributes].type=6;   // Service-Type
  attributes[anzattributes].data.integer=htonl(2);
  attributes[anzattributes].length=4;
  anzattributes++;

  if(calling_station_id[0]!=0)
   {
    attributes[anzattributes].type=31;   // Calling-Station-ID
    strmaxcpy(attributes[anzattributes].data.string, calling_station_id, TEXTLEN-1);
    attributes[anzattributes].length=strlen(calling_station_id);
    anzattributes++;
   }

  if(session_id[0]!=0)
   {
    attributes[anzattributes].type=44;   // Acct-Session-Id
    strmaxcpy(attributes[anzattributes].data.string, session_id, TEXTLEN-1);
    attributes[anzattributes].length=strlen(session_id);
    anzattributes++;
   }

  result=sendserver(radius_auth_server, 1645);
  //result=sendserver(radius_auth_server, 3000);

  if(result!=0) return result;

  for(i=0; i<anzattributes; i++)
    if(attributes[i].type==18)
     {
      strmaxcpy(message, attributes[i].data.string, TEXTLEN-1);
      break;
     }

  switch(code)
   {
   case 2:
    return 0;

   case 3:
    return 1;
   }

  return 4;
 }

int radiust::log_start()
 {
  anzattributes=0;
  code=4;

  attributes[anzattributes].type=1;   // Username
  strmaxcpy(attributes[anzattributes].data.string, username, TEXTLEN-1);
  attributes[anzattributes].length=strlen(username);
  anzattributes++;
  
  if(nas_ip_address!=0)
   {
    attributes[anzattributes].type=4;   // NAS-IP-Address
    attributes[anzattributes].data.integer=nas_ip_address;
    attributes[anzattributes].length=4;
    anzattributes++;
   }

  if(ip_address!=0)
   {
    attributes[anzattributes].type=8;   // Framed-IP-Address
    attributes[anzattributes].data.integer=ip_address;
    attributes[anzattributes].length=4;
    anzattributes++;

    attributes[anzattributes].type=9;   // Framed-IP-Netmask
    attributes[anzattributes].data.integer=getinternetaddress("255.255.255.255");
    attributes[anzattributes].length=4;
    anzattributes++;
   }

  if(port!=0)
   {
    attributes[anzattributes].type=5;   // NAS-Port
    attributes[anzattributes].data.integer=htonl((dword)port);
    attributes[anzattributes].length=4;
    anzattributes++;
   }

  attributes[anzattributes].type=6;   // Service-Type
  attributes[anzattributes].data.integer=htonl(2);
  attributes[anzattributes].length=4;
  anzattributes++;

  if(calling_station_id[0]!=0)
   {
    attributes[anzattributes].type=31;   // Calling-Station-ID
    strmaxcpy(attributes[anzattributes].data.string, calling_station_id, TEXTLEN-1);
    attributes[anzattributes].length=strlen(calling_station_id);
    anzattributes++;
   }

  attributes[anzattributes].type=40;    // Acct-Status-Type
  attributes[anzattributes].data.integer=htonl(1); // Start
  attributes[anzattributes].length=4;
  anzattributes++;

  if(session_id[0]!=0)
   {
    attributes[anzattributes].type=44;   // Acct-Session-Id
    strmaxcpy(attributes[anzattributes].data.string, session_id, TEXTLEN-1);
    attributes[anzattributes].length=strlen(session_id);
    anzattributes++;
   }

  attributes[anzattributes].type=45;    // Acct-Authentic
  attributes[anzattributes].data.integer=htonl(1); // RADIUS
  attributes[anzattributes].length=4;
  anzattributes++;

  return sendserver(radius_acct_server, 1646);
 }

int radiust::log_stop(dword dauer)
 {
  anzattributes=0;
  code=4;

  attributes[anzattributes].type=1;   // Username
  strmaxcpy(attributes[anzattributes].data.string, username, TEXTLEN-1);
  attributes[anzattributes].length=strlen(username);
  anzattributes++;
  
  if(nas_ip_address!=0)
   {
    attributes[anzattributes].type=4;   // NAS-IP-Address
    attributes[anzattributes].data.integer=nas_ip_address;
    attributes[anzattributes].length=4;
    anzattributes++;
   }

  if(port!=0)
   {
    attributes[anzattributes].type=5;   // NAS-Port
    attributes[anzattributes].data.integer=htonl((dword)port);
    attributes[anzattributes].length=4;
    anzattributes++;
   }

  attributes[anzattributes].type=6;   // Service-Type
  attributes[anzattributes].data.integer=htonl(2);
  attributes[anzattributes].length=4;
  anzattributes++;

  if(calling_station_id[0]!=0)
   {
    attributes[anzattributes].type=31;   // Calling-Station-ID
    strmaxcpy(attributes[anzattributes].data.string, calling_station_id, TEXTLEN-1);
    attributes[anzattributes].length=strlen(calling_station_id);
    anzattributes++;
   }

  attributes[anzattributes].type=40;    // Acct-Status-Type
  attributes[anzattributes].data.integer=htonl(2); // Stop
  attributes[anzattributes].length=4;
  anzattributes++;

  if(session_id[0]!=0)
   {
    attributes[anzattributes].type=44;   // Acct-Session-Id
    strmaxcpy(attributes[anzattributes].data.string, session_id, TEXTLEN-1);
    attributes[anzattributes].length=strlen(session_id);
    anzattributes++;
   }

  attributes[anzattributes].type=45;    // Acct-Authentic
  attributes[anzattributes].data.integer=htonl(1); // RADIUS
  attributes[anzattributes].length=4;
  anzattributes++;

  attributes[anzattributes].type=46;    // Acct-Session-Time
  attributes[anzattributes].data.integer=htonl(dauer);
  attributes[anzattributes].length=4;
  anzattributes++;

  return sendserver(radius_acct_server, 1646);
 }

/*
 * Function: rc_random_vector 
 *
 * Purpose: generates a random vector of AUTH_VECTOR_LEN octets.
 *
 * Returns: the vector (call by reference)
 *
 */

static void rc_random_vector (unsigned char *vector)
{
	int             randno;
	int             i;

	srand (time (0) + getppid() + getpid()); /* random enough :) */
	for (i = 0; i < AUTH_VECTOR_LEN;)
	{
		randno = rand ();
		memcpy ((char *) vector, (char *) &randno, sizeof (int));
		vector += sizeof (int);
		i += sizeof (int);
	}

	return;
} 

int radiust::sendserver(internetaddresst server_address, word udp_port)
 {
  int sockfd;
  struct sockaddr salocal;
  struct sockaddr saremote;
  struct sockaddr_in *sin;
  int i, j;
  struct timeval authtime;
  fd_set readfds;
  AUTH_HDR *auth, *recv_auth;
  //UINT4 auth_ipaddr;
  int total_length, retry_max, retries, secretlen;
  socklen_t length, salen;
  //int result;
  unsigned char vector[AUTH_VECTOR_LEN];
  char recv_buffer[1024], send_buffer[1024];
  unsigned char calc_digest[AUTH_VECTOR_LEN];
  unsigned char reply_digest[AUTH_VECTOR_LEN];
  unsigned char passbuf[AUTH_PASS_LEN];
  unsigned char md5buf[256];

  secret[MAX_SECRET_LENGTH]=0;

  sockfd=socket(AF_INET, SOCK_DGRAM, 0);
  if(sockfd<0) return 3;

  length=sizeof(salocal);
  sin=(struct sockaddr_in *)&salocal;
  memset((char *)sin, '\0', (size_t)length);
  sin->sin_family=AF_INET;
  sin->sin_addr.s_addr=INADDR_ANY;
  sin->sin_port=htons((unsigned short)0);

  if(bind(sockfd, (struct sockaddr *)sin, length)<0 ||
     getsockname(sockfd, (struct sockaddr *)sin, &length)<0)
   {
    close(sockfd);
    return 3;
   }
	
  retry_max=10;
  retries=0;

  auth=(AUTH_HDR *)send_buffer;
  auth->code=code;
  rc_random_vector(vector);
  memcpy(auth->vector, vector, AUTH_VECTOR_LEN);
  auth->id=identifier;

  total_length=AUTH_HDR_LEN;

  for(i=0; i<anzattributes; i++)
   {
    send_buffer[total_length]=attributes[i].type;
    total_length++;

    if(attributes[i].type==2)
     {
      send_buffer[total_length]=AUTH_PASS_LEN+2;
      total_length++;

      length=attributes[i].length; 
      if(length>AUTH_PASS_LEN) length=AUTH_PASS_LEN;
      memset(passbuf, '\0', AUTH_PASS_LEN);
      memcpy(passbuf, attributes[i].data.string, length);

      /* Calculate the MD5 Digest */
      strcpy((char *)md5buf, secret); // md5buf is assumed to be large enough
      memcpy(md5buf+strlen(secret), auth->vector, AUTH_VECTOR_LEN);
      md5_calc((unsigned char *)send_buffer+total_length, md5buf, 
               strlen(secret)+AUTH_VECTOR_LEN);

      /* Xor the password into the MD5 digest */
      for(j=0; j<AUTH_PASS_LEN; j++)
        send_buffer[total_length+j]^=passbuf[j];

      total_length+=AUTH_PASS_LEN;
     }
    else
     {
      send_buffer[total_length]=attributes[i].length+2;
      total_length++;
      memcpy(send_buffer+total_length, attributes[i].data.string, 
             attributes[i].length); 
      total_length+=attributes[i].length;
     }
   }

  auth->length=htons((unsigned short)total_length);

  if(auth->code==4)    // Acct-Request
   {
    memset(auth->vector, 0, AUTH_VECTOR_LEN);
    strcpy(send_buffer+total_length, secret); // md5buf is assumed to be large enough
    md5_calc(vector, (unsigned char *)send_buffer, total_length+strlen(secret));
    memcpy(auth->vector, vector, AUTH_VECTOR_LEN);
   }
   
  sin=(struct sockaddr_in *)&saremote;
  memset(sin, '\0', sizeof(saremote));
  sin->sin_family=AF_INET;
  sin->sin_addr.s_addr=server_address;
  sin->sin_port=htons(udp_port);

  for(;;)
   {
    sendto(sockfd, send_buffer, total_length, 0, 
           (struct sockaddr *)sin, sizeof(struct sockaddr_in));

    authtime.tv_usec=0L;
    authtime.tv_sec=3;   // 3 Sekunden Timeout
    FD_ZERO(&readfds);
    FD_SET(sockfd, &readfds);

    if(select(sockfd+1, &readfds, (fd_set *)NULL, (fd_set *)NULL, &authtime)<0)
     {
      if(errno==EINTR) continue;
      close(sockfd);
      return 3;
     }

    if(FD_ISSET(sockfd, &readfds)) break;

    /*
     * Timed out waiting for response.  Retry "retry_max" times
     * before giving up.  If retry_max = 0, don't retry at all.
     */

    if(++retries>=retry_max)
     {
      close(sockfd);
      return 2;
     }
   }

  salen=sizeof(saremote);
  length=recvfrom(sockfd, (char *)recv_buffer, sizeof(recv_buffer),
                  0, &saremote, &salen);

  close (sockfd);

  if(length<=0) return 3;
	
  recv_auth=(AUTH_HDR *)recv_buffer;
  code=recv_auth->code;
  if(recv_auth->id!=identifier) return 4;

  total_length=ntohs(recv_auth->length);
  memcpy(reply_digest, recv_auth->vector, AUTH_VECTOR_LEN);
  memcpy(recv_auth->vector, vector, AUTH_VECTOR_LEN);
  secretlen=strlen(secret);
  memcpy(recv_buffer+total_length, secret, secretlen);
  md5_calc(calc_digest, (unsigned char *)recv_auth, total_length+secretlen);
  
  if(recv_auth->code!=5) // Bug in Radiusd 1.16, behoben in 2.0
    if(memcmp((char *)reply_digest, (char *)calc_digest, AUTH_VECTOR_LEN)!=0)
      return 4;

  anzattributes=0;
  i=AUTH_HDR_LEN;

  while(i<total_length)
   {
    int finallen;

    attributes[anzattributes].type=recv_buffer[i];
    i++;
    attributes[anzattributes].length=recv_buffer[i];
    i++;
    if(attributes[anzattributes].length<2) break;
    finallen=min(attributes[anzattributes].length-2, TEXTLEN-1);
    memcpy(attributes[anzattributes].data.string, recv_buffer+i, finallen);
    attributes[anzattributes].data.string[finallen]=0;
    i+=attributes[anzattributes].length-2;
    anzattributes++;
   }

  return 0;

  #if 0	
  data->receive_pairs = rc_avpair_gen(recv_auth);
	
  result = rc_check_reply (recv_auth, secret, vector, data->seq_nbr);

  if (result != OK_RC) return (result);
	
	*msg = '\0';
	vp = data->receive_pairs;
	while (vp)
	{
		if ((vp = rc_avpair_get(vp, PW_REPLY_MESSAGE)))
		{
			strcat(msg, vp->strvalue);
			vp = vp->next;
		}
	}
	
	if ((recv_auth->code == PW_ACCESS_ACCEPT) ||
		(recv_auth->code == PW_PASSWORD_ACK) ||
		(recv_auth->code == PW_ACCOUNTING_RESPONSE))
	{
		result = OK_RC;
	}
	else
	{
		result = BADRESP_RC;
	}

	return (result);
 #endif
 }

