#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <ctype.h>

#include <fstream.h>

#include <tools.h>
#include <logf.h>
#include <folder.h>
#include <folderindex.h>
#include <stringtools.h>
#include <dboxpath.h>
#include <mtoolrfc.h>
#include <sfolderrfc.h>
#include <parsefrom.h>
#include <rfc1522.h>

static bool checkfolderdir(const char *directory)
 {
  char tempstr[DIRLEN];

  snprintf(tempstr, DIRLEN, "%s.", directory);
  if(access(tempstr, 00)==0) return FALSE;

  return TRUE;
 }

bool spool_in_folder(rfcmessaget &m, const string &folder,
                     bool forceextern, const string &envelope_to)
 {
  dword nr;
  folderpt fp(folder.c_str());

  if(fp.lock())
   {
    logf.printf("spool_in_folder",
      "fatal: folder %s is locked", fp.name);
    return TRUE;
   }

  bool result=spool_in_folder(m, fp, forceextern, envelope_to, nr);

  fp.unlock();

  return result;
 }

extern "C" time_t parsedate(char *line, int *tz);

time_t parsedate(const string &header)
 {
  int tz;

  return parsedate((char *)header.c_str(), &tz);
 }

bool spool_in_folder(rfcmessaget &m, folderpt &fp,
                     bool forceextern, const string &envelope_to,
                     dword &nr)
 {
  folderindext fi;
  char envelope_header[TEXTLEN*2];

  if(envelope_to!="")
    snprintf(envelope_header, TEXTLEN*2, "X-POP3-Rcpt:\t%s\r\n", 
      envelope_to.c_str());
  else
    envelope_header[0]=0;

  fp.setfilename();

  if(checkfolderdir(fp.getpath()))
   {
    logf.printf("spool_in_folder",
      "fatal: failed to access folder \"%s\"", fp.getpath());
    return TRUE;
   }

  fp.read();

   {
    char tempstr[DIRLEN];

    snprintf(tempstr, DIRLEN, "%sheader", fp.getpath());

    ofstream outfile(tempstr, ios::out|ios::app|ios::bin);
    if(!outfile)
     {
      logf.printf("spool_in_folder",
        "fatal: failed to open \"%s\"", tempstr);
      fp.unlock();
      return TRUE;
     }

    fi.clear();

    string header; 

    m.findsingleheader(header, "Subject");
    header=decode_rfc1522(header);
    iso2ibm(header);
    strmaxcpy(fi.bet, header.c_str(), TEXTLEN-1);

     {
      string mailto, name;
      m.findsingleheader(header, "From");
      parse_from(header, mailto, name);
      string parsed_from=mailto;
      if(name!="") parsed_from+=" ("+decode_rfc1522(name)+")";
      strmaxcpy(fi.abs, parsed_from.c_str(), TEXTLEN-1);
     }

    m.findsingleheader(header, "Message-ID"); // TODO: mehr parsen
    strmaxcpy(fi.mid, header.c_str(), TEXTLEN-1);

    m.findsingleheader(header, "Date"); // TODO: mehr parsen
    fi.eda=parsedate(header);

    time(&fi.received);
    fi.len=m.len;
    fi.headerlen=m.headerlen+strlen(envelope_header);
    if(forceextern || fi.len>EXTERNLEN) fi.flags.bodyextern=TRUE;
    fi.flags.binary=FALSE;
    fi.flags.rfc=TRUE;
    fi.headeroffs=outfile.tellp();
    fi.id=atol(fp|"nextid");
    if(fi.id==0) fi.id=1;

    outfile << envelope_header;

    if(fi.flags.bodyextern)
     {
      if(m.copyheader(outfile))
       {
        logf.printf("spool_in_folder",
          "fatal: failed to store header in \"%s\"", tempstr);
        return TRUE;
       }

      string externalfilename, hint;
      m.findsingleheader(hint, "Subject");
      makeexternfilename(externalfilename, hint, fp.getpath());

      ofstream outfile2(externalfilename.c_str(), ios::bin|ios::out);
      if(!outfile2)
       {
        logf.printf("spool_in_folder",
          "fatal: failed to open \"%s\"",
          externalfilename.c_str());
        return TRUE;
       }

      strmaxcpy(fi.file, strrchr(externalfilename.c_str(), 
        DIRCHAR[0])+1, TEXTLEN-1);

      m.copybody(outfile2);
     }
    else
     {
      if(m.copy(outfile))
       {
        logf.printf("spool_in_folder",
          "fatal: failed to store message in \"%s\"", tempstr);
        return TRUE;
       }
     }
   }

   {
    char tempstr[DIRLEN];

    snprintf(tempstr, DIRLEN, "%sindex", fp.getpath());

    ofstream indexfile(tempstr, ios::out|ios::app);

    if(!indexfile)
     {
      logf.printf("spool_in_folder",
        "fatal: failed to open \"%s\"", tempstr);
      return TRUE;
     }

    nr=indexfile.tellp()/sizeof fi;

    if(!indexfile.write(&fi, sizeof fi))
     {
      logf.printf("spool_in_folder",
        "fatal: failed to store index in \"%s\"", tempstr);
     }
   }

  fp.set("nextid", fi.id+1);
  fp.write();

  return FALSE;
 }
