#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <dirent.h>
#include <sys/stat.h>

#include <dbox.h>
#include <tools.h>
#include <stoolzc.h>
#include <logf.h>
#include <config.h>
#include <dboxpath.h>
#include <spoolp.h>

#include <spoolf.t>

class spool_puffert:public spoolft<zcmessaget>
 {
 public:
  filet outfile_mail, outfile_news;
  bool mail, news;

  void checkmailnews();
  void copymessage(FILE *outfile, bool mail);
  virtual void spoolf3();

  const char *fromsystem;

  virtual ~spool_puffert() { }
 };

void spool_puffert::checkmailnews()
 {
  const char *tptr, *inhalt;
  char tempstr[1000];

  news=mail=FALSE;

  tptr=m.data;
  while(!m.findheader(&tptr, "EMP", &inhalt))
   {
    m.copyheadertext(tempstr, inhalt, 999);

    if(isnewsemp(tempstr))
      news=TRUE;
    else
      mail=TRUE;
   }
 }

void spool_puffert::copymessage(FILE *outfile, bool mail)
 {
  const char *d=m.data, *kennung, *inhalt;
  char tempstr[200];

  while(TRUE)
   {
    if(m.parseline(&d, &kennung, &inhalt)) return;

    if(kennung==NULL)
     {
      fprintf(outfile, "\r\n");
      m.copybody(outfile);
      return;
     }

    m.copyheaderid(tempstr, kennung, 199);
    if(stricmp(tempstr, "EMP")==0)
     {
      m.copyheadertext(tempstr, inhalt, 199);
      
      if(isnewsemp(tempstr)==!mail)
        fwrite(kennung, 1, d-kennung, outfile);
      else
       {
        fprintf(outfile, "KOP:\t");
        fwrite(inhalt, 1, d-inhalt, outfile);
       }
     }
    else
      fwrite(kennung, 1, d-kennung, outfile);
   }
 }

void spool_puffert::spoolf3()
 {
  checkmailnews();

  if(mail) copymessage(outfile_mail.file, TRUE);
  if(news) copymessage(outfile_news.file, FALSE);  
 }

void spool_zc_puffer(const char *filename, const char *fromsystem)
 {
  spool_puffert sp;
  char filename_mail[DIRLEN], filename_news[DIRLEN];
  dword filelen;

  filelen=filelength(filename);

  if(filelen<10)
   {
    if(filelen!=0)
      logf.printf("spool_puffer", "\"%s\" too small", filename);

    remove(filename);
    return;
   }

  snprintf(filename_mail, DIRLEN, DBOXTEMPDIR "prearc.%d.zc.mail", config.dboxdir, getpid());
  snprintf(filename_news, DIRLEN, DBOXTEMPDIR "prearc.%d.zc.news", config.dboxdir, getpid());

  if(sp.outfile_mail.open(filename_mail, "wb") ||
     sp.outfile_news.open(filename_news, "wb"))
   {
    logf.printf("spool_puffer", "Failed to open outfile");
    return;
   }

  sp.fromsystem=fromsystem;

  if(sp.spoolf(filename))
    movetobad(filename, config.boxdir);

  sp.outfile_mail.close();
  sp.outfile_news.close();

  spool_zc_puffer_mail(filename_mail, fromsystem);
  spool_zc_puffer_news(filename_news, fromsystem);
 }

static void spool_zc_puffer(const char *filename,
                            const char *fromsystem,
                            bool mail)
 {
  char newfilename[DIRLEN];
  int i=0;
  dword filelen;

  filelen=filelength(filename);

  if(filelen<10)
   {
    if(filelen!=0) 
      logf.printf("spool_puffer", "\"%s\" too small", filename);

    remove(filename);
    return;
   }

  do
   {
    if(mail)
      snprintf(newfilename, DIRLEN, 
        BOXSPOOLINZCMAILDIR "%s.%d", config.boxdir,
        fromsystem, i++);
    else
      snprintf(newfilename, DIRLEN,
        BOXSPOOLINZCNEWSDIR "%s.%d", config.boxdir,
        fromsystem, i++);
   }
  while(access(newfilename, 00)==0);

  if(rename(filename, newfilename)!=0)
   {
    logf.printf("spool_puffer", "Fatal: rename(\"%s\", \"%s\"); failed",
      filename, newfilename);
   }
 }

void spool_zc_puffer_mail(const char *filename, const char *fromsystem)
 {
  spool_zc_puffer(filename, fromsystem, TRUE);
 }

void spool_zc_puffer_news(const char *filename, const char *fromsystem)
 {
  spool_zc_puffer(filename, fromsystem, FALSE);
 }

void spool_erfc(const string &filename, const string &fromsystem)
 {
  char newfilename[DIRLEN];
  int i=0;
  dword filelen;

  filelen=filelength(filename.c_str());

  if(filelen<10)
   {
    if(filelen!=0) 
      logf.printf("spool_erfc", "\"%s\" too small", filename.c_str());

    remove(filename.c_str());
    return;
   }

  do
   {
    snprintf(newfilename, DIRLEN, 
      BOXSPOOLINRFCMAILDIR "%s.%d", config.boxdir,
      fromsystem.c_str(), i++);
   }
  while(access(newfilename, 00)==0);

  if(rename(filename.c_str(), newfilename)!=0)
   {
    logf.printf("spool_erfc", "Fatal: rename(\"%s\", \"%s\"); failed",
      filename.c_str(), newfilename);
   }
 }
