#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <time.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>

#include <algorithm>

#include <general.h>
#include <config.h>
#include <dboxpath.h>
#include <oldtools.h>
#include <logf.h>
#include <iflags.h>
#include <tools.h>
#include <userdb.h>

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

void userpt::setfilename()
 {
  snprintf(path, DIRLEN, BOXUSERDIR, pconfig->boxdir, name);
  strmaxcpy(locktype, "userprofile", TEXTLEN-1);
  snprintf(filename, DIRLEN, "%sprofile", path);
 }
 
bool userpt::add()
 {
  char *tptr, tempstr[DIRLEN];

  setfilename();

  strmaxcpy(tempstr, path, DIRLEN-1);
  tptr=strrchr(tempstr, DIRCHAR[0]);
  if(tptr!=NULL) *tptr=0;

  if(mkdir(tempstr, 0700)!=0) return TRUE;

  write();

  snprintf(tempstr, DIRLEN, "%smail", path);
  if(mkdir(tempstr, 0700)!=0) return TRUE;
  return FALSE;
 }

bool userpt::del()
 {
  char tempstr[DIRLEN], *tptr;

  snprintf(tempstr, DIRLEN, BOXUSERDIR, pconfig->boxdir, "");
  tptr=strrchr(tempstr, DIRCHAR[0]);
  if(tptr!=NULL) *tptr=0;

  if(renamedirectory(tempstr, name)) return TRUE;

  if(config.homepagedir[0]!=0)
    renamedirectory(config.homepagedir, name);

  return FALSE;
 }

bool usercharisok(char ch)
 {
  if(!isascii(ch)) return FALSE;
  if(isdigit(ch) || isalpha(ch) || ch=='.' || ch=='-' || ch=='_')
    return TRUE;
  return FALSE;
 }

bool checkusername(const char *username)
 { 
  const char *tptr;

  tptr=username; 
 
  while(*tptr!=0)
   {
    if(!usercharisok(*tptr)) return TRUE;
    tptr++;
   } 

  if(username[0]==0 || username[0]=='.' ||
     username[0]=='_' || isdigit(username[0])) return TRUE;

  return FALSE;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

void grouppt::setfilename()
 {
  snprintf(path, DIRLEN, BOXGRUPPENDIR, pconfig->boxdir, name);
  strmaxcpy(locktype, "groupprofile", TEXTLEN-1);
  snprintf(filename, DIRLEN, "%sprofile", path);
 }
 
bool grouppt::add()
 {
  return TRUE;
 }

bool grouppt::del()
 {
  return TRUE;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

bool getpermissions(const char *permissionline, char *empmask,
  bool *read, bool *write, bool *sysop, userflagst *userflags)
 {
  char *tptr;
  int i, flag;
  stringlistt list;

  clear(userflags);

  *read=FALSE;
  *write=FALSE;
  *sysop=FALSE;
  empmask[0]=0;

  list.parsestring(permissionline);

  if(list.strings[0]==NULL || list.strings[1]==NULL) return TRUE;

  strmaxcpy(empmask, list.strings[0], TEXTLEN-1);

  tptr=list.strings[1];

  while(*tptr!=0)
   {
    switch(upcase(*tptr))
     {
     case 'R':
      *read=TRUE;
      break;

     case 'W':
      *write=TRUE;
      break;

     case 'S':
      *sysop=TRUE;
      break;

     case '-':
      break;

     default:
      return TRUE;
     }

    tptr++;
   }

  i=2;

  while(list.strings[i]!=NULL)
   {
    if(list.strings[i][0]=='!')
     {
      flag=config.userflagdb.index(list.strings[i]+1);
      if(flag==-1) return TRUE;
      (*userflags)[flag]=2;
     }
    else
     {
      flag=config.userflagdb.index(list.strings[i]);
      if(flag==-1) return TRUE;
      (*userflags)[flag]=1;
     }

    i++;
   }

  return FALSE;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

bool getinternalflag(userpt *up, grouppt *gp, int flag)
 {
  char flagname[TEXTLEN];

  if(flag<0) return FALSE;

  snprintf(flagname, TEXTLEN, "internalflag.%s", 
    up->pconfig->internalflagdb[flag]);

  if(atoi(*up|flagname)) return TRUE;
  if(atoi(*gp|flagname)) return TRUE;

  return FALSE;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

bool getuserflag(userpt *up, grouppt *gp, int flag)
 {
  char flagname[TEXTLEN];

  if(flag<0) return FALSE;

  snprintf(flagname, TEXTLEN, "userflag.%s", 
    up->pconfig->userflagdb[flag]);

  if(atoi(*up|flagname)) return TRUE;
  if(atoi(*gp|flagname)) return TRUE;

  return FALSE;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

bool getuserflag(userpt *up, grouppt *gp, const char *flag)
 {
  char flagname[TEXTLEN];

  snprintf(flagname, TEXTLEN, "userflag.%s", flag);

  if(atoi(*up|flagname)) return TRUE;
  if(atoi(*gp|flagname)) return TRUE;

  return FALSE;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

bool checkuserflags(userpt *up, grouppt *gp, userflagst *userflags)
 {
  int i;
  bool hasflag;

  for(i=0; i<MAXUSERFLAGS; i++)
   {
    switch((*userflags)[i])
     {
     case 0:
      break;

     case 1:
      hasflag=getuserflag(up, gp, i);
      if(!hasflag) return TRUE;
      break;

     case 2:
      hasflag=getuserflag(up, gp, i);
      if(hasflag) return TRUE;
      break;
     }
   }

  return FALSE;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

void getuser(userpt *up, grouppt *gp, const char *id, char *text)
 {
  up->get(id, text);
  if(text[0]!=0) return;

  gp->get(id, text);
  if(text[0]!=0) return;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

bool verifylogin(userpt *up, const char *pw)
 {
  char temppw[TEXTLEN];
  const char *cryptedpassword;

  cryptedpassword=*up|"cryptedpassword";
  if(cryptedpassword[0]==0)
    cryptedpassword=*up|"cryptedpasswort";

  if(cryptedpassword[0]==0 ||
     strcmp(cryptedpassword, "locked")==0)
    return TRUE;

  if(strcmp(cryptedpassword, "none")==0 && pw[0]==0)
    return FALSE;

  if((*up|"alias")[0]!=0)
    if(!atoi(*up|"account"))
      return TRUE;

  uncryptpassword(cryptedpassword, temppw);

  if(stricmp(temppw, pw)!=0) return TRUE;

  return FALSE;
 }

void cryptpassword(const char *source, char *dest)
 {
  char temppw[TEXTLEN*2];
  int i, len;

  if(source[0]==0)
   {
    strmaxcpy(dest, "none", TEXTLEN-1);
    return;
   }

  temppw[0]='A';
  
  len=strlen(source);
  for(i=0; i<len; i++)
   {
    temppw[(i*2)+1]='A'+((source[i]>>4)&0x0f);
    temppw[(i*2)+2]='A'+(source[i]&0x0f);
   }

  temppw[(i*2)+1]=0;

  strmaxcpy(dest, temppw, TEXTLEN-1);
 }

void uncryptpassword(const char *source, char *dest)
 {
  char temppw[TEXTLEN*2];
  int i, len;

  switch(source[0])
   {
   case 'A':
    len=strlen(source);
    for(i=0; i<((len-1)/2); i++)
      temppw[i]=((source[(i*2)+1]-'A')<<4)|
                (source[(i*2)+2]-'A');

    temppw[i]=0;
    strmaxcpy(dest, temppw, TEXTLEN-1);
    break;

   default:
    dest[0]=0;
   } 
 }

void getplaintextpassword(pt *up, char *pw)
 {
  const char *cryptedpassword;

  cryptedpassword=(*up)|"cryptedpassword";
  if(cryptedpassword[0]==0) cryptedpassword=(*up)|"cryptedpasswort";

  uncryptpassword(cryptedpassword, pw);
 }

// **********************************************************************
// *                                                                    *
// * Returns a sorted user list                                         *
// *                                                                    *
// **********************************************************************

void getuserlist(vector<string> &user, const string &mask)
 {
  string directory;
  DIR *dir;
  dirent *ent;

  getboxuserdir(directory, config.boxdir, ".");
  directory+=".";  
  if((dir=opendir(directory.c_str()))!=NULL)
   {
    while((ent=readdir(dir))!=NULL)
      if(ent->d_name[0]!='.' && 
         ent->d_name[0]!='_' &&
         checkjoker(mask.c_str(), ent->d_name, TRUE))
       {
        dnstring(ent->d_name);
        user.push_back(ent->d_name);
       }

    closedir(dir);
   }

  sort(user.begin(), user.end());
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

bool get_flag(userpt &up, userpt &up2, grouppt &gp, const char *flagname)
 {
  char data[TEXTLEN];

  up.get(flagname, data);
  if(data[0]!=0) return atoi(data);

  up2.get(flagname, data);
  if(data[0]!=0) return atoi(data);
  
  gp.get(flagname, data);
  if(data[0]!=0) return atoi(data);
  
  return FALSE;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

bool get_internalflag(userpt &up, userpt &up2, grouppt &gp, int flag)
 {
  char flagname[TEXTLEN];

  if(flag<0) return FALSE;

  snprintf(flagname, TEXTLEN, "internalflag.%s", 
    up.pconfig->internalflagdb[flag]);

  return get_flag(up, up2, gp, flagname);
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

bool get_userflag(userpt &up, userpt &up2, grouppt &gp, const char *flag)
 {
  char flagname[TEXTLEN];

  snprintf(flagname, TEXTLEN, "userflag.%s", flag);

  return get_flag(up, up2, gp, flagname);
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

void get_profiles(userpt &up, userpt &up2, grouppt &gp)
 {
  if((up|"alias")[0]!=0)
   {
    up2.setname(up|"alias");
    if(up2.read())
      up2.copy_from(&up);
   }
  else
    up2.copy_from(&up);

  gp.setname(up2|"gruppe");
  gp.read();
 }
