#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <fcntl.h>
#include <signal.h>

#include <strstream.h>

#include <dbox.h>
#include <folder.h>
#include <uuencode_str.h>
#include <mtoolzc.h>
#include <rfc1123date.h>
#include <zconnect.h>
#include <mtoolrfc.h>
#include <charsettools.h>
#include <stringtools.h>
#include <libdboxversion.h>
#include <logf.h>

class zc2rfct
 {
 public:
  zcmessaget *zcm;
  ostream *rfcm;
  string envelope_to, content_type;
  bool convert_charset, allow_zconnect_charset;

  void get_envelope_to();

  bool zc2rfc();
  bool zcbody2rfc();
  void converttext();
  void get_boundary(char *boundary);
  void get_content_type();
 };

static void strip_realname(char *address)
 {
  char *tptr=strchr(address, ' ');
  if(tptr!=NULL) *tptr=0;
 }

void zcdatetouucp(const char *zcdate, char *uucpdate)
 {
  #if 0
  int i, z1, z2;
  char tempstr[20];
  const char *tptr;
  char *tptr2;

  strncpy(uucpdate, zcdate+6, 2);
  uucpdate[2]=' ';

  strncpy(tempstr, zcdate+4, 2);
  tempstr[2]=0;
  i=atoi(tempstr);
  if(i!=0) i--;
  if(i>11) i=11;
  strncpy(uucpdate+3, rfc_months[i], 3);
  uucpdate[6]=' ';

  strncpy(uucpdate+7, zcdate, 4);
  uucpdate[11]=' ';

  strncpy(uucpdate+12, zcdate+8, 2);
  uucpdate[14]=':';

  strncpy(uucpdate+15, zcdate+10, 2);
  uucpdate[17]=':';

  strncpy(uucpdate+18, zcdate+12, 2);
  uucpdate[20]=' ';
  uucpdate[21]='+';

  tptr=zcdate+15;
  z1=z2=0;
  if(*tptr!=0)
   {
    uucpdate[21]=*tptr;
    tptr++;
    tptr2=strchr(tptr, ':');
    if(tptr2!=NULL) *tptr2=0;
    z1=atoi(tptr);
    if(tptr2!=NULL) z2=atoi(tptr2+1);
   }
  sprintf(uucpdate+22, "%02u%02u", z1, z2);
  #else

  time_t gm_time;
  long diff=0;

  if(parse_eda(zcdate, gm_time, diff))
    time(&gm_time);

  strmaxcpy(uucpdate, make_date(gm_time, diff).c_str(), TEXTLEN-1);   

  #endif
 }

void zcabstouucp(const char *abs, char *from)
 {
  char *tptr;

  ibm2ascii(abs, from);

  tptr=strchr(from, ' ');
  if(tptr!=NULL) *tptr=0;
  dnstring(from);
  if(tptr!=NULL) *tptr=' ';
 }

string iso2mime(const string &text)
 {
  string result;
  bool encode=FALSE;

  for(unsigned i=0; i<text.size(); i++)
   {
    char ch=text[i];

    if(!isascii(ch) && !encode)
     {
      result+="=?iso-8859-1?Q?";
      encode=TRUE;
     }
    else if(encode &&
            (ch==')' || ch==' ' || ch==':' || ch==',' ||
             ch=='(' || ch=='!' || ch=='.' || ch=='"'))
     {
      result+="?=";
      encode=FALSE;
     }

    if(encode && !isascii(ch))
     {
      char x[4];
      snprintf(x, 3, "%02X", (unsigned char)ch);
      result+='=';
      result+=x;
     }
    else
      result+=ch;
   }

  if(encode)
    result+="?=";

  return result;
 }

string zc2mime(const string &text)
 {
  string text2=text;
  ibm2iso(text2);
  return iso2mime(text2);
 }

void zc2rfct::get_envelope_to()
 {
  char tempstr[1000];

  if(!zcm->findsingleheader(tempstr,
      "X-" X_DBOX "-Envelope-To",
      1000-1))
   {
    strip_realname(tempstr);
    envelope_to=tempstr;
   }
  else
    envelope_to="";
 }

void zc2rfct::converttext()
 {
  for(const char *ptr=zcm->data+zcm->headerlen; ptr<zcm->endofmail; ptr++)
   {
    char data=*ptr;

    if(convert_charset)
      ibm2iso(&data, 1);

    (*rfcm) << data;
   }
 }

void zc2rfct::get_boundary(char *boundary)
 {
  string mid;

  if(zcm->mid==NULL)
    mid="";
  else
    zcm->copyheadertext(mid, zcm->mid);

  snprintf(boundary, TEXTLEN-1, "==========%s", mid.c_str());
 }

void zc2rfct::get_content_type()
 {
  char charset[TEXTLEN];

  convert_charset=FALSE;
  content_type="";

  if(!zcm->findsingleheader(content_type, "U-Content-Type"))
   {
    //if(strnicmp(content_type.c_str(), "multipart/", strlen("multipart/"))==0 ||
    //   strnicmp(content_type.c_str(), "application/", strlen("application/"))==0)
    return;
   }

  if(!zcm->findsingleheader(charset, "CHARSET", TEXTLEN-1))
   {
    if(strnicmp(charset, "ISO", 3)==0)
     {
      content_type="text/plain; charset=ISO-8859-";
      content_type+=charset[3];
      return;
     }
   }

  if(allow_zconnect_charset)
    content_type="text/plain; charset=X-IBM";
  else
   {
    content_type="text/plain; charset=ISO-8859-1";
    convert_charset=TRUE;
   }
 }

bool zc2rfct::zc2rfc()
 {
  dword kom=0;
  const char *start, *_id, *_text;
  char id[1000], text[1000], tempstr[2000];
  string mime_version;
  char multipart_boundary[TEXTLEN];
  string content_transfer_encoding;
  bool antwort_an=FALSE, bez=FALSE, to=FALSE;
  bool binary=FALSE;

  get_boundary(multipart_boundary); 
  get_envelope_to();
  get_content_type();

  (*rfcm) << "X-Gateway: " << "ZCONNECT UU "
          << config.boxname << "." << config.domain
          << " [DBOX " << LIBDBOXVERSIONNR << "]";

   {
    string gate;
    zcm->findsingleheader(gate, "GATE");
    if(gate!="") *(rfcm) << ", " << gate;
    *(rfcm) << "\n";
   }

  if(envelope_to!="")
    (*rfcm) << "X-POP3-Rcpt: " << envelope_to << "\r\n";

  start=zcm->data;

  while(!zcm->parseline(&start, &_id, &_text) && _id!=NULL)
   {
    zcm->copyheaderid(id, _id, 999);
    zcm->copyheadertext(text, _text, 999);

    // EB

    if(stricmp(id, "ABS")==0)
     {
      (*rfcm) << "From: " << zc2mime(text) << "\r\n";
     }
    else if(stricmp(id, "OEM")==0 ||
            stricmp(id, "EMP")==0 ||
            stricmp(id, "OAB")==0 ||
            stricmp(id, "KOP")==0)
     {
      (*rfcm) << "X-DBOX-" << id << ": " << zc2mime(text) << "\r\n";
     }
    else if(stricmp(id, "BET")==0)
     {
      (*rfcm) << "Subject: " << zc2mime(text) << "\r\n";
     }
    else if(stricmp(id, "KOM")==0)
      kom=atol(text);
    else if(stricmp(id, "MAILER")==0)
     {
      ibm2ascii(text, tempstr, TRUE);
      (*rfcm) << "X-Mailer: " << tempstr << "\r\n";
     }
    else if(stricmp(id, "MID")==0)
     {
      ibm2ascii(text, tempstr, TRUE);
      (*rfcm) << "Message-Id: <" << tempstr << ">\r\n";
     }
    else if(stricmp(id, "ORG")==0)
     {
      ibm2ascii(text, tempstr, TRUE);
      (*rfcm) << "Organization: " << tempstr << "\r\n";
     }
    else if(stricmp(id, "EDA")==0)
     {
      string date_header;

      (*rfcm) << "Date: ";

      if(!zcm->findsingleheader(date_header, "U-Date"))
        (*rfcm) << date_header;
      else
       {
        text[TEXTLEN-1]=0;
        zcdatetouucp(text, tempstr);
        (*rfcm) << tempstr;
       }

      (*rfcm) << "\r\n";
     }
    else if(stricmp(id, "ANTWORT-AN")==0 && !antwort_an)
     {
      antwort_an=TRUE;
      (*rfcm) << "Reply-To: " << zc2mime(text) << "\r\n";
     }
    else if(stricmp(id, "BEZ")==0 && !bez) // TODO: mehrere
     {
      bez=TRUE;
      ibm2ascii(text, tempstr, TRUE);
      (*rfcm) << "In-Reply-To: <" << tempstr << ">\r\n";
     }
    else if(stricmp(id, "TYP")==0)
     {
      if(stricmp(text, "MIME")!=0)
        binary=TRUE;
     }
    else if(strnicmp(id, "U-", 2)==0)
     {
      if(stricmp(id+2, "lines")!=0 &&
         stricmp(id+2, "date")!=0)
       {
        if(stricmp(id+2, "MIME-Version")==0)
          mime_version=text;
        else if(stricmp(id+2, "Content-Transfer-Encoding")==0)
          content_transfer_encoding=text;
        else if(stricmp(id+2, "Content-Type")==0)
         { // skip
         }
        else
         {
          ibm2ascii(text, tempstr, TRUE);
          (*rfcm) << id+2 << ": " << tempstr << "\r\n";

          if(stricmp(id+2, "To")==0)
            to=TRUE;
         }
       }
     }
   }

  if(!to)
    if(envelope_to!="")
      (*rfcm) << "To: " << envelope_to << "\r\n";
    else
     {
      (*rfcm) << "To: ";
      const char *start=zcm->data, *text;
      bool first=TRUE;
  
      while(!zcm->findheader(&start, "EMP", &text))
       {
        if(first) first=FALSE; else (*rfcm) << ",\n\t";
        string envelope_to;
        zcm->copyheadertext(envelope_to, text);
        (*rfcm) << zc2mime(envelope_to);
       }

      (*rfcm) << "\n";
     }

  if(mime_version=="" && 
     content_type[0]==0 &&
     content_transfer_encoding=="")
    content_transfer_encoding="8bit";

  if(binary)
   {
    (*rfcm) << "MIME-Version: 1.0\r\n"
            "Content-Type: multipart/mixed; boundary=\"" <<
            multipart_boundary << "\r\n"
            "Content-Transfer-Encoding: 8bit\r\n";
   }
  else
   {
    if(mime_version=="") mime_version="1.0";

    (*rfcm) << "MIME-Version: " << mime_version << "\r\n"
            "Content-Type: " << content_type << "\r\n";

    if(content_transfer_encoding!="")
      (*rfcm) << "Content-Transfer-Encoding: " << 
              content_transfer_encoding << "\r\n";
   }

  (*rfcm) << "\r\n";  // end of header

  return FALSE;
 }

bool zc2rfct::zcbody2rfc()
 {
  char typ[TEXTLEN], file[TEXTLEN]="", fileheader[TEXTLEN*2]="";
  char multipart_boundary[TEXTLEN];

  get_content_type();

  if(!zcm->findsingleheader(typ, "TYP", TEXTLEN-1))
   {
    const char *uufilename;
    istrstream zcbodystream(zcm->data+zcm->headerlen, zcm->len);

    get_boundary(multipart_boundary);

    zcm->findsingleheader(file, "FILE", TEXTLEN-1);
    ibm2ascii(file, fileheader, TRUE);

    if(fileheader[0]!=0)
      uufilename=fileheader;
    else
      uufilename="binaer.zer";

    (*rfcm) << "This is a multipart MIME message.\r\n"
            "--" << multipart_boundary << "\r\n"
            "Content-Type: " << content_type << "\r\n"
            "Content-Transfer-Encoding: 8bit\r\n"
            "\r\n";

    (*rfcm) << "File " << uufilename << " is attached\r\n";

    (*rfcm) << "\r\n"
            "--" << multipart_boundary << "\r\n"
            "Content-Type: application/octet-stream; name=\"" <<
            uufilename << "\"\r\n"
            "Content-Transfer-Encoding: x-uuencode\r\n"
            "Content-Disposition: attachment; filename=\"" <<
            uufilename << "\"\r\n"
            "\r\n";

    uuencode(zcbodystream, (*rfcm), zcm->len, uufilename, FALSE);

    (*rfcm) << "--" << multipart_boundary << "--\r\n";
   }
  else
    converttext();

  return FALSE;
 }

bool zc2rfc(zcmessaget &zcm, ostream &rfcm,
            bool headeronly, bool allow_zconnect_charset)
 {
  zc2rfct zc2rfc;

  zc2rfc.allow_zconnect_charset=allow_zconnect_charset;
  zc2rfc.zcm=&zcm;
  zc2rfc.rfcm=&rfcm;
  if(zc2rfc.zc2rfc()) return TRUE;

  if(!headeronly)
    zc2rfc.zcbody2rfc();

  return FALSE;
 }

bool zc2rfc(zcmessaget &zcm, rfcmessaget &rfcm,
            bool headeronly, bool allow_zconnect_charset)
 {
  zc2rfct zc2rfc;
  ostrstream rfcm_str;

  zc2rfc.allow_zconnect_charset=allow_zconnect_charset;
  zc2rfc.zcm=&zcm;
  zc2rfc.rfcm=&rfcm_str;
  if(zc2rfc.zc2rfc()) return TRUE;
  rfcm.headerlen=zc2rfc.rfcm->tellp();

  if(!headeronly)
    zc2rfc.zcbody2rfc();

  rfcm.data=rfcm_str.str();
  if(rfcm.data==NULL) logf.internalerror("zc2rfc", ".str() failed");
  rfcm.len=zc2rfc.rfcm->tellp()-rfcm.headerlen;
  rfcm.endofmail=rfcm.endoffile=rfcm.data+rfcm.len+rfcm.headerlen;

  return FALSE;
 }

bool zcbody2rfc(zcmessaget &zcm, rfcmessaget &rfcm,
                bool allow_zconnect_charset)
 {
  zc2rfct zc2rfc;
  char *body_str;
  ostrstream rfcm_str;

  zc2rfc.allow_zconnect_charset=allow_zconnect_charset;
  zc2rfc.zcm=&zcm;
  zc2rfc.rfcm=&rfcm_str;
  if(zc2rfc.zcbody2rfc()) return TRUE;

  body_str=rfcm_str.str();
  rfcm.len=strlen(body_str);

  rfcm.data=(const char *)realloc((void *)rfcm.data, rfcm.headerlen+rfcm.len);
  if(rfcm.data==(char *)NULL)
    logf.internalerror("zcbody2rfc", "out of memory");

  memcpy((char *)rfcm.data+rfcm.headerlen, body_str, rfcm.len);

  free(body_str);

  rfcm.endofmail=rfcm.endoffile=rfcm.data+rfcm.len+rfcm.headerlen;

  return FALSE;
 }

