/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: generic ZCONNECT(R) functions

*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <stdlib.h>

#include "dbox.h"
#include "zconnect.h"
#include "config.h"

byte readpufferline(FILE *f, char *kennung, char *text, word maxlen)
 {
  char tempstr[2000], *tptr;

  kennung[0]=0;
  if(fgets(tempstr, 1999, f)==NULL) return 1;
  tptr=tempstr+strlen(tempstr);
  if(tptr==tempstr) return 0;
  tptr--;
  while(tptr!=tempstr && (*tptr=='\n' || *tptr=='\r')) { *tptr=0; tptr--; }
  if(tptr==tempstr) return 0;
  tptr=strchr(tempstr, ':');
  if(tptr==NULL) return 2;
  *tptr=0;
  strmaxcpy(kennung, tempstr, TEXTLEN-1);
  do tptr++; while(*tptr==' ' || *tptr==9);
  strmaxcpy(text, tptr, maxlen-1);
  return 0;
 }

byte readkennung(FILE *f, char *kennung)
 {
  char data;
  word len;

  len=0;
  while(TRUE)
    switch(data=fgetc(f))
     {
     case -1:
      return len==0?1:2;

     case ':':
      if(len==0) return 2;
      *kennung=0;
      return 0;

     case '\r':
      if(len==0 && fgetc(f)=='\n')
       {
        *kennung=0;
        return 0;
       }
      return 2;

     default:
      /*
      if(data=='-' ||
         (data>='0' && data<='9') ||
         (data>='A' && data<='Z') ||
         (data>='a' && data<='z'))
      */
      if(data>' ' && data<127)
       {
        *(kennung++)=data;
        if((++len)>100) return 2;
       }
      else
        return 2;
     }
 }

byte skipinhalt(FILE *f)
 {
  char data;
  do if(fread(&data, 1, 1, f)==0) return 2; while(data!='\r');
  return fgetc(f)=='\n'?0:2;
 }

byte readinhalt(FILE *f, char *inhalt, word maxlen)
 {
  char data;
  word len;

  len=0;
  while(TRUE)
   {
    if(fread(&data, 1, 1, f)==0) return 2;
    switch(data)
     {
     case '\r':
      if(fgetc(f)=='\n')
       {
        *inhalt=0;
        return 0;
       }
      return 2;

     case ' ':
     case '\t':
      if(len==0) break;

     default:
      if(len<maxlen)
       {
        *(inhalt++)=data;
        len++;
       }
     }
   }
 }

byte copyinhalt(FILE *f, FILE *outfile)
 {
  char data;
  do
   {
    if(fread(&data, 1, 1, f)==0)
      return 2;

    fputc(data, outfile);
   }
  while(data!='\r');
  fputc('\n', outfile);
  return fgetc(f)=='\n'?0:2;
 }

time_t parseeda(const char *eda)
 {
  time_t t;
  tm ts;
  char jahr[5], mon[3], tag[3], std[3], min[3], sek[3];
  bool sommer;
  int diff;

  strncpy(jahr, eda, 4); jahr[4]=0;
  strncpy(mon, eda+4, 2); mon[2]=0;
  strncpy(tag, eda+6, 2); tag[2]=0;
  strncpy(std, eda+8, 2); std[2]=0;
  strncpy(min, eda+10, 2); min[2]=0;
  strncpy(sek, eda+12, 2); sek[2]=0;
  sommer=eda[14]=='S';
  diff=atoi(eda+15);

  ts.tm_year=atoi(jahr)-1900;
  ts.tm_mon=atoi(mon)-1;
  ts.tm_mday=atoi(tag);
  ts.tm_hour=atoi(std)+diff;
  ts.tm_min=atoi(min);
  ts.tm_sec=atoi(sek);

  t=mktime(&ts);

  return t;
 }

bool parse_eda(const char *eda, time_t &gm_time, long &diff)
 {
  tm ts;
  char jahr[5], mon[3], tag[3], std[3], min[3], sek[3];
  bool summer;

  gm_time=0;
  diff=0;

  if(strlen(eda)<15) return TRUE;

  strncpy(jahr, eda, 4); jahr[4]=0;
  strncpy(mon, eda+4, 2); mon[2]=0;
  strncpy(tag, eda+6, 2); tag[2]=0;
  strncpy(std, eda+8, 2); std[2]=0;
  strncpy(min, eda+10, 2); min[2]=0;
  strncpy(sek, eda+12, 2); sek[2]=0;

  summer=eda[14]=='S';
  diff=atoi(eda+15)*3600;
  const char *tptr=eda+15;
  while(*tptr!=0 && *tptr!=':') tptr++;
  if(*tptr==':')
   {
    long minutes=atoi(tptr+1)*60;
    if(diff<0)
      diff-=minutes;
    else
      diff+=minutes;
   }

  ts.tm_year=atoi(jahr)-1900;
  ts.tm_mon=atoi(mon)-1;
  ts.tm_mday=atoi(tag);
  ts.tm_hour=atoi(std);
  ts.tm_min=atoi(min);
  ts.tm_sec=atoi(sek);

  gm_time=mktime(&ts);
  if(gm_time==(time_t)-1) return TRUE;

  gm_time-=timezone;
  if(ts.tm_isdst) gm_time+=3600;

  return FALSE;
 }

void makeeda(time_t t, char *eda)
 {
  tm *s;

  if(config.zeitzoneplus)
   {
    t-=config.zeitzonestd*60*60;
    t-=config.zeitzonemin*60;
   }
  else
   {
    t-=config.zeitzonestd*60*60;
    t-=config.zeitzonemin*60;
   }

  s=gmtime(&t);

  snprintf(eda, TEXTLEN, "%04u%02u%02u%02u%02u%02u%c%c%u",
    s->tm_year+1900, s->tm_mon+1, s->tm_mday,
    s->tm_hour, s->tm_min, s->tm_sec,
    config.sommerzeit?'S':'W',
    config.zeitzoneplus?'+':'-',
    config.zeitzonestd);

  if(config.zeitzonemin!=0)
    sprintf(eda+strlen(eda), ":%02u", config.zeitzonemin);
 }

string make_eda_localtime()
 {
  time_t t;
  tm s;

  time(&t);
  localtime(&t); // sets timezone and daylight
  s=*gmtime(&t);

  char eda[TEXTLEN];

  long diff=-timezone;
  if(daylight) diff+=3600;

  snprintf(eda, TEXTLEN, "%04u%02u%02u%02u%02u%02u%c%c%ld",
    s.tm_year+1900, s.tm_mon+1, s.tm_mday,
    s.tm_hour, s.tm_min, s.tm_sec,
    daylight?'S':'W',
    (diff>=0)?'+':'-',
    labs(diff)/3600);

  unsigned minutes=(labs(diff)/60)%60;

  if(minutes!=0)
    sprintf(eda+strlen(eda), ":%02u", minutes);

  return eda;
 }

// **********************************************************************
// *                                                                    *
// *                                                                    *
// *                                                                    *
// **********************************************************************

bool zcfoldercharisok(char ch)
 {
  if(!isascii(ch)) return FALSE;
  if(isdigit(ch) || isalpha(ch) || ch=='/' || ch=='_' || ch=='-')
    return TRUE;
  return FALSE;
 }

