/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>

#include <dboxpath.h>
#include <logf.h>
#include <spoolp.h>
#include <account.h>
#include <profile.h>

class dtauspt:public pt
 {
 public:
  virtual void setfilename();
  virtual bool add();
  virtual bool del();

  dtauspt():pt() { }
  dtauspt(const char *_name):pt(_name) { }
 };

void dtauspt::setfilename()
 {
  snprintf(path, DIRLEN, BOXSYSTEMDIR, pconfig->boxdir);
  strmaxcpy(locktype, "dtaus", TEXTLEN-1);
  snprintf(filename, DIRLEN, "%s%s", path, name);
 }
 
bool dtauspt::add()
 {
  return TRUE;
 }

bool dtauspt::del()
 {
  return TRUE;
 }

void filtertext(char *s)
 {
  bool use_ibm=TRUE;

  while(*s!=0)
   {
    if(isascii(*s) && isalnum(*s))
     {
      *s=upcase(*s);
     }
    else
     {
      switch((byte)upcase(*s))
       {
       case ' ':
       case '.':
       case ',':
       case '&':
       case '-':
       case '+':
       case '*':
       case '%':
       case '/':
       case '$':
        break;

       case '_':
        *s='-';
        break;

       case 0x8e:
        if(use_ibm)
          *s=0x8e;
        else
          *s=0x5b;
        break;

       case 0x99:
        if(use_ibm)
          *s=0x99;
        else
          *s=0x5c;
        break;

       case 0x9a:
        if(use_ibm)
          *s=0x9a;
        else
          *s=0x5d;
        break;

       case 0xe1:
        if(use_ibm)
          *s=0xe1;
        else
          *s=0x7e;
        break;

       default:
        *s='*';
        break;
       }
     }
    s++;
   }
 }

void formatnumber(char *s, byte len)
 {
  char tempstr[100];
  byte i;
  
  for(i=0; i<len; i++)
    tempstr[i]='0';

  tempstr[len+1]=0;

  strcpy(tempstr+len-strlen(s), s);

  strcpy(s, tempstr);
 }

void sumnumber(char *sum, const char *number)
 {
  int i;
  byte tempsum, carry=0;

  if(strlen(sum)!=strlen(number))
    logf.internalerror("dtausprofilet::sumnumber", "assertion failed");

  i=strlen(sum);
  while(i!=0)
   {
    i--;
    tempsum=(sum[i]-'0')+(number[i]-'0')+carry;
    sum[i]=(tempsum%10)+'0';
    carry=tempsum/10;
   }
 }

void filterdigits(char *s)
 {
  char tempstr[1000], *tptr, *olds=s;

  tptr=tempstr;
  while(*s!=0)
   {
    if(isdigit(*s)) *(tptr++)=*s;
    s++;
   }
  *tptr=0;
  strcpy(olds, tempstr);
 }

signed long getscale2(signed long amount, byte scale)
 {
  if(scale<2)
    amount*=(signed long)power10(2-scale);
  else if(scale>2)
    amount/=(signed long)power10(scale-2);

  return amount;
 }

void dolastschrift(byte format, const char *gegenbuchungsdatei,
                   bool rueckzahlung)
 {
  userpt up;
  char username[TEXTLEN];
  char *tptr;
  dtauspt dtaus("dtaus.ini");
  struct tm *tm;
  time_t t;
  long laufende_nummer=1;
  dword sum_betraege=0, anz_buchungen=0;
  char sum_kontonummern[18], sum_blz[18];
  FILE *gbf=NULL;
  bool lastschrift=!rueckzahlung;

  time(&t);
  tm=localtime(&t);

  if(gegenbuchungsdatei!=NULL && gegenbuchungsdatei[0]!=0)
   {
    gbf=fopen(gegenbuchungsdatei, "wt");
    if(gbf!=NULL)
     {
      fprintf(gbf, ";\n; %sschriften vom %d.%d.%d\n;\n",
        lastschrift?"Last":"Gut",
        tm->tm_mday, tm->tm_mon+1, tm->tm_year+1900);
     }
   }

  switch(format)
   {
   case 2:
    if(dtaus.read())
     {
      fprintf(stderr, "Fehler beim Lesen von %sdtaus.ini\n",
        dtaus.getpath());
      if(gbf!=NULL) fclose(gbf);
      return;
     }

    if(strlen(dtaus|"blz")!=8)
     {
      fprintf(stderr, "Fehler beim Lesen von dtaus.ini: "
                      "Bankleitzahl ungueltig!\n");
      if(gbf!=NULL) fclose(gbf);
      return;
     }

    if(strchr(dtaus|"kontonummer", ' ')!=NULL || 
       strlen(dtaus|"kontonummer")>10)
     {
      fprintf(stderr, "Fehler beim Lesen von dtaus.ini: "
                      "Kontonummer ungueltig.\n");
      if(gbf!=NULL) fclose(gbf);
      return;
     }

    if(strlen(dtaus|"kontoinhaber")>27)
     {
      fprintf(stderr, "Fehler beim Lesen von dtaus.ini: "
                      "Kontoinhaber zu lang.\n");
      if(gbf!=NULL) fclose(gbf);
      return;
     }

    if(strlen(dtaus|"verwendungszweck")>27)
     {
      fprintf(stderr, "Fehler beim Lesen von dtaus.ini: "
                      "Verwendungszweck zu lang.\n");
      if(gbf!=NULL) fclose(gbf);
      return;
     }

    filtertext(dtaus.get_ptr("kontoinhaber"));
    filtertext(dtaus.get_ptr("verwendungszweck"));
    filterdigits(dtaus.get_ptr("blz"));
    filterdigits(dtaus.get_ptr("kontonummer"));
    formatnumber(dtaus.get_ptr("blz"), 8);
    formatnumber(dtaus.get_ptr("kontonummer"), 10);

    printf("0128"); // Satzlaenge
    printf("A");    // Satzart

    if(lastschrift)
      printf("L");  // Lastschrift
    else
      printf("G");  // Gutschrift

    printf("K");    // Kundendiskette
    formatnumber(dtaus.get_ptr("blz"), 8); // BLZ der Bank
    printf("%-8s", dtaus|"blz");

    printf("%08ld", 0L); // Reserve

    printf("%-27s", dtaus|"kontoinhaber"); // Diskettenabsender

    printf("%02d%02d%02d", tm->tm_mday, tm->tm_mon+1, tm->tm_year%100); 
      // Diskettenerstellungsdatum

    printf("%-4s", ""); // Reserve

    printf("%-10s", dtaus|"kontonummer");
      // Empfaenger/Absenderkontonummer

    printf("%010ld", 0L); // Referenznummer des Einreichers

    printf("%-15s", ""); // Reserve

    printf("%06ld", 0L); // Reserve
    printf("%-27s", ""); // Reserve 
    
    sprintf(sum_kontonummern, "%017ld", 0L);
    sprintf(sum_blz, "%017ld", 0L);

    break;
   }

  while(fgets(username, 900, stdin)!=NULL)
    if(username[0]!=0 && username[0]!='.' && username[0]!='_')
     {
      tptr=strchr(username, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(username, '\n'); if(tptr!=NULL) *tptr=0;

      username[TEXTLEN-1]=0;
      tptr=strrchr(username, '/');
      if(tptr!=NULL) *tptr=0;
      strcpy(up.name, username);
      if(up.read())
        fprintf(stderr, "Not found: %s\n", username);
      else
       {
        bankpt bp(up.name);
        if(!bp.read())
         {
          amountt balance=rescale(getbalance(&up), 2);

          filterdigits(bp.get_ptr("kontonummer"));
          filterdigits(bp.get_ptr("blz"));

          if(
             (lastschrift &&
              atoi(bp|"einzugsermaechtigung") && 
              atoi(bp|"automatisch_abbuchen") &&
              (bp|"kontoinhaber")[0]!=0 && 
              (bp|"blz")[0]!=0 &&
              (bp|"bank")[0]!=0 &&
              (bp|"kontonummer")[0]!=0  &&
              (balance.amount<0 || 
               balance.amount<(signed long)atol(bp|"abbuchungs_betrag")/4)
             ) ||
             (!lastschrift &&
              (bp|"kontoinhaber")[0]!=0 && 
              (bp|"blz")[0]!=0 &&
              (bp|"bank")[0]!=0 &&
              (bp|"kontonummer")[0]!=0  &&
              balance.amount>0
             ))
           {
            signed long betrag;
            char tempstr[TEXTLEN];

            if(lastschrift)
             {
              if(atol(bp|"abbuchungs_betrag")==0)
               {
                if(balance.amount>=0)
                  betrag=0;
                else
                 {
                  betrag=labs(balance.amount);
                  if(betrag<500) betrag=0;
                 }
               }
              else
               {
                betrag=0;

                while((balance.amount+betrag)<
                      ((signed long)(atol(bp|"abbuchungs_betrag")/3)))
                  betrag+=atol(bp|"abbuchungs_betrag");
               }
             }
            else
              betrag=balance.amount;

            if(betrag>0)
             {
              sum_betraege+=betrag;
              anz_buchungen++;

              if(gbf!=NULL)
               {
                fprintf(gbf, "U:%s;B:%s%ld;A:2;T:ZAHLUNG.%s;Z:%s vom %d.%d.%d\n",
                  up.name, 
                  lastschrift?"":"-", betrag,
                  lastschrift?"LASTSCHRIFT":"RUECKUEBERWEISUNG",
                  lastschrift?"Lastschrift":"Zahlung",
                  tm->tm_mday, tm->tm_mon+1, tm->tm_year+1900);
               }

              switch(format)
               {
               case 0:
                currency_string(tempstr, getbalance(&up), TRUE);
                printf("Username...............: %s\n", up.name);
                printf("Kontostand.............: %s\n", tempstr);
                printf("Kontoinhaber...........: %s\n", bp|"kontoinhaber");
                printf("Kontonummer............: %s\n", bp|"kontonummer");
                printf("BLZ....................: %s\n", bp|"blz");
                printf("Bank...................: %s\n", bp|"bank");
                printf("Betrag.................: %ld,%02ld DM\n", betrag/100, betrag%100);
                printf("Typ....................: %s\n", lastschrift?"Lastschrift":"Zahlung");
                printf("Verwendungszweck.......: Userkonto %s\n", up.name);
                printf("\n");
                break;

               case 1:
                break;

               case 2:
                if(strlen(bp|"kontoinhaber")>27) 
                  fprintf(stderr, 
                          "Warnung: Kontoinhaber von User %s ist zu lang!\n",
                          up.name);

                formatnumber(bp.get_ptr("blz"), 8);
                formatnumber(bp.get_ptr("kontonummer"), 10);
                filtertext(bp.get_ptr("kontoinhaber"));
                bp.get_ptr("kontoinhaber")[27]=0;
                snprintf(tempstr, TEXTLEN, "%s", up.name);
                tempstr[27]=0;
                filtertext(tempstr);
 
                printf("0216");
                printf("C");
                printf("%08ld", 0L);       // Erstbeteiligtes Institut
                printf("%-8s", bp|"blz");  // BLZ endbeg. Inst.
                printf("%-10s", bp|"kontonummer"); // Empfaenger/Zahlungspfl.
                printf("%013ld", 0L);      // Reserve/Interne Nummer

                if(lastschrift)
                  printf("05000");         // Textschluessel
                else
                  printf("51000");         // Textschluessel
 
                printf(" ");               // Reserve
                printf("%011ld", betrag);  // Betrag DM
                printf("%-8s", dtaus|"blz"); // BLZ erstbeauftr. Inst.
                printf("%-10s", dtaus|"kontonummer"); // Auftraggeber/Zahlungsempf.
                printf("%011ld", laufende_nummer++); // Laufende Nummer / Betrag Euro
                printf("%-3s", "");        // Reserve
                printf("%-27s", bp|"kontoinhaber"); // Empfnger/Zahlungspfl.
                printf("%-8s", "");        // Reserve
                printf("%-27s", dtaus|"kontoinhaber"); // Auftraggeber/Zahlungsemfaenger
                printf("%-27s", tempstr);  // Verwendungszweck
                printf("%-3s", "");        // Reserve
                printf("01");              // Ein Erweiterungsteil
                printf("02");              // Verwendungszweck
                printf("%-27s", dtaus|"verwendungszweck");
                printf("00");              // Endekennung
                printf("%-27s", "");       // Daten
                printf("%-11s", "");       // Reserve

                strmaxcpy(tempstr, bp|"kontonummer", TEXTLEN-1);
                formatnumber(tempstr, 17);
                sumnumber(sum_kontonummern, tempstr);
                strcpy(tempstr, bp|"blz");
                formatnumber(tempstr, 17);
                sumnumber(sum_blz, tempstr);

                break;
               }
             }
           }
         }
       }
     }

  switch(format)
   {
   case 2:
    printf("0128");
    printf("E");
    printf("%-5s", "");
    printf("%07lu", anz_buchungen);
    printf("%013lu", sum_betraege);
    printf("%-17s", sum_kontonummern);
    printf("%-17s", sum_blz);
    printf("%013lu", 0L);
    printf("%-51s", "");
    break;
   }

  if(gbf!=NULL)
   {
    fprintf(gbf, ";\n"
                 "; Gesamtsumme: %lu,%02lu DM\n"
                 "; Anzahl Buchungen: %lu\n"
                 ";\n", 
      sum_betraege/100, sum_betraege%100, anz_buchungen);
    fclose(gbf);
   }
 }

