/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>

#include <logf.h>
#include <dboxpath.h>
#include <oldtools.h>
#include <zconnect.h>
#include <account.h>
#include <spoolp.h>
#include <messageid.h>
#include "dta.h"

#define MAXPOSTEN 100

bool akkumulieren_vgl(buchungt *posten, buchungt *buchung)
 {
  if(posten->bearbeiter[0]!=0)
    if(!checkjoker(posten->bearbeiter, buchung->bearbeiter))
      return TRUE;

  #if 0
  if(posten->verwendungszweck[0]!=0)
    if(!checkjoker(posten->verwendungszweck, buchung->verwendungszweck))
      return TRUE;
  #endif

  if(posten->buchungstyp[0]!=0)
    if(!checkjoker(posten->buchungstyp, buchung->buchungstyp))
      return TRUE;

  if(posten->mwstsatz!=0)
    if(posten->mwstsatz!=buchung->mwstsatz)
      return TRUE;

  if(posten->haben!=buchung->haben)
      return TRUE;

  return FALSE;
 }

void sum_buchung(buchungt *buchung1, buchungt *buchung2)
 {
  signed long summe;

  if(buchung1->amount.scale>buchung2->amount.scale)
    buchung2->amount=rescale(buchung2->amount, buchung1->amount.scale);
  else if(buchung2->amount.scale>buchung1->amount.scale)
    buchung1->amount=rescale(buchung1->amount, buchung2->amount.scale);

  summe=(buchung1->amount.amount*(buchung1->haben?1:-1))+
        (buchung2->amount.amount*(buchung2->haben?1:-1));

  buchung1->amount.amount=labs(summe);
  buchung1->haben=(summe>=0);
 }

bool akkumulieren(dword anzposten, buchungt *posten, buchungt *buchung)
 {
  dword i;

  for(i=0; i<anzposten; i++)
    if(!akkumulieren_vgl(&posten[i], buchung))
     {
      sum_buchung(&posten[i], buchung);
      return FALSE;
     }

  return TRUE;
 }

void makekontoformat(userpt *up, char what, char format,
                     bool rechnung,
                     FILE *infile, FILE *outfile, 
                     const char *postenakkumulationsdatei) 
 {
  char tempstr[100], id[100];
  struct tm *tm;
  time_t t;
  dword pos, postennr;
  buchungt buchung, summenbuchung;
  signed long uebertrag;
  byte uebertrag_scale;
  int anzmwst, i;
  dword anzposten=0;
  buchungt posten[MAXPOSTEN];
  charsett charset;
  buchungt mwst[100];

  if(format=='l') charset=ISO; else charset=IBM;

  if(postenakkumulationsdatei[0]!=0)
   {
    FILE *infile2;
    char filename[DIRLEN];

    if(postenakkumulationsdatei[0]!=DIRCHAR[0])
      snprintf(filename, DIRLEN-1, "%s" DIRCHAR "%s", config.cwd, postenakkumulationsdatei);
    else
      strmaxcpy(filename, postenakkumulationsdatei, DIRLEN-1);
  
    infile2=fopen(filename, "rt");
    if(infile2!=NULL)
     {
      while(!posten[anzposten].read(infile2))
       {
        posten[anzposten].amount.amount=0;
        posten[anzposten].amount.scale=0;
        posten[anzposten].number=(dword)-1;
        posten[anzposten].datum=0;
        anzposten++;      
       }
      fclose(infile2);
     }
    else
     {
      fprintf(stderr, "%s not found", filename);
      return;
     }
   }

  if(rechnung)
   {
    dword first=0, last=0;
    bool isfirst=TRUE;
    dword rechnungsnummer;

    time(&t);
    tm=localtime(&t);
    anzmwst=0;
    pos=ftell(infile);

    if(what=='k')
      rechnungsnummer=1;
    else
     {
      snprintf(id, TEXTLEN, "auszug.%c.nummer", what);
      up->get(id, tempstr);
      rechnungsnummer=atol(tempstr)+1;
     }

    while(!buchung.read(infile))
     {
      if(isfirst) { first=buchung.number; isfirst=FALSE; }
      last=buchung.number;
     }

    switch(format)
     {
     case 'a':
      fprintf(outfile, "Rechnungsnummer: %ld (%lu-%lu)\n", 
                       rechnungsnummer, first, last);
      fseek(infile, pos, SEEK_SET);

      fprintf(outfile, "\n");
      fprintf(outfile, "A. LEISTUNGEN\n");
      fprintf(outfile, "\n");

      fprintf(outfile, "%s\n", formatkonto_start1);
      fprintf(outfile, "%s\n", formatkonto_start2);
      break;

     case 'l':
      fprintf(outfile, "Rechnungsnummer: {\\bf %ld} (%lu-%lu)\n", 
                       rechnungsnummer, first, last);
      fseek(infile, pos, SEEK_SET);

      fprintf(outfile, "\n");
      fprintf(outfile, "{\\bf A. LEISTUNGEN}\n");
      fprintf(outfile, "\n");

      fprintf(outfile,
        "\\begin{tabular*}{\\textwidth}{c|c|r|r|l@{\\extracolsep{\\fill}}}\n"
        "Nr. & Datum & \\multicolumn{1}{c|}{Betrag} & "
        "\\multicolumn{1}{c|}{MwSt} & Verwendungszweck \\hfill\\\\"
        "\\hline\n");
      break;

     case 'e':
      fseek(infile, pos, SEEK_SET);
      break;
     }

    summenbuchung.amount.amount=0;
    summenbuchung.amount.scale=2;
    summenbuchung.haben=TRUE;

    for(postennr=0; postennr<anzposten; postennr++)
      posten[postennr].haben^=1;

    while(!buchung.read(infile))
      if(strnicmp(buchung.buchungstyp, "ZAHLUNG.", 8)!=0 &&
                  buchung.number<(dword)atol(*up|"buchungsnummer"))
       {
        buchung.haben^=1;

        if(akkumulieren(anzposten, posten, &buchung))
          buchung.format(outfile, format, FALSE, charset);

        sum_buchung(&summenbuchung, &buchung);

        if(buchung.mwstsatz!=0)
         {
          for(i=0; i<anzmwst; i++)
            if(mwst[i].mwstsatz==buchung.mwstsatz)
              break;

          if(i==anzmwst) 
           {
            mwst[i].mwstsatz=buchung.mwstsatz;
            mwst[i].amount.amount=0;
            mwst[i].amount.scale=0;
            anzmwst++;
           }

          sum_buchung(&mwst[i], &buchung);
         }
       }

    for(postennr=0; postennr<anzposten; postennr++)
      if(posten[postennr].amount.amount!=0)
        posten[postennr].format(outfile, format, FALSE, charset);

    switch(format)
     {
     case 'a':
      fprintf(outfile, "%s\n", formatkonto_start2);
      break;

     case 'l':
      fprintf(outfile, "\\hline\n");
      break;

     case 'e':
      break;
     }

    strcpy(buchung.verwendungszweck, "Gesamt");

    summenbuchung.number=(dword)-1;
    summenbuchung.mwstsatz=(dword)-1;
    summenbuchung.format(outfile, format, FALSE, charset);

    switch(format)
     {
     case 'a':
      fprintf(outfile, "\n");
      if(anzmwst!=0) fprintf(outfile, "\n");
      break;

     case 'l':
      fprintf(outfile, "\\end{tabular*}\n"
                       "\n");
      break;

     case 'e':
      break;
     }

    for(i=0; i<anzmwst; i++)
     {
      amountt mwst_betrag;
      char mwst_betrag_s[TEXTLEN];

      mwst_betrag.amount=labs(mwst[i].amount.amount)-
                  (labs(mwst[i].amount.amount)*1000/(mwst[i].mwstsatz+1000));
      mwst_betrag.scale=mwst[i].amount.scale;

      mwst_betrag=rescale(mwst_betrag, 2);
      currency_string(mwst_betrag_s, mwst_betrag, TRUE);

      switch(format)
       {
       case 'a':
        fprintf(outfile, "In diesem Betrag sind %s "
                         "MwSt zu %ld,%ld%% enthalten.\n",
                         mwst_betrag_s,
                         mwst[i].mwstsatz/10, mwst[i].mwstsatz%10);
        break;

       case 'l':
        fprintf(outfile, "In diesem Betrag sind %s "
                         "MwSt zu %ld,%ld\\%% enthalten.\\\\\n",
                         mwst_betrag_s,
                         mwst[i].mwstsatz/10, mwst[i].mwstsatz%10);
        break;

       case 'e':
        break;
       }
     }

    switch(format)
     {
     case 'a':
      fprintf(outfile, "\n");
      fprintf(outfile, "B. ZAHLUNGEN\n");
      fprintf(outfile, "\n");

      fseek(infile, pos, SEEK_SET);

      fprintf(outfile, "%s\r\n", formatkonto_start1);
      fprintf(outfile, "%s\r\n", formatkonto_start2);
      break;

     case 'l':
      fprintf(outfile, "\n");
      fprintf(outfile, "\\vspace*{0.5ex}\n");
      fprintf(outfile, "{\\bf B. ZAHLUNGEN}\n");
      fprintf(outfile, "\n");

      fseek(infile, pos, SEEK_SET);

      fprintf(outfile,
        "\\begin{tabular*}{\\textwidth}{c|c|r|r|l@{\\extracolsep{\\fill}}}\n"
        "Nr. & Datum & \\multicolumn{1}{c|}{Betrag} & "
        "\\multicolumn{1}{c|}{MwSt} & Verwendungszweck \\hfill\\\\"
        "\\hline\n");
      break;

     case 'e':
      fseek(infile, pos, SEEK_SET);
      break;
     }
 
    if(summenbuchung.amount.amount!=0)
     {
      summenbuchung.datum=t;
      snprintf(summenbuchung.verwendungszweck, TEXTLEN,
              "Leistungen Rechnung %ld", rechnungsnummer);
      summenbuchung.format(outfile, format, FALSE, charset);
     }

    if(what=='k')
     {
      uebertrag=0;
      uebertrag_scale=0;
     }
    else
     {
      snprintf(id, TEXTLEN, "auszug.%c.uebertrag", what);
      up->get(id, tempstr);
      uebertrag=atol(tempstr);

      snprintf(id, TEXTLEN, "auszug.%c.uebertrag.scale", what);
      up->get(id, tempstr);
      if(tempstr[0]==0)
        uebertrag_scale=2;
      else
        uebertrag_scale=atol(tempstr);
     }

    if(uebertrag!=0)
     { 
      buchung.clear();
      strcpy(buchung.verwendungszweck, "bertrag der letzten Rechnung");
      buchung.number=(dword)-1;
      buchung.mwstsatz=(dword)-1;
      buchung.amount.amount=labs(uebertrag);
      buchung.haben=(uebertrag<0);
      buchung.amount.scale=uebertrag_scale;
      buchung.format(outfile, format, FALSE, charset);
     }
       
    while(!buchung.read(infile))
      if(strnicmp(buchung.buchungstyp, "ZAHLUNG.", 8)==0 &&
                  buchung.number<(dword)atol(*up|"buchungsnummer"))
       {
        buchung.haben^=1;
        buchung.mwstsatz=(dword)-1;
        buchung.format(outfile, format, FALSE, charset);
       }

    switch(format)
     {
     case 'a':
      fprintf(outfile, "%s\n", formatkonto_start2);
      break;

     case 'l':
      fprintf(outfile, "\\hline\n");
      break;
     }
       
    buchung.clear();
    buchung.number=(dword)-1;
    buchung.mwstsatz=(dword)-1;
    buchung.amount=abs(rescale(getbalance(up), 2));
    buchung.haben=(getbalance(up).amount<0);

    if(!buchung.haben)
      strmaxcpy(buchung.verwendungszweck, "Guthaben", TEXTLEN-1);
    else
      strmaxcpy(buchung.verwendungszweck, "Zu zahlender Betrag", TEXTLEN-1);

    buchung.format(outfile, format, FALSE, charset);

    switch(format)
     {
     case 'l':
      fprintf(outfile, "\\end{tabular*}\n\n");
      break;
     }
   }
  else
    switch(format)
     {
     case 'a':
      if(what=='k')
        fprintf(outfile, "Kontoauszge von......: %s\n", up->name);
      else
       {
        fprintf(outfile, "Kontoauszug von.......: %s\n", up->name);
        snprintf(id, TEXTLEN, "auszug.%c.nummer", what);
        up->get(id, tempstr);
        fprintf(outfile, "Nummer................: %ld\n", atol(tempstr)+1);
       }

      currency_string(tempstr, getbalance(up), TRUE);
      fprintf(outfile, "aktueller Kontostand..: %s\n", tempstr);
      //fprintf(outfile, "Kontoberzug erlaubt..: %s\n", noyes1(up->kontoueberzug));
      fprintf(outfile, "\n%s\n%s\n", formatkonto_start1, formatkonto_start2);

      while(!buchung.read(infile))
        if(buchung.number<(dword)atol(*up|"buchungsnummer"))
          if(akkumulieren(anzposten, posten, &buchung))
            buchung.format(outfile, format, TRUE, IBM);

      for(postennr=0; postennr<anzposten; postennr++)
        if(posten[postennr].amount.amount!=0)
          posten[postennr].format(outfile, format, TRUE, IBM);

      fprintf(outfile, "%s\n", formatkonto_ende);
      break;

     case 'e':
      if(what=='k')
        fprintf(outfile, "Kontoauszge von:\t%s\n", up->name);
      else
       {
        fprintf(outfile, "Kontoauszug von:\t%s\n", up->name);
        snprintf(id, TEXTLEN, "auszug.%c.nummer", what);
        up->get(id, tempstr);
        fprintf(outfile, "Nummer:\t%ld\n", atol(tempstr)+1);
       }

      currency_string(tempstr, getbalance(up), TRUE);
      fprintf(outfile, "aktueller Kontostand:\t%s\n", tempstr);
      fprintf(outfile, "Kontoberzug:\t%s\n", noyes1(atoi(*up|"kontoueberzug")));
      fprintf(outfile, "\n"
              "Nr.\tDatum\tBetrag\tMwSt\tVerwendungszweck\n");
          
      while(!buchung.read(infile))
        if(buchung.number<(dword)atol(*up|"buchungsnummer"))
          if(akkumulieren(anzposten, posten, &buchung))
            buchung.format(outfile, format, FALSE, IBM);

      for(postennr=0; postennr<anzposten; postennr++)
        if(posten[postennr].amount.amount!=0)
          posten[postennr].format(outfile, format, FALSE, IBM);

      fprintf(outfile, "\n");
      break;

     case 'l':
      while(!buchung.read(infile))
        if(buchung.number<(dword)atol(*up|"buchungsnummer"))
          if(akkumulieren(anzposten, posten, &buchung))
            buchung.format(outfile, format, FALSE, ISO);

      for(postennr=0; postennr<anzposten; postennr++)
        if(posten[postennr].amount.amount!=0)
          posten[postennr].format(outfile, format, FALSE, ISO);
      break;
     }
 }

void konto_seek(FILE *infile, dword buchungsnummer)
 {
  buchungt buchung;

  rewind(infile);
  
  while(buchungsnummer!=0)
   {
    buchung.read(infile);
    buchungsnummer--;
   }
 }

void mailkonto(userpt *up, char what, char format,
               const char *filename, const char *mailoutfilename)
 {
  FILE *tmpfile, *infile, *outfile;
  dword len;
  char id[TEXTLEN], tmpfilename[DIRLEN],
       tempstr[TEXTLEN], line[1000], *tptr;

  snprintf(tmpfilename, DIRLEN, DBOXTEMPDIR "listgen.%d.mail.tmp", 
    config.dboxdir, getpid());

  outfile=fopen(mailoutfilename, "ab");

  infile=fopen(filename, "rt");
  if(infile==NULL) return;

  tmpfile=fopen(tmpfilename, "wb");
  if(tmpfile==NULL)
    logf.internalerror("mailkonto", "Temporaerdatei konnte nicht geoeffnet werden");

  while(fgets(line, 900, infile)!=NULL)
   {
    tptr=strchr(line, '\r'); if(tptr!=NULL) *tptr=0;
    tptr=strchr(line, '\n'); if(tptr!=NULL) *tptr=0;

    fprintf(tmpfile, "%s\r\n", line);
   }

  fflush(tmpfile);
  len=ftell(tmpfile);
  fclose(infile);
  fclose(tmpfile);

  fprintf(outfile, "ABS:\tSYSOP@%s.%s\r\n", config.boxname, config.domain);

  switch(format)
   {
   case 'a':
    if(what=='k')
      fprintf(outfile, "BET:\tAuszge fr User %s\r\n", up->name);
    else
     {
      snprintf(id, TEXTLEN, "auszug.%c.nummer", what);
      up->get(id, tempstr);
      fprintf(outfile, "BET:\tAuszug %ld fr User %s\r\n", 
              atol(tempstr)+1, up->name);
     }
    break;

   case 'r':
    if(what=='k')
      fprintf(outfile, "BET:\tGesamtrechnung fr User %s\r\n", up->name);
    else
     {
      snprintf(id, TEXTLEN, "auszug.%c.nummer", what);
      up->get(id, tempstr);
      fprintf(outfile, "BET:\tRechnung %ld fr User %s\r\n", 
              atol(tempstr)+1, up->name);
     }
    break;

   case 'e':
    fprintf(outfile, "BET:\tBuchungs-Export fr User %s\r\n", up->name);
    break;

   }

  fprintf(outfile, "EDA:\t%s\r\n", make_eda_localtime().c_str());

  fprintf(outfile, "EMP:\t%s@%s.%s\r\n",
    up->name, config.boxname, config.domain);

  fprintf(outfile, "LEN:\t%lu\r\n", len);

  fprintf(outfile, "MAILER:\tDBOX Kontoverwaltung\r\n");

  fprintf(outfile, "MID:\t%s\r\n",
    gen_message_id("listgen").c_str());

  if(config.organization[0]!=0)
    fprintf(outfile, "ORG:\t%s\r\n", config.organization);

  fprintf(outfile, "ROT:\r\n");

  fprintf(outfile, "\r\n");

  infile=fopen(tmpfilename, "rb");
  filecopy(infile, outfile, len);
  fclose(infile);
  fclose(outfile);

  remove(tmpfilename);
  remove(filename);
 }

void makekonto(userpt *up, char what, char format, char action, 
               bool test, bool rechnung, const char *postenakkumulationsdatei,
               const char *mailoutfilename)
 {
  FILE *infile, *outfile;
  char filename[DIRLEN], tempstr[TEXTLEN], id[TEXTLEN];
  dword buchungsnummer=0;
  dword neu_buchungsnummer=atol(*up|"buchungsnummer");
  amountt neu_uebertrag=getbalance(up);

  switch(what)
   {
   case 'k':
    buchungsnummer=0;
    break;

   case '1': 
   case '2': 
    snprintf(id, TEXTLEN, "auszug.%c.buchungsnummer", what);
    up->get(id, tempstr);
    buchungsnummer=atol(tempstr);
    break;
   }

  snprintf(filename, DIRLEN, "%skonto", up->getpath());
  infile=fopen(filename, "rt");
  if(infile==NULL) return;

  konto_seek(infile, buchungsnummer);

  if(action=='m')
   {
    if(neu_buchungsnummer!=buchungsnummer)
     {
      snprintf(filename, DIRLEN, DBOXTEMPDIR "listgen.tmp", config.dboxdir);
      outfile=fopen(filename, "wt");
      if(outfile==NULL)
        logf.internalerror("makekonto", "Failed to open tempfile");

      makekontoformat(up, what, format, rechnung,
                      infile, outfile, postenakkumulationsdatei);
      fclose(outfile);

      mailkonto(up, what, format, filename, mailoutfilename);
 
      unlink(filename);
     }
    else
      test=TRUE;
   }
  else
    makekontoformat(up, what, format, rechnung,
                    infile, stdout, postenakkumulationsdatei);

  fclose(infile);

  if(!test)
   {
    switch(what)
     {
     case 'k':
      break;

     case '1':
     case '2':
      if(up->lock())
        fprintf(stderr, "locking %s failed\n", up->name);
      else
       {
        snprintf(id, TEXTLEN, "auszug.%c.nummer", what);
        up->get(id, tempstr);
        up->set(id, atol(tempstr)+1);

        sprintf(id, "auszug.%c.uebertrag", what);
        up->set(id, neu_uebertrag.amount);

        sprintf(id, "auszug.%c.uebertrag.scale", what);
        up->set(id, (int)neu_uebertrag.scale);

        sprintf(id, "auszug.%c.buchungsnummer", what);
        up->set(id, neu_buchungsnummer);

        up->write();
        up->unlock();
       }
      break;
     }
   }
 }

void makekonto(char what, char format, char action, bool test,
               bool rechnung, const char *postenakkumulationsdatei)
 {
  char username[TEXTLEN], *tptr;
  char mailoutfilename[DIRLEN];

  snprintf(mailoutfilename, DIRLEN, DBOXTEMPDIR "listgen.%d.spool.tmp",
          config.dboxdir, getpid());

  while(fgets(username, TEXTLEN-1, stdin)!=NULL)
    if(username[0]!=0 && username[0]!='.' && username[0]!='_')
     {
      tptr=strchr(username, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(username, '\n'); if(tptr!=NULL) *tptr=0;

      username[TEXTLEN-1]=0;
      tptr=strrchr(username, '/');
      if(tptr!=NULL) *tptr=0;
      userpt up(username);
      if(up.read())
        fprintf(stderr, "Not found: %s\n", username);
      else
        makekonto(&up, what, format, action, test, rechnung,
                  postenakkumulationsdatei, mailoutfilename);
     }

  if(action=='m')
    spool_zc_puffer_mail(mailoutfilename, config.boxname);
 }

char *zweckmakros[]=
 {
  "MONAT",
  "QUARTAL",
  "JAHR",
  NULL
 };

char *monate[]=
 {
  "Januar",
  "Februar",
  "Mrz",
  "April",
  "Mai",
  "Juni",
  "Juli",
  "August",
  "September",
  "Oktober",
  "November",
  "Dezember",
  NULL
 };

static void parsezweck(char *zweck)
 {
  char neu[1000], *in, *out;
  int i;
  time_t t;

  time(&t);

  in=zweck;
  out=neu;

  while(*in!=0)
   {
    if(*in=='$')
     {
      in++;
      for(i=0; zweckmakros[i]!=NULL; i++)
       {
        if(strnicmp(in, zweckmakros[i], strlen(zweckmakros[i]))==0)
         {
          in+=strlen(zweckmakros[i]);
          switch(i)
           {
           case 0:
            i=localtime(&t)->tm_mon;
            strcpy(out, monate[i]);
            out+=strlen(monate[i]);
            break;

           case 1:
            out[0]='0'+(localtime(&t)->tm_mon/3)+1;
            out++;
            break;

           case 2:
            sprintf(out, "%04d", localtime(&t)->tm_year+1900);
            out+=4;
            break;
           }
          break;
         }
       }
     }
    else
     {
      *out=*in;
      out++;
      in++;
     }
   }

  *out=0;
  strmaxcpy(zweck, neu, TEXTLEN-1);
 }

void dobuchung(bool test)
 {
  char line[1000], tempstr[1000];

  while(fgets(line, 900, stdin)!=NULL)
    if(line[0]!=0 && line[0]!='#' && line[0]!=';')
     {
      userpt up;
      char *tptr, username[TEXTLEN]="", *lineptr=line;
      long int betrag=0;
      buchungt buchung;
      char betrag_s[TEXTLEN];

      tptr=strchr(line, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(line, '\n'); if(tptr!=NULL) *tptr=0;

      buchung.clear();

      while(lineptr[0]!=0)
       {
        char ch=buchung.parse(&lineptr, tempstr);
        tempstr[TEXTLEN-1]=0;
        switch(upcase(ch))
         {
         case 'U':
          strcpy(username, tempstr);
          dnstring(username);
          break;

         case 'M':
          buchung.mwstsatz=atoi(tempstr);
          break;

         case 'A':
          buchung.amount.scale=atoi(tempstr);
          break;

         case 'Z':
          strcpy(buchung.verwendungszweck, tempstr);
          break;

         case 'B':
          betrag=atoi(tempstr);
          break;

         case 'T':
          strcpy(buchung.buchungstyp, tempstr);
          break;

         default:
          fprintf(stderr, "Ungueltiger Header!\n");
          if(ch!=0) fprintf(stderr, "Headername: %c\n", ch);
          lineptr="";
         }
       }

      if(username[0]==0 || betrag==0 || buchung.verwendungszweck[0]==0)
       {
        fprintf(stderr, "Buchung unvollstaendig!\n");
       }
      else
       {
        strcpy(up.name, username);
        if(up.read())
          fprintf(stderr, "Not found: %s\n", username);
        else
         {
          parsezweck(buchung.verwendungszweck);
  
          if(betrag>=0)
            buchung.haben=TRUE;
          else
           {
            buchung.haben=FALSE;
            betrag=-betrag;
           }

          buchung.amount.amount=betrag;

          strcpy(buchung.bearbeiter, "sysop");

          printf("Username........: \"%s\"\n", username);

          currency_string(betrag_s, buchung.amount, TRUE);
          printf("Betrag..........: %s (%s)\n",
                                    betrag_s,
                                    buchung.haben?"Haben":"Soll");

          printf("MwSt-Satz.......: %ld,%ld\n", 
                                    buchung.mwstsatz/10,
                                    buchung.mwstsatz%10);

          printf("Verwendungszweck: \"%s\"\n\n", buchung.verwendungszweck);

          if(!test)
            dobuchung(&up, &buchung);
         }
       }
     }
 }

void dorenumber()
 {
  char username[TEXTLEN], *tptr;

  while(fgets(username, TEXTLEN-1, stdin)!=NULL)
    if(username[0]!=0 && username[0]!='.' && username[0]!='_')
     {
      tptr=strchr(username, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(username, '\n'); if(tptr!=NULL) *tptr=0;

      username[TEXTLEN-1]=0;
      tptr=strrchr(username, '/');
      if(tptr!=NULL) *tptr=0;
      userpt up(username);
      if(up.read())
        fprintf(stderr, "Not found: %s\n", username);
      else
       {
        char filename[DIRLEN];
        snprintf(filename, DIRLEN, "%skonto", up.getpath());
        FILE *infile=fopen(filename, "rt");
        if(infile!=NULL)
         {
          dword nr=0;
          buchungt buchung;
          while(!buchung.read(infile))
           {
            buchung.number=nr++;
            buchung.write(stdout);
           }
          fclose(infile);
         }
       }
     }

 }
