/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/stat.h>

#include <dbox.h>
#include <tools.h>
#include <charsettools.h>
#include <folderindex.h>
#include <folder.h>
#include <dboxversion.h>
#include <libdboxversion.h>
#include <logf.h>
#include <userdb.h>
#include <account.h>
#include "konto.h"
#include "mail_ch.h"
#include "stats.h"
#include "tans.h"
#include "filtern.h"
#include "dta.h"
#include "checkkonto.h"
#include "chargemenu.h"
#include "../webonline/www_url.h"

bool followalias=FALSE;

void makeftpindex(const char *path, const char *brettname, 
     const char *img_url)
 {
  folderindext fi;
  int handle;
  FILE *outfile_INDEX, *outfile_index_html;
  char filename[TEXTLEN];

  sprintf(filename, "%sindex", path);

  printf("Makeftpindex: Bearbeite \"%s\": ", filename);

  handle=open(filename, O_RDWR|O_BINARY);
  if(handle<0)
   {
    printf("Folder ist leer!\n");
    return;
   }

  if((filelength(handle)/sizeof fi)==0)
   {
    close(handle);
    printf("Folder ist leer!\n");
    return;
   }

  sprintf(filename, "%sINDEX", path);
  outfile_INDEX=fopen(filename, "wt");
  sprintf(filename, "%sindex.html", path);
  outfile_index_html=fopen(filename, "wt");

  if(outfile_INDEX==NULL || outfile_index_html==NULL) return;

  fprintf(outfile_INDEX,
    "%-26s(INDEX)\n"
    "------------------------------------------------------------------------------\n",
    brettname);

  fprintf(outfile_index_html,
    "<html><head><title>Brett %s</title>\n"
    "<h1>Brett %s</h1>\n"
    "<table>", brettname, brettname);

  while(read(handle, &fi, sizeof fi)==sizeof fi)
   {
    if(!fi.flags.deleted && fi.flags.bodyextern)
     {
      struct stat statbuf;
      struct tm *tm;

      sprintf(filename, "%s%s", path, fi.file);
      if(stat(filename, &statbuf)==0)
       {
        char *tptr, *typ, tempstr[1000];

        tptr=strrchr(fi.file, '.');
        if(tptr==NULL)
          typ="binary";
        else
         {
          tptr++;
          if(stricmp(tptr, "mod")==0 ||
             stricmp(tptr, "wav")==0 ||
             stricmp(tptr, "voc")==0 ||
             stricmp(tptr, "au")==0 ||
             stricmp(tptr, "mid")==0)
            typ="sound";
          else if(stricmp(tptr, "txt")==0 ||
               stricmp(tptr, "doc")==0)
            typ="text";
          else
            typ="binary";
         }

        fprintf(outfile_index_html,
          "<tr><td><img src=\"%s/%s.xbm\">", img_url, typ);

        strcpy(tempstr, fi.abs);
        tptr=strchr(tempstr, ' ');
        if(tptr!=NULL) *tptr=0;

        fprintf(outfile_index_html, "<td><a href=\"mailto:%s\">", tempstr);
        fprintf(outfile_index_html, "%s</a>", tempstr);

        tm=localtime(&fi.eda);

        fprintf(outfile_index_html,
          "<td>%02u.%02u.%02u<td>%4luK ",
          tm->tm_mday, tm->tm_mon+1, tm->tm_year%100,
          fi.len/1024+1);

        strcpy(tempstr, fi.bet);
        ibm2iso(tempstr, strlen(tempstr));

        fprintf(outfile_index_html,
          "<td><a href=\"%s\">%s</a>", fi.file, tempstr);

        fprintf(outfile_index_html, "\n");

        strcpy(tempstr, fi.file);
        tempstr[25]=0;
        fprintf(outfile_INDEX,
          "%-26s", tempstr);

        strcpy(tempstr, fi.bet);
        ibm2iso(tempstr, strlen(tempstr));
        tempstr[56]=0;
        fprintf(outfile_INDEX,
          "%s\n", tempstr);
       }
     }
   }

  close(handle);

  fprintf(outfile_index_html, "</table></body></html>\n");
  fclose(outfile_INDEX);
  fclose(outfile_index_html);

  printf("Ok\n");
 }

void makefservindex(const char *path, const char *brettname)
 {
  folderindext fi;
  int handle;
  char filename[TEXTLEN];

  sprintf(filename, "%sindex", path);

  handle=open(filename, O_RDWR|O_BINARY);
  if(handle<0) return;
  if((filelength(handle)/sizeof fi)==0) return;

  printf(";******************************************************\n"
         ";* %-50s *\n"
         ";******************************************************\n"
         "\n",
         brettname);

  printf("B:%s\n\n", brettname);

  while(read(handle, &fi, sizeof fi)==sizeof fi)
   {
    if(!fi.flags.deleted && fi.flags.bodyextern)
     {
      struct stat statbuf;

      sprintf(filename, "%s%s", path, fi.file);
      if(stat(filename, &statbuf)==0)
       {
        char *bet=fi.bet;
        if(strnicmp(fi.file, bet, strlen(fi.file))==0)
         {
          bet+=strlen(fi.file);
          while(*bet==' ' || *bet=='-') bet++;
         }

        printf("N:%s\n", fi.file);
        printf("V:%s\n", path);
        printf("B:%s\n", bet);
        if(!fi.flags.binary) printf("F:T\n");
        printf("\n");
       }
     }
   }

  printf("\n");

  close(handle);
 }

void domakeindex(const char *img_url)
 {
  char tempstr[1000], *tptr;

  while(fgets(tempstr, TEXTLEN-1, stdin)!=NULL)
   {
    if(tempstr[0]=='/')
     {
      tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;

      dnstring(tempstr);
      folderpt fp(tempstr);
      fp.setfilename();
      makeftpindex(fp.getpath(), tempstr, img_url);
     }
   }
 }

void domakefservindex()
 {
  char tempstr[1000], *tptr;

  while(fgets(tempstr, TEXTLEN-1, stdin)!=NULL)
   {
    if(tempstr[0]=='/')
     {
      tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;

      dnstring(tempstr);
      folderpt fp(tempstr);
      fp.setfilename();
      makefservindex(fp.getpath(), tempstr);
     }
   }
 }

void dofollowalias(userpt *up)
 {
  if(followalias && (*up|"alias")[0]!=0)
   {
    strmaxcpy(up->name, *up|"alias", TEXTLEN-1);
    up->read();
   }
 }

void makeuserlist()
 {
  userpt up;
  char username[TEXTLEN];
  char *tptr;

  while(fgets(username, TEXTLEN-1, stdin)!=NULL)
    if(username[0]!=0 && username[0]!='.' && username[0]!='_')
     {
      tptr=strchr(username, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(username, '\n'); if(tptr!=NULL) *tptr=0;

      username[TEXTLEN-1]=0;
      tptr=strrchr(username, '/');
      if(tptr!=NULL) *tptr=0;
      strcpy(up.name, username);
      if(up.read())
        fprintf(stderr, "Not found: %s\n", username);
      else
       {
        char tempstr[TEXTLEN];
        struct tm *tm;
        time_t t;

        dofollowalias(&up);
    
        bankpt bp(up.name);
        bp.read();

        strmaxcpy(tempstr, up.name, 14);
        printf("%-14s  ", tempstr);
        strmaxcpy(tempstr, up|"gruppe", 13);
        printf("%-13s ", tempstr);
        currency_string(tempstr, getbalance(&up), FALSE);
        printf("%c%c%7s  ", 
          atoi(bp|"einzugsermaechtigung")?(atoi(bp|"automatisch_abbuchen")?'E':'e'):' ', 
          atoi(up|"kontoueberzug")?'K':' ', tempstr);

        t=atol(up|"eintragsdatum");
        tm=localtime(&t);

        if(t==0)
          printf("--.--.-- ");
        else
          printf("%02u.%02u.%02u ", tm->tm_mday, tm->tm_mon+1,
            tm->tm_year%100);

        t=atol(up|"letzteranruf");
        tm=localtime(&t);

        if(t==0)
          printf("--.--.--  ");
        else
          printf("%02u.%02u.%02u  ", tm->tm_mday, tm->tm_mon+1,
            tm->tm_year%100);

        strmaxcpy(tempstr, up|"vertretung", 15);
        printf("%s\n", tempstr);
       }
     }
 }

char *dboxadmexportup[]=
 {
  "vorname",
  "nachname",
  "firma",
  "strasse",
  "ort",
  "telefon",
  "fax",
  "sonstiges",
  "kundennummer",
  "anrede",
  "kontoueberzug",
  "letzteranruf",
  "eintragsdatum",
  "anrufe",
  NULL
 };

char *dboxadmexportbp[]=
 {
  "kontoinhaber",
  "bank",
  "blz",
  "kontonummer",
  "einzugsermaechtigung",
  "automatisch_abbuchen",
  "abbuchungs_betrag",
  NULL
 };

void dodboxadmuser()
 {
  userpt up;
  char username[TEXTLEN];
  char *tptr;

   {
    for(int i=0; config.groupdb[i]!=NULL; i++)
     {
      printf("_gruppe=%d\n", i);
      printf("_gruppenname=%s\n", config.groupdb[i]);
      printf("\n");
     }
   }

  while(fgets(username, TEXTLEN-1, stdin)!=NULL)
    if(username[0]!=0 && username[0]!='.' && username[0]!='_')
     {
      tptr=strchr(username, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(username, '\n'); if(tptr!=NULL) *tptr=0;

      username[TEXTLEN-1]=0;
      tptr=strrchr(username, '/');
      if(tptr!=NULL) *tptr=0;
      strcpy(up.name, username);
      if(up.read())
        fprintf(stderr, "Not found: %s\n", username);
      else
       {
        dofollowalias(&up);
    
        bankpt bp(up.name);
        bp.read();

        printf("username=%s\n", up.name);

        for(int i=0; dboxadmexportup[i]!=NULL; i++)
          printf("%s=%s\n", dboxadmexportup[i], up.get(dboxadmexportup[i]));

        printf("kontostand=%ld\n", rescale(getbalance(&up), 2).amount);
         // Rueckwaetskompatibilitaet 

        if((up|"gruppe")[0]!=0) 
          printf("gruppe=%d\n", config.groupdb.index(up|"gruppe"));

        for(int i=0; dboxadmexportbp[i]!=NULL; i++)
          printf("%s=%s\n", dboxadmexportbp[i], bp.get(dboxadmexportbp[i]));

        printf("\n");
       }
     }
 }

void makepointlist()
 {
  userpt up;
  char systemname[TEXTLEN];
  char *tptr;

  while(fgets(systemname, TEXTLEN-1, stdin)!=NULL)
    if(systemname[0]!=0 && systemname[0]!='.' && systemname[0]!='_')
     {
      tptr=strchr(systemname, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(systemname, '\n'); if(tptr!=NULL) *tptr=0;

      systemname[TEXTLEN-1]=0;
      tptr=strrchr(systemname, '/');
      if(tptr!=NULL) *tptr=0;
      strcpy(up.name, systemname);
      if(up.read())
        fprintf(stderr, "Not found: %s\n", systemname);
      else if(!atoi(up|"system"))
       {
        char tempstr[TEXTLEN];
        strmaxcpy(tempstr, up|"netztyp", 8);
        printf("%-8s  ", tempstr);
        strmaxcpy(tempstr, up|"packer", 5);
        printf("%-5s  ", tempstr);
        strmaxcpy(tempstr, systemname, 25);
        printf("%-25s\n", tempstr);
       }
     }
 }

void makesystemlist()
 {
  userpt up;
  char systemname[TEXTLEN];
  char *tptr;

  while(fgets(systemname, TEXTLEN-1, stdin)!=NULL)
    if(systemname[0]!=0 && systemname[0]!='.' && systemname[0]!='_')
     {
      tptr=strchr(systemname, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(systemname, '\n'); if(tptr!=NULL) *tptr=0;

      systemname[TEXTLEN-1]=0;
      tptr=strrchr(systemname, '/');
      if(tptr!=NULL) *tptr=0;
      strcpy(up.name, systemname);
      if(up.read())
        fprintf(stderr, "Not found: %s\n", systemname);
      else if(atoi(up|"system"))
       {
        char tempstr[TEXTLEN];
        strmaxcpy(tempstr, up|"netztyp", 8);
        printf("%-8s  ", tempstr);
        strmaxcpy(tempstr, up|"packer", 5);
        printf("%-5s  ", tempstr);
        strmaxcpy(tempstr, systemname, 25);
        printf("%-25s\n", tempstr);
       }
     }
 }

void pc2asc(const char *in, char *out)
 {
  while(*in!=0)
   {
    if(*in>=' ' && *in<=126)
     {
      *(out++)=*in;
     }
    else
     {
      switch(*in)
       {
       case '':
        *(out++)='a';
        *(out++)='e';
        break;

       case '':
        *(out++)='o';
        *(out++)='e';
        break;

       case '':
        *(out++)='u';
        *(out++)='e';
        break;

       case '':
        *(out++)='s';
        *(out++)='s';
        break;

       case '':
        *(out++)='A';
        *(out++)=islower(*(in+1))?'e':'E';
        break;

       case '':
        *(out++)='O';
        *(out++)=islower(*(in+1))?'e':'E';
        break;

       case '':
        *(out++)='U';
        *(out++)=islower(*(in+1))?'e':'E';
        break;
       }
     }
    in++;
   }
  *out=0;
 }

static void makefido()
 {
  userpt up;
  char username[TEXTLEN];
  char *tptr;

  while(fgets(username, TEXTLEN-1, stdin)!=NULL)
    if(username[0]!=0 && username[0]!='.' && username[0]!='_')
     {
      tptr=strchr(username, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(username, '\n'); if(tptr!=NULL) *tptr=0;

      username[TEXTLEN-1]=0;
      tptr=strrchr(username, '/');
      if(tptr!=NULL) *tptr=0;
      strcpy(up.name, username);
      if(up.read())
        fprintf(stderr, "Not found: %s\n", username);
      else
       {
        char realname[TEXTLEN*2], tempstr[TEXTLEN];

        dofollowalias(&up);

        pc2asc(up|"vorname", realname);
        pc2asc(up|"nachname", tempstr);
        sprintf(realname+strlen(realname), " %s", tempstr);

        printf("%s@%s.%s = %s\n", username, config.boxname, config.domain,
          realname);
       }
     }
 }

static void makemlistpoints()
 {
  userpt up;
  char systemname[TEXTLEN];
  char *tptr;

  while(fgets(systemname, TEXTLEN-1, stdin)!=NULL)
    if(systemname[0]!=0 && systemname[0]!='.' && systemname[0]!='_')
     {
      tptr=strchr(systemname, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(systemname, '\n'); if(tptr!=NULL) *tptr=0;

      systemname[TEXTLEN-1]=0;
      tptr=strrchr(systemname, '/');
      if(tptr!=NULL) *tptr=0;
      strcpy(up.name, systemname);
      if(up.read())
        fprintf(stderr, "Not found: %s\n", systemname);
      else if(!atoi(up|"system"))
        printf("M:%s@%s.zer\n", up.name, up.name);
     }
 }

static void makewww()
 {
  userpt up;
  char username[TEXTLEN], line[1000], url[TEXTLEN];
  char *tptr;
  char *strings[1000];
  int lines=0;

  while(fgets(username, TEXTLEN-1, stdin)!=NULL)
    if(username[0]!=0 && username[0]!='.' && username[0]!='_')
     {
      tptr=strchr(username, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(username, '\n'); if(tptr!=NULL) *tptr=0;

      username[TEXTLEN-1]=0;
      tptr=strrchr(username, '/');
      if(tptr!=NULL) *tptr=0;
      strmaxcpy(up.name, username, TEXTLEN-1);
      if(up.read())
        fprintf(stderr, "Not found: %s\n", username);
      else
       {
        char realname[TEXTLEN];

        dofollowalias(&up);

        if((up|"vorname")[0]==0 && (up|"nachname")[0]==0)
          strmaxcpy(realname, up|"firma", TEXTLEN-1);
        else
         {
          if((up|"vorname")[0]!=0 && (up|"nachname")[0]==0)
            strmaxcpy(realname, up|"vorname", TEXTLEN-1);
          else if((up|"vorname")[0]==0 && (up|"nachname")[0]!=0)
            strmaxcpy(realname, up|"nachname", TEXTLEN-1);
          else
            sprintf(realname, "%s %s", up|"vorname", up|"nachname");

          if((up|"firma")[0]!=0)
            sprintf(realname+strlen(realname), ", %s", up|"firma");
         }

        ibm2iso(realname, strlen(realname));
        wwwpt wp(username);
        wp.clear();
        wp.read();
        if((wp|"realname")[0]==0) wp.set("realname", realname);
        geturl(&wp, url);

        sprintf(line, "<li><a target=\"_top\" "
          "href=\"%s\">%s</a>", url, wp|"realname");

        if((wp|"betreff")[0]!=0)
          sprintf(line+strlen(line), " - %s", wp|"betreff");

        strings[lines]=(char *)malloc(strlen(line)+1);
        if(strings[lines]==NULL)
          logf.internalerror("makewww", "malloc failed");

        strcpy(strings[lines], line);
        lines++;
       }
     }

  if(lines!=0)
   {
    int a, b, smallest;
    char s1[1000], s2[1000];

    for(a=0; a<lines; a++)
     {
      smallest=a;
      for(b=a+1; b<lines; b++)
       {
        tptr=strstr(strings[smallest], "\">");
        if(tptr==NULL) 
          logf.internalerror("makewww", "assertion failed");

        tptr+=2;
        pc2asc(tptr, s1);

        tptr=strstr(strings[b], "\">");        
        if(tptr==NULL)
          logf.internalerror("makewww", "assertion failed");

        tptr+=2;
        pc2asc(tptr, s2);

        if(stricmp(s1, s2)>0)
          smallest=b;
       }

      if(smallest!=a)
       {
        tptr=strings[a];
        strings[a]=strings[smallest];
        strings[smallest]=tptr;
       }
     }

    for(a=0; a<lines; a++)
      printf("%s\n", strings[a]);
   }
 }

static bool parseoptions(int argc, char **argv)
 {
  int i;
  bool userlist=FALSE, pointlist=FALSE,
       fido=FALSE, mlistpoints=FALSE, mailcharge=FALSE,
       showmailclass=FALSE, statlogfiles=FALSE,
       statshow=FALSE, www=FALSE, tanserzeugen=FALSE, 
       tansauswerten=FALSE, systemlist=FALSE,
       buchung=FALSE, dboxadmuser=FALSE,
       filtern=FALSE, lastschrift=FALSE, ftp_index=FALSE,
       fileserv_index=FALSE, konto=FALSE, test=TRUE, _chargemenu=FALSE,
       rueckzahlung=FALSE, rechnung=FALSE, checkkonto=FALSE,
       renumber=TRUE;

  byte lastschrift_format=0;

  char boxname[TEXTLEN]="", tanspw[TEXTLEN]="", tansfolder[DIRLEN]="",
       programm[DIRLEN]="", gegenbuchungsdatei[TEXTLEN]="",
       postenakkumulationsdatei[TEXTLEN]="", charset_s[TEXTLEN]="",
       img_url[TEXTLEN]="", k_format=0, k_what=0, action=0;
  charsett charset=IBM;

  for(i=1; i<argc; i++)
   {
    if(argv[i][0]!='-') goto aufruf;

    switch(argv[i][1])
     {
     case '?':
     case 'h':
     case 'H':
      goto help;

     case 'b':
      strmaxcpy(boxname, argv[i]+2, TEXTLEN-1);
      break;

     case 'c':
      strmaxcpy(charset_s, argv[i]+2, TEXTLEN-1);
      break;

     case 'a':
      followalias=TRUE;
      break;

     case 'r':
      rechnung=TRUE;
      break;

     case 'l':
      switch(argv[i][2])
       {
       case 'u':
        userlist=TRUE;
        break;

       case 'p':
        pointlist=TRUE;
        break;

       case 's':
        systemlist=TRUE;
        break;

       case 'm':
        mlistpoints=TRUE;
        break;

       case 'f':
        fido=TRUE;
        break;
 
       case 'w':
        www=TRUE;
        break;

       case 'b':
        switch(argv[i][3])
         {
         case 'f':
          fileserv_index=TRUE;
          break;

         case 'w':
          ftp_index=TRUE;
          break;

         case 'u':
          strmaxcpy(img_url, argv[i]+4, TEXTLEN-1);
          break;

         default:
          goto help;
         }
        break;

       default:
        goto help;
       }
      break;

     case 'k':
      switch(argv[i][2])
       {
       case 'm':
        _chargemenu=TRUE;
        break;

       case 'c':
        checkkonto=TRUE;
        if(argv[i][3]=='x') test=FALSE;
        break;

       case 't':
        if(argv[i][3]=='r') renumber=TRUE;
        break;

       case '1':
       case '2':
       case 'k':
        konto=TRUE;
        k_what=argv[i][2];

        switch(argv[i][3])
         {
         case 'a':
         case 'e':
         case 'l':
          k_format=argv[i][3];
          switch(argv[i][4])
           {
           case 'a':
           case 'm':
            action=argv[i][4];
            if(argv[i][5]=='x') test=FALSE;
            break;

           default:
            goto help;
           }
          break;

         default:
          goto help;
         }
        break;
 
       case 'b':
        buchung=TRUE;
        if(argv[i][3]=='x') test=FALSE;
        break;

       case 'l':
        if(argv[i][3]=='f')
         {
          strmaxcpy(gegenbuchungsdatei, argv[i]+4, TEXTLEN-1);
         }
        else
         {
          lastschrift=TRUE;
          lastschrift_format=atoi(argv[i]+3);
         }
        break;

       case 'r':
        if(argv[i][3]=='f')
         {
          strmaxcpy(gegenbuchungsdatei, argv[i]+4, TEXTLEN-1);
         }
        else
         {
          lastschrift=TRUE;
          lastschrift_format=atoi(argv[i]+3);
          rueckzahlung=TRUE;
         }
        break;

       case 'a':
        if(argv[i][3]=='f')
         {
          strmaxcpy(postenakkumulationsdatei, argv[i]+4, TEXTLEN-1);
         }
        else
          goto help;
        break;

       default:
        goto help;
       }
      break;

     case 'f':
      strmaxcpy(programm, argv[i]+2, DIRLEN-1);
      filtern=TRUE;
      break;

     case 'm':
      switch(argv[i][2])
       {
       case 'm':
       case 'a':
        mailcharge=TRUE;
        action=argv[i][2];
        if(argv[i][3]=='x') test=FALSE;
        break;

       case 's':
        showmailclass=TRUE;
        break;

       default:
        goto help;
       }
      break;

     case 's':
      switch(argv[i][2])
       {
       case 'l':
        statlogfiles=TRUE;
        break;

       case 'a':
        statshow=TRUE;
        break;

       default:
        goto help;
       }
      break;
  
     case 't':
      switch(argv[i][2])
       {
       case 'a':
        tansauswerten=TRUE;
        strmaxcpy(tansfolder, argv[i]+3, TEXTLEN-1);
        break;

       case 'e':
        tanserzeugen=TRUE;
        strmaxcpy(tanspw, argv[i]+3, TEXTLEN-1);
        break;

       case 'l':
        dboxadmuser=TRUE;
        break;

       default:
        goto help;
       }
      break;

     default:
      goto aufruf;
     }
   }

  if(config.read(boxname))
    return TRUE;

  if(stricmp(charset_s, "IBM")==0)
    charset=IBM;
  else if(stricmp(charset_s, "ISO1")==0)
    charset=ISO;

  if(userlist)
    makeuserlist();
  else if(pointlist)
    makepointlist();
  else if(systemlist)
    makesystemlist();
  else if(konto)
    makekonto(k_what, k_format, action, test, rechnung,
              postenakkumulationsdatei);
  else if(fido)
    makefido();
  else if(mlistpoints)
    makemlistpoints();
  else if(mailcharge)
    domailcharge(action, test);
  else if(showmailclass)
    doshowmailclass();
  else if(statlogfiles)
    stat_logfiles();
  else if(statshow)
    stat_show();
  else if(www)
    makewww();
  else if(tanserzeugen)
    maketans(tanspw);
  else if(tansauswerten)
    dboxadm(tansfolder);
  else if(buchung)
    dobuchung(test);
  else if(dboxadmuser)
    dodboxadmuser();
  else if(filtern)
    dofiltern(programm, charset);
  else if(lastschrift)
    dolastschrift(lastschrift_format, gegenbuchungsdatei, rueckzahlung);
  else if(ftp_index)
    domakeindex(img_url);
  else if(fileserv_index)
    domakefservindex();
  else if(checkkonto)
    docheckkonto(!test);
  else if(_chargemenu)
    chargemenu();
  else if(renumber)
    dorenumber();
  else
    goto aufruf;

  return FALSE;

 aufruf:
  fprintf(stderr, "Falscher Aufruf!\n");

 help:
  fprintf(stderr,
    "\n"
    "* * LISTGEN - DBox Modul fr Datenverwaltung - Version " DBOXVERSIONNR " * *\n"
    "\n"
    "Aufruf:                      Aktion:\n"
    "\n"
    " listgen [-?] [-h]            Hilfe ausgeben\n"
    " listgen -lu                  Userliste formatieren\n"
    " listgen -fshellscript        Userliste filtern\n"
    " listgen -cIBM                Zeichensatz IBM\n"
    " listgen -cISO1               Zeichensatz ISO1\n");

  fprintf(stderr, 
    " listgen -lp                  Pointliste formatieren\n"
    " listgen -ls                  Systemliste formatieren\n"
    " listgen -lm                  Pointuserliste fr mlist\n");
  
  fprintf(stderr, 
    " listgen -lf                  Fido-E-Mail Liste formatieren\n");

  fprintf(stderr,
    " listgen -lw                  WWW-Userindex erzeugen\n"
    " listgen -lbw -lbuBilder_URL  Bretter: FTP/WWW-Index erstellen\n");

  fprintf(stderr, 
    " listgen -lbf                 Bretter: fileserv-Liste erstellen\n");

  fprintf(stderr,
    " listgen -kkFAx               Alle Buchungen\n"
    " listgen -k1FAx               Buchungen ab Posten 1\n"
    " listgen -k2FAx               Buchungen ab Posten 2\n"
    " listgen -kb                  Buchung (Test)\n"
    " listgen -kbx                 Buchung\n"
    " listgen -kl0                 automatische Lastschriften (ASCII)\n"
    " listgen -kl1                 automatische Lastschriften (ASCII, Export)\n"
    " listgen -kl2                 automatische Lastschriften (DTAUS)\n"
    " listgen -kr0                 Rueckzahlungen (ASCII)\n"
    " listgen -kr1                 Rueckzahlungen (ASCII, Export)\n"
    " listgen -kr2                 Rueckzahlungen (DTAUS)\n"
    " listgen -kc                  Kontostand ueberpruefen\n"
    " listgen -kcx                 Kontostand ueberpruefen und reparieren\n"
    " listgen -ktr                 Buchungen neu nummerieren\n"
    " listgen -mAx                 E-Mail-Rechnungen verschicken\n"
    " listgen -ms                  E-Mail-Klassen anzeigen\n"
    " listgen -km                  Mennutzung buchen\n"
    " listgen -tl                  dboxadm-Userliste erzeugen\n"
    " listgen -tePassword          Tans erzeugen\n"
    " listgen -taMailfolder        dboxadm-Mails auswerten\n");

   fprintf(stderr,
    " listgen -sl                  Auslastungsstatistik: Logfiles auswerten\n"
    " listgen -sa                  Auslastungsstatistik anzeigen\n"
    "\n"
    "Weitere Optionen:\n"
    "  -b   Boxname\n"
    "  -a   Aliasen folgen\n"
    "  -klf Dateiname Lastschriftgegenbuchungen\n"
    "  -kaf Dateiname Postenakkumulation\n"
    "  -r   Als Rechnung\n"
    "  F    Format: e=Export, a=ASCII, l=LaTex\n"
    "  A    Aktion: a=anzeigen, m=Mail\n"
    "\n"
   );

  return TRUE;
 }

int main(int argc, char **argv)
 {
  if(init_libdbox(LIBDBOXVERSION, LIBDBOXVERSIONNR)) return 1;

  parseoptions(argc, argv);

  return 0;
 }
