/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>

#include <dboxpath.h>
#include <logf.h>
#include <config.h>
#include <zconnect.h>
#include <flags.h>
#include <userdb.h>
#include <tools.h>
#include <account.h>
#include <spoolp.h>
#include <messageid.h>

#include "mail_ch.h"

char resolve_bang[]="hs-gate.handshake.de";

#define MAXMASK 1000

struct mailklasset
 {
  char *name, **mask;
  dword charge;          // Bytes/PF
 } mailklassen[11];

static void readmailclass()
 {
  FILE *infile;
  char kennung[500], text[500], filename[DIRLEN];
  byte mailclass=0xff;
  int anzmask=0;

  snprintf(filename, DIRLEN, DBOXSYSTEMDIR "mclass", config.dboxdir);
  infile=fopen(filename, "rt");

 schleife:
  switch(readconfigline(infile, kennung, text, 499))
   {
   case 0:
    if(stricmp(kennung, "name")==0)
     {
      if(mailclass==9) goto error;
      if(mailclass==0xff) mailclass=0; else mailclass++;
      mailklassen[mailclass].name=(char *)malloc(strlen(text)+1);
      if(mailklassen[mailclass].name==NULL)
        logf.internalerror("readmailclass", "malloc failed");
      strcpy(mailklassen[mailclass].name, text);
      mailklassen[mailclass].mask=(char **)malloc(MAXMASK*4);
      mailklassen[mailclass].mask[0]=NULL;
      mailklassen[mailclass].charge=0;
      anzmask=0;
     }
    else if(stricmp(kennung, "mask")==0)
     {
      if(mailclass==0xff) goto error;
      if(anzmask==(MAXMASK-2)) goto error;
      mailklassen[mailclass].mask[anzmask]=(char *)malloc(strlen(text)+1);
      if(mailklassen[mailclass].mask[anzmask]==NULL) 
        logf.internalerror("readmailclass", "malloc failed");
      strcpy(mailklassen[mailclass].mask[anzmask], text);
      anzmask++;
      mailklassen[mailclass].mask[anzmask]=NULL;
     }
    else
     {
     error:
      fprintf(stderr, "Fehler in mclass!\n");
      logf.internalerror("readmailclass", "Fehler in mclass");
     }
    goto schleife;

   case 1:
    break;

   default:
    goto error;
   }

  mailklassen[mailclass+1].name=NULL;
  fclose(infile);
 }

static void readmailcharge(const char *gruppe)
 {
  char *info, filename[DIRLEN], *tptr, *iptr, tempstr[DIRLEN];
  byte mailclass;

  if(gruppe[0]==0) return;

  sprintf(filename, BOXGRUPPENDIR "ch_mail", config.boxdir, gruppe);
  iptr=info=readtextconfigfile(filename);
  if(info==NULL) return;

  while(*iptr!=0)
   {
    strmaxcpy(tempstr, iptr, TEXTLEN-1);

    tptr=tempstr;
    while(*tptr!=0 && *tptr!=' ' && *tptr!='\t') tptr++;

    if(*tptr==0)
      break;

    *(tptr++)=0;

    while(*tptr==' ' || *tptr=='\t') tptr++;

    if(*tptr==0)
      break;

    for(mailclass=0; mailklassen[mailclass].name!=NULL; mailclass++)
     {
      if(stricmp(tempstr, mailklassen[mailclass].name)==0)
        mailklassen[mailclass].charge=atol(tptr);
     }

    iptr+=strlen(iptr)+1;
   }

  free(info);  
 }

void doshowmailclass()
 {
  int i, j;

  readmailclass();

  for(i=0; mailklassen[i].name!=NULL; i++)
   {
    printf("Name: %s\n", mailklassen[i].name);
    for(j=0; mailklassen[i].mask[j]!=NULL; j++)
      printf("  %s\n", mailklassen[i].mask[j]);
    printf("\n");
   }
 }

static bool compare(const char *mask, const char *host)
 {
  char tempstr[1000];

  if(mask[0]=='*')
    return checkjoker(mask, host, TRUE);

  if(checkjoker(mask, host, TRUE)) return TRUE;
  sprintf(tempstr, "*.%s", mask);
  return checkjoker(tempstr, host, TRUE);
 }

static byte getmailclass2(const char *host)
 {
  byte mailclass;
  int i;
  
  for(mailclass=0; mailklassen[mailclass].name!=NULL; mailclass++)
   {
    for(i=0; mailklassen[mailclass].mask[i]!=NULL; i++)
     {
      if(mailklassen[mailclass].mask[i][0]=='!')
       {
        if(compare(mailklassen[mailclass].mask[i]+1, host))
          goto nextclass;
       }
      else
       {
        if(compare(mailklassen[mailclass].mask[i], host))
          return mailclass;
       }
     }
   nextclass:;
   }

  return 0;
 }

static byte getmailclass(char *address)
 {
  char *tptr, *tptr2;
  byte mailclass;

  tptr=strrchr(address, '@');
  if(tptr==NULL) return 0;

  tptr++;
  if(stricmp(tptr, resolve_bang)==0)
   {
    *(tptr-1)=0;
    tptr2=strchr(address, '!');
    *(tptr-1)='@';
    if(tptr2!=NULL)
     {
      *tptr2=0;
      mailclass=getmailclass2(address);
      *tptr2='!';
      return mailclass;
     }
   }

  return getmailclass2(tptr);
 }

static void domailcharge(userpt *up, const char *mailoutfilename)
 {
  char tempstr[5000], tempstr2[TEXTLEN], filename[DIRLEN],
       tmpfilename[DIRLEN], *tptr, *tptr2, *tptr3;
  FILE *infile, *outfile, *tmpfile;
  dword len, groesse;
  byte mailclass;
  dword sums[10], sum, tmpsum;
  int i;
 
  snprintf(tmpfilename, DIRLEN-1, DBOXTEMPDIR "listgen.%d.main_ch.tmp",
    config.dboxdir, getpid());

  grouppt gp(*up|"gruppe");
  if(gp.read() || !getinternalflag(up, &gp, CHARGE_MAIL))
    return;

  sprintf(filename, BOXUSERDIR "mail.log", config.boxdir, up->name);
  if(access(filename, 00)!=0) return;

  readmailcharge(*up|"gruppe");

  infile=fopen(filename, "rt");
  if(infile==NULL) return;

  if(filelength(fileno(infile))<2000)
   {
    fclose(infile);
    return;
   }

  tmpfile=fopen(tmpfilename, "wb");
  if(tmpfile==NULL) 
    logf.internalerror("domailcharge", "Temporaerdatei konnte nicht geoeffnet werden");

  fprintf(tmpfile, "Mailabrechnung von.....: %s\r\n", up->name);
  fprintf(tmpfile, "\r\n"
    "    Datum/Zeit     Gre  Typ  Absender / Empfnger / Message-Id\r\n"
    ""
    "\r\n");

  for(mailclass=0; mailclass<10; mailclass++)
    sums[mailclass]=0;

  while(fgets(tempstr, 900, infile)!=NULL)
   {
    tptr=strrchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;
    tptr=strrchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;
    if(tempstr[0]!=0)
     {
      tempstr[20]=0;
      tptr=strrchr(tempstr+21, '\t');
      if(tptr==NULL) continue;
      groesse=atol(tptr+1);
      *tptr=0;
      tptr=strchr(tempstr+21, '\t');
      if(tptr==NULL) continue;
      tptr++;

      if(tempstr[22]=='I')
       {
        tptr2=strchr(tptr, '\t');
        if(tptr2==NULL) continue;
        tptr2++;
       }
      else
        tptr2=tptr;

      tptr3=strchr(tptr2, '\t');
      if(tptr3==NULL) continue;
      *tptr3=0;
      mailclass=getmailclass(tptr2);
      *tptr3='\t';

      if(groesse>9999)
        sprintf(tempstr2, "%luK", groesse/1000);
      else
        sprintf(tempstr2, "%lu", groesse);       

      tempstr[17]=0;
      
      fprintf(tmpfile, "%s %6s  %c%u   %s\r\n",
        tempstr, tempstr2, tempstr[22], mailclass, tptr);

      sums[mailclass]+=groesse;
     }
   }

  fclose(infile);

  fprintf(tmpfile, 
    ""
    "\r\n"
    "\r\n"
    "      Klasse            Preis          Bytes      Summe\r\n"
    "\r\n");

  sum=0;

  for(mailclass=0; mailklassen[mailclass].name!=NULL; mailclass++)
   {
    fprintf(tmpfile, " %u: %-14s",
      mailclass, mailklassen[mailclass].name);

    if(mailklassen[mailclass].charge==0)
      fprintf(tmpfile, " -- kostenlos --");
    else
      fprintf(tmpfile, "%7lu Bytes/PF", mailklassen[mailclass].charge);

    fprintf(tmpfile, "  %11lu ", sums[mailclass]);

    if(mailklassen[mailclass].charge==0)
      tmpsum=0;
    else
      tmpsum=sums[mailclass]/mailklassen[mailclass].charge;

    sum+=tmpsum;

    fprintf(tmpfile, "%5lu,%02lu DM\r\n", tmpsum/100, tmpsum%100);
   }

  fprintf(tmpfile,
    "\r\n"
    "                                     Gesamtsumme: %5lu,%02lu DM\r\n"
    "\r\n"
    "Der Betrag wird von Userkonto %s abgebucht.\r\n"
    "\r\n"
    "\r\nLegende:\r\n"
    "  1. Zeichen: I = incoming Mail, O = outgoing Mail\r\n"
    "  2. Zeichen: Mailklasse\r\n"
    "\r\n", sum/100, sum%100, up->name);

  fflush(tmpfile);
  len=ftell(tmpfile);
  fclose(tmpfile);

  outfile=fopen(mailoutfilename, "ab");
  if(outfile==NULL) 
    logf.internalerror("domailcharge", "Temporaerdatei konnte nicht geoeffnet werden");

  fprintf(outfile, "ABS:\tSYSOP@%s.%s\r\n", config.boxname, config.domain);

  fprintf(outfile, "BET:\tMailabrechnung fr %s\r\n", up->name);

  fprintf(outfile, "EDA:\t%s\r\n", make_eda_localtime().c_str());

  fprintf(outfile, "EMP:\t%s@%s.%s\r\n",
    up->name, config.boxname, config.domain);

  fprintf(outfile, "LEN:\t%lu\r\n", len);

  fprintf(outfile, "MAILER:\tDBOX Kontoverwaltung\r\n");

  fprintf(outfile, "MID:\t%s\r\n",
    gen_message_id("listgen").c_str());

  if(config.organization[0]!=0)
    fprintf(outfile, "ORG:\t%s\r\n", config.organization);

  fprintf(outfile, "ROT:\r\n");

  fprintf(outfile, "\r\n");

  infile=fopen(tmpfilename, "rb");
  filecopy(infile, outfile, len);
  fclose(infile);
  fclose(outfile);
  remove(tmpfilename);

  if(sum>0)
   {
    buchungt buchung;

    sprintf(buchung.verwendungszweck, 
            "Mailabrechnung %s", up->name);

    strcpy(buchung.buchungstyp, "MAIL");
    buchung.mwstsatz=config.mwstsatz;
    buchung.haben=FALSE;
    buchung.amount.amount=sum;
    strcpy(buchung.bearbeiter, "sysop");

    dobuchung(up, &buchung);
   }

  i=0;
  do
    sprintf(tempstr, "%s.%d", filename, i++); 
  while(access(tempstr, 0)==0);
  
  rename(filename, tempstr);
 }

void domailcharge(char action, bool test)
 {
  userpt up;
  char systemname[TEXTLEN];
  char *tptr;
  char mailoutfilename[DIRLEN];

  readmailclass();

  snprintf(mailoutfilename, DIRLEN-1, DBOXTEMPDIR "listgen.%d.mail_ch.spool", 
    config.dboxdir, getpid());

  while(fgets(systemname, TEXTLEN-1, stdin)!=NULL)
    if(systemname[0]!=0 && systemname[0]!='.' && systemname[0]!='_')
     {
      tptr=strchr(systemname, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(systemname, '\n'); if(tptr!=NULL) *tptr=0;

      systemname[TEXTLEN-1]=0;
      tptr=strrchr(systemname, '/');
      if(tptr!=NULL) *tptr=0;
      strcpy(up.name, systemname);
      if(up.read())
        fprintf(stderr, "Not found: %s\n", systemname);
      else
        domailcharge(&up, mailoutfilename);
     }

  spool_zc_puffer_mail(mailoutfilename, config.boxname);
 }
