/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>

#include <dboxpath.h>
#include <logf.h>
#include <config.h>
#include "stats.h"

struct statistikt
 {
  struct
   {
    unsigned long minuten;
    char tag;
    unsigned tage;
   } s[24];
 } statistik[7];

const char *tage[7]={ "Sonntag", "Montag", "Dienstag", "Mittwoch",
  "Donnerstag", "Freitag", "Samstag" };

void readstat(word port)
 {
  char filename[DIRLEN];
  FILE *sf;

  sprintf(filename, DBOXPORTDIR "stat.dat", config.dboxdir, port);

  if((sf=fopen(filename, "rb"))!=NULL)
   {
    fread(&statistik, sizeof statistik, 1, sf);
    fclose(sf);
   }
 }

void writestat(word port)
 {
  char filename[DIRLEN];
  FILE *sf;
  
  sprintf(filename, DBOXPORTDIR "stat.dat", config.dboxdir, port);

  if((sf=fopen(filename, "wb"))==NULL)
    fprintf(stderr, "Fehler!\n");
  else
   {
    fwrite(&statistik, sizeof statistik, 1, sf);
    fclose(sf);
   }
 }

void buchung(tm *t, int minuten)
 {
  printf("  Buche %d Minuten auf %d.\n", minuten, t->tm_wday);
  if(statistik[t->tm_wday].s[t->tm_hour].tag!=t->tm_mday)
   {
    statistik[t->tm_wday].s[t->tm_hour].tage++;
    statistik[t->tm_wday].s[t->tm_hour].tag=t->tm_mday;
   }
  statistik[t->tm_wday].s[t->tm_hour].minuten+=minuten;
 }

void addtime(time_t start, time_t ende)
 {
  tm ts, te;

  ts=*localtime(&start);
  te=*localtime(&ende);

  printf("%02u.%02u.%02u %02u:%02u:%02u - ", ts.tm_mday, ts.tm_mon+1,
    ts.tm_year, ts.tm_hour, ts.tm_min, ts.tm_sec);
  printf("%02u.%02u.%02u %02u:%02u:%02u \n", te.tm_mday, te.tm_mon+1,
    te.tm_year, te.tm_hour, te.tm_min, te.tm_sec);

  if(ts.tm_mday==te.tm_mday && ts.tm_hour==te.tm_hour)
    buchung(&ts, te.tm_min-ts.tm_min+1);
  else
   {
    buchung(&ts, 60-ts.tm_min);
    ts.tm_hour++;
    ts.tm_min=0;
    mktime(&ts);

    buchung(&te, te.tm_min+1);
    ts.tm_min=0;
    mktime(&ts);

    buchung(&te, te.tm_min+1);
    ts.tm_min=0;

    while(ts.tm_mday!=te.tm_mday || ts.tm_hour!=te.tm_hour)
     {
      buchung(&ts, 60);
      ts.tm_hour++;
      mktime(&ts);
     }
   }
 }

time_t parselogtime(const char *s)
 {
  tm t;

  t.tm_mday=atoi(s);
  t.tm_mon=atoi(s+3)-1;
  t.tm_year=atoi(s+6);
  t.tm_hour=atoi(s+9);
  t.tm_min=atoi(s+12);
  t.tm_sec=0;
  return mktime(&t);
 }

void stat_logfiles()
 {
  char tempstr[1000], dir[DIRLEN], filename[DIRLEN];
  FILE *infile;
  int i;

  while(fgets(tempstr, 900, stdin)!=NULL)
   {
    word port=atoi(tempstr);
    if(port<1 || port>MAXPORTS)
      logf.internalerror("stat_logfiles", "Port out of range");

    readstat(port);    
    sprintf(dir, DBOXPORTDIR, config.dboxdir, port);
    sprintf(filename, "%slogfile", dir);
    infile=fopen(filename, "rt");
    if(infile!=NULL)
     {
      while(fgets(tempstr, 900, infile)!=NULL)
        if(tempstr[0]!=0 && tempstr[0]!='\n')
         {

         }      
      
      fclose(infile);
      i=0;
      do sprintf(tempstr, "%s.%d", filename, i); while(access(tempstr, 00)==0);
      rename(filename, tempstr);
     }   
   }
 }

void showstat(int port)
 {
  int tag, stunde, i, werte[24];

  printf("\nAuslastungsstatistik fr die Mailbox %s Port %u\n"
         "\n\n\n", 
         config.boxname, port);

  for(tag=0; tag<7; tag++)
   {
    printf("%s\n", tage[tag]);
    for(i=strlen(tage[tag]); i!=0; i--) printf("");
    printf("\n\n");

    for(stunde=0; stunde<24; stunde++)
     {
      if(statistik[tag].s[stunde].tage==0)
        werte[stunde]=0;
      else
       {
        i=statistik[tag].s[stunde].minuten/statistik[tag].s[stunde].tage;
        if(i>59) i=59;
        werte[stunde]=1+i*9/59;
       }
     }

    for(i=0; i<10; i++)
     {
      printf("%c ", (i==0 || i==5)?'':'');
      for(stunde=0; stunde<24; stunde++)
        if(10-werte[stunde]<=i)
          printf(" ");
        else
          printf("   ");
      printf("\n");
     }

    printf("\n"
           " 00 01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23\n\n");
   }
 }

void stat_show()
 {
  char tempstr[1000], dir[DIRLEN];

  while(fgets(tempstr, 900, stdin)!=NULL)
   {
    word port=atoi(tempstr);
    if(port<1 || port>MAXPORTS)
      logf.internalerror("stat_logfiles", "Port out of range");
    
    readstat(port);
    sprintf(dir, DBOXPORTDIR, config.dboxdir, port);
    


   }
 }

