/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <fcntl.h>
#include <time.h>
#include <stdio.h>

#include <dbox.h>
#include <oldtools.h>
#include <tools.h>
#include <logf.h>
#include <zconnect.h>
#include <config.h>
#include <account.h>
#include <dboxpath.h>
#include <userdb.h>
#include <mtoolzc.h>
#include <messageid.h>

#include "tans.h"

char headertext[]=
  "Ŀ\r\n"
  "        DBOXADM - Tool zur Fernverwaltung des DBOX-Mailboxsystems         \r\n"
  "\r\n"
  "\r\n";

void maketans(const char *password)
 {
  char pw[TEXTLEN], outfilename[DIRLEN];
  dword pwcrc, seed;
  FILE *outfile;
  
  sprintf(outfilename, BOXSYSTEMDIR "dboxadm.tan", config.boxdir);
  outfile=fopen(outfilename, "at");
  if(outfile==NULL)
   {
    fprintf(stderr, "%s konnte nicht geoeffnet werden!\n", outfilename);
    return;
   }

  strcpy(pw, password);
  dnstring(pw);
  pwcrc=str2crc(pw);
  seed=time(NULL)^pwcrc;

  fprintf(outfile, "#\n"
                   "# Password: %s (Checksum: %u)\n"
                   "# Random-Seed: %lu\n"
                   "# Fingerprint: %x\n"
                   "#\n", pw, word(pwcrc&0xff), seed, word(seed&0xff));

  printf("--DBOXADM--\n"
    "%u %x\n", word(pwcrc&0xff), word(seed&0xff));

  srandom(seed);

  for(int i=0; i<50; i++)
   {
    word t1, t2, t3, t4;

    t1=random();
    t2=random();
    t3=random();
    t4=t1^t2^t3;

    fprintf(outfile, "T%u-%u-%u-%u\n", t1, t2, t3, t4);

    t1^=pwcrc;
    t2^=pwcrc>>16;
    t3^=pwcrc;
    t4^=pwcrc>>16;
    
    printf("T%u-%u-%u-%u\n", t1, t2, t3, t4);
   }

  fclose(outfile);
 }

static void printheader(FILE *outfile, const char *bez, const char *emp,
                        const char *bet, dword len)
 {
  fprintf(outfile, "ABS:\tMAPS@%s.%s\r\n", config.boxname, config.domain);
  fprintf(outfile, "BET:\t%s\r\n", bet);
  if(bez!=NULL) fprintf(outfile, "BEZ:\t%s\r\n", bez);
  fprintf(outfile, "EDA:\t%s\r\n", make_eda_localtime().c_str());
  fprintf(outfile, "EMP:\t%s\r\n", emp);
  fprintf(outfile, "LEN:\t%lu\r\n", len);
  fprintf(outfile, "MAILER:\tDBOXADM/DBOX (C) by Daniel Kroening\r\n");
  fprintf(outfile, "MID:\t%s\r\n", gen_message_id("maps").c_str());

  fprintf(outfile, "ROT:\r\n");
  if(config.organization[0]!=0)
    fprintf(outfile, "ORG:\t%s\r\n", config.organization);

  fprintf(outfile, "\r\n");
 }

static void reply(FILE *outfile, zcmessaget *m, const char *text)
 {
  char bez[TEXTLEN], emp[TEXTLEN], bet[TEXTLEN], tempstr[TEXTLEN],
       *tptr;

  m->copyheadertext(bez, m->mid, TEXTLEN-1);
  m->copyheadertext(emp, m->abs, TEXTLEN-1);
  m->copyheadertext(tempstr, m->bet, TEXTLEN-1);
  tempstr[TEXTLEN-10]=0;
  sprintf(bet, "Re: %s", tempstr);

  tptr=strchr(emp, ' ');
  if(tptr!=NULL) *tptr=0;

  printheader(outfile, bez, emp, bet, strlen(text)+strlen(headertext));
  fwrite(headertext, strlen(headertext), 1, outfile);
  fwrite(text, strlen(text), 1, outfile);
 }

bool gettan(const char *tan)
 {
  FILE *tanfile, *newtanfile;
  char filename[DIRLEN], *tptr, line[TEXTLEN], newfilename[DIRLEN];
  bool found=FALSE;

  sprintf(filename, BOXSYSTEMDIR "dboxadm.tan", config.boxdir);
  tanfile=fopen(filename, "rt");
  if(tanfile==NULL)
   {
    fprintf(stderr, "dboxadm.tan nicht gefunden!\n");
    return TRUE;
   }

  sprintf(newfilename, BOXSYSTEMDIR "dboxadm.tmp", config.boxdir);
  newtanfile=fopen(newfilename, "wt");
  if(newtanfile==NULL)
   {
    fprintf(stderr, "dboxadm.tmp konnte nicht geoeffnet werden!\n");
    fclose(tanfile);
    return TRUE;
   }

  while(fgets(line, TEXTLEN-1, tanfile)!=NULL)
   {
    tptr=strrchr(line, '\r'); if(tptr!=NULL) *tptr=0;
    tptr=strrchr(line, '\n'); if(tptr!=NULL) *tptr=0;
    if(line[0]=='T' && strcmp(tan, line+1)==0)
     {
      found=TRUE;
      fprintf(newtanfile, "* ");
     }

    fprintf(newtanfile, "%s\n", line);
   }

  fclose(tanfile);
  fclose(newtanfile);

  if(found)
   {
    remove(filename);
    rename(newfilename, filename);
    return FALSE;
   }
  else
   {
    remove(newfilename);
    return TRUE; 
   }

 }

void mailauszuege(userpt *up, FILE *outfile, zcmessaget *m)
 {
  FILE *tmpfile, *infile;
  dword len;
  char filename[DIRLEN], 
       tmpfilename[DIRLEN];
  char tempstr[TEXTLEN*2];

  sprintf(tmpfilename, DBOXTEMPDIR "listgen.tmp", config.dboxdir);

  sprintf(filename, BOXUSERDIR "konto", config.boxdir, up->name);
  infile=fopen(filename, "rt");
  if(infile==NULL)
   {
    sprintf(tempstr, "User %s hat keinen Kontoauszug.\r\n", up->name);
    reply(outfile, m, tempstr);
    return;
   }

  tmpfile=fopen(tmpfilename, "wb");
  if(tmpfile==NULL)
    logf.internalerror("mailauszuege", "Temporaerdatei konnte nicht geoeffnet werden");

  currency_string(tempstr, getbalance(up), TRUE);
  fprintf(tmpfile, "Kontoauszug von User...: %s\r\n", up->name);
  fprintf(tmpfile, "aktueller Kontostand...: %s\r\n", tempstr);
  fprintf(tmpfile, "Kontoberzug...........: %s\r\n", noyes1(atoi(*up|"kontoueberzug")));
  fprintf(tmpfile, "\r\n"
    " Nr.  Datum/Zeit   Betrag  Verwendungszweck\r\n"
    ""
    "\r\n");

  formatkonto(infile, tmpfile, 'a', TRUE, IBM);
  fprintf(tmpfile, 
    ""
    "\r\n");

  fclose(infile);
  len=ftell(tmpfile);
  fclose(tmpfile);

   {
    char bez[TEXTLEN], emp[TEXTLEN];
    sprintf(tempstr, "Kontoauszge von User %s", up->name);
    m->copyheadertext(bez, m->mid, TEXTLEN-1);
    m->copyheadertext(emp, m->abs, TEXTLEN-1);
    printheader(outfile, bez, emp, tempstr, len);
   }

  infile=fopen(tmpfilename, "rb");
  filecopy(infile, outfile, len);
  fclose(infile);
  remove(tmpfilename);
 }

void buchung(char *args, userpt *up, FILE *outfile, 
  zcmessaget *m)
 {
  char *tptr;
  char msg[TEXTLEN*3];
  buchungt buchung;
  char betrag[TEXTLEN];

  if(buchung.parse(args))
   {
    reply(outfile, m, "Ungueltige Syntax.\r\n");
    return;
   }

  m->copyheadertext(buchung.bearbeiter, m->abs, TEXTLEN-1);
  tptr=strchr(buchung.bearbeiter, ' ');
  if(tptr!=NULL) *tptr=0;

  if(dobuchung(up, &buchung))
   {
    reply(outfile, m, "Buchung fehlgeschlagen!\r\n");
    return;
   }

  currency_string(betrag, buchung.amount, TRUE);

  sprintf(msg,
    " Konto: %s\r\n"
    "Betrag: %s\r\n"
    "   Typ: %s\r\n"
    "  Text: %s\r\n",
    up->name,
    betrag,
    buchung.haben?"Haben":"Soll",
    buchung.verwendungszweck); 

  reply(outfile, m, msg);
 }

void docommand(char *command, FILE *outfile, zcmessaget *m)
 {
  char *tptr, *tptr2;
  userpt up;

  tptr=strchr(command, ' ');
  if(tptr==NULL)
   {
    char text[TEXTLEN*2];
    sprintf(text, "Ungueltiges Kommando: \"%s\"\r\n", command);
    reply(outfile, m, text);
    return;
   }

  *tptr=0;
  tptr++;
  while(*tptr==' ') tptr++;

  tptr2=strchr(tptr, ' ');
  if(tptr2!=NULL)
   {
    *tptr2=0;
    tptr2++;
    while(*tptr2==' ') tptr2++;
   }

  strmaxcpy(up.name, tptr, TEXTLEN-1);
  dnstring(up.name);
  
  if(up.read())
   {
    char text[TEXTLEN*2];
    sprintf(text, "Ungueltiger Username: \"%s\"\r\n", up.name);
    reply(outfile, m, text);
    return;
   }
 
  if(stricmp(command, "KONTOUEBERZUG-AN")==0 ||
     stricmp(command, "KONTOUEBERZUG-AUS")==0)
   {
    char text[TEXTLEN*2];

    if(!up.lock())
     {
      bool kontoueberzug=(stricmp(command, "KONTOUEBERZUG-AN")==0);
      up.set("kontoueberzug", kontoueberzug);
      up.write();
      up.unlock();
      snprintf(text, TEXTLEN*2, 
               "Der Kontoueberzug von User \"%s\" wurde "
               "%s.\r\n", up.name, kontoueberzug?"freigeschaltet":"gesperrt");
      reply(outfile, m, text);
     }
    else
     {
      fprintf(stderr, "Locking %s failed\n", up.name);
      snprintf(text, TEXTLEN*2, 
               "Fehler: Der Kontoueberzug von User \"%s\" konnte nicht "
               "geaendert werden.\r\n", up.name);
      reply(outfile, m, text);
     }
   }
  else if(stricmp(command, "BUCHUNG")==0)
   {
    buchung(tptr2, &up, outfile, m);
   }
  else if(stricmp(command, "BUCHUNG")==0)
   {
    buchung(tptr2, &up, outfile, m);
   }
  else if(stricmp(command, "AUSZUEGE")==0)
   {
    mailauszuege(&up, outfile, m);
   }
  else
   {
    char text[TEXTLEN*2];
    sprintf(text, "Ungueltiges Kommando: \"%s\"\r\n", command);
    reply(outfile, m, text);
    return;
   }
 }

static void dodboxadmmail(FILE *outfile, zcmessaget *m)
 {
  const char *pptr;
  char tan[TEXTLEN], line[TEXTLEN];

  m->copyheadertext(tan, m->bet, TEXTLEN-1);

  if(gettan(tan))
   {
    reply(outfile, m,
      "Ungueltige TAN!\r\n");
    logf.printf("listgen", "Illegale Tan!");
    return;
   }

  pptr=m->data+m->headerlen;

  while(!getline(m, line, TEXTLEN-1, &pptr))
    if(line[0]!=0)
      docommand(line, outfile, m);
 }

void dodboxadm(int handle, dword laenge)
 {
  bool error=FALSE;
  char *posting=NULL;
  zcmessaget m;
  FILE *outfile;
  char tempstr[DIRLEN];
  dword i=0;

  do
    sprintf(tempstr, BOXSPOOLINZCMAILDIR "%s.listgen-%lu", config.boxdir, 
      config.boxname, i++);
  while(access(tempstr, 00)==0);

  outfile=fopen(tempstr, "ab");
  if(outfile==NULL)
   {
    logf.printf("domaps", "Fatal: Failed to open \"%s\"", tempstr);
    return;
   }

  posting=(char *)malloc(laenge);
  if(posting==NULL)
    logf.internalerror("dodboxadm", "malloc failed");

  if(read(handle, posting, laenge)!=(ssize_t)laenge)
    logf.internalerror("dodboxadm", "read failed");

  m.data=posting;
  m.endoffile=posting+laenge;

  while(!m.eof() && !error)
   {
    if(m.parse())
      error=TRUE;
    else
     {
      dodboxadmmail(outfile, &m);
      m.data=m.endofmail;
     }
   }

  free(posting);
  fclose(outfile);

  if(error)
    logf.printf("listgen", "FATAL: Fehler in der Pufferdatei!");
 }

void dboxadm(const char *folderfile)
 {
  int handle;
  dword laenge;
 
  handle=open(folderfile, O_RDONLY|O_BINARY);

  if(handle<0) return;

  laenge=filelength(handle);

  if(laenge!=0) dodboxadm(handle, laenge);

  remove(folderfile);
 }
