/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <dirent.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <time.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/mman.h>

#include <dbox.h>
#include <tools.h>
#include <folder.h>
#include <folderindex.h>
#include <mtoolzc.h>
#include <spoolp.h>
#include <config.h>
#include <dboxpath.h>
#include <logf.h>
#include <userdb.h>
#include <dboxversion.h>
#include <libdboxversion.h>
#include <lock.h>
#include <oldtools.h>
#include <flags.h>
#include <zconnect.h>
#include <messageid.h>
#include "textdb.h"

char *systempermissions[MAXGRUPPEN];

word midcounter=0;

char headertext[]=
  "Ŀ\r\n"
  "        MAPS - Tool fr Brettbestellungen des DBOX-Mailboxsystems         \r\n"
  "\r\n"
  "\r\n";

FILE *openconfigfile(char *filename)
 {
  char tempstr[DIRLEN];
  FILE *f;

  sprintf(tempstr, BOXSYSTEMDIR "%s", config.boxdir, filename);
  if((f=fopen(tempstr, "rt"))!=NULL) return f;

  sprintf(tempstr, DBOXSYSTEMDIR "%s", config.dboxdir, filename);
  if((f=fopen(tempstr, "rt"))!=NULL) return f;

  return NULL;
 }

class mapst
 {
 public:
  char *mapsdata;
  char command[TEXTLEN];
  userpt up;
  grouppt gp;
  zcmessaget *m;
  FILE *outfile;
  char *args;
  textfiledbt mapslist;
  FILE *mapslistfile;

  bool checkpermission(const char *brett);
  bool checkbestellt(const char *brett);
  bool checksysop(const char *user);
  bool getsystem(const char *abs);
  bool getline(char *line, const char **pptr);
  dword getmapsentrylen();
  void doadd3(const char *brett, FILE *tempfile);
  FILE *openmapslist();
  void doadd2(const char *mask, FILE *tempfile);
  void writemaps();
  void doadd();
  void dohold();
  void dohold(bool hold_off);
  void dohelp();
  void dodel();
  void dolist();
  void doindex();
  void doinhaltindex1(bool index);
  void doinhaltindex2(bool index, const char *mask, FILE *outfile);
  void doinhaltindex3(bool index, folderpt *fp, FILE *outfile);
  void doinhalt();
  void dolesenorder1(bool order);
  void dolesenorder2(bool order, const char *brettname,
                     const char *msgid, FILE *tempfile);
  void dolesenorder3(bool order, folderpt *fp, 
                     const char *msgid, FILE *tempfile);
  void dolesen();
  void doorder();
  void dopmloeschen();
  void dopm();
  void doversion();
  int getcommand(const char *bet);
  void domapsmail();
  void printheader(const char *bet, dword len, const char *abs);
  void reply(const char *text);
  void replybyfile(const char *filename, bool header, const char *newbet=NULL);

  mapst();
  ~mapst();
 };

mapst::mapst()
 {
  mapslistfile=openmapslist();
  if(mapslistfile!=NULL)
    mapslist.read(mapslistfile);
 }

mapst::~mapst()
 { 
  if(mapslistfile!=NULL)
    free(mapslistfile);
 }

void mapst::printheader(const char *bet, dword len, const char *abs)
 {
  char bez[TEXTLEN], emp[TEXTLEN], *tptr;

  if(atoi(up|"system"))
   {
    m->copyheadertext(emp, m->abs, TEXTLEN-1);
    tptr=strchr(emp, ' ');
    if(tptr!=NULL) *tptr=0;
   }
  else
    sprintf(emp, "%s@%s-%s.zer", up.name, config.boxname, up.name);

  m->copyheadertext(bez, m->mid, TEXTLEN-1);

  if(abs==NULL)
    fprintf(outfile, "ABS:\tMAPS@%s.%s\r\n", config.boxname, config.domain);
  else
   {
    fprintf(outfile, "ABS:\t%s\r\n", abs);
    fprintf(outfile, "WAB:\tMAPS@%s.%s\r\n", config.boxname, config.domain);
   }

  fprintf(outfile, "BET:\t%s\r\n", bet);
  fprintf(outfile, "BEZ:\t%s\r\n", bez);
  fprintf(outfile, "EDA:\t%s\r\n", make_eda_localtime().c_str());
  fprintf(outfile, "EMP:\t%s\r\n", emp);
  fprintf(outfile, "LEN:\t%lu\r\n", len);
  fprintf(outfile, "MAILER:\tMAPS/DBOX by Daniel Krning\r\n");
  fprintf(outfile, "MID:\t%s\r\n", gen_message_id("maps").c_str());

  fprintf(outfile, "ROT:\r\n");
  if(config.organization[0]!=0)
    fprintf(outfile, "ORG:\t%s\r\n", config.organization);

  fprintf(outfile, "\r\n");
 }

void mapst::reply(const char *text)
 {
  char bet[TEXTLEN], tempstr[TEXTLEN];

  m->copyheadertext(tempstr, m->bet, TEXTLEN-1);
  tempstr[TEXTLEN-10]=0;
  sprintf(bet, "Your %s", tempstr);
  printheader(bet, strlen(text)+strlen(headertext), NULL);
  fwrite(headertext, strlen(headertext), 1, outfile);
  fwrite(text, strlen(text), 1, outfile);
 }

void mapst::replybyfile(const char *filename,
                        bool header, const char *newbet)
 {
  char bet[TEXTLEN], tempstr[TEXTLEN];
  dword flen, len;
  FILE *infile;

  if(newbet==NULL)
   {
    m->copyheadertext(tempstr, m->bet, TEXTLEN-1);
    tempstr[TEXTLEN-10]=0;
    sprintf(bet, "Your %s", tempstr);
   }
  else
    strmaxcpy(bet, newbet, TEXTLEN-1);

  infile=fopen(filename, "rb");
  if(infile==NULL) return;
  flen=filelength(fileno(infile));
  len=flen;

  if(header)
    len+=strlen(headertext);

  printheader(bet, len, NULL);

  if(header)
    fwrite(headertext, strlen(headertext), 1, outfile);

  filecopy(infile, outfile, flen);

  fclose(infile);
 }

void getpermissions()
 {
  int i;
  char tempstr[DIRLEN];

  for(i=0; i<MAXGRUPPEN; i++)
   {
    systempermissions[i]=NULL;

    if(config.groupdb[i]!=NULL)
     {
      sprintf(tempstr, BOXGRUPPENDIR "bretter", config.boxdir, config.groupdb[i]);
      systempermissions[i]=readtextconfigfile(tempstr);
     }
   }
 }

void freepermissions()
 {
  dword i;

  for(i=0; i<MAXGRUPPEN; i++)
    if(systempermissions[i]!=NULL)
      free(systempermissions[i]);
 }

bool mapst::checkpermission(const char *brett) // FALSE=ok
 {
  const char *perm;
  char empmask[TEXTLEN];
  bool read, write, sysop;
  userflagst userflags;
  int group;

  group=config.groupdb.index(up|"gruppe");

  if(group<0) return TRUE;
  perm=systempermissions[group];
  if(perm==NULL) return TRUE;

  while(*perm!=0)
   {
    if(!::getpermissions(perm, empmask, &read, &write, &sysop, &userflags))
     {
      if(checkjoker(empmask, brett, TRUE))
        if(!checkuserflags(&up, &gp, &userflags))
          return !read;
     }
    perm+=strlen(perm)+1;
   }

  return TRUE;
 }

bool mapst::checkbestellt(const char *brett) // TRUE=bestellt
 {
  char *tptr;

  if(mapsdata==NULL) return FALSE;

  tptr=mapsdata;

  while(*tptr!=0)
   {
    if(stricmp(tptr, brett)==0) return TRUE;
    tptr+=strlen(tptr)+1;
   }

  return FALSE;
 }

bool mapst::checksysop(const char *user)
 {
  char *berechtigte, *tptr;
  char tempstr[TEXTLEN];

  strmaxcpy(tempstr, up|"maps", TEXTLEN-1);
  if(tempstr[0]==0) strmaxcpy(tempstr, "sysop root", TEXTLEN-1);

  berechtigte=tempstr;

  while(berechtigte[0]!=0)
   {
    tptr=berechtigte;
    while(*tptr!=0 && *tptr!=';' && *tptr!=',' && *tptr!=' ')
      tptr++;

    if(*tptr!=0)
     {
      *tptr=0;
      tptr++;
      while(*tptr==';' || *tptr==',' || *tptr==' ')
        tptr++;
     }

    while(*berechtigte==' ') berechtigte++;
    if(stricmp(user, berechtigte)==0) return FALSE;
    berechtigte=tptr;
   }

  return TRUE;
 }

bool mapst::getsystem(const char *abs)
 {
  char tempabs[TEXTLEN], *tptr, user[TEXTLEN], system[TEXTLEN];
  strmaxcpy(tempabs, abs, TEXTLEN-1);
  dnstring(tempabs);

  tptr=strchr(abs, ' ');
  if(tptr!=NULL) *tptr=0;

  tptr=strchr(tempabs, '@');
  if(tptr==NULL) return TRUE;

  *tptr=0;
  strmaxcpy(user, tempabs, TEXTLEN-1);
  strmaxcpy(system, tptr+1, TEXTLEN-1);

  tptr=strchr(system, '.');
  if(tptr==NULL) return TRUE;
  *tptr=0;

  if(stricmp(system, config.boxname)==0)
   {
    if(strchr(user, '/')!=NULL) return TRUE;
    strmaxcpy(up.name, user, TEXTLEN-1);
    dnstring(up.name);
    if(up.read()) return TRUE;

    if((up|"alias")[0]!=0)
     {
      strmaxcpy(up.name, up|"alias", TEXTLEN-1);
      if(up.read()) return TRUE;
     }

    if(atoi(up|"system")) return TRUE;

    strmaxcpy(gp.name, up|"gruppe", TEXTLEN-1);
    if(gp.read()) return TRUE;
    if(!getinternalflag(&up, &gp, LOGIN_NETZ)) return TRUE;
    return FALSE;
   }

  if(strchr(system, '/')!=NULL) return TRUE;
  strmaxcpy(up.name, system, TEXTLEN-1);
  dnstring(up.name);
  if(up.read()) return TRUE;
  //if(!up.system) return TRUE;
  if(checksysop(user)) return TRUE;
  return FALSE;
 }

void mapst::dohelp()
 {
  if(args[0]==0 || stricmp(args, "*")==0)
   {
    reply(
      "Dieses Maps-Tool empfngt seine Befehle ber die Betreffszeile.\r\n"
      "Schreiben Sie keine Befehle in den Nachrichtentext.\r\n"
      "\r\n"
      "Es folgt die Befehlsliste:\r\n"
      "\r\n"
      "  Befehl   Funktion\r\n"
      " \r\n"
      "  ADD      Bretter bestellen\r\n"
      "  DEL      Bretter abbestellen\r\n"
      "  HOLD     \"Urlaubsfunktion\"\r\n"
      "  LIST     verschiedene Listen anfordern\r\n"
      "  HILFE    Hilfe anfordern\r\n"
      "  VERSION  Versionsinformationen anfordern\r\n"
      "  PM       Postfach verwalten\r\n"
      "\r\n"
      "Um genauere Informationen zu einem bestimmten Befehl zu erhalten,\r\n"
      "schicken Sie einfach eine Nachricht an dieses Maps-Tool mit dem Betreff\r\n"
      "\"HILFE Befehl\" (ohne die Anfhrungszeichen).\r\n");
   }
  else if(stricmp(args, "ADD")==0)
   {
    reply(
      "Mit dem Befehl ADD knnen Sie Bretter bestellen. Die Namen der\r\n"
      "Bretter, die Sie bestellen wollen, mssen im Nachrichtentext stehen\r\n"
      "(NICHT im Betreff). Sie knnen auch mehere Bretter auf einmal "
      "bestellen,\r\ndann mssen Sie pro Brettname eine Zeile verwenden.\r\n"
      "\r\n"
      "Sie knnen auch Jokerzeichen verwenden:\r\n"
      " - das ganze Z-Netz: /Z-NETZ/*\r\n"
      " - alle Basic-Bretter: /*BASIC*\r\n");
   }
  else if(stricmp(args, "DEL")==0)
   {
    reply(
      "Mit dem Befehl DEL knnen Sie Bretter abbestellen. Die Namen der\r\n"
      "Bretter, die Sie abbestellen wollen, mssen im Nachrichtentext stehen\r\n"
      "(NICHT im Betreff). Sie knnen auch mehere Bretter auf einmal "
      "abbestellen,\r\ndann mssen Sie pro Brettname eine Zeile verwenden.\r\n"
      "\r\n"
      "Sie knnen auch Jokerzeichen verwenden:\r\n"
      " - das ganze Z-Netz: /Z-NETZ/*\r\n"
      " - alle Basic-Bretter: /*BASIC*\r\n"
      " - alle Bretter: /*\r\n");
   }
  else if(stricmp(args, "LIST")==0)
   {
    reply(
      "Mit dem Befehl LIST knnen Sie verschiedene Listen anfordern.\r\n"
      "\r\n"
      "  Befehl                 Funktion\r\n"
      " \r\n"
      "  LIST BRETTER           Liste der bestellbaren Bretter\r\n"
      "  LIST VERBOSE BRETTER   ausfhrliche Liste der bestellbaren Bretter\r\n"
      "  LIST MY BRETTER        Liste der bestellten Bretter\r\n"
      "  LIST OTHER BRETTER     Liste der nicht bestellten Bretter\r\n");
   }
  else if(stricmp(args, "VERSION")==0)
   {
    reply(
      "Hilfe zum Befehl VERSION\r\n"
      "\r\n"
      "\r\n"
      "Mit diesem Befehl erhalten Sie Informationen zur eingesetzten\r\n"
      "DBOX-Version.\r\n");
   }
  else if(stricmp(args, "HOLD")==0)
   {
    reply(
      "Hilfe zum Befehl HOLD\r\n"
      "\r\n"
      "\r\n"
      "Mit diesem Befehl und dem Parameter ON werden Ihre Brettbestellungen\r\n"
      "gelscht, bis sie wieder mit dem Parameter OFF aktiviert werden.\r\n");
   }
  else if(stricmp(args, "PM")==0)
   {
    reply(
      "Hilfe zum Befehl PM\r\n"
      "\r\n"
      "\r\n"
      "Mit dem Befehl PM LOESCHEN wird Ihr Postfach gelscht.\r\n");
   }
  else
   {
    reply(
      "Zu diesem Befehl ist leider keine Hilfe verfgbar. Sie knnen\r\n"
      "mit dem Betreff \"HILFE\" eine Befehlsliste anfordern.\r\n");
   }
 }

bool mapst::getline(char *line, const char **pptr)
 {
  return ::getline(m, line, TEXTLEN-1, pptr);
 }

dword mapst::getmapsentrylen()
 {
  dword len=1, i;
  char *tptr=mapsdata;

  while(*tptr!=0)
   {
    i=strlen(tptr)+1;
    tptr+=i;
    len+=i;
   }

  return len;
 }

void mapst::doadd3(const char *brett, FILE *tempfile)
 {
  if(checkbestellt(brett))
   {
    fprintf(tempfile,
      "Brett ist bereits bestellt:\r\n"
      "  \"%s\"\r\n\r\n", brett);
    return;
   }

  dword oldlen=getmapsentrylen();
  dword newlen=oldlen+strlen(brett)+1;

  fprintf(tempfile,
    "Brett bestellt:\r\n"
    "  \"%s\"\r\n\r\n", brett);

  mapsdata=(char *)realloc(mapsdata, newlen);
  if(mapsdata==NULL)
    logf.internalerror("mapst::doadd3", "malloc failed");

  strcpy(mapsdata+oldlen-1, brett); // mapsdata is assumed to be large enough
  mapsdata[newlen-1]=0;
 }

FILE *mapst::openmapslist()
 {
  char filename[DIRLEN];
  FILE *infile;

  sprintf(filename, BOXSYSTEMDIR "mapslist", config.boxdir);
  infile=fopen(filename, "rt");
  if(infile!=NULL) return infile;
  
  sprintf(filename, DBOXSYSTEMDIR "mapslist", config.dboxdir);
  infile=fopen(filename, "rt");
  return infile;
 }

void mapst::doadd2(const char *mask, FILE *tempfile)
 {
  bool found=FALSE;

  printf("ADD: %s\n", mask);

  if(strchr(mask, '?')==NULL && strchr(mask, '*')==NULL) // no jokers
   {
    char betreff[TEXTLEN];

    if(!mapslist.get(mask, betreff))
     {
      if(!checkpermission(mask))
       {
        found=TRUE;
        doadd3(mask, tempfile);
       }
     }
   }
  else if(mapslistfile!=NULL)
   {
    char mapsline[TEXTLEN], *tptr;

    fseek(mapslistfile, 0, SEEK_SET);

    while(fgets(mapsline, TEXTLEN-1, mapslistfile)!=NULL)
      if(mapsline[0]=='/')
       {
        tptr=strchr(mapsline, '\r'); if(tptr!=NULL) *tptr=0;
        tptr=strchr(mapsline, '\n'); if(tptr!=NULL) *tptr=0;

        tptr=mapsline;
        while(*tptr!=' ' && *tptr!=0 && *tptr!='\t') tptr++;
        if(*tptr!=0)
         {
          *tptr=0;
          tptr++;
          while(*tptr==' ' || *tptr=='\t') tptr++;
         }

        upstring(mapsline);

        if(checkjoker(mask, mapsline, TRUE))
         {
          if(!checkpermission(mapsline))
           {
            found=TRUE;
            doadd3(mapsline, tempfile);
           }
         }
       }
   }

  if(!found)
   {
    fprintf(tempfile,
      "Keine passenden Bretter gefunden:\r\n"
      "  \"%s\"\r\n\r\n", mask);
   }
 }

void mapst::writemaps()
 {
  FILE *mapsfile;
  char tempstr[DIRLEN];
  char *tptr=mapsdata;

  sprintf(tempstr, BOXUSERDIR "maps", config.boxdir, up.name);
  mapsfile=fopen(tempstr, "wb");
  if(mapsfile!=NULL)
   {
    while(*tptr!=0)
     {
      if(tptr[0]=='/')
        fprintf(mapsfile, "%s\r\n", tptr);

      tptr+=strlen(tptr)+1;
     }

    fclose(mapsfile);
   }
 }

void mapst::doadd()
 {
  char tempfilename[DIRLEN], line[TEXTLEN];
  const char *pptr;
  FILE *tempfile;
  dword oldlen=getmapsentrylen();

  dohold(TRUE);

  if(args!=NULL && args[0]!=0)
   {
    reply("Befehl ADD: Falsche Syntax!\r\n");
    return;
   }

  sprintf(tempfilename, DBOXTEMPDIR "maps.tmp", config.dboxdir);
  tempfile=fopen(tempfilename, "wb");
  if(tempfile==NULL)
    logf.internalerror("mapst::doadd", "Temporaerdatei konnte nicht geoeffnet werden");

  fprintf(tempfile, "DBox-Maps-Logfile:\r\n\r\n");

  pptr=m->data+m->headerlen;
  while(!getline(line, &pptr))
    if(line[0]!=0)
     {
      upstring(line);
      doadd2(line, tempfile);
     }

  fclose(tempfile);
  replybyfile(tempfilename, TRUE);
  remove(tempfilename);

  if(getmapsentrylen()!=oldlen)
    writemaps();
 }

void mapst::dodel()
 {
  char tempfilename[DIRLEN], line[TEXTLEN], *tptr;
  const char *pptr;
  FILE *tempfile;
  bool changed=FALSE, found;

  dohold(TRUE);

  if(args!=NULL && args[0]!=0)
   {
    reply("Befehl DEL: Falsche Syntax!\r\n");
    return;
   }

  sprintf(tempfilename, DBOXTEMPDIR "maps.tmp", config.dboxdir);
  tempfile=fopen(tempfilename, "wb");
  if(tempfile==NULL)
    logf.internalerror("mapst::dodel", "Temporaerdatei konnte nicht geoeffnet werden");

  fprintf(tempfile, "DBox-Maps-Logfile:\r\n\r\n");

  pptr=m->data+m->headerlen;
  while(!getline(line, &pptr))
    if(line[0]!=0)
     {
      upstring(line);

      printf("DEL: %s\n", line);

      found=FALSE;

      if(mapsdata!=NULL)
       {
        tptr=mapsdata;

        while(*tptr!=0)
         {
          if(checkjoker(line, tptr, TRUE))
           {
            fprintf(tempfile,
              "Brett abbestellt:\r\n"
              "  \"%s\"\r\n\r\n", tptr);

            tptr[0]='-';
            found=TRUE;
            changed=TRUE;
           }
          tptr+=strlen(tptr)+1;
         }
       }

      if(!found)
       {
        fprintf(tempfile,
          "Brett nicht gefunden oder nicht bestellt:\r\n"
          "  \"%s\"\r\n\r\n", line);
       }
     }

  if(changed)
    writemaps();

  fclose(tempfile);
  replybyfile(tempfilename, TRUE);
  remove(tempfilename);
 }

void mapst::dolist()
 {
  bool verbose, my, other, bretter, systeme, header=FALSE;
  bool bestellt, listit;
  FILE *tempfile;
  char tempfilename[DIRLEN], tempstr[DIRLEN], *tptr;

  if(args==NULL || args[0]==0)
   {
   syntax:
    reply("Befehl LIST: Falsche Syntax!\r\n");
    return;
   }

  verbose=(strstr(args, "VERBOSE")!=NULL);
  my=(strstr(args, "MY")!=NULL);
  other=(strstr(args, "OTHER")!=NULL);
  bretter=(strstr(args, "BRETTER")!=NULL);
  systeme=(strstr(args, "SYSTEME")!=NULL);

  if(my && other) goto syntax;
  if(bretter && systeme) goto syntax;
  if(!bretter && !systeme) goto syntax;

  sprintf(tempfilename, DBOXTEMPDIR "maps.tmp", config.dboxdir);

  if(bretter)
   {
    tempfile=fopen(tempfilename, "wb");
    if(tempfile==NULL)
      logf.internalerror("mapst::dolist", "Temporaerdatei konnte nicht geoeffnet werden");

    if(mapslistfile!=NULL)
     {
      fseek(mapslistfile, 0, SEEK_SET);

      while(fgets(tempstr, TEXTLEN-1, mapslistfile)!=NULL)
        if(tempstr[0]=='/')
         {
          tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;
          tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;

          tptr=tempstr;
          while(*tptr!=' ' && *tptr!=0 && *tptr!='\t') tptr++;
          if(*tptr!=0)
           {
            *tptr=0;
            tptr++;
            while(*tptr==' ' || *tptr=='\t') tptr++;
           }

          upstring(tempstr);

          bestellt=checkbestellt(tempstr);
          listit=FALSE;

          if(my)
           {
            if(bestellt) listit=TRUE;
           }            
          else if(other)
           {
            if(!bestellt) listit=TRUE;
           }
          else
            listit=TRUE;

          if(listit && !checkpermission(tempstr))
           {
            if(verbose)
              fprintf(tempfile, "%c %-40s %s\r\n",
                bestellt?'J':'N', tempstr, tptr);
            else
              fprintf(tempfile, "%s\r\n", tempstr);
           }
         }
     }

    if(ftell(tempfile)==0)
     {
      fprintf(tempfile,
        "Es wurden leider keine passenden Bretter gefunden!\r\n");
      header=TRUE;
     }

    char *bet;

    if(my)
      bet="YOUR BRETTER";
    else if(other)
      bet="OTHER BRETTER";
    else
      bet="BRETTER";

    fclose(tempfile);
    replybyfile(tempfilename, header, bet);
    remove(tempfilename);
    return;
   }

  if(systeme)
   {
    reply("Dieser Befehl ist z.Zt. leider noch nicht verfgbar.\r\n");
    return;
   }
 }

time_t parse_ab(const char *s)
 {
  struct tm tm;

  tm.tm_sec=0;
  tm.tm_min=0;
  tm.tm_hour=0;
  tm.tm_wday=0;
  tm.tm_yday=0;
  tm.tm_isdst=0;

  sscanf(s, "%ud.%ud.%ud", 
    &tm.tm_mday, &tm.tm_mon, &tm.tm_year);

  tm.tm_mon--;

  if(tm.tm_year>1000)
    tm.tm_year-=1900;
  else if(tm.tm_year<50)
    tm.tm_year+=100;

  return mktime(&tm);
 }

void mapst::doinhaltindex3(bool index, folderpt *fp, FILE *outfile)
 {
  folderindext fi;
  int handle;
  FILE *infile;
  char filename[DIRLEN], tempstr[TEXTLEN];
  time_t ab_datum=0;
  bool verbose=FALSE;
  dword nr;

  if(!index && args!=NULL)
   {
    stringlistt arglist;
    int i;

    arglist.parsestring(args);

    for(i=0; arglist.strings[i]!=NULL; i++)
     {
      if(stricmp(arglist.strings[i], "VERBOSE")==0)
        verbose=TRUE;
      else if(stricmp(arglist.strings[i], "AB")==0)
       {
        if(arglist.strings[i+1]!=NULL)
         {
          i++;
          strmaxcpy(tempstr, arglist.strings[i], TEXTLEN-1);
          ab_datum=parse_ab(tempstr);
         }
       }
     }
   }

  printf("DO%s: %s\n", index?"INDEX":"INHALT", fp->name);

  logf.printf("doinhaltindex3", 
    "%s: DO%s %s", up.name, index?"INDEX":"INHALT", fp->name);

  if(!index)
   {
    strmaxcpy(tempstr, fp->name, TEXTLEN-1);
    upstring(tempstr);

    fprintf(outfile, 
      "Brett            : %s\r\n"
      "Betrifft         : %s\r\n"
      "Betreuung durch  : %s\r\n"
      "Routgrenze       : \r\n"
      "neueste Mail     : \r\n" // 14.11.97 04:35:33
      "\r\n"
      "\r\n",
      tempstr, *fp|"betreff", *fp|"betreuer");
   }

  sprintf(filename, "%sindex", fp->getpath());
  handle=open(filename, O_RDONLY|O_BINARY);
  if(handle<0) return;

  sprintf(filename, "%sheader", fp->getpath());
  infile=fopen(filename, "rb");
  if(infile==NULL) { close(handle); return; }

  nr=0;

  while(read(handle, &fi, sizeof fi)==sizeof fi)
   {
    nr++;

    if(!fi.flags.deleted && fi.received>=ab_datum)
     {
      if(index)
       {
        fseek(infile, fi.headeroffs, SEEK_SET);
        filecopy(infile, outfile, fi.headerlen);
        fprintf(outfile, "\r\n\r\n");
       }
      else
       {
        if(verbose)
         {
          char eda[TEXTLEN];

          makeeda(fi.eda, eda);

          fprintf(outfile, 
            "Nachricht Nummer      : %lu\r\n"
            "Absender              : %s\r\n"
            "Betreff               : %s\r\n"
            "Erstellungsdatum      : %s\r\n"
            "Message ID            : %s\r\n"
            "Lnge in Bytes        : %lu\r\n"
            "\r\n",
            nr, fi.abs, fi.bet, eda, fi.mid, fi.len);
         }
        else
         {
          fprintf(outfile, 
            "%lu %lu %s %s\r\n", nr, fi.len, fi.abs, fi.bet);
         }
       }
     }
   }

  close(handle);
  fclose(infile);
 }

void mapst::doinhaltindex2(bool index, const char *mask, FILE *outfile)
 {
  char mapsline[TEXTLEN], *tptr;
  FILE *infile;

  infile=openmapslist();
  if(infile!=NULL)
   {
    while(fgets(mapsline, TEXTLEN-1, infile)!=NULL)
      if(mapsline[0]=='/' && strchr(mapsline, '@')==NULL)
       {
        tptr=strchr(mapsline, '\r'); if(tptr!=NULL) *tptr=0;
        tptr=strchr(mapsline, '\n'); if(tptr!=NULL) *tptr=0;

        tptr=mapsline;
        while(*tptr!=' ' && *tptr!=0 && *tptr!='\t') tptr++;
        if(*tptr!=0)
         {
          *tptr=0;
          tptr++;
          while(*tptr==' ' || *tptr=='\t') tptr++;
         }

        upstring(mapsline);

        if(checkjoker(mask, mapsline, TRUE))
         {
          if(!checkpermission(mapsline))
           {
            folderpt fp;

            strmaxcpy(fp.name, mapsline, TEXTLEN-1);
            dnstring(fp.name);
            if(!fp.read())
              doinhaltindex3(index, &fp, outfile);
           }
         }
       }

    fclose(infile);
   }
 }

void mapst::doinhaltindex1(bool index)
 {
  char tempfilename[DIRLEN], line[TEXTLEN];
  const char *pptr;
  FILE *tempfile;

  if(args!=NULL && args[0]!=0)
   {
    if(index)
     {
      reply("Befehl INDEX: Falsche Syntax!\r\n");
      return;
     }
    else
     {
     }
   }

  sprintf(tempfilename, DBOXTEMPDIR "maps.tmp", config.dboxdir);
  tempfile=fopen(tempfilename, "wb");
  if(tempfile==NULL)
    logf.internalerror("mapst::doinhaltindex1", "Temporaerdatei konnte nicht geoeffnet werden");

  pptr=m->data+m->headerlen;
  while(!getline(line, &pptr))
    if(line[0]!=0)
     {
      upstring(line);

      if(!index)
        fprintf(tempfile, "Brettmaske:  %s\r\n\r\n", line);

      doinhaltindex2(index, line, tempfile);

      if(!index)
        fprintf(tempfile, 
          "\r\n"
          "------------------------------------------------------------------------------\r\n"
          "\r\n");
     }

  fclose(tempfile);
  replybyfile(tempfilename, TRUE);
  remove(tempfilename);
 }

void mapst::doinhalt()
 {
  doinhaltindex1(FALSE);
 }

void mapst::doindex()
 {
  doinhaltindex1(TRUE);
 }

void mapst::dolesenorder3(bool order, folderpt *fp, 
                          const char *msgid, FILE *tempfile)
 {
  folderindext fi;
  int handle;
  FILE *infile, *infile2;
  char filename[DIRLEN];
  dword nr;

  printf("DO%s: %s\n", order?"ORDER":"LESEN", fp->name);

  logf.printf("dolesenorder3",
    "%s: DO%s %s", up.name, order?"ORDER":"LESEN", fp->name);

  sprintf(filename, "%sindex", fp->getpath());
  handle=open(filename, O_RDONLY|O_BINARY);
  if(handle<0) return;

  sprintf(filename, "%sheader", fp->getpath());
  infile=fopen(filename, "rb");
  if(infile==NULL) { close(handle); return; }

  nr=0;

  while(read(handle, &fi, sizeof fi)==sizeof fi)
   {
    nr++;

    if(!fi.flags.deleted && strcmp(msgid, fi.mid)==0)
     {
      logf.printf("dolesenorder3", 
        "%s: Versende Nachricht aus %s", up.name, fp->name);

      if(fi.flags.bodyextern)
       {
        sprintf(filename, "%s%s", fp->getpath(), fi.file);
        infile2=fopen(filename, "rb");
        if(infile2==NULL)
         {
          close(handle);
          fclose(infile);
          return;
         }
       }
      else
        infile2=NULL;

      if(order)
       {
        fseek(infile, fi.headeroffs, SEEK_SET);

        if(fi.flags.bodyextern)
         {
          filecopy(infile, tempfile, fi.headerlen);
          filecopy(infile2, tempfile, fi.len);
         }
        else
          filecopy(infile, tempfile, fi.headerlen+fi.len);
       }
      else
       {
        fprintf(tempfile, 
          "Nachricht Nummer      : %lu\r\n"
          "Absender              : %s\r\n"
          "Betreff               : %s\r\n"
          "Message ID            : %s\r\n"
          "Lnge in Bytes        : %lu\r\n"
          "\r\n",
          nr, fi.abs, fi.bet, fi.mid, fi.len);

        if(fi.flags.binary)
          fprintf(outfile, "TYP:\tBIN\r\n");

        printheader(fi.bet, fi.len, fi.abs);

        if(fi.flags.bodyextern)
          filecopy(infile2, outfile, fi.len);
        else
         {
          fseek(infile, fi.headeroffs+fi.headerlen, SEEK_SET);
          filecopy(infile, outfile, fi.len);
         }
       }
     }
   }

  close(handle);
  fclose(infile);
 }

void mapst::dolesenorder2(bool order, const char *brettname,
                          const char *msgid, FILE *tempfile)
 {
  if(!checkpermission(brettname))
   {
    folderpt fp;

    strmaxcpy(fp.name, brettname, TEXTLEN-1);
    dnstring(fp.name);
    if(!fp.read())
     {
      dolesenorder3(order, &fp, msgid, tempfile);
     }
    else
     {
      if(!order)
       {
        fprintf(tempfile, "Brett %s nicht gefunden!\r\n", brettname);
       }
     }
   }
  else
   {
    if(!order)
     {
      fprintf(tempfile, "Zugriff auf %s verweigert!\r\n", brettname);
     }
   }
 }

void mapst::dolesenorder1(bool order)
 {
  char tempfilename[DIRLEN], line[TEXTLEN], *tptr;
  const char *pptr;
  char brettname[TEXTLEN]="";
  FILE *tempfile;

  sprintf(tempfilename, DBOXTEMPDIR "maps.tmp", config.dboxdir);
  tempfile=fopen(tempfilename, "wb");
  if(tempfile==NULL)
    logf.internalerror("mapst::doinhaltindex1", "Temporaerdatei konnte nicht geoeffnet werden");

  pptr=m->data+m->headerlen;
  while(!getline(line, &pptr))
    if(line[0]!=0)
     {
      tptr=strchr(line, ' ');
      if(tptr!=NULL)
       {
        *tptr=0;
        tptr++;
        while(*tptr==' ') tptr++;

        if(order)
         {
          dolesenorder2(order, line, tptr, tempfile);
         }
        else
         {
          if(stricmp(line, "BRETT")==0)
           {
            strmaxcpy(brettname, tptr, TEXTLEN-1);
            fprintf(tempfile, "Brett: %s\r\n", brettname);
           }
          else if(stricmp(line, "ID")==0)
           {
            dolesenorder2(order, brettname, tptr, tempfile);
           }
         }
       }

      if(!order)
        fprintf(tempfile, 
          "\r\n"
          "------------------------------------------------------------------------------\r\n"
          "\r\n");
     }

  fclose(tempfile);
  replybyfile(tempfilename, !order);
  remove(tempfilename);
 }

void mapst::dolesen()
 {
  dolesenorder1(FALSE);
 }

void mapst::doorder()
 {
  dolesenorder1(TRUE);
 }

void mapst::dopmloeschen()
 {
  folderindext fi;
  int index;
  dword pos=0;
  char filename[DIRLEN];

  reply("Ihr Postfach wurde gelscht.\r\n");

  sprintf(filename, BOXUSERDIR "mail" DIRCHAR "index", 
    config.boxdir, up.name);

  index=::open(filename, O_RDWR|O_BINARY);

  while(::read(index, &fi, sizeof fi)==sizeof fi)
   {
    if(!fi.flags.deleted)
     {
      lseek(index, pos*sizeof fi, SEEK_SET);
      fi.flags.deleted=TRUE;
      ::write(index, &fi, sizeof fi);
     }

    pos++;
   }

  close(index);

   {
    char filename[DIRLEN];
    FILE *outfile;

    sprintf(filename, BOXTODODIR "crunch.%s", config.boxdir,
      up.name);

    outfile=fopen(filename, "wt");
    if(outfile!=NULL) fclose(outfile);
   }

 }

void mapst::dopm()
 {
  if(args==NULL) goto syntax;

  if(stricmp(command, "CLEARPM")==0)
   {
    dopmloeschen();
    return;
   }

  if(stricmp(args, "INHALT")==0)
   {
    reply("Dieser Befehl ist z.Zt. leider noch nicht verfgbar.\r\n");
    return;
   }

  if(stricmp(args, "LOESCHEN")==0 || stricmp(args, "LSCHEN")==0)
   {
    dopmloeschen();
    return;
   }

 syntax:
  reply("Befehl PM: Falsche Syntax!\r\n");
 }

void mapst::doversion()
 {
  if(args!=NULL && args[0]!=0)
   {
    reply("Befehl VERSION: Falsche Syntax!\r\n");
    return;
   }

  char text[1000];
  sprintf(text, "Dies ist die DBOX Version " DBOXVERSIONNR " vom " DBOXVERSION ".\r\n");
  reply(text);
 }

#define DOHELP 1
#define DOADD 2
#define DODEL 3
#define DOLIST 4
#define DOVERSION 5
#define DOINDEX 6
#define DOINHALT 7
#define DOLESEN 8
#define DOORDER 9
#define DOPM 10
#define DOHOLD 11

struct
 {
  char *command;
  byte commandnr;
 } commandtable[]=
 {
   { "HILFE", DOHELP },
   { "HELP", DOHELP },
   { "ADD", DOADD },
   { "HOLD", DOHOLD },
   { "BESTELLEN", DOADD },
   { "DEL", DODEL },
   { "ABBESTELLEN", DODEL },
   { "LIST", DOLIST },
   { "VERSION", DOVERSION },
   { "INHALT", DOINHALT },
   { "INDEX", DOINDEX },
   { "LESEN", DOLESEN },
   { "ORDER", DOORDER },
   { "PM", DOPM },
   { "CLEARPM", DOPM },
   { NULL, 0 }
 };

int mapst::getcommand(const char *bet)
 {
  char tempstr[TEXTLEN], *tptr;
  byte erg=0;

  tptr=tempstr;

  while(*bet!=0 && *bet!='\t' && *bet!=' ')
   {
    *tptr=*bet;
    tptr++;
    bet++;
   }

  *tptr=0;
  if(*bet!=0)
   {
    while(*bet=='\t' || *bet==' ') bet++;
   }

  args=(char *)bet;

  while(commandtable[erg].command!=NULL)
   {
    if(stricmp(commandtable[erg].command, tempstr)==0)
      return erg;

    erg++;
   }

  return -1;
 }

void mapst::dohold(bool hold_off)
 {
  char tempfilename1[DIRLEN], tempfilename2[DIRLEN];

  if(!hold_off && args==NULL)
   {
    reply("Befehl HOLD: Kein Parameter!\r\n");
    return;
   }

  sprintf(tempfilename1, BOXUSERDIR "maps", config.boxdir, up.name);
  sprintf(tempfilename2, BOXUSERDIR "mapshold", config.boxdir, up.name);

  if(hold_off || stricmp(args, "OFF")==0)
   {
    if(access(tempfilename2, 00)!=0)
     {
      if(!hold_off)
        reply("HOLD-Funktion ist bereits deaktiviert!\r\n");

      return;
     }
    else
     {
      unlink(tempfilename1);      
      rename(tempfilename2, tempfilename1);

      if(!hold_off)
        reply("HOLD-Funktion deaktiviert!\r\n");

      return;
     }
   }
  else if(stricmp(args, "ON")==0)
   {
    if(access(tempfilename2, 00)==0)
     {
      reply("HOLD-Funktion ist bereits aktiviert!\r\n");
      return;
     }
    else
     {
      FILE *outfile;
 
      rename(tempfilename1, tempfilename2);
      outfile=fopen(tempfilename1, "w");
      if(outfile!=NULL) fclose(outfile);

      reply("HOLD-Funktion aktiviert!\r\n");

      return;
     }
   }
  else
   {
    reply("Befehl HOLD: Falsche Syntax!\r\n");
    return;
   }
 }

void mapst::dohold()
 {
  dohold(FALSE);
 }

void mapst::domapsmail()
 {
  char abs[TEXTLEN], *tptr;
  int commandnr;

  m->copyheadertext(abs, m->abs, TEXTLEN-1);
  tptr=strchr(abs, ' ');
  if(tptr!=NULL) *tptr=0;

  m->copyheadertext(command, m->bet, TEXTLEN-1);
  upstring(command);

  if(getsystem(abs))
   {
    logf.printf("domapsmail", "Illegaler Absender \"%s\"", abs);

    tptr=strchr(abs, '@');
    if(tptr==NULL) return;
    *tptr=0;
    
    if(stricmp(abs, "mailer-daemon")==0 ||
       stricmp(abs, "maps")==0)
      return;

    reply("Sie haben leider keine Berechtigung, das Maps-Tool dieses "
      "Mailboxsystems\r\nzu benutzen. Bitte wenden Sie sich an Ihre "
      "Systembetreuung!\r\n");

    return;
   }

  commandnr=getcommand(command);
  if(commandnr<0)
   {
    reply("Das von Ihnen verwendete Kommando ist diesem Maps-Tool leider "
      "unbekannt.\r\nSie knnen mit dem Betreff \"HILFE\" eine Hilfeseite "
      "anfordern.\r\n");

    logf.printf("domapsmail", "Unbekanntes Kommando \"%s\" von \"%s\"", 
      command, up.name);
    return;
   }

  printf("%s: Bearbeite Kommando \"%s\"...\n", up.name, command);
  logf.printf("domapsmail", "%s: Bearbeite Kommando \"%s\"", up.name, command);

   {
    char tempstr[TEXTLEN];
    sprintf(tempstr, BOXUSERDIR "maps", config.boxdir, up.name);
    mapsdata=readtextconfigfile(tempstr);
    if(mapsdata==NULL)
     {
      mapsdata=(char *)malloc(1);
      if(mapsdata==NULL)
        logf.internalerror("mapst::domapsmail", "malloc failed");

      mapsdata[0]=0;
     }
   }

  switch(commandtable[commandnr].commandnr)
   {
   case DOHELP: dohelp(); break;
   case DOADD: doadd(); break;
   case DODEL: dodel(); break;
   case DOLIST: dolist(); break;
   case DOVERSION: doversion(); break;
   case DOINDEX: doindex(); break;
   case DOINHALT: doinhalt(); break;
   case DOLESEN: dolesen(); break;
   case DOORDER: doorder(); break;
   case DOPM: dopm(); break;
   case DOHOLD: dohold(); break;
   default: dohelp(); break;
   }

  free(mapsdata);
 }

static void domaps(int handle, dword laenge)
 {
  bool mapped=FALSE, error=FALSE;
  char *posting=NULL;
  zcmessaget m;
  FILE *outfile;
  char outfilename[DIRLEN];
  mapst maps;

  sprintf(outfilename, DBOXTEMPDIR "maps.%d", config.dboxdir, getpid());
  outfile=fopen(outfilename, "ab");
  if(outfile==NULL)
   {
    logf.printf("domaps", "Fatal: Failed to open \"%s\"", outfilename);
    return;
   }

  if(laenge>60000)
   {
    posting=(char *)mmap(NULL, laenge, PROT_READ, MAP_FILE|MAP_PRIVATE, handle, 0);
    mapped=TRUE;
    if(posting==NULL)
     {
      logf.printf("domaps", "Fatal: mmap failed!");
      return;
     }
   }

  if(posting==NULL)
   {
    posting=(char *)malloc(laenge);
    if(posting==NULL)
      logf.internalerror("domaps", "malloc failed");

    if(read(handle, posting, laenge)!=(ssize_t)laenge)
      logf.internalerror("domaps", "read failed");
   }

  m.data=posting;
  m.endoffile=posting+laenge;

  while(!m.eof() && !error)
   {
    if(m.parse())
      error=TRUE;
    else
     {
      maps.outfile=outfile;
      maps.m=&m;
      maps.domapsmail();
      m.data=m.endofmail;
     }
   }

  if(mapped)
    munmap(posting, laenge);
  else
    free(posting);

  fclose(outfile);

  if(error)
    logf.printf("domaps", "FATAL: Fehler in der Pufferdatei!");

  spool_zc_puffer_mail(outfilename, "maps");
 }

void maps()
 {
  char filename[DIRLEN];
  dword laenge;
  int handle;

  if(lock("puffer.prv", "maps", config.boxname)) 
   {
    printf("maps.prv is locked...\r\n");
    return;
   }

  sprintf(filename, BOXSPOOLDIR "maps.prv", config.boxdir, "maps");

  handle=open(filename, O_RDONLY|O_BINARY);

  if(handle>=0)
   {
    laenge=filelength(handle);
    if(laenge!=0) domaps(handle, laenge);
    close(handle);
   }

  remove(filename);

  unlock("puffer.prv", "maps", config.boxname);
 }

