/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <dirent.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <time.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/mman.h>

#include <dbox.h>
#include <logf.h>
#include "textdb.h"

void textfiledbt::read(FILE *infile)
 {
  struct stat stat;
  dword len, i, j, line;
  bool newline;

  fstat(fileno(infile), &stat);

  len=stat.st_size;

  file=(char *)malloc(len);
  if(file==NULL)
    logf.internalerror("textfiledbt::read", "malloc failed");

  fseek(infile, 0, SEEK_SET);

  fread(file, len, 1, infile);

  anzlines=i=0;

  while(i<len)
   {
    if(file[i]=='\n') anzlines++;
    i++;
   }

  if(anzlines>0)
   {
    index=(char **)malloc(sizeof(char *)*anzlines);
    if(index==NULL)
      logf.internalerror("textfiledbt::read", "malloc failed");

    line=i=0;
    newline=TRUE;

    while(i<len)
     {
      if(newline)
       {
        index[line++]=file+i;
        newline=FALSE;
       }

      if(file[i]=='\n')
       {
        newline=TRUE;
        file[i]=0;
       }

      i++;
     }
   }

  hash_next=(dword *)malloc(anzlines*sizeof(dword));

  for(i=0; i<TEXTDBHASHENTRIES; i++)
    hash[i]=HASH_NIL;

  for(i=0; i<anzlines; i++)
   {
    char key[TEXTLEN], data[TEXTLEN];
    dword hashvalue;

    hash_next[i]=HASH_NIL;

    get(i, key, data);
    hashvalue=(str2icrc(key)&TEXTDBHASHMASK);

    if(hash[hashvalue]==HASH_NIL)
      hash[hashvalue]=i;
    else
     {
      j=hash[hashvalue];
      while(hash_next[j]!=HASH_NIL) j=hash_next[j];
      hash_next[j]=i;
     }
   }
 }

textfiledbt::textfiledbt()
 {
  file=NULL;
  index=NULL;
  anzlines=0;
  hash_next=NULL;
 }

textfiledbt::~textfiledbt()
 {
  if(file!=NULL) free(file);
  if(index!=NULL) free(index);
  if(hash_next!=NULL) free(hash_next);
 }

void textfiledbt::get(dword line, char *key, char *data)
 {
  char *tptr, tmpline[1000];

  key[0]=data[0]=0;

  if(line>=anzlines) return;

  strmaxcpy(tmpline, index[line], 900);
  tptr=strchr(tmpline, '\r'); if(tptr!=NULL) *tptr=0;
  tptr=strchr(tmpline, '\n'); if(tptr!=NULL) *tptr=0;

  tptr=tmpline;
  while(*tptr!=' ' && *tptr!='\t' && *tptr!=0) tptr++;
  if(*tptr==0)
    tptr=NULL;
  else
    *tptr=0;

  strmaxcpy(key, tmpline, TEXTLEN-1);

  if(tptr!=NULL)
   {
    tptr++;
    while(*tptr==' ' || *tptr=='\t') tptr++;
    strmaxcpy(data, tptr, TEXTLEN-1);
   }
 }

bool textfiledbt::get(const char *key, char *data)
 {
  dword i;
  char tmpkey[TEXTLEN], tmpdata[TEXTLEN];

  i=(str2icrc(key)&TEXTDBHASHMASK);

  i=hash[i];

  while(i!=HASH_NIL)
   {
    get(i, tmpkey, tmpdata);
    if(stricmp(tmpkey, key)==0)
     {
      strmaxcpy(data, tmpdata, TEXTLEN-1);
      return FALSE;
     }

    i=hash_next[i];
   }

  return TRUE;
 }

