/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <dirent.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <time.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/mman.h>

#include <dboxpath.h>
#include <dbox.h>
#include <tools.h>
#include <config.h>
#include <userdb.h>
#include <logf.h>
#include <cmdline.h>
#include <dboxversion.h>
#include <libdboxversion.h>

void docrunch()
 {
  DIR *dir;
  struct dirent *de;
  char command[TEXTLEN*2], tempstr[DIRLEN];

  sprintf(tempstr, BOXTODODIR ".", config.boxdir);
  dir=opendir(tempstr);
  if(dir==NULL) return;

  while((de=readdir(dir))!=NULL)
   {
    if(strncmp(de->d_name, "crunch.", 7)==0)
     {
      printf("Crunche Postfach von User %s: ", de->d_name+7);
      fflush(stdout);

      sprintf(command, "cd " BOXUSERDIR "mail", config.boxdir, de->d_name+7);
      sprintf(command+strlen(command), " ; " DBOXBINDIR "onews -f", config.dboxdir);

      system(command);

      sprintf(command, BOXTODODIR "%s", config.boxdir, de->d_name);
      unlink(command);
     }
   }

  closedir(dir);
 }

void expiremailbox(userpt *up, cmdlinet *cmdline)
 {
  char command[TEXTLEN*10];
  char *passargs="tsnq";
  
  printf("Crunche Postfach von User %s: ", up->name);
  fflush(stdout);

  sprintf(command, "cd " BOXUSERDIR "mail", config.boxdir, up->name);
  sprintf(command+strlen(command), " ; " DBOXBINDIR "onews -f", config.dboxdir);

  for(int i=0; passargs[i]!=0; i++)
    if(cmdline->isset(passargs[i]))
      sprintf(command+strlen(command), " -%c \"%s\"",
        passargs[i], cmdline->getval(passargs[i]));

  system(command);
 }

void doexpire(cmdlinet *cmdline)
 {
  char username[TEXTLEN];

  while(fgets(username, TEXTLEN-1, stdin)!=NULL)
    if(username[0]!=0 && username[0]!='.' && username[0]!='_')
     {
      char *tptr;

      tptr=strchr(username, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(username, '\n'); if(tptr!=NULL) *tptr=0;

      username[TEXTLEN-1]=0;
      tptr=strrchr(username, '/');
      if(tptr!=NULL) *tptr=0;
      dnstring(username);
     
      userpt up(username);

      if(up.read())
        fprintf(stderr, "Not found: %s\n", username);
      else
        expiremailbox(&up, cmdline);
     }
 }

static bool parseoptions(int argc, char **argv)
 {
  cmdlinet cmdline;
  char boxname[TEXTLEN]="";
  const char *val;

  if(cmdline.parse(argc, argv, "?hb:acp:t:n:rs:eq:"))
    goto aufruf;

  val=cmdline.getval('b');
  if(val!=NULL) strmaxcpy(boxname, val, TEXTLEN-1);

  if(cmdline.isset('?') || cmdline.isset('h'))
    goto help;

  if(config.read(boxname))
   {
   }
  else
   {

     {
      char logfilename[DIRLEN];
      sprintf(logfilename, BOXLOGDIR "omail.log", config.boxdir);
      logf.setlogfile(logfilename);
     }

    if(cmdline.isset('c'))
     {
      docrunch();
     }
    else if(cmdline.isset('e'))
     {
      doexpire(&cmdline);
     }
    else
      goto help;
   }

  return FALSE;

 aufruf:
  fprintf(stderr, "Falscher Aufruf!\n");

 help:
  fprintf(stderr,
    "\n"
    "* * OMAIL - DBOX Module for Mailbox Management - Version " 
    DBOXVERSIONNR " * *\n"
    "\n"
    "Usage:\n"
    "\n"
    " omail [-?] [-h]              print this help text\n"
    " omail -c                     crunch mailboxes\n"
    " omail -e options             expire mail in mailboxes (stdin)\n"
    "\n"
    "Additional options:\n"
    "  -b   bbs name\n"
    "\n"
    "See also onews -h for expire options.\n"
    "\n"
   );

  return TRUE;
 }

int main(int argc, char **argv)
 {
  if(init_libdbox(LIBDBOXVERSION, LIBDBOXVERSIONNR)) return 1;

  parseoptions(argc, argv);

  return 0;
 }
