/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <dirent.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <time.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <unistd.h>

#include <dbox.h>
#include <tools.h>
#include <folder.h>
#include <folderindex.h>
#include <mtoolzc.h>
#include <logf.h>
#include <config.h>
#include <netdefs.h>
#include <dboxpath.h>
#include <cmdline.h>
#include <dboxversion.h>
#include <libdboxversion.h>
#include <sfolderzc.h>
#include <spoolf.t>

class spool_onewst:public spoolft<zcmessaget>
 {
 public:
  void cancelarticle(folderpt *fp, const char *msgid);
  void dospoolarticle(const char *folder);
  virtual void spoolf3();
 };

void spool_onewst::cancelarticle(folderpt *fp, const char *msgid)
 {
  folderindext fi;
  int handle;
  char filename[DIRLEN];
  char tempstr[TEXTLEN], mid[TEXTLEN];
  dword i;
  const char *tptr, *tptr2, *tptr3;
  char *tptr4;

  printf("  Cancel: %s\n", msgid);

  tptr3=msgid;
  if(tptr3[0]=='<') tptr3++;
  strmaxcpy(mid, tptr3, TEXTLEN-1);
  tptr4=strrchr(mid, '>');
  if(tptr4!=NULL) *tptr4=0;

  tptr=m.data;
  if(!m.findheader(&tptr, "U-Approved", &tptr2))
   {
    m.copyheadertext(tempstr, tptr2, TEXTLEN-1);
    printf("    -> Approved: %s\n", tempstr);
   }

  sprintf(filename, "%sindex", fp->getpath());

  handle=open(filename, O_RDWR|O_BINARY);
  if(handle<0) return;

  i=0;

  while(read(handle, &fi, sizeof fi)==sizeof fi)
   {
    i++;

    if(strcmp(mid, fi.mid)==0)
     {
      if(!fi.flags.deleted)
       {
        fi.flags.deleted=TRUE;
        lseek(handle, -sizeof fi, SEEK_CUR);
        write(handle, &fi, sizeof fi);
        printf("    -> Nachricht %lu geloescht!\n", i);
        logf.printf("cancelarticle", "Nachricht <%s> geloescht", fi.mid);
       }

      break;
     }
   }

  close(handle);
 }

void spool_onewst::dospoolarticle(const char *folder)
 {
  folderpt fp;
  const char *tptr, *tptr2;
  char tempstr[TEXTLEN];

  strmaxcpy(fp.name, folder, TEXTLEN-1);

  if(fp.read())
   {
    return;
   }

  if((fp|"type")[0]!=0 && atoi(fp|"type")!=GROUPTYPE_ZCONNECT)
   {
    printf("%s kein ZC-Brett\n", folder);
    return;
   }

  printf("%s\n", folder);

  tptr=m.data;
  if(!m.findheader(&tptr, "U-Control", &tptr2))
   {
    m.copyheadertext(tempstr, tptr2, TEXTLEN-1);
    char *tptr3=strchr(tempstr, ' ');
    if(tptr3!=NULL)
     {
      *tptr3=0;
      tptr3++;
      while(*tptr3==' ') tptr3++;
     }

    if(stricmp(tempstr, "cancel")==0)
     {
      cancelarticle(&fp, tptr3);
      return;
     }
   }

  spoolinfolder(m, folder, atoi(fp|"forceextern"), NULL);
 }

void spool_onewst::spoolf3()
 {
  const char *dataptr, *inhalt, *tptr;
  char tempstr[1000];

  dataptr=m.data;
  while(!m.findheader(&dataptr, "EMP", &inhalt))
   {
    m.copyheadertext(tempstr, inhalt, 999);
    dnstring(tempstr);
    tptr=strchr(tempstr, '@');
    if(tptr==NULL)
      dospoolarticle(tempstr);
   }
 }

void doonews()
 {
  char filename[DIRLEN];
  spool_onewst so;

  sprintf(filename, BOXSPOOLDIR "%s.brt", 
    config.boxdir, config.boxname, config.boxname);

  so.spoolf(filename);
 }

void domakedir(const char *dir)
 {
  char *tptr;
  char d[DIRLEN];
  bool again;

  do
   {
    strmaxcpy(d, dir, TEXTLEN-1);
    again=FALSE;

   schleife:
    if(mkdir(d, 0700)!=0)
     {
      tptr=strrchr(d, DIRCHAR[0]);
      if(tptr==NULL) return;
      *tptr=0;
      if(strlen(d)!=strlen(dir))
       {
        again=TRUE;
        goto schleife;
       }
     }
   }
  while(again);
 }

void doeintragen()
 {
  char tempstr[1000], *tptr;

  while(fgets(tempstr, TEXTLEN-1, stdin)!=NULL)
   {
    tptr=strchr(tempstr, ' ');
    if(tptr!=NULL) *tptr=0;
    tptr=strchr(tempstr, '\t');
    if(tptr!=NULL) *tptr=0;
    tptr=strchr(tempstr, '\n');
    if(tptr!=NULL) *tptr=0;
    tptr=strchr(tempstr, '\r');
    if(tptr!=NULL) *tptr=0;

    if(tempstr[0]=='/')
     {
      printf("Trage ein: \"%s\"\n", tempstr);
      dnstring(tempstr);

      folderpt fp;

      strmaxcpy(fp.name, tempstr, TEXTLEN-1);
      if(fp.read())
       {
        domakedir(fp.getpath());

        if(fp.lock())
          printf("lock failed\n");
        else
         {
          if(fp.write())
            printf("write failed\n");

          fp.unlock();
         }
       }
     }
   }
 }

struct minmaxt
 {
  dword min, max;

  minmaxt()
   {
    min=max=0;
   }
 };

void parseminmax(const char *string, minmaxt *minmax) 
 {
  if(string==NULL) return;

  while(string[0]!=0)
   {
    switch(string[0])
     {
     case '+':
      string++;
      if(!isdigit(string[0])) return;
      minmax->max=atol(string);
      while(isdigit(string[0])) string++;
      break;

     case '-':
      string++;
      if(!isdigit(string[0])) return;
      minmax->min=atol(string);
      while(isdigit(string[0])) string++;
      break;

     default:
      return;
     }
   }
 }

void doexpirepath(const char *path, cmdlinet *cmdline)
 {
  folderindext fi;
  int handle_i, handle_h, tmp_handle_i, tmp_handle_h;
  char filename_i[DIRLEN], tmp_filename_i[DIRLEN];
  char filename_h[DIRLEN], tmp_filename_h[DIRLEN];
  char datafilename[DIRLEN];
  dword realanz, realquota=0;
  time_t ct;
  char newpath[DIRLEN];

  if(strlen(path)>(TEXTLEN-10)) return;

  if(path[0]==0)
    sprintf(newpath, "%s" DIRCHAR, config.cwd);
  else if(path[0]==DIRCHAR[0])
    sprintf(newpath, "%s" DIRCHAR, path);
  else
    sprintf(newpath, "%s" DIRCHAR "%s", config.cwd, path);

  time(&ct);

  sprintf(filename_i, "%sindex", newpath);
  sprintf(filename_h, "%sheader", newpath);
  sprintf(tmp_filename_i, "%s.tmp", filename_i);
  sprintf(tmp_filename_h, "%s.tmp", filename_h);

  if(path[0]!=0)
    printf("Crunch \"%s\": ", newpath);

  handle_i=open(filename_i, O_RDWR|O_BINARY);
  if(handle_i<0)
   {
    printf("Folder ist leer!\n");
    return;
   }

  realanz=filelength(handle_i)/sizeof fi;

  printf("%lu ", realanz);

  if(realanz==0)
   {
    close(handle_i);
    printf("Folder ist leer!\n");
    return;
   }

  while(read(handle_i, &fi, sizeof fi)==sizeof fi)
   {
    if(fi.flags.deleted)
      realanz--;
    else
      realquota+=(fi.headerlen+fi.len+(sizeof fi));
   }

  lseek(handle_i, 0, SEEK_SET);

  if(cmdline->isset('t') || cmdline->isset('s') || cmdline->isset('n'))
   {
    minmaxt m_days, m_size, m_number, m_quota;
    dword age, messagequota; // Age in days

    if(cmdline->isset('t')) parseminmax(cmdline->getval('t'), &m_days);
    if(cmdline->isset('s')) parseminmax(cmdline->getval('s'), &m_size);
    if(cmdline->isset('n')) parseminmax(cmdline->getval('n'), &m_number);
    if(cmdline->isset('q')) parseminmax(cmdline->getval('q'), &m_quota);

    while(read(handle_i, &fi, sizeof fi)==sizeof fi)
     {
      age=(ct-fi.received)/60/60/24;
      messagequota=(fi.headerlen+fi.len+(sizeof fi));

      #if 0
      fprintf(stderr, "DK Debug 1: %ld>%ld %ld>%ld %ld>%ld %ld>%ld\n",
        age, m_days.min, messagequota, m_size.min*1000,  
        realanz, m_number.min, realquota, m_quota.min*1000);

      fprintf(stderr, "DK Debug 2: %ld>%ld %ld>%ld %ld>%ld %ld>%ld\n",
        age, m_days.max, messagequota, m_size.max*1000,  
        realanz, m_number.max, realquota, m_quota.max*1000);
      #endif

      if(fi.flags.deleted)
       {
       }
      else if((age>m_days.min) &&
              (messagequota>(m_size.min*1000)) &&
              (realanz>m_number.min) &&
              (realquota>(m_quota.min*1000)) &&
              ((age>m_days.max && m_days.max!=0) ||
               (messagequota>(m_size.max*1000) && m_size.max!=0) ||
               (realanz>m_number.max && m_number.max!=0) ||
               (realquota>(m_quota.max*1000) && m_quota.max!=0)))
       {
        fi.flags.deleted=TRUE;
        lseek(handle_i, -sizeof fi, SEEK_CUR);
        write(handle_i, &fi, sizeof fi);
        realanz--;
        realquota-=messagequota;
       }
     }

    lseek(handle_i, 0, SEEK_SET);
   }

  remove(tmp_filename_i);
  remove(tmp_filename_h);

  handle_h=open(filename_h, O_RDONLY|O_BINARY);
  if(handle_h<0)
   {
    logf.printf("crunch", "Failed to open headerfile %s!", filename_h);
    close(handle_i);
    printf("Fehler!\n");
    return;
   }

  tmp_handle_i=open(tmp_filename_i, O_WRONLY|O_CREAT|O_TRUNC|O_BINARY, 
    S_IREAD|S_IWRITE);

  if(tmp_handle_i<0)
   {
    logf.printf("crunch", "Failed to open indexfile %s!", tmp_filename_i);
    close(handle_i);
    close(handle_h);
    printf("Fehler!\n");
    return;
   }

  tmp_handle_h=open(tmp_filename_h, O_WRONLY|O_CREAT|O_TRUNC|O_BINARY,
    S_IREAD|S_IWRITE);

  if(tmp_handle_h<0)
   {
    logf.printf("crunch", "Failed to open headerfile %s!", tmp_filename_h);
    close(handle_i);
    close(handle_h);
    close(tmp_handle_i);
    printf("Fehler!\n");
    return;
   }

  realanz=0;

  while(read(handle_i, &fi, sizeof fi)==sizeof fi)
   {
    if(fi.flags.deleted)
     {
      if(fi.flags.bodyextern)
       {
        sprintf(datafilename, "%s%s", newpath, fi.file);
        if(remove(datafilename)!=0)
          printf("remove(\"%s\"); failed!\n", datafilename);
       }
     }
    else
     {
      lseek(handle_h, fi.headeroffs, SEEK_SET);
      fi.headeroffs=lseek(tmp_handle_h, 0, SEEK_CUR);

      if(fi.flags.bodyextern)
        filecopy(handle_h, tmp_handle_h, fi.headerlen);
      else
        filecopy(handle_h, tmp_handle_h, fi.headerlen+fi.len);

      write(tmp_handle_i, &fi, sizeof fi);

      realanz++;
     }
   }

  close(handle_i);
  close(handle_h);
  close(tmp_handle_i);
  close(tmp_handle_h);

  remove(filename_i);
  remove(filename_h);

  if(rename(tmp_filename_i, filename_i)!=0)
    logf.printf("crunch", "Failed to rename %s to %s!", 
      tmp_filename_i, filename_i);

  if(rename(tmp_filename_h, filename_h)!=0)
    logf.printf("crunch", "Failed to rename %s to %s!", 
      tmp_filename_h, filename_h);

  printf("%lu\n", realanz);
 }

void doexpire(const char *board, cmdlinet *cmdline)
 {
  folderpt fp;
  strmaxcpy(fp.name, board, TEXTLEN-1);
  fp.setfilename();
  doexpirepath(fp.getpath(), cmdline);
 }

void doexpire(cmdlinet *cmdline)
 {
  char tempstr[1000], *tptr;

  while(fgets(tempstr, TEXTLEN-1, stdin)!=NULL)
   {
    tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;
    tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;

    if(tempstr[0]=='/')
     {
      dnstring(tempstr);
      doexpire(tempstr, cmdline);
     }
   }
 }

void doexport(const char *brett, FILE *outfile)
 {
  folderpt fp;
  folderindext fi;
  int handle_i;
  FILE *infile_h, *infile_d;
  char filename_i[DIRLEN];
  char filename_h[DIRLEN];
  char datafilename[DIRLEN];

  strmaxcpy(fp.name, brett, TEXTLEN-1);
  if(fp.read())
   {
    fprintf(stderr, "Brett %s nicht gefunden!\r\n", fp.name);
    return;
   }

  printf("Exportiere %s\n", fp.name);

  sprintf(filename_i, "%sindex", fp.getpath());
  sprintf(filename_h, "%sheader", fp.getpath());

  handle_i=open(filename_i, O_RDWR|O_BINARY);
  if(handle_i<0)
   {
    printf("Folder ist leer!\n");
    return;
   }

  infile_h=fopen(filename_h, "rb");
  if(infile_h==NULL)
   {
    logf.printf("doexport", "Failed to open headerfile %s!", filename_h);
    close(handle_i);
    printf("Fehler!\n");
    return;
   }

  while(read(handle_i, &fi, sizeof fi)==sizeof fi)
   {
    if(!fi.flags.deleted)
     {
      fseek(infile_h, fi.headeroffs, SEEK_SET);

      if(fi.flags.bodyextern)
       {
        filecopy(infile_h, outfile, fi.headerlen);

        sprintf(datafilename, "%s%s", fp.getpath(), fi.file);
        infile_d=fopen(datafilename, "rb");
        filecopy(infile_d, outfile, fi.len);
        fclose(infile_d);
       }
      else
        filecopy(infile_h, outfile, fi.headerlen+fi.len);
     }
   }

  close(handle_i);
  fclose(infile_h);
 }

void doexport(const char *filename)
 {
  char tempstr[1000], *tptr;
  FILE *outfile;

  if(filename[0]=='/')
    strmaxcpy(tempstr, filename, TEXTLEN-1);
  else
    sprintf(tempstr, "%s" DIRCHAR "%s", config.cwd, filename);

  outfile=fopen(tempstr, "ab");
  if(outfile==NULL)
   {
    fprintf(stderr, "Datei %s konnte nicht geoeffnet werden!\r\n", tempstr);
    return;
   }

  printf("Exportiere nach %s\n", tempstr);

  while(fgets(tempstr, TEXTLEN-1, stdin)!=NULL)
   {
    tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;
    tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;

    if(tempstr[0]=='/')
     {
      dnstring(tempstr);
      doexport(tempstr, outfile);
     }
   }

  fclose(outfile);
 }

void dobrettliste()
 {
  char tempstr[1000], *tptr;
  FILE *infile;

  sprintf(tempstr, "cd " BOXBRETTERDIR "; find ./ -type d -follow",
    config.boxdir);

  infile=popen(tempstr, "r");
  if(infile==NULL) return;
 
  while(fgets(tempstr, 900, infile)!=NULL)
   {
    if(tempstr[0]=='.' && tempstr[1]=='/')
     {
      folderpt fp;

      tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;

      strmaxcpy(fp.name, tempstr+1, TEXTLEN-1);
      if(!fp.read())
       {
        switch(atoi(fp|"type"))
         {
         case GROUPTYPE_ZCONNECT:
          strmaxcpy(tempstr, fp.name, TEXTLEN-1);
          upstring(tempstr);
          printf("%-40s %s\n", tempstr, fp|"betreff");
          break;

         default:;
         }
       }
     }
   }

  fclose(infile);
 }

static void dostorearticle(zcmessaget *m)
 {
  const char *dataptr, *inhalt;
  char tempstr[1000];
  FILE *outfile;
  int i;

  dataptr=m->data;
  if(!m->findheader(&dataptr, "FILE", &inhalt))
   {
    m->copyheadertext(tempstr, inhalt, 999);
   }
  else
   {
    dataptr=m->data;
    if(!m->findheader(&dataptr, "BET", &inhalt))
     {
      m->copyheadertext(tempstr, inhalt, 999);
     }
    else
      abort();
   }

  char *tptr2=tempstr+strlen(tempstr);
  i=1;
  while(access(tempstr, 00)==0)
    sprintf(tptr2, ".%d", i++);

  tptr2=tempstr;
  while(*tptr2!=0)
   {
    if(isascii(*tptr2) &&
       (isalnum(*tptr2) || *tptr2=='.' || *tptr2=='_' ||
        *tptr2=='-'))
     {
     }
    else
      *tptr2='_';

    tptr2++;
   }

  outfile=fopen(tempstr, "wb");
  if(outfile==NULL) abort();

  m->copybody(outfile);

  fclose(outfile);
 }

static void dostore(int handle, dword laenge)
 {
  bool mapped=FALSE, error=FALSE;
  char *posting=NULL;
  zcmessaget m;

  if(laenge>60000)
   {
    posting=(char *)mmap(NULL, laenge, PROT_READ, MAP_FILE|MAP_PRIVATE, handle, 0);
    mapped=TRUE;
    if(posting==NULL)
     {
      logf.printf("dostore", "Fatal: mmap failed!");
      return;
     }
   }

  if(posting==NULL)
   {
    posting=(char *)malloc(laenge);
    if(posting==NULL) abort();

    if(read(handle, posting, laenge)!=(ssize_t)laenge) abort();
   }

  m.data=posting;
  m.endoffile=posting+laenge;

  while(!m.eof() && !error)
   {
    if(m.parse())
      error=TRUE;
    else
     {
      dostorearticle(&m);
      m.data=m.endofmail;
     }
   }

  if(mapped)
   {
    munmap(posting, laenge);
   }
  else
   {
    free(posting);
   }

  if(error)
    logf.printf("dostore", "FATAL: Fehler in der Pufferdatei!");
 }

void dostore(const char *filename)
 {
  dword laenge;
  int handle;

  handle=open(filename, O_RDONLY|O_BINARY);

  if(handle<0) return;

  laenge=filelength(handle);

  if(laenge!=0) dostore(handle, laenge);

  close(handle);
 }

