/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <dirent.h>
#include <time.h>

#include <dboxpath.h>
#include <logf.h>
#include <userdb.h>
#include "userlog.h"
#include "ostatus.h"
#include "alias.h"

// Loescht alle Aliase eines Users

bool del_all_alias(userpt *up)
 {
  FILE *infile;
  char filename[DIRLEN], tempstr[DIRLEN], *tptr;

  snprintf(filename, DIRLEN, "%salias", up->getpath());
  infile=fopen(filename, "rt");

  if(infile!=NULL)
   {
    while(fgets(tempstr, DIRLEN-1, infile)!=NULL)
     {
      tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;
      if(tempstr[0]!=0)
       {
        userpt up2(tempstr);
        if(!up2.read())
         {
          if(stricmp(up2|"alias", up->name)!=0)
           {
            logf.printf("del_all_alias",
              "Alias \"%s\" von User \"%s\" inkonsistent: \"%s\"",
              up2.name, up->name, up2|"alias");
           }
          else
           {
            if(up2.del())
             {
              logf.printf("del_all_alias",
                "Loeschung von Alias \"%s\" von User \"%s\" fehlgeschlagen!",
                up2.name, up->name);
             }
            else
             {
              userlogprintf2(status, up->name, "del_all_alias",
                "Alias \"%s\" von User \"%s\" geloescht", up2.name, up->name);
             }
           }
         }
        else
          logf.printf("del_all_alias", 
            "Alias \"%s\" von User \"%s\" nicht gefunden!",
            up2.name, up->name);
       }
     }

    fclose(infile);
   }

  snprintf(filename, DIRLEN, "%salias", up->getpath());
  snprintf(tempstr, DIRLEN, "%s_alias", up->getpath());
  rename(filename, tempstr);

  return FALSE;
 }

// Loescht einen Alias

bool del_alias(userpt *up)
 {
  FILE *infile, *outfile;
  char filename[DIRLEN], tempstr[DIRLEN], *tptr;

  if((*up|"alias")[0]==0) return TRUE;

  userpt up2(*up|"alias");
  if(up2.read()) return TRUE;

  snprintf(filename, DIRLEN, "%salias", up2.getpath());
  infile=fopen(filename, "rt");

  snprintf(filename, DIRLEN, "%salias.tmp", up2.getpath());
  outfile=fopen(filename, "wt");

  if(outfile==NULL) { fclose(infile); return TRUE; }

  if(infile!=NULL)
   {
    while(fgets(tempstr, DIRLEN-1, infile)!=NULL)
     {
      tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;
      if(tempstr[0]!=0 && stricmp(tempstr, up->name)!=0)
        fprintf(outfile, "%s\n", tempstr);
     }

    fclose(infile);
   }

  fclose(outfile);

  snprintf(filename, DIRLEN, "%salias.tmp", up2.getpath());
  snprintf(tempstr, DIRLEN, "%salias", up2.getpath());
  rename(filename, tempstr);

  if(up->del()) return TRUE;

  return FALSE;
 }

// Fuegt einen Alias hinzu

bool add_alias(pt *profile, const char *alias)
 {
  FILE *outfile;
  char filename[DIRLEN];
  userpt up(alias);

  if(up.lock()) return TRUE;

  up.set("eintragsdatum", time(NULL));
  up.set("alias", profile->name);

  if(up.add())
   {
    up.unlock();
    return TRUE;
   }
  else
   {
    up.unlock();

    snprintf(filename, DIRLEN, "%salias", profile->getpath());
    outfile=fopen(filename, "at");

    if(outfile==NULL) return TRUE;

    fprintf(outfile, "%s\n", alias);

    fclose(outfile);     
   }        

  return FALSE;
 }

