/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <time.h>

#include <dbox.h>
#include <portinfo.h>
#include <oldtools.h>
#include <oldui.h>
#include <dboxpath.h>
#include <logf.h>
#include <nls.h>

#include "online.h"
#include "selport.h"
#include "chat.h"
#include "help.h"

static void callsysop()
 {
  system("callsysop &");
  infobox(&online, "Sysop wird gerufen!");
 }

static bool calluser() // TRUE=kein Redraw
 {
  word port;

  if((port=selectport(&online, 5, 0))==0xffff) return TRUE;

  portinfo.read(port);

  switch(portinfo.info.status)
   {
   case portinfot::infot::NETCALL:
    errorbox(&online, "Ein Netzsytem kann nicht chatten!");
    return FALSE;

   case portinfot::infot::SLIP:
    errorbox(&online, "Dieser User verwendet SLIP!");
    return FALSE;

   case portinfot::infot::CHATTING:
    errorbox(&online, "Der User befindet sich bereits im Chat!");
    return FALSE;

   case portinfot::infot::ONLINE:
    break;

   default:
    errorbox(&online, "Auf diesem Port ist kein User eingeloggt!");
    return FALSE;
   }

  if(stricmp(portinfo.info.username, status.username)==0)
   {
    errorbox(&online, "Das ist Ihr Useraccount!");
    return FALSE;
   }

  return TRUE;
 }

void doonlinechatmenu()
 {
  bool ende=FALSE;

  do
   {
    byte port=0, anzchatting=0, anzonline=0;
    bool chatting[MAXPORTS], onliners[MAXPORTS];
    char tempstr[TEXTLEN];
    int i;

    drawframe(nls.get("online.chat.menutitle"));

    online.setcursor(26, 3);
    online.setattr(1, TRUE);
    online.printf(nls.get("online.chat.user"));
    online.setattr(1, FALSE);

    printtimeout();

    while(port!=MAXPORTS && port<32)
     {
      portinfo.read(port);
      online.setcursor(port%4*18+6, port/4+5);
      online.printf("%2u: ", port);
      chatting[port]=onliners[port]=FALSE;

      if(stricmp(portinfo.info.username, status.username)!=0)
       {
        switch(portinfo.info.status)
         {
         case portinfot::infot::CHATTING:
          chatting[port]=TRUE;
          anzchatting++;
          break;

         case portinfot::infot::ONLINE:
          onliners[port]=TRUE;
          anzonline++;
          break;

         default:;
         }
       }

      if(portinfo.info.versteckt)
        strmaxcpy(tempstr, "* belegt *", TEXTLEN-1);
      else
       {
        portinfo.getstatusstring(tempstr);
        tempstr[12]=0;
       }

      if(chatting[port])
        online.setattr(1, TRUE);
      else if(port==config.port)
        online.setattr(10, FALSE);

      online.printf(tempstr);
      online.setattr(1, FALSE);
      for(i=12-strlen(tempstr); i>0; i--)
        #ifdef DEBUG
        online.putch('');
        #else
        online.putch(' ');
        #endif

      port++;
     }

    byte y=14;
    met mes[10];
    byte ids[10];

    for(i=0; i<10; i++)
     {
      mes[i].type=met::COMMANDBUTTON;
      mes[i].x=25;
      mes[i].y=y+i*2;
      mes[i].sizex=30;
      mes[i].key=0;
      ids[i]=0xff;
     }

    i=0;

    if(TRUE)
     { ids[0]=i; mes[i++].text=(char *)nls.get("online.chat.conference"); }

    if(anzonline>0)
     { ids[1]=i; mes[i++].text=(char *)nls.get("online.chat.calluser"); }

    if(TRUE)
     { ids[2]=i; mes[i++].text=(char *)nls.get("online.chat.callsysop"); }

    ids[3]=i; mes[i++].text=(char *)nls.get("online.chat.quit");

    mes[i].type=met::END_OF_LIST;

   schleife:
    i=domenu(&online, mes, 0, TRUE);

    if(i==ids[0])
      doonlinechat();
    else if(i==ids[1])
     {
      if(calluser() && !status.dologout) goto schleife;
     }
    else if(i==ids[2])
      callsysop();
    else if(i==MENU_HELP)
      displayhelp("chat");
    else
      ende=TRUE;

    if(status.dologout) ende=TRUE;
   }
  while(!ende);
 }

class doonlinechatt
 {
 public:
  char ownfile[DIRLEN];
  FILE *outfile;
  dword lastoffset[MAXPORTS];
  char line[80];
  byte starty, y, xoffs;
  const char *chatanrede;

  void initscreen();
  void printwelcome();
  void doit();
  void sendline(const char *l);
  bool checkkey();
  void checkchat();
  bool openownfile();
  bool closeownfile();
  void printmsg(char *text);
 };

void doonlinechatt::initscreen()
 {
  int i;

  chatanrede=status.userprofile|"chatanrede";

  if(chatanrede[0]==0)
    chatanrede=status.username;

  online.clrscr();
  drawtitle(&online, nls.get("online.chat.title"));

  online.setcursor(1, 2);
  online.printf("%s:", chatanrede);
  xoffs=strlen(chatanrede)+3;
  online.setcursor(1, 3);
  online.setattr(10, FALSE);
  for(i=online.printf(" %s %u", nls.get("online.chat.port"),
    config.port); i<80; i++) online.putch(' ');
  online.setattr(1, FALSE);

  for(i=0; i<MAXPORTS; i++)
   {
    struct stat statbuf;
    char filename[DIRLEN];

    snprintf(filename, DIRLEN, DBOXCHATDIR "%u", config.dboxdir, i);
    if(stat(filename, &statbuf)==0)
      lastoffset[i]=statbuf.st_size;
    else
      lastoffset[i]=0xffffffff;
    lastoffset[0]=0;
   }

  starty=y=4;
  line[0]=0;

 }

void doonlinechatt::printwelcome()
 {
  byte port=0, anzchatting=0;
  char tempstr[TEXTLEN];

  while(port!=MAXPORTS)
   {
    portinfo.read(port);

    if(portinfo.info.status==portinfot::infot::CHATTING)
      anzchatting++;

    port++;
   }

  snprintf(tempstr, TEXTLEN, "Es %s zur Zeit %u User im Konferenzbrett.",
    anzchatting==1?"ist":"sind", anzchatting);

  printmsg("");
  printmsg(tempstr);
  printmsg("");
 }

bool doonlinechatt::openownfile()
 {
  snprintf(ownfile, DIRLEN, DBOXCHATDIR "%u", config.dboxdir, config.port);

  outfile=fopen(ownfile, "wt");

  if(outfile==NULL)
   {
    logf.printf("doonlinechat::openownfile",
                "%s konnte nicht geffnet werden!", ownfile);
    return TRUE;
   }

  return FALSE;
 }

void doonlinechatt::checkchat()
 {
  int i;

  for(i=1; i<MAXPORTS; i++)
    if(i!=config.port)
     {
      struct stat statbuf;
      char filename[DIRLEN];

      snprintf(filename, DIRLEN, DBOXCHATDIR "%u", config.dboxdir, i);
      if(stat(filename, &statbuf)==0)
       {
        if(statbuf.st_size!=lastoffset[i])
         {
          if(lastoffset[i]==0xffffffff)
           {
            portinfo.read(i);
            if(portinfo.info.status==portinfot::infot::CHATTING)
             {
              char tempstr[TEXTLEN];
              snprintf(tempstr, TEXTLEN, "User %s hat das Konferenzbrett "
                "betreten!", portinfo.info.username);
              printmsg("");
              printmsg(tempstr);
              printmsg("");
             }
           }
          else
           {
            FILE *infile;
            char tempstr[100], *tptr;

            if(lastoffset[i]>statbuf.st_size)
              lastoffset[i]=0;

            infile=fopen(filename, "rt");

            if(infile!=NULL)
             {
              fseek(infile, lastoffset[i], SEEK_SET);
              while(fgets(tempstr, 90, infile)!=NULL)
               {
                tptr=strchr(tempstr, '\n');
                if(tptr!=NULL)
                 {
                  *tptr=0;
                  tptr=strchr(tempstr, '\r');
                  if(tptr!=NULL) *tptr=0;
                  tempstr[79]=0;
                  printmsg(tempstr);
                  lastoffset[i]=ftell(infile);
                 }
               }
              fclose(infile);
             }
           }
          lastoffset[i]=statbuf.st_size;
         }
       }
      else if(lastoffset[i]!=0xffffffff)
       {
        char tempstr[TEXTLEN];

        lastoffset[i]=0xffffffff;
        portinfo.read(i);
        if(portinfo.info.status==portinfot::infot::ONLINE)
          snprintf(tempstr, TEXTLEN, "User %s hat das Konferenzbrett "
            "verlassen!", portinfo.info.username);
        else
          snprintf(tempstr, TEXTLEN, "Der User auf Port %u hat das "
            "Konferenzbrett verlassen!", i);
        
        printmsg("");
        printmsg(tempstr);
        printmsg("");
       }
     }
 }

bool doonlinechatt::checkkey()
 {
  word key;
  byte len=strlen(line);

  do
   {
    switch(key=online.doreadkey())
     {
     case Key_C_X:
     case Key_C_C:
     case Key_Esc:
     case 0xffff:
      return TRUE;

     case Key_Ent:
      len=0;
      sendline(line);
      line[0]=0;
      online.setcursor(xoffs+len, 2);
      online.cleareol();
      break;

     case Key_BS:
      if(len>0)
       {
        online.putch(8); online.putch(' '); online.putch(8);
        len--;
        line[len]=0;
       }
      break;

     case 0:
      break;

     default:
      if((xoffs+strlen(line))<80 && key>=' ')
       {
        online.printf("%c", key);
        line[len]=key;
        len++;
        line[len]=0;
       }
     }
   }
  while(key!=0);

  return FALSE;
 }

void doonlinechatt::sendline(const char *l)
 {
  char tempstr[TEXTLEN];
  snprintf(tempstr, TEXTLEN, "%s: %s", chatanrede, l);
  fprintf(outfile, "%s\n", tempstr);
  fflush(outfile);
  printmsg(tempstr);
 }

void doonlinechatt::doit()
 {
  do checkchat(); while(!checkkey());
 }

bool doonlinechatt::closeownfile()
 {
  fclose(outfile);
  remove(ownfile);
  return FALSE;
 }

void doonlinechatt::printmsg(char *text)
 {
  online.setcursor(1, y);
  online.printf("%s\r\n", text);
  if(y==online.sizey-2)
   {
    online.setcursor(1, starty);
    online.deleteline();
    online.setcursor(1, y);
    online.cleareol();
   }
  else
    y++;

  online.setcursor(xoffs+strlen(line), 2);
 }

void doonlinechat()
 {
  doonlinechatt doc;
  doc.initscreen();
  portinfo.owninfo.status=portinfot::infot::CHATTING;
  portinfo.write(config.port);
  if(doc.openownfile()) return;
  doc.printwelcome();
  doc.doit();
  portinfo.owninfo.status=portinfot::infot::ONLINE;
  portinfo.write(config.port);
  doc.closeownfile();
 }
