/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: command line interface

*/

#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <time.h>
#include <fcntl.h>

#include <dboxpath.h>
#include <folder.h>
#include <portinfo.h>
#include <junkerr.h>
#include <dboxversion.h>
#include <zconnect.h>
#include <folderindex.h>
#include <oldtools.h>
#include <newsdb.h>
#include <charsettools.h>
#include "online.h"
#include "verw.h"
#include "chat.h"
#include "settings.h"
#include "openconfig.h"
#include "help.h"
#include "menu.h"
#include "readmail.h"
#include "wmail.h"
#include "readnews.h"
#include <tools.h>
#include "telnet.h"
#include <nls.h>

#define MAXCOMMANDS 100
#define MAXHISTORY 20

class commandlinet
 {
 public:
  byte historypos;
  bool first;
  word morecounter;
  char brett[TEXTLEN];
  
  struct historyt
   {
    char command[TEXTLEN];
   } history[MAXHISTORY];

  struct commandt
   {
    char name[TEXTLEN], command[TEXTLEN];
   } commands[MAXCOMMANDS];

  bool getcommand(char *command);
  bool docommand(char *c);
  bool cmpcommand(const char *command, const char *c, int minlen);
  void readcommands();
  void doit(bool f);

  bool more();
  void cmd_online();
  void cmd_showline(folderindext *fi, dword pos);
  bool cmd_inhaltparseargs(folderindext *fi, char *args, dword pos);
  void cmd_inhalt(char *args);
  void cmd_lesen(char *args);
  bool cmd_lesen(FILE *infile, dword laenge, const char *charset);
  bool cmd_lesen(folderindext *fi, dword pos);
  void cmd_loeschen(char *args);

 };

bool commandlinet::more()
 {
  if(morecounter==(online.console.sizey-3))
   {
    word key;

    online.setattr(1, TRUE);
    online.printf("<MORE>");
    online.setattr(1, FALSE);
    online.movecursor(-6, 0);

   schleife:
    key=online.doreadkey();

    if(key==0xffff ||
       key==Key_C_X ||
       key==Key_C_C ||
       key==Key_Esc)
     {
      if(key!=0xffff)
       {
        online.printf("      ");
        online.movecursor(-6, 0);
       }
      return TRUE;
     }

    if(key==0) goto schleife;

    online.printf("      ");
    online.movecursor(-6, 0);

    morecounter=0;
   }
  else
    morecounter++;

  return FALSE;
 }

void commandlinet::cmd_online()
 {
  int i;
  char tempstr[2*TEXTLEN];

  online.printf("\r\nPort Login                     "
                "Name\r\n");

  morecounter+=2;

  for(i=0; i<MAXPORTS; i++)
   {
    portinfo.read(i);
    switch(portinfo.info.status)
     {
     case portinfot::infot::OFFLINE:
      break;

     case portinfot::infot::WORKING:
      online.printf("%3d  DBOX\r\n", i);
      if(more()) return;
      break;

     case portinfot::infot::FREE:
      online.printf("%3d  frei\r\n", i);
      if(more()) return;
      break;

     case portinfot::infot::RING:
      online.printf("%3d  Anruf\r\n", i);
      if(more()) return;
      break;
 
     case portinfot::infot::LOGIN:
      online.printf("%3d  Login\r\n", i);
      if(more()) return;
      break;

     case portinfot::infot::ERROR:
      online.printf("%3d  FEHLER\r\n", i);
      if(more()) return;
      break;

     case portinfot::infot::ONLINE:
     case portinfot::infot::CHATTING:
     case portinfot::infot::SLIP:
      if(portinfo.info.status==portinfot::infot::SLIP)
        snprintf(tempstr, TEXTLEN, "SLIP %s", portinfo.info.username);
      else
        strmaxcpy(tempstr, portinfo.info.username, TEXTLEN-1);

      tempstr[20]=0;
      online.printf("%3d  User %-20s", i, tempstr);
      if(portinfo.info.username[0]!=0 &&
         stricmp(config.boxname, portinfo.info.boxname)==0)
       {
        userpt up(portinfo.info.username);
        if(!up.read())
         {
          snprintf(tempstr, TEXTLEN, "%s %s", up|"vorname", up|"nachname");
          //ibm2iso(tempstr, strlen(tempstr));
          tempstr[25]=0;
          online.printf(" %-25s", tempstr);
         }
       }
      online.printf("\r\n");
      if(more()) return;
      break;

     case portinfot::infot::NETCALL:
      online.printf("%3d  Netcall %s\r\n", i, portinfo.info.username);
      if(more()) return;
      break;
     }
   }

  online.printf("\r\n");
  if(more()) return;
 }

void commandlinet::cmd_showline(folderindext *fi, dword pos)
 {
  char t1[TEXTLEN*2], t2[TEXTLEN*2], tempstr[TEXTLEN];
  bool gelesen;
  struct tm *tm;
  
  if(brett[0]==0)
    gelesen=fi->flags.seen;
  else
    gelesen=(status.letzteranruf>fi->received);

  strmaxcpy(t1, fi->abs, (TEXTLEN*2)-1);
  t1[25]=0;

  if(fi->flags.deleted)
    snprintf(t2, TEXTLEN*2, "-- Gelscht -- (war: %s)", fi->bet);
  else
    strmaxcpy(t2, fi->bet, (TEXTLEN*2)-1);

  t2[35]=0;

  if(fi->len<10000)
   {
    snprintf(tempstr, TEXTLEN, "%4lu", fi->len);
   }
  else
   {
    dword l=fi->len/1024;
    if(l>999)
     {
      l/=1024;
      if(l==0) l=1;
      snprintf(tempstr, TEXTLEN, "%3luM", l);
     }
    else
     {
      if(l==0) l=1;
      snprintf(tempstr, TEXTLEN, "%3luK", l);
     }
   }
  
  tm=localtime(&fi->eda);

  online.printf("%c%4ld %02u.%02u %s %-25s %-35s\r\n", 
    gelesen?' ':'*', pos, tm->tm_mday, tm->tm_mon+1, tempstr, t1, t2);
 }

bool commandlinet::cmd_inhaltparseargs(folderindext *fi, char *args, dword pos)
 {
  if(fi->flags.deleted) 
    return FALSE;

  if(args[0]==0 ||
     stricmp(args, "N")==0 ||
     stricmp(args, "NEU")==0)
   {
    if(!fi->flags.seen)
      return TRUE;
   }
  else if(stricmp(args, "*")==0)
   {
    return TRUE;
   }
  else
   {
    char *tptr1, *tptr2, tempstr[TEXTLEN];

    strmaxcpy(tempstr, args, TEXTLEN-1);
    tptr1=strchr(tempstr, ' ');
    tptr2=strchr(tempstr, ',');

    if(tptr1==NULL && tptr2==NULL)
     {
      if(isdigit(args[0]) || args[0]=='-')
       {
        tptr1=strchr(args, '-');
        if(tptr1==NULL)
         {
          return pos==(dword)atol(args);
         }
        else
         {
          dword a1, a2;

          if(args[0]==0)
            a1=0;
          else
            a1=atol(args);

          tptr1++;

          if(tptr1[0]==0)
            return pos>=a1;

          a2=atol(tptr1);
          return pos>=a1 && pos<=a2;
         }
       }
      else
       {
        // online.printf("\r\nParameter \"%s\" unbekannt!\r\n\r\n", args);
        return FALSE;
       }
     }
    else
     {
      if(tptr1==NULL || (tptr1>tptr2 && tptr2!=NULL))
        tptr1=tptr2;

      *tptr1=0;
      tptr1++;
      while(*tptr1==' ' || *tptr1==',') tptr1++;
      return cmd_inhaltparseargs(fi, tempstr, pos) ||
             cmd_inhaltparseargs(fi, tptr1, pos);
     }
   }

  return FALSE;
 }

void commandlinet::cmd_inhalt(char *args)
 {
  folderindext fi;
  int index;
  bool neu=FALSE;
  char filename[DIRLEN];

  snprintf(filename, DIRLEN, BOXUSERDIR "mail" DIRCHAR "index",
    config.boxdir, status.username);

  index=::open(filename, O_RDWR|O_BINARY);

  morecounter++;
  online.printf("\r\n");

  if(atoi(status.userprofile|"nachrichtenanzeige"))
   {
    dword pos;

    pos=filelength(index)/sizeof fi;

    while(pos!=0 && 
          lseek(index, (pos-1)*sizeof fi, SEEK_SET)!=-1 &&
          ::read(index, &fi, sizeof fi)==sizeof fi)
     {
      if(cmd_inhaltparseargs(&fi, args, pos))
       {
        cmd_showline(&fi, pos);
        if(more()) return;
        neu=TRUE;
       }

      pos--;
     }
   }
  else
   {
    dword pos=1;

    while(::read(index, &fi, sizeof fi)==sizeof fi)
     {
      if(cmd_inhaltparseargs(&fi, args, pos))
       {
        cmd_showline(&fi, pos);
        if(more()) return;
        neu=TRUE;
       }

      pos++;
     }
   }

  close(index);

  if(!neu && 
     (stricmp(args, "NEU")==0 || stricmp(args, "N")==0 || args[0]==0))
   {
    online.setattr(1, TRUE);
    online.printf("Keine neuen Mails gefunden.\r\n");
    online.setattr(1, FALSE);
   }

  online.printf("\r\n");
  if(more()) return;
 }

bool commandlinet::cmd_lesen(FILE *infile, dword laenge, const char *charset)
 {
  char c;
  byte x=0, zeichensatz;

  if(charset[0]==0)
    zeichensatz=0;
  else if(stricmp(charset, "ISO1")==0)
    zeichensatz=1;
  else 
    zeichensatz=0xff;

  while(laenge!=0)
   {
    c=fgetc(infile);
    laenge--;

    switch(c)
     {
     case '\r':
      break;

     case '\n':
      x=0;
      online.printf("\r\n");
      if(more()) return TRUE;
      break;
 
     default:
      if(x<(online.sizex-2) && (c<0 || c>=' '))
       {
        x++;
        switch(zeichensatz)
         {
         case 0:
          online.printf("%c", c);
          break;

         case 1:
          iso2ibm(&c, 1);
          online.printf("%c", c);
          break;

         default:
          online.putch(c);
         }
       }
     }
   }

  online.printf("\r\n");
  if(more()) return TRUE;

  return FALSE;
 }

bool commandlinet::cmd_lesen(folderindext *fi, dword pos)
 {
  FILE *infile;
  char kennung[200], tempstr[DIRLEN], path[DIRLEN], charset[TEXTLEN]="";
  byte width;
  dword laenge;

  online.printf("\r\n");
  if(more()) return TRUE;

  if(fi->flags.deleted)
   {
    online.setattr(1, TRUE);
    online.printf("Nachricht %lu ist gelscht!\r\n");
    online.setattr(1, FALSE);
    if(more()) return TRUE;
    online.printf("\r\n");
    if(more()) return TRUE;
    return TRUE;
   }

  width=min(TEXTLEN-1, online.sizex-21);

  if(fi->flags.binary)
    laenge=0;
  else
    laenge=fi->len;

  online.setattr(1, TRUE);

  snprintf(path, DIRLEN, BOXUSERDIR "mail" DIRCHAR, 
    config.boxdir, status.username);

  snprintf(tempstr, DIRLEN, "%sheader", path);
  infile=fopen(tempstr, "rb");
  if(infile==NULL) return TRUE;
  fseek(infile, fi->headeroffs, SEEK_SET);

  while(!readkennung(infile, kennung) && kennung[0]!=0)
   {
    if(stricmp(kennung, "BET")==0)
     {
      readinhalt(infile, tempstr, width);
      online.printf("Betreff.......: %s\r\n", tempstr);
      if(more()) { fclose(infile); return TRUE; }
     }
    else if(stricmp(kennung, "X-" X_DBOX "-JUNKERROR")==0)
     {
      readinhalt(infile, tempstr, 60);
      online.printf("Fehler........: %s\r\n",
        junkerrors[atoi(tempstr)]);
      if(more()) { fclose(infile); return TRUE; }
     }
    else if(stricmp(kennung, "ABS")==0)
     {
      readinhalt(infile, tempstr, width);
      online.printf("Absender......: %s\r\n", tempstr);
      if(more()) { fclose(infile); return TRUE; }
     }
    else if(stricmp(kennung, "WAB")==0)
     {
      readinhalt(infile, tempstr, width);
      online.printf("Weiterleitung.: %s\r\n", tempstr);
      if(more()) { fclose(infile); return TRUE; }
     }
    else if(stricmp(kennung, "ORG")==0)
     {
      readinhalt(infile, tempstr, width);
      online.printf("Organisation..: %s\r\n", tempstr);
      if(more()) { fclose(infile); return TRUE; }
     }
    else if(stricmp(kennung, "FILE")==0)
     {
      readinhalt(infile, tempstr, width);
      online.printf("Dateiname.....: %s\r\n", tempstr);
      if(more()) { fclose(infile); return TRUE; }
     }
    else if(stricmp(kennung, "KOM")==0)
     {
      readinhalt(infile, tempstr, 60);

      if(fi->flags.binary)
        laenge=min((dword)atol(tempstr), fi->len);
     }
    else if(stricmp(kennung, "CHARSET")==0)
     {
      readinhalt(infile, charset, 60);
     }
    else
      skipinhalt(infile);
   }

  online.printf("Textlnge.....: %lu Bytes\r\n", laenge);
  if(more()) { fclose(infile); return TRUE; }

  if(fi->flags.binary && fi->len>laenge)
   {
    online.printf("Dateigre....: %lu Bytes\r\n", fi->len-laenge);
    if(more()) { fclose(infile); return TRUE; }
   }

   {
    struct tm *t;
    t=localtime(&fi->eda);
    online.printf("Datum.........: %02u.%02u.%04u, %02u:%02u:%02u "
                  "(local time)\r\n",
      t->tm_mday, t->tm_mon+1, t->tm_year+1900, t->tm_hour, t->tm_min,
      t->tm_sec);
    if(more()) { fclose(infile); return TRUE; }
   }

  online.setattr(1, FALSE);

  for(int i=0; i<online.sizex-1; i++) online.printf("");
  online.printf("\r\n");
  if(more()) { fclose(infile); return TRUE; }

  if(fi->flags.bodyextern)
   {
    FILE *infile2;
    char filename[DIRLEN];

    snprintf(filename, DIRLEN, "%s%s", path, fi->file);
    infile2=fopen(filename, "rb");
    if(infile2==NULL)
     {
      online.setattr(1, TRUE);
      online.printf("\r\nINTERNER FEHLER!\r\n");
      online.setattr(1, FALSE);
     }
    else
     {
      if(laenge!=0)
       {
        if(cmd_lesen(infile2, laenge, charset)) 
         {
          fclose(infile2);
          fclose(infile);
          return TRUE;
         }
       }
      fclose(infile2);
     }
   }
  else
   {
    if(laenge!=0)
     {
      if(cmd_lesen(infile, laenge, charset))
       {
        fclose(infile);
        return TRUE;
       }
     }
   }

  fclose(infile);

  if(laenge<fi->len && fi->flags.binary)
   {
    online.printf("\r\n");
    if(more()) return TRUE;
    for(int i=0; i<online.sizex-1; i++) online.printf("");
    online.printf("\r\n");
    if(more()) return TRUE;
    online.printf("\r\n");
    if(more()) return TRUE;
    online.printf("Wichtiger Hinweis: Dieser Nachricht ist eine Datei "
      "angehngt!\r\n");
    if(more()) return TRUE;
   }

  return FALSE;
 }

void commandlinet::cmd_lesen(char *args)
 {
  folderindext fi;
  int index;
  bool neu=FALSE;
  char filename[DIRLEN];

  snprintf(filename, DIRLEN, BOXUSERDIR "mail" DIRCHAR "index",
    config.boxdir, status.username);

  index=::open(filename, O_RDWR|O_BINARY);

  if(atoi(status.userprofile|"nachrichtenanzeige"))
   {
    dword pos;

    pos=filelength(index)/sizeof fi;

    while(pos!=0 && 
          lseek(index, (pos-1)*sizeof fi, SEEK_SET)!=-1 &&
          ::read(index, &fi, sizeof fi)==sizeof fi)
     {
      if(cmd_inhaltparseargs(&fi, args, pos))
       {
        if(cmd_lesen(&fi, pos)) { close(index); return; }
        neu=TRUE;
       }

      pos--;
     }
   }
  else
   {
    dword pos=1;

    while(::read(index, &fi, sizeof fi)==sizeof fi)
     {
      if(cmd_inhaltparseargs(&fi, args, pos))
       {
        if(cmd_lesen(&fi, pos)) { close(index); return; }
        neu=TRUE;
       }

      pos++;
     }
   }

  close(index);

  if(!neu && 
     (stricmp(args, "NEU")==0 || stricmp(args, "N")==0 || args[0]==0))
   {
    online.setattr(1, TRUE);
    online.printf("\r\nKeine neuen Mails gefunden.\r\n");
    online.setattr(1, FALSE);
   }
 }

void commandlinet::cmd_loeschen(char *args)
 {
  folderindext fi;
  int index;
  bool neu=FALSE;
  char filename[DIRLEN];

  if(args[0]==0)
   {
    online.printf("\r\nWas soll gelscht werden?\r\n\r\n");
    return;
   }

  snprintf(filename, DIRLEN, BOXUSERDIR "mail" DIRCHAR "index",
    config.boxdir, status.username);

  index=::open(filename, O_RDWR|O_BINARY);

  morecounter++;
  online.printf("\r\n");

  if(atoi(status.userprofile|"nachrichtenanzeige"))
   {
    dword pos;

    pos=filelength(index)/sizeof fi;

    while(pos!=0 && 
          lseek(index, (pos-1)*sizeof fi, SEEK_SET)!=-1 &&
          ::read(index, &fi, sizeof fi)==sizeof fi)
     {
      if(cmd_inhaltparseargs(&fi, args, pos))
       {
        fi.flags.deleted=TRUE;
        lseek(index, (pos-1)*sizeof fi, SEEK_SET);
        write(index, &fi, sizeof fi);
        online.printf("Nachricht %lu gelscht.\r\n", pos);
        if(more()) return;
        neu=TRUE;
       }

      pos--;
     }
   }
  else
   {
    dword pos=1;

    while(::read(index, &fi, sizeof fi)==sizeof fi)
     {
      if(cmd_inhaltparseargs(&fi, args, pos))
       {
        fi.flags.deleted=TRUE;
        lseek(index, (pos-1)*sizeof fi, SEEK_SET);
        write(index, &fi, sizeof fi);
        online.printf("Nachricht %lu gelscht.\r\n", pos);
        if(more()) return;
        neu=TRUE;
       }

      pos++;
     }
   }

  close(index);

  if(neu)
   {
    char filename[DIRLEN];
    FILE *outfile;

    snprintf(filename, DIRLEN, BOXTODODIR "crunch.%s", config.boxdir,
      status.username);

    outfile=fopen(filename, "wt");
    if(outfile!=NULL) fclose(outfile);
   }

  if(!neu && 
     (stricmp(args, "NEU")==0 || stricmp(args, "N")==0))
   {
    online.setattr(1, TRUE);
    online.printf("Keine neuen Mails gefunden.\r\n");
    online.setattr(1, FALSE);
   }

  online.printf("\r\n");
  if(more()) return;
 }

bool commandlinet::cmpcommand(const char *command, const char *s, 
  int minlen)
 {
  int i=strlen(command);
  if(i<minlen) return FALSE;
  return strnicmp(command, s, i)==0;
 }

bool commandlinet::docommand(char *c)
 {
  char *tptr;
  char tempstr[1000], command[TEXTLEN], args[TEXTLEN];
  bool redraw=FALSE;

  while(*c==' ' || *c=='\t') c++;
  strmaxcpy(tempstr, c, TEXTLEN-1);

  tptr=tempstr+strlen(tempstr);
  if(tptr==tempstr) return FALSE;
  tptr--;
  while(*tptr==' ' || *tptr=='\t') { *tptr=0; tptr--; }

  tptr=strchr(tempstr, ' ');
  if(tptr!=NULL)
   {
    *tptr=0;
    tptr++;
    while(*tptr==' ' || *tptr=='\t') tptr++;
   }

  strmaxcpy(command, tempstr, TEXTLEN-1);
  if(tptr==NULL)
    strmaxcpy(args, "", TEXTLEN-1);
  else
    strmaxcpy(args, tptr, TEXTLEN-1);

  for(int i=0; commands[i].name[0]!=0; i++)
   {
    if(stricmp(command, commands[i].name)==0)
     {
      strmaxcpy(command, commands[i].command, TEXTLEN-1);
      break;
     }
   }

  if(command[0]==0)
    return FALSE;

  morecounter=0;

  if(cmpcommand(command, "SHELL", 2))
   {
   }
  else if(cmpcommand(command, "QUIT", 1) ||
          cmpcommand(command, "BYE", 2) ||
          cmpcommand(command, "EXIT", 2) ||
          cmpcommand(command, "ENDE", 3))
   {
    online.printf("Ok, Quit.\r\n");
    return TRUE;
   }
  else if(cmpcommand(command, "PASSWORD", 4) ||
          cmpcommand(command, "KENNWORT", 4) ||
          cmpcommand(command, "PASSWORT", 4))
   {
    char password[TEXTLEN];
    getplaintextpassword(&status.userprofile, password);
    if(!changepassword(password))
     {
      if(!status.userprofile.lock())
       {
        cryptpassword(password, status.userprofile.get_ptr("cryptedpassword"));
        status.userprofile.write();
        status.userprofile.unlock();
       }
     }
    redraw=TRUE;
   }
  else if(cmpcommand(command, "LOGOUT", 5))
   {
    if(first) return TRUE;
    online.printf("\r\nKommando hier nicht implementiert!\r\n\r\n");
   }
  else if(cmpcommand(command, "RELOGIN", 4) ||
          cmpcommand(command, "LOGIN", 5))
   {
    if(first)
     {
      status.relogin=TRUE;
      return TRUE;
     }
    online.printf("\r\nKommando hier nicht implementiert!\r\n\r\n");
   }
  else if(stricmp(command, "?")==0 ||
          cmpcommand(command, "HILFE", 4) || 
          cmpcommand(command, "HELP", 4))
   {
    displayhelp("commandline");
    redraw=TRUE;
   }
  else if(((cmpcommand(command, "MAIL", 1) && args[0]!=0) ||
          cmpcommand(command, "SENDEN", 1) ||
          cmpcommand(command, "BRIEF", 3)))
   {
    if(args[0]=='/' && strchr(args, '@')==NULL)
     {
      if(get_news_folder_grants(args, status.userprofile, status.gruppe,
                                FALSE, TRUE, FALSE))
        online.printf("\r\nSchreibzugriff verweigert!\r\n\r\n");
      else
       {
        dnstring(args);
        folderpt fp(args);
        if(fp.read())
          online.printf("\r\nBrett nicht gefunden!\r\n\r\n");
        else
         {
          dowritemail(args);
          redraw=TRUE;  
         }
       }
     }
    else
     {
      dowritemail(args);
      redraw=TRUE;
     }
   }
  #if 0
  else if(stricmp(command, "VERWALTUNG")==0)
   {
    doverwaltungsmenue();
    redraw=TRUE;
   }
  #endif
  else if(stricmp(command, "CHAT")==0 ||
          cmpcommand(command, "DIALOG", 3))
   {
    doonlinechatmenu();
    redraw=TRUE;
   }
  else if(cmpcommand(command, "SETTINGS", 3))
   {
    doonlinemenu("setting2", &status.userprofile, 
                 FALSE, FALSE, TRUE, NULL);
    redraw=TRUE;
   }
  else if(cmpcommand(command, "TELNET", 3))
   {
    dointernet_telnet(args);
   }
  else if(cmpcommand(command, "STATUS", 4))
   {
    doonlinemenu("setting1", &status.userprofile, 
                 FALSE, FALSE, TRUE, NULL);
    redraw=TRUE;
   }
  #if 0
  else if(stricmp(command, "ANTRAG")==0)
   {
    doantrag();
    redraw=TRUE;
   }
  #endif
  else if(cmpcommand(command, "MAIL", 1) && args[0]==0)
   {
    doreadmail();
    redraw=TRUE;
   }
  else if(cmpcommand(command, "INHALT", 1))
   {
    cmd_inhalt(args);
   }
  else if(cmpcommand(command, "LESEN", 1))
   {
    cmd_lesen(args);
   }
  else if(isdigit(command[0]))
   {
    snprintf(tempstr, TEXTLEN, "%s %s", command, args);
    cmd_lesen(tempstr);
   }
  else if(cmpcommand(command, "LOESCHEN", 3))
   {
    cmd_loeschen(args);
   }
  else if(stricmp(command, "VERSION")==0 ||
          stricmp(command, "#VERSION")==0)
   {
    online.printf("\r\nDBOX Version " DBOXVERSIONNR " vom "
                  DBOXVERSION ".\r\n\r\n");
   }
  else if(stricmp(command, "NEWSREADER")==0 ||
          cmpcommand(command, "BRETT", 1))
   {
    if(args[0]==0)
     {
      dobretter();
      redraw=TRUE;
     }
    else if(args[0]=='/')
     {
      if(get_news_folder_grants(args, status.userprofile, status.gruppe,
                                TRUE, FALSE, FALSE))
        online.printf("\r\nBrett nicht gefunden!\r\n\r\n");
      else
       {
        dnstring(args);
        folderpt fp(args);
        if(fp.read())
          online.printf("\r\nBrett nicht gefunden!\r\n\r\n");
        else
         {
          //doreadnews(args, FALSE); // TODO
          redraw=TRUE;  
         }
       }
     }
    else
     {
     }
   }
  else if(stricmp(command, "ONLINE")==0 ||
          cmpcommand(command, "OSTAT", 1))
   {
    cmd_online();    
   }
  else if(cmpcommand(command, "MENU", 3))
   {
    const char *menu=status.gruppe|"mainmenu";
    if(menu[0]==0) menu="main";
    doonlinemenu(menu, &status.userprofile, 
                 FALSE, FALSE, TRUE, NULL);
    redraw=TRUE;
   }
  #if 0
  else if(stricmp(command, "FINGER")==0)
    dointernet_finger(tptr);
  else if(stricmp(command, "TRACEROUTE")==0)
    dointernet_traceroute(tptr);
  else if(stricmp(command, "NSLOOKUP")==0)
    dointernet_nslookup(tptr);
  else if(stricmp(command, "ARCHIE")==0)
    dointernet_archie();
  else if(stricmp(command, "WWW")==0)
    dointernet_www(tptr);
  #endif
  else
    online.printf("\r\n%s\r\n\r\n",
      nls.get("online.commandline.invalidcommand"));

  if(status.dologout) return TRUE;

  if(redraw)
   {
    online.setattr(1, FALSE);
    online.clrscr();
    online.setcursor(1, 1);
   }

  return FALSE;
 }

bool commandlinet::getcommand(char *command)
 {
  char *tptr;
  word key;
  int i;
  byte posx, prompt;

  if(status.dologout) return TRUE;

  command[0]=0;
  posx=0;
  historypos=0;

  if(brett[0]==0)
    prompt=online.printf("%s@%s.%s> ", 
      status.username, config.boxname, config.domain);
  else
    prompt=online.printf("%s> ", brett);

  while(TRUE)
   {
    key=online.doreadkey();

    if(key==0xffff)
      return TRUE;
    else if(key==Key_F1)
     {
      strmaxcpy(command, "?", TEXTLEN-1);
      return FALSE;
     }
    else if(key>=' ' && key<126)
     {
      if(int(strlen(command))<(79-(int)prompt))
       {
        tptr=command+strlen(command);
        key=dncase(key);
        *(tptr++)=key;
        *tptr=0;
        online.putch(key);
       }
     }
    else if(key=='\r' || key=='\n')
     {
      online.printf("\r\n");
      if(command[0]!=0)
       {
        strmaxcpy(history[0].command, command, TEXTLEN-1);
        for(i=(MAXHISTORY-2); i!=0; i--)
          history[i]=history[i-1];
       }
      return FALSE;
     }
    else if(key==Key_Up)
     {
      if(history[historypos+1].command[0]!=0)
       {
        if(historypos==0)
          strmaxcpy(history[0].command, command, TEXTLEN-1);

        historypos++;
        strmaxcpy(command, history[historypos].command, TEXTLEN-1);
        online.movecursor(-posx, 0);
        posx=online.printf("%s", command);
        online.cleareol();
       }
     }
    else if(key==Key_Dwn)
     {
      if(historypos!=0)
       {
        historypos--;
        strmaxcpy(command, history[historypos].command, TEXTLEN-1);
        online.movecursor(-posx, 0);
        posx=online.printf("%s", command);
        online.cleareol();
       }
     }
    else if(key==8 || key==127)
     {
      if(strlen(command)!=0)
       {
        online.putch(8);
        online.putch(' ');
        online.putch(8);
        command[strlen(command)-1]=0;
       }
     }
   }

  return FALSE;
 }

void commandlinet::readcommands()
 {
  FILE *infile;
  char name[TEXTLEN], command[TEXTLEN];
  int i;

  infile=openconfigfile(status, "commands");
  if(infile==NULL)
   {
    strmaxcpy(commands[0].name, "QUIT", TEXTLEN-1);
    strmaxcpy(commands[0].command, "QUIT", TEXTLEN-1);
    commands[1].name[0]=commands[1].command[0]=0;
    return;
   }

  i=0;
  while(readconfigline(infile, name, command, TEXTLEN-1)==0)
   {
    strmaxcpy(commands[i].name, name, TEXTLEN-1);
    strmaxcpy(commands[i].command, command, TEXTLEN-1);
    i++;
   }

  commands[i].name[0]=commands[i].command[0]=0;

  fclose(infile);
 }

void checkmail()
 {
  folderindext fi;
  int index;
  bool new_mail=FALSE;
  char filename[DIRLEN];

  snprintf(filename, DIRLEN, BOXUSERDIR "mail" DIRCHAR "index",
    config.boxdir, status.username);

  index=::open(filename, O_RDWR|O_BINARY);

  while(::read(index, &fi, sizeof fi)==sizeof fi && !new_mail)
   {
    if(!fi.flags.seen && !fi.flags.deleted)
      new_mail=TRUE;
   }

  close(index);

  if(new_mail)
   {
    online.setattr(1, TRUE);
    online.printf("%s\r\n\r\n", nls.get("online.commandline.newmail"));
    online.setattr(1, FALSE);
   }
 }

void printtimeout_cmdline()
 {
  if(status.timeouttime!=0)
   {
    long int min=status.timeouttime-time(NULL)-1;
    if(min<0) min=0; else min=(min/60)+1;
    if(min==1)
     {
      online.setattr(1, TRUE);
      online.printf("\r\n%s\r\n\r\n", nls["online.oneminuteleft"]);
      online.setattr(1, FALSE);
     }
    else
     {
      online.printf("\r\n");
      online.printf(nls["online.minutesleft"], min);
      online.printf("\r\n\r\n");
     }
   }
 }

void commandlinet::doit(bool f)
 {
  char command[TEXTLEN];
  time_t lastcheck;
  int i;

  if(status.dologout) return;

  online.setattr(1, FALSE);
  online.clrscr();
  online.setcursor(1, 2);

  online.setattr(1, TRUE);
  online.printf(nls.get("online.commandline.welcome"),
                DBOXVERSIONNR, DBOXVERSION);
  online.printf("\r\n\r\n");
  online.setattr(1, FALSE);

  historypos=0;
  first=f;
  brett[0]=0;

  for(i=0; i<MAXHISTORY; i++)
    history[i].command[0]=0;

  readcommands();

  time(&lastcheck);
  checkmail();
  printtimeout_cmdline();

  while(!getcommand(command) && !status.dologout)
   {
    if(docommand(command))
      return;

    if(time(NULL)>(lastcheck+120))
     {
      checkmail();
      time(&lastcheck);
     }

    printtimeout_cmdline();
   }
 }

void docommandline(bool first)
 {
  commandlinet commandline;

  commandline.doit(first);
 }

