/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <sys/stat.h>
#include <dirent.h>
#include <sys/wait.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <signal.h>
#include <errno.h>
// #include <waitstatus.h>

#include <dbox.h>
#include <logf.h>
#include <dboxpath.h>
#include <asyncio.h>
#include <oldui.h>
#include <oldtools.h>
#include "online.h"
#include "file.h"
#include "help.h"
#include "extern.h"

int waitprocess(int pid)
 {
  int erg, sta=-1;

  status.child=pid;
  status.hup_child=TRUE;

  do
   {
    erg=wait(&sta);

    if(erg<0)
     {
      kill(pid, SIGHUP);
      status.hup_child=FALSE;
      return 0;
     }
   }
  while(erg!=pid);

  status.hup_child=FALSE;

  return sta;
 }

time_t estimate(dword size)
 {
  if(status.linespeed==0) return 0;
  return size*9/status.linespeed;
 }

void estimate(dword size, char *s)
 {
  time_t t;
  t=estimate(size);
  if(t==0)
    strmaxcpy(s, "nicht feststellbar", TEXTLEN-1);
  else
   {
    t=t/60+1;
    snprintf(s, TEXTLEN, "%lu Minute%s", t, t==1?"":"n");
   }
 }

// Upload: filename = Verzeichnisname

bool doupdownload(bool down, char prot, const char *filename,
  const char *d1, const char *d2)
 {
  int pipefd[2], udpid, sta=-1, n, oldflags, linefd;
  bool abbruch=FALSE, dowait=FALSE;
  char buf[128], linebuf[128]="";
  time_t last_no_dcd=0, lastbufflush=0;

  if(status.online2)
   {
    online.console.clrscr();

    if(down)
     {
      struct stat statbuf;
      char tempstr[TEXTLEN];

      drawtitle(&online.console, "Download-Modul");
      online.console.setcursor(1, 3); online.console.printf("Dateiname:");
      online.console.setcursor(1, 4); online.console.printf("Dateigre:");
      online.console.setcursor(1, 5); online.console.printf("Username:");
      online.console.setcursor(1, 6); online.console.printf("Gesch. Zeit:");
      online.console.setcursor(1, 7); online.console.printf("Modem-Speed:");
      online.console.setattr(1, TRUE);
      if(strchr(filename, ' ')==NULL)
       {
        if(stat(filename, &statbuf)!=0) return TRUE;
        online.console.setcursor(15, 3); online.console.printf("%s", filename);
        online.console.setcursor(15, 4); online.console.printf("%lu Bytes", statbuf.st_size);
        online.console.setcursor(15, 5); online.console.printf("%s", status.username);
        estimate(statbuf.st_size, tempstr);
        online.console.setcursor(15, 6); online.console.printf(tempstr);
       }
      else
       {
        online.console.setcursor(15, 3); online.console.printf("mehrere");
        online.console.setcursor(15, 4); online.console.printf("n/a");
        online.console.setcursor(15, 5); online.console.printf("%s", status.username);
        online.console.setcursor(15, 6); online.console.printf("n/a");
       }

      if(status.linespeed==0)
        strmaxcpy(tempstr, "nicht feststellbar", TEXTLEN-1);
      else
        snprintf(tempstr, TEXTLEN, "%lu BPS", status.linespeed);
      online.console.setcursor(15, 7); online.console.printf(tempstr);
      online.console.setattr(1, FALSE);
      online.console.setcursor(1, 9);
      if(d1!=NULL) online.console.printf("%s\r\n", d1);
      if(d2!=NULL) online.console.printf("%s\r\n", d2);
      online.console.printf("\r\n");
     }
    else
     {
      drawtitle(&online.console, "Upload-Modul");
      online.console.setcursor(1, 3); online.console.printf("Verzeichnis:");
      online.console.setcursor(1, 4); online.console.printf("Dateigre:");
      online.console.setcursor(1, 5); online.console.printf("Username:");
      online.console.setcursor(1, 6); online.console.printf("Gesch. Zeit:");
      online.console.setattr(1, TRUE);
      online.console.setcursor(15, 3); online.console.printf("%s", filename);
      online.console.setcursor(15, 4); online.console.printf("-");
      online.console.setcursor(15, 5); online.console.printf("%s", status.username);
      online.console.setcursor(15, 6); online.console.printf("-");
      online.console.setattr(1, FALSE);
      online.console.setcursor(1, 8);
      if(d1!=NULL) online.console.printf("%s\r\n", d1);
      if(d2!=NULL) online.console.printf("%s\r\n", d2);
      online.console.printf("\r\n");
     }
    online.comport.clrscr();
    online.comport.setcursor(1, 1);
    online.comport.printf("Bitte starten Sie Ihr bertragungsprotokoll!\r\n\r\n");
    linefd=online.comport.fd;
   }
  else
   {
    online.clrscr();
    online.setcursor(1, 1);
    linefd=online.console.fd;
   }

  asyncio.flush_output();

  oldflags=fcntl(linefd, F_GETFL);
  pipe(pipefd);

  switch(udpid=fork())
   {
   case -1:
    logf.printf("doupdownload", "Fatal: fork failed!");
    if(status.online2)
     {
      close(pipefd[0]);
      close(pipefd[1]);
     }
    return TRUE;

   case 0: /* Child */
    if(status.online2)
     {
      dup2(online.comport.fd, 0);
      dup2(online.comport.fd, 1);
      dup2(pipefd[1], 2);
      close(pipefd[0]);
      // if (pipefd[1] != 2) close(pipefd[1]);
     }
    else
     {
      dup2(online.console.fd, 0);
      dup2(online.console.fd, 1);
      dup2(open("/dev/null", O_RDWR), 2);
     }
    fcntl(linefd, F_SETFL, 0);
    fcntl(pipefd[1], F_SETFD, 0);

    char pname[DIRLEN];

    prot=tolower(prot);
    switch(prot)
     {
     case 'x':
     case 'y':
     case 'z':
      pname[0]=down?'s':'r';
      pname[1]=prot;
      pname[2]=0;
      break;
     }

    if(down)
     {
      if(strchr(filename, ' ')==NULL)
       {
        execlp(pname, pname, "-vv", filename, NULL);
       }
      else
       {
        #define MAXFILES 1000
        char *tmp, *pointer[MAXFILES], *tptr, *tptr2;
        int i;

        tmp=strdup(filename);
        if(tmp==NULL) exit(1);
        i=0;
        pointer[i++]=pname;
        pointer[i++]="-vv";
        
        tptr=tmp;
        while(tptr!=NULL && *tptr!=0)
         {
          tptr2=strchr(tptr, ' ');
          if(tptr2!=NULL) *(tptr2++)=0;
          if(tptr[0]!=0) pointer[i++]=tptr;
          tptr=tptr2;
         }

        pointer[i]=NULL;

        execvp(pname, pointer);
       }
     }
    else
     {
      chdir(filename);
      setenv("SHELL", "rksh", TRUE);
      execlp(pname, pname, "-vv", NULL);
     }
    exit(1);

   default: /* Parent */
    break;
   }

  dowait=TRUE;

  fcntl(pipefd[0], F_SETFL, O_NONBLOCK);
  close(pipefd[1]);

 schleife:
   {
    vector<int> handles;
    handles.push_back(pipefd[0]);
    if(status.online2) handles.push_back(online.console.fd);
    asyncio.input(handles);
   }

  for(n=0; n<sizeof buf; n++) buf[n]='#';

 readagain:
  n=read(pipefd[0], buf, 100);
  if(n==0) // Client Tot
   {
    if(waitpid(udpid, &sta, WNOHANG)!=udpid) 
      sta=100;
    else
     {
      if(WIFEXITED(sta))
        sta=WEXITSTATUS(sta);
      else
        sta=102;

      dowait=FALSE;
     }
   }
  else 
   {
    if(status.online2)
     {
      word key;

      if(n>0 && !abbruch)
       {
        for(int i=0; i<n; i++)
         {
          if(buf[i]=='\r' || buf[i]=='\n')
           {
            if(time(NULL)>(lastbufflush+3) || buf[i]=='\n')
             {
              online.console.printf("%s\r", linebuf);
              lastbufflush=time(NULL);
              if(buf[i]=='\n') online.console.printf("\n");
             }

            linebuf[0]=0;              
           }
          else
           {
            int len=strlen(linebuf);

            if(len>100)
              linebuf[0]=0;
            else
             {
              linebuf[len]=buf[i];
              linebuf[len+1]=0;
             }
           }
         }
        goto readagain;
       }

      status.online2=FALSE;
      status.lastkey=0;
      key=online.doreadkey();
      status.online2=TRUE;

      switch(key)
       {
       case 0xffff:
        kill(udpid, 1);
        abbruch=TRUE;
        logf.printf("doupdownload", "Logout");
        sta=101;
        break;

       case Key_Esc:
        kill(udpid, 1);
        online.console.printf("\r\n\r\nTransfer wird abgebrochen!\r\n");
        logf.printf("doupdownload", "Abbruch");
        abbruch=TRUE;
        sleep(5);
        break;     
       }

      if(!online.comport.dcd())
       {
        if(last_no_dcd==0)
          last_no_dcd=time(NULL);
        else if(time(NULL)>last_no_dcd+4)
         {
          kill(udpid, 1);
          logf.printf("doupdownload", "Carrier lost");
          online.console.printf("\r\n\r\nCarrier lost!\r\n");
          abbruch=TRUE;
          sleep(5);
         }
       }
     }
    if(!abbruch)
      goto schleife;
   }

  close(pipefd[0]);

  fcntl(linefd, F_SETFL, O_NONBLOCK);

  online.flushkeys();

  if(sta!=0)
   {
    if(sta==100)
      logf.printf("doupdownload", 
        "warning: protocol error %d during %sload ignored", sta,
        down?"down":"up");
    else
     {
      logf.printf("doupdownload", "protocol error %d during %sload", sta,
        down?"down":"up");
      return TRUE;
     }
   }

  if(dowait)
   {
    sleep(1);
    waitpid(udpid, &sta, WNOHANG);
   }

  if(abbruch) return TRUE;

  if(status.online2)
    online.console.printf("\r\nTRANSFER ERFOLGREICH ABGESCHLOSSEN!\r\n");

  return FALSE;
 }

bool dodownload(char prot, const char *filename, const char *d1, const char *d2)
 {
  return doupdownload(TRUE, prot, filename, d1, d2);
 }

char selprot(const char *d1, const char *d2)
 {
  int i;
  met mes[10];

  online.setcursor(25, 12);
  online.printf("Bitte whlen Sie ein Protokoll:");

  for(i=0; i<10; i++)
   {
    mes[i].type=met::COMMANDBUTTON;
    mes[i].x=20;
    mes[i].y=14+i*2;
    mes[i].sizex=40;
    mes[i].key=0;
   }

  mes[0].text="^Z-Modem";
  mes[1].text="^Y-Modem";
  mes[2].text="^X-Modem";
  mes[3].text="Men ^verlassen";
  mes[4].type=met::END_OF_LIST;

 schleife:
  i=domenu(&online, mes, 3, TRUE);
  switch(i)
   {
   case 0: return 'z';
   case 1: return 'y';
   case 2: return 'x';

   case MENU_HELP:
    //displayhelp(M_SELPROT_HELP);
    //goto startover;
    goto schleife;
   }
  return 0;
 }

void doonlinedownload(const char *filename, const char *d1, const char *d2)
 {
  char prot;
  const char *f;
  struct stat statbuf;
  char tempstr[TEXTLEN];

  if(stat(filename, &statbuf)!=0) return;

  f=strrchr(filename, DIRCHAR[0]);
  if(f==NULL) f=filename; else f++;

  online.clrscr();
  online.box(1, 1, online.sizex, online.sizey-1, BOXFRAMEDBL,
    "Dateibertragung", FALSE);
  printtimeout();

  online.setcursor(4, 3);
  online.printf("Dateiname:    %s", f);
  online.setcursor(4, 4);
  online.printf("Dateigre:   %lu Bytes", statbuf.st_size);
  online.setcursor(4, 5);
  estimate(statbuf.st_size, tempstr);
  online.printf("Gesch. Zeit:  %s", tempstr);

  if(d1!=NULL)
   {
    online.setcursor(4, 7);
    online.printf("%s", d1);
    if(d2!=NULL)
     {
      online.setcursor(4, 8);
      online.printf("%s", d2);
     }
   }

  prot=selprot(d1, d2);
  if(prot!=0)
    dodownload(prot, filename, d1, d2);
 }

bool doupload_specialfile(char prot, const char *filename,
  const char *d1, const char *d2)
 {
  char tempstr[DIRLEN], *tptr, filename2[DIRLEN], tempstr2[DIRLEN];
  DIR *dir;
  struct dirent *ent;

  strmaxcpy(tempstr, filename, DIRLEN-1);
  tptr=strrchr(tempstr, DIRCHAR[0]);
  if(tptr!=NULL) *tptr=0;

  if(tempstr[0]==0)
    strmaxcpy(tempstr, ".", DIRLEN-1);

  if((dir=opendir(tempstr))==NULL) return TRUE;
  closedir(dir);

  deldir(tempstr);

  if(doupdownload(FALSE, prot, tempstr, d1, d2)) return TRUE;

  if((dir=opendir(tempstr))==NULL) return TRUE;

  filename2[0]=0;

  while((ent=readdir(dir))!=NULL)
   {
    if(ent->d_name[0]!='.')
     {
      snprintf(tempstr2, DIRLEN, "%s" DIRCHAR "%s", tempstr, ent->d_name);
      if(filename2[0]==0)
        strmaxcpy(filename2, tempstr2, DIRLEN-1);
      else
        remove(tempstr2);
     }
   }

  closedir(dir);

  if(filename2[0]==0) return TRUE;

  if(rename(filename2, filename)==0)
    return FALSE;

  return TRUE;
 }

bool doupload_anyfile(char prot, const char *filename,
  const char *d1, const char *d2)
 {
  return TRUE;
 }

void doextern(char *command, const char *arg1, 
  const char *arg2, const char *arg3)
 {
  int fd, sta, pid, savefd;
  struct termio told;
 
  if(status.online2)
   {
    char tempstr[TEXTLEN];

    snprintf(tempstr, TEXTLEN, "Es wird zur Zeit das Programm \"%s\" ausgefhrt.",
      command);
    online.console.clrscr();
    online.console.box(1, 1, online.sizex, online.sizey-1, BOXFRAMEDBL,
      NULL, FALSE);
    online.console.setcursor(
      (online.console.sizex-strlen(tempstr))/2, online.console.sizey/2);
    online.console.printf("%s", tempstr);

    fd=online.comport.fd;
    savefd=online.comport.fd;
    // online.comport.fd=dup(online.comport.fd);
    // online.comport.close();
   }
  else
   {
    fd=online.console.fd;
    savefd=online.console.fd;
    online.console.fd=dup(online.console.fd);
    online.console.close();
   }

  ioctl(savefd, TCGETA, &told);

  /*
  if(status.online2)
    ioctl(fd, TCSETA, &online.comport.told);
  else
    ioctl(fd, TCSETA, &online.console.told);
  */

  switch(pid=fork())
   {
   case -1:
    logf.printf("doextern", "FATAL: Failed to fork");
    break;

   case 0:
    if(fd!=1)
     {
      dup2(fd, 0);
      dup2(fd, 1);
      dup2(fd, 2);
     }
    fcntl(0, F_SETFL, 0);
    fcntl(1, F_SETFL, 0);
    fcntl(2, F_SETFL, 0);
    execl(command, command, arg1, arg2, arg3, NULL);
    printf("NOT FOUND!\r\n");
    sleep(1);
    exit(1);
   }

  sta=waitprocess(pid);

  ioctl(savefd, TCSETA, &told);

  if(status.online2)
   {
    // online.comport.fd=savefd;
    // online.comport.open();
   }
  else
   {
    online.console.fd=savefd;
    online.console.open();
   }
 }

void doextern2(char *command)
 {
  system(command);
 }

void dogetty()
 {
 }

static void doshell()
 {
  online.clrscr();
  drawtitle(&online, "Online-Shell-Zugang - drcken Sie Ctrl-D, um abzubrechen");
  online.setcursor(1, 2);
  dogetty();
 }
