/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <dirent.h>
#include <time.h>

#include <dboxpath.h>
#include <blzdb.h>
#include <nls.h>
#include <oldui.h>
#include <oldtools.h>
#include <account.h>
#include "online.h"
#include "selport.h"
#include "help.h"
#include "fibu.h"
#include "html.h"

#define MAXLEN 10000

bool parsebetrag(const char *betrag, dword *b)
 {
  char *tptr;

  tptr=strchr(betrag, ',');
  if(tptr==NULL) return TRUE;

  if(strlen(tptr)!=3) return TRUE;
  if(tptr==betrag) return TRUE;
  *b=atol(betrag)*100+atol(tptr+1);
  return FALSE;
 }

static void showkonto(const char *username)
 {
  FILE *infile, *outfile;
  char tempstr[DIRLEN], url[TEXTLEN];
  userpt up(username);

  snprintf(tempstr, DIRLEN, BOXUSERDIR "konto", config.boxdir, username);
  infile=fopen(tempstr, "rt");

  snprintf(tempstr, DIRLEN, DBOXPORTTEMPDIR "konto.html", config.dboxdir, config.port);
  outfile=fopen(tempstr, "wb");
  if(outfile==NULL)
   {
    fclose(infile);
    return;
   }

  fprintf(outfile, "<html>\n"
                   "<head>\n"
                   "<title>Kontoauszge von User %s</title>\n"
                   "</head>\n"
                   "<body>\n"
                   "<dbox-charset-%d>\n"
                   "<pre>\n", username, IBM);

  if(infile==NULL)
   {
    fprintf(outfile, "<p>\nEs liegen keine Buchungen vor.<p>\n");
   }
  else
   {
    fprintf(outfile, "%s\n%s\n", formatkonto_start1, formatkonto_start2);
    formatkonto(infile, outfile, 'a', TRUE, IBM);
    fprintf(outfile, "%s\n", formatkonto_ende);
    fclose(infile);
   }

  fprintf(outfile, "</pre>\n"
                   "</body>\n"
                   "</html>\n");

  fclose(outfile);

  strmaxcpy(url, "internal:", TEXTLEN-1);

  snprintf(tempstr, DIRLEN, DBOXPORTTEMPDIR "konto.html",
    config.dboxdir, config.port);

  dohtmlview(tempstr, url);
  unlink(tempstr);
 }

static void userfibu()
 {
  char tempstr[TEXTLEN*3], balance_s[TEXTLEN];
  met mes[20];
  bool redraw;
  int i, newi=0;
  byte y;

  if(status.userprofile.read())
   {
    errorbox(&online, "profile konnte nicht gelesen werden!");
    return;
   }

 startover:
  if(status.dologout) return;

  snprintf(tempstr, TEXTLEN, "Konto von User %s", status.username);
  drawframe(tempstr);

  currency_string(balance_s, getbalance(&status.userprofile), TRUE);
  online.setcursor(3, 3); online.printf("Name/Firma....:");
  online.setcursor(3, 4); online.printf("Kundennummer..:");
  online.setcursor(3, 5); online.printf("Kontostand....:");
  online.setcursor(3, 6); online.printf("Kontoberzug..:");
  online.setcursor(3, 7); online.printf("Buchungsnummer:");
  online.setcursor(3, 8); online.printf("Gruppe........:");
  online.setattr(1, TRUE);

  online.setcursor(19, 3); 

  tempstr[0]=0;

  if((status.userprofile|"nachname")[0]!=0 || (status.userprofile|"vorname")[0]!=0)
   {
    if((status.userprofile|"vorname")[0]!=0)
     {
      strmaxcpy(tempstr+strlen(tempstr),
        status.userprofile|"vorname", TEXTLEN*3-strlen(tempstr)-1);

      if((status.userprofile|"nachname")[0]!=0)
        strmaxcpy(tempstr+strlen(tempstr), " ",
          TEXTLEN*3-strlen(tempstr)-1);
     }
    
    strmaxcpy(tempstr+strlen(tempstr), status.userprofile|"nachname",
      TEXTLEN*3-strlen(tempstr)-1);

    if((status.userprofile|"firma")[0]!=0)
      strmaxcpy(tempstr+strlen(tempstr), ", ", TEXTLEN*3-strlen(tempstr)-1);
   }

  strmaxcpy(tempstr+strlen(tempstr), status.userprofile|"firma",
    TEXTLEN*3-strlen(tempstr)-1);

  tempstr[online.sizex-20]=0;
  online.setcursor(19, 3); online.printf("%s", tempstr);

  status.userprofile.get("kundennummer", tempstr);
  online.setcursor(19, 4); online.printf("%s", tempstr);
  online.setcursor(19, 5); online.printf("%s", balance_s);
  online.setcursor(19, 6); online.printf(noyes1(atoi(status.userprofile|"kontoueberzug")));
  online.setcursor(19, 7); online.printf("%lu", atol(status.userprofile|"buchungsnummer"));
  online.setcursor(19, 8); online.printf("%s", status.userprofile|"gruppe");

  y=11;

  for(i=0; i<10; i++)
   {
    mes[i].type=met::COMMANDBUTTON;
    mes[i].x=25;
    mes[i].y=y+i*2;
    mes[i].sizex=30;
    mes[i].key=0;
   }

  mes[0].text="Men ^verlassen";
  mes[1].text="^Kontoauszge";
  mes[2].text="^berweisung";
  mes[3].text="^Bankeinzug";

  mes[4].type=met::END_OF_LIST;

  redraw=TRUE;

  i=domenu(&online, mes, newi, redraw);
  redraw=FALSE;

  switch(i)
   {
   case 1:
    showkonto(status.username);
    newi=i;
    goto startover;

   case 2:
    errorbox(&online, nls["online.notavailable"]);
    newi=i;
    goto startover;

   case 3:
    errorbox(&online, nls["online.notavailable"]);
    newi=i;
    goto startover;

   case MENU_HELP:
    newi=1;
    displayhelp("settings/fibu.html");
    goto startover;
   }
 }

#define ANZVERWENDUNG 4

static void doueberweisung(const char *username, userpt *up,
                           bool eingehend)
 {
  met mes[20];
  int i;
  char betrag[TEXTLEN]="0,00";
  char verwendung[ANZVERWENDUNG][TEXTLEN];

  for(i=0; i<ANZVERWENDUNG; i++)
    verwendung[i][0]=0;

  online.setattr(2, FALSE);
  online.box(10, 5, 70, 20, BOXFRAMEDBL,
    eingehend?"eingehende berweisung":"ausgehende berweisung",
    TRUE);

  online.setcursor(12, 7);  online.printf("Bearbeiter................:");
  online.setcursor(12, 8);  online.printf("Kontoinhaber (Username)...:");
  online.setcursor(12, 10); online.printf("Betrag....................:"
                                "          %s", config.currency_symbol);
  online.setcursor(12, 12); online.printf("Verwendungszweck:");

  online.setattr(2, TRUE);
  online.setcursor(40, 7);  online.printf("%s", status.username);
  online.setcursor(40, 8);  online.printf("%s", username);

  mes[0].type=met::EDIT;
  mes[0].text=betrag;
  mes[0].x=40;
  mes[0].y=10;
  mes[0].sizex=6;
  mes[0].sizex=6;
  mes[0].key=0;
  mes[0].maxlen=10;

  for(i=1; i<(ANZVERWENDUNG+1); i++)
   {
    mes[i].type=met::EDIT;
    mes[i].text=verwendung[i-1];
    mes[i].x=13;
    mes[i].y=12+i;
    mes[i].sizex=53;
    mes[i].key=0;
    mes[i].maxlen=TEXTLEN-1;
   }

  mes[ANZVERWENDUNG+1].type=met::COMMANDBUTTON;
  mes[ANZVERWENDUNG+1].text="Buchung ^durchfhren";
  mes[ANZVERWENDUNG+1].x=12;
  mes[ANZVERWENDUNG+1].y=18;
  mes[ANZVERWENDUNG+1].sizex=25;

  mes[ANZVERWENDUNG+2].type=met::COMMANDBUTTON;
  mes[ANZVERWENDUNG+2].text="Buchung ^abbrechen";
  mes[ANZVERWENDUNG+2].x=42;
  mes[ANZVERWENDUNG+2].y=18;
  mes[ANZVERWENDUNG+2].sizex=25;

 schleife:
  i=domenu(&online, mes, 0, TRUE);
  if(i==MENU_HELP) goto schleife;

  if(i==ANZVERWENDUNG+1)
   {
    char v[ANZVERWENDUNG*TEXTLEN]="";
    buchungt buchung;

    buchung.amount=parse_currency_string(betrag);
    if(buchung.amount.amount<=0)
     {
      errorbox(&online, "Betrag fehlerhaft!");
      return;
     }

    for(i=0; i<ANZVERWENDUNG; i++)
      if(verwendung[i][0]!=0)
       {
        if(v[0]==0)
          strmaxcpy(v, verwendung[i], TEXTLEN-1);
        else
          sprintf(v+strlen(v), " %s", verwendung[i]); // v is assumed to be large enough
       }

    strmaxcpy(buchung.verwendungszweck, v, (TEXTLEN*2)-1);
    strmaxcpy(buchung.bearbeiter, status.username, TEXTLEN-1);
    buchung.haben=eingehend;
    buchung.mwstsatz=0;

    dobuchung(up, &buchung);
   }
 }

static void bankeinzug(userpt *up)
 {
  met mes[20];
  int i, newi=0;
  bankpt bp(up->name);
  char betrag[TEXTLEN], kontoinhaber[TEXTLEN],
       blz[TEXTLEN], bank[TEXTLEN], kontonummer[TEXTLEN];
  bool useblzdb=TRUE;

  bp.read();

 startover:
  if(status.dologout) return;

  online.setattr(2, FALSE);
  online.box(5, 5, 75, 18, BOXFRAMEDBL, "Bankeinzug", TRUE);

  online.setcursor(7, 7);  online.printf("Username.............:");
  online.setcursor(7, 8);  online.printf("Kontoinhaber.........:");
  online.setcursor(7, 9);  online.printf("Bankleitzahl.........:");
  online.setcursor(7, 10); online.printf("Bank.................:");
  online.setcursor(7, 11); online.printf("Kontonummer..........:");
  online.setcursor(7, 12); online.printf("Einzugsermchtigung..:");
  online.setcursor(7, 13); online.printf("Automatisch Abbuchen.:");
  online.setcursor(7, 14); online.printf("Abbuchungsbetrag.....:");

  online.setattr(2, TRUE);
  online.setcursor(30, 7); online.printf("%s", up->name);

  for(i=0; i<7; i++)
   {
    mes[i].type=met::EDIT;
    mes[i].x=30;
    mes[i].y=8+i;
    mes[i].sizex=30;
    mes[i].key=0;
    mes[i].maxlen=TEXTLEN-1;
   }

  bp.get("kontoinhaber", kontoinhaber);

  mes[0].text=kontoinhaber;
  mes[0].maxlen=27;

  bp.get("blz", blz);

  mes[1].text=blz;
  mes[1].maxlen=8;
  mes[1].edittype=met::BLZ;

  bp.get("bank", bank);

  if(useblzdb)
   {
    mes[1].blzx=mes[2].x;
    mes[1].blzy=mes[2].y;
    mes[1].sizex=9;
    mes[2].type=met::IGNORE;
   }
  else
   {
    mes[2].text=bank;
    mes[2].maxlen=27;
   }

  bp.get("kontonummer", kontonummer);

  mes[3].text=kontonummer;
  mes[3].sizex=11;
  mes[3].maxlen=10;

  mes[4].type=met::COMMANDBUTTON;
  mes[4].text=(char *)noyes1(atoi(bp|"einzugsermaechtigung"));
  mes[4].sizex=4;

  mes[5].type=met::COMMANDBUTTON;
  mes[5].text=(char *)noyes1(atoi(bp|"automatisch_abbuchen"));
  mes[5].sizex=4;

  mes[6].text=betrag;
  mes[6].sizex=6;
  snprintf(betrag, TEXTLEN, "%lu,%02lu", atol(bp|"abbuchungs_betrag")/100,
    atol(bp|"abbuchungs_betrag")%100);

  mes[7].type=met::COMMANDBUTTON;
  mes[7].text="^Ok";
  mes[7].x=12;
  mes[7].y=16;
  mes[7].sizex=25;

  mes[8].type=met::COMMANDBUTTON;
  mes[8].text="^Abbruch";
  mes[8].x=42;
  mes[8].y=16;
  mes[8].sizex=25;

 schleife:
  i=domenu(&online, mes, newi, TRUE);

  switch(i)
   {
   case MENU_HELP:
    newi=0;
    goto schleife;

   case 4:
    bp.set("einzugsermaechtigung", (bool)!atoi(bp|"einzugsermaechtigung"));
    mes[4].text=(char *)noyes1(atoi(bp|"einzugsermaechtigung"));
    newi=4;
    goto schleife;

   case 5:
    bp.set("automatisch_abbuchen", (bool)!atoi(bp|"automatisch_abbuchen"));
    mes[5].text=(char *)noyes1(atoi(bp|"automatisch_abbuchen"));
    newi=5;
    goto schleife;

   case 7:
     {
      dword abbuchungs_betrag;
      if(parsebetrag(betrag, &abbuchungs_betrag))
       {
        errorbox(&online, "Betrag fehlerhaft!");
        goto startover;
       }

      bp.set("abbuchungs_betrag", abbuchungs_betrag);
     }

    bankpt newbp(bp.name);

    if(newbp.lock())
      errorbox(&online, "Daten konnten nicht gespeichert werden!");
    else
     {
      newbp.set("blz", blz);
      newbp.set("abbuchungs_betrag", bp|"abbuchungs_betrag");
      newbp.set("einzugsermaechtigung", bp|"einzugsermaechtigung");
      newbp.set("automatisch_abbuchen", bp|"automatisch_abbuchen");
      newbp.set("kontoinhaber", kontoinhaber);
      newbp.set("kontonummer", kontonummer);

      if(useblzdb)
       {
        char tempstr[TEXTLEN];
        blzdb.lookup(atol(newbp|"blz"), tempstr);
        newbp.set("bank", tempstr);
       }
      else
        newbp.set("bank", bank);

      newbp.write();
      newbp.unlock();
     }
    break;
   }
 }

static void verwfibu(const char *username)
 {
  char tempstr[TEXTLEN*2], balance_s[TEXTLEN];
  met mes[20];
  bool redraw;
  int i, newi=1;
  byte y;
  bool kontoueberzug;

  userpt profile(username);
  if(profile.read())
   {
    errorbox(&online, "profile konnte nicht gelesen werden!");
    return;
   }

  kontoueberzug=atoi(profile|"kontoueberzug");

 startover:
  if(status.dologout) return;

  snprintf(tempstr, TEXTLEN, "Konto von User %s", username);
  drawframe(tempstr);

  currency_string(balance_s, getbalance(&profile), TRUE);
  online.setcursor(3, 3); online.printf("Name/Firma....:");
  online.setcursor(3, 4); online.printf("Telefon.......:");
  online.setcursor(3, 5); online.printf("Kundennummer..:");
  online.setcursor(3, 6); online.printf("Kontostand....:");
  online.setcursor(3, 7); online.printf("Kontoberzug..:");
  online.setcursor(3, 8); online.printf("Buchungsnummer:");
  online.setcursor(3, 9); online.printf("Gruppe........:");
  online.setattr(1, TRUE);

  online.setcursor(19, 3); 

  if((profile|"nachname")[0]!=0 || (profile|"vorname")[0]!=0)
   {
    if((profile|"vorname")[0]!=0)
     {
      online.printf("%s", profile|"vorname");

      if((profile|"nachname")[0]!=0)
        online.printf(" ");
     }
    
    online.printf("%s", profile|"nachname");

    if((profile|"firma")[0]!=0)
      online.printf(", ");
   }

  online.printf("%s", profile|"firma");

  profile.get("telefon", tempstr);
  online.setcursor(19, 4); online.printf("%s", tempstr);
  profile.get("kundennummer", tempstr);
  online.setcursor(19, 5); online.printf("%s", tempstr);
  online.setcursor(19, 6); online.printf("%s", balance_s);
  online.setcursor(19, 8); online.printf("%lu", atol(profile|"buchungsnummer"));
  online.setcursor(19, 9); online.printf("%s", profile|"gruppe");

  mes[0].type=met::COMMANDBUTTON;
  mes[0].x=19;
  mes[0].y=7;
  mes[0].sizex=4;
  mes[0].text=(char *)noyes2(kontoueberzug);

  y=9;

  for(i=1; i<10; i++)
   {
    mes[i].type=met::COMMANDBUTTON;
    mes[i].x=25;
    mes[i].y=y+i*2;
    mes[i].sizex=30;
    mes[i].key=0;
   }

  mes[1].text="Men ^verlassen";
  mes[2].text="^Kontoauszge";
  mes[3].text="^eingehende berweisung";
  mes[4].text="^ausgehende berweisung";
  mes[5].text="^Mahnung verschicken";
  mes[6].text="^Bankeinzug";

  mes[7].type=met::END_OF_LIST;

  redraw=TRUE;

 schleife:
  i=domenu(&online, mes, newi, redraw);
  redraw=FALSE;

  switch(i)
   {
   case 0:
    kontoueberzug^=1;
    mes[i].text=(char *)noyes2(kontoueberzug);
    newi=i;
    goto schleife;

   case 1:
    if(!profile.lock())
     {
      profile.set("kontoueberzug", kontoueberzug);
      profile.write();
      profile.unlock();
     }
    break;

   case 2:
    showkonto(username);
    newi=i;
    goto startover;

   case 3:
    doueberweisung(username, &profile, TRUE);
    newi=i;
    goto startover;

   case 4:
    doueberweisung(username, &profile, FALSE);
    newi=i;
    goto startover;

   case 5:
    errorbox(&online, nls["online.notavailable"]);
    newi=i;
    goto startover;

   case 6:
    bankeinzug(&profile);
    newi=i;
    goto startover;

   case MENU_HELP:
    newi=1;
    displayhelp("verw/fibu.html");
    goto startover;
   }
 }

void dofibu(const char *username)
 {
  if(username==NULL)
    userfibu();
  else
    verwfibu(username);
 }
