/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <dirent.h>
#include <fcntl.h>
#include <time.h>
#include <sys/stat.h>

#include <dboxpath.h>
#include <charsettools.h>
#include <oldtools.h>
#include <oldui.h>
#include <logf.h>
#include "online.h"
#include "file.h"
#include "extern.h"
#include "help.h"
#include "editor.h"
#include "vfolder.h"

#define MAXFILES 1000

class viewfilet:public viewfoldert
 {
 public:
  char basedirectory[DIRLEN], directory[DIRLEN];
  byte layer;
  bool enable_edit;
  int w_umask;

  struct stackt
   {
    dword pos, scroll;
   } stack[100];

  struct indexentryt
   {
    bool directory;
    char *filename, *description;
    dword size;
    time_t date;
   } index[MAXFILES];

  enum { NONE, UNKNOWN, SUNSITE, FILES_BBS, HOBBES_DIRS, INFOMAGIC } type;

  virtual void readindex();
  virtual void freeindex();
  virtual void displayline(dword line);
  virtual void draw(bool complete);
  virtual void read();
  virtual void view();
  virtual void edit();
  virtual void del();
  virtual void info();
  virtual void help();
  virtual void extract();
  virtual void createfile(bool createdir);
  virtual void renamefile();
  virtual void key(word k);

  void viewfile(char *dir, bool e_edit, int write_umask);

  viewfilet() 
   {
    type=NONE;
    w_umask=0;
   }
 };

void viewfilet::key(word k)
 {
  switch(k)
   {
   case Key_Lft:
    if(stricmp(directory, basedirectory)!=0)
     {
      pos=0;
      read();
     }
    return;

   case Key_Rgt:
    read();
    return;
   }

  viewfoldert::key(k);
 }

void viewfilet::readindex()
 {
  FILE *infile;
  char tempstr[DIRLEN], tempstr2[DIRLEN], *tptr, *tptr2;
  struct stat statbuf;

  anz=1;

  index[0].directory=TRUE;
  index[0].filename=index[0].description=0;
  index[0].size=0;
  index[0].date=0;

  online.clrscr();
  online.setcursor(1, 1);
  online.printf("Bitte warten...");

  if(type==NONE)
   {
    snprintf(tempstr, DIRLEN, "%sdirs.txt", basedirectory);
    if(access(tempstr, 00)==0)
      type=HOBBES_DIRS;
    else
     {
      snprintf(tempstr, DIRLEN, "%sINDEX", basedirectory);
      if(access(tempstr, 00)==0)
        type=SUNSITE;
      else
       {
        snprintf(tempstr, DIRLEN, "%s!INDEX", basedirectory);
        if(access(tempstr, 00)==0)
          type=SUNSITE;
        else
         {
          snprintf(tempstr, DIRLEN, "%sFULLINDX.TXT", basedirectory);
          if(access(tempstr, 00)==0)
            type=INFOMAGIC;
          else
           {
            type=UNKNOWN;
           }
         }
       }
     }
   }

  if(type==NONE || type==UNKNOWN || type==FILES_BBS)
   {
    snprintf(tempstr, DIRLEN, "%sfiles.bbs", directory);
    if(access(tempstr, 00)==0)
      type=FILES_BBS;
    else
     {
      snprintf(tempstr, DIRLEN, "%sFILES.BBS", directory);
      if(access(tempstr, 00)==0)
        type=FILES_BBS;
      else
        type=UNKNOWN;
     }
   }

  switch(type)
   {
   case HOBBES_DIRS:
    snprintf(tempstr, DIRLEN, "%sdirs.txt", basedirectory);
    if((infile=fopen(tempstr, "rt"))==NULL)
      break;

    while(fgets(tempstr, DIRLEN-1, infile)!=NULL && anz!=MAXFILES)
     {
      tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=tempstr;
      while(*tptr!=0 && *tptr!=' ') { if(*tptr=='\\') *tptr='/'; tptr++; }

      tptr=directory+strlen(basedirectory)-1;
      if(strncmp(tptr, tempstr, strlen(tptr))==0)
       {
        tptr=tempstr+strlen(tptr);
        if(tptr[0]!=' ')
         {
          char *tptr3;

          tptr2=strchr(tptr, ' ');
          if(tptr2!=NULL)
           {
            *tptr2=0;
            tptr2++;
            while(*tptr2==' ') tptr2++;
           }

          tptr3=strchr(tptr, '/');
          if(tptr3!=NULL)
           {
            *tptr3=0;
            if(tptr3[1]!=0) tptr2=NULL;
           }

          if(anz==1 || stricmp(tptr, index[anz-1].filename)!=0)
           {
            index[anz].filename=strdup(tptr);
            if(index[anz].filename==NULL) return;
            index[anz].directory=TRUE;
            index[anz].size=0;
            index[anz].date=0;
            if(tptr2==NULL)
              index[anz].description=NULL;
            else
             {
              index[anz].description=strdup(tptr2);
              if(index[anz].description==NULL) return;
             }
            anz++;
           }
         }
       }
     }
    fclose(infile);

    snprintf(tempstr, DIRLEN, "%sdescript.ion", directory);
    if((infile=fopen(tempstr, "rt"))==NULL) break;

    while(fgets(tempstr, DIRLEN-1, infile)!=NULL && anz!=MAXFILES)
     {
      tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;

      if(tempstr[0]!=0)
       {
        tptr=strchr(tempstr, ' ');
        if(tptr!=NULL)
         {
          *tptr=0;
          tptr++;
          while(*tptr==' ') tptr++;
         }

        if(tempstr[0]!=0)
         {
          dnstring(tempstr); // HACK!
          snprintf(tempstr2, DIRLEN, "%s%s", directory, tempstr);
          if(stat(tempstr2, &statbuf)==0)
           {
            index[anz].filename=strdup(tempstr);
            if(index[anz].filename==NULL) return;
            index[anz].directory=FALSE;
            index[anz].size=statbuf.st_size;
            index[anz].date=statbuf.st_mtime;
            if(tptr==NULL)
              index[anz].description=NULL;
            else
             {
              index[anz].description=strdup(tptr);
              if(index[anz].description==NULL) return;
             }
            anz++;
           }
         }
       }
     }

    fclose(infile);
    break;

   case INFOMAGIC:
   case UNKNOWN:
     {
      DIR *dir;
      struct dirent *ent;

      snprintf(tempstr, DIRLEN, "%s.", directory);
      if((dir=opendir(tempstr))==NULL) return;

      while((ent=readdir(dir))!=NULL && anz!=MAXFILES)
        if(ent->d_name[0]!='.')
         {
          snprintf(tempstr, DIRLEN, "%s%s", directory, ent->d_name);
          if(stat(tempstr, &statbuf)==0)
           {
            index[anz].filename=strdup(ent->d_name);
            if(index[anz].filename==NULL) return;

            index[anz].description=strdup("");
            if(index[anz].description==NULL) return;

            index[anz].directory=(statbuf.st_mode&S_IFDIR)!=0;
            index[anz].size=statbuf.st_size;
            index[anz].date=statbuf.st_mtime;
            anz++;
           }
         }
      closedir(dir);
     }
    break;

   case SUNSITE:
    snprintf(tempstr, DIRLEN, "%sINDEX", directory);
    if((infile=fopen(tempstr, "rt"))==NULL) 
     {
      snprintf(tempstr, DIRLEN, "%s!INDEX", directory);
      if((infile=fopen(tempstr, "rt"))==NULL) return;
     }

    fgets(tempstr, DIRLEN-1, infile); // zwei Zeilen berspringen
    fgets(tempstr, DIRLEN-1, infile);

    while(fgets(tempstr, DIRLEN-1, infile)!=NULL && anz!=MAXFILES)
     {
      tptr=strchr(tempstr, '\n');
      if(tptr!=NULL) *tptr=0;
      if(tempstr[0]!=0)
       {
        tptr=strchr(tempstr, ' ');
        if(tptr!=NULL)
         {
          *tptr=0;
          tptr++;
          while(*tptr==' ') tptr++;
         }

        if(tempstr[0]!=0)
         {
          tptr2=tempstr+strlen(tempstr)-1;
          if(*tptr2=='/') *tptr2=0;

          snprintf(tempstr2, DIRLEN, "%s%s", directory, tempstr);
          if(stat(tempstr2, &statbuf)==0)
           {
            index[anz].filename=strdup(tempstr);
            if(index[anz].filename==NULL) return;
            index[anz].directory=(statbuf.st_mode&S_IFDIR)!=0;
            index[anz].size=statbuf.st_size;
            index[anz].date=statbuf.st_mtime;
            if(tptr==NULL)
              index[anz].description=NULL;
            else
             {
              iso2ibm(tptr, strlen(tptr));
              index[anz].description=strdup(tptr);
              if(index[anz].description==NULL) return;
             }
            anz++;
           }
         }
       }
     }

    fclose(infile);
    break;
 
   case FILES_BBS:
     {
      bool files_bbs_lower_case=TRUE;

      snprintf(tempstr, DIRLEN, "%sfiles.bbs", directory);
      if((infile=fopen(tempstr, "rt"))==NULL) 
       {
        snprintf(tempstr, DIRLEN, "%sFILES.BBS", directory);
        if((infile=fopen(tempstr, "rt"))==NULL) return;
        files_bbs_lower_case=FALSE;
       }

      while(fgets(tempstr, DIRLEN-1, infile)!=NULL && anz!=MAXFILES)
       {
        tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;
        tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;
  
        if(tempstr[0]!=0 && tempstr[0]!=' ' && tempstr[0]!='\t')
         {
          tptr=strchr(tempstr, ' ');
          if(tptr!=NULL)
           {
            *tptr=0;
            tptr++;
            while(*tptr==' ') tptr++;
           }

          if(files_bbs_lower_case) dnstring(tempstr);

          snprintf(tempstr2, DIRLEN, "%s%s", directory, tempstr);

          if(stat(tempstr2, &statbuf)==0)
           {
            index[anz].filename=strdup(tempstr);
            if(index[anz].filename==NULL) return;
            index[anz].directory=(statbuf.st_mode&S_IFDIR)!=0;
            index[anz].size=statbuf.st_size;
            index[anz].date=statbuf.st_mtime;
            if(tptr==NULL)
              index[anz].description=NULL;
            else
             {
              index[anz].description=strdup(tptr);
              if(index[anz].description==NULL) return;
             }
            anz++;
           }
         }
       }

      fclose(infile);
     }
    break;

   case NONE:
    break;
   }

  pos=stack[layer].pos;
  scroll=stack[layer].scroll;
 }

void viewfilet::freeindex()
 {
  dword i;

  for(i=0; i<anz; i++)
   {
    if(index[i].filename!=NULL) free(index[i].filename);
    if(index[i].description!=NULL) free(index[i].description);
   }
 }

void viewfilet::displayline(dword line)
 {
  if(line>=anz) return;

  if(line==0)
    viewfoldert::displayline(line, " Vorhergehendes Verzeichnis");
  else
    viewfoldert::displayline(line, FALSE, index[line].date,
      index[line].directory?0xffffffff:index[line].size,
      index[line].filename, index[line].description);
 }

void viewfilet::draw(bool complete)
 {
  snprintf(title, TEXTLEN, " Inhalt von %s - drcken Sie ? fr Hilfe ",
    directory+strlen(basedirectory)-1);
  title[79]=0;
  viewfoldert::draw(complete);
 }

void viewfilet::read()
 {
  if(index[pos].directory)
   {
    if(pos==0)
     {
      if(stricmp(directory, basedirectory)!=0)
       {
        char *tptr=strrchr(directory, DIRCHAR[0]);
        if(tptr!=NULL)
         {
          *tptr=0;
          tptr=strrchr(directory, DIRCHAR[0]);
          if(tptr!=NULL) *(tptr+1)=0;
          reread=TRUE;
          layer--;
         }
       }
     }
    else
     {
      snprintf(directory+strlen(directory), DIRLEN-strlen(directory),
              "%s" DIRCHAR,
              index[pos].filename);
      reread=TRUE;
      stack[layer].pos=pos;
      stack[layer].scroll=scroll;
      layer++;
      stack[layer].pos=0;
      stack[layer].scroll=0;
     }
   }
  else if(enable_edit)
    edit();
  else
   {
    char tempstr[DIRLEN];
    snprintf(tempstr, DIRLEN, "%s%s", directory, index[pos].filename);
    if(index[pos].description==NULL)
      doonlinedownload(tempstr, NULL, NULL);
    else
     {
      char tempstr2[TEXTLEN*2], tempstr3[TEXTLEN*2]="", *tptr;
      int i=60;

      if(strlen(index[pos].description)>=60)
       {
        if(index[pos].description[i]!=' ')
         {
          while(i>30 &&
                index[pos].description[i]!=' ' &&
                index[pos].description[i-1]!='-') i--;
          if(i==0) i=60;
         }

        tptr=index[pos].description+i;
        while(*tptr==' ') tptr++;

        snprintf(tempstr3, TEXTLEN, "              ");
        strmaxcpy(tempstr3+strlen(tempstr3), tptr, 60);
       }

      snprintf(tempstr2, TEXTLEN, "Beschreibung: ");
      strmaxcpy(tempstr2+strlen(tempstr2), index[pos].description, i);

      doonlinedownload(tempstr, tempstr2, tempstr3[0]!=0?tempstr3:(char *)NULL);
     }
    if(!status.dologout) draw(TRUE);
   }
 }

void viewfilet::view()
 {
  FILE *f;
  char tempstr[DIRLEN], tempstr2[DIRLEN];

  if(index[pos].directory) return;
  online.clrscr(); online.setcursor(1, 1); online.printf("Bitte warten...");
  snprintf(tempstr, DIRLEN, "%s%s", directory, index[pos].filename);
  snprintf(tempstr2, DIRLEN, TEMPDIR "dbox.tmp");
  if(viewarchive(tempstr, tempstr2))
    f=fopen(tempstr, "rt");
  else
    f=fopen(tempstr2, "rt");

  viewtextfile(&online, f,
    index[pos].description==NULL?"Ohne Beschreibung":index[pos].description,
    IBM, FALSE);

  fclose(f);
  if(!status.dologout) draw(TRUE);
 }

void viewfilet::del()
 {
  met mes[3];

  if(pos==0 || !enable_edit) return;

  online.setattr(2, FALSE);
  online.box(20, 10, 60, 16, BOXFRAMEDBL, NULL, TRUE);

  online.setcursor(22, 12);
  if(index[pos].directory)
    online.printf("Verzeichnis wirklich lschen?");
  else
    online.printf("Datei wirklich lschen?");

  mes[0].type=met::COMMANDBUTTON;
  mes[0].x=27;
  mes[0].y=14;
  mes[0].key=Key_C_Ent;
  mes[0].sizex=6;
  mes[0].text="^Ja";

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=46;
  mes[1].y=14;
  mes[1].key=0;
  mes[1].sizex=6;
  mes[1].text="^Nein";

  mes[2].type=met::END_OF_LIST;

  switch(domenu(&online, mes, 1, TRUE))
   {
   case 0:
    char filename[DIRLEN];
    snprintf(filename, DIRLEN, "%s%s", directory, index[pos].filename);

    if(index[pos].directory) 
     {
      if(rmdir(filename)!=0)
        errorbox(&online, "Lschung fehlgeschlagen!");
     }
    else
     {
      if(remove(filename)!=0)
        errorbox(&online, "Lschung fehlgeschlagen!");
     }

    reread=TRUE;
    break;

   case 1:
    break;
   }
 }

static bool checkfilename(const char *filename)
 {
  if(filename[0]=='.') return TRUE;

  while(*filename!=0 && *filename!=' ' && *filename!='/')
    filename++;

  return *filename!=0;   
 }

void viewfilet::createfile(bool createdir)
 {
  met mes[4];
  char title[TEXTLEN], name[DIRLEN]="";

  if(status.dologout) return;

 startover:
  online.setattr(2, FALSE);
  snprintf(title, TEXTLEN, "%s anlegen", createdir?"Verzeichnis":"Datei");
  online.box(20, 10, 60, 16, BOXFRAMEDBL, title, TRUE);

  online.setcursor(22, 12);
  online.printf("Name:");

  mes[0].type=met::EDIT;
  mes[0].key=0;
  mes[0].x=28;
  mes[0].y=12;
  mes[0].sizex=29;
  mes[0].maxlen=60;
  mes[0].text=name;

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=26;
  mes[1].y=14;
  mes[1].key=Key_C_Ent;
  mes[1].sizex=6;
  mes[1].text="O^k";

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=43;
  mes[2].y=14;
  mes[2].key=0;
  mes[2].sizex=9;
  mes[2].text="^Abbruch";

  mes[3].type=met::END_OF_LIST;

  switch(domenu(&online, mes, 0, TRUE))
   {
   case 1:
    if(checkfilename(name))
      errorbox(&online, "Name ist ungltig!");
    else
     {
      char filename[DIRLEN];
      snprintf(filename, DIRLEN, "%s%s", directory, name);

      if(createdir)
       {
        int oldumask=0;
        if(w_umask!=0) oldumask=umask(w_umask);
        mkdir(filename, 0755);
        if(w_umask!=0) umask(oldumask);
       }
      else
       {
        int oldumask=0;
        if(w_umask!=0) oldumask=umask(w_umask);
        doedit(filename, name);       
        if(w_umask!=0) umask(oldumask);
       }

      reread=TRUE;
     }
    break;

   case MENU_HELP:
    /*
    displayhelp(helpnr);
    online.clrscr();
    */
    if(status.dologout) break;
    goto startover;
   }
 }

void viewfilet::renamefile()
 {
  infobox(&online, "Funktion noch nicht verfuegbar!");
 }

void viewfilet::edit()
 {
  met mes[10];
  int i;

  if(!enable_edit) return;
  if(pos>=anz) return;

  online.box(20, 7, 60, 21, BOXFRAMEDBL, "Datei bearbeiten", TRUE);

  mes[0].type=met::COMMANDBUTTON;
  mes[0].x=23;
  mes[0].y=9;
  mes[0].sizex=33;
  mes[0].text="Datei ^umbenennen";

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=23;
  mes[1].y=11;
  mes[1].sizex=33;
  mes[1].text="Neue Datei ^anlegen";

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=23;
  mes[2].y=13;
  mes[2].sizex=33;
  mes[2].text="^Verzeichnis anlegen";

  mes[3].type=met::COMMANDBUTTON;
  mes[3].x=23;
  mes[3].y=15;
  mes[3].sizex=33;
  mes[3].text="Datei/Verzeichnis ^lschen";

  mes[4].type=met::COMMANDBUTTON;
  mes[4].x=23;
  mes[4].y=17;
  mes[4].sizex=33;
  mes[4].text="Datei ^editieren";

  mes[5].type=met::COMMANDBUTTON;
  mes[5].x=23;
  mes[5].y=19;
  mes[5].sizex=33;
  mes[5].text="^Men verlassen";

  mes[6].type=met::END_OF_LIST;

 schleife:
  i=domenu(&online, mes, 0, TRUE);
  if(i==MENU_HELP) goto schleife;

  switch(i)
   {
   case 0:
    renamefile();
    break;

   case 1:
    createfile(FALSE);
    break;

   case 2:
    createfile(TRUE);
    break;

   case 3:
    del();
    break;

   case 4:
    if(!index[pos].directory && pos!=0) 
     {
      char filename[DIRLEN];
      snprintf(filename, DIRLEN, "%s%s", directory, index[pos].filename);
      doedit(filename, index[pos].filename);
      reread=TRUE;
     }
    break;
   }

  if(!status.dologout) draw(TRUE);
 }

void viewfilet::info()
 {
  online.clrscr();
  //getch();
  if(!status.dologout) draw(TRUE);
 }

void viewfilet::help()
 {
  displayhelp("file");
  if(!status.dologout) draw(TRUE);
 }

void viewfilet::extract()
 {

  if(!status.dologout) draw(TRUE);
 }

void viewfilet::viewfile(char *dir, bool e_edit, int write_umask)
 {
  enable_edit=e_edit;
  w_umask=write_umask;
  strmaxcpy(directory, dir, DIRLEN-1);
  if(directory[strlen(directory)-1]!=DIRCHAR[0])
    strcat(directory, DIRCHAR);
  strmaxcpy(basedirectory, directory, DIRLEN-1);
  layer=0;
  stack[0].pos=0;
  stack[0].scroll=0;

  viewfolder();
 }

void dofile(char *path)
 {
  viewfilet *vf=new viewfilet;
  if(vf==NULL)
    logf.printf("dofile", "FATAL: Zu wenig RAM!");
  else
   {
    vf->viewfile(path, FALSE, 0);
    delete vf;
   }
 }

void dofile(char *path, bool e_edit, int w_umask)
 {
  viewfilet *vf=new viewfilet;
  if(vf==NULL)
    logf.printf("dofile", "FATAL: Zu wenig RAM!");
  else
   {
    vf->viewfile(path, e_edit, w_umask);
    delete vf;
   }
 }
