/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <time.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include <dboxpath.h>
#include <oldui.h>
#include <oldtools.h>
#include <flags.h>
#include <logf.h>
#include <account.h>
#include "online.h"
#include "userlog.h"
#include "verw.h"
#include "chat.h"
#include "extern.h"
#include "settings.h"
#include "openconfig.h"
#include "folder.h"
#include "help.h"
#include "menu.h"
#include "file.h"
#include "fibu.h"
#include "antrag.h"
#include "readmail.h"
#include "wmail.h"
#include "readnews.h"
#include "editor.h"
#include "finger.h"
#include "trace.h"
#include "archie.h"
#include "nslookup.h"
#include "telnet.h"
#include "www.h"

static char *anreden[]=
 {
  "Herr",
  "Frau",
  "n/a",
  NULL
 };

#define MAXINFOS 1000

void showuserlist()
 {

 }

void doedit_www_homepage(const char *basedirectory)
 {
  char tempstr[DIRLEN], tempstr2[DIRLEN];

  strmaxcpy(tempstr, basedirectory, DIRLEN-1);

  if(basedirectory[0]==0) return;

  if(tempstr[strlen(tempstr)-1]!=DIRCHAR[0])
    snprintf(tempstr+strlen(tempstr), DIRLEN-strlen(tempstr), DIRCHAR);

  snprintf(tempstr+strlen(tempstr), DIRLEN-strlen(tempstr),
    "%s" DIRCHAR, status.username);

  snprintf(tempstr2, DIRLEN, "%s.", tempstr);

  if(access(tempstr2, 00)!=0)
   { 
    strmaxcpy(tempstr2, tempstr, DIRLEN-1);
    tempstr2[strlen(tempstr2)-1]=0;

    mkdir(tempstr2, 0755);
    chmod(tempstr2, 0755);
   }

  dofile(tempstr, TRUE, 022);
 }

class doonlinemenut
 {
 public:
  struct infot
   {
    char command[DIRLEN];
    char name[TEXTLEN], subst_name[TEXTLEN], value[TEXTLEN];
    char flag[TEXTLEN], typ[TEXTLEN], data[TEXTLEN];
    int sizex, posx, posy, maxlen, center;
   } infos[MAXINFOS];

  met mes[MAXINFOS];
  int nr[MAXINFOS];
  bool ende, first, save, dowrite, docharge;
  word anz;
  const char *help;
  char helpstr[TEXTLEN];

  void formatvalue(const char *value, const char *typ, char *data);
  void substtext(char *text, pt *profile);
  bool readinfos(const char *name, pt *profile);
  void draw(pt *profile);
  void docommand(const char *command, pt *profile);
  int doonlinemenu(const char *name, pt *profile);

  doonlinemenut() { }
 };

void doonlinemenut::formatvalue(const char *value, const char *typ, char *data)
 {
  data[0]=0;

  if(stricmp(typ, "BOOL")==0)
   {
    strmaxcpy(data, noyes1(atoi(value)), TEXTLEN-1);
   }
  else if(stricmp(typ, "DATE")==0 || stricmp(typ, "DATUM")==0)
   {
    struct tm *tm;
    time_t t;

    t=atol(value);
    if(t!=0)
     {
      tm=localtime(&t);
      snprintf(data, TEXTLEN, "%02u.%02u.%04u", 
        tm->tm_mday, tm->tm_mon+1, tm->tm_year+1900);
     }
   }
  else if(stricmp(typ, "CURRENCY")==0)
   {
    amountt amount=parse_currency_string(value);
    currency_string(data, amount, TRUE);
   }
  else if(strnicmp(typ, "SELECT:", strlen("SELECT:"))==0)
   {
    const char *liste;
    int i;

    i=atoi(value);
    liste=typ+strlen("SELECT:");

    if(stricmp(liste, "charset")==0)
     {
      if(charsets[i]==NULL)
        data[0]=0;
      else
        strmaxcpy(data, charsets[i], TEXTLEN-1);
     }
    else if(stricmp(liste, "anrede")==0)
     {
      if(anreden[i]==NULL)
        data[0]=0;
      else
        strmaxcpy(data, anreden[i], TEXTLEN-1);
     }
    else if(stricmp(liste, "group")==0)
     {
      strmaxcpy(data, value, TEXTLEN-1);
     }
    else
     {
      stringlistt list;
      int j;

      list.parsestring(typ+strlen("SELECT:"));
      for(j=0; list.strings[j]!=NULL; j++);

      if(i<j)
        strmaxcpy(data, list.strings[i], TEXTLEN-1);
     }
   }
  else
    strmaxcpy(data, value, TEXTLEN-1);
 }

void doonlinemenut::substtext(char *text, pt *profile)
 {
  char neu[1000], *in, *out, *tptr, lastchar, 
       tempstr[1000], typecast[TEXTLEN],
       value[TEXTLEN];
  bool klammer;

  in=text;
  out=neu;
  lastchar=0;

  while(*in!=0)
   {
    if(*in=='$' && lastchar!='\\')
     {
      in++;
      strmaxcpy(tempstr, in, 999);
      tptr=tempstr;

      klammer=FALSE;
      while(*tptr!=0 && (isalpha(*tptr) || 
        *tptr=='_' || *tptr=='-' || *tptr=='.' || 
        *tptr=='[' || *tptr==']' || klammer))
       {
        if(*tptr=='[') 
          klammer=TRUE;
        else if(*tptr==']')
          klammer=FALSE;
        tptr++;
       }

      *tptr=0;
      in+=strlen(tempstr);
      if(tempstr[0]=='[')
       {
        tptr=strchr(tempstr, ']');
        if(tptr!=NULL)
         {
          *tptr=0;
          tptr++;
          strmaxcpy(typecast, tempstr+1, TEXTLEN-1);

          if(stricmp(typecast, "CURRENCY")==0)  // HACK
           {
            amountt amount;
            char tempstr2[TEXTLEN];
            profile->get(tptr, value);
            amount.amount=atol(value);
            snprintf(tempstr2, TEXTLEN, "%s.scale", tptr);
            profile->get(tempstr2, value);
            amount.scale=atol(value);
            currency_string(value, amount, FALSE);
           }
          else
            profile->get(tptr, value);
         }
        else
          profile->get(tempstr, value);
       }
      else
       {
        profile->get(tempstr, value);
        typecast[0]=0;
       }

      formatvalue(value, typecast, out);
      out+=strlen(out);
      lastchar='$';
     }
    else
     {
      lastchar=*in;
      *out=*in;
      out++;
      in++;
     }
   }

  *out=0;
  strmaxcpy(text, neu, TEXTLEN-1);
 }

bool doonlinemenut::readinfos(const char *name, pt *profile)
 {
  char tempstr[DIRLEN];
  char t1[TEXTLEN], t2[TEXTLEN];
  FILE *infile;
  bool error=FALSE;

  snprintf(tempstr, DIRLEN, "%s.mnu", name);
  infile=openconfigfile(status, tempstr);
  if(infile==NULL)
   {
    logf.printf("doonlinemenu", "Fatal: Mendatei %s nicht gefunden!", tempstr);
    return TRUE;
   }
  anz=0;

 schleife:
  switch(readconfigline(infile, t1, t2, TEXTLEN-1))
   {
   case 0:
    if(stricmp(t1, "Name")==0)
     {
      anz++;
      infos[anz-1].command[0]=0;
      infos[anz-1].flag[0]=0;
      strmaxcpy(infos[anz-1].name, t2, TEXTLEN-1);
      strmaxcpy(infos[anz-1].subst_name, infos[anz-1].name, TEXTLEN-1);
      substtext(infos[anz-1].subst_name, profile);
      infos[anz-1].sizex=40;
      infos[anz-1].maxlen=TEXTLEN-2;
      infos[anz-1].posx=0;
      infos[anz-1].posy=0;
      infos[anz-1].typ[0]=0;
      infos[anz-1].data[0]=0;
      infos[anz-1].value[0]=0;
      infos[anz-1].center=-1;
     }
    else if(stricmp(t1, "Helpfile")==0)
     {
      help=helpstr;
      strmaxcpy(helpstr, t2, TEXTLEN-1);
     }
    else if(stricmp(t1, "Command")==0 || stricmp(t1, "Befehl")==0)
     {
      if(anz==0) goto error;
      strmaxcpy(infos[anz-1].command, t2, TEXTLEN-1);
     }
    else if(stricmp(t1, "Flag")==0)
     {
      if(anz==0) goto error;
      strmaxcpy(infos[anz-1].flag, t2, TEXTLEN-1);
     }
    else if(stricmp(t1, "Typ")==0)
     {
      if(anz==0) goto error;
      strmaxcpy(infos[anz-1].typ, t2, TEXTLEN-1);
      if(infos[anz-1].typ[0]!=0)
        profile->get(infos[anz-1].name, infos[anz-1].value);
     }
    else if(stricmp(t1, "Sizex")==0)
     {
      if(anz==0) goto error;
      infos[anz-1].sizex=atoi(t2);
     }
    else if(stricmp(t1, "Center")==0)
     {
      if(anz==0) goto error;
      infos[anz-1].center=atoi(t2);
     }
    else if(stricmp(t1, "Maxlen")==0)
     {
      if(anz==0) goto error;
      infos[anz-1].maxlen=max(atoi(t2), TEXTLEN-1);
     }
    else if(stricmp(t1, "Posx")==0)
     {
      if(anz==0) goto error;
      infos[anz-1].posx=atoi(t2);
     }
    else if(stricmp(t1, "Posy")==0)
     {
      if(anz==0) goto error;
      infos[anz-1].posy=atoi(t2);
      if(infos[anz-1].posy<0)
        infos[anz-1].posy+=online.sizey;
     }
    else
      goto error;

    goto schleife;

   case 1:
    break;

   case 2:
   error:
    logf.printf("readexterninfos", "Konfigurationsdatei fehlerhaft");
    error=TRUE;
   }

  fclose(infile);

  return error;
 }

void doonlinemenut::draw(pt *profile)
 {
  byte y=2;
  int menr=0;

  online.setattr(1, FALSE);

  for(int i=1; i<anz; i++)
    if(infos[i].flag[0]==0 || 
       getuserflag(&status.userprofile, &status.gruppe, infos[i].flag))
     {
      if(infos[i].posy!=0)
        y=infos[i].posy;

      if(infos[i].command[0]==0 && infos[i].typ[0]==0)
       {
        strmaxcpy(infos[i].subst_name, infos[i].name, TEXTLEN-1);
        substtext(infos[i].subst_name, profile);

        if(infos[i].posx==0)
          online.setcursor(3, y);
        else
          online.setcursor(infos[i].posx, y);

        if(infos[i].name[0]=='.' && infos[i].name[1]==0)
         {
          for(int j=2; j<online.sizex-2; j++)
            online.printf("");
         }
        else if(infos[i].center==1)
         {
          char *text;
          bool escaped=FALSE;
          int width=0;

          text=infos[i].subst_name;

          while(*text!=0)
           {
            if(*text=='\\' && !escaped)
              escaped=TRUE;
            else if(*text=='~' && !escaped)
             {
             }
            else
             {
              escaped=FALSE;
              if(*text>=' ' || *text<=0) width++;
             }

            text++;   
           }

          if(width>75) width=75;

          online.setcursor((online.sizex-width)/2+1, y);

          printattrstr(&online, infos[i].subst_name, 75, 1);
         }
        else
         {
          printattrstr(&online, infos[i].subst_name, 75, 1);
         }

        y++;
       }
      else
       {
        nr[menr]=i;

        if(infos[i].posx==0)
          mes[menr].x=(online.sizex-(infos[i].sizex+2))/2;
        else
          mes[menr].x=infos[i].posx;

        mes[menr].y=y;
        mes[menr].sizex=infos[i].sizex;
        mes[menr].key=0;

        if(infos[i].typ[0]==0)
         {
          mes[menr].type=met::COMMANDBUTTON;

          strmaxcpy(infos[i].subst_name, infos[i].name, TEXTLEN-1);
          substtext(infos[i].subst_name, profile);
          mes[menr].text=infos[i].subst_name;
          if(infos[i].center<0) infos[i].center=TRUE;
         }
        else if(stricmp(infos[i].typ, "BOOL")==0)
         {
          mes[menr].type=met::COMMANDBUTTON;
          mes[menr].text=(char *)noyes1(atoi(infos[i].value));
          if(infos[i].center<0) infos[i].center=TRUE;
         }
        else if(stricmp(infos[i].typ, "STRING")==0 || stricmp(infos[i].typ, "INTEGER")==0)
         {
          mes[menr].type=met::EDIT;
          mes[menr].text=infos[i].value;
          mes[menr].maxlen=infos[i].maxlen;
          if(infos[i].center<0) infos[i].center=FALSE;
         }
        else if(stricmp(infos[i].typ, "DATE")==0 || 
                stricmp(infos[i].typ, "DATUM")==0)
         {
          formatvalue(infos[i].value, infos[i].typ, infos[i].data);
          mes[menr].type=met::EDIT;
          mes[menr].text=infos[i].data;
          mes[menr].maxlen=11;
          if(infos[i].center<0) infos[i].center=FALSE;
         }
        else if(strnicmp(infos[i].typ, "SELECT:", strlen("SELECT:"))==0)
         {
          formatvalue(infos[i].value, infos[i].typ, infos[i].data);
          mes[menr].type=met::COMMANDBUTTON;
          mes[menr].text=infos[i].data;
          if(infos[i].center<0) infos[i].center=FALSE;
         }

        mes[menr].center=infos[i].center;

        menr++;
        y+=2;
       }
     }

  mes[menr].type=met::END_OF_LIST;
  printtimeout();
  printkonto();
 }

int doonlinemenut::doonlinemenu(const char *name, pt *profile)
 {
  int i=0, j;
  bool redraw, clear=TRUE, locked=FALSE;
  time_t starttime;

  time(&starttime);

  if(docharge)
   {
    char error[TEXTLEN];

    if(checkchargemenu(&status.userprofile, &status.gruppe,
                       name, error))
     {
      errorbox(&online, error);
      return -1;
     }
   }

  if(readinfos(name, profile) || anz<2) return -1;
  if(status.dologout) return -1;

  save=FALSE;

  do
   {
    ende=TRUE;

    strmaxcpy(infos[0].subst_name, infos[0].name, TEXTLEN-1);
    substtext(infos[0].subst_name, profile);

    drawframe(infos[0].subst_name);
    draw(profile);

    redraw=TRUE;
   schleife:
    i=domenu(&online, mes, i, redraw);
    switch(i)
     {
     default:
      if(i>=0)
       {
        j=nr[i];
        if(infos[j].typ[0]!=0)
         {
          if(stricmp(infos[j].typ, "BOOL")==0)
           {
            snprintf(infos[j].value, TEXTLEN, "%u", !atoi(infos[j].value));
            mes[i].text=(char *)noyes1(atoi(infos[j].value));
           }
          else if(stricmp(infos[j].typ, "SELECT:GROUP")==0)
           {
            int gruppe=config.groupdb.index(infos[j].value);

            gruppe++;
            if(config.groupdb[gruppe]==NULL) gruppe=-1;
            strmaxcpy(infos[j].value, config.groupdb[gruppe], TEXTLEN-1);
            formatvalue(infos[j].value, infos[j].typ, infos[j].data);
           }
          else if(strnicmp(infos[j].typ, "SELECT:", strlen("SELECT:"))==0)
           {
            snprintf(infos[j].value, TEXTLEN, "%u", atoi(infos[j].value)+1);
            formatvalue(infos[j].value, infos[j].typ, infos[j].data);
            if(infos[j].data[0]==0) 
             {
              strmaxcpy(infos[j].value, "0", TEXTLEN-1);
              formatvalue(infos[j].value, infos[j].typ, infos[j].data);
             }
           }

          redraw=FALSE;
          
          goto schleife;
         }
 
        ende=FALSE;
        docommand(infos[j].command, profile);
       }
      else if(i==-1 && first)
       {
        if(!doquit()) ende=FALSE;
        i=0;
       }
      break;

     case MENU_HELP:
      displayhelp(help==NULL?"menu":help);
      ende=FALSE;
      break;
     }
    if(status.dologout) { ende=TRUE; clear=FALSE; }
   }
  while(!ende);

  if(clear)
   {
    if(ende && first) online.setattr(1, FALSE);
    online.clrscr();
    online.setcursor(1, 1);
   }

  if(docharge)
    chargemenu(&status.userprofile, name, infos[0].subst_name, 
               starttime, time(NULL));

  if(save)
   {
    bool changed=FALSE, item_changed;
    char tempstr[TEXTLEN];

    if(dowrite)
     {
      profile->lock();
      locked=TRUE;
     }
    else
      profile->read();
 
    for(i=1; i<anz; i++)
      if(infos[i].flag[0]==0 ||
         getuserflag(&status.userprofile, &status.gruppe, infos[i].flag))
       {
        item_changed=FALSE;

        if(stricmp(infos[i].typ, "BOOL")==0 ||
           stricmp(infos[i].typ, "STRING")==0 ||
           stricmp(infos[i].typ, "INTEGER")==0 ||
           stricmp(infos[i].typ, "GROUP")==0)
         {
          item_changed=TRUE;
         }
        else if(strnicmp(infos[i].typ, "SELECT:", strlen("SELECT:"))==0)
         {
          item_changed=TRUE;
         }
        else if(stricmp(infos[i].typ, "DATE")==0)
         {
          struct tm tm;
          time_t t;

          if(infos[i].data[0]==0)
           {
            snprintf(infos[i].value, TEXTLEN, "0");
            item_changed=TRUE;
           }
          else if(sscanf(infos[i].data, "%u.%u.%u", 
            &tm.tm_mday, &tm.tm_mon, &tm.tm_year)==3)
           {
            tm.tm_sec=0;
            tm.tm_min=0;
            tm.tm_hour=0;
            tm.tm_mon--;
            tm.tm_isdst=0;

            if(tm.tm_year>100) 
              tm.tm_year-=1900;
            else if(tm.tm_year<90)
              tm.tm_year+=100;

            t=mktime(&tm);

            snprintf(infos[i].value, TEXTLEN, "%ld", t);
            item_changed=TRUE;
           }
         }

        if(item_changed)
         {
          profile->get(infos[i].name, tempstr);
          if(strcmp(tempstr, infos[i].value)!=0)
           {
            profile->set(infos[i].name, infos[i].value);
            changed=TRUE;
            
            userlogprintf1(status, "doonlinemenu", "%s.%s: \"%s\"->\"%s\"",
              profile->name, infos[i].name, tempstr, 
              stricmp(infos[i].name, "_password")==0?"*":infos[i].value);

            if(profile==&status.userprofile)
             {
              if(stricmp(infos[i].name, "usecolor")==0)
               {
                online.set_usecolor(atoi(status.userprofile|"usecolor"));
               }
              else if(stricmp(infos[i].name, "charset")==0)
               {
                online.set_hw_charset((charsett)atoi(status.userprofile|"charset"));
               }
              else if(stricmp(infos[i].name, "lines")==0)
               {
                online.set_screensize(80, atoi(status.userprofile|"lines"));
               }
             }
           }
         }
       }

    if(changed && dowrite)
      profile->write();

    if(locked)
      profile->unlock();

    return 0;
   }

  return -1;
 }

bool doquit()
 {
  met mes[3];
  byte y=online.sizey/2-5;
  int i;

  drawframe("Quit");
  online.setcursor(30, y);
  online.printf("Box wirklich verlassen?");

  printtimeout();

  mes[0].type=met::COMMANDBUTTON;
  mes[0].x=25;
  mes[0].y=y+3;
  mes[0].sizex=30;
  mes[0].key=0;
  mes[0].text="^Ja, Box verlassen";

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=25;
  mes[1].y=y+5;
  mes[1].sizex=30;
  mes[1].key=0;
  mes[1].text="^Nein, zurck zum Hauptmen";

  mes[2].type=met::END_OF_LIST;

 schleife:
  i=domenu(&online, mes, 0, TRUE);
  if(i==0) return TRUE;
  if(i==MENU_HELP) goto schleife;
  return FALSE;
 }

#define MESSAGELINES 14

void shortmessage(const char *emp)
 {
  met mes[30];
  int i;
  char message[MESSAGELINES][80];
  byte lines;

  for(i=0; i<MESSAGELINES; i++)
   {
    mes[i].type=met::EDIT;
    mes[i].x=3;
    mes[i].y=5+i;
    mes[i].sizex=74;
    mes[i].key=0;
    mes[i].maxlen=78;
    mes[i].text=message[i];
    message[i][0]=0;
   }

  mes[MESSAGELINES].type=met::COMMANDBUTTON;
  mes[MESSAGELINES].x=5;
  mes[MESSAGELINES].y=online.sizey-4;
  mes[MESSAGELINES].sizex=30;
  mes[MESSAGELINES].key=0;
  mes[MESSAGELINES].text="^Ja, Nachricht abschicken";

  mes[MESSAGELINES+1].type=met::COMMANDBUTTON;
  mes[MESSAGELINES+1].x=45;
  mes[MESSAGELINES+1].y=online.sizey-4;
  mes[MESSAGELINES+1].sizex=30;
  mes[MESSAGELINES+1].key=0;
  mes[MESSAGELINES+1].text="^Nein, Nachricht lschen";

  mes[MESSAGELINES+2].type=met::END_OF_LIST;

 startover:
  drawframe("Kurznachricht");
  online.setcursor(3, 3);
  online.printf("Kurznachricht an %s: "
                "Bitte geben Sie Ihren Text ein!",
    emp==NULL?"die Systembetreuung":emp);

  printtimeout();

  i=domenu(&online, mes, 0, TRUE);
  if(i==MENU_HELP)
   {
    goto startover;
   }
  if(i!=MESSAGELINES) return;

  lines=MESSAGELINES;
  while(message[lines-1][0]==0) lines--;

  if(emp==NULL)
   {
    FILE *outfile;
    char filename[DIRLEN];

    i=1;
    do
     {
      snprintf(filename, DIRLEN, DBOXSYSTEMDIR "message.%d", 
        config.dboxdir, i);
      i++;
     }
    while(access(filename, 00)==0);

    if((outfile=fopen(filename, "wt"))!=NULL)
     {
      char tempstr[TEXTLEN];

      formatdate(tempstr, time(NULL));

      fprintf(outfile,
        "Nachricht von: %s\n"
        "Mailboxport:   %u\n"
        "Datum/Uhrzeit: %s\n"
        "\n"
        "Nachrichtentext:\n"
        "\n",
        status.username,
        config.port,
        tempstr);

      for(i=0; i<lines; i++)
        fprintf(outfile, "%s\n", message[i]);

      fprintf(outfile, "\nEnde der Nachricht\n");
      fclose(outfile);
      infobox(&online, "Nachricht wurde verschickt!");
     }
   }
 }

static bool askpassword()
 {
  met mes[3];

  online.setattr(2, FALSE);
  online.box(20, 10, 60, 17, BOXFRAMEDBL, NULL, TRUE);

  online.setcursor(22, 12);
  online.printf("Dieser User hat bereits ein Passwort.");
  online.setcursor(22, 13);
  online.printf("Soll dieses Passwort gelscht werden?");

  mes[0].type=met::COMMANDBUTTON;
  mes[0].x=27;
  mes[0].y=15; 
  mes[0].key=0;
  mes[0].sizex=6;
  mes[0].text="^Ja";

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=46;
  mes[1].y=15;
  mes[1].key=Key_C_Ent;
  mes[1].sizex=6;
  mes[1].text="^Nein";

  mes[2].type=met::END_OF_LIST;

  switch(domenu(&online, mes, 1, TRUE))
   {
   case 0:
    return FALSE;
   }

  return TRUE;
 }

static void doeditsystem(pt *_profile)
 {
  userpt profile;
  met mes[20];
  int i, newi=0;
  char tempstr[TEXTLEN];

  profile.copy_from(_profile);

  if(((atoi(profile|"system") && !getinternalflag(&status.userprofile, &status.gruppe, VERWMENU_NETZ)) ||
     (!atoi(profile|"system") && !getinternalflag(&status.userprofile, &status.gruppe, VERWMENU_POINT))))
   {
    errorbox(&online, "Zugriff verweigert!");
    return;
   }

 startover:
  online.setattr(2, FALSE);
  online.box(10, 5, 70, 21, BOXFRAMEDBL,
             "Point- bzw. Systemeinstellungen", TRUE);

  online.setcursor(12, 7);  online.printf("Typ...............:");
  online.setcursor(12, 8);  online.printf("Netztyp...........:");
  online.setcursor(12, 9);  online.printf("Packer............:");
  online.setcursor(12, 10); online.printf("Alle EMPs.........:");
  online.setcursor(12, 11); online.printf("Postfach aus......:");
  online.setcursor(12, 12); online.printf("Domains...........:");
  online.setcursor(12, 13); online.printf("Hauptdomain.......:");
  online.setcursor(12, 14); online.printf("MAPS Berechtigte..:");

 startover1:
  online.setattr(2, TRUE);
  online.setcursor(32, 13); online.printf("%s", profile|"primarydomain");
  online.setattr(2, FALSE);

  if(atoi(profile|"system"))
   {
    online.setcursor(12, 15); online.printf("Telefonnummer.....:");
    online.setcursor(12, 16); online.printf("Mail Routinggrenze:");
    online.setcursor(12, 17); online.printf("Routingtabelle....:");
   }
  else
   {
    char clr[]="                                                         ";
   
    online.setattr(2, FALSE);
    online.setcursor(12, 15); online.printf(clr);
    online.setcursor(12, 16); online.printf(clr);
    online.setcursor(12, 17); online.printf(clr);
   }

  mes[0].type=met::COMMANDBUTTON;
  mes[0].x=32;
  mes[0].y=7; 
  mes[0].key=0;
  mes[0].sizex=6;
  mes[0].text=(char *)(atoi(profile|"system")?"System":"Point");
  mes[0].center=FALSE;

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=32;
  mes[1].y=8;
  mes[1].sizex=15;
  mes[1].text=profile.get_ptr("netztyp");
  mes[1].center=FALSE;

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=32;
  mes[2].y=9;
  mes[2].sizex=10;
  mes[2].text=profile.get_ptr("packer");
  mes[2].center=FALSE;

  mes[3].type=met::COMMANDBUTTON;
  mes[3].x=32;
  mes[3].y=10;
  mes[3].sizex=4;
  mes[3].text=(char *)noyes1(atoi(profile|"allemps"));

  mes[4].type=met::COMMANDBUTTON;
  mes[4].x=32;
  mes[4].y=11;
  mes[4].sizex=4;
  mes[4].text=(char *)noyes1(atoi(profile|"disablepop"));

  mes[5].type=met::EDIT;
  mes[5].x=32;
  mes[5].y=12;
  mes[5].sizex=35;
  mes[5].maxlen=TEXTLEN-1;
  mes[5].text=profile.get_ptr("domains");

  mes[6].type=met::EDIT;
  mes[6].x=32;
  mes[6].y=14;
  mes[6].sizex=35;
  mes[6].maxlen=TEXTLEN-1;
  mes[6].text=profile.get_ptr("maps");

  if(atoi(profile|"system"))
   {
    mes[7].type=met::EDIT;
    mes[7].x=32;
    mes[7].y=15;
    mes[7].sizex=35;
    mes[7].maxlen=TEXTLEN-1;
    mes[7].text=profile.get_ptr("telefon_data");

    mes[8].type=met::EDIT;
    mes[8].x=32;
    mes[8].y=16;
    mes[8].sizex=13;
    mes[8].maxlen=10;
    mes[8].text=profile.get_ptr("maxlen");

    profile.get("routing", tempstr);
    mes[9].type=met::COMMANDBUTTON;
    mes[9].x=32;
    mes[9].y=17;
    mes[9].sizex=4;
    mes[9].text=(char *)noyes1(atoi(tempstr));

    for(i=10; i<10; i++)
      mes[i].type=met::IGNORE;
   }
  else
    for(i=7; i<10; i++)
      mes[i].type=met::IGNORE;

  mes[10].type=met::COMMANDBUTTON;
  mes[10].x=13;
  mes[10].y=19;
  mes[10].key=Key_C_Ent;
  mes[10].sizex=6;
  mes[10].text="O^k";

  mes[11].type=met::COMMANDBUTTON;
  mes[11].x=23;
  mes[11].y=19;
  mes[11].sizex=9;
  mes[11].text="^Abbruch";

  mes[12].type=met::END_OF_LIST;

 schleife:
  switch(i=domenu(&online, mes, newi, TRUE))
   {
   case 0:
    if(atoi(profile|"system"))
     {
      if(getinternalflag(&status.userprofile, &status.gruppe, VERWMENU_POINT))
        profile.set("system", FALSE);
      else
       {
        errorbox(&online, "Zugriff verweigert!");
        goto startover;
       }
     }
    else
     {
      if(getinternalflag(&status.userprofile, &status.gruppe, VERWMENU_NETZ))
        profile.set("system", TRUE);
      else
       {
        errorbox(&online, "Zugriff verweigert!");
        goto startover;
       }
     }
    newi=i;
    goto startover1;

   case 1:
    newi=i;
     {
      int netztyp=config.nettypedb.index(profile|"netztyp");
      if(netztyp==-1) netztyp=0; else netztyp++;
      if(config.nettypedb[netztyp]==NULL) netztyp=0;
      mes[i].text=(char *)config.nettypedb[netztyp];
      profile.set("netztyp", config.nettypedb[netztyp]);
     }
    goto schleife;

   case 2:
    newi=i;
     {
      int packer=config.netpackerdb.index(profile|"packer");
      if(packer==-1) packer=0; else packer++;
      if(config.netpackerdb[packer]==NULL) packer=0;
      mes[i].text=(char *)config.netpackerdb[packer];
      profile.set("packer", config.netpackerdb[packer]);
     }
    goto schleife;

   case 3:
    newi=i;
    profile.set("allemps", (bool)!atoi(profile|"allemps"));
    mes[i].text=(char *)noyes1(atoi(profile|"allemps"));
    goto schleife;

   case 4:
    newi=i;
    profile.set("disablepop", (bool)!atoi(profile|"disablepop"));
    mes[i].text=(char *)noyes1(atoi(profile|"disablepop"));
    goto schleife;

   case 9:
    newi=i;
    profile.set("routing", (bool)!atoi(profile|"routing"));
    mes[i].text=(char *)noyes1(atoi(profile|"routing"));
    goto schleife;

   case 10:
    if(!_profile->lock())
     {
       {
        char primarydomain[TEXTLEN], *tptr;
        const char *domains=profile|"domains";
        while(*domains==' ' || *domains=='.') domains++;
        strmaxcpy(primarydomain, domains, TEXTLEN-1);
        tptr=primarydomain;
        while(*tptr!=0 && *tptr!=' ' && *tptr!=';' && *tptr!=',') tptr++;
        *tptr=0;
        _profile->set("primarydomain", primarydomain);
       }

      _profile->set("routing", profile|"routing");
      _profile->set("disablepop", profile|"disablepop");
      _profile->set("packer", profile|"packer");
      _profile->set("routing", profile|"routing");
      _profile->set("allemps", profile|"allemps");
      _profile->set("netztyp", profile|"netztyp");
      _profile->set("system", profile|"system");
      _profile->set("domains", profile|"domains");
      _profile->set("maps", profile|"maps");
      _profile->set("telefon_data", profile|"telefon_data");
      _profile->set("maxlen", profile|"maxlen");
      _profile->write();
      _profile->unlock();
     }
    break;
   }
 }

void doonlinemenut::docommand(const char *command, pt *profile)
 {
  char *tptr, *command2;
  char tempstr[1000];

  while(*command==' ' || *command=='\t') command++;
  strmaxcpy(tempstr, command, 999);

  tptr=strchr(tempstr, ';');
  if(tptr!=NULL)
   {
    *tptr=0;
    tptr++;
    command2=tptr;
   }
  else
    command2=NULL;

  tptr=tempstr+strlen(tempstr);
  if(tptr==tempstr) return;
  tptr--;
  while(*tptr==' ' || *tptr=='\t') { *tptr=0; tptr--; }

  tptr=strchr(tempstr, ' ');
  if(tptr!=NULL)
   {
    *tptr=0;
    tptr++;
    while(*tptr==' ' || *tptr=='\t') tptr++;
   }

  if(stricmp(tempstr, "SHELL")==0)
   {
   }
  else if(stricmp(tempstr, "QUIT")==0)
   {
    if(first)
     {
      if(doquit())
        ende=TRUE;
     }
    else
      ende=TRUE;
   }
  else if(stricmp(tempstr, "SAVE")==0)
   {
    ende=TRUE;
    save=TRUE;
   }
  else if(stricmp(tempstr, "RELOGIN")==0)
   {
    ende=TRUE;
    status.relogin=TRUE;
   }
  else if(stricmp(tempstr, "MENU")==0)
   {
    if(tptr==NULL)
      logf.printf("doonlinemenut::docommand",
        "Fatal: MENU bentigt Argument!");
    else
      doonlinemenu(tptr, profile);                                 // VORSICHT!
   }
  else if(stricmp(tempstr, "CALLMENU")==0)
   {
    if(tptr==NULL)
      logf.printf("doonlinemenut::docommand",
        "Fatal: CALLMENU bentigt Argument!");
    else
      ::doonlinemenu(tptr, profile, FALSE, FALSE, dowrite, help);  // VORSICHT!
   }
  else if(stricmp(tempstr, "CALLCHARGEDMENU")==0)
   {
    if(tptr==NULL)
      logf.printf("doonlinemenut::docommand",
        "Fatal: CALLCHARGEDMENU bentigt Argument!");
    else
      ::doonlinemenu(tptr, profile, FALSE, TRUE, dowrite, help);   // VORSICHT!
   }
  else if(stricmp(tempstr, "FILE")==0)
   {
    if(tptr==NULL)
      logf.printf("doonlinemenut::docommand",
        "Fatal: FILE bentigt Argument!");
    else
      dofile(tptr);
   }
  else if(stricmp(tempstr, "DOWNLOAD")==0 ||
          stricmp(tempstr, "DODOWNLOAD")==0)
   {
    if(tptr==NULL)
      logf.printf("doonlinemenut::docommand",
        "Fatal: DOWNLOAD bentigt Argument!");
    else
     {
      char *tptr2, beschreibung[TEXTLEN];

      tptr2=strchr(tptr, ' ');
      if(tptr2!=NULL)
       {
        *tptr2=0;
        tptr2++;
        while(*tptr2==' ') tptr2++;
        snprintf(beschreibung, TEXTLEN, "Beschreibung: %s", tptr2);
        tptr2=beschreibung;
       }

      doonlinedownload(tptr, tptr2);
     }
   }
  else if(stricmp(tempstr, "VIEW")==0 || stricmp(tempstr, "VIEWATTR")==0)
   {
    bool useattr=strlen(tempstr)!=4;

    if(tptr==NULL)
      logf.printf("doonlinemenut::docommand",
                  "Fatal: VIEW/VIEWATTR bentigt Argument!");
    else
     {
      char *tptr2, beschreibung[TEXTLEN];

      tptr2=strchr(tptr, ' ');
      if(tptr2!=NULL)
       {
        *tptr2=0;
        tptr2++;
        while(*tptr2==' ') tptr2++;
        snprintf(beschreibung, TEXTLEN, "%s", tptr2);
        tptr2=beschreibung;
       }

      FILE *infile=fopen(tptr, "rb");
      if(infile!=NULL)
       {
        viewtextfile(&online, infile, tptr2, IBM, useattr);
        fclose(infile);
       }
      else
        logf.printf("doonlinemenut::docommand",
          "Fatal: Datei %s nicht gefunden!", tptr);
     }
   }
  else if(stricmp(tempstr, "EDIT")==0)
   {
    if(tptr==NULL)
      logf.printf("doonlinemenut::docommand",
                  "Fatal: EDIT bentigt Argument!");
    else
     {
      doedit(tptr, "Editor");
     }
   }
  else if(stricmp(tempstr, "MAIL")==0)
    dowritemail(tptr);
  else if(stricmp(tempstr, "VERWALTUNG")==0)
    doverwaltungsmenue();
  else if(stricmp(tempstr, "CHAT")==0)
    doonlinechatmenu();
  else if(stricmp(tempstr, "SETTINGS")==0)
    dosettings(tptr);
  else if(stricmp(tempstr, "EDITSYSTEM")==0)
   {
    if(profile!=NULL)
      doeditsystem(profile);
   }
  else if(stricmp(tempstr, "EDITALIAS")==0)
   {
    bool verwaltung=FALSE;

    if(tptr!=NULL && stricmp(tptr, "Verwaltung")==0)
      verwaltung=TRUE;

    if(profile!=NULL)
      doeditalias(profile, TRUE);
   }
  else if(stricmp(tempstr, "FIBU")==0)
   {
    if(profile!=NULL)
      dofibu(profile->name);
   }
  else if(stricmp(tempstr, "ANTRAG")==0)
    doantrag();
  else if(stricmp(tempstr, "MAILREADER")==0)
    doreadmail();
  else if((stricmp(tempstr, "NEWSREADER")==0 ||
          stricmp(tempstr, "BRETT")==0))
   {
    if(tptr==NULL) 
      dobretter();
    //else
    //  doreadnews(tptr, FALSE); // TODO
   }
  else if(stricmp(tempstr, "SHOWUSERLIST")==0)
    showuserlist();
  else if(stricmp(tempstr, "SHORTMESSAGE")==0)
    shortmessage(tptr);
  else if(stricmp(tempstr, "FINGER")==0)
    dointernet_finger(tptr);
  else if(stricmp(tempstr, "TRACEROUTE")==0)
    dointernet_traceroute(tptr);
  else if(stricmp(tempstr, "NSLOOKUP")==0)
    dointernet_nslookup(tptr);
  else if(stricmp(tempstr, "ARCHIE")==0)
    dointernet_archie();
  else if(stricmp(tempstr, "WWW")==0)
    dointernet_www(tptr);
  else if(stricmp(tempstr, "TELNET")==0)
    dointernet_telnet(tptr);
  else if(stricmp(tempstr, "EDIT_WWW_HOMEPAGE")==0)
   {
    if(tptr==NULL)
      logf.printf("doonlinemenut::docommand",
                  "Fatal: EDIT_WWW_HOMEPAGE bentigt Argument!");
    else
     {
      doedit_www_homepage(tptr);
     }
   }
  else if(stricmp(tempstr, "HELP")==0 || stricmp(tempstr, "HILFE")==0)
   {
    displayhelp(help==NULL?"menu":help);
   }
  else if(stricmp(tempstr, "PASSWORT")==0 || stricmp(tempstr, "PASSWORD")==0)
   {
    char temppw[TEXTLEN], cryptedpw[TEXTLEN*2];
    char lastpasswordchange[TEXTLEN];

    if(profile==NULL) profile=&status.userprofile;

    getplaintextpassword(profile, temppw);
    snprintf(lastpasswordchange, TEXTLEN-1, "%lu", time(NULL));

    if(atoi(status.gruppe|"security")==MAXSECURITY ||
       stricmp(status.username, profile->name)==0)
     {
      if(!changepassword(temppw))
       {
        cryptpassword(temppw, cryptedpw);
        if(!profile->lock())
         {
          profile->set("cryptedpassword", cryptedpw);
          profile->set("lastpasswordchange", lastpasswordchange);
          profile->write();
          profile->unlock();

          if(stricmp(status.username, profile->name)==0)
            userlogprintf1(status, 
              "doonlinemenut::docommand", "Passwort geaendert!");
         }
       }
     }
    else
     {
      if(!askpassword())
       {
        temppw[0]=0;

        if(!changepassword(temppw))
         {
          cryptpassword(temppw, cryptedpw);
          if(!profile->lock())
           {
            profile->set("cryptedpassword", cryptedpw);
            profile->set("lastpasswordchange", lastpasswordchange);
            profile->write();
            profile->unlock();
           }
         }
       }
     }
   }
  else if(stricmp(tempstr, "SHOWUSERLOGFILE")==0)
   {
    if(profile==NULL)
      showuserlog(status.username);
    else
      showuserlog(profile->name);
   }
  else
    logf.printf("doonlinemenut::docommand",
      "Fatal: unbekannter Befehl \"%s\"", tempstr);

  if(command2!=NULL) docommand(command2, profile);
 }

int doonlinemenu(const char *name, pt *profile, 
                 bool first, bool docharge, bool dowrite,
                 const char *help)
 {
  int erg;
  doonlinemenut *dm=new doonlinemenut;

  if(dm==NULL)
   {
    logf.printf("doonlinemenu", "FATAL: Zu wenig RAM!");
    return -1;
   }

  dm->first=first;
  dm->docharge=docharge;
  dm->dowrite=dowrite;
  dm->help=help;

  erg=dm->doonlinemenu(name, profile);
  delete dm;
  
  return erg;
 }

bool docommand(const char *command)
 {
  bool ende;
  doonlinemenut *dm=new doonlinemenut;

  if(dm==NULL)
   {
    logf.printf("docommand", "FATAL: Zu wenig RAM!");
    return TRUE;
   }

  dm->first=FALSE;
  dm->ende=FALSE;
  dm->docommand(command, &status.userprofile);
  ende=dm->ende;
  delete dm;
  return ende;
 }
