/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <signal.h>
#include <getopt.h>

#include <dboxpath.h>
#include <portinfo.h>
#include <oldtools.h>
#include <flags.h>
#include <dboxversion.h>
#include <libdboxversion.h>
#include <logf.h>
#include <nls.h>
#include "online.h"
#include "settings.h"
#include "menu.h"
#include "openconfig.h"
#include "command.h"
#include "html.h"

static void printlogout()
 {
  online.printf("\r\n");
  FILE *file=openconfigfile(status, "logout");
  if(file!=NULL) printtextfile(&online, file);
 }

static void douser(bool use_console)
 {
  online.open();

  if(use_console || status.online2)
   {
    snprintf(status.deviceconfig.name, TEXTLEN, "%u", config.port);
    if(status.deviceconfig.read()) return;

    online.console.close();
    online.console.assign(config.console);
    online.console.open();

    online.set_app_out_charset(IBM);
    online.console.set_hw_charset(config.console_charset);
   }

  if(status.online2)
   {
    if(online.comport.assign(config.comport))
      return;

    online.comport.open();
   }

  strmaxcpy(status.userprofile.name, status.username, TEXTLEN-1);
  status.userprofile.read();
  status.letzteranruf=atol(status.userprofile|"letzteranruf");

  if(!status.userprofile.lock()) 
   {
    status.userprofile.set("letzteranruf", time(NULL));
    status.userprofile.set("anrufe", atol(status.userprofile|"anrufe")+1);

    status.userprofile.write();
    status.userprofile.unlock();
   }

  strmaxcpy(status.gruppe.name, status.userprofile|"gruppe", TEXTLEN-1);
  if(status.gruppe.name[0]!=0) status.gruppe.read();

  snprintf(status.userlogfile, DIRLEN, BOXUSERDIR "logfile",
    config.boxdir, status.username);

  portinfo.read(config.port);
  portinfo.owninfo=portinfo.info;
  portinfo.owninfo.status=portinfot::infot::ONLINE;
  strmaxcpy(portinfo.owninfo.boxname, config.boxname, TEXTLEN-1);
  strmaxcpy(portinfo.owninfo.username, status.username, TEXTLEN-1);
  strmaxcpy(portinfo.info.username, status.username, TEXTLEN-1);
  portinfo.write(config.port);

  online.set_screensize(online.console.sizex, online.console.sizey);

  if(status.online2)
   {
    int lines=atoi(status.userprofile|"lines");
    if(lines<24)
      lines=24;
    else if(lines>50)
      lines=50;

    online.set_screensize(80, lines);
   }

  online.set_hw_charset((charsett)atoi(status.userprofile|"charset"));
  online.set_usecolor(atoi(status.userprofile|"usecolor"));

   {
    char tempstr[TEXTLEN];
    word timeout;

    getuser(&status.userprofile, &status.gruppe, "timeout", tempstr);
    timeout=atoi(tempstr);

    if(timeout==0)
      status.timeouttime=0;
    else
      status.timeouttime=time(NULL)+timeout*60;
   }

   {
    char language[TEXTLEN];
    getuser(&status.userprofile, &status.gruppe, "language", language);
    nls.set_language(language);
   }  

  status.lastkey=time(NULL);

  if(!status.dologout)
   {
    if(atoi(status.userprofile|"dogastsettings")==1)
      dogastsettings();
    else if(atoi(status.userprofile|"dogastsettings")==2)
     {
      bool usecolor;
      charsett charset;

      dogastsettings();
      usecolor=atoi(status.userprofile|"usecolor");
      charset=(charsett)atoi(status.userprofile|"charset");

      if(!status.userprofile.lock())
       {
        status.userprofile.set("dogastsettings", "0");
        status.userprofile.set("usecolor", usecolor);
        status.userprofile.set("charset", (long int)charset);

        status.userprofile.write();
        status.userprofile.unlock();
       }
     }
   }

  if(!status.dologout &&
     !getinternalflag(&status.userprofile, &status.gruppe, DONT_SHOW_NEWSFLASH))
   {
    FILE *file=openconfigfile(status, "news");
    if(file!=NULL)
     {
      online.printf("\r\n");
      viewtextfile(&online, file, "-- Newsflash --", IBM, TRUE);
      fclose(file);
     }
   }

  if(!status.dologout && 
     !getinternalflag(&status.userprofile, &status.gruppe, DONT_SHOW_WELCOME))
   {
    FILE *file=openconfigfile(status, "welcome");
    if(file!=NULL)
     {
      online.printf("\r\n");
      viewtextfile(&online, file, "-- Willkommen --", IBM, TRUE);
      fclose(file);
     }
   }

  if(!status.dologout)
   {
    switch(atoi(status.userprofile|"oberflaeche"))
     {
     case 1:
      docommandline(TRUE);
      break;

     default:
       {
        const char *mainmenu=status.gruppe|"mainmenu";
        if(mainmenu[0]==0) mainmenu="main";
        doonlinemenu(mainmenu, &status.userprofile, 
                   TRUE, FALSE, TRUE, NULL);
       }
     }
   }

  if(!status.killed && (!status.online2 || online.comport.dcd()))
    // kann ich noch was ausgeben?
   {
    status.dologout=FALSE;
    time(&status.lastkey);

    online.setattr(0xff, FALSE);
    if(!getinternalflag(&status.userprofile, &status.gruppe, DONT_SHOW_LOGOUT))
     {
      status.dologout=FALSE;
      printlogout();
     }
  
    if(status.relogin && !status.killed)
     {
      char reloginfile[DIRLEN];
      FILE *outfile;

      snprintf(reloginfile, DIRLEN, DBOXPORTTEMPDIR "relogin", config.dboxdir, config.port);
      outfile=fopen(reloginfile, "w");
      if(outfile!=NULL) fclose(outfile);
     }
   }

  strmaxcpy(status.lastcall, status.username, TEXTLEN-1);

  online.sizex=online.console.sizex;
  online.sizey=online.console.sizey;

  if(status.online2)
    online.comport.close();

  online.close();
 }

static bool parseoptions(int argc, char **argv)
 {
  bool ibm=FALSE, use_console=FALSE;
  char boxname[TEXTLEN]="";

  status.debug=FALSE;
  config.port=0;
  status.dologout=FALSE;
  status.killed=FALSE;
  status.from[0]=0;
  status.username[0]=0;

   {
    char *tptr;
    tptr=getenv("CALL_FROM");
    if(tptr!=NULL) strmaxcpy(status.from, tptr, TEXTLEN-1);
   }

 schleife:
  switch(getopt(argc, argv, "?ho:idf:lp:u:c:s:b:"))
   {
   case '?':
   case 'h':
    goto help;

   case 'o':
    config.port=atoi(optarg);
    if(config.port==0) goto aufruf;
    status.online2=FALSE;
    goto schleife;

   case 'c':
    config.port=atoi(optarg);
    if(config.port==0) goto aufruf;
    status.online2=FALSE;
    use_console=TRUE;
    goto schleife;

   case 'i':
    ibm=TRUE;
    goto schleife;

   case 'd':
    status.debug=TRUE;
    logf.printf("parseoptions", "Debugging on.");
    goto schleife;

   case 'f':
    strmaxcpy(status.from, optarg, TEXTLEN-1);
    goto schleife;

   case 'p':
    config.port=atoi(optarg);
    if(config.port==0) goto aufruf;
    status.online2=TRUE;
    goto schleife;

   case 'u':
    strmaxcpy(status.username, optarg, TEXTLEN-1);
    goto schleife;

   case 's':
    status.linespeed=atol(optarg);
    goto schleife;

   case 'b':
    strmaxcpy(boxname, optarg, TEXTLEN-1);
    goto schleife;

   case -1:
    break;

   case ':':
    fprintf(stderr, "\n\nParameter fehlt! - ");
    goto aufruf;

   default:
    goto aufruf;
   }

  if(config.port==0 || status.username[0]==0)
    goto aufruf;

  if(config.read(boxname)) return TRUE;

  logf.setport(config.port);

  online.console.assign("");
  online.console.open();

  if(ibm)
   {
    online.console.close();
    online.console.linuxconsole=TRUE;
    online.console.open();
   }

  online.sizex=online.console.sizex;
  online.sizey=online.console.sizey;

  douser(use_console);
  //DK
  //online.open();
  //dohtmlview("/home/kroening/test.html", "http://www.test.de/");

  online.flushkeys();
  online.console.close();
  fflush(stdin);

  return FALSE;

 aufruf:
  fprintf(stderr, "Falscher Aufruf!\n");

 help:
  fprintf(stderr,
    "\n"
    "* * ONLINE - DBox Modul fuer Onlinesitzungen - "
    "Version " DBOXVERSIONNR " vom " DBOXVERSION " * *\n"
    "\n"
    "Aufruf:                         Aktion:\n"
    "\n"
    " online [-?] [-h]                Hilfe ausgeben\n"
    " online -oPortnummer -uUsername  stdin/stdout-Offline-Port starten\n"
    " online -cPortnummer -uUsername  Consolen-Offline-Port starten\n"
    " online -pPortnummer -uUsername  Online-Port starten\n"
    " online -l                       Lizenzinformationen anzeigen\n"
    "\n"
    "Weitere Optionen:\n"
    "  -i   IBM Zeichensatz\n"
    "  -d   Debugging on\n"
    "  -f   Caller-ID (Telefon oder Netzadresse)\n"
    "  -s   Linespeed\n"
    "\n"
   );

  return TRUE;
 }

void sig_hangup(int dummy)
 {
  /*
  if(config.port!=0)
   {
    portinfo.owninfo.status=portinfot::infot::OFFLINE;
    portinfo.write(config.port);
   }*/
  logf.printf("sig_hangup", "online received signal");
  status.killed=status.dologout=TRUE;
  online.comport.sigalarm=TRUE;
  online.console.sigalarm=TRUE;
 }

void sig_alarm(int dummy)
 {
  logf.printf("sig_alarm", "received signal");
  status.dologout=TRUE;
  online.comport.sigalarm=TRUE;
  online.console.sigalarm=TRUE;
 }

int main(int argc, char **argv)
 {
  if(init_libdbox(LIBDBOXVERSION, LIBDBOXVERSIONNR)) return 1;

  signal(SIGHUP, sig_hangup);
  signal(SIGINT, sig_hangup);
  signal(SIGQUIT, sig_hangup);
  signal(SIGTERM, sig_hangup);
  signal(SIGALRM, sig_alarm);
  signal(SIGPIPE, SIG_IGN); // wg. socket
  signal(SIGTTIN, SIG_IGN); // wg. Bash
  signal(SIGTTOU, SIG_IGN); // wg. Bash

  parseoptions(argc, argv);

  return 0;
 }

