/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <nls.h>
#include <netdb.h>
#include <dboxpath.h>
#include <logf.h>
#include <portinfo.h>
#include <oldtools.h>
#include <account.h>
#include "device.h"
#include "online.h"
#include "extern.h"

onlinet online;

void drawstatusbar()
 {
  byte x=1;
  char tempstr[TEXTLEN];

  if(!status.online2) return;

  strmaxcpy(tempstr, status.username, 20);
  online.console.setcursor(1, online.console.sizey);
  online.console.setattr(10, FALSE);
  x+=online.console.printf(" %-21s ", tempstr);

  if(status.linespeed==0)
    x+=online.console.printf("------");
  else
    x+=online.console.printf("%6lu", status.linespeed);

  x+=online.console.printf(" BPS  ");

  if(status.timeouttime==0)
    x+=online.console.printf("       ");
  else
   {
    long int min=status.timeouttime-time(NULL)-1;
    if(min<0) min=0; else min=(min/60)+1;
    x+=online.console.printf("%3ld Min", min);
   }

  x+=online.console.printf("  %4s  ", charsets[online.comport.hw_out_charset]);

  if(status.username[0]==0)
    tempstr[0]=0;
  else
    strmaxcpy(tempstr, status.gruppe.name, 19);

  x+=online.console.printf("%-20s ", tempstr);

  x+=online.console.printf("%u ", config.port);

  while(x<online.console.sizex) { x++; online.console.printf(" "); }

  online.console.setattr(1, FALSE);
 }

void drawframe(const char *title)
 {
  online.setattr(1, FALSE);

  online.clrscr();

  drawstatusbar();

  online.box(1, 1, online.sizex, online.sizey-1, BOXFRAMEDBL,
    title, FALSE);
 }

void onlinet::waitkey()
 {
  if(status.online2)
   {
    vector<int> handles;
    handles.push_back(console.fd);
    handles.push_back(comport.fd);
    asyncio.input(handles);
   }
  else
    console.waitkey();
 }

int onlinet::printf(const char *format, ...)
 {
  va_list args;
  char buffer[2000];
  int chars;

  va_start(args, format);

  chars=vsnprintf(buffer, 2000, format, args);

  va_end(args);

  console.printf("%s", buffer);
  if(status.online2 && !status.dologout) comport.printf("%s", buffer);
  return chars;
 }

bool onlinet::readchar(char *ch)
 {
  if(status.online2)
   {
    if(console.readchar(ch)) return TRUE;
    return comport.readchar(ch);
   }

  return console.readchar(ch);
 }

word onlinet::readkey()
 {
  word key;
  time_t uhrzeit;

  time(&uhrzeit);

  if(status.dologout) return 0xffff;

  if(status.online2 && !comport.dcd())
   {
    online.set_app_out_charset(ISO);
    console.setattr(0xff, FALSE);
    console.clrscr();
    console.setcursor(1, 1);
    console.printf("\r\n%s\r\n", nls["online.carrierlost"]);
    online.set_app_out_charset(IBM);
    status.dologout=TRUE;
    return 0xffff;
   }

  if(status.timeouttime!=0 && uhrzeit>status.timeouttime)
   {
   timeout:
    online.set_app_out_charset(ISO);
    online.setattr(0xff, FALSE);
    online.clrscr();
    online.setcursor(1, 1);
    online.printf("%s\r\n", nls["online.timelimit"]);
    online.set_app_out_charset(IBM);
    status.dologout=TRUE;
    return 0xffff;
   }

  // Eingeschlafen?
  if(status.lastkey!=0 && uhrzeit>status.lastkey+5*60) goto timeout;

  key=console.readkey();

  if(key==Key_C_Q)
    goto timeout;

  if(status.online2)
   {
    switch(key)
     {
     case 0:
      key=comport.readkey();
      break;

     case Key_C_R:  // -M
      if(status.timeouttime==0)
        status.timeouttime=uhrzeit+5*60;
      else
        status.timeouttime-=60;
      break;

     case Key_C_T:  // +
      if(status.timeouttime==0)
        status.timeouttime=uhrzeit+5*60;
      else
        status.timeouttime+=60;
      break;

     case Key_C_L:
      goto timeout;
     }
   }

  if(key!=0) status.lastkey=uhrzeit;

  return key;
 }

word onlinet::doreadkey()
 {
  word key;
 
  asyncio.flush_output();

  key=readkey();

  if(key!=0) return key;

  waitkey();

  return readkey();
 }

void onlinet::flushkeys()
 {
  while(console.readkey()!=0); 

  if(status.online2)
    while(!status.dologout && comport.readkey()!=0);
 }

bool onlinet::open()
 {
  col1v=config.col1v;
  col2v=config.col2v;
  col3v=config.col3v;
  col1h=config.col1h;
  col2h=config.col2h;
  col3h=config.col3h;

  comport.col1v=config.col1v;
  comport.col2v=config.col2v;
  comport.col3v=config.col3v;
  comport.col1h=config.col1h;
  comport.col2h=config.col2h;
  comport.col3h=config.col3h;

  console.col1v=config.col1v;
  console.col2v=config.col2v;
  console.col3v=config.col3v;
  console.col1h=config.col1h;
  console.col2h=config.col2h;
  console.col3h=config.col3h;

  return screent::open();
 }

void printtimeout()
 {
  if(status.timeouttime!=0)
   {
    long int min=status.timeouttime-time(NULL)-1;
    if(min<0) min=0; else min=(min/60)+1;
    if(min==1)
     {
      online.setcursor(online.sizex-14, online.sizey-2);
      online.setattr(1, TRUE);
      online.set_app_out_charset(ISO);
      online.printf("%s", nls["online.oneminuteleft"]);
      online.set_app_out_charset(IBM);
      online.setattr(1, FALSE);
     }
    else
     {
      char tempstr[TEXTLEN];
      online.setcursor(online.sizex-21, online.sizey-2);
      snprintf(tempstr, TEXTLEN, nls["minutesleft"], min);
      online.set_app_out_charset(ISO);
      online.printf("%20s", tempstr);
      online.set_app_out_charset(IBM);
     }
   }

   {
    time_t t;
    struct tm *tm;
    time(&t);
    tm=localtime(&t);
    online.setcursor(3, online.sizey-2);
    online.setattr(1, FALSE);
    online.printf("%u.%u.%02u, %02u:%02u    ",
      tm->tm_mday, tm->tm_mon+1, tm->tm_year%100, tm->tm_hour, tm->tm_min);
   }
 }

void printkonto()
 {
  if(getbalance(&status.userprofile).amount!=0)
   {
    char tempstr[100];
    amountt balance=getbalance(&status.userprofile);

    currency_string(tempstr, balance, TRUE);
    online.setcursor(online.sizex-3-strlen(tempstr), 1);
    online.printf(" %s ", tempstr);
   }
 }

void gen_abs(const char *username, char *abs, char *org)
 {
  char tempstr[TEXTLEN*2], absneu[TEXTLEN*4];

  snprintf(absneu, TEXTLEN*2, "%s@", username);

  status.userprofile.get("abs_host", tempstr);

  if(tempstr[0]!=0)
    strcat(absneu, tempstr);
  else if((status.gruppe|"abs_host")[0]!=0)
    strcat(absneu, status.gruppe|"abs_host");
  else if(config.abs_host[0]!=0)
    strcat(absneu, config.abs_host);
  else
    sprintf(absneu+strlen(absneu), "%s.%s", 
      config.boxname, config.domain);

  if((status.userprofile|"vorname")[0]!=0)
   {
    strcpy(tempstr, status.userprofile|"vorname");

    if((status.userprofile|"nachname")[0]!=0)
      sprintf(tempstr+strlen(tempstr), " %s", 
        status.userprofile|"nachname");
   }
  else
    strcpy(tempstr, status.userprofile|"nachname");

  if(tempstr[0]!=0)
    sprintf(absneu+strlen(absneu), " (%s)", tempstr);

  strmaxcpy(abs, absneu, TEXTLEN-1);

  if((status.userprofile|"firma")[0]!=0)
    strmaxcpy(org, status.userprofile|"firma", TEXTLEN-1);
  else
    strmaxcpy(org, config.organization, TEXTLEN-1);
 }

void gen_from(const string &username, string &from, string &org)
 {
  char _from[TEXTLEN], _org[TEXTLEN];
  gen_abs(username.c_str(), _from, _org);
  from=_from;
  org=_org;
 }
