/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <dirent.h>
#include <fcntl.h>
#include <time.h>
#include <sys/stat.h>

#include <dboxpath.h>
#include <logf.h>
#include <oldui.h>
#include <newsdb.h>
#include <tools.h>
#include <oldtools.h>
#include <netdefs.h>
#include "online.h"
#include "userlog.h"
#include "help.h"
#include "openconfig.h"
#include "vfolder.h"
#include "readmail.h"
#include "wmail.h"
#include "internet.h"
#include "readnews.h"
#include "readfolder.h"
#include "readrfcnews.h"
#include "readzcfolder.h"

#define MAXBRETTER 2000

class brettbaumt:public viewfoldert
 {
 public:
  char brett[TEXTLEN];  // Ohne fhrenden /
  byte layer;
  bool verw, newssystem, schnellsuche, nntp;
  char nntp_server[TEXTLEN];
  word schnellsuche_zeichen;

  struct stackt
   {
    dword pos, scroll;
    bool newssystem;
   } stack[100];

  struct indexentryt
   {
    char *name, *description;
    bool highlight;
   } index[MAXBRETTER];

  struct activeentryt
   {
    char *name;
    char type;
    activeentryt *n;
   } *active;

  void readpermissions(char **permissions);
  void freepermissions(char *permissions);
  bool checkpermissions(const char *brett, const char *permissions);

  virtual void readindex();
  virtual void freeindex();
  virtual void displayline(dword line);
  virtual void draw(bool complete);
  virtual void setcursor();
  //virtual void viewfolder();

  virtual void read();
  virtual void help();
  virtual void info();
  //virtual void mark() { }
  //virtual void extract() { }
  virtual void del();
  virtual void key(word k);

  void neue_nachricht();
  void goback();
  void doread_newssystem(folderpt *fp, bool enter);
  void doread(bool enter);
  void editbrett(folderpt *fp);
  void eintragen();
  void umbenennen();

  void readactive();
  void freeactive();

  brettbaumt()
   {
    brett[0]=0;
    layer=0;
    verw=newssystem=nntp=FALSE;
    nntp_server[0]=0;
    active=NULL;
    schnellsuche=FALSE;
    stack[0].pos=0;
    stack[0].scroll=0;
    stack[0].newssystem=FALSE;
   }
 };

static void domakedir(const char *dir)
 {
  char *tptr;
  char d[DIRLEN];
  bool again;

  do
   {
    strmaxcpy(d, dir, DIRLEN-1);
    again=FALSE;

   schleife:
    if(mkdir(d, 0700)!=0)
     {
      tptr=strrchr(d, DIRCHAR[0]);
      if(tptr==NULL) return;
      *tptr=0;
      if(strlen(d)!=strlen(dir))
       {
        again=TRUE;
        goto schleife;
       }
     }
   }
  while(again);
 }

static void dodeldir(const char *dir)
 {
  char *tptr;
  char d[DIRLEN];

  deldir(dir);
  strmaxcpy(d, dir, DIRLEN-1);
  if(d[0]==0) return;
  if(d[strlen(d)-1]==DIRCHAR[0])
    d[strlen(d)-1]=0;

 schleife:
  if(rmdir(d)==0)
   {
    tptr=strrchr(d, DIRCHAR[0]);
    if(tptr==NULL) return;
    *tptr=0;
    goto schleife;
   }
 }

void brettbaumt::readpermissions(char **permissions)
 {
  char filename[DIRLEN];

  if(status.gruppe.read())
    *permissions=NULL;
  else
   {
    snprintf(filename, DIRLEN, "%sbretter", status.gruppe.getpath());
    *permissions=readtextconfigfile(filename);
   }
 }

void brettbaumt::freepermissions(char *permissions)
 {
  if(permissions!=NULL)
    free(permissions);
 }

bool brettbaumt::checkpermissions(const char *brett, const char *permissions)
 {
  char empmask[TEXTLEN];
  bool read, write, sysop;
  userflagst userflags;
  char brett2[TEXTLEN];

  snprintf(brett2, TEXTLEN, "%s/*", brett);

  if(permissions==NULL) return TRUE;

  while(*permissions!=0)
   {
    getpermissions(permissions, empmask, &read, &write, &sysop, &userflags);

    if(!checkuserflags(&status.userprofile, &status.gruppe, &userflags))
     {
      if(stricmp(brett2, empmask)==0)
        return !read;

      if(checkjoker(empmask, brett, TRUE))
        return !read;

      if(checkjoker(brett2, empmask, TRUE))
        if(read) return FALSE;
     }

    permissions+=strlen(permissions)+1;
   }

  return TRUE;
 }

static int sort_string(const void *a, const void *b)
 {
  brettbaumt::indexentryt *aa, *bb;

  aa=(brettbaumt::indexentryt *)a;
  bb=(brettbaumt::indexentryt *)b;

  return stricmp(aa->name, bb->name);
 }

void brettbaumt::readindex()
 {
  folderpt fp;
  DIR *dir;
  struct dirent *ent;
  char filename[DIRLEN];
  dword i;
  char *permissions;

  pos=stack[layer].pos;
  scroll=stack[layer].scroll;
  newssystem=stack[layer].newssystem;

 schleife:
  anz=layer;
  for(i=0; i<anz; i++)
   {
    index[i].name=NULL;
    index[i].description=NULL;
    index[i].highlight=FALSE;
   }

  readpermissions(&permissions);

  snprintf(fp.name, TEXTLEN, "/%s", brett);
  fp.setfilename();

  if(newssystem)
   {
    activeentryt *ae;
    byte len;
    char cmptext[TEXTLEN], *tptr;

    snprintf(cmptext, TEXTLEN, "%s.", fp.name+1);
    tptr=cmptext;
    while(*tptr!=0)
     {
      if(*tptr=='/') *tptr='.';
      tptr++;
     }

    len=strlen(cmptext);

    ae=active;
    while(ae!=NULL)
     {
      if(strncmp(cmptext, ae->name, len)==0)
       {
        char b[TEXTLEN], *tptr;

        snprintf(b, TEXTLEN, "/%s", ae->name);
        tptr=strchr(b+1+len, '.');
        if(tptr!=NULL) *tptr=0;
        
        tptr=b;
        while(*tptr!=0)
         {
          if(*tptr=='.') *tptr='/';
          tptr++;
         }

        tptr=b+1+len;

        for(i=0; i<anz; i++)
          if(index[i].name!=NULL && strcmp(index[i].name, tptr)==0) break;

        if(i==anz && !checkpermissions(b, permissions))
         {
          index[anz].name=strdup(tptr);
          if(index[anz].name==NULL) abort();
          index[anz].description=NULL;
          index[anz].highlight=FALSE;
          anz++;
         }
       }

      ae=ae->n;
     } 
   }
  else
   {
    snprintf(filename, DIRLEN, "%s.", fp.getpath());
    if((dir=opendir(filename))==NULL) return;

    while((ent=readdir(dir))!=NULL && anz!=MAXBRETTER)
      if(ent->d_name[0]!='.')
       {
        struct stat statbuf;

        snprintf(filename, DIRLEN, "%s%s", fp.getpath(), ent->d_name);
        if(stat(filename, &statbuf)==0)
          if(statbuf.st_mode&S_IFDIR)
           {
            char b[TEXTLEN];
            if(layer==0)
              snprintf(b, TEXTLEN, "/%s", ent->d_name);
            else
              snprintf(b, TEXTLEN, "/%s/%s", brett, ent->d_name);
  
            if(!checkpermissions(b, permissions))
             {
              index[anz].name=strdup(ent->d_name);
              if(index[anz].name==NULL) 
                logf.internalerror("readindex", "strdup failed");

              index[anz].highlight=FALSE;
              index[anz].description=NULL;

              if(verw)
               {
               }
              else
               {
                snprintf(filename, DIRLEN, "%s%s" DIRCHAR "index", fp.getpath(), ent->d_name);

                if(stat(filename, &statbuf)==0)
                  if(statbuf.st_mtime>status.letzteranruf)
                    index[anz].highlight=TRUE;

                folderpt tmp_bp(b);
                if(!tmp_bp.read() && (tmp_bp|"betreff")[0]!=0)
                 {
                  index[anz].description=strdup(tmp_bp|"betreff");
                  if(index[anz].description==NULL)
                    logf.internalerror("readindex", "strdup failed");
                 }
               }

              anz++;
             }
           }
       }

    qsort(index+layer, anz-layer, sizeof(indexentryt), sort_string);

    closedir(dir);

    if(layer==anz)
     {
      infobox(&online, "Brett ist leer!");
      if(layer!=0)
       {
        goback();
        goto schleife;
       }
     }
   }

  freepermissions(permissions);

 }

void brettbaumt::freeindex()
 {
  dword i;
  for(i=0; i<anz; i++)
   {
    if(index[i].name!=NULL)
      free(index[i].name);

    if(index[i].description!=NULL)
      free(index[i].description);
   }
 }

void brettbaumt::displayline(dword line)
 {
  char tempstr[1000]="";
  dword i;

  if(line>=layer)
   {
    if(line>=anz) return;

    for(i=0; i<layer; i++)
      strcat(tempstr, "  ");

    if(line==(anz-1))
      strcat(tempstr, "");
    else
      strcat(tempstr, "");

    strcat(tempstr, index[line].name);

    if(index[line].highlight) 
      snprintf(tempstr+strlen(tempstr), 1000-strlen(tempstr), " *");

    if(index[line].description!=NULL)
     {
      i=strlen(tempstr);
      for(; i<39; i++) strcat(tempstr, " ");
      strcat(tempstr, index[line].description);
      tempstr[online.sizex-1]=0;
     }
   }
  else
   {
    char tempstr2[TEXTLEN], *tptr;

    for(i=0; i<line; i++)
      strcat(tempstr, "  ");

    strcat(tempstr, "");

    tptr=brett;
    for(i=0; i<line; i++)
     {
      tptr=strchr(tptr, '/');
      if(tptr==NULL) abort();
      tptr++;
     }

    strmaxcpy(tempstr2, tptr, TEXTLEN-1);
    tptr=strchr(tempstr2+1, '/');
    if(tptr!=NULL) *tptr=0;

    strcat(tempstr, tempstr2);
   }

  viewfoldert::displayline(line, tempstr);
 }

void brettbaumt::draw(bool complete)
 {
  if(verw)
    strmaxcpy(title, "Bretterverwaltung - drcken Sie ? fr Hilfe", TEXTLEN-1);
  else
    strmaxcpy(title, "Brettbaum - drcken Sie ? fr Hilfe", TEXTLEN-1);

  viewfoldert::draw(complete);

  setcursor(); 
 }

void brettbaumt::setcursor()
 {
  if(schnellsuche)
    online.setcursor(schnellsuche_zeichen+(layer+1)*2+1, 2+pos-scroll);
  else
    online.setcursor(1, 2+pos-scroll);
 }

void brettbaumt::read()
 {
  doread(TRUE);
 }

void brettbaumt::doread_newssystem(folderpt *fp, bool enter)
 {
  activeentryt *ae;
  byte len;
  bool brett_gef=FALSE, unterbrett_gef=FALSE;
  char cmptext1[TEXTLEN], cmptext2[TEXTLEN], *tptr;

  strmaxcpy(cmptext1, fp->name+1, TEXTLEN-1);
  tptr=cmptext1;
  while(*tptr!=0)
   {
    if(*tptr=='/') *tptr='.';
    tptr++;
   }

  snprintf(cmptext2, TEXTLEN, "%s.", cmptext1);
  len=strlen(cmptext2);

  ae=active;
  while(ae!=NULL)
   {
    if(strcmp(cmptext1, ae->name)==0)
      brett_gef=TRUE;
    else if(strncmp(cmptext2, ae->name, len)==0)
      unterbrett_gef=TRUE;
    ae=ae->n;
   } 

  if((enter && brett_gef) || !unterbrett_gef)
   {
    doreadnews(fp);
    if(!status.dologout) draw(TRUE);
   }
  else
   {
    strmaxcpy(brett, fp->name+1, TEXTLEN-1);
    stack[layer].pos=pos;
    stack[layer].scroll=scroll;
    stack[layer].newssystem=newssystem;
    layer++;
    stack[layer].pos=layer;
    stack[layer].scroll=0; // TODO!
    stack[layer].newssystem=TRUE;
    reread=TRUE;
   }
 }

void brettbaumt::doread(bool enter)
 {
  if(pos>=layer)
   {
    char tempstr[TEXTLEN];
    folderpt fp;

    if(layer==0)
      strmaxcpy(tempstr, index[pos].name, TEXTLEN-1);
    else
      snprintf(tempstr, TEXTLEN, "%s/%s", brett, index[pos].name);

    snprintf(fp.name, TEXTLEN, "/%s", tempstr);
    fp.setfilename();

    if(newssystem)
     {
      fp.set("type", nntp?GROUPTYPE_NNTP:GROUPTYPE_LOCAL_NEWS);
      fp.set("nntp_server", nntp_server);
      doread_newssystem(&fp, enter);
     }
    else
     {
      if(fp.read())
        fp.set("type", GROUPTYPE_NONE);

      switch(atoi(fp|"type"))
       {
       case GROUPTYPE_LOCAL_NEWS:
       case GROUPTYPE_NNTP:
        if(verw)
         {
          editbrett(&fp);
          if(!status.dologout) draw(TRUE);
         }
        else
         {
          nntp=(atoi(fp|"type")==GROUPTYPE_NNTP);
          strmaxcpy(nntp_server, fp|"nntp_server", TEXTLEN-1);
          doread_newssystem(&fp, enter);
         }
        break;

       case GROUPTYPE_ZCONNECT:
        if(!enter)
         {
          // Pruefen, ob "Unterbretter" da sind
         }

        if(verw)
         {
          if(enter)
           {
            editbrett(&fp);
            if(!status.dologout) draw(TRUE);
           }
         }
        else
         {
          doreadnews(&fp);
          if(!status.dologout) draw(TRUE);
         }
        break;

       case GROUPTYPE_FILESYSTEM:
        break;

       case GROUPTYPE_NONE:
        if(verw && enter)
         {
          editbrett(&fp);
          if(!status.dologout) draw(TRUE);
         }
        else
         {
          strmaxcpy(brett, tempstr, TEXTLEN-1);
          stack[layer].pos=pos;
          stack[layer].scroll=scroll;
          stack[layer].newssystem=newssystem;
          layer++;
          stack[layer].pos=layer;
          stack[layer].scroll=0; // TODO!
          stack[layer].newssystem=FALSE;
          reread=TRUE;
         }
        break;
       }
     }
   }
  else
   {
    while(pos<layer) goback();
    reread=TRUE;
   }
 }

void brettbaumt::help()
 {
  displayhelp(verw?"verw/tree.html":"news/tree.html");
  if(!status.dologout) draw(TRUE);
 }

void brettbaumt::info()
 {

  if(!status.dologout) draw(TRUE);
 }

void brettbaumt::key(word k)
 {
  if(schnellsuche)
   { 
    if(k==Key_BS || k==Key_Del)
     {
      if(schnellsuche_zeichen!=0)
       {
        schnellsuche_zeichen--;
        setcursor();
       }

      return;
     }
    else if(k==Key_Ent || k==Key_Esc)
     {
      schnellsuche=FALSE;
      setcursor();
      return;
     }
    else if(lo(k)<' ')
     {
      schnellsuche=FALSE;
      setcursor();
     } 
    else if(lo(k)=='/')
     {
      k=Key_Ent;
      schnellsuche_zeichen=0;
     }
    else
     {
      dword i;
      char string[TEXTLEN];

      strmaxcpy(string, index[pos].name, TEXTLEN-1);
      string[schnellsuche_zeichen]=lo(k);
      string[schnellsuche_zeichen+1]=0;

      for(i=layer; i<anz; i++)
        if(strnicmp(index[i].name, string, schnellsuche_zeichen+1)==0)
          break;

      if(i<anz)
       {
        schnellsuche_zeichen++;
        setpos(i, TRUE);
        setcursor();
        return;
       }
     }
   }

  switch(lo(k))
   {
   case 0:
    switch(k)
     {
     case Key_Lft:
      if(layer!=0)
       {
        goback();
        reread=TRUE;
       }
      return;

     case Key_Rgt:
      doread(FALSE);
      return;
     }
    break;

   case 'b':
   case 'B':
   case 's':
   case 'S':
    if(!verw)
      neue_nachricht();
    break;

   case 'e':
    if(verw)
      eintragen();
    break;

   case 'u':
    if(verw)
      umbenennen();
    break;

   case '/':
    if(anz>0)
     {
      schnellsuche=TRUE;
      schnellsuche_zeichen=0;
      if(pos<layer)
       {
        dword oldpos=pos;
        pos=layer;
        displayline(oldpos);
        displayline(pos);
       }
      setcursor();
     }
    break;
   }
  viewfoldert::key(k);
 }

void brettbaumt::neue_nachricht()
 {
  folderpt fp;

  if(layer==0)
    snprintf(fp.name, TEXTLEN, "/%s", index[pos].name);
  else
    snprintf(fp.name, TEXTLEN, "/%s/%s", brett, index[pos].name);

  if(newssystem)
   {
   do_newssystem:
    activeentryt *ae;
    byte len;
    bool brett_gef=FALSE, unterbrett_gef=FALSE;
    char cmptext1[TEXTLEN], cmptext2[TEXTLEN], *tptr;

    strmaxcpy(cmptext1, fp.name+1, TEXTLEN-1);
    tptr=cmptext1;
    while(*tptr!=0)
     {
      if(*tptr=='/') *tptr='.';
      tptr++;
     }

    snprintf(cmptext2, TEXTLEN, "%s.", cmptext1);
    len=strlen(cmptext2);

    ae=active;
    while(ae!=NULL)
     {
      if(strcmp(cmptext1, ae->name)==0)
        brett_gef=TRUE;
      else if(strncmp(cmptext2, ae->name, len)==0)
        unterbrett_gef=TRUE;
      ae=ae->n;
     } 

    if(!brett_gef)
     {
      if(unterbrett_gef) doread(FALSE);
      return;
     }

    if(get_news_folder_grants(fp.name, status.userprofile, status.gruppe,
                              FALSE, TRUE, FALSE))
      errorbox(&online, "Schreibzugriff verweigert!");
    else
      dowritemail(fp.name);

    if(!status.dologout) draw(TRUE);
    return;
   }

  if(fp.read())
    doread(FALSE);
  else
   {
    switch(atoi(fp|"type"))
     {
     case GROUPTYPE_LOCAL_NEWS:
     case GROUPTYPE_NNTP:
      goto do_newssystem;

     case GROUPTYPE_FILESYSTEM:
      errorbox(&online, "Schreibzugriff verweigert!");
      if(!status.dologout) draw(TRUE);
      break;

     case GROUPTYPE_NONE:
      doread(FALSE);
      break;

     case GROUPTYPE_ZCONNECT:
      if(get_news_folder_grants(fp.name, status.userprofile, status.gruppe,
                                FALSE, TRUE, FALSE))
        errorbox(&online, "Schreibzugriff verweigert!");
      else
        dowritemail(fp.name);

      if(!status.dologout) draw(TRUE);
      break;
     }
   }
 }

void brettbaumt::goback()
 {
  if(layer==1)
   {
    brett[0]=0;
    layer=0;
   }
  else
   {
    char *tptr;
    tptr=strrchr(brett, '/');
    *tptr=0;
    layer--;
   }
 }

void brettbaumt::del()
 {
  folderpt fp;
  met mes[10];
  char tempstr[TEXTLEN];

  if(!verw) return;
  snprintf(fp.name, TEXTLEN, "/%s/%s", brett, index[pos].name);

  if(fp.read())
   {
    errorbox(&online, "Dies ist kein Brett!");
    if(!status.dologout) draw(TRUE);
    return;
   }

  online.setattr(2, FALSE);
  online.box(10, 8, 70, 16, BOXFRAMEDBL, "Brett lschen", TRUE);

  online.setcursor(13, 10);
  online.printf("Brettname: ");
  online.setattr(2, TRUE);
  strmaxcpy(tempstr, fp.name, 45);
  online.printf("%s", tempstr);
  online.setattr(2, FALSE);
  online.setcursor(28, 12);
  online.printf("Brett wirklich austragen?");

  mes[0].type=met::COMMANDBUTTON;
  mes[0].x=26;
  mes[0].y=14;
  mes[0].key=Key_C_Ent;
  mes[0].sizex=6;
  mes[0].text="O^k";

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=43;
  mes[1].y=14;
  mes[1].key=0;
  mes[1].sizex=9;
  mes[1].text="^Abbruch";

  mes[2].type=met::END_OF_LIST;

 schleife:
  switch(domenu(&online, mes, 1, TRUE))
   {
   case 0:
    dodeldir(fp.getpath());
    userlogprintf1(status, "brettbaumt::del", 
      "Brett %s wurde durch User %s gelscht!",
      fp.name, status.username);
    reread=TRUE;
    pos=0;
    scroll=0;
    layer=0;
    stack[0].pos=0;
    stack[0].scroll=0;
    stack[0].newssystem=FALSE;
    brett[0]=0;
    break;

   case MENU_HELP:
    goto schleife;
   }

  online.setattr(1, FALSE);

  if(!status.dologout && !reread) draw(TRUE);
 }

void brettbaumt::editbrett(folderpt *fp)
 {
  int i, newi=20;
  met mes[30];
  bool redraw;

 startover:
  if(status.dologout) return;

  online.setattr(2, FALSE);
  online.box(10, 4, 70, 18, BOXFRAMEDBL, fp->name, TRUE);

  online.setcursor(12, 6);  online.printf("Betreff.......:");
  online.setcursor(12, 7);  online.printf("Memo..........:");
  online.setcursor(12, 8);  online.printf("Bretttyp......:");
  online.setcursor(12, 9);  online.printf("Betreuuer.....:");
  online.setcursor(12, 10); online.printf("Immer extern..:");
  online.setcursor(12, 11); online.printf("NNTP-Server...:");
  online.setcursor(12, 12); online.printf("Verzeichnis...:");

  mes[0].type=met::EDIT;
  mes[0].x=28;
  mes[0].y=6;
  mes[0].sizex=39;
  mes[0].maxlen=TEXTLEN-1;
  mes[0].text=fp->get_ptr("betreff");

  mes[1].type=met::EDIT;
  mes[1].x=28;
  mes[1].y=7;
  mes[1].sizex=39;
  mes[1].maxlen=TEXTLEN-1;
  mes[1].text=fp->get_ptr("memo");

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=28;
  mes[2].y=8;
  mes[2].sizex=20;
  mes[2].center=FALSE;
  mes[2].text=(char *)config.grouptypedb[atoi(*fp|"type")];

  mes[3].type=met::EDIT;
  mes[3].x=28;
  mes[3].y=9;
  mes[3].sizex=39;
  mes[3].maxlen=TEXTLEN-1;
  mes[3].disabled=(atoi(*fp|"type")!=GROUPTYPE_ZCONNECT);
  mes[3].text=fp->get_ptr("betreuer");

  mes[4].type=met::COMMANDBUTTON;
  mes[4].x=28;
  mes[4].y=10;
  mes[4].sizex=4;
  mes[4].disabled=(atoi(*fp|"type")!=GROUPTYPE_ZCONNECT);
  mes[4].text=(char *)noyes2(atoi(*fp|"forceextern"));

  mes[5].type=met::EDIT;
  mes[5].x=28;
  mes[5].y=11;
  mes[5].sizex=39;
  mes[5].maxlen=TEXTLEN-1;
  mes[5].disabled=(atoi(*fp|"type")!=GROUPTYPE_NNTP);
  mes[5].text=fp->get_ptr("nntp_server");

  mes[6].type=met::EDIT;
  mes[6].x=28;
  mes[6].y=12;
  mes[6].sizex=39;
  mes[6].maxlen=TEXTLEN-1;
  mes[6].disabled=(atoi(*fp|"type")!=GROUPTYPE_FILESYSTEM);
  mes[6].text=fp->get_ptr("fs_path");

  for(i=7; i<20; i++)
    mes[i].type=met::IGNORE;

  mes[20].type=met::COMMANDBUTTON;
  mes[20].x=15;
  mes[20].y=16;
  mes[20].key=Key_C_Ent;
  mes[20].sizex=6;
  mes[20].text="O^k";

  mes[21].type=met::COMMANDBUTTON;
  mes[21].x=25;
  mes[21].y=16;
  mes[21].sizex=9;
  mes[21].text="^Abbruch";

  mes[22].type=met::COMMANDBUTTON;
  mes[22].x=38;
  mes[22].y=16;
  mes[22].sizex=7;
  mes[22].text="^Hilfe";

  mes[23].type=met::END_OF_LIST;

  redraw=TRUE;

 schleife:
  i=domenu(&online, mes, newi, redraw);
  redraw=FALSE;

  switch(i)
   {
   case 2:
    fp->set("type", atoi(*fp|"type")+1);
    if(config.grouptypedb[atoi(*fp|"type")]==NULL) fp->set("type", 0);
    mes[i].text=(char *)config.grouptypedb[atoi(*fp|"type")];
    newi=i;
    goto startover;

   case 4:
    fp->set("forceextern", (bool)!atoi(*fp|"forceextern"));
    mes[i].text=(char *)noyes2(*fp|"forceextern");
    newi=i;
    goto schleife;

   case 20:
     {
      folderpt _fp(fp->name);
      if(!_fp.lock())
       {
        _fp.copy_from(fp);
        _fp.write();
        _fp.unlock();
       }
     }
    break;

   case 22:
    newi=i;
    displayhelp("verw/folder.html");
    goto startover;

   case MENU_HELP:
    newi=20;
    displayhelp("verw/folder.html");
    goto startover;
   }
 }

void brettbaumt::eintragen()
 {
  char brettname[TEXTLEN]="";
  met mes[3];
  char *tptr;

  online.setattr(2, FALSE);
  online.box(20, 10, 60, 16, BOXFRAMEDBL, "Brett eintragen", TRUE);

  online.setcursor(25, 12);
  online.printf("Brettname:");

  mes[0].type=met::EDIT;
  mes[0].key=0;
  mes[0].x=36;
  mes[0].y=12;
  mes[0].sizex=16;
  mes[0].maxlen=40;
  mes[0].text=brettname;

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=26;
  mes[1].y=14;
  mes[1].key=Key_C_Ent;
  mes[1].sizex=6;
  mes[1].text="O^k";

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=43;
  mes[2].y=14;
  mes[2].key=0;
  mes[2].sizex=9;
  mes[2].text="^Abbruch";

  mes[3].type=met::END_OF_LIST;

 schleife:
  switch(domenu(&online, mes, 0, TRUE))
   {
   case 1:
    dnstring(brettname);
    tptr=brettname;
    while(*tptr!=0)
     {
      if(!zcfoldercharisok(*tptr)) break;
      tptr++;
     }

    if(*tptr==0 && brettname[0]!='.' && brettname[0]!='_' &&
       !isdigit(brettname[0]) && brettname[0]=='/')
     {
      folderpt profile(brettname);
      if(!profile.read())
       {
        errorbox(&online, "Brett existiert bereits!");
       }
      else
       {
        profile.clear();
        strmaxcpy(profile.name, brettname, TEXTLEN-1);
        domakedir(profile.getpath());
        if(profile.lock())
          errorbox(&online, "Eintragung fehlgeschlagen!");
        else
         {
          profile.set("type", GROUPTYPE_ZCONNECT);
          profile.write();
          profile.unlock();

          userlogprintf1(status, "brettbaumt::eintragen",
            "Brett %s durch %s eingetragen", 
             brettname, status.username);
          reread=TRUE;
          editbrett(&profile);
         }
       }
     }
    else
     {
      errorbox(&online, "Brettname enthlt ungltiges Zeichen!");
     }
    break;

   case MENU_HELP:
    goto schleife;
   }

  online.setattr(1, FALSE);
  if(!status.dologout && !reread) draw(TRUE);
 }

void brettbaumt::umbenennen()
 {

 }

void brettbaumt::readactive()
 {
  FILE *infile;
  activeentryt *ae, *last=NULL;
  char tempstr[1000], *tptr;

  active=NULL;
  infile=openconfigfile(status, "active");
  if(infile==NULL) return;

  while(fgets(tempstr, 900, infile)!=NULL)
   {
    tptr=strchr(tempstr, ' ');
    if(tptr==NULL) continue;
    *tptr=0;
    tptr=strchr(tptr+1, ' ');
    if(tptr==NULL) continue;
    tptr=strchr(tptr+1, ' ');
    if(tptr==NULL) continue;
    ae=new activeentryt;
    ae->type=tptr[1];
    ae->name=strdup(tempstr);
    if(ae->name==NULL) abort();
    ae->n=NULL;
    if(active==NULL)
      active=ae;
    else
      last->n=ae;

    last=ae;
   }
  
  fclose(infile);
 }

void brettbaumt::freeactive()
 {
  activeentryt *ae, *n;

  ae=active;
  while(ae!=NULL)
   {
    n=ae->n;
    free(ae->name);
    delete ae;
    ae=n;
   }

  active=NULL;
 }

void dobretter()
 {
  brettbaumt bb;
  bb.readactive();
  bb.viewfolder();
  bb.freeactive();
 }

void dobretter_verw()
 {
  brettbaumt bb;
  bb.verw=TRUE;
  bb.viewfolder();
 }

void doreadnews(folderpt *fp)
 {
  switch(atoi(*fp|"type"))
   {
   case GROUPTYPE_LOCAL_NEWS:
   case GROUPTYPE_NNTP:
     {
      readrfcnewst rm;

      rm.nntp=(atoi(*fp|"type")==GROUPTYPE_NNTP);
      strmaxcpy(rm.nntp_server, *fp|"nntp_server", TEXTLEN-1);
      rm.folder=fp->name;
      rm.viewfolder();
     }
    break;

   case GROUPTYPE_ZCONNECT:
     {
      readzcfoldert rm;

      rm.folder=fp->name;
      rm.viewfolder();
     }

   default:;
   }
 }
