/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <dirent.h>
#include <fcntl.h>
#include <time.h>
#include <sys/stat.h>

#include <dboxpath.h>
#include <nls.h>
#include <oldtools.h>
#include <logf.h>
#include <newsdb.h>
#include <charsettools.h>
#include "online.h"
#include "help.h"
#include "extern.h"
#include "vfolder.h"
#include "readmail.h"
#include "editor.h"
#include "wmail.h"
#include "readnews.h"
#include "readfolder.h"
#include "readrfcnews.h"
#include "html.h"

#define NEWSSPOOLDIR "/var/spool/news"

extern "C" time_t parsedate(char *line, int *tz);

bool readrfcnewst::getheader(const char *name, char *text)
 {
  FILE *infile;
  char kennung[200], tempstr[DIRLEN];

  if(nntp)
   {
    // TODO
   }
  else
   {
    snprintf(tempstr, DIRLEN, NEWSSPOOLDIR "%s/%ld", folder, msgindex[pos].msgnr);
    infile=fopen(tempstr, "rb");
    if(infile==NULL) return 0;

    while(!readrfcheaderline(infile, kennung, text, TEXTLEN-1) && kennung[0]!=0)
     {
      if(stricmp(kennung, name)==0)
       {
        fclose(infile);
        return FALSE;
       }
     }

    fclose(infile);
    text[0]=0;
   }

  return TRUE;
 }

bool readrfcnewst::readrfcheaderline(FILE *infile, 
  char *header, char *text, dword maxlen)
 {
  char line[10000], *tptr;
  bool fertig=FALSE;

  header[0]=text[0]=0;

  do
   {
    if(fgets(line, 9000, infile)==NULL) return TRUE;
    tptr=strchr(line, '\r'); if(tptr!=NULL) *tptr=0;
    tptr=strchr(line, '\n'); if(tptr!=NULL) *tptr=0;

    if(line[0]==0) return FALSE;

    if(line[0]!=' ' && line[0]!='\t')
     {
      tptr=strchr(line, ':');
      if(tptr==NULL) return TRUE;
      *tptr=0;
      strmaxcpy(header, line, maxlen-1);
      tptr++;
      while(*tptr==' ' || *tptr=='\t') tptr++;
      strmaxcpy(text, tptr, maxlen-1);
      fertig=TRUE;
     }
   }
  while(!fertig);

  return FALSE;
 }

int msgindex_sort(const void *a, const void *b)
 {
  dword a_msgnr, b_msgnr;

  a_msgnr=((readrfcnewst::msgindext *)a)->msgnr;
  b_msgnr=((readrfcnewst::msgindext *)b)->msgnr;

  if(a_msgnr<b_msgnr)
    return -1;
  else if(a_msgnr>b_msgnr)
    return 1;

  return 0;
 }

int msgindex_sort_reverse(const void *a, const void *b)
 {
  return -msgindex_sort(a, b);
 }

void readrfcnewst::readindex()
 {
  dword i;
  char *tptr;
 
  strmaxcpy(newsgroup, folder+1, TEXTLEN-1);
  tptr=newsgroup;
  while(*tptr!=0)
   {
    *tptr=tolower(*tptr);
    if(*tptr=='/') *tptr='.';
    tptr++;
   }

  mail=FALSE;

  if(marked!=NULL) free(marked);
  marked=NULL;

  if(msgindex!=NULL) free(msgindex);
  msgindex=NULL;

  anz=0;

  if(nntp)
   {
    if(readindex_nntp())
      ende=TRUE;
   }
  else
    readindex_local();

  qsort(msgindex, anz, sizeof(msgindext), 
    atoi(status.userprofile|"nachrichtenanzeige")?
    msgindex_sort_reverse:msgindex_sort);

  marked=(bool *)malloc(anz+1);

  for(i=0; i<(anz+1); i++) marked[i]=FALSE;
 }

bool readrfcnewst::opensocket(bool sendgroup)
 {
  char tempstr[TEXTLEN];

  if(socket.is_open()) 
   {
    socket.form("date\r\n");
    if(socket.gets(tempstr, TEXTLEN-1)==NULL ||
       atoi(tempstr)!=111)
     {
      socket.close();
     }
    else
      return FALSE;
   }

  if(socket.open(nntp_server, 119)) return TRUE;
  if(socket.gets(tempstr, TEXTLEN-1)==NULL) return TRUE;
  if(atoi(tempstr)!=200) return TRUE;

  socket.form("mode reader\r\n");
  if(socket.gets(tempstr, TEXTLEN-1)==NULL) return TRUE;
  if(atoi(tempstr)!=200) return TRUE;

  if(sendgroup)
   {
    socket.form("group %s\r\n", newsgroup);
    if(socket.gets(tempstr, TEXTLEN-1)==NULL) return TRUE;
   }

  return FALSE;
 }

bool readrfcnewst::readindex_nntp()
 {
  dword i;
  char *tptr, tempstr[TEXTLEN];

   {
    char title[TEXTLEN];
    byte x=online.sizex/2;

    online.setattr(2, FALSE);
    snprintf(title, TEXTLEN, "Internet - News");
    online.box(x-20, 10, x+20, 15, BOXFRAMEDBL, title, TRUE);
    snprintf(title, TEXTLEN, "Verbindung luft - drcken Sie die");
    online.setcursor(x-strlen(title)/2, 12); online.printf("%s", title);
    online.setcursor(x-12, 13); online.printf("Taste ESC um abzubrechen");
    online.setattr(1, FALSE);
   }

  if(opensocket(FALSE))
   {
    if(!status.dologout)
      errorbox(&online, "Serverfehler (1)!");
    return TRUE;
   }

  socket.form("group %s\r\n", newsgroup);
  
  if(socket.gets(tempstr, TEXTLEN-1)==NULL)
   {
    if(!status.dologout)
      errorbox(&online, "Serverfehler (2)!");
    return TRUE;
   }
   
  switch(atoi(tempstr))
   {
   case 211:
    tptr=tempstr;
    while(isdigit(*tptr)) tptr++;
    while(*tptr==' ') tptr++;
    anz=atol(tptr);
    break;

   case 411:
    if(!status.dologout)
      errorbox(&online, "Gruppe nicht verfuegbar!");
    return TRUE;

   default:
    if(!status.dologout)
      errorbox(&online, "Serverfehler (3)!");
    return TRUE;
   }

  if(anz!=0)
   {
    msgindex=(msgindext *)malloc(sizeof(msgindext)*anz);

    if(msgindex==NULL)
      logf.internalerror("readindex_nntp", "malloc failed");

    for(i=0; i<anz; i++)
     {
      msgindex[i].index_gelesen=FALSE;
      msgindex[i].date=0;
      msgindex[i].erhaltenam=0;
      msgindex[i].len=0;
      msgindex[i].subject[0]=0;
      msgindex[i].from[0]=0;
      msgindex[i].msgnr=0;
     }

    socket.form("listgroup\r\n");

    if(socket.gets(tempstr, TEXTLEN-1)==NULL) return TRUE;
    if(atoi(tempstr)!=211)
      return TRUE;

    for(i=0; i<anz; i++)
     {
      if(socket.gets(tempstr, TEXTLEN-1)==NULL) return TRUE;
      if(tempstr[0]=='.') return FALSE;
      msgindex[i].msgnr=atoi(tempstr);
     }

    do
     {
      if(socket.gets(tempstr, TEXTLEN-1)==NULL) return TRUE;
     }
    while(tempstr[0]!='.');
   }

  return FALSE;
 }

void readrfcnewst::readindex_local()
 {
  dword i;
  DIR *dir;
  char path[DIRLEN];
  struct dirent *ent;

  snprintf(path, DIRLEN, NEWSSPOOLDIR "%s", folder);
  dir=opendir(path);

  if(dir!=NULL)
   {
    while((ent=readdir(dir))!=NULL)
      if(isdigit(ent->d_name[0]))
        anz++;

    if(anz!=0)
     {
      msgindex=(msgindext *)malloc(sizeof(msgindext)*anz);

      if(msgindex==NULL)
        logf.internalerror("readindex_nntp", "malloc failed");

      for(i=0; i<anz; i++)
       {
        msgindex[i].index_gelesen=FALSE;
        msgindex[i].date=0;
        msgindex[i].erhaltenam=0;
        msgindex[i].len=0;
        msgindex[i].subject[0]=0;
        msgindex[i].from[0]=0;
        msgindex[i].msgnr=0;
       }

      rewinddir(dir);

      i=0;
      while((ent=readdir(dir))!=NULL && i<anz)
        if(isdigit(ent->d_name[0]))
         {
          msgindex[i].msgnr=atol(ent->d_name);
          i++;
         }
     }

    closedir(dir);
   }
 }

void readrfcnewst::freeindex()
 {
  if(msgindex!=NULL) free(msgindex);
  msgindex=NULL;
  readfoldert::freeindex();
 }

void readrfcnewst::process_xover(char *xover)
 {
  dword i, msgnr, line=0;
  char *tptr, *pointer[10];

  for(i=0; i<7; i++)
   {
    pointer[i]=xover;
    tptr=strchr(xover, '\t');
    if(tptr==NULL) return;
    *tptr=0;
    tptr++;
    xover=tptr;
   }

  msgnr=atol(pointer[0]);

  //logf.printf("xover", "GOT %ld", msgnr);

  for(i=0; i<anz; i++)
    if(msgindex[i].msgnr==msgnr)
     {
      line=i;
      break;
     }

  if(i==anz) return;

  iso2ibm(pointer[1], strlen(pointer[1]));
  strmaxcpy(msgindex[line].subject, pointer[1], TEXTLEN-1);

  iso2ibm(pointer[2], strlen(pointer[2]));
  strmaxcpy(msgindex[line].from, pointer[2], TEXTLEN-1);

  int tz;
  msgindex[line].date=parsedate(pointer[3], &tz);
  if(msgindex[line].date==-1) msgindex[line].date=0;

  msgindex[line].len=atol(pointer[6]);

  msgindex[line].index_gelesen=TRUE;  
 }

void readrfcnewst::displayline(dword line)
 {
  bool seen;
  char tempstr[TEXTLEN*2];

  if(line>=anz) return;

  if(!msgindex[line].index_gelesen)
   {
    if(nntp)
     {
      if(msgindex[line].msgnr!=0)
       {
        if(opensocket(TRUE)) return;

         {
          dword low, high, i;

          low=msgindex[line].msgnr;
          for(i=1; i<10 && line>=i && !msgindex[line-i].index_gelesen; i++)
            low=msgindex[line-i].msgnr;

          high=msgindex[line].msgnr;
          for(i=1; i<10 && (line+i)<anz && !msgindex[line+i].index_gelesen; i++)
            high=msgindex[line+i].msgnr;

          if(low>high) { i=low; low=high; high=i; }

          socket.form("xover %lu-%lu\r\n", low, high);
          //logf.printf("xover", "XOVER %ld-%ld", low, high);
         }

        if(socket.gets(tempstr, TEXTLEN-1)==NULL) return;
        if(atoi(tempstr)!=224) return;
        //logf.printf("xover", "STARTING %ld", msgindex[line].msgnr);
        while(TRUE)
         {
          char xover[10000];
          if(socket.gets(xover, 9000)==NULL) return;
          if(xover[0]=='.') break;
          process_xover(xover);
         }
        //logf.printf("xover", "STOP %ld", msgindex[line].msgnr);
       }
     }
    else
     {
      FILE *infile;
      char header[TEXTLEN], text[TEXTLEN];
      char filename[DIRLEN];
      struct stat statbuf;
    
      snprintf(filename, DIRLEN, NEWSSPOOLDIR "%s/%lu", 
        folder, msgindex[line].msgnr);

      infile=fopen(filename, "rt");
      if(infile!=NULL)
       {
        if(fstat(fileno(infile), &statbuf)==0)
         {
          msgindex[line].len=statbuf.st_size;
          msgindex[line].erhaltenam=statbuf.st_mtime;
         }

        while(!readrfcheaderline(infile, header, text, TEXTLEN-1))
         {
          if(stricmp(header, "Subject")==0)
           {
            iso2ibm(text, strlen(text));
            strmaxcpy(msgindex[line].subject, text, TEXTLEN-1);
           }
          else if(stricmp(header, "From")==0)
           {
            iso2ibm(text, strlen(text));
            strmaxcpy(msgindex[line].from, text, TEXTLEN-1);
           }
          else if(stricmp(header, "Date")==0)
           {
            int tz;
            msgindex[line].date=parsedate(text, &tz);
            if(msgindex[line].date==-1) msgindex[line].date=0;
           }
         }

        fclose(infile);
       }
     }

    msgindex[line].index_gelesen=TRUE;
   }
  
  seen=(status.letzteranruf>msgindex[line].erhaltenam);

  snprintf(tempstr, TEXTLEN, "%c%c %s", seen?' ':'*', ' ', 
          msgindex[line].from);

  viewfoldert::displayline(line, marked[line], 
    msgindex[line].date, 
    msgindex[line].len, tempstr, 
    msgindex[line].subject);
 }

void readrfcnewst::read()
 {
  char filename[DIRLEN], url[TEXTLEN];

  if(pos>=anz) return;

  snprintf(filename, DIRLEN, DBOXPORTTEMPDIR "posting.html", config.dboxdir, config.port);

  if(!formatposting(filename, FALSE, ""))
   {
    strmaxcpy(url, "internal:", TEXTLEN-1);
    dohtmlview(filename, url);
   }
  else
    infobox(&online, "Diese Nachricht ist nicht lesbar!");

  remove(filename);

  if(!status.dologout) draw(TRUE);
 }

void readrfcnewst::doweiterleiten()
 {
 }

void readrfcnewst::fileinclude()
 {
  #if 0
  char tempstr[TEXTLEN], bet[TEXTLEN]="", filename[DIRLEN]="";
  met mes[10];
  FILE *infile, *outfile;

  if(status.gruppe.security!=MAXSECURITY)
   {
    if(!status.dologout)
      errorbox(&online, "Zugriff verweigert!");
    if(!status.dologout) draw(TRUE);
    return;
   }

 startover:
  online.setattr(2, FALSE);
  online.box(20, 8, 60, 18, BOXFRAMEDBL, "Fileinclude", TRUE);

  online.setcursor(22, 10);
  online.printf("Empfnger:");
  online.setcursor(24, 12);
  online.printf("Betreff:");
  online.setcursor(26, 14);
  online.printf("Datei:");

  online.setattr(2, TRUE);
  online.setcursor(33, 10);
  online.printf("%s", folder);
  online.setattr(2, FALSE);

  mes[0].type=met::EDIT;
  mes[0].key=0;
  mes[0].x=33;
  mes[0].y=12;
  mes[0].sizex=24;
  mes[0].maxlen=60;
  mes[0].text=bet;

  mes[1].type=met::EDIT;
  mes[1].key=0;
  mes[1].x=33;
  mes[1].y=14;
  mes[1].sizex=24;
  mes[1].maxlen=TEXTLEN-1;
  mes[1].text=filename;

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=25;
  mes[2].y=16;
  mes[2].key=Key_C_Ent;
  mes[2].sizex=6;
  mes[2].text="O^k";

  mes[3].type=met::COMMANDBUTTON;
  mes[3].x=42;
  mes[3].y=16;
  mes[3].key=0;
  mes[3].sizex=9;
  mes[3].text="^Abbruch";

  mes[4].type=met::END_OF_LIST;

 schleife:
  switch(domenu(&online, mes, 0, TRUE))
   {
   case 2:
    break;

   case MENU_HELP:
    /*
    displayhelp();
    */
    online.clrscr();
    if(status.dologout) break;
    goto startover;
   }

  if(!status.dologout) draw(TRUE);
  #endif
 }

void readrfcnewst::reply(bool quote, bool followup, bool quoteall)
 {
  char bet[TEXTLEN*2], bez[TEXTLEN*2]="";
  FILE *quotefile=NULL;
  char quotestring[TEXTLEN*2]="";
  char quotefilename[DIRLEN];

  if(pos>=anz) return;

  if(strncmp(msgindex[pos].subject, "Re: ", 4)==0)
    strmaxcpy(bet, msgindex[pos].subject, TEXTLEN-1);
  else
   {
    snprintf(bet, TEXTLEN, "Re: %s", msgindex[pos].subject);
    bet[TEXTLEN-1]=0;
   }

   {
    char tempstr[TEXTLEN*2], *tptr;

    getheader("Message-ID", tempstr);
    tptr=strrchr(tempstr, '>');
    if(tptr!=NULL) *tptr=0;
    tptr=tempstr;
    if(*tptr=='<') tptr++;
    strmaxcpy(bez, tptr, TEXTLEN-1);
   }

  if(quoteall)
   {
    infobox(&online, nls["online.notavailable"]);
    if(!status.dologout) draw(TRUE);
    return;
   }

  if(quote)
   {
    snprintf(quotefilename, DIRLEN, DBOXPORTTEMPDIR "posting",
      config.dboxdir, config.port);

    if(followup)
      snprintf(quotestring, TEXTLEN, nls["online.quotestringmail"],
        msgindex[pos].from);
     else
      {
       struct tm *tm;
       tm=localtime(&msgindex[pos].date);
       snprintf(quotestring, TEXTLEN, nls["online.quotestringnews"], 
       tm->tm_mday, tm->tm_mon+1, tm->tm_year+1900);
      }

    if(!formatposting(quotefilename, TRUE, quotestring))
     {
      quotefile=fopen(quotefilename, "rb");
     }
   }
   
  if(followup)
   {
    if(get_news_folder_grants(folder, status.userprofile, status.gruppe,
                              FALSE, TRUE, FALSE))
      errorbox(&online, "Schreibzugriff verweigert!");
    else
     {
      // TODO: Followup-To
      dowritemail(folder, bet, bez, quotefile);
      reread=TRUE;
     }
   }
  else
   {
    char emp[TEXTLEN], *tptr;

    if(getheader("Reply-To", emp))
      strmaxcpy(emp, msgindex[pos].from, TEXTLEN-1);

    tptr=strchr(emp, ' ');
    if(tptr!=NULL) *tptr=0;

    // dowritemail(emp, bet, bez, quotefile); // TODO!!!
    infobox(&online, nls["online.notavailable"]);
   }

  if(quotefile!=NULL) 
   { 
    fclose(quotefile);
    remove(quotefilename);
   }

  if(!status.dologout && !reread) draw(TRUE);
 }

bool readrfcnewst::nntp_getposting(FILE *outfile)
 {
  char tempstr[DIRLEN], line[1000], *tptr;

  if(opensocket(TRUE))
    return TRUE;

  socket.form("article %ld\r\n", msgindex[pos].msgnr);

  if(socket.gets(tempstr, TEXTLEN-1)==NULL) return TRUE;
  if(atoi(tempstr)!=220) return TRUE;

  while(TRUE)
   {
    if(socket.gets(line, 900)==NULL) return TRUE;
    tptr=strchr(line, '\r'); if(tptr!=NULL) *tptr=0;
    tptr=strchr(line, '\n'); if(tptr!=NULL) *tptr=0;

    if(line[0]=='.' && line[1]==0)
      break;

    fprintf(outfile, "%s\n", line);
   }

  return FALSE;
 }

bool readrfcnewst::formatposting(const char *filename, bool quote,
                                 const char *quotestring)
 {
  FILE *outfile, *infile;
  char headername[TEXTLEN], text[TEXTLEN], tempstr[DIRLEN], tmpfile[DIRLEN]="";

  if(nntp)
   {
    snprintf(tmpfile, DIRLEN, DBOXPORTTEMPDIR "posting.nntp",
      config.dboxdir, config.port);
    outfile=fopen(tmpfile, "wb");
    if(outfile==NULL) return TRUE;

    if(nntp_getposting(outfile))
     {
      fclose(outfile);
      return TRUE;
     }

    fclose(outfile);

    infile=fopen(tmpfile, "rb");
    if(infile==NULL) return TRUE; 
   }
  else
   {
    snprintf(tempstr, DIRLEN, NEWSSPOOLDIR "%s/%ld", 
      folder, msgindex[pos].msgnr);
    infile=fopen(tempstr, "rb");
    if(infile==NULL) return TRUE;
   }
  
  outfile=fopen(filename, "wb");
  if(outfile==NULL)
   {
    fclose(infile);
    return TRUE;
   }

  if(!quote)
    fprintf(outfile, 
      "<html>\n"
      "<head>\n"
      "<title>%s</title>\n"
      "</head>\n"
      "<body>\n"
      "<dbox-charset-%d>\n"
      "<pre>", msgindex[pos].subject, ISO);

  if(!quote)
   {
    strmaxcpy(tempstr, msgindex[pos].subject, 60);
    fprintf(outfile, "<b>Betreff:</b>       %s\r\n", tempstr);
    strmaxcpy(tempstr, msgindex[pos].from, 60);
    fprintf(outfile, "<b>Absender:</b>      %s\r\n", tempstr);
   }
  
  while(!readrfcheaderline(infile, headername, text, TEXTLEN-1) &&
    headername[0]!=0)
   {
    if(stricmp(headername, "Sender")==0)
     {
      if(!quote)
       {
        text[60]=0;
        fprintf(outfile, "<b>Weiterleitung:</b> %s\r\n", text);
       }
     }
    else if(stricmp(headername, "Organization")==0)
     {
      if(!quote)
       {
        text[60]=0;
        fprintf(outfile, "<b>Organisation:</b>  %s\r\n", text);
       }
     }
   }

  if(!quote)
   {
    fprintf(outfile, "<b>Textl&auml;nge:</b>     %lu Bytes\r\n", msgindex[pos].len);
  
    if(msgindex[pos].date!=0)
     {
      struct tm *t;
      t=localtime(&msgindex[pos].date);
      fprintf(outfile, "<b>Datum:</b>         %02u.%02u.%04u, %02u:%02u:%02u "
                       "(local time)\r\n",
        t->tm_mday, t->tm_mon+1, t->tm_year+1900, t->tm_hour, t->tm_min,
        t->tm_sec);
     }
  
    fprintf(outfile, "<hr>");
   }
  
  if(quotestring[0]!=0 && quote)
   {
    fprintf(outfile, "%s\r\n\r\n", quotestring);
   }

   {
    dword pos, length;
    pos=ftell(infile);
    fseek(infile, 0, SEEK_END);
    length=(ftell(infile)-pos);
    fseek(infile, pos, SEEK_SET);

    formattext(infile, outfile, length, quote, !quote, quote);
   }

  fclose(infile);
  fclose(outfile);

  return FALSE;
 }

void readrfcnewst::send()
 {
  if(get_news_folder_grants(folder, status.userprofile, status.gruppe,
                            FALSE, TRUE, FALSE))
    errorbox(&online, "Schreibzugriff verweigert!");
  else
   {
    dowritemail(folder);
    reread=TRUE;
   }

  if(!status.dologout && !reread) draw(TRUE);
 }
