/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <dirent.h>
#include <fcntl.h>
#include <time.h>
#include <sys/stat.h>

#include <minmax.h>
#include <fstream.h>

#include <dboxpath.h>
#include <oldtools.h>
#include <folder.h>
#include <logf.h>
#include <stringtools.h>
#include <nls.h>
#include <oldui.h>
#include <newsdb.h>
#include <tools.h>
#include <zconnect.h>
#include <messageid.h>
#include <htmltools.h>
#include "../webonline/webonline.h"
#include "online.h"
#include "gen_from.h"
#include "help.h"
#include "extern.h"
#include "vfolder.h"
#include "readmail.h"
#include "editor.h"
#include "www.h"
#include "wmail.h"
#include "readnews.h"
#include "readfolder.h"
#include "readzcfolder.h"
#include "html.h"

dword readzcfoldert::getnr(dword line)
 {
  if(atoi(status.userprofile|"nachrichtenanzeige"))
    return (anz-1-line);

  return line;
 }

dword readzcfoldert::getindexpos(dword line)
 {
  return getnr(line)*sizeof fi;
 }

void readzcfoldert::readindex()
 {
  char filename[DIRLEN];
  dword i;

  if(marked!=NULL) free(marked);
  marked=NULL;

  mail=folder[0]!='/';

  if(mail)
   {
    userpt up(folder);
    up.setfilename();
    snprintf(path, DIRLEN, "%smail" DIRCHAR, up.getpath());
    unzustellbar=FALSE;
   }
  else
   {
    folderpt fp(folder);
    fp.setfilename();
    strmaxcpy(path, fp.getpath(), DIRLEN-1);

    unzustellbar=(stricmp(folder, config.prvunzustellbar)==0 ||
                  stricmp(folder, config.brtunzustellbar)==0);
   }

  snprintf(filename, DIRLEN, "%s%s", path, "index");

  index=open(filename, O_RDWR);
  if(index<0)
   {
    anz=0;
    return;
   }

  anz=filelength(index)/sizeof(folderindext);

  marked=(bool *)malloc(anz+1);
  for(i=0; i<(anz+1); i++) marked[i]=FALSE;
 }

void readzcfoldert::freeindex()
 {
  if(index>=0)
   {
    close(index);
    index=-1;
    anz=0;
   }

  readfoldert::freeindex();
 }

void readzcfoldert::displayline(dword line)
 {
  folderindext fi;
  char t1[TEXTLEN*2], t2[TEXTLEN*2];
  bool seen;

  if(line>=anz) return;

  lseek(index, getindexpos(line), SEEK_SET);
  if(::read(index, &fi, sizeof fi)!=sizeof fi) return;

  if(mail)
    seen=fi.flags.seen;
  else
    seen=(status.letzteranruf>fi.received);

  snprintf(t1, TEXTLEN*2, "%c%c %s", seen?' ':'*', fi.flags.binary?'B':' ', fi.abs);

  if(fi.flags.deleted)
    snprintf(t2, TEXTLEN*2, "-- Gelscht -- (war: %s)", fi.bet);
  else
    strmaxcpy(t2, fi.bet, (TEXTLEN*2)-1);

  viewfoldert::displayline(line, marked[line], fi.eda, fi.len, t1, t2);
 }

void readzcfoldert::getmailfile(char *filename, bool mitkom, bool *temporary)
 {
  *temporary=FALSE;

  if(fi.flags.bodyextern && (mitkom || kom==0))
    snprintf(filename, DIRLEN, "%s%s", path, fi.file);
  else
   {
    FILE *outfile;
    char file[TEXTLEN], *tptr;

    if(getheader("FILE", file))
      strmaxcpy(file, fi.bet, TEXTLEN-1);
    
    tptr=file;
    while(*tptr!=0)
     {
      if(isascii(*tptr) &&
         (isalpha(*tptr) || isdigit(*tptr) || *tptr=='.' ||
         *tptr=='-' || *tptr=='_'))
       {
       }
      else
        *tptr='_';

      tptr++;
     }

    if(file[0]==0) strmaxcpy(file, "mail", TEXTLEN-1);

    snprintf(filename, DIRLEN, DBOXPORTTEMPDIR "%s", config.dboxdir, config.port, file);

    outfile=fopen(filename, "wb");
    if(outfile==NULL) return;

    getmailfile(outfile, mitkom);

    fclose(outfile);

    *temporary=TRUE;
   }
 }

void readzcfoldert::getmailfile(FILE *outfile, bool mitkom)
 {
  FILE *infile;
  char filename[DIRLEN];

  if(fi.flags.bodyextern)
   {
    snprintf(filename, DIRLEN, "%s%s", path, fi.file);
    infile=fopen(filename, "rb");
    if(infile==NULL) return;
   }
  else
   {
    snprintf(filename, DIRLEN, "%s%s", path, "header");
    infile=fopen(filename, "rb");
    if(infile==NULL) return;
    fseek(infile, fi.headeroffs+fi.headerlen, SEEK_SET);
   }

  if(mitkom || kom==0)
    filecopy(infile, outfile, fi.len);
  else
   {
    fseek(infile, kom, SEEK_CUR);
    filecopy(infile, outfile, max(fi.len-kom, (unsigned long)(0L)));
   }

  fclose(infile);
 }

void readzcfoldert::dodownload2()
 {
  char filename[DIRLEN];
  char d1[TEXTLEN*2], d2[TEXTLEN*2];
  bool temporary;

  getmailfile(filename, FALSE, &temporary);

  snprintf(d1, TEXTLEN, "Mail aus %s", folder);
  d1[70]=0;
  strmaxcpy(d2, fi.bet, 70);

  doonlinedownload(filename, d1, d2);

  if(temporary) unlink(filename);
 }

bool readzcfoldert::formatposting(const char *filename, dword laenge,
                                  bool quote, const char *quotestring)
 {
  if(mail && !quote)
   {
    string tmpfilename;

     {
      getdboxporttempdir(tmpfilename, config.dboxdir, config.port);
      tmpfilename+="http.tmp";
      ofstream out(tmpfilename.c_str(), ios::out|ios::bin);
      char nr[TEXTLEN];
      snprintf(nr, TEXTLEN, "%lu", getnr(pos));
      string url="/email?mailbox=INBOX&action=read&nr=";
      url+=nr;
      dowebonline(url, out);
     }

     {
      ifstream in(tmpfilename.c_str(), ios::in|ios::bin);
      ofstream out(filename, ios::out|ios::bin);
      char tempstr[1000];

      while(!in.eof() && !in.fail())
       {
        if(in.get(tempstr, 900))
         {
          char ch, *tptr;
          in.get(ch); // \n lesen

          tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;
          tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;
         }
        if(tempstr[0]==0) break;
       }
      filecopy(in, out, filelength(tmpfilename.c_str())-in.tellg());
     }
    return FALSE;
   }

  FILE *outfile, *infile;
  char kennung[200], tempstr[DIRLEN], charset_header[TEXTLEN]="";
  bool iso2ibm=FALSE;

  snprintf(tempstr, DIRLEN, "%s%s", path, "header");
  infile=fopen(tempstr, "rb");
  if(infile==NULL) return TRUE;
  fseek(infile, fi.headeroffs, SEEK_SET);

  outfile=fopen(filename, "wb");
  if(outfile==NULL)
   {
    fclose(infile);
    return TRUE;
   }

  if(!quote)
    fprintf(outfile, 
      "<html>\n"
      "<head>\n"
      "<dbox-charset-%d>\n"
      "<title>%s</title>\n"
      "</head>\n"
      "<body>\n"
      "<dbox-charset-%d>\n"
      "<pre>", IBM, htmlize_string(fi.bet).c_str(), IBM);

  while(!readkennung(infile, kennung) && kennung[0]!=0)
   {
    if(stricmp(kennung, "BET")==0)
     {
      readinhalt(infile, tempstr, 60);
      if(!quote)
        fprintf(outfile, "<b>Betreff:</b>       %s\r\n", 
          htmlize_string(tempstr).c_str());
     }
    else if(stricmp(kennung, "X-" X_DBOX "-Envelope-To")==0)
     {
      readinhalt(infile, tempstr, 60);
      if(!quote)
        fprintf(outfile, "<b>Empf&auml;nger:</b>     %s\r\n", 
          htmlize_string(tempstr).c_str());
     }
    else if(stricmp(kennung, "X-" X_DBOX "-JUNKERROR")==0)
     {
      readinhalt(infile, tempstr, 60);
      if(!quote)
        fprintf(outfile, "<b>Fehler:</b>        %s\r\n",
                         junkerrors[atoi(tempstr)]);
     }
    else if(stricmp(kennung, "X-" X_DBOX "-JUNKERROR-FROM")==0)
     {
      readinhalt(infile, tempstr, 60);
      if(!quote)
        fprintf(outfile, "<b>Puffer von:</b>    %s\r\n", tempstr);
     }
    else if(stricmp(kennung, "ABS")==0)
     {
      readinhalt(infile, tempstr, 60);
      if(!quote)
        fprintf(outfile, "<b>Absender:</b>      %s\r\n", 
          htmlize_string(tempstr).c_str());
     }
    else if(stricmp(kennung, "CHARSET")==0)
     {
      readinhalt(infile, charset_header, TEXTLEN-2);
     }
    else if(stricmp(kennung, "WAB")==0)
     {
      readinhalt(infile, tempstr, 60);
      if(!quote)
        fprintf(outfile, "<b>Weiterleitung:</b> %s\r\n",
          htmlize_string(tempstr).c_str());
     }
    else if(stricmp(kennung, "ORG")==0)
     {
      readinhalt(infile, tempstr, 60);
      if(!quote)
        fprintf(outfile, "<b>Organisation:</b>  %s\r\n", 
          htmlize_string(tempstr).c_str());
     }
    else
      skipinhalt(infile);
   }

  if(!quote)
   {
    struct tm *t;

    fprintf(outfile, "<b>Textl&auml;nge:</b>     %lu Bytes\r\n", laenge);

    t=localtime(&fi.eda);

    fprintf(outfile, "<b>Datum:</b>         %02u.%02u.%04u, %02u:%02u:%02u "
                     "(local time)\r\n",
      t->tm_mday, t->tm_mon+1, t->tm_year+1900, t->tm_hour, t->tm_min,
      t->tm_sec);

    fprintf(outfile, "<hr>");
   }
  else if(quotestring[0]!=0)
    fprintf(outfile, "%s\r\n\r\n", quotestring);

  if(charset_header[0]==0)
   {
    if(!quote)
      fprintf(outfile, "<dbox-charset-%d>\n", IBM);
   }
  else if(stricmp(charset_header, "ISO1")==0)
   {
    if(quote)
      iso2ibm=TRUE;
    else
      fprintf(outfile, "<dbox-charset-%d>\n", ISO);
   }

  if(fi.flags.bodyextern)
   {
    FILE *infile2;
    char filename[DIRLEN];

    snprintf(filename, DIRLEN, "%s%s", path, fi.file);
    infile2=fopen(filename, "rb");
    if(infile2==NULL)
     {
      fprintf(outfile, "\r\nINTERNER FEHLER!\r\n");
     }
    else
     {
      formattext(infile2, outfile, laenge, quote, !quote, iso2ibm);
      fclose(infile2);
     }
   }
  else
    formattext(infile, outfile, laenge, quote, !quote, iso2ibm);

  fclose(infile);

  if(laenge!=fi.len && !quote)
   {
    fprintf(outfile, "\r\n<hr>\r\n");
    fprintf(outfile,
      "\r\n<p>\r\n~Wichtiger Hinweis:~ Dieser Nachricht ist eine Datei "
      "angeh&auml;ngt!\r\n");
   }

  fclose(outfile);
  return FALSE;
 }

void readzcfoldert::readtextmail()
 {
  if(!mail)
   {
    char filename[DIRLEN], url[TEXTLEN];

    snprintf(filename, DIRLEN, DBOXPORTTEMPDIR "posting.html", config.dboxdir, config.port);

    if(!formatposting(filename, fi.len, FALSE, ""))
     {
      strmaxcpy(url, "internal:", TEXTLEN-1);
      dohtmlview(filename, url);
     }

    remove(filename);
   }
  else
   {
    string url;
    url="internal://webonline/email?mailbox=INBOX"
        "&action=read&nr="+i2string(unsigned(getnr(pos)));
    do_www(url);
   }
 }

dword readzcfoldert::getkom()
 {
  char tempstr[TEXTLEN];

  if(getheader("KOM", tempstr))
    return 0;

  return atol(tempstr);
 }

bool readzcfoldert::getheader(const char *name, char *text)
 {
  FILE *infile;
  char kennung[200], tempstr[DIRLEN];

  text[0]=0;

  snprintf(tempstr, DIRLEN, "%s%s", path, "header");
  infile=fopen(tempstr, "rb");
  if(infile==NULL) return 0;
  fseek(infile, fi.headeroffs, SEEK_SET);

  while(!readkennung(infile, kennung) && kennung[0]!=0)
   {
    if(stricmp(kennung, name)==0)
     {
      readinhalt(infile, text, TEXTLEN-1);
      fclose(infile);
      return FALSE;
     }
    else
      skipinhalt(infile);
   }

  fclose(infile);
  text[0]=0;

  return TRUE;
 }

junkerrort readzcfoldert::getjunkerror(char *junktext,
  char *junkfrom, char *junkemp)
 {
  FILE *infile;
  char kennung[200], tempstr[DIRLEN];
  junkerrort junkerror=JERR_NO_EMP;

  junktext[0]=0;
  junkfrom[0]=0;
  junkemp[0]=0;
  snprintf(tempstr, DIRLEN, "%s%s", path, "header");
  infile=fopen(tempstr, "rb");
  if(infile==NULL) return junkerror;
  fseek(infile, fi.headeroffs, SEEK_SET);

  while(!readkennung(infile, kennung) && kennung[0]!=0)
   {
    if(stricmp(kennung, "X-" X_DBOX "-JUNKERROR")==0)
     {
      readinhalt(infile, tempstr, 60);
      junkerror=(junkerrort)atol(tempstr);
     }
    else if(stricmp(kennung, "X-" X_DBOX "-JUNKERROR-TEXT")==0)
      readinhalt(infile, junktext, TEXTLEN-1);
    else if(stricmp(kennung, "X-" X_DBOX "-JUNKERROR-FROM")==0)
      readinhalt(infile, junkfrom, TEXTLEN-1);
    else if(stricmp(kennung, "X-" X_DBOX "-JUNKERROR-EMP")==0)
      readinhalt(infile, junkemp, TEXTLEN-1);
    else
      skipinhalt(infile);
   }

  fclose(infile);

  return junkerror;
 }

void readzcfoldert::doreadmail(bool download)
 {
  if(pos>=anz) return;

  lseek(index, getindexpos(pos), SEEK_SET);
  if(::read(index, &fi, sizeof fi)!=sizeof fi) return;

  if(fi.flags.deleted)
   {
    infobox(&online, "Diese Nachricht ist geloescht!");
    if(!status.dologout) draw(TRUE);
    return;
   }

  kom=getkom();

  if(!fi.flags.seen && mail)
   {
    fi.flags.seen=TRUE;
    lseek(index, getindexpos(pos), SEEK_SET);
    if(::write(index, &fi, sizeof fi)!=sizeof fi) return;
   }

  if(download)
    dodownload2();
  else
   {
    if(fi.flags.binary && kom==0)
      dodownload2();
    else
      readtextmail();
   }

  if(!status.dologout) draw(TRUE);
 }

void readzcfoldert::dodownload()
 {
  doreadmail(TRUE);
 }

void readzcfoldert::read()
 {
  doreadmail(FALSE);
 }

bool readzcfoldert::checkmodify()
 {
  if(atoi(status.gruppe|"security")!=MAXSECURITY)
   {
    char *berechtigte, *tptr;
    char tempstr[TEXTLEN];

    if(!get_news_folder_grants(folder, status.userprofile, status.gruppe,
                               FALSE, FALSE, TRUE))
      return FALSE;

    folderpt fp(folder);
    if(fp.read())
      return TRUE;

    strmaxcpy(tempstr, fp|"betreuer", TEXTLEN-1);
    berechtigte=tempstr;

    while(berechtigte[0]!=0)
     {
      tptr=berechtigte;
      while(*tptr!=0 && *tptr!=';' && *tptr!=',' && *tptr!=' ')
        tptr++;

      if(*tptr!=0)
       {
        *tptr=0;
        tptr++;
        while(*tptr==';' || *tptr==',' || *tptr==' ')
          tptr++;
       }

      while(*berechtigte==' ') berechtigte++;
      if(stricmp(status.username, berechtigte)==0) return FALSE;
      berechtigte=tptr;
     }

    errorbox(&online, "Zugriff verweigert!");
    if(!status.dologout) draw(TRUE);
    return TRUE;
   }

  return FALSE;
 }

void readzcfoldert::del()
 {
  if(!mail)
   {
    if(checkmodify())
      return;
   }

  if(pos>=anz) return;

  lseek(index, getindexpos(pos), SEEK_SET);
  if(::read(index, &fi, sizeof fi)!=sizeof fi) return;

  fi.flags.deleted=!fi.flags.deleted;

  lseek(index, getindexpos(pos), SEEK_SET);
  if(::write(index, &fi, sizeof fi)!=sizeof fi) return;

  displayline(pos);

  if(mail)
   {
    char filename[DIRLEN];
    FILE *outfile;

    snprintf(filename, DIRLEN, BOXTODODIR "crunch.%s", config.boxdir, status.username);
    outfile=fopen(filename, "wt");
    if(outfile!=NULL) fclose(outfile);
   }
  
 }

void readzcfoldert::del_marked()
 {
  dword i;
 
  if(!mail)
   {
    if(checkmodify())
      return;
   }

  for(i=0; i<anz; i++)
    if(marked[i])
     {
      lseek(index, getindexpos(i), SEEK_SET);
      if(::read(index, &fi, sizeof fi)!=sizeof fi) return;

      fi.flags.deleted=!fi.flags.deleted;

      lseek(index, getindexpos(i), SEEK_SET);
      if(::write(index, &fi, sizeof fi)!=sizeof fi) return;
     }

  if(mail)
   {
    char filename[DIRLEN];
    FILE *outfile;

    snprintf(filename, DIRLEN, BOXTODODIR "crunch.%s", config.boxdir, status.username);
    outfile=fopen(filename, "wt");
    if(outfile!=NULL) fclose(outfile);
   }

  if(!status.dologout) draw(TRUE);
 }

void readzcfoldert::doweiterleiten()
 {
  met mes[4];
  FILE *infile;
  char filename[DIRLEN], junkfrom[TEXTLEN], junktext[TEXTLEN],
       emp[TEXTLEN]="";
  bool temporary, news;
  junkerrort junkerror=JERR_NO_EMP;

  if(pos>=anz) return;

  lseek(index, getindexpos(pos), SEEK_SET);
  if(::read(index, &fi, sizeof fi)!=sizeof fi) return;

  if(unzustellbar)
   {
    lseek(index, getindexpos(pos), SEEK_SET);
    if(::read(index, &fi, sizeof fi)!=sizeof fi) return;
    kom=getkom();
    junkerror=getjunkerror(junktext, junkfrom, emp);

    if(junkerror!=JERR_UNKNOWN_EMP && junkerror!=JERR_INVALID_SYSTEM &&
       junkerror!=JERR_INVALID_EMP && junkerror!=JERR_ACCESS_DENIED &&
       junkerror!=JERR_MAIL_TOO_BIG && junkerror!=JERR_INVALID_ABS) //TODO!
      errorbox(&online, "Warnung: Nachricht kann evtl. nicht weitergeleitet werden!");
   }

  online.setattr(2, FALSE);

  if(unzustellbar)
    online.box(10, 10, 70, 16, BOXFRAMEDBL,
      "unzustellbare Nachricht weiterleiten", TRUE);
  else
    online.box(10, 10, 70, 16, BOXFRAMEDBL,
      "Nachricht weiterleiten", TRUE);

  online.setcursor(12, 12);
  online.printf("Empfnger:");

  mes[0].type=met::EDIT;
  mes[0].key=0;
  mes[0].x=23;
  mes[0].y=12;
  mes[0].sizex=44;
  mes[0].maxlen=TEXTLEN-1;
  mes[0].text=emp;

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=27;
  mes[1].y=14;
  mes[1].key=Key_C_Ent;
  mes[1].sizex=6;
  mes[1].text="O^k";

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=46;
  mes[2].y=14;
  mes[2].key=0;
  mes[2].sizex=9;
  mes[2].text="^Abbruch";

  mes[3].type=met::END_OF_LIST;

 schleife:
  switch(domenu(&online, mes, 0, TRUE))
   {
   case 1:
    FILE *outfile;
    char file1[DIRLEN], file2[DIRLEN], kennung[200], 
         abs[TEXTLEN], org[TEXTLEN];
    dword headerbeginpos, headerendpos;

    if(emp[0]==0) break;

    news=(strchr(emp, '@')==NULL && emp[0]=='/');

    if(strchr(emp, '@')==NULL && !news)
      snprintf(emp+strlen(emp), TEXTLEN-strlen(emp),
        "@%s.%s", config.boxname, config.domain);

    snprintf(file1, DIRLEN, DBOXPORTTEMPDIR "msg.tmp", config.dboxdir, config.port);
    outfile=fopen(file1, "wb");
    if(outfile==NULL)
     {
      logf.printf("readfoldert::doweiterleiten",
        "Failed to create \"%s\"", file1);
      return;
     }

    if(unzustellbar)
     {
      getmailfile(filename, TRUE, &temporary);
      infile=fopen(filename, "rb");
      if(infile==NULL) return;
     }
    else
     {
      snprintf(filename, DIRLEN, "%s%s", path, "header");
      infile=fopen(filename, "rb");
      filename[0]=0;
      if(infile==NULL) return;
      fseek(infile, fi.headeroffs, SEEK_SET);
      temporary=FALSE;
     }

    headerbeginpos=ftell(infile);

    while(!readkennung(infile, kennung) && kennung[0]!=0)
     {
      if(stricmp(kennung, "EMP")==0)
       {
        fprintf(outfile, "X-" X_DBOX "-O-EMP:");
        copyinhalt(infile, outfile);
       }
      else if(stricmp(kennung, "ROT")==0)
       {
        fprintf(outfile, "X-" X_DBOX "-O-ROT:");
        copyinhalt(infile, outfile);
       }
      else if(stricmp(kennung, "MID")==0)
       {
        fprintf(outfile, "X-" X_DBOX "-O-MID:");
        copyinhalt(infile, outfile);
       }
      else
       {
        fprintf(outfile, "%s:", kennung);
        copyinhalt(infile, outfile);
       }
     }

    gen_abs(status, status.username, abs, org, TRUE);
  
    fprintf(outfile, "ROT:\r\n");
    fprintf(outfile, "EMP:\t%s\r\n", emp);

    if(unzustellbar)
      fprintf(outfile, "X-" X_DBOX "-WAB:\t%s\r\n", abs);
    else
      fprintf(outfile, "WAB:\t%s\r\n", abs);

    if(unzustellbar)
      fprintf(outfile, "MID:\tverwaltung.%lu.%lu@%s.%s\r\n",
        status.midcounter++, time(NULL), config.boxname, config.domain);
    else
      fprintf(outfile, "MID:\tmailer.%lu.%lu@%s.%s\r\n",
        status.midcounter++, time(NULL), config.boxname, config.domain);

    fprintf(outfile, "\r\n");

    headerendpos=ftell(infile);

    if(unzustellbar)
      filecopy(infile, outfile, fi.len-(headerendpos-headerbeginpos));
    else
      getmailfile(outfile, TRUE);

    fclose(infile);
    fclose(outfile);

    if(temporary) unlink(filename);

    if(news)
      snprintf(file2, DIRLEN, BOXSPOOLINZCNEWSDIR "%s.%lu-%lu-%u",
        config.boxdir, config.boxname, status.midcounter++, 
        time(NULL), config.port);
    else if(unzustellbar)
      snprintf(file2, DIRLEN, BOXSPOOLINZCMAILDIR "%s.%lu-%lu-%u",
        config.boxdir, config.boxname, status.midcounter++, 
        time(NULL), config.port);
    else
      snprintf(file2, DIRLEN, BOXSPOOLINZCMAILDIR "%s.%lu-%lu-%u",
        config.boxdir, status.username, status.midcounter++, 
        time(NULL), config.port);

    rename(file1, file2);

    // TODO: news

    break;

   case MENU_HELP:
    goto schleife;
   }

  if(!status.dologout) draw(TRUE);
 }

void readzcfoldert::dozurueckschicken()
 {
  char emp[TEXTLEN*2]="", errormsg[TEXTLEN]="";
  met mes[10];
  FILE *infile;
  char filename[DIRLEN], kennung[200], junktext[TEXTLEN],
    junkfrom[TEXTLEN], junkemp[TEXTLEN];
  junkerrort junkerror=JERR_NO_EMP;
  bool temporary, unsubscribe=FALSE;

  if(pos>=anz) return;

  lseek(index, getindexpos(pos), SEEK_SET);
  if(::read(index, &fi, sizeof fi)!=sizeof fi) return;

  online.setattr(2, FALSE);
  online.box(10, 10, 70, 17, BOXFRAMEDBL,
    "unzustellbare Nachricht zurckschicken", TRUE);

  kom=getkom();
  junkerror=getjunkerror(junktext, junkfrom, junkemp);
  if(junkfrom[0]==0) strmaxcpy(junkfrom, "(unknown)", TEXTLEN-1);
  if(junkemp[0]==0) strmaxcpy(junkemp, "(unknown)", TEXTLEN-1);

  getmailfile(filename, TRUE, &temporary);

  infile=fopen(filename, "rb");
  if(infile==NULL) return;

  while(!readkennung(infile, kennung) && kennung[0]!=0)
   {
    if(stricmp(kennung, "ABS")==0 && emp[0]==0)
      readinhalt(infile, emp, TEXTLEN-1);
    else if(stricmp(kennung, "WAB")==0)
      readinhalt(infile, emp, TEXTLEN-1);
    else
      skipinhalt(infile);
   }

  fclose(infile);

  strmaxcpy(errormsg, junkerrors[junkerror], TEXTLEN-1);

  #if 0
  if(junkerror!=JERR_UNKNOWN_EMP && junkerror==JERR_INVALID_SYSTEM &&
     junkerror!=JERR_INVALID_EMP && junkerror==JERR_ACCESS_DENIED)
   {
    errorbox(&online, "Nachricht kann nicht weitergeleitet werden!");
    return;
   }
  #endif

  online.setcursor(12, 12);
  online.printf("Empfnger:");

  online.setcursor(12, 13);
  online.printf("Fehler:");

  mes[0].type=met::EDIT;
  mes[0].key=0;
  mes[0].x=23;
  mes[0].y=12;
  mes[0].sizex=44;
  mes[0].maxlen=TEXTLEN-1;
  mes[0].text=emp;

  mes[1].type=met::EDIT;
  mes[1].key=0;
  mes[1].x=23;
  mes[1].y=13;
  mes[1].sizex=44;
  mes[1].maxlen=TEXTLEN-1;
  mes[1].text=errormsg;

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=22;
  mes[2].y=15;
  mes[2].key=Key_C_Ent;
  mes[2].sizex=6;
  mes[2].text="O^k";

  mes[3].type=met::COMMANDBUTTON;
  mes[3].x=33;
  mes[3].y=15;
  mes[3].key=0;
  mes[3].sizex=9;
  mes[3].text="^Abbruch";

  mes[4].type=met::COMMANDBUTTON;
  mes[4].x=47;
  mes[4].y=15;
  mes[4].key=0;
  mes[4].sizex=13;
  mes[4].text="^Unsubscribe";

  mes[5].type=met::END_OF_LIST;

 schleife:
  switch(domenu(&online, mes, 0, TRUE))
   {
   case 2:
    FILE *outfile;
    char file1[DIRLEN], file2[DIRLEN], text[1000];

    if(emp[0]==0) break;

    if(strchr(emp, '@')==NULL)
      snprintf(emp+strlen(emp), TEXTLEN-strlen(emp),
        "@%s.%s", config.boxname, config.domain);

    infile=fopen(filename, "rb");
    if(infile==NULL) return;

    snprintf(file1, DIRLEN, DBOXPORTTEMPDIR "msg.tmp", config.dboxdir, config.port);
    outfile=fopen(file1, "wb");
    if(outfile==NULL)
     {
      logf.printf("readfoldert::dozurueckschicken",
        "Failed to create \"%s\"", file1);
      return;
     }

    if(unsubscribe)
     {
      snprintf(text, 1000, "%s\r\n", errormsg);

      fprintf(outfile, "ABS:\t%s\r\n", junkemp);

      fprintf(outfile, "BET:\t%s\r\n", errormsg);

      fprintf(outfile, "LEN:\t%d\r\n", strlen(text));
     }
    else
     {
      snprintf(text, 1000,
        "The original message was received at (unknown)\r\n"
        "from unknown@%s\r\n"
        "\r\n"
        "   ----- The following addresses had delivery problems -----\r\n"
        "%s (%s)\r\n"
        "\r\n"
        "   ----- Transcript of session follows -----\r\n"
        "550 %s... %s\r\n"
        "\r\n"
        "   ----- Original message follows -----\r\n",
        junkfrom, junkemp, errormsg, junkemp, junktext);

      fprintf(outfile, "ABS:\tMAILER-DAEMON@%s.%s\r\n",
        config.boxname, config.domain);

      fprintf(outfile, "BET:\tReturned mail: %s\r\n", errormsg);

      fprintf(outfile, "LEN:\t%lu\r\n", strlen(text)+fi.len);
     }

    fprintf(outfile, "ANTWORT-AN:\t%s@%s.%s\r\n", status.username,
      config.boxname, config.domain);

    fprintf(outfile, "EDA:\t%s\r\n", make_eda_localtime().c_str());

    fprintf(outfile, "EMP:\t%s\r\n", emp);

    fprintf(outfile, "MAILER:\tDBOX Verwaltungsmodul\r\n"); 

    fprintf(outfile, "MID:\t%s\r\n",
      gen_message_id("verwaltung").c_str());

    if(config.organization[0]!=0)
      fprintf(outfile, "ORG:\t%s\r\n", config.organization);

    fprintf(outfile, "ROT:\r\n");

    fprintf(outfile, "\r\n");

    fwrite(text, 1, strlen(text), outfile);

    if(!unsubscribe)
      filecopy(infile, outfile, fi.len);

    fclose(infile);
    fclose(outfile);

    snprintf(file2, DIRLEN, BOXSPOOLINZCMAILDIR "%s.%lu-%lu-%u", config.boxdir,
      config.boxname, status.midcounter++, time(NULL), config.port);
    rename(file1, file2);
    break;

   case 4:
    online.setattr(2, FALSE);
    online.setcursor(12, 13);
    online.printf("Befehl:");
    online.setattr(1, FALSE);

     {
      char list[TEXTLEN*2], *tptr, tempstr[TEXTLEN*2];

      strmaxcpy(list, emp, (TEXTLEN*2)-1);
      tptr=strchr(list, '@'); if(tptr!=NULL) *tptr=0;
      tptr=strstr(list, "-request"); 
      if(tptr!=NULL)
        *tptr=0;
      else
       {
        if(strncmp(list, "owner-", 6)==0)
         {
          strmaxcpy(tempstr, list, (TEXTLEN*2)-1);
          strmaxcpy(list, tempstr+6, (TEXTLEN*2)-1);
         }
       }

      list[50]=0;
      snprintf(errormsg, TEXTLEN, "unsubscribe %s", list);
     }

    unsubscribe=TRUE;
    mes[4].disabled=TRUE;
    goto schleife;

   case MENU_HELP:
    goto schleife;
   }

  if(temporary) unlink(filename);

  if(!status.dologout) draw(TRUE);
 }

void readzcfoldert::fileinclude()
 {
  char bet[TEXTLEN]="", filename[DIRLEN]="";
  met mes[10];

  if(atoi(status.gruppe|"security")!=MAXSECURITY)
   {
    errorbox(&online, "Zugriff verweigert!");
    if(!status.dologout) draw(TRUE);
    return;
   }

 startover:
  online.setattr(2, FALSE);
  online.box(20, 8, 60, 18, BOXFRAMEDBL, "Fileinclude", TRUE);

  online.setcursor(22, 10);
  online.printf("Empfnger:");
  online.setcursor(24, 12);
  online.printf("Betreff:");
  online.setcursor(26, 14);
  online.printf("Datei:");

  online.setattr(2, TRUE);
  online.setcursor(33, 10);
  online.printf("%s", folder);
  online.setattr(2, FALSE);

  mes[0].type=met::EDIT;
  mes[0].key=0;
  mes[0].x=33;
  mes[0].y=12;
  mes[0].sizex=24;
  mes[0].maxlen=60;
  mes[0].text=bet;

  mes[1].type=met::EDIT;
  mes[1].key=0;
  mes[1].x=33;
  mes[1].y=14;
  mes[1].sizex=24;
  mes[1].maxlen=TEXTLEN-1;
  mes[1].text=filename;

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=25;
  mes[2].y=16;
  mes[2].key=Key_C_Ent;
  mes[2].sizex=6;
  mes[2].text="O^k";

  mes[3].type=met::COMMANDBUTTON;
  mes[3].x=42;
  mes[3].y=16;
  mes[3].key=0;
  mes[3].sizex=9;
  mes[3].text="^Abbruch";

  mes[4].type=met::END_OF_LIST;

  switch(domenu(&online, mes, 0, TRUE))
   {
   case 2:
    break;

   case MENU_HELP:
    /*
    displayhelp();
    */
    online.clrscr();
    if(status.dologout) break;
    goto startover;
   }

  if(!status.dologout) draw(TRUE);
 }

void readzcfoldert::doedit_bet()
 {
  met mes[10];
  int i;
  char data[1000];

  strmaxcpy(data, fi.bet, TEXTLEN-1);

  online.box(10, 9, 71, 15, BOXFRAMEDBL, "Betreff bearbeiten", TRUE);

  mes[0].type=met::EDIT;
  mes[0].x=13;
  mes[0].y=11;
  mes[0].sizex=54;
  mes[0].text=data;
  mes[0].maxlen=TEXTLEN-1;

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=13;
  mes[1].y=13;
  mes[1].sizex=25;
  mes[1].text="O^k";

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=42;
  mes[2].y=13;
  mes[2].sizex=25;
  mes[2].text="^Abbruch";

  mes[3].type=met::END_OF_LIST;

 schleife:
  i=domenu(&online, mes, 0, TRUE);
  if(i==MENU_HELP) goto schleife;

  if(i==1)
   {
    lseek(index, getindexpos(pos), SEEK_SET);
    if(::read(index, &fi, sizeof fi)!=sizeof fi) return;

    strmaxcpy(fi.bet, data, TEXTLEN-1);

    lseek(index, getindexpos(pos), SEEK_SET);
    if(::write(index, &fi, sizeof fi)!=sizeof fi) return;
   }
 }

void readzcfoldert::doedit_typ()
 {
  met mes[10];
  int i;
  bool data;

  data=fi.flags.binary;

  online.box(10, 9, 71, 15, BOXFRAMEDBL, "Nachrichtentyp bearbeiten", TRUE);

  mes[0].type=met::COMMANDBUTTON;
  mes[0].x=13;
  mes[0].y=11;
  mes[0].sizex=54;
  mes[0].text=(char *)(data?"Binrnachricht":"Textnachricht");
  mes[0].maxlen=TEXTLEN-1;

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=13;
  mes[1].y=13;
  mes[1].sizex=25;
  mes[1].text="O^k";

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=42;
  mes[2].y=13;
  mes[2].sizex=25;
  mes[2].text="^Abbruch";

  mes[3].type=met::END_OF_LIST;

 schleife:
  i=domenu(&online, mes, 0, TRUE);
  if(i==MENU_HELP) goto schleife;

  if(i==0)
   {
    data=!data;
    mes[0].text=(char *)(data?"Binrnachricht":"Textnachricht");
    goto schleife;
   }

  if(i==1)
   {
    lseek(index, getindexpos(pos), SEEK_SET);
    if(::read(index, &fi, sizeof fi)!=sizeof fi) return;

    fi.flags.binary=data;

    lseek(index, getindexpos(pos), SEEK_SET);
    if(::write(index, &fi, sizeof fi)!=sizeof fi) return;
   }
 }

void readzcfoldert::doedit_abs()
 {
  met mes[10];
  int i;
  char data[1000];

  strmaxcpy(data, fi.abs, TEXTLEN-1);

  online.box(10, 9, 71, 15, BOXFRAMEDBL, "Absender bearbeiten", TRUE);

  mes[0].type=met::EDIT;
  mes[0].x=13;
  mes[0].y=11;
  mes[0].sizex=54;
  mes[0].text=data;
  mes[0].maxlen=TEXTLEN-1;

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=13;
  mes[1].y=13;
  mes[1].sizex=25;
  mes[1].text="O^k";

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=42;
  mes[2].y=13;
  mes[2].sizex=25;
  mes[2].text="^Abbruch";

  mes[3].type=met::END_OF_LIST;

 schleife:
  i=domenu(&online, mes, 0, TRUE);
  if(i==MENU_HELP) goto schleife;

  if(i==1)
   {
    lseek(index, getindexpos(pos), SEEK_SET);
    if(::read(index, &fi, sizeof fi)!=sizeof fi) return;

    strmaxcpy(fi.abs, data, TEXTLEN-1);

    lseek(index, getindexpos(pos), SEEK_SET);
    if(::write(index, &fi, sizeof fi)!=sizeof fi) return;
   }
 }

void readzcfoldert::doedit_file()
 {
  met mes[10];
  int i;
  char data[1000];

  strmaxcpy(data, fi.file, TEXTLEN-1);

  online.box(10, 9, 71, 15, BOXFRAMEDBL, "Dateiname bearbeiten", TRUE);

  mes[0].type=met::EDIT;
  mes[0].x=13;
  mes[0].y=11;
  mes[0].sizex=54;
  mes[0].text=data;
  mes[0].maxlen=TEXTLEN-1;

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=13;
  mes[1].y=13;
  mes[1].sizex=25;
  mes[1].text="O^k";

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=42;
  mes[2].y=13;
  mes[2].sizex=25;
  mes[2].text="^Abbruch";

  mes[3].type=met::END_OF_LIST;

 schleife:
  i=domenu(&online, mes, 0, TRUE);
  if(i==MENU_HELP) goto schleife;

  if(i==1)
   {
    #if 0
    lseek(index, getindexpos(pos), SEEK_SET);
    if(::read(index, &fi, sizeof fi)!=sizeof fi) return;

    strmaxcpy(fi.file, data, TEXTLEN-1);

    lseek(index, getindexpos(pos), SEEK_SET);
    if(::write(index, &fi, sizeof fi)!=sizeof fi) return;
    #endif
   }
 }

void readzcfoldert::doedit()
 {
  met mes[10];
  int i;

  if(pos>=anz) return;

  lseek(index, getindexpos(pos), SEEK_SET);
  if(::read(index, &fi, sizeof fi)!=sizeof fi) return;

  if(!mail)
   {
    if(checkmodify())
      return;
   }

  online.box(20, 7, 60, 19, BOXFRAMEDBL, "Nachricht bearbeiten", TRUE);

  mes[0].type=met::COMMANDBUTTON;
  mes[0].x=23;
  mes[0].y=9;
  mes[0].sizex=33;
  mes[0].text="^Betreff editieren";

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=23;
  mes[1].y=11;
  mes[1].sizex=33;
  mes[1].text="^Dateiname editieren";

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=23;
  mes[2].y=13;
  mes[2].sizex=33;
  mes[2].text="^Nachrichtentyp";

  mes[3].type=met::COMMANDBUTTON;
  mes[3].x=23;
  mes[3].y=15;
  mes[3].sizex=33;
  mes[3].text="^Absender";

  mes[4].type=met::COMMANDBUTTON;
  mes[4].x=23;
  mes[4].y=17;
  mes[4].sizex=33;
  mes[4].text="^Men verlassen";

  mes[5].type=met::END_OF_LIST;

 schleife:
  i=domenu(&online, mes, 0, TRUE);
  if(i==MENU_HELP) goto schleife;

  switch(i)
   {
   case 0:
    doedit_bet();
    break;

   case 1:
    doedit_file();
    break;

   case 2:
    doedit_typ();
    break;

   case 3:
    doedit_abs();
    break;
   }

  if(!status.dologout) draw(TRUE);
 }

void readzcfoldert::reply(bool quote, bool followup, bool quoteall)
 {
  char bet[TEXTLEN*2], bez[TEXTLEN*2]="";
  FILE *quotefile=NULL;
  char quotestring[TEXTLEN*2]="";
  char quotefilename[DIRLEN];

  lseek(index, getindexpos(pos), SEEK_SET);
  if(::read(index, &fi, sizeof fi)!=sizeof fi) return;

  if(strncmp(fi.bet, "Re: ", 4)==0)
    strmaxcpy(bet, fi.bet, TEXTLEN-1);
  else
   {
    snprintf(bet, TEXTLEN, "Re: %s", fi.bet);
    bet[TEXTLEN-1]=0;
   }

  getheader("MID", bez);

  if(quoteall)
   {
    infobox(&online, nls["online.notavailable"]);
    if(!status.dologout) draw(TRUE);
    return;
   }

  if(quote)
   {
    dword laenge;

    kom=getkom();

    if(fi.flags.binary && kom==0)
     {
      errorbox(&online, "Nachricht ist binaer!");
      if(!status.dologout) draw(TRUE);
      return;
     }
 
    snprintf(quotefilename, DIRLEN, DBOXPORTTEMPDIR "posting", config.dboxdir, config.port);

    if(kom==0 || kom>fi.len)
      laenge=fi.len;
    else 
      laenge=kom;
  
    if(!followup)
      snprintf(quotestring, TEXTLEN*2, nls["online.quotestringmail"], fi.abs);
    else
     {
      struct tm *tm;
      tm=localtime(&fi.eda);
      snprintf(quotestring, TEXTLEN*2, nls["online.quotestringnews"],
      tm->tm_mday, tm->tm_mon+1, tm->tm_year%100);
     }

    if(!formatposting(quotefilename, laenge, TRUE, quotestring))
      quotefile=fopen(quotefilename, "rb");
   }
   
  if(followup)
   {
    if(get_news_folder_grants(folder, status.userprofile, status.gruppe,
                              FALSE, TRUE, FALSE))
      errorbox(&online, "Schreibzugriff verweigert!");
    else
     {
      // TODO: Diskussion-In
      dowritemail(folder, bet, bez, quotefile);
      reread=TRUE;
     }
   }
  else
   {
    char emp[TEXTLEN], *tptr;

    if(getheader("ANTWORT-AN", emp))  // TODO: antwort-an mehrfach
      strmaxcpy(emp, fi.abs, TEXTLEN-1);

    tptr=strchr(emp, ' ');
    if(tptr!=NULL) *tptr=0;

    dowritemail(emp, bet, bez, quotefile);
   }

  if(quotefile!=NULL) 
   { 
    fclose(quotefile);
    remove(quotefilename);
   }

  if(!status.dologout && !reread) draw(TRUE);
 }

void readzcfoldert::send()
 {
  if(mail)
    dowritemail("");
  else
   {
    if(get_news_folder_grants(folder, status.userprofile, status.gruppe,
                              FALSE, TRUE, FALSE))
      errorbox(&online, "Schreibzugriff verweigert!");
    else
     {
      dowritemail(folder);
      reread=TRUE;
     }
   }

  if(!status.dologout && !reread) draw(TRUE);
 }
