/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <time.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netdb.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/wait.h>
#include <signal.h>
#include <arpa/telnet.h>

#include <dbox.h>
#include <oldui.h>
#include <oldtools.h>
#include "online.h"
#include "help.h"
#include "sockett.h"
#include "telnet.h"

// #define DEBUG_TELNET

class telnett
 {
 public:
  bool iac, iac2, sb, sgaflg;
  byte iac2_char, sb_buffer[TEXTLEN];
  int sb_buffer_bytes;
  sockett socket;

  void processchar(byte ch);
  void sendchar(byte ch);
  void sendescseq(const char *seq);
  void send_iac(byte cmd, byte opt);

  void doit(const char *host);

  telnett()
   {
    iac=iac2=sb=sgaflg=FALSE;
   }
 };

void telnett::send_iac(byte cmd, byte opt)
 {
  byte io_data[3];

  io_data[0]=IAC;
  io_data[1]=cmd;
  io_data[2]=opt;

  socket.write(io_data, 3);
 }

void telnett::sendchar(byte ch)
 {
  if(ch==IAC)
   {
    socket.write(&ch, 1);
    socket.write(&ch, 1);
   }
  else
    socket.write(&ch, 1);
 }

void telnett::processchar(byte ch)
 {
  if(iac2)
   {
    iac2=FALSE;
    switch(ch)
     {
     case TELOPT_ECHO:                 /* ECHO negotiation. */
      switch(iac2_char)
       {
       case WILL:                      /* Host says it will echo. */
        #ifdef DEBUG_TELNET
        online.printf("DK REMOTE WILL ECHO\r\n");
        #endif
        break;

       case WONT:                      /* Host says it won't echo. */
        #ifdef DEBUG_TELNET
        online.printf("DK REMOTE WONT ECHO\r\n");
        #endif
        break;

       case DO:                        /* Host wants me to echo */
        #ifdef DEBUG_TELNET
        online.printf("DK REMOTE DOES ECHO\r\n");
        #endif
        send_iac(WONT, ch);            /* I say I won't, */
        send_iac(DO, ch);              /* and ask the host to echo. */
        break;

       case DONT:                      /* Host wants me not to echo */
        #ifdef DEBUG_TELNET
        online.printf("DK REMOTE DOES NOT ECHO\r\n");
        #endif
        send_iac(WONT, ch);            /* I say I won't. */
        break;
       }
      break;

      case TELOPT_SGA:                 /* Suppress Go-Ahead */
       switch(iac2_char)   
        {
        case WONT:                     /* Host says it won't. */
         sgaflg=TRUE;                  /* Remember. */
         send_iac(DONT,ch);            /* acknowledge, */
         break;

        case WILL:                     /* Host says it will. */
         sgaflg=FALSE;                 /* Remember. */
        }
       break;

     case TELOPT_TTYPE:                /* Terminal Type */
      switch(iac2_char)
       {
       case DO:                        /* DO terminal type. */
        #ifdef DEBUG_TELNET
        online.printf("DK REMOTE DOES TTYPE\r\n");
        #endif
        send_iac(WILL, ch);            /* Say I'll send it if asked. */
        break;

       case SB:                        /* enter subnegociations */
        #ifdef DEBUG_TELNET
        online.printf("DK REMOTE SB TTYPE\r\n");
        #endif
        sb=TRUE;
        sb_buffer_bytes=1;
        sb_buffer[0]=ch;
        break;
       }
      break;

     default:                          /* All others: refuse */
      switch(iac2_char) 
       {
       case WILL:                      /* You will? */
        #ifdef DEBUG_TELNET
        online.printf("DK REMOTE WILL %d\r\n", ch);
        #endif
        send_iac(DONT, ch);            /* Please don't. */
        break;
    
       case DO:                        /* You want me to? */
        #ifdef DEBUG_TELNET
        online.printf("DK REMOTE DOES %d\r\n", ch);
        #endif
        send_iac(WONT, ch);            /* I won't. */
        send_iac(DONT, ch);            /* Don't you either. */
        break;

       case DONT:                      /* (fall thru...) */
        #ifdef DEBUG_TELNET
        online.printf("DK REMOTE DOES NOT %d\r\n", ch);
        #endif
        send_iac(WONT, ch);            /* I won't. */
        break;

       case WONT:                      /* You won't? */
        #ifdef DEBUG_TELNET
        online.printf("DK REMOTE WONT %d\r\n", ch);
        #endif
        break;                         /* Good. */
       }
     }
   }
  else if(iac)
   {
    iac=FALSE;

    if(ch==IAC)
     {
      if(sb)
       {
        if(sb_buffer_bytes<TEXTLEN)
          sb_buffer[sb_buffer_bytes++]=ch;
       }
      else
        online.printf("%c", ch);
     }
    else if(ch==SE)
     {
      if(sb)
       {
        sb=FALSE;
        if(sb_buffer[0]==TELOPT_TTYPE && sb_buffer[1]==1)
         {
          #ifdef DEBUG_TELNET
          online.printf("DK TTYPE?\r\n");
          #endif
          send_iac(SB, TELOPT_TTYPE);
          sb_buffer[0]=0;
          socket.write(sb_buffer, 1);
          socket.form("vt100");
          sb_buffer[0]=IAC;
          sb_buffer[1]=SE;
          socket.write(sb_buffer, 2);
         }
       }
     }
    else
     {
      iac2=TRUE;
      iac2_char=ch;
     }
   }
  else if(ch==IAC)
    iac=TRUE;
  else
   {
    if(sb)
     {
      if(sb_buffer_bytes<TEXTLEN)
        sb_buffer[sb_buffer_bytes++]=ch;
     }
    else
      online.printf("%c", ch);
   }
 }

void telnett::sendescseq(const char *seq)
 {
  sendchar(27);
  while(seq[0]!=0) sendchar(*(seq++));
 }

void telnett::doit(const char *host)
 {
  socket.set_verbose(TRUE);

  online.setattr(1, FALSE);

  online.printf("Escape character is '^]'.\r\n");

  socket.open(host, 23);

  online.set_app_out_charset(ISO);

  do
   {
    byte buffer[1000];
    int n, i;
    word key;
    bool fast;

    n=socket.read(buffer, 900);

    if(n>0)
     {
      fast=TRUE;

      for(i=0; i<n && fast; i++)
        if(buffer[i]==IAC)
          fast=FALSE;

      if(fast && !iac && !sb && !iac2)
       {
        buffer[n]=0;
        online.printf("%s", buffer);
       }
      else
        for(i=0; i<n; i++)
          processchar(buffer[i]);
     }

    key=online.readkey();

    switch(key)
     {
     case 0:
      break;

     case Key_C_2:
      sendchar(0);
      break;

     case Key_BS:
      sendchar(127);
      break;

     case Key_Up:  sendescseq("[A"); break;
     case Key_Dwn: sendescseq("[B"); break;
     case Key_Lft: sendescseq("[D"); break;
     case Key_Rgt: sendescseq("[C"); break;
     case Key_Home: sendescseq("[1~"); break;
     case Key_End: sendescseq("[4~"); break; 
     case Key_PgUp: sendescseq("[5~"); break; 
     case Key_PgDn: sendescseq("[6~"); break; 

     case 0x1d:
      socket.close();
      online.set_app_out_charset(IBM);
      online.setattr(1, FALSE);
      online.console.printf("\r\n\r\nConnection aborted.\r\n");
      return;

     case 0xffff:
      socket.close();
      online.set_app_out_charset(IBM);
      return;

     default:
      if(lo(key)!=0)
        sendchar(lo(key));
     }
   }
  while(socket.is_open());

  socket.close();

  online.set_app_out_charset(IBM);

  online.setattr(1, FALSE);
  
  if(socket.get_error_number()==0)
    online.printf("\r\nConnection closed by foreign host.\r\n");
  else
    online.printf("\r\nConnection closed. (%s)\r\n",
      strerror(socket.get_error_number()));
 }

void dointernet_telnet(const char *host)
 {
  telnett telnet;
  char hostname[TEXTLEN]="";

  if(host==NULL || host[0]==0)
   {
    met mes[4];

    if(status.dologout) return;

   startover:
    online.setattr(2, FALSE);
    online.box(20, 10, 60, 16, BOXFRAMEDBL, "Internet - Telnet", TRUE);

    online.setcursor(22, 12);
    online.printf("Hostname:");

    mes[0].type=met::EDIT;
    mes[0].key=0;
    mes[0].x=32;
    mes[0].y=12;
    mes[0].sizex=25;
    mes[0].maxlen=60;
    mes[0].text=hostname;

    mes[1].type=met::COMMANDBUTTON;
    mes[1].x=26;
    mes[1].y=14;
    mes[1].key=Key_C_Ent;
    mes[1].sizex=6;
    mes[1].text="O^k";

    mes[2].type=met::COMMANDBUTTON;
    mes[2].x=43;
    mes[2].y=14;
    mes[2].key=0;
    mes[2].sizex=9;
    mes[2].text="^Abbruch";

    mes[3].type=met::END_OF_LIST;

    switch(domenu(&online, mes, 0, TRUE))
     {
     case 1:
      if(status.dologout) return;
      host=hostname;
      break;

     case MENU_HELP:
      //displayhelp(helpurl);
      if(status.dologout) return;
      online.setattr(1, FALSE);
      online.clrscr();
      goto startover;

     default:
      host="";
     }

    online.setattr(1, FALSE);
    online.clrscr();
    online.setcursor(1, 1);
   }  

  if(host[0]!=0)
    telnet.doit(host);
 }

