/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <dirent.h>
#include <time.h>

#include <userdb.h>
#include <dboxpath.h>
#include <portinfo.h>
#include <dboxversion.h>
#include <oldtools.h>
#include <oldui.h>
#include <logf.h>
#include <zconnect.h>
#include <messageid.h>
#include <flags.h>
#include "online.h"
#include "selport.h"
#include "userlog.h"
#include "help.h"
#include "vfolder.h"
#include "fibu.h"
#include "menu.h"
#include "readnews.h"
#include "alias.h"
#include "verw.h"

bool verify_access(userpt &up)
 {
  grouppt gp(up|"gruppe");

  gp.read();

  if(atoi(status.gruppe|"security")==MAXSECURITY)
    return FALSE;

  if(atoi(status.gruppe|"security")>=atoi(gp|"security"))
    return FALSE;

  if(stricmp(status.username, up.name)==0)
    return FALSE;

  return TRUE;
 }

static dword countdir(char *dirname)
 {
  DIR *dir;
  struct dirent *ent;
  dword count=0;

  if((dir=opendir(dirname))==NULL)
    return 0;

  while((ent=readdir(dir))!=NULL)
   {
    if(ent->d_name[0]!='.' && ent->d_name[0]!='_')
      count++;
   }

  closedir(dir);

  return count;
 }

static dword countuser()
 {
  char dirname[DIRLEN];
  snprintf(dirname, DIRLEN, BOXUSERDIR ".", config.boxdir, ".");
  return countdir(dirname);
 }

static dword countgruppen()
 {
  dword count=0;
  while(config.groupdb[count]!=NULL) count++;
  return count;
 }

static void showportstatus(word port)
 {
  byte y=11;

  clearworkscreen(&online);
  portinfo.read(port);

  online.setcursor(26, y);
  online.setattr(1, TRUE);
  online.printf("Statusinformationen fr Port %u", port);
  online.setattr(1, FALSE);
  y+=2;

  online.setcursor(3, y);
  y+=2;

  if(port==0)
   {
    online.printf("Dies ist der DBOX-Masterport.");
   }
  else if(stricmp(portinfo.info.boxname, config.boxname)!=0 &&
          !config.mainsystem)
   {
    online.printf("Dieser Port ist gehrt nicht zu Ihrer Mailbox!");
   }
  else
   {
    switch(portinfo.info.status)
     {
     case portinfot::infot::OFFLINE:
      online.printf("Dieser Port ist zur Zeit nicht in Benutzung.");
      break;

     case portinfot::infot::WORKING:
      online.printf("Dieser Port ist zur Zeit interner Arbeit beschftigt.");
      break;

     case portinfot::infot::FREE:
      online.printf("Dieser Port ist zur Zeit frei und erwartet einen Anruf.");
      break;

     case portinfot::infot::RING:
      online.printf("Dieser Port wartet zur Zeit auf einen CONNECT.");
      break;

     case portinfot::infot::LOGIN:
      online.printf("Auf diesem Port loggt sich zur Zeit einen User ein.");
      break;

     case portinfot::infot::ONLINE:
     case portinfot::infot::CHATTING:
      online.printf("Auf diesem Port ist zur Zeit ein User eingeloggt.");
     showuserinfo:
      online.setcursor(3, y);
      online.printf("Username:       %s", portinfo.info.username);
      y++;

      if(stricmp(portinfo.info.boxname, config.boxname)!=0)
       {
        online.setcursor(3, y);
        online.printf("Fehler! Dieser User ist nicht in Ihrer Box!");
        y++;
       }
      else
       {
        userpt up(portinfo.info.username);

        if(up.read())
         {
          online.setcursor(3, y);
          online.printf("Fehler! ber diesen User sind keine Informationen verfgbar!");
          y++;
         }
        else
         {
          online.setcursor(3, y);
          online.printf("  Name:         %s %s",
            up|"vorname", up|"nachname");
          y++;

          if((up|"telefon")[0]!=0)
           {
            online.setcursor(3, y);
            online.printf("  Telefon:      %s", up|"telefon");
            y++;
           }

          if(portinfo.info.from[0]!=0)
           {
            online.setcursor(3, y);
            online.printf("  From:         %s", portinfo.info.from);
            y++;
           }

          online.setcursor(3, y);
          online.printf("  Gruppe:       %s", up|"gruppe");
          y++;
         }
        y++;
       }
      break;

     case portinfot::infot::NETCALL:
      online.printf("Auf diesem Port findet zur Zeit ein Netcall statt.");
      online.setcursor(3, y);
      online.printf("Boxname:        %s", portinfo.info.username);
      y++;

      if(portinfo.info.from[0]!=0)
       {
        online.setcursor(3, y);
        online.printf("  From:         %s", portinfo.info.from);
        y++;
       }

      y++;
      break;

     case portinfot::infot::SLIP:
      online.printf("Auf diesem Port ist zur Zeit SLIP in Verwendung.");
      goto showuserinfo;

     case portinfot::infot::ERROR:
      online.printf("Auf diesem Port liegt ein Hardwarefehler vor.");
      break;
     }

    if(portinfo.info.modemname[0]!=0)
     {
      online.setcursor(3, y);
      online.printf("Modemname:      %s", portinfo.info.modemname);
      y++;
     }

    if(portinfo.info.telefon[0]!=0)
     {
      online.setcursor(3, y);
      online.printf("Telefonnummer:  %s", portinfo.info.telefon);
      y++;
     }
   }

  while(online.doreadkey()==0);
 }

static void showstatus()
 {
  bool ende=FALSE;
  word port=0;

  do
   {
    char tempstr[TEXTLEN], tempstr2[TEXTLEN];

    clearworkscreen(&online);
    printtimeout();
    strmaxcpy(tempstr2, config.boxname, TEXTLEN-1);
    upstring(tempstr2);
    snprintf(tempstr, TEXTLEN, "Status der Mailbox %s", tempstr2);
    online.setcursor(40-strlen(tempstr)/2, 11);
    online.setattr(1, TRUE);
    online.printf(tempstr);
    online.setattr(1, FALSE);

    online.setcursor(3, online.sizey-2);
    online.printf("User: %-4lu Gruppen: %-4lu",
      countuser(), countgruppen());

    drawports(&online, 13, 0);
    port=selectport(&online, 13, port);
    if(port==0xffff)
      ende=TRUE;
    else
      showportstatus(port);

    if(status.dologout) ende=TRUE;
   }
  while(!ende);
  if(!status.dologout) clearworkscreen(&online);
 }

static void einstellungen()
 {
  int i, newi=7;
  met mes[20];
  byte x;
  bool redraw;
  char tempstr[TEXTLEN];

 startover:
  drawframe("Allgemeine Einstellungen");
  snprintf(tempstr, TEXTLEN, "Einstellungen fr Mailbox %s.%s",
          config.boxname, config.domain);
  i=strlen(tempstr);
  x=(online.sizex-i)/2;
  online.setcursor(x, 3);
  online.setattr(1, TRUE);
  online.printf("%s", tempstr);
  online.setattr(1, FALSE);
  online.setcursor(x, 4);
  for(; i>0; i--) online.printf("");

  mes[0].type=met::COMMANDBUTTON;
  mes[0].x=12;
  mes[0].y=online.sizey-3;
  mes[0].key=Key_C_Ent;
  mes[0].sizex=6;
  mes[0].text="O^k";

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=22;
  mes[1].y=online.sizey-3;
  mes[1].sizex=9;
  mes[1].text="^Abbruch";

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=35;
  mes[2].y=online.sizey-3;
  mes[2].sizex=22;
  mes[2].text="^xxxxxxxxxxxxxxxxxxxx";

  mes[3].type=met::COMMANDBUTTON;
  mes[3].x=61;
  mes[3].y=online.sizey-3;
  mes[3].sizex=7;
  mes[3].text="^Hilfe";

  mes[4].type=met::END_OF_LIST;
  redraw=TRUE;

 //schleife:
  i=domenu(&online, mes, newi, redraw);
  redraw=FALSE;

  switch(i)
   {
   case 3:
    newi=i;
    displayhelp("verw/settings.html");
    goto startover;

   case MENU_HELP:
    newi=7;
    displayhelp("verw/settings.html");
    goto startover;
   }
 }

#define MAXENTRIES 20000

struct indexentryt
 {
  char *name, *description;
 };

static int sort_string(const void *a, const void *b)
 {
  indexentryt *aa, *bb;

  aa=(indexentryt *)a;
  bb=(indexentryt *)b;

  return stricmp(aa->name, bb->name);
 }

class userverwt:public viewfoldert
 {
 public:
  indexentryt index[MAXENTRIES];
  bool schnellsuche;
  word schnellsuche_zeichen;

  virtual void readindex();
  virtual void freeindex();
  virtual void displayline(dword line);
  virtual void key(word k);
  virtual bool check(const char *name) { return FALSE; }
  virtual void draw(bool complete);
  virtual void setcursor();
  virtual void read();
  virtual void help();
  virtual void eintragen();
  virtual void umbenennen();
  virtual void del();

  userverwt()
   {
    schnellsuche=FALSE;
   }

 private:
  void doedit(const char *username);
 };

void userverwt::readindex()
 {
  DIR *dir;
  struct dirent *ent;
  char tempstr[TEXTLEN];

  online.clrscr();
  online.setcursor(1, 1);
  online.printf("Bitte warten...");

  anz=0;

  snprintf(tempstr, TEXTLEN, BOXUSERDIR ".", config.boxdir, ".");
  if((dir=opendir(tempstr))==NULL) return;

  while((ent=readdir(dir))!=NULL && anz<MAXENTRIES)
    if(ent->d_name[0]!='.' && ent->d_name[0]!='_' && !check(ent->d_name))
     {
      index[anz].name=strdup(ent->d_name);
      if(index[anz].name==NULL)
       {
        logf.printf("userverwt::readindex", "FATAL: Zu wenig RAM!");
        return;
       }
      index[anz].description=NULL;
      anz++;
     }

  qsort(index, anz, sizeof(indexentryt), sort_string);

  closedir(dir);
 }

void userverwt::freeindex()
 {
  dword i;
  for(i=0; i<anz; i++)
   {
    free(index[i].name);
    if(index[i].description!=NULL) free(index[i].description);
   }
 }

void userverwt::displayline(dword line)
 {
  char tempstr[TEXTLEN*2], tempstr2[TEXTLEN*2];

  if(line>=anz) return;

  if(index[line].description==NULL)
   {
    userpt up(index[line].name);

    if(up.read())
      index[line].description=strdup("User nicht gefunden!");
    else
     {
      if((up|"alias")[0]!=0)
        snprintf(tempstr, TEXTLEN, "-> %s", up|"alias");
      else
       {
        if((up|"firma")[0]!=0)
          strmaxcpy(tempstr, up|"firma", TEXTLEN-1);
        else if((up|"vorname")[0]!=0)
          snprintf(tempstr, TEXTLEN, "%s, %s", up|"nachname", up|"vorname");
        else
          strmaxcpy(tempstr, up|"nachname", TEXTLEN-1);
       }

      tempstr[online.sizex-43]=0;
      index[line].description=strdup(tempstr);
     }
   }
 
  strmaxcpy(tempstr2, index[line].name, TEXTLEN-1);
  tempstr2[40]=0;
  snprintf(tempstr, TEXTLEN, " %-40s ", tempstr2);

  if(index[line].description!=NULL)
    strcat(tempstr, index[line].description);

  viewfoldert::displayline(line, tempstr);
 }

void userverwt::key(word k)
 {
  char ch=upcase(lo(k));

  if(schnellsuche)
   {
    if(k==0xffff)
     {
      ende=TRUE;
      schnellsuche=FALSE;
      return;
     }
    else if(k==Key_BS || k==Key_Del)
     {
      if(schnellsuche_zeichen!=0)
       {
        schnellsuche_zeichen--;
        setcursor();
       }

      return;
     }
    else if(k==Key_Ent || k==Key_Esc)
     {
      schnellsuche=FALSE;
      setcursor();
      return;
     }
    else if(lo(k)<' ')
     {
      schnellsuche=FALSE;
      setcursor();
     } 
    else if(lo(k)=='/')
     {
      k=Key_Ent;
      schnellsuche_zeichen=0;
      schnellsuche=FALSE;
     }
    else
     {
      dword i;
      char string[TEXTLEN];

      strmaxcpy(string, index[pos].name, TEXTLEN-1);
      string[schnellsuche_zeichen]=lo(k);
      string[schnellsuche_zeichen+1]=0;

      for(i=0; i<anz; i++)
        if(strnicmp(index[i].name, string, schnellsuche_zeichen+1)==0)
          break;

      if(i<anz)
       {
        schnellsuche_zeichen++;
        setpos(i, TRUE);
        setcursor();
       }

      return;
     }
   }

  switch(ch)
   {
   case '/':
    schnellsuche=TRUE;
    schnellsuche_zeichen=0;
    setcursor();
    break;

   case 'E':
    eintragen();
    break;

   case 'U':
    if(anz!=0) umbenennen();
    break;

   default:
    viewfoldert::key(k);
   }
 }

void userverwt::draw(bool complete)
 {
  snprintf(title, TEXTLEN, "Userliste %s - drcken Sie ? fr Hilfe",
    config.boxname);

  viewfoldert::draw(complete);
 }

void userverwt::setcursor()
 {
  if(schnellsuche)
    online.setcursor(schnellsuche_zeichen+2, 2+pos-scroll);
  else
    online.setcursor(1, 2+pos-scroll);
 }

bool changepassword(char *password)
 {
  met mes[4];
  bool erg=TRUE;
  char temppw[TEXTLEN];

  strmaxcpy(temppw, password, TEXTLEN-1);

  online.setattr(2, FALSE);
  online.box(20, 10, 60, 16, BOXFRAMEDBL, "Passwort ndern", TRUE);
  online.setcursor(25, 12);
  online.printf("Passwort:");

  mes[0].type=met::EDIT;
  mes[0].key=0;
  mes[0].x=35;
  mes[0].y=12;
  mes[0].sizex=18;
  mes[0].maxlen=30;
  mes[0].text=temppw;

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=26;
  mes[1].y=14;
  mes[1].key=Key_C_Ent;
  mes[1].sizex=6;
  mes[1].text="O^k";

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=43;
  mes[2].y=14;
  mes[2].key=0;
  mes[2].sizex=9;
  mes[2].text="^Abbruch";

  mes[3].type=met::END_OF_LIST;

 schleife:
  switch(domenu(&online, mes, 0, TRUE))
   {
   case 1:
    erg=FALSE;
    strmaxcpy(password, temppw, TEXTLEN-1);
    break;

   case MENU_HELP:
    goto schleife;
   }
  online.setattr(1, FALSE);

  return erg;
 }

void showuserlog(const char *username)
 {
  char title[TEXTLEN];
  char tempstr[DIRLEN];
  FILE *f;

  snprintf(tempstr,  DIRLEN, BOXUSERDIR "logfile", config.boxdir, username);
  f=fopen(tempstr, "rt");
  if(f==NULL)
   {
    errorbox(&online, "Keine Logdatei vorhanden!");
   }
  else
   {
    snprintf(title, TEXTLEN, "Userlogdatei von %s", username),
    title[78]=0;
    viewtextfile(&online, f, title, IBM, FALSE);
    fclose(f);
   }
 }

void userverwt::read()
 {
  doedit(index[pos].name);

  if(!status.dologout && !reread) draw(TRUE);
 }

void userverwt::doedit(const char *username)
 {
  userpt up(username);

  if(up.read())
   {
    errorbox(&online, "profile konnte nicht gelesen werden!");
    return;
   }

  const char *alias=up|"alias";

  if(alias[0]!=0)
   {
    dword i;

    for(i=0; i<anz && (stricmp(alias, index[i].name)!=0); i++);

    if(i<anz)
      setpos(i, FALSE);
    else
      errorbox(&online, "User nicht gefunden!");

    return;
   }

  if(verify_access(up))
   {
    errorbox(&online, "Zugriff verweigert!");
    logf.printf("userverwt::doedit", "Illegaler Versuch User %s zu editieren!",
      username);
    return;
   }

  doonlinemenu("userverw", &up, FALSE, FALSE, TRUE, NULL);

  if(stricmp(username, status.username)==0)
    status.userprofile.read();

  reread=TRUE;
 }

void userverwt::help()
 {
  displayhelp("verw/userlist.html");
  if(!status.dologout) draw(TRUE);
 }

void userverwt::del()
 {
  met mes[3];
  const char *username;
  grouppt gruppe;

  username=index[pos].name;

  if(stricmp(username, status.username)==0)
   {
    errorbox(&online, "Sie knnen sich nicht selbst austragen!");
    if(!status.dologout) draw(TRUE);
    return;
   }

  userpt up(username);

  if(up.read())
   {
    errorbox(&online, "profile konnte nicht gelesen werden!");
    if(!status.dologout) draw(TRUE);
    return;
   }

  bool alias=((up|"alias")[0]!=0);

  if(alias)
   {
    userpt up2(up|"alias");

    if(up2.read())
     {
      errorbox(&online, "Alias fehlerhaft!");
      return;
     }
    else
      strmaxcpy(gruppe.name, up2|"gruppe", TEXTLEN-1);
   }
  else
    strmaxcpy(gruppe.name, up|"gruppe", TEXTLEN-1);

  gruppe.read();

  if(atoi(status.gruppe|"security")!=MAXSECURITY &&
     atoi(status.gruppe|"security")<=atoi(gruppe|"security"))
   {
    errorbox(&online, "Zugriff verweigert!");
    if(!status.dologout) draw(TRUE);
    logf.printf("userverwt::del", "Illegaler Versuch User %s zu lschen!",
      username);
    return;
   }

  online.setattr(2, FALSE);
  online.box(20, 8, 60, 16, BOXFRAMEDBL, 
    alias?"Alias lschen":"User lschen", TRUE);

  online.setcursor(23, 10);
  online.printf("Username: ");
  online.setattr(2, TRUE);

   {
    char tempstr[TEXTLEN];
    strmaxcpy(tempstr, username, 25);
    online.printf("%s", tempstr);
   }

  online.setattr(2, FALSE);
  online.setcursor(28, 12);
  online.printf("%s wirklich austragen?", alias?"Alias":"User");

  mes[0].type=met::COMMANDBUTTON;
  mes[0].x=26;
  mes[0].y=14;
  mes[0].key=Key_C_Ent;
  mes[0].sizex=6;
  mes[0].text="O^k";

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=43;
  mes[1].y=14;
  mes[1].key=0;
  mes[1].sizex=9;
  mes[1].text="^Abbruch";

  mes[2].type=met::END_OF_LIST;

 schleife:
  switch(domenu(&online, mes, 1, TRUE))
   {
   case 0:
    if(alias)
     {
      userlogprintf2(status, up.name, "uservertw::del", 
        "Alias \"%s\" wird durch User \"%s\" geloescht!",
        up.name, status.username);

      del_alias(&up); 
      reread=TRUE;
     }
    else
     {
      del_all_alias(&up);

      userlogprintf2(status, up.name, "uservertw::del", 
        "User \"%s\" wird durch User \"%s\" geloescht!",
        up.name, status.username);

      if(up.del())
       {
        errorbox(&online, "Lschung fehlgeschlagen!");
        logf.printf("userverwt::del", 
          "Loeschung von \"%s\" fehlgeschlagen!");
       }
      else
       {
        userlogprintf1(status, "userverwt::del", 
          "User \"%s\" wurde durch User \"%s\" geloescht!",
          username, status.username);
        reread=TRUE;
       }
     }
    break;

   case MENU_HELP:
    goto schleife;
   }

  online.setattr(1, FALSE);

  if(!status.dologout && !reread) draw(TRUE);
 }

void userverwt::eintragen()
 {
  char username[TEXTLEN]="";
  met mes[3];

  if(atoi(status.gruppe|"security")==0)
   {
    errorbox(&online, "Zugriff verweigert!");
    if(!status.dologout) draw(TRUE);
    return;
   }

  online.setattr(2, FALSE);
  online.box(20, 10, 60, 16, BOXFRAMEDBL, "User eintragen", TRUE);

  online.setcursor(26, 12);
  online.printf("Username:");

  mes[0].type=met::EDIT;
  mes[0].key=0;
  mes[0].x=36;
  mes[0].y=12;
  mes[0].sizex=16;
  mes[0].maxlen=40;
  mes[0].text=username;

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=26;
  mes[1].y=14;
  mes[1].key=Key_C_Ent;
  mes[1].sizex=6;
  mes[1].text="O^k";

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=43;
  mes[2].y=14;
  mes[2].key=0;
  mes[2].sizex=9;
  mes[2].text="^Abbruch";

  mes[3].type=met::END_OF_LIST;

 schleife:
  switch(domenu(&online, mes, 0, TRUE))
   {
   case 1:
    dnstring(username);

    if(checkusername(username))
     {
      errorbox(&online, "Username ist ungltig!");
     }
    else
     {
      userpt up(username);

      if(!up.read())
        errorbox(&online, "User existiert bereits!");
      else if(up.lock())
        errorbox(&online, "Eintragung fehlgeschlagen!");
      else
       {
        up.set("eintragsdatum", time(NULL));
        up.set("dogastsettings", 2);

        if(up.add())
         {
          errorbox(&online, "Eintragung fehlgeschlagen!");
          up.unlock();
         }
        else
         {
          up.unlock();
          userlogprintf2(status, username, "userverwt::eintragen",
            "User %s durch %s eingetragen", username, status.username);
          reread=TRUE;
          doedit(username);
         }
       }
     }
    break;

   case MENU_HELP:
    goto schleife;
   }

  online.setattr(1, FALSE);
  if(!status.dologout && !reread) draw(TRUE);
 }

void userverwt::umbenennen()
 {
  char *oldusername, newusername[TEXTLEN]="";
  met mes[3];
  char tempstr[DIRLEN];

  oldusername=index[pos].name;

  if(stricmp(oldusername, status.username)==0)
   {
    errorbox(&online, "Sie knnen sich nicht selbst umbenennen!");
    if(!status.dologout) draw(TRUE);
    return;
   }

  userpt up(oldusername);
  if(up.read())
   {
    errorbox(&online, "profile konnte nicht gelesen werden!");
    if(!status.dologout) draw(TRUE);
    return;
   }

  if(verify_access(up))
   {
    errorbox(&online, "Zugriff verweigert!");
    if(!status.dologout) draw(TRUE);
    logf.printf("userverwt::del", "Illegaler Versuch User %s umzubenennen!",
      oldusername);
    return;
   }

  online.setattr(2, FALSE);
  online.box(20, 10, 60, 18, BOXFRAMEDBL, "User umbenennen", TRUE);

  online.setcursor(22, 12);
  online.printf("alter Username:");
  online.setcursor(22, 14);
  online.printf("neuer Username:");

  online.setattr(2, TRUE);
  online.setcursor(38, 12);
  online.printf("%s", oldusername);

  mes[0].type=met::EDIT;
  mes[0].key=0;
  mes[0].x=38;
  mes[0].y=14;
  mes[0].sizex=16;
  mes[0].maxlen=40;
  mes[0].text=newusername;

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=26;
  mes[1].y=16;
  mes[1].key=Key_C_Ent;
  mes[1].sizex=6;
  mes[1].text="O^k";

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=43;
  mes[2].y=16;
  mes[2].key=0;
  mes[2].sizex=9;
  mes[2].text="^Abbruch";

  mes[3].type=met::END_OF_LIST;

 schleife:
  switch(domenu(&online, mes, 0, TRUE))
   {
   case 1:
    dnstring(newusername);

    if(checkusername(newusername))
     {
      errorbox(&online, "Username ist ungltig!");
     }
    else     
     {
      snprintf(tempstr, DIRLEN, BOXUSERDIR "profile", config.boxdir, newusername);
      if(access(tempstr, 00)==0)
       {
        errorbox(&online, "User existiert bereits!");
       }
      else
       {

        #if 0
        up.clear();
        strmaxcpy(profile.name, username, TEXTLEN-1);
        strmaxcpy(profile.kontext, status.kontext, TEXTLEN-1);
        profile.eintragsdatum=time(NULL);
        profile.profile_ok=TRUE;
        if(profile.add())
         {
          errorbox(&online, "Eintragung fehlgeschlagen!");
         }
        else
         {
          userlogprintf2(status, username, "userverwt::eintragen",
            "User %s durch %s eingetragen", username, status.username);
          reread=TRUE;
          doedit(username);
         }
        #endif
       }
     }
    break;

   case MENU_HELP:
    goto schleife;
   }

  online.setattr(1, FALSE);
  if(!status.dologout && !reread) draw(TRUE);
 }

static void userverwaltung()
 {
  userverwt *uv=new userverwt;
  if(uv!=NULL)
   {
    uv->viewfolder();
    delete uv;
   }
 }

class editaliast:public userverwt
 {
 public:
  pt *profile;
  bool verwaltung;

  virtual void readindex();
  virtual void draw(bool complete);
  virtual void displayline(dword line);
  virtual void read();
  virtual void help();
  virtual void eintragen();
  virtual void umbenennen();
  virtual void del();
 };

void editaliast::readindex()
 {
  char filename[DIRLEN], tempstr[TEXTLEN*2], *tptr;
  FILE *infile;

  online.clrscr();
  online.setcursor(1, 1);
  online.printf("Bitte warten...");

  anz=0;

  snprintf(filename, DIRLEN, "%salias", profile->getpath());
  if((infile=fopen(filename, "rt"))==NULL) return;

  while((fgets(tempstr, TEXTLEN, infile))!=NULL && anz<MAXENTRIES)
   {
    tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;
    tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;

    if(tempstr[0]!=0 && !check(tempstr))
     {
      index[anz].name=strdup(tempstr);
      if(index[anz].name==NULL)
       {
        logf.printf("editaliast::readindex", "FATAL: Zu wenig RAM!");
        return;
       }
      index[anz].description=NULL;
      anz++;
     }
   }

  qsort(index, anz, sizeof(indexentryt), sort_string);

  fclose(infile);
 }

void editaliast::draw(bool complete)
 {
  snprintf(title, TEXTLEN, "Aliasliste von User %s - drcken Sie ? fr Hilfe",
    profile->name);

  viewfoldert::draw(complete);
 }

void editaliast::displayline(dword line)
 {
  char tempstr[TEXTLEN*2], tempstr2[TEXTLEN*2];

  if(line>=anz) return;

  strmaxcpy(tempstr2, index[line].name, (TEXTLEN*2)-1);
  tempstr2[40]=0;
  snprintf(tempstr, TEXTLEN, " %-40s ", tempstr2);

  //if(index[line].description!=NULL)
  //  strcat(tempstr, index[line].description);

  viewfoldert::displayline(line, tempstr);
 }

void editaliast::read()
 {
  char *username;

  username=index[pos].name;

  //doedit(username);

  //if(!status.dologout) draw(TRUE);
 }

void editaliast::help()
 {
  displayhelp("verw/alias.html");
  if(!status.dologout) draw(TRUE);
 }

void editaliast::del()
 {
  met mes[3];
  char *username, tempstr[TEXTLEN];
  grouppt gruppe;

  username=index[pos].name;

  userpt up(username);

  if(up.read())
   {
    errorbox(&online, "profile konnte nicht gelesen werden!");
    if(!status.dologout) draw(TRUE);
    return;
   }

  online.setattr(2, FALSE);
  online.box(20, 8, 60, 16, BOXFRAMEDBL, "Alias lschen", TRUE);

  online.setcursor(23, 10);
  online.printf("Username: ");
  online.setattr(2, TRUE);
  strmaxcpy(tempstr, username, 25);
  online.printf("%s", tempstr);
  online.setattr(2, FALSE);
  online.setcursor(28, 12);
  online.printf("Alias wirklich austragen?");

  mes[0].type=met::COMMANDBUTTON;
  mes[0].x=26;
  mes[0].y=14;
  mes[0].key=Key_C_Ent;
  mes[0].sizex=6;
  mes[0].text="O^k";

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=43;
  mes[1].y=14;
  mes[1].key=0;
  mes[1].sizex=9;
  mes[1].text="^Abbruch";

  mes[2].type=met::END_OF_LIST;

 schleife:
  switch(domenu(&online, mes, 1, TRUE))
   {
   case 0:
    if(stricmp(up|"alias", profile->name)!=0)
     {
      errorbox(&online, "Alias inkonsistent!");
     }
    else
     {
      del_alias(&up);
      reread=TRUE;
     }
    break;

   case MENU_HELP:
    goto schleife;
   }

  online.setattr(1, FALSE);

  if(!status.dologout && !reread) draw(TRUE);
 }

void editaliast::eintragen()
 {
  char username[TEXTLEN]="";
  met mes[3];

  if(verwaltung && atoi(status.gruppe|"security")==0)
   {
    errorbox(&online, "Zugriff verweigert!");
    if(!status.dologout) draw(TRUE);
    return;
   }

  if(!verwaltung)
   {
    char tempstr[TEXTLEN];
    grouppt gp((*profile)|"gruppe");

    gp.read();
    
    profile->get("maxalias", tempstr);
    if(tempstr[0]==0)
      gp.get("maxalias", tempstr);

    if(anz>=(dword)atol(tempstr))
     {
      errorbox(&online, "Maximale Anzahl Aliase bereits erreicht!");
      if(!status.dologout) draw(TRUE);
      return;
     }
   }

  online.setattr(2, FALSE);
  online.box(20, 10, 60, 16, BOXFRAMEDBL, "Alias eintragen", TRUE);

  online.setcursor(26, 12);
  online.printf("Username:");

  mes[0].type=met::EDIT;
  mes[0].key=0;
  mes[0].x=36;
  mes[0].y=12;
  mes[0].sizex=16;
  mes[0].maxlen=40;
  mes[0].text=username;

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=26;
  mes[1].y=14;
  mes[1].key=Key_C_Ent;
  mes[1].sizex=6;
  mes[1].text="O^k";

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=43;
  mes[2].y=14;
  mes[2].key=0;
  mes[2].sizex=9;
  mes[2].text="^Abbruch";

  mes[3].type=met::END_OF_LIST;

 schleife:
  switch(domenu(&online, mes, 0, TRUE))
   {
   case 1:
    dnstring(username);

    if(checkusername(username))
     {
      errorbox(&online, "Username ist ungltig!");
     }
    else
     {
      userpt tmp_up(username);

      if(!tmp_up.read())
       {
        errorbox(&online, "User existiert bereits!");
       }
      else
       {
        if(add_alias(profile, username))
          errorbox(&online, "Eintragung fehlgeschlagen!");
        else
         {
          userlogprintf2(status, username, "editaliast::eintragen",
            "Alias \"%s\" von User \"%s\" durch \"%s\" eingetragen", 
            username, profile->name, status.username);

          reread=TRUE;
         }
       }
     }
    break;

   case MENU_HELP:
    goto schleife;
   }

  online.setattr(1, FALSE);
  if(!status.dologout && !reread) draw(TRUE);
 }

void editaliast::umbenennen()
 {
  char newusername[TEXTLEN]="";
  met mes[3];
  char tempstr[DIRLEN];
  //grouppt gruppe;
  const char *oldusername=index[pos].name;

  userpt up(oldusername);
  if(up.read())
   {
    errorbox(&online, "profile konnte nicht gelesen werden!");
    if(!status.dologout) draw(TRUE);
    return;
   }

  online.setattr(2, FALSE);
  online.box(20, 10, 60, 18, BOXFRAMEDBL, "Alias umbenennen", TRUE);

  online.setcursor(22, 12);
  online.printf("alter Username:");
  online.setcursor(22, 14);
  online.printf("neuer Username:");

  online.setattr(2, TRUE);
  online.setcursor(38, 12);
  online.printf("%s", oldusername);

  mes[0].type=met::EDIT;
  mes[0].key=0;
  mes[0].x=38;
  mes[0].y=14;
  mes[0].sizex=16;
  mes[0].maxlen=40;
  mes[0].text=newusername;

  mes[1].type=met::COMMANDBUTTON;
  mes[1].x=26;
  mes[1].y=16;
  mes[1].key=Key_C_Ent;
  mes[1].sizex=6;
  mes[1].text="O^k";

  mes[2].type=met::COMMANDBUTTON;
  mes[2].x=43;
  mes[2].y=16;
  mes[2].key=0;
  mes[2].sizex=9;
  mes[2].text="^Abbruch";

  mes[3].type=met::END_OF_LIST;

 schleife:
  switch(domenu(&online, mes, 0, TRUE))
   {
   case 1:
    dnstring(newusername);

    if(checkusername(newusername))
     {
      errorbox(&online, "Username ist ungltig!");
     }
    else
     {
      snprintf(tempstr, DIRLEN, BOXUSERDIR "profile", config.boxdir, newusername);
      if(access(tempstr, 00)==0)
       {
        errorbox(&online, "User existiert bereits!");
       }
      else
       {

        #if 0
        profile.clear();
        strmaxcpy(profile.name, username, TEXTLEN-1);
        profile.eintragsdatum=time(NULL);
        profile.profile_ok=TRUE;
        if(profile.add())
         {
          errorbox(&online, "Eintragung fehlgeschlagen!");
         }
        else
         {
          userlogprintf2(status, username, "userverwt::eintragen",
            "User %s durch %s eingetragen", username, status.username);
          reread=TRUE;
          doedit(username);
         }
        #endif

       }
     }
    break;

   case MENU_HELP:
    goto schleife;
   }

  online.setattr(1, FALSE);
  if(!status.dologout && !reread) draw(TRUE);
 }

void doeditalias(pt *profile, bool verwaltung)
 {
  editaliast alias;

  alias.profile=profile;
  alias.verwaltung=verwaltung;

  alias.viewfolder();
 }

class cronverwt:public viewfoldert
 {
 public:
  struct commandt
   {
    enum { USER, NETCALL, NETCALL_RM, CRUNCH, UNKNOWN } typ;
    char option1[TEXTLEN], option2[TEXTLEN], option3[TEXTLEN],
         option4[TEXTLEN];
    char text[TEXTLEN];
   };

  struct crontabentryt
   {
    char minute[TEXTLEN], hour[TEXTLEN], dayofmonth[TEXTLEN], 
         month[TEXTLEN], dayofweek[TEXTLEN];
    char command[DIRLEN];
    commandt parsed_command;
   } *entries[MAXENTRIES];

  void parsecommand(commandt *command, const char *c);

  virtual void readindex();
  virtual void freeindex();
  virtual void displayline(dword line);
  // virtual bool check(const char *name);
  // virtual void draw(bool complete);
  // virtual void read();
  virtual void help();
  // virtual void eintragen();
  // virtual void umbenennen();
  // virtual void del();

  void cronverw()
   {
    // snprintf(directory, DIRLEN, BOXUSERDIR, config.boxdir, ".");
    strmaxcpy(title, "DBOX Cron-Tabelle", TEXTLEN-1);
    viewfolder();
   }

 // private:
  // void doedit(const char *username);
 };

void cronverwt::parsecommand(commandt *command, const char *c)
 {
  int i;
  char tempstr[1000], *tptr, *tptr2, tempstr2[1000];

  strmaxcpy(tempstr2, c, 900);

  command->text[0]=0;

  snprintf(tempstr, 1000, "echo > " BOXTODODIR, config.boxdir);
  i=strlen(tempstr);

  if(strnicmp(tempstr2, tempstr, i)==0)
   {
    tptr=tempstr2+i;
    while(*tptr!=0 && (isalnum(*tptr) || *tptr=='-' ||
          *tptr=='_' || *tptr=='.')) tptr++;

    if(*tptr==0)
     {
      if(strnicmp(tempstr2+i, "cron.crunch.", 12)==0)
       {
        command->typ=commandt::CRUNCH;
        strmaxcpy(command->option1, tempstr2+i+12, TEXTLEN-1);
        if(stricmp(command->option1, "mail")==0)
          strmaxcpy(command->text, "Postfcher crunchen", TEXTLEN-1);
        else if(stricmp(command->option1, "news")==0)
          strmaxcpy(command->text, "Bretter crunchen", TEXTLEN-1);
        else
          strmaxcpy(command->text, command->option1, TEXTLEN-1);
       }
      else
       {
        command->typ=commandt::USER;
        strmaxcpy(command->option1, tempstr2+i, TEXTLEN/2);
        snprintf(command->text, TEXTLEN, "Flag \"%s\"", command->option1);
       }

     }
   }
  else if(strnicmp(tempstr2, "echo ", 5)==0)
   {
    tptr=tempstr2+5;

    while(*tptr==' ' || *tptr=='\t') tptr++;
    
    tptr2=tptr;
    while(*tptr2!=' ' && *tptr2!='\t' && *tptr2!=0) tptr2++;
    *(tptr2++)=0;
    strmaxcpy(command->option1, tptr, TEXTLEN-1);
    tptr=tptr2;
    while(*tptr==' ' || *tptr=='\t') tptr++;

    tptr2=tptr;
    while(*tptr2!=' ' && *tptr2!='\t' && *tptr2!=0) tptr2++;
    *(tptr2++)=0;
    strmaxcpy(command->option2, tptr, TEXTLEN-1);
    tptr=tptr2;
    while(*tptr==' ' || *tptr=='\t') tptr++;
      
    if(tptr[0]=='>')
      command->option3[0]=0;
    else
     {
      tptr2=tptr;
      while(*tptr2!=' ' && *tptr2!='\t' && *tptr2!=0) tptr2++;
      *(tptr2++)=0;
      strmaxcpy(command->option3, tptr, TEXTLEN-1);
      tptr=tptr2;
      while(*tptr==' ' || *tptr=='\t') tptr++;
     }
      
    snprintf(tempstr, 1000, "> " BOXTODODIR "netcall.", config.boxdir);
    i=strlen(tempstr);

    if(strnicmp(tptr, tempstr, i)==0)
     {
      tptr+=i;
      strmaxcpy(command->option4, tptr, TEXTLEN-1);
  
      while(*tptr!=0 && isdigit(*tptr)) tptr++;

      if(*tptr==0)
       {
        command->typ=commandt::NETCALL;
        snprintf(tempstr, 1000, "Netcall %s Port %s", 
          command->option1, command->option4);
 
        if(command->option3[0]!=0)
          snprintf(tempstr+strlen(tempstr), 1000-strlen(tempstr),
            " Nr. %s",
            command->option3);

        strmaxcpy(command->text, tempstr, TEXTLEN-1);
       }
     }
   }
  else if(strnicmp(tempstr2, "rm ", 3)==0)
   {
    snprintf(tempstr, DIRLEN, BOXTODODIR "netcall.", config.boxdir);
    i=strlen(tempstr);

    tptr=tempstr2+3;
    if(strnicmp(tptr, tempstr, i)==0)
     {
      tptr+=i;
      strmaxcpy(command->option3, tptr, TEXTLEN-1);
  
      while(*tptr!=0 && isdigit(*tptr)) tptr++;

      if(*tptr==0)
       {
        command->typ=commandt::NETCALL;
        snprintf(tempstr, 1000, "Netcall Port %s beenden", 
          command->option3);
        strmaxcpy(command->text, tempstr, TEXTLEN-1);
       }
     }
   }

  if(command->text[0]==0)
   {
    command->typ=commandt::UNKNOWN;
    strmaxcpy(command->text, c, TEXTLEN-1);
   }
 }

void cronverwt::readindex()
 {
  FILE *infile;
  char tempstr[1000], *tptr, *tptr2;
  crontabentryt e;

  anz=0;

  infile=popen("crontab -l", "r");
  if(infile==NULL)
   {
    errorbox(&online, "cron konnte nicht gefunden werden!");
    return;
   }

  while(fgets(tempstr, 900, infile)!=NULL)
   {
    tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;
    tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;

    tptr=tempstr;
    while(*tptr==' ' || *tptr=='\t') tptr++;
    if(tptr[0]!='#' && tptr[0]!=';')
     {
      tptr2=tptr;
      while(*tptr2!=' ' && *tptr2!='\t' && *tptr2!=0) tptr2++;
      *(tptr2++)=0;
      strmaxcpy(e.minute, tptr, TEXTLEN-1);
      tptr=tptr2;
      while(*tptr==' ' || *tptr=='\t') tptr++;

      tptr2=tptr;
      while(*tptr2!=' ' && *tptr2!='\t' && *tptr2!=0) tptr2++;
      *(tptr2++)=0;
      strmaxcpy(e.hour, tptr, TEXTLEN-1);
      tptr=tptr2;
      while(*tptr==' ' || *tptr=='\t') tptr++;
      
      tptr2=tptr;
      while(*tptr2!=' ' && *tptr2!='\t' && *tptr2!=0) tptr2++;
      *(tptr2++)=0;
      strmaxcpy(e.dayofmonth, tptr, TEXTLEN-1);
      tptr=tptr2;
      while(*tptr==' ' || *tptr=='\t') tptr++;
      
      tptr2=tptr;
      while(*tptr2!=' ' && *tptr2!='\t' && *tptr2!=0) tptr2++;
      *(tptr2++)=0;
      strmaxcpy(e.month, tptr, TEXTLEN-1);
      tptr=tptr2;
      while(*tptr==' ' || *tptr=='\t') tptr++;
      
      tptr2=tptr;
      while(*tptr2!=' ' && *tptr2!='\t' && *tptr2!=0) tptr2++;
      *(tptr2++)=0;
      strmaxcpy(e.dayofweek, tptr, TEXTLEN-1);
      tptr=tptr2;
      while(*tptr==' ' || *tptr=='\t') tptr++;
      
      strmaxcpy(e.command, tptr, DIRLEN-1);

      if(e.minute[0]!=0 && e.hour[0]!=0 && e.dayofmonth[0]!=0 && 
         e.month[0]!=0 && e.dayofweek[0]!=0 && e.command[0]!=0)
       {
        parsecommand(&e.parsed_command, e.command);
        entries[anz]=new crontabentryt;
        *(entries[anz])=e;
        anz++;
       }
     }
   }

  pclose(infile);
 }

void cronverwt::freeindex()
 {
  for(dword i=0; i<anz; i++)
    delete entries[i];
 }

char *wochentage[]=
 {
  "Sonntag", "Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag",
  "Samstag", "Sonntag"
 };

void cronverwt::displayline(dword line)
 {
  char tempstr[TEXTLEN*2], tempstr2[TEXTLEN*2];
  int i;

  if(line>=anz) return;

  snprintf(tempstr2, TEXTLEN*2, "%2s:%-2s", entries[line]->hour, entries[line]->minute);
  tempstr2[10]=0;
  snprintf(tempstr, TEXTLEN*2, "%-10s ", tempstr2);

  snprintf(tempstr2, TEXTLEN*2, "%s.%s", entries[line]->dayofmonth, entries[line]->month);
  tempstr2[10]=0;
  snprintf(tempstr+strlen(tempstr), TEXTLEN*2-strlen(tempstr), "%-10s ", tempstr2);

  i=atoi(entries[line]->dayofweek);

  if(isdigit(entries[line]->dayofweek[0]) && 
     entries[line]->dayofweek[1]==0 &&
     i>=0 && i<=7)
    strmaxcpy(tempstr2, wochentage[i], (TEXTLEN*2)-1);
  else
   {
    snprintf(tempstr2, TEXTLEN*2, "%s", entries[line]->dayofweek);
    tempstr2[10]=0;
   }

  snprintf(tempstr+strlen(tempstr), TEXTLEN*2-strlen(tempstr),
    "%-10s ", tempstr2);

  strmaxcpy(tempstr+strlen(tempstr), entries[line]->parsed_command.text, 46);

  viewfoldert::displayline(line, tempstr);
 }

void cronverwt::help()
 {
  displayhelp("verw/cron.html"); 
  if(!status.dologout) draw(TRUE);
 }

void docron_verw()
 {
  cronverwt *cv=new cronverwt;
  if(cv!=NULL)
   {
    cv->cronverw();
    delete cv;
   }
 }

void doverwaltungsmenue()
 {
  bool ende=FALSE;
  int newi=0;

  userlogprintf1(status, "doverwaltungsmenue", "Start des Verwaltungsmens");

  do
   {
    int i;
    byte y=11;
    met mes[10];
    byte ids[10];

     {
      char title[TEXTLEN];
      snprintf(title, TEXTLEN, "Verwaltungs-Hauptmen fr %s", config.boxname);
      drawscreen(&online, title, "Version " DBOXVERSIONNR " vom " DBOXVERSION);
     }

    drawstatusbar();

    /*
    online.setcursor(3, 2);
    online.printf("      Benutzer:     %lu", countuser());
    online.setcursor(3, 3);
    online.printf("       Gruppen:     %lu", countgruppen());
    */

    for(i=0; i<10; i++)
     {
      mes[i].type=met::COMMANDBUTTON;
      mes[i].x=(online.sizex-30)/2;
      mes[i].y=y+i*2;
      mes[i].sizex=30;
      ids[i]=0xff;
     }

    i=0;

    if(TRUE)
     { ids[0]=i; mes[i++].text="^Userverwaltung"; }

    if(getinternalflag(&status.userprofile, &status.gruppe, VERWMENU_BRETTER))
     { ids[1]=i; mes[i++].text="^Bretterverwaltung"; }

    //if(status.gruppe.internalflags[VERWMENU_ALLGEMEIN])
    // { ids[2]=i; mes[i++].text="^Allgemeine Einstellungen"; }

    if(getinternalflag(&status.userprofile, &status.gruppe, VERWMENU_CRON))
     { ids[3]=i; mes[i++].text="^Cron Tabelle"; }

    if(TRUE)
     { ids[4]=i; mes[i++].text="Box^status Anzeigen"; }

    ids[5]=i; mes[i++].text="Verwaltungsmen ^verlassen";

    mes[i].type=met::END_OF_LIST;

   schleife:
    printtimeout();
    i=domenu(&online, mes, newi, TRUE);

    if(i==ids[0])
     {
      userverwaltung();
      newi=i;
     }
    else if(i==ids[1])
     {
      dobretter_verw();
      newi=i;
     }
    else if(i==ids[2])
     {
      einstellungen();
      newi=i;
     }
    else if(i==ids[3])
     {
      docron_verw();
      newi=i;
     }
    else if(i==ids[4])
     {
      showstatus();
      newi=i;
      if(!status.dologout) goto schleife;
     }
    else if(i==MENU_HELP)
     {
      displayhelp("verw");
     }
    else
      ende=TRUE;

    if(status.dologout) ende=TRUE;
   }
  while(!ende);

  userlogprintf1(status, "doverwaltungsmenue", "Ende des Verwaltungsmens");
 }
