/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <dirent.h>
#include <fcntl.h>
#include <time.h>
#include <sys/stat.h>

#include <dboxpath.h>
#include <oldtools.h>
#include "online.h"
#include "vfolder.h"

void viewfoldert::setpos(dword newpos, bool updatescreen)
 {
  dword oldpos, oldscroll;

  if(pos==newpos) return;

  oldpos=pos;
  oldscroll=scroll;
  pos=newpos;

  if(pos<scroll)
    scroll=pos;
  else if(pos-scroll>online.sizey-3)
    scroll=pos-(online.sizey-3);

  if(updatescreen)
   {
    if(scroll==oldscroll)
     {
      displayline(oldpos);
      displayline(pos);
     }
    else // TODO: Optimierbar bei kleinen scroll-oldscroll
     {
      draw(FALSE);
     }
   }
 }

void viewfoldert::displayline(dword line)
 {
  displayline(line, "");
 }

void viewfoldert::displayline(dword line, char *text)
 {
  int i;

  online.setcursor(1, line-scroll+2+headersize);

  if(online.usecolor)
    online.setattr(line==pos?11:1, FALSE);
  else
    online.setattr(line==pos?10:1, FALSE);

  i=online.sizex-online.printf("%s", text);
  for(; i>0; i--) online.putch(' ');
  //online.cleareol();

  online.setattr(1, FALSE);
 }

void viewfoldert::displayline(dword line, bool marked, time_t t,
                             dword size, char *t1, char *t2)
 {
  struct tm *tm;
  char text[1000], tempstr[TEXTLEN], text1[1000], text2[1000];
  
  if(size==0xffffffff)
    strcpy(tempstr, "DIR ");
  else
   {
    if(size<10000)
     {
      snprintf(tempstr, TEXTLEN, "%4lu", size);
     }
    else
     {
      dword l=size/1024;
      if(l>999)
       {
        l/=1024;
        if(l==0) l=1;
        snprintf(tempstr, TEXTLEN, "%3luM", l);
       }
      else
       {
        if(l==0) l=1;
        snprintf(tempstr, TEXTLEN, "%3luK", l);
       }
     }
   }

  strmaxcpy(text1, t1, 22);

  if(t2!=NULL)
    strmaxcpy(text2, t2, online.sizex-40);
  else
    strcpy(text2, "  -- keine Beschreibung vorhanden --");

  if(t==0)
   {
    snprintf(text, 1000, "%c --.--.-- %s %-22s %-40s ",
      marked?'>':' ', tempstr, text1, text2);
   }
  else
   {
    tm=localtime(&t);

    snprintf(text, 1000, "%c %02u.%02u.%02u %s %-22s %-40s ",
      marked?'>':' ', tm->tm_mday, tm->tm_mon+1, 
        tm->tm_year%100, tempstr, text1, text2);
   }

  displayline(line, text);
 }

void viewfoldert::draw(bool complete)
 {
  byte y;

  if(status.dologout) return;

  if(complete)
   {
    online.clrscr();
    drawtitle(&online, title);
   }

  for(y=2+headersize; y<online.sizey; y++)
    displayline(scroll+y-2-headersize);

  drawstatusbar();

  online.setcursor(1, 2+pos-scroll+headersize);
 }

void viewfoldert::key(word k)
 {
  switch(k)
   {
   case 0xffff:
    ende=TRUE;
    break;

   case Key_Ent:
    if(anz!=0) read();
    break;

   case Key_Esc:
    ende=TRUE;
    break;

   case Key_Home:
    if(anz!=0 && pos!=0)
     {
      if(scroll==0)
       {
        dword oldpos=pos;
        pos=0;
        displayline(oldpos);
        displayline(pos);
       }
      else
       {
        scroll=0;
        pos=0;
        draw(FALSE);
       }
      online.setcursor(1, 2+pos-scroll+headersize);
     }
    break;

   case Key_End:
    if(anz!=0 && pos!=anz-1)
     {
      dword oldpos=pos;
      pos=anz-1;
      if(pos-scroll>online.sizey-3)
       {
        scroll+=(pos-scroll)-(online.sizey-3);
        draw(FALSE);
       }
      else
       {
        displayline(oldpos);
        displayline(pos);
       }
      online.setcursor(1, 2+pos-scroll+headersize);
     }
    break;

   case Key_Up:
    if(anz!=0 && pos!=0)
     {
      pos--;
      if(pos<scroll)
       {
        displayline(pos+1);
        scroll--;
        online.setcursor(1, online.sizey-1);
        online.deleteline();
        online.setcursor(1, 2+headersize);
        online.insertline();
        drawstatusbar();
        displayline(pos);
       }
      else
       {
        displayline(pos+1);
        displayline(pos);
       }
      online.setcursor(1, 2+pos-scroll+headersize);
     }
    break;

   case Key_F1:
    help();
    break;

   case Key_Dwn:
    if(anz!=0 && pos!=anz-1)
     {
      pos++;
      if(pos-scroll>online.sizey-3)
       {
        if(status.online2)
         {
          online.console.setcursor(1, online.console.sizey);
          online.console.deleteline();
         }

        displayline(pos-1);
        scroll++;
        online.setcursor(1, 2+headersize),
        online.deleteline();
        drawstatusbar();
        displayline(pos);
       }
      else
       {
        displayline(pos-1);
        displayline(pos);
       }
      online.setcursor(1, 2+pos-scroll+headersize);
     }
    break;

   case Key_PgUp:
    if(pos!=0)
     {
      if(scroll==0)
       {
        dword oldpos=pos;
        pos=0;
        displayline(oldpos);        
        displayline(pos);
       }
      else
       {
        pos-=online.sizey-3;
        if(scroll>(online.sizey-3))
          scroll-=online.sizey-3;
        else
          scroll=0;

        draw(FALSE); 
       }

      online.setcursor(1, 2+pos-scroll+headersize);
     }
    break;

   case Key_PgDn:
    if(anz!=0 && pos!=anz-1)
     {
      dword oldscroll=scroll, oldpos=pos;

      pos=min(pos+online.sizey-3, anz-1);
      if((anz-1)>=(online.sizey-3))
        scroll=min(scroll+online.sizey-3, (anz-1)-(online.sizey-3));

      if(scroll!=oldscroll)
        draw(FALSE);
      else
       {
        displayline(oldpos);
        displayline(pos);
       }

      online.setcursor(1, 2+pos-scroll+headersize);
     }
    break;

   case Key_Spa:
    mark();
    break;

   case Key_C_E:
    unmarkall();
    break;

   default:
    switch(upcase(lo(k)))
     {
     case '?':
     case 'H':
      help();
      break;

     case 'I':
      if(anz!=0) info();
      break;

     case 'E':
      if(anz!=0) edit();
      break;

     case 'V':
      if(anz!=0) view();
      break;

     case 'X':
      if(anz!=0) extract();
      break;

     case 'L':
      if(anz!=0) del();
      break;
     }
    break;
   }
 }

void viewfoldert::viewfolder()
 {
  word k;
  ende=FALSE;
  pos=scroll=0;

  do
   {
    readindex();

    if(pos>=anz)
     {
      if(anz==0)
       {
        scroll=pos=0;
       }
      else
       {
        pos=anz-1;
        if(scroll>pos) scroll=pos;
       }
     }

    draw(TRUE);
    reread=FALSE;

    do
     {
      k=online.doreadkey();
      if(k!=0) key(k);
     }
    while(!ende && !reread);

    freeindex();
   }
  while(!ende && !status.dologout);
 }
