/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <time.h>
#include <dirent.h>

#include <netdefs.h>
#include <logf.h>
#include <userdb.h>
#include <dboxpath.h>
#include <oldtools.h>
#include <portinfo.h>
#include "../online/extern.h"
#include "../online/online.h"
#include "puffer.h"
#include "netz.h"

class docallt
 {
 public:
  const char *systemname;
  char nummer[TEXTLEN];
  userpt up;
  char systemlogfile[DIRLEN];
  string input_log;

  bool checkcarrier;

  int waitforstring(char **strings, time_t timeout);
  int waitforstring(char *string, time_t timeout);
  void write_log();

  bool dialer();
  void makenetcall();

  bool docall(const char *sys, const char *nr);
 };

int docallt::waitforstring(char **strings, time_t timeout)
 {
  byte counter[20], anzstrings, i;
  char ch;
  time_t starttime=time(NULL);

  anzstrings=0;
  while(strings[anzstrings]!=NULL) anzstrings++;
  for(i=0; i<anzstrings; i++) counter[i]=0;

  while(TRUE)
   {
    do
     {
      status.online2=TRUE;
      online.waitkey();
      status.online2=FALSE;

      ch=online.console.readkey();
      if(ch!=0)
       {
        if(ch==Key_Esc) return -1;
        if(ch=='\n') ch='\r';
        online.comport.putch(ch);
       }

      if(time(NULL)>(starttime+timeout)) return -2;

      if(!online.comport.readchar(&ch)) ch=0;
     }
    while(ch==0);

    online.console.putch(ch);
    input_log+=ch;

    for(i=0; i<anzstrings; i++)
      if(upcase(strings[i][counter[i]])!=upcase(ch))
        counter[i]=0;
      else
       {
        counter[i]++;
        if(strings[i][counter[i]]==0)
          return i;
       }
   }
 }

int docallt::waitforstring(char *string, time_t timeout)
 {
  char *strings[2];
  strings[0]=string;
  strings[1]=NULL;
  return waitforstring(strings, timeout);
 }

bool docallt::dialer()
 {
  char *strings1[]=
   {
    "CONNECT",
    "NO DIALTONE",
    "NO CARRIER",
    "ERROR",
    "DELAYED",
    "VOICE",
    "BUSY",
    NULL
   };

  online.comport.printf("\rAT\r");
  asyncio.flush_output();

  online.console.clrscr();
  online.console.setattr(0xff, TRUE);
  online.console.setcursor(1, 1);
  online.console.printf("Anwahl von %s...\r\n\r\n", systemname);
  online.console.setattr(0xff, FALSE);

  logf.fileprintf(systemlogfile, "dialer", "Port %u: Anwahl", config.port);

  online.console.printf("Whlbefehl: %s%s\r\n\r\n",
    status.deviceconfig|"waehlen", nummer);

  asyncio.flush_output();
  sleep(1);
  online.comport.flushkeys();

  online.comport.printf("%s%s\r",
    status.deviceconfig|"waehlen", nummer);

  checkcarrier=FALSE;
  input_log="";

  switch(waitforstring(strings1, 120))
   {
   case 0:
    waitforstring("\r", 2);
    write_log();
    logf.fileprintf(systemlogfile, "dialer", "CONNECT");
    break;

   case -1:     // ESC
    write_log();
    online.console.setattr(0xff, TRUE);
    online.console.printf("\r\nAbbruch\r\n");
    online.console.setattr(0xff, FALSE);
    logf.fileprintf(systemlogfile, "dialer", "manueller Abbruch");
    sleep(5);
    return TRUE;

   case -2:     // Timeout
    write_log();
    online.console.setattr(0xff, TRUE);
    online.console.printf("\r\nTimeout\r\n");
    online.console.setattr(0xff, FALSE);
    logf.fileprintf(systemlogfile, "dialer", "Timeout");
    sleep(5);
    return TRUE;

   default:
    waitforstring("\r", 4);
    write_log();
    online.console.printf("\r\n");
    logf.fileprintf(systemlogfile, "dialer", "Fehler");
    sleep(5);
    return TRUE;
   }

  return FALSE;
 }

void docallt::makenetcall()
 {
  char tempstr[TEXTLEN], outfilename[DIRLEN], newoutfilename[DIRLEN],
       infilename[DIRLEN], netcallverfahren[TEXTLEN], *tptr,
       tmpfilename[DIRLEN];
  int i;

  switch(getnettype(up))
   {
   case NETZ_GSBOX:
   case NETZ_JANUS:
   case NETZ_JANUS2:
   case NETZ_Z38:
    snprintf(netcallverfahren, TEXTLEN, 
      "Netcallverfahren %s", config.nettypedb[getnettype(up)]);
 
    switch(getnettype(up))
     {
     case NETZ_GSBOX:
      tptr="namen ein:";
      break;

     default:
      tptr="Username:";
     }
 
    if(waitforstring(tptr, 30)!=0)
     {
      write_log();
      logf.fileprintf(systemlogfile, "makenetcall",
        "\"%s\" nicht erhalten", tptr);

      online.console.setattr(0xff, TRUE);
      online.console.printf("\r\nFehler!\r\n");
      online.console.setattr(0xff, FALSE);
      return;
     }

    sleep(1);

    switch(getnettype(up))
     {
     case NETZ_GSBOX:
      online.comport.printf("NETCALL\r");
      break;

     case NETZ_JANUS:
      online.comport.printf("JANUS\r");
      break;

     case NETZ_JANUS2:
      online.comport.printf("JANUS2\r");
      break;

     case NETZ_Z38:
      online.comport.printf("ZERBERUS\r");
      break;
     }

    if(waitforstring("Systemname:", 60)!=0)
     {
      write_log();
      logf.fileprintf(systemlogfile, "makenetcall",
        "Systemname: nicht erhalten");

      online.console.setattr(0xff, TRUE);
      online.console.printf("\r\nFehler!\r\n");
      online.console.setattr(0xff, FALSE);
      return;
     }

    sleep(1);
    strmaxcpy(tempstr, config.boxname, TEXTLEN-1);
    upstring(tempstr);
    online.comport.printf("%s\r", tempstr);

    if(waitforstring("Passwort:", 30)!=0)
     {
      write_log();
      logf.fileprintf(systemlogfile, "makenetcall",
        "Passwort: nicht erhalten");

      online.console.setattr(0xff, TRUE);
      online.console.printf("\r\nFehler!\r\n");
      online.console.setattr(0xff, FALSE);
      return;
     }

    sleep(1);

     {
      char password[TEXTLEN];
      getplaintextpassword(&up, password);
      online.comport.printf("%s\r", password);
     }

    write_log();

    if(getnettype(up)==NETZ_GSBOX)
     {
      if(waitforstring("**", 60)!=0)
       {
        write_log();
        logf.fileprintf(systemlogfile, "makenetcall",
          "ZModem Kennung nicht erhalten");

        online.console.setattr(0xff, TRUE);
        online.console.printf("\r\nFehler!\r\n");
        online.console.setattr(0xff, FALSE);
        return;
       }

      // ???
     }
    else
     {
      if(waitforstring("Running ARC", 30)!=0)
       {
        write_log();
        logf.fileprintf(systemlogfile, "makenetcall",
          "Running ARC nicht erhalten");

        online.console.setattr(0xff, TRUE);
        online.console.printf("\r\nFehler!\r\n");
        online.console.setattr(0xff, FALSE);
        return;
       }
 
      write_log();

      tempstr[0]=21;
      tempstr[1]=0;

      if(waitforstring(tempstr, 10*60)!=0)
       {
        write_log();
        logf.fileprintf(systemlogfile, "makenetcall",
          "NAK nicht erhalten");

        online.console.setattr(0xff, TRUE);
        online.console.printf("\r\nFehler!\r\n");
        online.console.setattr(0xff, FALSE);
        return;
       }

      for(i=0; i<5; i++)
        online.comport.putch(0);

      tempstr[0]=6;
      tempstr[1]=0;

      if(waitforstring(tempstr, 10*60)!=0)
       {
        write_log();
        logf.fileprintf(systemlogfile, "makenetcall",
          "ACK nicht erhalten");

        online.console.setattr(0xff, TRUE);
        online.console.printf("\r\nFehler!\r\n");
        online.console.setattr(0xff, FALSE);
        return;
       }

      if(getnetcallfile(&up, outfilename))
       {
        logf.printf("makenetcall", "%s: failed to get netcallfile", systemname);
        logf.fileprintf(systemlogfile, "makenetcall", "failed to get netcallfile");
        online.console.printf("\r\nFatal: Netcallfile gelockt!\r\n");
        return;
       }

      if(getnettype(up)==NETZ_JANUS2)
       {
        #define MAXFILESLEN 1000
        #define MAXFILES 50

        DIR *dir;
        struct dirent *ent;
        bool error=FALSE;
        char files[MAXFILESLEN]="-u";
        int i=0;

        snprintf(tmpfilename, DIRLEN, BOXSPOOLDIR "%s.%s", 
          config.boxdir, systemname, systemname,
          packerextention(up|"packer"));

        if(access(tmpfilename, 00)==0)
         {
          char newfilename[DIRLEN];
          dword t=time(NULL);

          do
           {
            snprintf(newfilename, DIRLEN, BOXSPOOLDIR "NP%06lu.%s", 
              config.boxdir, systemname, t%1000000,
              packerextention(up|"packer"));
            t++;
           }
          while(access(newfilename, 00)==0);

          rename(tempstr, newfilename);
         }

        snprintf(tmpfilename, DIRLEN, BOXSPOOLDIR ".", 
          config.boxdir, systemname);

        dir=opendir(tmpfilename);
        if(dir!=NULL)
         {
          while((ent=readdir(dir))!=NULL && i<MAXFILES &&
                strlen(files)<MAXFILESLEN)
           {
            if(isjanus2packet(ent->d_name, &up))
             {
              snprintf(files+strlen(files), MAXFILESLEN-strlen(files),
                " " BOXSPOOLDIR "%s", 
                config.boxdir, systemname, ent->d_name);
              i++;
             }
           }
          closedir(dir);
         }

        if(i==0)
         {
          snprintf(outfilename, DIRLEN, BOXSPOOLDIR "NP000000.%s", 
            config.boxdir, systemname,
            packerextention(up|"packer"));

          if(createnetcallfile(outfilename, up|"packer"))
           {
            logf.fileprintf(systemlogfile, "makenetcall",
              "failed to create netcallfile");
            freenetcallfile(&up);
            return;
           }

          snprintf(tempstr, TEXTLEN, "Ausgangspuffer von Netzwerksystem %s", systemname);
          status.online2=TRUE;
          if(doupdownload(TRUE, 'Z', outfilename, tempstr, netcallverfahren))
           {
            logf.fileprintf(systemlogfile, "makenetcall",
              "Download fehlgeschlagen");
            error=TRUE;
           }
          status.online2=FALSE;
         }        
        else
         {
          snprintf(tempstr, TEXTLEN, "Ausgangspuffer von Netzwerksystem %s", systemname);
          status.online2=TRUE;
          if(doupdownload(TRUE, 'Z', files, tempstr, netcallverfahren))
           {
            logf.fileprintf(systemlogfile, "makenetcall", 
              "Download fehlgeschlagen");
            error=TRUE;
           }
          status.online2=FALSE;
         }

        if(!online.comport.dcd())
         {
          logf.printf("makenetcall", "%s: Carrier lost", systemname);
          logf.fileprintf(systemlogfile, "makenetcall", "Carrier lost");
         }
        else if(!error)
         {
          snprintf(infilename, DIRLEN, DBOXPORTTEMPDIR ".", config.dboxdir, config.port);
          snprintf(tempstr, TEXTLEN, "Eingangspuffer von Netzwerksystem %s", systemname);

          status.online2=TRUE;
          if(doupdownload(FALSE, 'Z', infilename, tempstr, netcallverfahren))
           {
            logf.fileprintf(systemlogfile, "makenetcall", 
              "Upload fehlgeschlagen");
            error=TRUE;
           }

          status.online2=FALSE;

          dir=opendir(infilename);
          if(dir!=NULL)
           {
            while((ent=readdir(dir))!=NULL)
             {
              if(ent->d_name[0]!='.')
               {
                snprintf(infilename, DIRLEN, DBOXPORTTEMPDIR "%s", 
                  config.dboxdir, config.port, ent->d_name);
                storearchive(&up, infilename);
               }
             }
            closedir(dir);
           } 
         }
       }
      else
       {
        if(access(outfilename, 00)!=0)
         {
          if(createnetcallfile(outfilename, up|"packer"))
           {
            logf.printf("makenetcall", "%s: failed to create netcallfile", systemname);
            logf.fileprintf(systemlogfile, "makenetcall", "failed to create netcallfile");
            online.console.printf("\r\nFatal: Ein Netcallfile konnte nicht erstellt werden!\r\n");
            freenetcallfile(&up);
            return;
           }
         }

        snprintf(newoutfilename, DIRLEN, BOXSPOOLDIR "caller.%s", 
          config.boxdir, systemname, packerextention(up|"packer"));

        remove(newoutfilename);

        if(rename(outfilename, newoutfilename)!=0)
         {
          logf.printf("makenetcall", "Fatal: rename(\"%s\", \"%s\"); failed",
            outfilename, newoutfilename);
          logf.fileprintf(systemlogfile, "makenetcall", "Fatal: "
            "rename(\"%s\", \"%s\"); failed", outfilename, newoutfilename);
          online.console.printf("\r\nrename fehlgeschlagen!\r\n");
          freenetcallfile(&up);
          return;
         }
       
        if(!online.comport.dcd())
         {
          logf.printf("makenetcall", "%s: Carrier lost", systemname);
          logf.fileprintf(systemlogfile, "makenetcall", "Carrier lost");
          rename(newoutfilename, outfilename);
          freenetcallfile(&up);
          return;
         }

        snprintf(tempstr, TEXTLEN, "Ausgangspuffer von Netzwerksystem %s", systemname);
        status.online2=TRUE;
        if(dodownload('Z', newoutfilename, tempstr, netcallverfahren))
         {
          status.online2=FALSE;
          logf.printf("makenetcall", "%s: Download fehlgeschlagen", systemname);
          logf.fileprintf(systemlogfile, "makenetcall", "Download fehlgeschlagen");
          rename(newoutfilename, outfilename);
          freenetcallfile(&up);
          sleep(10);
          return;
         }

        status.online2=FALSE;

        online.comport.flushkeys();

        snprintf(tmpfilename, DIRLEN, DBOXPORTTEMPDIR, config.dboxdir, config.port);
        deldir(tmpfilename);

        snprintf(infilename, DIRLEN, DBOXPORTTEMPDIR "infile.%s",
          config.dboxdir, config.port, packerextention(up|"packer"));

        remove(infilename);

        snprintf(tempstr, TEXTLEN, "Eingangspuffer von Netzwerksystem %s", systemname);
        status.online2=TRUE;
        if(doupload_specialfile('Z', infilename, tempstr, netcallverfahren))
         {
          status.online2=FALSE;
          logf.printf("makenetcall", "%s: Upload fehlgeschlagen", systemname);
          logf.fileprintf(systemlogfile, "makenetcall", "Upload fehlgeschlagen");
          rename(newoutfilename, outfilename);
          freenetcallfile(&up);
          return;
         }

        status.online2=FALSE;

        storearchive(&up, infilename);

        remove(newoutfilename);
       }

      freenetcallfile(&up);

      logf.fileprintf(systemlogfile, "makenetcall",
        "Netcall erfolgreich abgeschlossen");
     }
    break;
   }
 }

void docallt::write_log()
 {
  char line[100];
  word linecount=0;

  for(dword i=0; i<input_log.size(); i++)
   {
    if(input_log[i]=='\r')
     {
     }
    else if(input_log[i]=='\n')
     {
      line[linecount]=0;
      logf.fileprintf(systemlogfile, "log", "%s", line);
      linecount=0;
     }
    else if(linecount<60)
     {
      line[linecount++]=input_log[i];
     }
   }

  if(linecount!=0)
   {
    line[linecount]=0;
    logf.fileprintf(systemlogfile, "log", "%s", line);
    linecount=0;
   }

  input_log="";
 }

bool docallt::docall(const char *sys, const char *nr)
 {
  systemname=sys;

  strmaxcpy(up.name, systemname, TEXTLEN-1);
  if(up.read())
   {
    logf.printf("docall", "System %s nicht gefunden!", systemname);
    return TRUE;
   }

  if(nr[0]!=0)
    strmaxcpy(nummer, nr, TEXTLEN-1);
  else
    strmaxcpy(nummer, up|"telefon_data", TEXTLEN-1);

  snprintf(systemlogfile, DIRLEN, BOXSPOOLDIR "netcall", 
    config.boxdir, systemname);

  portinfo.owninfo.status=portinfot::infot::NETCALL;
  strmaxcpy(portinfo.owninfo.username, systemname, TEXTLEN-1);
  portinfo.write(config.port);

  if(getnettype(up)==NETZ_UUCP)
   {
    char command[DIRLEN];

    online.console.clrscr();
    online.console.setattr(0xff, TRUE);
    online.console.setcursor(1, 1);
    online.console.printf("Anwahl von %s...\r\n\r\n", systemname);
    online.console.setattr(0xff, FALSE);

    snprintf(command, DIRLEN, "/usr/lib/uucp/uucico -s %s -r1 -q -f -D", 
      systemname);

    online.console.printf("Kommando: %s\r\n", command);

    online.comport.close();
    sleep(1);
    system(command);
   }
  else
   {
    if(dialer())
      return TRUE;
    else
      makenetcall();
   }

  return FALSE;
 }

void docall(const char *systemname, const char *nummer)
 {
  docallt dc;
  dc.docall(systemname, nummer);
 }

void doautomaticcall()
 {
  FILE *infile, *outfile;
  char filename[DIRLEN], command[TEXTLEN], nummer[TEXTLEN], *tptr;
  byte versuche;
  docallt dc;

  snprintf(filename, DIRLEN, BOXTODODIR "netcall.%u",
    config.boxdir, config.port);

  infile=fopen(filename, "rt");
  if(infile==NULL)
   {
   error:
    remove(filename);
    return;
   }

  if(fgets(command, TEXTLEN-1, infile)==NULL) 
   {
    fclose(infile);
    goto error;
   }

  fclose(infile);

  tptr=strchr(command, ' ');
  if(tptr==NULL) goto error;
  *(tptr++)=0;
  
  versuche=atoi(tptr);
  if(versuche==0) goto error;

  nummer[0]=0;

  tptr=strchr(tptr, ' ');
  if(tptr!=NULL) strmaxcpy(nummer, tptr+1, TEXTLEN-1);

  versuche--;
  if(versuche==0)
    remove(filename);
  else
   {
    outfile=fopen(filename, "wt");
    fprintf(outfile, "%s %u", command, versuche);
    if(nummer[0]!=0) fprintf(outfile, " %s", nummer);
    fprintf(outfile, "\n");
    fclose(outfile);
   }

  dnstring(command);

  if(!dc.docall(command, nummer))
    remove(filename);
 }
