/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
//#include <waitstatus.h>

#include <dboxpath.h>
#include <logf.h>
#include <portinfo.h>
#include <oldtools.h>
#include <dboxversion.h>
#include <flags.h>
#include <logins.h>
#include <radius.h>
#include "../online/online.h"
#include "../online/extern.h"
#include "../online/userlog.h"
#include "../online/openconfig.h"
#include "netz.h"

bool use_console=FALSE, ibm=FALSE, boxname_changed=FALSE;

static void startonline()
 {
  int pid, sta=-1;
  
  char arg1[TEXTLEN], arg2[TEXTLEN], *arg3=NULL, *arg4=NULL, *arg5=NULL,
       *arg6=NULL, fromarg[TEXTLEN], linespeedarg[TEXTLEN],
       boxnamearg[TEXTLEN*2], programname[DIRLEN];
 
  switch(pid=fork())
   {
   case -1:
    logf.printf("startonline", "FATAL: Failed to fork");
    break;

   case 0:
    if(status.online2)
      snprintf(arg1, TEXTLEN, "-p%u", config.port);
    else if(use_console)
      snprintf(arg1, TEXTLEN, "-c%u", config.port);
    else
      snprintf(arg1, TEXTLEN, "-o%u", config.port);

    snprintf(arg2, TEXTLEN, "-u%s", status.username);

    if(boxname_changed)
     {
      snprintf(boxnamearg, TEXTLEN, "-b%s", config.boxname);
      arg3=boxnamearg;
     }

    if(status.from[0]!=0)
     {
      snprintf(fromarg, TEXTLEN, "-f%s", status.from);
      if(arg3==NULL) arg3=fromarg; else arg4=fromarg;
     }

    if(status.linespeed!=0)
     {
      snprintf(linespeedarg, TEXTLEN, "-s%lu", status.linespeed);

      if(arg3==NULL)
        arg3=linespeedarg;
      else if(arg4==NULL)
        arg4=linespeedarg;
      else
        arg5=linespeedarg;
     }

    if(ibm && !status.online2)
     {
      if(arg3==NULL)
        arg3="-i";
      else if(arg4==NULL)
        arg4="-i";
      else if(arg5==NULL)
        arg5="-i";
      else
        arg6="-i";
     }

    snprintf(programname, DIRLEN-1, DBOXBINDIR "online", config.dboxdir);

    execl(programname, programname, arg1, arg2, arg3, 
      arg4, arg5, arg6, NULL);

    printf("NOT FOUND!\r\n");
    sleep(1);
    exit(1);
   }

  sta=waitprocess(pid);

  if(sta!=0)
    logf.printf("startonline", "FATAL: online failed");
 }

static void printlogin()
 {
  FILE *file=openconfigfile(status, "login");
  online.printf("\r\n");
  if(file!=NULL) printtextfile(&online, file);
 }

bool startport()
 {
  char tmpdirname[DIRLEN];

  snprintf(tmpdirname, DIRLEN, DBOXPORTDIR ".", config.dboxdir, config.port);

  if(access(tmpdirname, 0)!=0)
   {
    logf.printf("startport", "warning: directory of port %u was created",
                config.port);

    snprintf(tmpdirname, DIRLEN, DBOXPORTDIR, config.dboxdir, config.port);
    tmpdirname[strlen(tmpdirname)-1]=0;

    if(mkdir(tmpdirname, 0700)!=0)
     {
      logf.printf("startport", "fatal: failed to create directory %s",
                  tmpdirname);
      return TRUE;
     }

   }

  snprintf(tmpdirname, DIRLEN, DBOXPORTTEMPDIR ".", config.dboxdir, config.port);
  if(access(tmpdirname, 0)!=0)
   {
    logf.printf("startport", "warning: created temp directory of port %u",
                config.port);

    snprintf(tmpdirname, DIRLEN, DBOXPORTTEMPDIR, config.dboxdir, config.port);
    tmpdirname[strlen(tmpdirname)-1]=0;

    if(mkdir(tmpdirname, 0700)!=0)
     {
      logf.printf("startport", "fatal: failed to create directory %s",
                  tmpdirname);
      return TRUE;
     }
   }
  return FALSE;
 }

bool readusername()
 {
  online.flushkeys();
  online.printf("Username:");
  return readline(&online, status.username, FALSE, TRUE, FALSE, FALSE);
 }

void doonlineuser()
 {
  charsett old_in_charset, old_out_charset;
  bool oldusecolor;

  snprintf(status.userlogfile, DIRLEN, BOXUSERDIR "logfile", 
    config.boxdir, status.username);

  status.userprofile.read();
  status.letzteranruf=atol(status.userprofile|"letzteranruf");

  portinfo.owninfo.status=portinfot::infot::ONLINE;
  strmaxcpy(portinfo.owninfo.username, status.username, 39);
  portinfo.write(config.port);

  online.sizex=80;
  online.sizey=atoi(status.userprofile|"lines");
  if(status.online2)
   {
    old_in_charset=online.comport.hw_in_charset;
    old_out_charset=online.comport.hw_out_charset;
    online.comport.hw_in_charset=(charsett)atoi(status.userprofile|"charset");
    online.comport.hw_out_charset=online.comport.hw_in_charset;
   }
  else
   {
    old_in_charset=online.console.hw_in_charset;
    old_out_charset=online.console.hw_out_charset;
    online.console.hw_in_charset=(charsett)atoi(status.userprofile|"charset");
    online.console.hw_out_charset=online.console.hw_in_charset;
   }

  oldusecolor=online.usecolor;
  online.usecolor=atoi(status.userprofile|"usecolor");

  status.timeouttime=0;

  status.lastkey=time(NULL);

  if(!getinternalflag(&status.userprofile, &status.gruppe, LOGIN_ONLINE))
   {
    userlogprintf1(status, "doonlineuser", "Online login of %s denied",
      status.username);
    online.printf("\r\nAccess denied\r\n");
   }
  else
   {
    userlogprintf1(status, "doonlineuser", "%lu. Online-Login von User %s",
      atol(status.userprofile|"anrufe"), status.username);

    startonline();

    strmaxcpy(status.lastcall, status.username, TEXTLEN-1);
    userlogprintf1(status, "doonlineuser", "Ausloggen");
   }

  online.sizex=online.console.sizex;
  online.sizey=online.console.sizey;
  if(status.online2)
   {
    online.comport.hw_in_charset=old_in_charset;
    online.comport.hw_out_charset=old_out_charset;
   }
  else
   {
    online.console.hw_in_charset=old_in_charset;
    online.console.hw_out_charset=old_out_charset;
   }

  online.usecolor=oldusecolor;
 }

static bool checkuserlogin()
 {
  char password[TEXTLEN];

  strmaxcpy(status.userprofile.name, status.username, TEXTLEN-1);
  if(status.userprofile.read())
   {
    online.flushkeys();
    online.printf("Passwort:");
    if(readline(&online, password, TRUE, FALSE, FALSE, FALSE)) return TRUE;
    logf.printf("checkuserlogin", "Falscher Username: %s", status.username);

   wrongname:
    online.printf("\r\nName oder Passwort falsch!\r\n\n");
    return TRUE;
   }

  strmaxcpy(status.gruppe.name, status.userprofile|"gruppe", TEXTLEN-1);

  status.gruppe.read();

  if(strcmp(status.userprofile|"cryptedpassword", "none")==0)
   {
    doonlineuser();
    return FALSE;
   }

  online.flushkeys();
  online.printf("Passwort:");
  if(readline(&online, password, TRUE, FALSE, FALSE, FALSE)) return TRUE;

  if(verifylogin(&status.userprofile, password))
   {
    userlogprintf2(status, status.username, "checkuserlogin",
      "User \"%s\": Falsches Passwort!", status.username);
    goto wrongname;
   }

  doonlineuser();

  return FALSE;
 }

void dologin()
 {
  char reloginfile[DIRLEN];

  snprintf(reloginfile, DIRLEN, DBOXPORTTEMPDIR "relogin", 
    config.dboxdir, config.port);

  do
   {
    time_t startlogin;
    byte trials=3;

    status.timeouttime=status.lastkey=0;
    status.dologout=status.killed=FALSE;
    status.relogin=FALSE;

    remove(reloginfile);

    portinfo.owninfo.status=portinfot::infot::LOGIN;
    portinfo.owninfo.username[0]=0;
    strmaxcpy(portinfo.owninfo.from, status.from, TEXTLEN-1);
    portinfo.write(config.port);

    printlogin();

    time(&startlogin);

   schleife:
    if(status.dologout || (time(NULL)>(startlogin+120)) || readusername())
     {
      logf.printf("dologin", "Timeout bei Login");
     }
    else
     {
      if(status.username[0]==0)
       {
        online.printf("\r\n");
        goto schleife;
       }

      if(stricmp(status.username, "@QUIT")==0)
       {
        online.printf("Ok, Quit!\r\n");
        logf.printf("dologin", "@QUIT");
       }
      #if 0
      else if(stricmp(status.username, "@GETTY")==0)
       {
        logf.printf("dologin", "@GETTY");
        dogetty();
        online.printf("\r\n");
        goto schleife;
       }
      #endif
      else if(checknetlogin(status.username))
       {
        if(checkuserlogin())
         {
          trials--;
          if(trials==0)
            online.printf("Zu viele Versuche, bitte legen Sie auf!\r\n\n");
          else
            goto schleife;
         }
       }
     }

    if(access(reloginfile, 00)==0)
      status.relogin=TRUE;

    status.username[0]=0;
    portinfo.owninfo.status=portinfot::infot::OFFLINE;
    portinfo.write(config.port);

    if(status.relogin)
      logf.printf("dologin", "Relogin");
   }
  while(status.relogin);

  status.timeouttime=status.lastkey=0;
  status.dologout=status.killed=FALSE;
  sleep(1);
 }

void doofflinelogin()
 {
  config.port=MAXPORTS-1;
  while(config.port>(MAXPORTS-(MAXPORTS/3)))
   {
    portinfo.read(config.port);
    if(portinfo.info.status==portinfot::infot::OFFLINE)
     {
      if(startport()) return;
      logf.setport(config.port);

      if(status.from[0]==0)
        logf.printf("doofflinelogin", "Offline Anruf");
      else
        logf.printf("doofflinelogin", "Offline Anruf von %s", status.from);

      dologin();
      logf.printf("doofflinelogin", "Offline Auflegen");
      logf.setport(0);

      return;
     }

    config.port--;
   }

  online.printf("DBOX: FATAL: Kein freier Port mehr!\r\n");
  logf.printf("doofflinelogin", "Fatal: Kein freier Port mehr!");
 }
