/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <time.h>
#include <dirent.h>

#include <logf.h>
#include <dboxpath.h>
#include <netdefs.h>
#include <logins.h>
#include "../online/extern.h"
#include "../online/online.h"
#include "../online/userlog.h"
#include "netz.h"
#include "puffer.h"
#include "z38.h"
#include "janus.h"

class gsboxnetlogint:public z38netlogint
 {
 public:
  virtual bool checklicense();
  virtual void handle_local();
 };
 
void gsboxnetlogint::handle_local()
 {
  char infilename[DIRLEN], 
       outfilename[DIRLEN], 
       newoutfilename[DIRLEN], 

  del_port_temp_dir();

  snprintf(infilename, DIRLEN, DBOXPORTTEMPDIR "infile.%s", 
    config.dboxdir, config.port, packerextention(up|"packer"));

  remove(infilename);

  if(getnetcallfile(&up, outfilename))
   {
    userlogprintf2(status, up.name, "gsboxnetlogin", "failed to get netcallfile");
    online.printf("\r\nFatal: Ein Transfer ist zur Zeit nicht mglich!\r\n");
    return;
   }

  if(access(outfilename, 00)!=0)
   {
    if(createnetcallfile(outfilename, up|"packer"))
     {
      userlogprintf2(status, up.name, "gsboxnetlogin", "failed to create netcallfile");
      online.printf("\r\nFatal: Ein Transfer ist zur Zeit nicht mglich!\r\n");
      freenetcallfile(&up);
      return;
     }
   }

  snprintf(newoutfilename, DIRLEN, BOXSPOOLDIR "puffer.%s", config.boxdir,
    up.name, packerextention(up|"packer"));

  remove(newoutfilename);
  if(rename(outfilename, newoutfilename)!=0)
   {
    userlogprintf2(status, up.name, "gsboxnetlogin",
      "Fatal: rename(\"%s\", \"%s\"); failed", outfilename, newoutfilename);
    online.printf("\r\nFatal: Ein Transfer ist zur Zeit nicht mglich!\r\n");
    freenetcallfile(&up);
    return;
   }

  if(status.online2)
   {
    if(!online.comport.dcd())
     {
      userlogprintf2(status, up.name, "gsboxnetlogin", "Carrier lost");
      rename(newoutfilename, outfilename);
      freenetcallfile(&up);
      return;
     }
   }

  // EMPFANGEN

   {
    char tempstr[TEXTLEN];

    snprintf(tempstr, TEXTLEN, "Eingangspuffer von Netzwerksystem %s", up.name);
    if(doupload_specialfile('Z', infilename, tempstr, logintype))
     {
      userlogprintf2(status, up.name, "gsboxnetlogin", "Upload fehlgeschlagen");
      rename(newoutfilename, outfilename);
      freenetcallfile(&up);
      return;
     }
   }

  online.flushkeys();

  // SENDEN
   
   {
    char tempstr[TEXTLEN];

    snprintf(tempstr, TEXTLEN, "Ausgangspuffer von Netzwerksystem %s", up.name);
    if(dodownload('Z', newoutfilename, tempstr, logintype))
     {
      userlogprintf2(status, up.name, "gsboxnetlogin", "Download fehlgeschlagen");
      rename(newoutfilename, outfilename);
      freenetcallfile(&up);
      sleep(5);
      return;
     }
   }

  storearchive(&up, infilename);

  remove(newoutfilename);
  freenetcallfile(&up);
 }

bool gsboxnetlogint::checklicense()
 {
  return FALSE;
 }

void gsboxlogin(const char *login)
 {
  gsboxnetlogint gsboxnetlogin;

  strmaxcpy(gsboxnetlogin.loginstring, login, TEXTLEN-1);
  gsboxnetlogin.login=NETZ_GSBOX;
  gsboxnetlogin.donetlogin();
 }

