/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <time.h>
#include <dirent.h>

#include <logf.h>
#include <dboxpath.h>
#include <asyncio.h>
#include <netdefs.h>
#include <logins.h>
#include "../online/extern.h"
#include "../online/online.h"
#include "../online/userlog.h"
#include "netz.h"
#include "puffer.h"
#include "z38.h"
#include "janus.h"

class janusnetlogint:public z38netlogint
 {
 public:
  virtual bool checklicense();
  virtual void handle_local();
  virtual void handle_local_janus(char *outfilename);
  virtual void handle_local_janus2(char *outfilename);
 };

bool janusnetlogint::checklicense()
 {
  return FALSE;
 }

void janusnetlogint::handle_local_janus(char *outfilename)
 {
  char newoutfilename[DIRLEN], infilename[DIRLEN];

  snprintf(infilename, DIRLEN, DBOXPORTTEMPDIR "infile.%s",
    config.dboxdir, config.port, packerextention(up|"packer"));

  remove(infilename);

  if(access(outfilename, 00)!=0)
   {
    if(createnetcallfile(outfilename, up|"packer"))
     {
      userlogprintf2(status, up.name, "janusnetlogin", "failed to create netcallfile");
      online.printf("\r\nFatal: Ein Transfer ist zur Zeit nicht mglich!\r\n");
      return;
     }
   }

  snprintf(newoutfilename, DIRLEN, BOXSPOOLDIR "called.%s",
    config.boxdir, up.name, packerextention(up|"packer"));

  remove(newoutfilename);
  if(rename(outfilename, newoutfilename)!=0)
   {
    userlogprintf2(status, up.name, "janusnetlogin",
    "Fatal: rename(\"%s\", \"%s\"); failed", outfilename, newoutfilename);
    online.printf("\r\nFatal: Ein Transfer ist zur Zeit nicht mglich!\r\n");
    return;
   }

  online.putch(21);
  asyncio.flush_output();
  sleep(1);
  online.putch(6);
  online.flushkeys();
  asyncio.flush_output();

  if(status.online2)
   {
    if(!online.comport.dcd())
     {
      userlogprintf2(status, up.name, "janusnetlogin", "Carrier lost");
      rename(newoutfilename, outfilename);
      return;
     }
   }

  // EMPFANGEN

  if(transfersinglefile(TRUE, infilename))
   {
    userlogprintf2(status, up.name, "janusnetlogin", "Upload fehlgeschlagen");
    rename(newoutfilename, outfilename);
    return;
   }

  online.flushkeys();

  // SENDEN
  
  if(transfersinglefile(FALSE, newoutfilename))
   {
    userlogprintf2(status, up.name, "janusnetlogin", "Download fehlgeschlagen");
    rename(newoutfilename, outfilename);
    return;
   }

  storearchive(&up, infilename);

  remove(newoutfilename);
 }

void janusnetlogint::handle_local_janus2(char *outfilename)
 {
  DIR *dir;
  struct dirent *ent;
  bool error=FALSE;
  char infilename[DIRLEN];

  online.putch(21);
  asyncio.flush_output();
  sleep(1);
  online.putch(6);
  asyncio.flush_output();
  online.flushkeys();

  snprintf(infilename, DIRLEN, DBOXPORTTEMPDIR ".", config.dboxdir, config.port);

   {
    char tempstr[TEXTLEN];

    snprintf(tempstr, TEXTLEN, "Eingangspuffer von Netzwerksystem %s", up.name);
    if(doupdownload(FALSE, 'Z', infilename, tempstr, logintype))
     {
      userlogprintf2(status, up.name, "janusnetlogin", "Upload fehlgeschlagen");
      error=TRUE;
     }
   }

  dir=opendir(infilename);
  if(dir!=NULL)
   {
    while((ent=readdir(dir))!=NULL)
     {
      if(ent->d_name[0]!='.')
       {
        snprintf(infilename, DIRLEN, DBOXPORTTEMPDIR "%s", 
          config.dboxdir, config.port, ent->d_name);
        storearchive(&up, infilename);
       }
     }
    closedir(dir);
   }

  if(!error)
   {
    #define MAXFILESLEN 1000
    #define MAXFILES 50
    char files[MAXFILESLEN]="-u";
    int i=0;
    bool sendmail=TRUE, sendnews=TRUE;

    if(stricmp(loginstring, "JANUS2H")==0)
     {
      sendmail=sendnews=FALSE;
      userlogprintf2(status, up.name, "janusnetlogin", "Sende leeren Puffer");
     }
    else if(stricmp(loginstring, "JANUS2N")==0)
     {
      sendmail=FALSE;
      userlogprintf2(status, up.name, "janusnetlogin", "Sende nur News");
     }
    else if(stricmp(loginstring, "JANUS2P")==0)
     {
      sendnews=FALSE;
      userlogprintf2(status, up.name, "janusnetlogin", "Sende nur Mail");
     }

    if(sendnews)
     {
      char tmpfilename[DIRLEN];

      snprintf(tmpfilename, DIRLEN, BOXSPOOLDIR "%s.%s", 
        config.boxdir, up.name, up.name,
        packerextention(up|"packer"));

      if(access(tmpfilename, 00)==0)
       {
        char newfilename[DIRLEN];
        dword t=time(NULL);

        do
         {
          snprintf(newfilename, DIRLEN, BOXSPOOLDIR "NP%06lu.%s", 
            config.boxdir, up.name, t%1000000,
            packerextention(up|"packer"));
          t++;
         }
        while(access(newfilename, 00)==0);

        rename(tmpfilename, newfilename);
       }
     }

    if(sendmail || sendnews)
     {
      char dirname[DIRLEN];

      snprintf(dirname, DIRLEN, BOXSPOOLDIR ".", config.boxdir, up.name);
      dir=opendir(dirname);
      if(dir!=NULL)
       {
        while((ent=readdir(dir))!=NULL && i<MAXFILES &&
              strlen(files)<MAXFILESLEN)
         {
          if(isjanus2packet(ent->d_name, &up))
           {
            if((ent->d_name[0]=='P' && sendmail) ||
               (ent->d_name[0]=='N' && sendnews))
             {
              snprintf(files+strlen(files), MAXFILESLEN-strlen(files), 
                " " BOXSPOOLDIR "%s", config.boxdir, up.name, ent->d_name);
              i++;
             }
           }
         }
        closedir(dir);
       }
     }

    if(i==0)
     {
      snprintf(outfilename, DIRLEN, BOXSPOOLDIR "NP000000.%s", 
        config.boxdir, up.name,
        packerextention(up|"packer"));

      if(createnetcallfile(outfilename, up|"packer"))
       {
        userlogprintf2(status, up.name, "janusnetlogin", "failed to create netcallfile");
        return;
       }

       {
        char tempstr[TEXTLEN];

        snprintf(tempstr, TEXTLEN, "Ausgangspuffer von Netzwerksystem %s", up.name);
        if(doupdownload(TRUE, 'Z', outfilename, tempstr, logintype))
         {
          userlogprintf2(status, up.name, "janusnetlogin", "Download fehlgeschlagen");
          error=TRUE;
         }
       }
     }        
    else
     {
      char tempstr[TEXTLEN];

      snprintf(tempstr, TEXTLEN, "Ausgangspuffer von Netzwerksystem %s", up.name);
      if(doupdownload(TRUE, 'Z', files, tempstr, logintype))
       {
        userlogprintf2(status, up.name, "janusnetlogin", "Download fehlgeschlagen");
        error=TRUE;
       }
     }
   }
 }

void janusnetlogint::handle_local()
 {
  char outfilename[DIRLEN];

  del_port_temp_dir();

  online.printf("Running ARC\r\n");
  online.printf("Running ARC\r\n");
  online.printf("Running ARC\r\n");

  if(getnetcallfile(&up, outfilename))
   {
    userlogprintf2(status, up.name, "janusnetlogin", "Failed to get netcallfile");
    online.printf("\r\nFatal: Ein Transfer ist zur Zeit nicht mglich!\r\n");
    return;
   }

  if(login==NETZ_JANUS2)
    handle_local_janus2(outfilename);
  else
    handle_local_janus(outfilename);

  freenetcallfile(&up);
 }

void januslogin(const char *login)
 {
  janusnetlogint janusnetlogin;

  strmaxcpy(janusnetlogin.loginstring, login, TEXTLEN-1);
  janusnetlogin.login=NETZ_JANUS;
  janusnetlogin.donetlogin();
 }

void janus2login(const char *login)
 {
  janusnetlogint janusnetlogin;

  strmaxcpy(janusnetlogin.loginstring, login, TEXTLEN-1);
  janusnetlogin.login=NETZ_JANUS2;
  janusnetlogin.donetlogin();
 }

void znetzlogin(const char *login)
 {
  janusnetlogint janusnetlogin;

  strmaxcpy(janusnetlogin.loginstring, login, TEXTLEN-1);
  janusnetlogin.login=NETZ_Z38;
  janusnetlogin.donetlogin();
 }

