/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <time.h>
#include <dirent.h>

#include <dbox.h>
#include <tools.h>
#include <logf.h>
#include <dboxpath.h>
#include <oldtools.h>
#include <radius.h>
#include <account.h>
#include <netdefs.h>
#include <flags.h>
#include <portinfo.h>

#include "../online/online.h"
#include "../online/userlog.h"
#include "puffer.h"
#include "uucp.h"
#include "fido.h"
#include "ppp.h"
#include "slip.h"
#include "zclogin.h"
#include "janus.h"
#include "alpha.h"
#include "gsbox.h"
#include "netz.h"

struct netloginst
 {
  char *login;
  void (*loginproc)(const char *loginname);
 } netlogins[]=
 {
  { "JANUS", januslogin },
  { "JANUS2", janus2login },
  { "JANUS2H", janus2login },
  { "JANUS2N", janus2login },
  { "JANUS2P", janus2login },
  { "ZERBERUS", znetzlogin },
  { "ZCONNECT", zconnectlogin },
  { "UUCP", uucplogin },
  { "@ALPHA1", alpha1login },
  { "NET410", gsboxlogin },
  { "NETCALL", gsboxlogin },
  { "SLIP", sliplogin },
  { "CSLIP", csliplogin },
  { "@PPP", autoppplogin },
  { "FIDO", fidologin },
  { NULL, NULL }
 };

byte getnettype(userpt &up)
 {
  int login;

  login=up.pconfig->nettypedb.index(up|"netztyp");
  if(login<0) login=NETZ_JANUS;

  return login;
 }

bool ispoint(char *systemname)
 {
  char prefix[TEXTLEN], tempstr[TEXTLEN];

  snprintf(prefix, TEXTLEN, "%s-", config.boxname);
  if(strnicmp(systemname, prefix, strlen(prefix))!=0)
    return FALSE;

  strmaxcpy(tempstr, systemname+strlen(prefix), TEXTLEN-1);
  strmaxcpy(systemname, tempstr, TEXTLEN-1);

  return TRUE;   
 }

bool checknetlogin(const char *login)
 {
  int i;

  i=0;
 schleife:
  if(netlogins[i].login!=NULL)
   {
    if(stricmp(login, netlogins[i].login)==0)
     {
      status.timeouttime=0;

      logf.printf("checknetlogin", "Anruf mit Netcall-Verfahren \"%s\"",
        netlogins[i].login);

      netlogins[i].loginproc(login);
      return FALSE;
     }

    i++;
    goto schleife;
   }

  return TRUE;
 }

char *netzprotocols[]=
 {
  "janus",      // NETZ_JANUS 
  "janus2",     // NETZ_JANUS2
  "z38",        // NETZ_Z38 
  "zconnect",   // NETZ_ZCONNECT
  "uucp",       // NETZ_UUCP
  "alphabox",   // NETZ_ALPHABOX
  "gsbox",      // NETZ_GSBOX
  "fido",       // NETZ_FIDO
  "pop3",       // NETZ_POP3
  NULL
 };

static bool processpointlist(char *username, const char *password)
 {
  FILE *infile;
  char tempstr[1000], *tptr, *tptr2, filename[DIRLEN];

  snprintf(filename, DIRLEN, BOXSYSTEMDIR "pointlist", config.boxdir);
  infile=fopen(filename, "rt");
  if(infile!=NULL)
   {
   s:
    if(fgets(tempstr, 900, infile)!=NULL)
     {
      tptr=strrchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strrchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(tempstr, ' ');
      if(tptr==NULL) goto s;
      *(tptr++)=0;
      if(stricmp(username, tempstr)==0)
       {
        tptr2=strrchr(tptr, ' ');
        if(tptr2!=NULL)
         {
          *(tptr2++)=0;
          if(stricmp(password, tptr)==0)
           {
            strmaxcpy(username, tptr2, TEXTLEN-1);
            fclose(infile);
            logf.printf("donetzlogin",
              "Login ueber pointlist von User \"%s\"", username);
            return TRUE;
           }
         }
       }
      else
        goto s;
     }
    fclose(infile);
   } /* Temp ende */

  return FALSE;
 }

void netlogint::init()
 {
  strmaxcpy(protocol, netzprotocols[login], TEXTLEN-1);
  snprintf(logintype, TEXTLEN, "login type %s", config.nettypedb[login]);

  rclient.session_id[0]=0;
  ip[0]=0;

  strmaxcpy(cp.name, "localhost.dbox", TEXTLEN-1);
  cp.read();
 }

bool netlogint::radiuslogin(const char *password)
 {
  char message[TEXTLEN]="";
 
  snprintf(rclient.username, TEXTLEN, "%s-%s", protocol, check_login1.username);
  rclient.identifier=((status.midcounter++)+time(NULL))&255;
  rclient.radius_auth_server=getinternetaddress(check_login1.radius_auth_server);
  rclient.radius_acct_server=getinternetaddress(check_login1.radius_acct_server);
  rclient.nas_ip_address=config.radius_ip_address;
  snprintf(rclient.session_id, TEXTLEN, "%d.%lu.%lu",
    config.port, status.midcounter++, time(NULL));
  strcpy(rclient.secret, check_login1.radius_auth_secret);

  logf.printf("netlogint::radiuslogin", "Authorisierung von User %s",
    rclient.username);

  online.printf("\r\nFuehre Authorisierung durch...");

  switch(rclient.authenticate(password, message))
   {
   case 0:
    online.printf(" Ok\r\n%s\r\n", message);
    break;

   case 1:
    online.printf(" Fehlgeschlagen\r\n%s\r\n", message);
    logf.printf("netlogint::radiuslogin", 
                "Fehler: %s", message);
    return TRUE;

   case 2:
    online.printf(" Timeout!\r\n");
    logf.printf("netlogint::radiuslogin", "radius: Timeout");
    return TRUE;

   default:
    online.printf(" Fehlgeschlagen\r\n");
    logf.printf("netlogint::radiuslogin", "radius: FEHLER!");
    return TRUE;
   }

  if(up.read())
   {
    if(check_login1.radius_autoadd)
     {
      #if 0 // TODO
      logf.printf("netlogint::radiuslogin", "User %s angelegt");
      up.lock();
      up.clear();
      up.profile_ok=TRUE;
      up.eintragsdatum=time(NULL);
      up.netztyp=login;
      // up.packer=;
      // up.potokoll=;
      // up.system=;
      if(up.add())
       {
        logf.printf("netlogint::radiuslogin", "FEHLER: User konnte nicht %s angelegt werden");
        return TRUE;
       }
      #else
      return TRUE;
      #endif
     }
    else
     {
      logf.printf("netlogint::radiuslogin", "User %s nicht gefunden", up.name);
      return TRUE;
     }
   }

  return FALSE;
 }

void netlogint::del_port_temp_dir()
 { 
  char dirname[DIRLEN];

  snprintf(dirname, DIRLEN, DBOXPORTTEMPDIR, 
    config.dboxdir, config.port);

  deldir(dirname);
 } 

FILE *netlogint::open_login()
 {
  char filename[DIRLEN];
  FILE *infile;

  snprintf(filename, DIRLEN, DBOXPORTDIR "login", 
    config.dboxdir, config.port);

  if((infile=fopen(filename, "rt"))==NULL)
   {
    snprintf(filename, DIRLEN, BOXSYSTEMDIR "login", config.boxdir);
    if((infile=fopen(filename, "rt"))==NULL)
     {
      snprintf(filename, DIRLEN, "%slogin", cp.getpath());
      if((infile=fopen(filename, "rt"))==NULL)
       {
        snprintf(filename, DIRLEN, DBOXSYSTEMDIR "login", config.dboxdir);
        infile=fopen(filename, "rt");
       }
     }
   }

  return infile;
 }

FILE *netlogint::open_charge()
 {
  char filename[DIRLEN];
  FILE *infile;

  snprintf(filename, DIRLEN, DBOXPORTDIR "charge", 
    config.dboxdir, config.port);

  if((infile=fopen(filename, "rt"))==NULL)
   {
    snprintf(filename, DIRLEN, BOXSYSTEMDIR "charge", config.boxdir);
    if((infile=fopen(filename, "rt"))==NULL)
     {
      snprintf(filename, DIRLEN, "%scharge", cp.getpath());
      if((infile=fopen(filename, "rt"))==NULL)
       {
        snprintf(filename, DIRLEN, DBOXSYSTEMDIR "charge", config.dboxdir);
        infile=fopen(filename, "rt");
       }
     }
   }

  return infile;
 }

bool netlogint::verifylogin(const char *password, char *message)
 {
  filet infile;
  bool pointlist=FALSE, pw_radius_verified=FALSE;

  message[0]=0;
  local=TRUE;
  dnstring(loginname);

  pointlist=processpointlist(loginname, password);

  infile.file=open_login();

  check_login1.login=loginname;
  check_login1.protocol=protocol;

  if(infile.file==NULL) // Kompatibilitt
   {
    strmaxcpy(up.name, loginname, TEXTLEN-1);
    ispoint(up.name);
    strmaxcpy(check_login1.username, up.name, TEXTLEN-1);
   }
  else
   {
    if(check_login1.check_login1(infile.file))
     {
      logf.printf("verifylogin", 
                  "%s-login of %s denied by logins-profile",
                  protocol, check_login1.username);

      if(check_login1.errormsg[0]!=0)
       {
        strmaxcpy(message, check_login1.errormsg, TEXTLEN-1);
        logf.printf("verifylogin", "%s", check_login1.errormsg);
       }

      return TRUE;
     }

    strmaxcpy(up.name, check_login1.username, TEXTLEN-1);

    if(up.name[0]==0)
      return TRUE;

    if(check_login1.radius_auth_server[0]!=0 && !pointlist)
     {
      if(radiuslogin(password))
        return TRUE;

      pw_radius_verified=TRUE;
     }
   }

  if(up.name[0]==0)
    return TRUE;

  check_login2.charge_menu[0]=0;

  if(local)
   {
    if(up.read())
     {
      logf.printf("verifylogin", 
                  "%s-Login of %s: user not found", 
                  protocol, up.name);
      return TRUE;
     }
     
    if(login!=getnettype(up))
     {
      strcpy(message, "wrong protocol");
      userlogprintf2(status, up.name, "verifylogin", "wrong protocol");
      return TRUE;
     }
  
    if(!pointlist && !pw_radius_verified && ::verifylogin(&up, password))
     {
      userlogprintf2(status, up.name, "dochecknetlogin", "wrong password");
      return TRUE;
     }

    if(check_login1.radius_auth_server[0]==0)
     {
      strmaxcpy(gp.name, up|"gruppe", TEXTLEN-1);
  
      gp.read();
      if(!getinternalflag(&up, &gp, LOGIN_NETZ))
       {
        userlogprintf2(status, up.name, "dochecknetlogin", "net access denied: no login_netz flag");
        strcpy(message, "net access denied");
        return TRUE;
       }

      infile.close();
      infile.file=open_charge();

      check_login2.protocol=protocol;
      check_login2.up=&up;

      if(infile.file==NULL) // Kompatibilitt
       {
        strmaxcpy(check_login2.charge_menu, "netcall", TEXTLEN-1);
       }
      else if(check_login2.check_login2(infile.file))
       {
        logf.printf("verifylogin", 
                    "%s-login of %s denied by charge-profile",
                    protocol, check_login2.up->name);

        if(check_login2.errormsg[0]!=0)
         {
          strmaxcpy(message, check_login2.errormsg, TEXTLEN-1);
          logf.printf("verifylogin", "%s", check_login2.errormsg);
         }

        return TRUE;
       }

      if(check_login2.charge_menu[0]!=0)
       {
        if(checkchargemenu(&up, &gp, check_login2.charge_menu, message))
         {
          userlogprintf2(status, up.name, "netlogin", "%s", message);
          return TRUE;
         }
       }
     }
   }

  accounting_start();

  if(local)
    handle_local();
  else
    handle_remote();

  accounting_stop();

  return FALSE;
 }

void netlogint::accounting_start()
 {
  time(&starttime);
  if(local && !up.lock())
   {
    char anrufe[TEXTLEN], letzteranruf[TEXTLEN];
    snprintf(anrufe, TEXTLEN, "%lu", atol(up|"anrufe")+1);
    snprintf(letzteranruf, TEXTLEN, "%lu", time(NULL));
    up.set("anrufe", anrufe);
    up.set("letzteranruf", letzteranruf);
    up.write();
    up.unlock();
   }

  strmaxcpy(status.lastcall, up.name, TEXTLEN-1);

  if(local)
    userlogprintf2(status, up.name, "accounting_start", "Start %s",
      logintype);
  else
    logf.printf("accounting_start", "Start %s %s",
      up.name, logintype);

  if(ip[0]==0)
    portinfo.owninfo.status=portinfot::infot::NETCALL;
  else
   {
    portinfo.owninfo.status=portinfot::infot::SLIP;
    portinfo.owninfo.slipaddr=getinternetaddress(ip);
   }

  if(local)
    strmaxcpy(portinfo.owninfo.boxname, config.boxname, TEXTLEN-1);
  else
    portinfo.owninfo.boxname[0]=0;

  strmaxcpy(portinfo.owninfo.username, up.name, TEXTLEN-1);
  portinfo.write(config.port);

  if(check_login1.radius_acct_server[0]!=0)
   {
    rclient.identifier=((status.midcounter++)+time(NULL))&255;
    rclient.radius_acct_server=getinternetaddress(check_login1.radius_acct_server);
    rclient.nas_ip_address=config.radius_ip_address;
    rclient.ip_address=getinternetaddress(ip);
    strcpy(rclient.secret, check_login1.radius_acct_secret);

    if(rclient.session_id[0]==0)
      snprintf(rclient.session_id, TEXTLEN, "%d.%lu.%lu",
        config.port, status.midcounter++, starttime);
    
    strcpy(rclient.calling_station_id, status.from);
    rclient.port=config.port;

    if(status.online2)
      online.console.printf("Radius-accounting ");

    if(rclient.log_start()!=0)
     {
      if(local)
        userlogprintf2(status, up.name, "netlogint::accounting_start", "radius: start accounting failed!");
      else
        logf.printf("netlogint::accounting_start", "radius: start accounting failed!");

      if(status.online2)
        online.console.printf("failed\r\n");
     }
    else
     {
      if(status.online2)
        online.console.printf("ok\r\n");
     }
   }
 }

void netlogint::accounting_stop()
 {
  time_t stoptime, duration;
  char flagfile[DIRLEN];

  time(&stoptime);
  duration=stoptime-starttime;

  if(local)
    userlogprintf2(status, up.name, "accounting_stop", "Completed %s",
      logintype);
  else
    logf.printf("accounting_stop", "Completed %s %s",
      up.name, logintype);

  snprintf(flagfile, DIRLEN, BOXTODODIR "nocharge.%u", 
    config.boxdir, config.port);

  if(access(flagfile, 00)==0)
   {
    remove(flagfile);

    if(local)
      userlogprintf2(status, up.name, "accounting_stop", "nocharge",
        logintype);
    else
      logf.printf("accounting_stop", "nocharge",
        up.name, logintype);

    duration=0;
   }

  if(local)
   {
    if(check_login2.charge_menu[0]!=0)
     {
      logf.printf("accounting_stop", "charge: %s %lds",
        check_login2.charge_menu, duration);
 
      chargemenu(&up, check_login2.charge_menu, "Netcall Zeitgebhr", 
        starttime, stoptime);
     }
    //else
    //  logf.printf("accounting_stop", "charge: no menu");
   }

  if(check_login1.radius_acct_server[0]!=0)
   {
    rclient.identifier=((status.midcounter++)+time(NULL))&255;
    rclient.nas_ip_address=config.radius_ip_address;
    rclient.ip_address=getinternetaddress(ip);

    if(status.online2)
      online.console.printf("Radius-accounting ");

    if(rclient.log_stop(duration)!=0)
     {
      if(local)
        userlogprintf2(status, up.name, "netlogint::accounting_start", "radius: stop accounting failed!");
      else
        logf.printf("netlogint::accounting_start", "radius: stop accounting failed!");

      if(status.online2)
        online.console.printf("failed\r\n");
     }
    else
     {
      if(status.online2)
        online.console.printf("ok\r\n");
     }
   }
 }
