/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <signal.h>
#include <getopt.h>

#include <dbox.h>
#include <device.h>
#include <radius.h>
#include <dboxversion.h>
#include <libdboxversion.h>
#include <dboxpath.h>
#include <portinfo.h>
#include <logf.h>
#include "../online/online.h"
#include "waitcall.h"
#include "doonline.h"

static bool parseoptions(int argc, char **argv)
 {
  char boxname[TEXTLEN]="";
  bool radius=FALSE, ibm=FALSE;

  config.port=0;
  status.debug=FALSE;
  status.dologout=FALSE;
  status.killed=FALSE;
  status.from[0]=0;

   {
    char *tptr;

    tptr=getenv("CALL_FROM");
    if(tptr!=NULL) strmaxcpy(status.from, tptr, TEXTLEN-1);

    tptr=getenv("DBOX_BOXNAME");
    if(tptr!=NULL) 
     { 
      strmaxcpy(boxname, tptr, TEXTLEN-1);
      boxname_changed=TRUE;
     }
   }  

 schleife:
  switch(getopt(argc, argv, "?ho:idf:lsb:r"))
   {
   case '?':
   case 'h':
    goto help;

   case 'o':
    config.port=atoi(optarg);
    if(config.port==0) goto aufruf;
    goto schleife;

   case 'i':
    ibm=TRUE;
    goto schleife;

   case 'd':
    status.debug=TRUE;
    logf.printf("parseoptions", "Debugging on.");
    goto schleife;

   case 'f':
    strmaxcpy(status.from, optarg, TEXTLEN-1);
    goto schleife;

   case 'b':
    strmaxcpy(boxname, optarg, TEXTLEN-1);
    boxname_changed=TRUE;
    goto schleife;

   case 'r':
    radius=TRUE;
    goto schleife;

   case -1:
    break;

   case ':':
    online.printf("\r\n\r\nParameter fehlt! - ");
    goto aufruf;

   default:
    goto aufruf;
   }

  online.console.assign("");
  online.console.open();

  if(config.read(boxname))
   {
    online.console.close();
    return FALSE;
   }

  online.open();

  if(ibm)
   { 
    online.console.set_hw_charset(IBM);
    online.console.close();
    online.console.linuxconsole=TRUE;
    online.console.open();
   }

  online.sizex=online.console.sizex;
  online.sizey=online.console.sizey;

  strmaxcpy(portinfo.owninfo.boxname, config.boxname, TEXTLEN-1);
  strmaxcpy(portinfo.owninfo.sessionid, "", TEXTLEN-1);

  if(radius)
    radius_test();
  else
   {
    if(config.port==0)
      doofflinelogin(); 
    else
     {
      use_console=TRUE;

      if(config.public_terminal)
        public_terminal();
      else
        waitforcall();
     }
   }

  online.flushkeys();
  online.console.close();
  fflush(stdin);

  if(status.restart)
   {
    #define MAXARGS 100

    char *new_argv[MAXARGS];
    char programname[DIRLEN];
    int i;

    for(i=0; i<argc && i<(MAXARGS-1); i++)
      new_argv[i]=argv[i];

    new_argv[i]=NULL;

    snprintf(programname, DIRLEN-1, DBOXBINDIR "oserv", config.dboxdir);
    execv(programname, new_argv);
   }

  return FALSE;

 aufruf:
  fprintf(stderr, "Falscher Aufruf!\n");

 help:
  fprintf(stderr, 
    "\n"
    "* * OSERV - DBox Modul fuer Onlineverbindungen - "
    "Version " DBOXVERSIONNR " vom " DBOXVERSION " * *\n"
    "\n"
    "Aufruf:                      Aktion:\n"
    "\n"
    " oserv [-?] [-h]              Hilfe ausgeben\n"
    " oserv                        Offline-Port starten\n"
    " oserv -oPortnummer           Online-Port starten\n"
    " oserv -r                     RADIUS-Test\n"
    "\n"
    "Weitere Optionen:\n"
    "  -i   IBM Zeichensatz\n"
    "  -d   Debugging on\n"
    "  -f   Caller-ID (Telefon oder Netzadresse)\n"
    "  -b   Boxname\n"
    "\n"
   );

  return TRUE;
 }

void sig_hangup(int dummy)
 {
  /*
  if(config.port!=0)
   {
    portinfo.owninfo.status=portinfot::infot::OFFLINE;
    portinfo.write(config.port);
   }*/
  if(status.hup_child) kill(status.child, SIGHUP);
  logf.printf("sig_hangup", "oserv received signal");
  status.killed=status.dologout=TRUE;
  online.comport.sigalarm=TRUE;
  online.console.sigalarm=TRUE;
 }

void sig_alarm(int dummy)
 {
  logf.printf("sig_alarm", "received signal");
  status.dologout=TRUE;
  online.comport.sigalarm=TRUE;
  online.console.sigalarm=TRUE;
  signal(SIGALRM, sig_alarm);
 }

int main(int argc, char **argv)
 {
  if(init_libdbox(LIBDBOXVERSION, LIBDBOXVERSIONNR)) return 1;

  signal(SIGHUP, sig_hangup);
  signal(SIGINT, sig_hangup);
  signal(SIGQUIT, sig_hangup);
  signal(SIGTERM, sig_hangup);
  signal(SIGALRM, sig_alarm);
  signal(SIGPIPE, SIG_IGN); // wg. socket
  signal(SIGTTIN, SIG_IGN); // wg. Bash
  signal(SIGTTOU, SIG_IGN); // wg. Bash

  parseoptions(argc, argv);

  return 0;
 }
