/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <time.h>
#include <dirent.h>

#include <dbox.h>
#include <tools.h>
#include <userdb.h>
#include <dboxpath.h>
#include <logf.h>
#include "../online/online.h"
#include "../online/userlog.h"
#include <netdefs.h>
#include "puffer.h"

bool getnetcallfile(userpt *up, char *filename)
 {
  char directory[DIRLEN], tmpfilename1[DIRLEN], tmpfilename2[DIRLEN];
  byte trials=60;

  snprintf(directory, DIRLEN, BOXSPOOLDIR, up->pconfig->boxdir, up->name);
  snprintf(tmpfilename1, DIRLEN, "%sunlocked", directory);
  snprintf(tmpfilename2, DIRLEN, "%slocked", directory);

 schleife:
  if(rename(tmpfilename1, tmpfilename2)!=0)
   {
    if(access(tmpfilename1, 00)!=0 && access(tmpfilename2, 00)!=0)
     {
      FILE *f=fopen(tmpfilename1, "wb");
      if(f==NULL) return TRUE;
      fclose(f);
      sleep(1);
      logf.printf("getnetcallfile", "warning: creating lockfile");
     }
    trials--;
    if(trials==0) return TRUE;

    sleep(1);
    goto schleife;
   }
  
  snprintf(filename, DIRLEN, "%s%s.%s", directory, up->name,
    packerextention(*up|"packer"));

  return FALSE;
 }

void freenetcallfile(userpt *up)
 {
  char directory[DIRLEN], tmpfilename1[DIRLEN], tmpfilename2[DIRLEN];

  snprintf(directory, DIRLEN, BOXSPOOLDIR, config.boxdir, up->name);
  snprintf(tmpfilename1, DIRLEN, "%sunlocked", directory);
  snprintf(tmpfilename2, DIRLEN, "%slocked", directory);
  if(rename(tmpfilename2, tmpfilename1)!=0)
    logf.printf("freenetcallfile", "warning: unlock failed");
 }

void storearchive(userpt *up, const char *filename)
 {
  char tmpfilename[DIRLEN];
  dword i=0;
  struct stat statbuf;

  if(stat(filename, &statbuf)!=0)
   {
    userlogprintf2(status, up->name, "storearchive", "stat fehlgeschlagen");
    return;
   }

  userlogprintf2(status, up->name, "storearchive", "%lu Bytes", statbuf.st_size);

  do
   {
    snprintf(tmpfilename, DIRLEN, BOXSPOOLDIR "in_%lu.%s", config.boxdir,
      up->name, i, packerextention(*up|"packer"));

    i++;
   }
  while(access(tmpfilename, 00)==0);

  if(rename(filename, tmpfilename)!=0)
   {
    userlogprintf2(status, up->name, "storearchive",
      "rename(\"%s\", \"%s\"); failed!", filename, tmpfilename);
   }

   {
    FILE *outfile;
    snprintf(tmpfilename, DIRLEN, BOXTODODIR "in.%s", config.boxdir, up->name);
    outfile=fopen(tmpfilename, "wb");
    if(outfile!=NULL) fclose(outfile);
   }
 }

bool createnetcallfile(const char *out_filename, const char *packer)
 {
  char in_filename[DIRLEN];
  FILE *infile, *outfile;

  snprintf(in_filename, DIRLEN, DBOXSYSTEMDIR "empty.%s", 
          config.dboxdir, packerextention(packer));

  if((infile=fopen(in_filename, "rb"))==NULL)
   {
    logf.printf("createnetcallfile", "fatal: %s not found", in_filename);
    return TRUE;
   }

  if((outfile=fopen(out_filename, "wb"))==NULL)
   {
    logf.printf("createnetcallfile", "fatal: failed to create %s",
      out_filename);

    fclose(infile);
    return TRUE;
   }

  if(filecopy(infile, outfile, filelength(fileno(infile)))!=0)
   {
    logf.printf("createnetcallfile",
      "fatal: failed to copy %s", in_filename);

    fclose(outfile);
    fclose(infile);
    return TRUE;
   }
  fclose(outfile);
  fclose(infile);
  return FALSE;
 }

bool isjanus2packet(const char *filename, userpt *up)
 {
  return (strchr("NPED", filename[0])!=NULL && 
    filename[1]=='P' &&
    filename[8]=='.' && 
    stricmp(filename+9, packerextention(*up|"packer"))==0);
 }

