/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <time.h>
#include <dirent.h>

#include <dbox.h>
#include <tools.h>
#include <logf.h>
#include <logins.h>
#include <flags.h>
#include <oldtools.h>
#include <account.h>
#include "../online/extern.h"
#include "../online/online.h"
#include "../online/userlog.h"
#include "netz.h"
#include "slip.h"

class slipnetlogint:public netlogint
 {
 public:
  bool cslip;

  virtual void getip();
  virtual void handle_local();
  virtual void donetlogin();
  virtual bool verifylogin(const char *password, char *message);
 };

void slipnetlogint::getip()
 {
  if((up|"slipaddr")[0]!=0)
    up.get("slipaddr", ip);
  else if(config.slipaddr!=0)
    makeinternetdotaddress(config.slipaddr, ip);
  else
    snprintf(ip, TEXTLEN, "192.168.21.%u", config.port);
 }

static void doslip(const char *ip, bool cslip)
 {
  if(cslip)
   {
    if(status.online2)
      doextern("/home/dbox/bin/doslip", ip, "-v", "-c");
    else
      doextern("/home/dbox/bin/doslip", ip, "-v");
   }
  else
   {
    if(status.online2)
      doextern("/home/dbox/bin/doslip", ip, "-c");
    else
      doextern("/home/dbox/bin/doslip", ip);
   }
 }

void slipnetlogint::handle_local()
 {
  if(local)
    userlogprintf2(status, up.name, "slipnetlogint::handlelocal",
      "%sSLIP-Login, IP %s", cslip?"C":"", ip);
  else
    logf.printf("slipnetlogint::handlelocal", 
      "%sSLIP-Login, IP %s", cslip?"C":"", ip);

  doslip(ip, cslip);
 }

void slipnetlogint::donetlogin()
 {
  char password[TEXTLEN], message[TEXTLEN];

  login=0;
  init();
  strmaxcpy(protocol, cslip?"cslip":"slip", TEXTLEN-1);
  snprintf(logintype, TEXTLEN, "login type %s", protocol);

  online.flushkeys();
  online.printf("Online-Username:");
  if(readline(&online, loginname, FALSE, FALSE, FALSE, FALSE)) return;

  online.flushkeys();
  online.printf("Online-Passwort:");

  if(readline(&online, password, TRUE, FALSE, FALSE, FALSE)) return;

  if(!verifylogin(password, message))
    return;

  online.printf("\r\n\nName oder Passwort falsch.\r\n");
 }
 
bool slipnetlogint::verifylogin(const char *password, char *message)
 {
  filet infile;

  local=FALSE;
  message[0]=0;
  dnstring(loginname);

  infile.file=open_login();

  check_login1.login=loginname;
  check_login2.protocol=protocol;

  if(infile.file==NULL) // Kompatibilitt
   {
    //strmaxcpy(check_login2.charge_menu, "slip", TEXTLEN-1);
    strmaxcpy(up.name, loginname, TEXTLEN-1);
    strmaxcpy(check_login1.username, up.name, TEXTLEN-1);
    local=TRUE;
   }
  else
   {
    if(check_login1.check_login1(infile.file))
     {
      logf.printf("verifylogin",
                  "%s-login of %s denied by logins-profile",
                  protocol, check_login1.username);

      if(check_login1.errormsg[0]!=0)
       {
        strmaxcpy(message, check_login1.errormsg, TEXTLEN-1);
        logf.printf("verifylogin", "%s", check_login1.errormsg);
       }

      return TRUE;
     }

    strmaxcpy(up.name, check_login1.username, TEXTLEN-1);

    if(up.name[0]==0)
      return TRUE;

    if(check_login1.radius_auth_server[0]!=0)
     {
      if(radiuslogin(password))
        return TRUE;
     }
    else
      local=TRUE;
   }

  if(up.name[0]==0)
    return TRUE;

  if(local)
   {
    if(up.read()) 
     {
      logf.printf("sliplogin", "Username falsch");
      strcpy(message, "account not found");
      return TRUE;
     }

    if(::verifylogin(&up, password))
     {
      userlogprintf2(status, up.name, "sliplogin", "Falsches Passwort!");
      return TRUE;
     }

    strmaxcpy(gp.name, up|"gruppe", TEXTLEN-1);
    gp.read();
    if(!getinternalflag(&up, &gp, LOGIN_SLIP))
     {
      userlogprintf2(status, up.name, "sliplogin",
        "SLIP login denied: no login_slip flag");
      strcpy(message, "access denied");
      return TRUE;
     }

    check_login2.up=&up;
    check_login2.protocol=protocol;

    infile.close();
    infile.file=open_charge();

    check_login1.login=loginname;
    check_login2.protocol=protocol;

    if(infile.file==NULL) // Kompatibilitt
     {
      strmaxcpy(check_login2.charge_menu, "slip", TEXTLEN-1);
     }
    else if(check_login2.check_login2(infile.file))
     {
      logf.printf("verifylogin",
                  "%s-login of %s denied by charge-profile",
                  protocol, check_login2.up->name);

      if(check_login2.errormsg[0]!=0)
       {
        strmaxcpy(message, check_login2.errormsg, TEXTLEN-1);
        logf.printf("verifylogin", "%s", check_login2.errormsg);
       }

      return TRUE;
     }

    if(check_login2.charge_menu[0]!=0)
     {
      if(checkchargemenu(&up, &gp, check_login2.charge_menu, message))
       {
        userlogprintf2(status, up.name, "sliplogin", "%s", message);
        return TRUE;
       }

       {
        char balance_s[TEXTLEN];
        amountt balance=getbalance(&status.userprofile);

        currency_string(balance_s, balance, TRUE);
        online.printf("Ihr Kontostand: %s\r\n", balance_s);
       }
     }
   }

  getip();

  accounting_start();
  handle_local();
  accounting_stop();

  return FALSE;
 }

void sliplogin(const char *loginname)
 {
  slipnetlogint slip;

  slip.cslip=FALSE;
  slip.donetlogin();
 }

void csliplogin(const char *loginname)
 {
  slipnetlogint slip;

  slip.cslip=TRUE;
  slip.donetlogin();
 }

