/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <getopt.h>

#include <dboxpath.h>
#include <userdb.h>
#include <config.h>
#include <charsettools.h>
#include <dboxversion.h>
#include <libdboxversion.h>
#include <portinfo.h>

void printkontext(const char *kontext)
 {
  if(kontext[0]==0)
    printf("  ");
  else
    printf("K ");
 }

void doshowstatus(const char *client, bool verbose)
 {
  int i, j;
  bool loggedin=FALSE;
  userpt up;
  char tempstr[2*TEXTLEN], statusfile[DIRLEN], lastcall[TEXTLEN];
  struct tm *tm;
  portinfot portinfo;

  if(client[0]==0)
    client="localhost.dbox";

  sprintf(statusfile, DBOXCLIENTDIR "portinfo", config.dboxdir, client);
 
  if(access(statusfile, 00)!=0)
   {
    printf("Client nicht gefunden!\n");
    return;
   }

  strmaxcpy(portinfo.filename, statusfile, DIRLEN-1);

  if(verbose)
    printf("Port Login                     "
           "Name                      Anruf       IP\n");
  else
    printf("Port Login                     "
           "Name                      Anruf\n");

  for(i=0; i<MAXPORTS; i++)
   {
    portinfo.read(i);
  
    if(portinfo.info.lastcall==0)
      lastcall[0]=0;
    else
     {
      tm=localtime(&portinfo.info.lastcall);
      sprintf(lastcall, "%02u.%02u %02u:%02u", 
        tm->tm_mday, tm->tm_mon+1, tm->tm_hour, tm->tm_min);
     }

    switch(portinfo.info.status)
     {
     case portinfot::infot::OFFLINE:
      break;
  
     case portinfot::infot::WORKING:
      printf("%3d  DBOX                                                "
             "%s\n", i, lastcall);
      loggedin=TRUE;
      break;
 
     case portinfot::infot::FREE:
      printf("%3d  frei                                                "
             "%s\n", i, lastcall);
      loggedin=TRUE;
      break;
 
     case portinfot::infot::RING:
      printf("%3d  Anruf                                               "
             "%s\n", i, lastcall);
      loggedin=TRUE;
      break;
 
     case portinfot::infot::LOGIN:
      printf("%3d  Login                                               "
             "%s\n", i, lastcall);
      loggedin=TRUE;
      break;

     case portinfot::infot::ERROR:
      printf("%3d  FEHLER                                              "
             "%s\n", i, lastcall);
      loggedin=TRUE;
      break;
 
     case portinfot::infot::ONLINE:
     case portinfot::infot::CHATTING:
     case portinfot::infot::SLIP:
      if(portinfo.info.status==portinfot::infot::SLIP)
        sprintf(tempstr, "SLIP %s", portinfo.info.username);
      else
        strmaxcpy(tempstr, portinfo.info.username, TEXTLEN-1);

      tempstr[20]=0;
      printf("%3d  User %-20s", i, tempstr);
      loggedin=TRUE;

      tempstr[0]=0;
      if(portinfo.info.username[0]!=0 &&
         stricmp(config.boxname, portinfo.info.boxname)==0)
       {
        strmaxcpy(up.name, portinfo.info.username, TEXTLEN-1);
        if(!up.read())
         {
          sprintf(tempstr, "%s %s", up|"vorname", up|"nachname");
          ibm2iso(tempstr, strlen(tempstr));
          tempstr[25]=0;
         }
       }

      printf(" %-25s %-12s", tempstr, lastcall);
      if(verbose)
       {
        if(portinfo.info.status==portinfot::infot::SLIP)
         {
          struct in_addr in;
          in.s_addr=portinfo.info.slipaddr;
          printf("%s", inet_ntoa(in));
         }
       }

      printf("\n");
      break;

     case portinfot::infot::NETCALL:
      j=57-printf("%3d  Netcall %s", i, portinfo.info.username);
      for(; j>0; j--) printf(" ");
      printf("%s\n", lastcall );
      loggedin=TRUE;
      break;
     }
   }

  if(!loggedin)
    printf("\nNiemand eingeloggt!\n");
 }

void doresetport(const char *client, word port)
 {
  char statusfile[DIRLEN];
  //char lastcall[TEXTLEN];
  portinfot portinfo;

  if(client[0]==0)
    client="localhost.dbox";

  sprintf(statusfile, DBOXCLIENTDIR "portinfo", config.dboxdir, client);
 
  if(access(statusfile, 00)!=0)
   {
    printf("Client nicht gefunden!\n");
    return;
   }

  strmaxcpy(portinfo.filename, statusfile, DIRLEN-1);

  portinfo.read(port);
  
  if(portinfo.info.status!=portinfot::infot::OFFLINE)
   {
    portinfo.owninfo.status=portinfot::infot::OFFLINE;
    portinfo.write(port);
   }
 }

static bool parseoptions(int argc, char **argv)
 {
  char client[TEXTLEN]="";
  int resetport=-1;
  bool verbose=FALSE;

 schleife:
  switch(getopt(argc, argv, "?hr:v"))
   {
   case '?':
   case 'h':
    goto help;

   case -1:
    break;

   case 'v':
    verbose=TRUE;
    break;

   case 'r':
    resetport=atoi(optarg);
    goto schleife;

   case ':':
    printf("\n\nParameter fehlt! - ");
    goto aufruf;

   default:
    goto aufruf;
   }

  if((argc-optind)>0)
    strmaxcpy(client, argv[optind], TEXTLEN-1);

  if(resetport>=0)
    doresetport(client, resetport);
  else
    doshowstatus(client, verbose);

  return FALSE;

 aufruf:
  printf("Falscher Aufruf!\n");

 help:
  printf(
    "\n"
    "* * OSTAT - DBox Status Display Module - Version " DBOXVERSIONNR " * *\n"
    "\n"
    "Aufruf:                      Aktion:\r\n"
    "\n"
    " ostat [-?] [-h]              Hilfe ausgeben\r\n"
    " ostat Client                 Status des Clients anzeigen\r\n"
    " ostat                        Boxstatus ausgeben\r\n"
    "\r\n"
    "Weitere Optionen:\r\n"
    " -r Port                      reset port \r\n"
    " -v                           verbose\r\n"
    "\r\n"
   );

  return TRUE;
 }

int main(int argc, char **argv)
 {
  if(init_libdbox(LIBDBOXVERSION, LIBDBOXVERSIONNR) || config.read(""))
    return 1;

  parseoptions(argc, argv);

  return 0;
 }
