/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>

#include <dbox.h>
#include <tools.h>
#include <mtoolzc.h>
#include <spoolp.h>
#include <userdb.h>
#include <logf.h>
#include <config.h>
#include <dboxpath.h>
#include <netdefs.h>
#include <lock.h>
#include <dboxversion.h>
#include "prearc.h"
#include "gsbox.h"

static char gsgate[TEXTLEN]="UUCP", 
            defaultzcdomain[TEXTLEN]="ZER";

int readline(FILE *f, char *s, dword maxlen)
 {
  char ch, *starts=s;

 schleife:
  if(fread(&ch, 1, 1, f)==0) return TRUE;
  if(ch=='\n')
   {
    *s=0;
    while(s!=starts && *(s-1)==' ') *(--s)=0;
    return FALSE;
   }
  else if(ch!='\r')
   {
    if(maxlen!=0)
     {
      maxlen--;
      *(s++)=ch;
     }
   }
  goto schleife;
 }

/*

Results:

0 Ok
1 Rekursiv
2 abgewiesen

*/

byte gsabstozc(char *gsabs, char *abs, userpt *up)
 {
  char *tptr;

  strcpy(abs, gsabs);

  tptr=abs;
  while(*tptr!=0 && *tptr!='%' && *tptr!='@')
   {
    if(*tptr==' ') *tptr='_';
    tptr++;
   }

  tptr=strchr(abs, '@');
  if(tptr!=NULL)
   {
    tptr++;

    if(stricmp(tptr, up->name)!=0)
      sprintf(tptr+strlen(tptr), ".%s", up->name);

    sprintf(tptr+strlen(tptr), ".%s", *up|"primarydomain");
   }

  dnstring(abs);
  return 0;
 }

void gsmidtozc(char *gsmid, char *mid, userpt *up)
 {
  char *tptr, *tptr2;

  strcpy(mid, gsmid);

  tptr=strchr(mid, '@');
  if(tptr!=NULL)
   {
    tptr++;
    tptr2=tptr;

    if(strchr(tptr2, '.')!=NULL) return;

    if(stricmp(tptr, up->name)!=0)
      sprintf(tptr+strlen(tptr), ".%s", *up|"primarydomain");

    sprintf(tptr+strlen(tptr), ".%s", *up|"primarydomain");
   }
  else
   {
    tptr2=mid+strlen(mid);
    sprintf(tptr2, "@%s.%s", up->name, *up|"primarydomain");
    tptr2++;
   }

  dnstring(tptr2);
 }

static bool checklen(const char *len)
 {
  while(*len==' ' || *len=='\t') len++;
  if(!isdigit(*len)) return TRUE;
  len++;
  while(*len!=0)
   {
    if(!isdigit(*len)) return TRUE;
    len++;
   }
  return FALSE;
 }

static int gsbox2zc(userpt *up, FILE *infile, 
                    FILE *outfilemail, FILE *outfilenews)
 {
  char header[8][200], emp[200], abs[200], mid[200], tempstr[300], *tptr;
  int i;
  dword laenge;
  bool ignore=FALSE;
  FILE *outfile;

  for(i=0; i<8; i++)
    if(readline(infile, header[i], 190))
      return i!=0;

  strcpy(emp, header[0]);

  tptr=strchr(emp, '@');
  if(tptr==NULL)
   {
    strcpy(tempstr, emp);
    emp[0]='/';
    strcpy(emp+1, tempstr);
    upstring(emp+1);
    outfile=outfilenews;
   }
  else
   {
    tptr++;
    outfile=outfilemail;

    if(stricmp(tptr, gsgate)==0)
     {
      *(tptr-1)=0;
      tptr=strrchr(emp, '%');
      if(tptr!=0) *tptr='@';
     }

    if(strchr(strchr(emp, '@')+1, '.')==NULL)
      sprintf(emp+strlen(emp), ".%s", defaultzcdomain);

    tptr=emp;
    while(*tptr!=0)
     {
      if(*tptr==' ') *tptr='_';
      tptr++;
     }

    dnstring(emp);
   }

  switch(gsabstozc(header[2], abs, up))
   {
   case 1:
    logf.printf("gsbox2zc", "Warnung: Rekursive Nachricht in \"%s\" ignoriert!", emp);
    ignore=TRUE;
    break;

   case 2:
    /*
    if(strnicmp(emp, NODENYNET, strlen(NODENYNET))!=0)
     {
      logf.printf("gsbox2zc", "Nachricht von \"%s\" an \"%s\" abgewiesen!", abs, emp);
      ignore=TRUE;
     }
    */
    break;
   }

  laenge=atol(header[7]);
  if(checklen(header[7]))
   {
    logf.printf("gsbox2zc", "Pufferdatei defekt!\n");
    return TRUE;
   }

  gsmidtozc(header[5], mid, up);

  if(ignore)
    fseek(infile, laenge, SEEK_CUR);
  else
   {
    fprintf(outfile, "EMP:\t%s\r\n", emp);
    fprintf(outfile, "ABS:\t%s\r\n", abs);
    fprintf(outfile, "EDA:\t19%s00S+2\r\n", header[3]);
    fprintf(outfile, "BET:\t%s\r\n", header[1]);
    fprintf(outfile, "MID:\t%s\r\n", mid);
    if(toupper(header[6][0])=='B')
      fprintf(outfile, "TYP:\tBIN\r\n");
    fprintf(outfile, "LEN:\t%lu\r\n", laenge);
    fprintf(outfile, "ROT:\t%s.%s\r\n", up->name, *up|"primarydomain");
    fprintf(outfile, "GATE:\tGSBOX %s.%s [GS-Mailbox-Gate vom "
                     DBOXVERSION " by Daniel Kroening]\r\n",
                     config.boxname, config.domain);
    fprintf(outfile, "\r\n");

    if(filecopy(infile, outfile, laenge)) return TRUE;
   }

  return FALSE;
 }

bool zcemptogs(char *emp)
 {
  char *tptr1, *tptr2;
  char tempstr[300];

  tptr1=strchr(emp, ' ');
  if(tptr1!=NULL) *tptr1=0;

  if(emp[0]=='/')
   {
    strcpy(tempstr, emp);
    strcpy(emp, tempstr+1);
   }
  else
   {
    tptr1=strchr(emp, '@');
    if(tptr1==NULL) return TRUE;
    tptr2=strchr(tptr1+1, '.');
    if(tptr2!=NULL) *tptr2=0;

    tptr1=emp;
    while(*tptr1!=0 && *tptr1!='%' && *tptr1!='@')
     {
      if(*tptr1=='_') *tptr1=' ';
      tptr1++;
     }
   }

  upstring(emp);
  return FALSE;
 }

static int zctogsbox(userpt *up, FILE *infile, FILE *outfile)
 {
  char emps[20][100], bet[100]="", abs[100]="", wab[100]="",
       eda[30]="", mid[100]="", *emp;

  char tempstr[300], *tptr;

  int binaer=FALSE, anzemp=0, empnr;
  dword laenge=0xffffffff;

 schleife:
  if(readline(infile, tempstr, 100))
    return FALSE;

  if(tempstr[0]!=0)
   {
    tptr=strchr(tempstr, ':');
    if(tptr==NULL) goto error;
    *(tptr++)=0;
    while(*tptr=='\t' || *tptr==' ') tptr++;

    tptr[98]=0;

    if(stricmp(tempstr, "ABS")==0)
      strcpy(abs, tptr);
    else if(stricmp(tempstr, "BET")==0)
      strcpy(bet, tptr);
    else if(stricmp(tempstr, "EDA")==0)
     {
      tptr[28]=0;
      strcpy(eda, tptr);
     }
    else if(stricmp(tempstr, "EMP")==0 && anzemp!=19)
     {
      char *tptr2;
      if((tptr2=strchr(tptr, ' '))!=NULL) *tptr2=0;
      strcpy(emps[anzemp++], tptr);
     }
    else if(stricmp(tempstr, "LEN")==0)
      laenge=atol(tptr);
    else if(stricmp(tempstr, "MID")==0)
      strcpy(mid, tptr);
    else if(stricmp(tempstr, "TYP")==0)
      binaer=TRUE;
    else if(stricmp(tempstr, "WAB")==0)
      strcpy(wab, tptr);
    goto schleife;
   }

  #if 0
  if(wab[0]!=0) strcpy(abs, wab);
  #endif

  if(laenge==0xffffffff || abs[0]==0 || eda[0]==0 || anzemp==0 || mid[0]==0)
   {
   error:
    logf.printf("zctogsbox", "FEHLER: Pufferdatei defekt, die Datei wurde "
                "nicht vollstndig konvertiert.");
    return TRUE;
   }

   {
    char *tptr;

    tptr=strchr(abs, ' ');
    if(tptr!=NULL) *tptr=0;

    tptr=strchr(abs, '@');
    if(tptr!=NULL)
     {
      *tptr='%';
      strcat(abs, "@");
      strcat(abs, gsgate);
     }

    upstring(abs);
   }

   {
    strcpy(tempstr, eda+2);
    tempstr[10]=0;
    strcpy(eda, tempstr);
   }

  for(empnr=0; empnr<anzemp; empnr++)
   {
    if(empnr!=0) fseek(infile, -laenge, SEEK_CUR);
    emp=emps[empnr];

    if(zcemptogs(emp)) goto dump;

    fprintf(outfile, "%s\r\n", emp);
    fprintf(outfile, "%s\r\n", bet);
    fprintf(outfile, "%s\r\n", abs);
    fprintf(outfile, "%s\r\n", eda);
    fprintf(outfile, "%s\r\n", gsgate);
    fprintf(outfile, "%s\r\n", mid);
    fprintf(outfile, "%c\r\n", binaer?'B':'T');
    fprintf(outfile, "%lu\r\n", laenge);

    filecopy(infile, outfile, laenge);

   goto weiter;

  dump:
   logf.printf("zctogsbox", "Nachricht an \"%s\" ignoriert!", emp);
   fseek(infile, laenge, SEEK_CUR);
  weiter:;
   }

  return FALSE;
 }

void gsbox2zc(userpt *up, const char *infilename,
              const char *outfilenamemail,
              const char *outfilenamenews)
 {
  FILE *infile, *outfilemail, *outfilenews;

  infile=fopen(infilename, "rb");
  if(infile==NULL)
   {
    logf.printf("gsbox2zc", "Fatal: Failed to open \"%s\"!", infilename);
    return;
   }

  outfilemail=fopen(outfilenamemail, "ab");
  if(outfilemail==NULL)
   {
    logf.printf("gsbox2zc", "Fatal: Failed to open \"%s\"!", outfilenamemail);
    fclose(infile);
    return;
   }

  outfilenews=fopen(outfilenamenews, "ab");
  if(outfilenews==NULL)
   {
    logf.printf("gsbox2zc", "Fatal: Failed to open \"%s\"!", outfilenamenews);
    fclose(infile);
    fclose(outfilemail);
    return;
   }
 
  while(!feof(infile))
   {
    if(gsbox2zc(up, infile, outfilemail, outfilenews)!=0)
     {
      logf.printf("gsbox2zc", "Fatal: gsbox2zc failed!");
      break;
     }
   }

  fclose(infile);
  fclose(outfilemail);
  fclose(outfilenews);
 }

void zc2gsbox(userpt *up, const char *infilename,
              const char *outfilename)
 {
  FILE *infile, *outfile;

  infile=fopen(infilename, "rb");
  if(infile==NULL)
   {
    logf.printf("zc2gsbox", "Fatal: Failed to open \"%s\"!", infilename);
    return;
   }
 
  outfile=fopen(outfilename, "ab");
  if(outfile==NULL)
   {
    logf.printf("zc2gsbox", "Fatal: Failed to open \"%s\"!", outfilename);
    fclose(infile);
    return;
   }

  while(!feof(infile))
   {
    if(zctogsbox(up, infile, outfile)!=0)
     {
      logf.printf("zc2gsbox", "Fatal: zctogsbox failed!");
      break;
     }
   }

  fclose(infile);
  fclose(outfile);   
 }

void gsbox2zc(userpt *up, const char *dirname)
 {
  DIR *dir;
  struct dirent *ent;
  char tempstr[DIRLEN];

  sprintf(tempstr, "%s.", dirname);

  if((dir=opendir(tempstr))==NULL)
   {
    console.printf("opendir() failed!\r\n");
    logf.printf("gsbox2zc", "Fatal: opendir(\"%s\") failed!", tempstr);
    return;
   }

  while((ent=readdir(dir))!=NULL)
   {
    if(ent->d_name[0]!='.')
     {
      char oldfilename[DIRLEN],
           newfilenamemail[DIRLEN],
           newfilenamenews[DIRLEN];

      sprintf(oldfilename, "%s%s", dirname, ent->d_name);
      sprintf(newfilenamemail, DBOXTEMPDIR "prearc.%d.gsboxmail", config.dboxdir, getpid());
      sprintf(newfilenamenews, DBOXTEMPDIR "prearc.%d.gsboxnews", config.dboxdir, getpid());

      gsbox2zc(up, oldfilename, newfilenamemail, newfilenamenews);

      spool_zc_puffer_mail(newfilenamemail, up->name);
      spool_zc_puffer_news(newfilenamenews, up->name);
     }
   }

  closedir(dir);
 }

void doprearcgsboxunpack(userpt *up)
 {
  DIR *dir;
  struct dirent *ent;
  char dirname[DIRLEN];

  sprintf(dirname, BOXSPOOLDIR ".", config.boxdir, up->name);

  if((dir=opendir(dirname))==NULL)
   {
    console.printf("opendir() failed!\r\n");
    logf.printf("doprearcgsboxunpack", "Fatal: opendir(\"%s\") failed!", dirname);
    return;
   }

  while((ent=readdir(dir))!=NULL)
   {
    if(strnicmp(ent->d_name, "in_", 3)==0)
     {
      char filename[DIRLEN];

      sprintf(filename, BOXSPOOLDIR "%s", 
        config.boxdir, up->name, ent->d_name);

      if(filelength(filename)<10)
       {
        console.printf("Datei zu klein: %s\r\n", filename);
        logf.printf("doprearcgsboxunpack", "Datei zu klein: %s", filename);
        remove(filename);
       }
      else
       {
        char tempdir[DIRLEN];

        snprintf(tempdir, DIRLEN-1, DBOXSPOOLTEMPDIR, config.dboxdir);

        if(unpack(getpacker(up), filename, tempdir))
         {
          logf.printf("doprearcgsboxunpack", "unpack(\"%s\") failed!", filename);
         }
        else
         {
          remove(filename);
          gsbox2zc(up, tempdir);
         }
       }
     }
   }

  closedir(dir);
 }

void doprearcgsboxpack(userpt *up)
 {
  char filename[DIRLEN], tempstr[DIRLEN], directory[DIRLEN];
  dword packetcount;
  byte packer;

  sprintf(directory, BOXSPOOLDIR, config.boxdir, up->name);

  packetcount=time((time_t *)NULL)%100000000;
  packer=getpacker(up);

  if(packer==PACKER_INTERN ||
     packer==PACKER_NONE ||            //TODO!
     packer==PACKER_ZOO ||
     packer==PACKER_ARJ)
    return;

  if(!checkpack(up)) return;

  sprintf(tempstr, BOXSPOOLDIR "%s.%s",
    config.boxdir, up->name, up->name, packerextentions[packer]);

  dword maxsize1;

   {
    char temp[TEXTLEN];

    strmaxcpy(temp, up->get("prearc.maxsize1"), TEXTLEN-1);
    if(temp[0]!=0)
      maxsize1=atol(temp);
    else // TODO: Gruppe mit getuser()
      maxsize1=config.prearc_maxsize1;
   }

  if(filelength(tempstr)>maxsize1) return;

  if(locksystem(up)) return;

  sprintf(tempstr, BOXSPOOLDIR "%s.prv",
    config.boxdir, up->name, up->name);

  if(access(tempstr, 00)==0)
   {
    packetcount++;
    packetcount%=100000000;
    sprintf(filename, BOXSPOOLDIR "%lu.pkt", 
      config.boxdir, up->name, packetcount);

    while(lock("puffer.prv", up->name, config.boxname))
     {
      console.printf("%s is locked... ", up->name);
      sleep(10);
     }

    zc2gsbox(up, tempstr, filename);
    remove(tempstr);

    unlock("puffer.prv", up->name, config.boxname);

    sprintf(tempstr, BOXSPOOLDIR "%s.%s",
      config.boxdir, up->name, up->name, packerextentions[packer]);
    pack(packer, tempstr, filename, directory);
    remove(filename);
   }

  sprintf(tempstr, BOXSPOOLDIR "%s.brt", 
    config.boxdir, up->name, up->name);

  if(access(tempstr, 00)==0)
   {
    packetcount++;
    packetcount%=100000000;
    sprintf(filename, BOXSPOOLDIR "%lu.pkt",
      config.boxdir, up->name, packetcount);
    zc2gsbox(up, tempstr, filename);
    remove(tempstr);

    sprintf(tempstr, BOXSPOOLDIR "%s.%s",
      config.boxdir, up->name, up->name, packerextentions[packer]);
    pack(packer, tempstr, filename, directory);
    remove(filename);
   }

  unlocksystem(up);
 }

void doprearcgsbox(userpt *up)
 {
  doprearcgsboxunpack(up);

  if(!onlyunpack)
    doprearcgsboxpack(up);
 }
