/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>

#include <dbox.h>
#include <tools.h>
#include <mtoolzc.h>
#include <spoolp.h>
#include <userdb.h>
#include <logf.h>
#include <config.h>
#include <dboxpath.h>
#include <cmdline.h>
#include <lock.h>
#include <dboxversion.h>
#include <libdboxversion.h>
#include "prearc.h"
#include "alpha.h"

static void help()
 { 
  fprintf(stderr,
    "\n"
    "* * PREARC - DBOX Module - Version "
    DBOXVERSIONNR " * *\n"
    "\n"
    "Aufruf:                      Aktion:\n"
    "\n"
    " prearc [-?] [-h]             Hilfe ausgeben\n"
    " prearc -d                    Konfiguration anzeigen\n"
    " prearc                       packen & konvertieren\n"
    " prearc Systemname            Nur System \"Systemname\" bearbeiten\n"
    "\n"
    "Weitere Optionen:\n"
    "  -e   nur eingehende Archive bearbeiten\n"
    "  -l   Maxlimale Systemlast\n"
    //"  -i   IBM Zeichensatz\n"
    //"  -d   Debugging on\n"
    "\n"
   );
 }

static void aufruf()
 {
  fprintf(stderr, "Falscher Aufruf!\n");
  help();
 }

static bool parseoptions(int argc, char **argv)
 {
  cmdlinet cmdline;
  const char *boxname, *maxloadarg;
  int i;

  if(cmdline.parse(argc, argv, "?hdb:l:e"))
   {
    aufruf();
    return TRUE;
   }

  if(cmdline.isset('?') ||
     cmdline.isset('h'))
   {
    help();
    return FALSE;
   }

  boxname=cmdline.getval('b');
  if(boxname==NULL) boxname="";

  maxloadarg=cmdline.getval('l');
  if(maxloadarg!=NULL) maxload=atoi(maxloadarg);

  onlyunpack=cmdline.isset('e');

  if(config.read(boxname))
    return TRUE;

   {
    char logfilename[DIRLEN];
    sprintf(logfilename, BOXLOGDIR "prearc.log", config.boxdir);
    logf.setlogfile(logfilename);
   }

  if(cmdline.isset('d'))
   {
    if(cmdline.argcount!=0)
     {
      aufruf();
      return TRUE;
     }
    doshowalphaconfig();
    return FALSE;
   }

  if(cmdline.argcount==0)
    doprearc("");
  else
    for(i=0; i<cmdline.argcount; i++)
      doprearc(cmdline.args[i]);

  return FALSE;
 }

int main(int argc, char **argv)
 {
  if(init_libdbox(LIBDBOXVERSION, LIBDBOXVERSIONNR)) return 1;

  parseoptions(argc, argv);

  return 0;
 }
