/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>

#include <dbox.h>
#include <tools.h>
#include <mtoolzc.h>
#include <spoolp.h>
#include <oldtools.h>
#include <userdb.h>
#include <logf.h>
#include <config.h>
#include <dboxpath.h>
#include <netdefs.h>
#include <lock.h>
#include "prearc.h"
#include "alpha.h"
#include "gsbox.h"

consolet console;
bool onlyunpack=FALSE;
byte maxload=0;

extentiont extentions[]=
 { 
  { "prv", 'P', TRUE  },
  { "brt", 'N', FALSE },
  { (char *)NULL, 0, FALSE }
 };

int getpacker(userpt *up)
 {
  int packer;

  packer=config.netpackerdb.index(*up|"packer");  
  if(packer<0) packer=0;

  return packer;
 }

bool unpack(byte packer, const char *filename, const char *dirname)
 {
  char tempstr[DIRLEN*2];
  bool error=TRUE;

  console.setattr(0xff, TRUE);
  console.printf("Entpacke: %s (%s)\r\n", filename, config.netpackerdb[packer]);
  console.setattr(0xff, FALSE);
  deldir(dirname);

  sprintf(tempstr, "%s.", dirname);
  chdir(tempstr);                    // MSDOS!

  console.close();
  switch(packer)
   {
   case PACKER_ZIP:
   case PACKER_ZIP2:
    sprintf(tempstr, "unzip -jo \"%s\"", filename);  // sicherheitsluecke!
    system(tempstr);
    error=FALSE;
    break;

   case PACKER_LHA:
   case PACKER_LHA2:
    sprintf(tempstr, "lharc efi \"%s\"", filename);  // sicherheitsluecke!
    system(tempstr);
    error=FALSE;
    break;

   case PACKER_ZOO:
    break;

   case PACKER_ARJ:
    break;

   case PACKER_RAR:
    break;

   case PACKER_NONE:
    break;
   }

  console.assign("");
  console.open();

  config.dboxcd();

  return error;
 }

bool pack(byte packer, const char *archive, const char *filename, 
          const char *dirname)
 {
  char tempstr[DIRLEN*2], *tptr;
  bool error=TRUE;

  console.setattr(0xff, TRUE);
  console.printf("Packe: %s (%s)\r\n", filename, config.netpackerdb[packer]);
  console.setattr(0xff, FALSE);

  sprintf(tempstr, "%s.", dirname);
  chdir(tempstr);

  console.close();
  switch(packer)
   {
   case PACKER_ZIP:
    sprintf(tempstr, DBOXBINDIR "oldzip -k -j -9 %s %s",  
      config.dboxdir, archive, filename);
    system(tempstr);
    error=FALSE;
    break;

   case PACKER_ZIP2:
    sprintf(tempstr, DBOXBINDIR "newzip -k -j -9 %s %s",
      config.dboxdir, archive, filename);
    system(tempstr);
    error=FALSE;
    break;

   case PACKER_LHA:
    sprintf(tempstr, "lharc ago %s %s", archive, filename);
    system(tempstr);
    error=FALSE;
    strmaxcpy(tempstr, archive, (DIRLEN*2)-1);
    tptr=strrchr(tempstr, '.');
    if(tptr!=NULL)
     {
      tptr++;
      strcpy(tptr, "bak");
      remove(tempstr);
     }
    break;

   case PACKER_LHA2:
    sprintf(tempstr, "lharc ag %s %s", archive, filename);
    system(tempstr);
    error=FALSE;
    break;

   case PACKER_ZOO:
    break;

   case PACKER_ARJ:
    break;

   case PACKER_RAR:
    break;

   case PACKER_NONE:
    break;
   }

  console.assign("");
  console.open();

  config.dboxcd();

  return error;
 }

void storefiles(const char *systemname, const char *dirname)
 {
  DIR *dir;
  struct dirent *ent;
  char tempstr[DIRLEN];

  sprintf(tempstr, "%s.", dirname);

  if((dir=opendir(tempstr))==NULL)
   {
    console.printf("opendir() failed!\r\n");
    logf.printf("storefiles", "Fatal: opendir(\"%s\") failed!", tempstr);
    return;
   }

  while((ent=readdir(dir))!=NULL)
   {
    if(ent->d_name[0]!='.')
     {
      char filename[DIRLEN];

      sprintf(filename, "%s%s", dirname, ent->d_name);

      spool_zc_puffer(filename, systemname);
     }
   }

  closedir(dir);
 }

void doprearcjanusunpack(userpt *up)
 {
  DIR *dir;
  struct dirent *ent;
  char dirname[DIRLEN];

  sprintf(dirname, BOXSPOOLDIR ".", config.boxdir, up->name);

  if((dir=opendir(dirname))==NULL)
   {
    console.printf("opendir() failed!\r\n");
    logf.printf("doprearcjanusunpack", "Fatal: opendir(\"%s\") failed!", dirname);
    return;
   }

  while((ent=readdir(dir))!=NULL)
   {
    if(strnicmp(ent->d_name, "in_", 3)==0)
     {
      char filename[DIRLEN];

      sprintf(filename, BOXSPOOLDIR "%s", config.boxdir, up->name, ent->d_name);

      if(filelength(filename)<10)
       {
        console.printf("Datei zu klein: %s\r\n", filename);
        logf.printf("doprearcjanusunpack", "Datei zu klein: %s", filename);
        remove(filename);
       }
      else
       {
        char tempdir[DIRLEN];

        snprintf(tempdir, DIRLEN-1, DBOXSPOOLTEMPDIR, config.dboxdir);

        if(unpack(getpacker(up), filename, tempdir))
         {
          logf.printf("doprearcjanusunpack", "unpack(\"%s\") failed!", filename);
         }
        else
         {
          remove(filename);
          storefiles(up->name, tempdir);
         }
       }
     }
   }

  closedir(dir);
 }

bool checkpack(userpt *up) // TRUE=Es gibt was zu packen
 {
  char filename[DIRLEN];
  int e;

  for(e=0; extentions[e].ext!=NULL; e++)
   {
    sprintf(filename, BOXSPOOLDIR "%s.%s", 
      config.boxdir, up->name, up->name, extentions[e].ext);

    if(access(filename, 00)==0) return TRUE;
   }

  return FALSE;
 }

bool locksystem(userpt *up)
 {
  char tempstr[DIRLEN], tempstr2[DIRLEN];
  byte trials=3;

  sprintf(tempstr, BOXSPOOLDIR "unlocked", config.boxdir, up->name);
  sprintf(tempstr2, BOXSPOOLDIR "locked", config.boxdir, up->name);

 schleife:
  if(rename(tempstr, tempstr2)!=0)
   {
    if(access(tempstr, 00)!=0 && access(tempstr2, 00)!=0)
     {
      FILE *f=fopen(tempstr, "wb");
      if(f==NULL) return TRUE;
      fclose(f);
      sleep(1);
      logf.printf("locksystem", "Warnung: Erzeuge Lockfile von %s", up->name);
     }
    trials--;
    if(trials==0) return TRUE;

    sleep(1);
    goto schleife;
   }
  
  return FALSE;
 }

void unlocksystem(userpt *up)
 {
  char tempstr[DIRLEN], tempstr2[DIRLEN];

  sprintf(tempstr, BOXSPOOLDIR "locked", config.boxdir, up->name);
  sprintf(tempstr2, BOXSPOOLDIR "unlocked", config.boxdir, up->name);

  if(rename(tempstr, tempstr2)!=0)
    logf.printf("unlocksystem", "Fatal: rename() failed!");
 }

void doprearcjanuspack(userpt *up, const char *file, 
                       const char *ext, const char *archivfile,
                       const char *directory)
 {
  char filename[DIRLEN], tempstr[DIRLEN];

  num2str(time((time_t *)NULL), tempstr, 7);
  sprintf(filename, BOXSPOOLDIR "%s.%s", 
    config.boxdir, up->name, tempstr, ext);

  while(lock("puffer.prv", up->name, config.boxname))
   {
    console.printf("%s is locked... ", up->name);
    sleep(10);
   }

  rename(file, filename);

  while(unlock("puffer.prv", up->name, config.boxname));

  pack(getpacker(up), archivfile, filename, directory);

  remove(filename);
 }

void doprearcjanuspack(userpt *up)
 {
  char filename[DIRLEN], archivfile[DIRLEN], *extention;
  int e;
  byte packer;

  packer=getpacker(up);

  if(packer==PACKER_INTERN ||
     packer==PACKER_NONE ||            //TODO
     packer==PACKER_ZOO ||
     packer==PACKER_ARJ)
    return;

  if(!checkpack(up)) return;

  if(locksystem(up)) return;

  for(e=0; extentions[e].ext!=NULL; e++)
   {
    extention=extentions[e].ext;
    sprintf(filename, BOXSPOOLDIR "%s.%s", config.boxdir, up->name, up->name, extention);
    sprintf(archivfile, BOXSPOOLDIR "%s.%s", config.boxdir, up->name, up->name, packerextentions[packer]);

    if(access(filename, 00)==0)
     {
      switch(config.nettypedb.index(*up|"netztyp"))
       {
       case -1:
       case NETZ_JANUS:
        dword maxsize1;
        char tempstr[TEXTLEN];

        up->get("prearc.maxsize1", tempstr);
        if(tempstr[0]!=0)
          maxsize1=atol(tempstr);
        else // TODO: Gruppe mit getuser()
          maxsize1=config.prearc_maxsize1;

        if((filelength(archivfile)<maxsize1 ||
           filelength(filename)>maxsize1))
         {
          char directory[DIRLEN];
          sprintf(directory, BOXSPOOLDIR, config.boxdir, up->name);
          doprearcjanuspack(up, filename, extention, archivfile, directory);
         }
        break;

       case NETZ_JANUS2:
        archivfile[0]=0;
          
         {
          DIR *dir;
          dirent *ent;
          char directory[DIRLEN];

          sprintf(directory, BOXSPOOLDIR ".", config.boxdir, up->name);

          dir=opendir(directory);
          if(dir==NULL) 
           {
            logf.printf("doprearcjanuspack", "Fatal: opendir failed!");
            break;
           }

          while((ent=readdir(dir))!=NULL)
           {
            if(ent->d_name[0]==extentions[e].janus2filename &&
               ent->d_name[1]=='P' && ent->d_name[8]=='.' &&
               stricmp(ent->d_name+9, packerextentions[packer])==0)
             {
              dword maxsize2;
              char tempstr[DIRLEN];

              up->get("prearc.maxsize2", tempstr);
              if(tempstr[0]!=0)
                maxsize2=atol(tempstr);
              else // TODO: Gruppe mit getuser()
                maxsize2=config.prearc_maxsize2;

              sprintf(tempstr, BOXSPOOLDIR "%s", 
                config.boxdir, up->name, ent->d_name);

              if(filelength(tempstr)<maxsize2)
               {
                strmaxcpy(archivfile, tempstr, DIRLEN-1);
                break;
               }
             }
           }

          closedir(dir);
         }

        if(archivfile[0]==0)
         {
          dword i=time((time_t *)NULL);

          do
           {
            sprintf(archivfile, BOXSPOOLDIR "%cP%06lu.%s", 
              config.boxdir,
              up->name,
              extentions[e].janus2filename, 
              i%1000000,
              packerextentions[packer]);

            i++;
           }
          while(access(archivfile, 00)==0);
         }
 
         {
          char directory[DIRLEN];

          sprintf(directory, BOXSPOOLDIR, config.boxdir, up->name);
          doprearcjanuspack(up, filename, extention, archivfile, directory);
         }
        break;

       default:;
        // Uh?
       }
     }
   }

  unlocksystem(up);
 }

void doprearcjanus(userpt *up)
 {
  doprearcjanusunpack(up);

  if(!onlyunpack)
    doprearcjanuspack(up);
 }

void doprearcpop3(userpt *up)
 {
  char filename[DIRLEN], command[DIRLEN*2],
       filename2[DIRLEN], tempstr[TEXTLEN];
  FILE *infile, *outfile;

  sprintf(filename, BOXSPOOLDIR "%s.pop",
    config.boxdir, up->name, up->name);

  if(access(filename, 00)==0)
   {
    sprintf(filename2, "%smaps", up->getpath());
    if(access(filename2, 00)==0)
     {
      if((infile=fopen(filename, "rb"))!=NULL)
       {
        sprintf(filename2, BOXSPOOLDIR "%s.prv", 
          config.boxdir, up->name, up->name);

        if((outfile=fopen(filename2, "ab"))!=NULL)
         {
          filecopy(infile, outfile, filelength(fileno(infile)));
          fclose(outfile);
         }
        fclose(infile);
       }
     }

    up->get("disablepop", tempstr);

    if(!atoi(tempstr))
     {
      sprintf(command, DBOXBINDIR "omail -p %s -b %s", 
        config.dboxdir, up->name, config.boxname);

      console.close();
      system(command);
      console.assign("");
      console.open();
     }
    else
      unlink(filename);
   }
 }

void doprearc2(const char *systemname)
 {
  if(onlyunpack)
   {
    char filename[DIRLEN];
    sprintf(filename, BOXTODODIR "in.%s", config.boxdir, systemname);
    if(access(filename, 00)!=0) return;
    remove(filename);
   }

  console.setattr(0xff, TRUE);
  console.printf("Username: %s\r\n", systemname);
  console.setattr(0xff, FALSE);

  userpt up(systemname);
 
  if(up.read())
   {
    console.printf("FATAL: Profile von %s konnte nicht gelesen werden!\r\n", systemname);
    logf.printf("doprearc", "Profile von %s konnte nicht gelesen werden!", systemname);
    return;
   }

  if(!onlyunpack)
    doprearcpop3(&up);

  switch(config.nettypedb.index(up|"netztyp"))
   {
   case -1:
   case NETZ_JANUS:
   case NETZ_JANUS2:
   case NETZ_ZCONNECT:
    doprearcjanus(&up);
    break;

   case NETZ_Z38:
    break;

   case NETZ_UUCP:
    break;

   case NETZ_ALPHABOX:
    doprearcalpha(&up);
    break;

   case NETZ_GSBOX:
    doprearcgsbox(&up);
    break;

   case NETZ_FIDO:
    break;

   case NETZ_POP3:
    break;
   }
 }

void domaxload()
 {
  time_t t, starttime;
  char ch;
 
  time(&starttime);

  console.printf("\r\nPrearc wegen zu hoher "
                 "Systemlast angehalten.\r\n\r\n");

  do               
   {               
    ch=upcase(lo(console.doreadkey()));
    time(&t);      
   }               
  while(ch!=' ' && (t-starttime)<10);

  if(ch==' ') maxload=0;
 }

void doprearc(const char *systemname)
 {
  DIR *dir;
  struct dirent *ent;
  char dirname[DIRLEN];
  time_t lastt=0, t=0;

  console.assign("");
  console.open();

  if(systemname[0]!=0)
   {
    doprearc2(systemname);
    console.close();
    return;
   }

  sprintf(dirname, BOXUSERDIR ".", config.boxdir, ".");

  if((dir=opendir(dirname))==NULL)
   {
    printf("opendir() failed!\n");
    return;
   }

  while((ent=readdir(dir))!=NULL && console.readkey()==0)
   {
    if(ent->d_name[0]!='.' && ent->d_name[0]!='_' &&
       strcmp(ent->d_name, "lost+found")!=0)
     {
      char systemname[TEXTLEN];
      strmaxcpy(systemname, ent->d_name, TEXTLEN-1);
      doprearc2(systemname);
      time(&t);
      if((t-lastt)>5)
       {
        lastt=t;
        if(checkload(maxload))
          domaxload();
       }
     }
   }

  console.close();

  closedir(dir);
 }
