/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>

#include <config.h>
#include <userdb.h>
#include <dboxpath.h>
#include <logf.h>
#include "mtoolzc.h"
#include "stool.h"
#include "mroute.h"
#include "logmail.h"

static const char *strendistr(const char *s1, const char *s2)
 {
  int l1, l2;
  const char *tptr;

  l1=strlen(s1);
  l2=strlen(s2);

  if(l2>l1) return NULL;

  tptr=s1+l1-l2;

  if(stricmp(tptr, s2)!=0) return NULL;

  return tptr;
 }

#define MAXABSMASKLEN 100000

bool checkabsmask(const char *absmask, const char *abs)
 {
  char tempstr[MAXABSMASKLEN*2], *tptr, *absm;

  strmaxcpy(tempstr, absmask, MAXABSMASKLEN-1);
  absm=tempstr;

  while(absm[0]!=0)
   {
    tptr=absm;
    while(*tptr!=0 && *tptr!=';' && *tptr!=',' && *tptr!=' ')
      tptr++;

    if(*tptr!=0)
     {
      *tptr=0;
      tptr++;
      while(*tptr==';' || *tptr==',' || *tptr==' ')
        tptr++;
     }

    while(*absm==' ') absm++;
    if(checkjoker(absm, abs, TRUE)) return TRUE;
    absm=tptr;
   }

  return FALSE;
 }

bool checksourcemask(const char *sourcemask, const char *source)
 {
  char tempstr[MAXABSMASKLEN*2], *tptr, *sourcem;

  strmaxcpy(tempstr, sourcemask, MAXABSMASKLEN-1);
  sourcem=tempstr;

  while(sourcem[0]!=0)
   {
    tptr=sourcem;
    while(*tptr!=0 && *tptr!=';' && *tptr!=',' && *tptr!=' ')
      tptr++;

    if(*tptr!=0)
     {
      *tptr=0;
      tptr++;
      while(*tptr==';' || *tptr==',' || *tptr==' ')
        tptr++;
     }

    while(*sourcem==' ') sourcem++;
    if(checkjoker(sourcem, source, TRUE)) return TRUE;
    sourcem=tptr;
   }

  return FALSE;
 }

bool mailroute1(char *dest, const char *boxname, dword *maxlen)
 {
  char systemname[TEXTLEN], tempstr[TEXTLEN*2], *domain, *tptr;

  strmaxcpy(systemname, boxname, TEXTLEN-1);
  tptr=strrchr(systemname, '.');
  snprintf(tempstr, TEXTLEN, "%s-", config.boxname);

  if(tptr!=NULL && strnicmp(boxname, tempstr, strlen(tempstr))==0 &&
     stricmp(tptr, ".zer")==0)
   {
    *tptr=0;
    strmaxcpy(tempstr, systemname+strlen(tempstr), TEXTLEN-1);
    strmaxcpy(systemname, tempstr, TEXTLEN-1);

    userpt up(systemname);
    if(up.read()) return TRUE;

    if(atoi(up|"system")) return TRUE;
    *maxlen=0;

    strmaxcpy(dest, up.name, TEXTLEN-1);

    return FALSE;
   }

  domain=strchr(systemname, '.');

  if(domain==NULL)
    domain="zer";
  else
    *(domain++)=0;

  userpt up(systemname);
  if(!up.read())
   {
    up.get("routing", tempstr);
    if(atoi(tempstr)) return TRUE;
    if(checkdomain(domain, up|"domains")) return TRUE;
    *maxlen=atol(up|"maxlen");
    strmaxcpy(dest, up.name, TEXTLEN-1);
    return FALSE;
   }

  return TRUE;
 }

bool mailroute2(char *dest, const char *source,
                const char *boxname,
                dword *maxlen, const char *abs)
 {
  FILE *infiles[100];
  int includestack=0;
  char kennung[500]="", text[500];
  char absmask[MAXABSMASKLEN]="*", domainserver[TEXTLEN]="",
       sourcemask[MAXABSMASKLEN]="*";
  bool ende=FALSE;
  char routingfilename[DIRLEN];
  char lastkennung[TEXTLEN];

  snprintf(routingfilename, DIRLEN, BOXSYSTEMDIR "routing", config.boxdir);

  infiles[0]=fopen(routingfilename, "rt");
  if(infiles[0]==NULL) return TRUE;

 schleife:
  if(kennung[0]!=0 && stricmp(kennung, "include")!=0)
    strmaxcpy(lastkennung, kennung, TEXTLEN-1);

  switch(readconfigline(infiles[includestack], kennung, text, 499))
   {
   case 0:
    if(stricmp(kennung, "absmask")==0)
     {
      if(stricmp(lastkennung, "absmask")==0 && 
         strlen(absmask)+strlen(text)<(MAXABSMASKLEN-2))
        sprintf(absmask+strlen(absmask), " %s", text); // see if
      else
        strmaxcpy(absmask, text, MAXABSMASKLEN-1);
     }
    else if(stricmp(kennung, "sourcemask")==0)
     {
      if(stricmp(lastkennung, "sourcemask")==0 && 
         strlen(sourcemask)+strlen(text)<(MAXABSMASKLEN-2))
        sprintf(sourcemask+strlen(sourcemask), " %s", text); // see if
      else
        strmaxcpy(sourcemask, text, MAXABSMASKLEN-1);
     }
    else if(stricmp(kennung, "domainserver")==0)
      strmaxcpy(domainserver, text, TEXTLEN-1);
    else if(stricmp(kennung, "maxlen")==0)
      *maxlen=atol(text);
    else if(stricmp(kennung, "stop")==0)
     {
     }
    else if(stricmp(kennung, "domain")==0)
     {
      if(checkabsmask(absmask, abs) &&
         checksourcemask(sourcemask, source))
       {
        if(strcmp(text, ".")==0 || strcmp(text, "*")==0)
         {
          ende=TRUE;
          strmaxcpy(dest, domainserver, TEXTLEN-1);
         }
        else if(text[0]=='.')
         {
          if(strendistr(boxname, text)!=NULL)
           {
            ende=TRUE;
            strmaxcpy(dest, domainserver, TEXTLEN-1);
           }
         }
        else
         {
          if(stricmp(boxname, text)==0)
           {
            ende=TRUE;
            strmaxcpy(dest, domainserver, TEXTLEN-1);
           }
         }
       }
     }
    else if(stricmp(kennung, "include")==0)
     {
      includestack++;
      text[TEXTLEN]=0;
    
      if(text[0]==DIRCHAR[0])
        strmaxcpy(routingfilename, text, DIRLEN-1);
      else
        snprintf(routingfilename, DIRLEN, BOXSYSTEMDIR "%s",
                 config.boxdir, text);

      infiles[includestack]=fopen(routingfilename, "rt");
      if(infiles[includestack]==NULL) includestack--;
     }
    else
     {
      logf.printf("resolvesystem2",
        "Unbekannter Eintrag \"%s\" in der Routingtabelle",
        kennung);
      ende=TRUE;
     }

    if(ende) break;

    goto schleife;

   case 1:
    if(includestack>0)
     {
      fclose(infiles[includestack]);
      includestack--;
      goto schleife;
     }
    break;

   default:
    logf.printf("resolvesystem2", "Fehler in der Routingfile");
   }

  for(int i=0; i<=includestack; i++)
    fclose(infiles[i]);

  userpt up(dest);
  if(up.read())
   {
    logf.printf("mailroute2", "No domainserver %s", up.name);
    return TRUE;  
   }

  return FALSE;
 }

bool mailroute(char *dest, const char *source, 
               const char *boxname,
               dword *maxlen, 
               const char *abs)
 {
  dest[0]=0;

  if(!mailroute1(dest, boxname, maxlen))
    return FALSE;

  if(!mailroute2(dest, source, boxname, maxlen, abs))
    return FALSE;
 
  return TRUE;
 }

void logmail(const char *source, const char *dest, 
             zcmessaget *m, const char *emp)
 {
  FILE *outfilefrom, *outfileto;
  char tempstr[DIRLEN];

  snprintf(tempstr, DIRLEN, BOXUSERDIR "mail.log",
    config.boxdir, dest);

  outfileto=fopen(tempstr, "at");
  if(outfileto==NULL)
   {
    logf.printf("logmail", "Failed to open \"%s\"", tempstr);
    return;
   }

  snprintf(tempstr, DIRLEN, BOXUSERDIR "mail.log",
    config.boxdir, source);

  outfilefrom=fopen(tempstr, "at");
  if(outfilefrom==NULL)
   {
    logf.printf("logmail", "Failed to open \"%s\"", tempstr);
    fclose(outfileto);
    return;
   }

  logmail(outfilefrom, outfileto, source, dest, m, emp);

  fclose(outfileto);
  fclose(outfilefrom);
 }

bool checkstop(const char *abs, const char *emp)
 {
  #if 0
  FILE *infile;
  char kennung[500], text[500];
  char absmask[TEXTLEN]="*";
  bool ende=FALSE, erg=FALSE;
  char routingfilename[DIRLEN];

  snprintf(routingfilename, DIRLEN, BOXSYSTEMDIR "routing", config.boxdir);
  infile=fopen(routingfilename, "rt");
  if(infile==NULL) return FALSE;

 schleife:
  switch(readconfigline(infile, kennung, text, 499))
   {
   case 0:
    if(stricmp(kennung, "absmask")==0)
      strmaxcpy(absmask, text, TEXTLEN-1);
    else if(stricmp(kennung, "stop")==0)
     {
      if(checkabsmask(absmask, abs))
       {
        if(text[0]=='.')
         {
          if(strendistr(emp, text)!=NULL)
            ende=erg=TRUE;
         }
        else
         {
          if(checkabsmask(text, emp))
            ende=erg=TRUE;
         }
       }
     }

    if(ende) break;
    goto schleife;

   case 1:
    break;

   default:
    logf.printf("checkstop", "Fehler in der Routingfile");
   }

  fclose(infile);

  return erg;
  #endif

  return FALSE;
 }

