/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <stdio.h>
#include <time.h>
#include <sys/stat.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <unistd.h>

#include <dbox.h>
#include <tools.h>
#include <dboxpath.h>
#include <config.h>
#include "msgid.h"

#define NIL 0xffffffff

class msgidbaset msgidbase;

bool msgidbaset::open()
 {
  int indexfd;
  char filename[DIRLEN];
  dword indexsize;

  snprintf(filename, DIRLEN, DBOXSYSTEMDIR "msgid.idx", config.dboxdir);

  if((indexfd=::open(filename, O_RDWR|O_CREAT|O_BINARY, S_IREAD|S_IWRITE))<0)
    return TRUE;

  indexsize=filelength(indexfd);

  if(indexsize==0)
   {
    dword pos[1024];
    int i;

    memset(pos, 0xff, sizeof pos);

    for(i=0; i<(1l<<(HASH_BITS-10)); i++)
      write(indexfd, pos, sizeof pos);

    lseek(indexfd, 0, SEEK_SET);
   }
  else if(indexsize!=HASH_SIZE)
   {
    return TRUE;
   }

  if((index=(dword *)malloc(HASH_SIZE))==NULL) return TRUE;
  read(indexfd, index, HASH_SIZE);
  ::close(indexfd);

  snprintf(filename, DIRLEN, DBOXSYSTEMDIR "msgid", config.dboxdir);

  datafd=::open(filename, O_RDWR|O_CREAT|O_BINARY, S_IREAD|S_IWRITE);
  if(datafd<0) return TRUE;

  return FALSE;
 }

bool msgidbaset::close()
 {
  char filename[DIRLEN];

  snprintf(filename, DIRLEN, DBOXSYSTEMDIR "msgid.idx", config.dboxdir);
  indexfd=::open(filename, O_RDWR|O_CREAT|O_BINARY, S_IREAD|S_IWRITE);
  write(indexfd, index, HASH_SIZE);
  free(index);
  ::close(indexfd);

  return ::close(datafd)!=0;
 }

bool msgidbaset::add(char *msgid)
 {
  dword crc, pos, len;

  if(msgid[0]==0) return TRUE;

  crc=str2crc(msgid);
  pos=getindex(crc);
  len=strlen(msgid)+1;
  if(pos==NIL)
   {
    pos=lseek(datafd, 0, SEEK_END);
    setindex(crc, pos);
   }
  else
   {
   schleife1:
    lseek(datafd, pos, SEEK_SET);
    read(datafd, &dataentry, sizeof dataentry);
    if(len==dataentry.len)
     {
      dword i=0;
      char ch;

     schleife2:
      read(datafd, &ch, 1);
      if(ch==msgid[i])
       {
        i++;
        if(i==len) return TRUE;
        goto schleife2;
       }
     }
    if(dataentry.next!=NIL)
     {
      pos=dataentry.next;
      goto schleife1;
     }

    dataentry.next=lseek(datafd, 0, SEEK_END);
    lseek(datafd, pos, SEEK_SET);
    write(datafd, &dataentry, sizeof dataentry);
    lseek(datafd, 0, SEEK_END);
   }
  dataentry.len=len;
  dataentry.next=NIL;
  write(datafd, &dataentry, sizeof dataentry);
  write(datafd, msgid, len);
  return FALSE;
 }

bool msgidbaset::check()
 {
  dword datasize;
  word i;
  dword pos;

  datasize=filelength(datafd);

  i=0;
  do
   {
    pos=getindex(i);
    if(pos!=0xffffffff && pos>datasize-5)
      return TRUE;

    i++;
   }
  while(i!=0);

  return FALSE;
 }

