/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <dirent.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <time.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/mman.h>

#include <dboxpath.h>
#include <tools.h>
#include <mtoolzc.h>
#include <stool.h>
#include <lock.h>
#include <logf.h>
#include <stoolzc.h>
#include <junkerr.h>
#include <sfolderzc.h>
#include "logmail.h"
#include "mroute.h"
#include "junkzcm.h"
#include "spool.h"
#include "spoolzcm.h"
#include "sdisp.h"
#include "deliverzc.h"

bool user_is_box(const char *username)
 {
  char filename[DIRLEN];
  
  snprintf(filename, DIRLEN, 
          "%s" DIRCHAR "%s" DIRCHAR "system" DIRCHAR ".",
          config.dboxdir, username);

  if(access(filename, 00)==0)
   {
    snprintf(filename, DIRLEN, 
             "%s" DIRCHAR "%s" DIRCHAR "user" DIRCHAR ".",
             config.dboxdir, username);

    if(access(filename, 00)==0)
      return FALSE;
   }

  return TRUE;
 }

class spoolzcmt
 {
 public:
  struct destusert
   {
    char username[TEXTLEN];
    byte emps[MAXEMPS];
   } *destuser;

  dword anzdestuser;
  userpt up;
  const char *alias;
  zcmessaget m;

  bool checkpointusername(const char *username);
  junkerrort checkpointabs(const char *abs);
  junkerrort checkabswab(const char *abs, const char *wab);
  junkerrort checkabswab();
  void docopymail(dword anzemps);
  void addemp(dword emp_nr, dword anzemps, const char *zielsystem);
  void doemp(dword emp_nr, dword anzemps,
             const char *zielsystem, const char *emp);
  void spoolmail(dword anzemps);
  void spoolposting(dword pos);
  bool spoolfile(int handle);
  void spoolfile(const char *filename);
  void storepop(const char *username, const char *emp);
  void posteb(const char *emp);
  void dospooloutgoingmail(const char *emp, 
       const char *envelope_to, const char *outfilename);

  spoolzcmt();
  ~spoolzcmt();
 };

spoolzcmt::spoolzcmt()
 {
  alias=NULL;
 }

spoolzcmt::~spoolzcmt()
 {
  if(alias!=NULL) free((void *)alias);
 }

bool spoolzcmt::checkpointusername(const char *username)
 {
  const char *tptr;

  if(stricmp(username, up.name)==0) return FALSE;

  if(alias!=NULL)
   {
    tptr=alias;

    while(tptr[0]!=0)
     {
      if(stricmp(username, tptr)==0) return FALSE;
      tptr+=strlen(tptr)+1;
     }
   }

  return TRUE;

  // TODO 

  return FALSE;
 }

junkerrort spoolzcmt::checkpointabs(const char *abs)
 {
  char host[TEXTLEN], fqdnhost[TEXTLEN],
       username[TEXTLEN*2], domain[TEXTLEN], *tptr;

  // Teil nach dem @ in fqdnhost speichern
  // Teil vor dem @ in username speichern

  strmaxcpy(username, abs, (TEXTLEN*2)-1);
  tptr=strchr(username, '@');
  if(tptr==NULL) return JERR_INVALID_ABS_SYNTAX;
  *(tptr++)=0;
  strmaxcpy(fqdnhost, tptr, TEXTLEN-1);

  // Realname entfernen

  tptr=strchr(fqdnhost, ' ');
  if(tptr!=NULL) *tptr=0;

  // Domain berechnen
  // Site-Name isolieren in host

  strmaxcpy(host, fqdnhost, TEXTLEN-1);
  tptr=strchr(host, '.');
  if(tptr==NULL) return JERR_INVALID_ABS_SYNTAX;
  *(tptr++)=0;
  strmaxcpy(domain, tptr, TEXTLEN-1);

  // Domain mit Domainliste vergleichen

  if(stricmp(host, up.name)==0 &&
     !checkdomain(domain, up|"domains"))
    return JERR_NO_ERROR;

  // Username testen

  if(checkpointusername(username))
    return JERR_INVALID_ABS_USERNAME;

  // Hauptboxname vergleichen

  if(stricmp(host, config.boxname)==0)
   {
    // TODO: Domain (domain) berprfen
    return JERR_NO_ERROR;
   }

  // Abshost vergleichen

  if(config.abs_host[0]!=0 && 
     stricmp(fqdnhost, config.abs_host)==0)
   {
    return JERR_NO_ERROR;
   }

  // TODO: Gruppen / User abs_host testen

  return JERR_INVALID_ABS_HOST;
 }

junkerrort spoolzcmt::checkabswab(const char *abs, const char *wab)
 {
  if(checkabscharset(abs, atoi(up|"system")))
    return JERR_INVALID_ABS_SYNTAX;

  if(wab[0]!=0)
    if(checkabscharset(wab, atoi(up|"system"))) 
      return JERR_INVALID_ABS_SYNTAX;

  if(!atoi(up|"system"))
   {
    junkerrort error;

    if(wab[0]!=0)
      error=checkpointabs(wab);
    else
      error=checkpointabs(abs);

    if(error!=JERR_NO_ERROR)
      return error;
   }
     
  return JERR_NO_ERROR;
 }

junkerrort spoolzcmt::checkabswab()
 {
  char abs[1000], wab[1000];

  m.copyheadertext(abs, m.abs, 999);

  if(m.wab==NULL)
    wab[0]=0;
  else
    m.copyheadertext(wab, m.wab, 999);

  return checkabswab(abs, wab);
 }

bool iszer(const char *emp)
 {
  while(*emp!=0 && *emp!='@')
    emp++;

  while(*emp!=0)
   {
    if(emp[0]=='.' &&
       tolower(emp[1])=='z' &&
       tolower(emp[2])=='e' &&
       tolower(emp[3])=='r' &&
       (emp[4]==' ' || emp[4]==0))
      return TRUE;

    if(emp[0]==' ') return FALSE;

    emp++;
   }

  return FALSE;
 }

void spoolzcmt::docopymail(dword anzemps)
 {
  dword systemnr;
  char rot[TEXTLEN];

  snprintf(rot, TEXTLEN, "%s.%s", config.boxname, config.domain);

  for(systemnr=0; systemnr<anzdestuser; systemnr++)
   {
    filet outfile;
    char filename[DIRLEN];

    if(!user_is_box(destuser[systemnr].username) &&
       strcasecmp(config.boxname, destuser[systemnr].username)!=0)
     {
      char tmp_boxdir[DIRLEN];

      snprintf(tmp_boxdir, DIRLEN, DBOXBOXDIR, config.dboxdir, destuser[systemnr].username);

      snprintf(filename, DIRLEN, BOXSPOOLINZCMAILDIR "%s.prv", 
              tmp_boxdir, config.boxname);

      display.versenden(destuser[systemnr].username, "", "inbound");
     }
    else
     {
      snprintf(filename, DIRLEN, BOXSPOOLDIR "%s.prv", 
              config.boxdir, destuser[systemnr].username, 
              destuser[systemnr].username);

      display.versenden(config.boxname, destuser[systemnr].username, "Puffer");
     }

    while(lock("puffer.prv", destuser[systemnr].username, config.boxname))
     {
      display.console.printf("%s is locked... ",
         destuser[systemnr].username);
      sleep(10);
     }

    if(outfile.open(filename, "ab"))
     {
      logf.printf("docopymail", "Fatal: ffnen von der Datendatei "
                  "von %s fehlgeschlagen (\"%s\")!", 
                  destuser[systemnr].username, filename);
     }
    else
     {
      m.copyheader_special(outfile.file, destuser[systemnr].emps, rot, TRUE);
      m.copybody(outfile.file);
     }

    unlock("puffer.prv", destuser[systemnr].username, config.boxname);
   }
 }

void spoolzcmt::addemp(dword emp_nr, dword anzemps, 
                       const char *zielsystem)
 {
  dword i, j;

  for(i=0; i<anzdestuser; i++)
    if(strcmp(destuser[i].username, zielsystem)==0)
      break;

  if(i==anzdestuser)
   {
    anzdestuser++;

    for(j=0; j<anzemps; j++)
      destuser[i].emps[j]=m.stat_nokop?DOSKIP:DOKOP;
   }

  strmaxcpy(destuser[i].username, zielsystem, TEXTLEN-1);

  destuser[i].emps[emp_nr]=DOEMP;
 }

void spoolzcmt::posteb(const char *emp)
 {
  // TODO
 }

void spoolzcmt::dospooloutgoingmail(const char *emp, 
  const char *envelope_to, const char *outfilename)
 {
  FILE *outfile;
  const char *d=m.data, *kennung, *inhalt;
  char tempstr[DIRLEN], envelope_header[TEXTLEN*2];
  int i;

  if(envelope_to!=NULL)
    snprintf(envelope_header, TEXTLEN*2, "X-%s-Envelope-To:\t%s\r\n",
      X_DBOX, envelope_to);
  else
    envelope_header[0]=0;

  if(outfilename!=NULL)
    strmaxcpy(tempstr, outfilename, DIRLEN-1);
  else
   {
    i=0;
    do
     {
      snprintf(tempstr, DIRLEN, BOXSPOOLINZCMAILDIR "%s.%d", 
        config.boxdir, config.boxname, i);
      i++;
     }
    while(access(tempstr, 00)==0);
   }

  outfile=fopen(tempstr, "ab");
  if(outfile==NULL)
   {
    logf.printf("dospooloutgoingmail", "Fatal: Failed to open spool");
    return;
   }

  fprintf(outfile, "%s", envelope_header);

  while(TRUE)
   {
    if(m.parseline(&d, &kennung, &inhalt))
     {
      logf.printf("dospooloutgoingmail", "Fatal: Pufferfehler");
      return;
     }

    if(kennung==NULL)
     {
      fprintf(outfile, "EMP:\t%s\r\n", emp);

      fprintf(outfile, "\r\n");
      m.copybody(outfile);
      fclose(outfile);
      return;
     }

    m.copyheaderid(tempstr, kennung, 100);
    if(stricmp(tempstr, "EMP")==0)
     {
      fprintf(outfile, "OEM:\t");
      fwrite(inhalt, 1, d-inhalt, outfile);
     }
    else
     {
      fwrite(kennung, 1, d-kennung, outfile);
     }
   }
 }

void spoolzcmt::doemp(dword emp_nr, dword anzemps, 
                      const char *zielsystem, const char *emp)
 {
  bool pop=FALSE, puf=FALSE;
  char filename[DIRLEN], tempstr[TEXTLEN*2];
  userpt destup;

  if(stricmp(zielsystem, config.boxname)==0)
   {
    char username[TEXTLEN], *tptr;

    strmaxcpy(username, emp, TEXTLEN-1);
    tptr=strchr(username, '@');
    if(tptr!=NULL) *tptr=0;
    dnstring(username);

    if(strchr(username, '/')!=NULL)
     {
      snprintf(tempstr, TEXTLEN, "Username %s invalid", username);
      junkposting(&m, &up, JERR_UNKNOWN_EMP, tempstr, emp);
      return;
     }

    strmaxcpy(destup.name, username, TEXTLEN-1);
    if(destup.read())
     {
      snprintf(tempstr, TEXTLEN, "User %s unbekannt", destup.name);
      junkposting(&m, &up, JERR_UNKNOWN_EMP, tempstr, emp);
      return;
     }

    if((destup|"alias")[0]!=0 &&
       !atoi(destup|"account"))
     {
      strmaxcpy(destup.name, destup|"alias", TEXTLEN-1);
      if(destup.read())
       {
        snprintf(tempstr, TEXTLEN, "alias %s unknown", up.name);
        junkposting(&m, &up, JERR_UNKNOWN_EMP, tempstr, emp);
        return;
       }
     }

    if((destup|"vertretung")[0]!=0)
     {
      char newemp[1000], *tptr;
      
      destup.get("vertretung", newemp);
      tptr=strchr(newemp, '@');
      if(tptr==NULL)
        logf.printf("doemp", "Illegaler Vertreter bei User %s", destup.name);
      else
       {
        if(strchr(tptr+1, '.')==NULL) strcat(newemp, ".zer");
        display.versenden(config.boxname, destup.name, "Vertretung");
        dospooloutgoingmail(newemp, emp, NULL);
       }
     }

    destup.get("disablepop", tempstr);
    if(!atoi(tempstr)) pop=TRUE;

    snprintf(filename, DIRLEN, "%smaps", destup.getpath());
    if(access(filename, 00)==0 && !atoi(destup|"system"))
      puf=TRUE;
   }
  else
   {
    strmaxcpy(destup.name, zielsystem, TEXTLEN-1);
    if(destup.read())
      logf.internalerror("doemp", "FATAL: zielsystem nicht gefunden!");

    destup.get("disablepop", tempstr);

    if(!atoi(destup|"system") && !atoi(tempstr) && !iszer(emp))
      pop=TRUE;

    snprintf(filename, DIRLEN, "%smaps", destup.getpath());
    if(access(filename, 00)==0 || atoi(destup|"system"))
      puf=TRUE;
   }

  if(pop)
   {
    display.versenden(config.boxname, destup.name, "mailbox");
    #if 0
    spoolinfolder(m, destup.name, FALSE, emp);
    #else
    deliver_mail(m, destup, emp);
    #endif
    posteb(emp);
   }

  if(puf)
    addemp(emp_nr, anzemps, destup.name);

  logmail(up.name, destup.name, &m, emp);
 }

void spoolzcmt::spoolmail(dword anzemps)
 {
  char tempstr[1000], junkemp[TEXTLEN];

   {
    const char *dataptr, *inhalt, *boxname;
    dword count=0;

    dataptr=m.data;
    while(!m.findheader(&dataptr, "EMP", &inhalt) && count!=MAXEMPS)
     {
      m.copyheadertext(tempstr, inhalt, 999);
      strmaxcpy(junkemp, tempstr, TEXTLEN-1);
      dnstring(tempstr);

      if(!isnewsemp(tempstr))
       {
        char *tptr=strstr(tempstr, " (");
        if(tptr!=NULL) *tptr=0;

        boxname=strrchr(tempstr, '@');
        if(boxname==NULL || strchr(tempstr, ' ')!=NULL ||
           strchr(boxname, '.')==NULL)
         {
          junkposting(&m, &up, JERR_INVALID_EMP, 
            "EMP-Formfehler", junkemp);
         }
        else
         {
          char abs[1000];

          boxname++;

          if(m.wab==NULL)
            m.copyheadertext(abs, m.abs, 999);
          else
            m.copyheadertext(abs, m.wab, 999);
 
          tptr=strchr(abs, ' ');
          if(tptr!=NULL) *tptr=0;

          if(checkstop(abs, tempstr))
           {
            junkposting(&m, &up, JERR_INVALID_EMP, 
              "E-Mail nicht erwnscht", junkemp);
           }
          else
           {
            char tempstr2[1000];
            char zielsystem[TEXTLEN];
            dword maxlen;
   
            if(mailroute(zielsystem, up.name, boxname, &maxlen, abs))
             {
              snprintf(tempstr2, TEXTLEN, "No route to %s", boxname);
              junkposting(&m, &up, JERR_INVALID_SYSTEM, 
                 tempstr2, junkemp);
             }
            else
             {
              if(m.len>maxlen && maxlen!=0)
               {
                snprintf(tempstr2, TEXTLEN, "Mail larger than %lu bytes", maxlen);
                junkposting(&m, &up, JERR_MAIL_TOO_BIG, 
                  tempstr2, junkemp);
               }
              else
                doemp(count, anzemps, zielsystem, tempstr);
             }
           }
         }
       }

      count++;
     }

    if(count!=anzemps)
      logf.internalerror("spoolnews", "count!=anzemps");
   }
 }

void spoolzcmt::spoolposting(dword pos)
 {
  bool news, mail;
  dword anzemps;
  char tempstr[1000];
  junkerrort error;

  display.startmail(config.boxname, up.name, &m, pos);

  checkmailnews(&m, &mail, &news, &anzemps);

  if(anzemps>MAXEMPS)
   {
    anzemps=MAXEMPS;
    logf.printf("spoolposting", "Warnung: Zu viele EMPs!");
   }

  if(!mail && !news) // Kein EMP!
   {
    junkposting(&m, &up, JERR_NO_EMP, "", "");
    return;
   }

  if(m.abs==NULL)
   {
    junkposting(&m, &up, JERR_NO_ABS, "", "");
    return;
   }

  if(m.bet==NULL)
   {
    junkposting(&m, &up, JERR_NO_BET, "", "");
    return;
   }

  if(m.mid==NULL)
   {
    junkposting(&m, &up, JERR_NO_MID, "", "");
    return;
   }

  if(m.eda==NULL)
   {
    junkposting(&m, &up, JERR_NO_EDA, "", "");
    return;
   }

  m.copyheadertext(tempstr, m.mid, 999);
  if(checkmid(tempstr, atoi(up|"system")))
   {
    junkposting(&m, &up, JERR_INVALID_MID, "", "");
   }
  else 
   {
    error=checkabswab();

    if(error!=JERR_NO_ERROR)
     {
      junkposting(&m, &up, error, "", "");
     }
    else
     {
      destuser=(destusert *)malloc(sizeof(destusert)*anzemps);
      if(destuser==NULL)
        logf.internalerror("spoolposting", "FATAL: malloc failed!");

      anzdestuser=0;

      if(mail)
        spoolmail(anzemps);

      if(news)
        logf.printf("spoolposting", "FATAL: spoolzcm got news!");

      docopymail(anzemps);

      free(destuser);
     }
   }
 }

bool spoolzcmt::spoolfile(int handle)
 {
  dword laenge;
  char *posting=NULL;
  bool error=FALSE, mapped=FALSE;
  time_t t, lastcheck=0;

  laenge=filelength(handle);

  logf.printf("spoolfile", "Verteile %lu Bytes von System "
    "%s@%s", laenge, up.name, config.boxname);

  if(laenge<=2) return FALSE;

  if(laenge>60000)
   {
    posting=(char *)mmap(NULL, laenge, PROT_READ, MAP_FILE|MAP_PRIVATE, handle, 0);
    mapped=TRUE;
    if(posting==NULL)
     {
      logf.printf("spoolfile", "Fatal: mmap failed!");
      return TRUE;
     }
   }

  if(posting==NULL)
   {
    posting=(char *)malloc(laenge);
    if(posting==NULL) return TRUE;

    if(read(handle, posting, laenge)!=(ssize_t)laenge)
      return TRUE;
   }

  m.data=posting;
  m.endoffile=posting+laenge;

  while(!m.eof() && !error)
   {
    if(display.checkabort())
     {
      error=TRUE;
      logf.printf("spoolfile", "Einsortieren wurde manuell abgebrochen!");
     }
    else if((time(&t)-lastcheck)>10)
     {
      lastcheck=t;
      if(checkload(maxload))
       {
        if(display.waitload())
          maxload=0;
       }
     }
    else
     {
      if(m.parse())
        error=TRUE;
      else
       {
        spoolposting(m.data-posting);
        m.data=m.endofmail;
       }
     }
   }

  if(mapped)
    munmap(posting, laenge);
  else
    free(posting);

  return error;
 }

void spoolzcmt::spoolfile(const char *filename)
 {
  char *tptr, tempstr[DIRLEN];
  int handle;
  bool error;

  strmaxcpy(tempstr, filename, DIRLEN-1);
  tptr=strrchr(tempstr, '.');
  *tptr=0;
  strmaxcpy(up.name, tempstr, TEXTLEN-1);

  if(up.read())
   {
    logf.printf("spoolfile", "Fatal: Unbekannter User %s@%s!", 
      up.name, config.boxname);
    movetobad(TRUE, filename, config.boxdir);
    return;
   }

  snprintf(tempstr, DIRLEN, "%salias", up.getpath());
  alias=readtextconfigfile(tempstr);

  snprintf(tempstr, DIRLEN, BOXSPOOLINZCMAILDIR "%s", 
    config.boxdir, filename);

  handle=open(tempstr, O_RDONLY|O_BINARY);

  if(handle<0)
   {
    logf.printf("spoolfile", "Fatal: Failed to open \"%s\"!", tempstr);
    movetobad(TRUE, filename, config.boxdir);
    return;
   }

  error=spoolfile(handle);

  close(handle);

  if(error)
    movetobad(TRUE, filename, config.boxdir);
  else
   {
    logf.printf("spoolfile", "Daten verteilt");

    snprintf(tempstr, DIRLEN, BOXSPOOLINZCMAILDIR "%s",
      config.boxdir, filename);

    if(remove(tempstr)!=0) 
      logf.printf("spoolfile", "remove(\"%s\") failed!", tempstr);
   }
 }

void getfilename(char *filename)
 {
  DIR *dir;
  struct dirent *ent;
  char tempstr[DIRLEN];

  filename[0]=0;

  snprintf(tempstr, DIRLEN, BOXSPOOLINZCMAILDIR ".", config.boxdir);
  if((dir=opendir(tempstr))==NULL)
   {
    logf.printf("dospool", "Fatal: Failed to open SPOOLINDIR!");
    return;
   }

  while((ent=readdir(dir))!=NULL)
    if(ent->d_name[0]!='.' && ent->d_name[0]!='_')
     {
      if(strchr(ent->d_name, '.')!=NULL)
       {
        strmaxcpy(filename, ent->d_name, DIRLEN-1);
        closedir(dir);
        return;
       }
     }

  closedir(dir);
 }

void dospoolzcm()
 {
  char filename[DIRLEN];
  bool ende=FALSE, verteilt=FALSE;

  printf("Starte Einsortiervorgang\n");

  display.open();

  do
   {
    getfilename(filename);
    if(filename[0]==0)
      ende=TRUE;
    else
     {
      spoolzcmt spoolzcm;

      display.console.printf("\r\nVerteile Datei \"%s\" von Box %s...\r\n", 
        filename, config.boxname);

      spoolzcm.spoolfile(filename);

      verteilt=TRUE;
     }
   }
  while(!ende);

  display.close();
  printf("\n");

  if(!verteilt)
   {
    logf.timefprintf(stdout);
    printf("Nichts zu verteilen!\n");
   }
 }

void dotest()
 {
  while(TRUE)
   {
    char tempstr[1000];

    printf("\nTestmodus:\n");
    printf("1: resolvesystem\n");
    printf("Q: Quit\n");
    printf("\nEingabe: ");

    fgets(tempstr, 90, stdin);
    switch(upcase(tempstr[0]))
     {
     case '1':
       {
        char abs[1000], sys[1000], *tptr;

        printf("\nSystemname: ");
        fgets(sys, TEXTLEN-1, stdin);
        tptr=strchr(sys, '\r'); if(tptr!=NULL) *tptr=0;
        tptr=strchr(sys, '\n'); if(tptr!=NULL) *tptr=0;
        if(sys[0]==0) break;

        printf("Absender: ");
        fgets(abs, TEXTLEN-1, stdin);
        tptr=strchr(abs, '\r'); if(tptr!=NULL) *tptr=0;
        tptr=strchr(abs, '\n'); if(tptr!=NULL) *tptr=0;
        if(abs[0]==0) break;

        #if 0
        s=resolvesystem(source, sys, &maxlen, abs);
        if(s==NULL)
          printf("\nNULL!\n");
        else
          printf("\nSystemname: %s\n"
                 "Maxlen: %lu\n", s->name, maxlen);
        #endif
       }
      break;

     case 'Q':
      return;
     }
   }
 }
