/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <dirent.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <time.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/mman.h>

#include <dbox.h>
#include <tools.h>
#include <dboxpath.h>
#include <logf.h>
#include <mtoolzc.h>
#include <stool.h>
#include <stoolzc.h>
#include <config.h>
#include <profile.h>
#include <config.h>
#include <junkerr.h>
#include "spool.h"
#include "spoolzcn.h"
#include "msgid.h"
#include "junkzcn.h"
#include "dowrite.h"
#include "sdisp.h"
#include <oldtools.h>
#include <userdb.h>

checkmapst checkmaps;
dword anzsystems=0;
byte anzboxen=0;
systemt *systems;
mailboxt *boxen;

void getuserflags(pt *profile, userflagst *userflags)
 {
  int i;

  clear(userflags);

  for(i=0; i<profile->size(); i++)
   {
    const char *id=profile->data[i].id;

    if(strnicmp(id, "userflag.", strlen("userflag."))==0 &&
       atoi(profile->data[i].data)!=0)
     {
      const char *tptr=id+strlen("userflag.");
      int flagnr=profile->pconfig->userflagdb.index(tptr);

      if(flagnr>=0)
        (*userflags)[flagnr]=1;
     }
   }
 }

bool checkmid(const char *mid, systemt *s)
 {
  if(s==NULL || !s->point)
   {
    if(strchr(mid, '@')==NULL) return TRUE;
    if(mid[0]==0) return TRUE;

    while(*mid!=0)
     {
      if(*mid>=0 && *mid<=' ') return TRUE;
      if(*mid<0) return TRUE;
      mid++;
     }

    return FALSE;
   }

  // TODO

  return FALSE;
 }

bool checkpointusername(systemt *s, const char *username)
 {
  char *tptr;

  if(stricmp(username, s->name)==0) return FALSE;

  if(s->alias!=NULL)
   {
    tptr=s->alias;

    while(tptr[0]!=0)
     {
      if(stricmp(username, tptr)==0) return FALSE;
      tptr+=strlen(tptr)+1;
     }
   }

  return TRUE;
 }

junkerrort checkpointabs(systemt *s, const char *abs)
 {
  char host[TEXTLEN], fqdnhost[TEXTLEN], username[TEXTLEN*2],
       domain[TEXTLEN], *tptr;

  // Teil nach dem @ in fqdnhost speichern
  // Teil vor dem @ in username speichern

  strmaxcpy(username, abs, (TEXTLEN*2)-1);
  tptr=strchr(username, '@');
  if(tptr==NULL) return JERR_INVALID_ABS_SYNTAX;
  *(tptr++)=0;
  strmaxcpy(fqdnhost, tptr, TEXTLEN-1);

  // Realname entfernen

  tptr=strchr(fqdnhost, ' ');
  if(tptr!=NULL) *tptr=0;

  // Domain berechnen
  // Site-Name isolieren in host

  strmaxcpy(host, fqdnhost, TEXTLEN-1);
  tptr=strchr(host, '.');
  if(tptr==NULL) return JERR_INVALID_ABS_SYNTAX;
  *(tptr++)=0;
  strmaxcpy(domain, tptr, TEXTLEN-1);

  // Domain mit Domainliste vergleichen

  if(stricmp(host, s->name)==0 &&
     !checkdomain(domain, s->domains))
    return JERR_NO_ERROR;

  // Username testen

  if(checkpointusername(s, username))
    return JERR_INVALID_ABS_USERNAME;

  // Hauptboxname vergleichen

  if(stricmp(host, boxen[s->box].name)==0)
   {
    // TODO: Domain (domain) berprfen
    return JERR_NO_ERROR;
   }

  // Abshost vergleichen

  if(boxen[s->box].config.abs_host[0]!=0 && 
     stricmp(fqdnhost, boxen[s->box].config.abs_host)==0)
   {
    return JERR_NO_ERROR;
   }

  // TODO: Gruppen / User abs_host testen

  return JERR_INVALID_ABS_HOST;
 }

junkerrort checkabswab(systemt *s, const char *abs, const char *wab)
 {
  if(checkabscharset(abs, !s->point)) return JERR_INVALID_ABS_SYNTAX;

  if(wab[0]!=0)
    if(checkabscharset(wab, !s->point)) return JERR_INVALID_ABS_SYNTAX;

  if(s->point && boxen[s->box].config.spoolzcn_checkabs)
   {
    junkerrort error;

    if(wab[0]!=0)
      error=checkpointabs(s, wab);
    else
      error=checkpointabs(s, abs);

    if(error!=JERR_NO_ERROR)
      return error;
   }
     
  return JERR_NO_ERROR;
 }

junkerrort checkabswab(systemt *s, zcmessaget *m)
 {
  char abs[1000], wab[1000];

  m->copyheadertext(abs, m->abs, 999);

  if(m->wab==NULL)
    wab[0]=0;
  else
    m->copyheadertext(wab, m->wab, 999);

  return checkabswab(s, abs, wab);
 }

void allocempflags(dword anzemps)
 {
  dword i, j;

  for(i=0; i<anzsystems; i++)
   {
    systems[i].emps=(byte *)malloc(anzemps*sizeof(byte));

    if(systems[i].emps==NULL)
      logf.internalerror("allocempflags", "Zu wenig RAM!");

    for(j=0; j<anzemps; j++)
      systems[i].emps[j]=DOKOP;
   }
 }

void freeempflags()
 {
  dword i;

  for(i=0; i<anzsystems; i++)
    free(systems[i].emps);
 }

void docopymail(systemt *s, zcmessaget *m, dword anzemps)
 {
  dword systemnr, empnr;
  char rot[TEXTLEN];

  snprintf(rot, TEXTLEN, "%s.%s", config.boxname, config.domain);

  for(systemnr=0; systemnr<anzsystems; systemnr++)
   {
    bool docopy=FALSE;

    for(empnr=0; empnr<anzemps && !docopy; empnr++)
      if(systems[systemnr].emps[empnr]==DOEMP)
        docopy=TRUE;

    if(docopy)
     {
      FILE *outfile;

      display.versenden(boxen[systems[systemnr].box].name,
                        systems[systemnr].name, "");

      outfile=dowrite.openfile(systems+systemnr);

      if(outfile==NULL)
       {
        logf.printf("docopymail", "Fatal: ffnen von der Datendatei "
                   "von %s (%s) fehlgeschlagen!", 
                   systems[systemnr].name, 
                   boxen[systems[systemnr].box].name);
       }
      else
       {
        m->copyheader_special(outfile, systems[systemnr].emps, rot, FALSE);
        m->copybody(outfile);
        dowrite.closefile(outfile);
       }
     }
   }
 }

systemt *findsystem(const char *systemname, byte box)
 {
  dword i;

  for(i=0; i<anzsystems; i++)
    if(strcmp(systemname, systems[i].name)==0 && systems[i].box==box)
      return systems+i;

  return NULL;
 }


dword checkmapst::hashfkt(const char *emp)
 {
  /*
  dword val;
  dword len=strlen(emp), i;

  val=emp[0]-96;

  for(i=1; i<len; i++)
   {
    val=(val<<5)+(emp[i]-96)%101;
   }

  return val&HASHMASK;*/

  return str2crc(emp)&HASHMASK;
 }

bool checkmapst::check(systemt *s, const char *emp)
 {
  dword hashval;
  hashentryt *h;
  dword systemnr=s-systems;

  hashval=hashfkt(emp);

  if(hashval>=HASHSIZE || systemnr>=anzsystems || systems>s) 
    logf.internalerror("checkmapst::check", "assertion failed");

  h=hashtable[hashval];

  while(h!=NULL)
   {
    if(strcmp(emp, h->emp)==0)
      return h->bestellt[systemnr];

    h=h->nexthash;
   }

  return FALSE;
 }

void checkmapst::add(systemt *s, const char *emp)
 {
  dword hashval;
  hashentryt *h;
  dword systemnr=s-systems;

  hashval=hashfkt(emp);
  if(hashval>=HASHSIZE || systemnr>=anzsystems || systems>s) 
    logf.internalerror("checkmapst::add", "assertion failed");

  h=hashtable[hashval];

  while(h!=NULL)
   {
    if(strcmp(emp, h->emp)==0)
     {
      h->bestellt[systemnr]=TRUE;
      return;
     }

    h=h->nexthash;
   }

  h=new hashentryt;
  if(h==NULL)
    logf.internalerror("checkmapst::add", "new failed");

  h->n=list;
  h->nexthash=NULL;
  h->emp=strdup(emp);
  h->bestellt=(bool *)malloc(anzsystems*sizeof(bool));
  if(h->emp==NULL || h->bestellt==NULL) 
    logf.internalerror("checkmapst::add", "malloc failed");

  memset(h->bestellt, FALSE, anzsystems);
  h->bestellt[systemnr]=TRUE;
  list=h;

  if(hashtable[hashval]==NULL)
    hashtable[hashval]=h;
  else
   {
    h->nexthash=hashtable[hashval];
    hashtable[hashval]=h;
   }

  anzentries++;
 }

void checkmapst::freeall()
 {
  hashentryt *h;

  while(list!=NULL)
   {
    free(list->emp);
    free(list->bestellt);
    h=list;
    list=list->n;
    delete h;
   }

  anzentries=0;
 }

checkmapst::checkmapst()
 {
  list=NULL;
  anzentries=0;

  for(dword i=0; i<HASHSIZE; i++)
    hashtable[i]=NULL;
 }

static bool checkfeed(const char *feed, const char *emp) // TRUE=bestellt
 {
  if(feed==NULL) return FALSE;

  while(feed[0]!=0)
   {
    if(feed[0]=='!')
     {
      if(checkjoker(feed+1, emp, TRUE))
        return FALSE;
     }
    else
     {
      if(checkjoker(feed, emp, TRUE))
        return TRUE;
     }

    feed=feed+strlen(feed)+1;
   }

  return FALSE;
 }

static bool checkemp(systemt *s, const char *emp) // TRUE=bestellt
 {
  if(checkfeed(s->feed, emp)) return TRUE;
  if(checkmaps.check(s, emp)) return TRUE;

  return FALSE;
 }

static bool checkuserflags(systemt *s, userflagst *userflags)
 {
  int i;
  bool hasflag;

  for(i=0; i<MAXUSERFLAGS; i++)
   {
    hasflag=(s->userflags[i] ||
             boxen[s->box].gruppen[s->gruppe].userflags[i]);

    switch((*userflags)[i])
     {
     case 0:
      break;

     case 1:
      if(!hasflag) return TRUE;
      break;

     case 2:
      if(hasflag) return TRUE;
      break;
     }
   }

  return FALSE;
 }

static bool verifyemp(systemt *s, const char *emp) // TRUE=Fehler
 {
  char *tptr;

  if(s==NULL) return FALSE;

  if(s->gruppe<0) return TRUE;

  tptr=boxen[s->box].gruppen[s->gruppe].bretter;
  if(tptr==NULL) return TRUE;

  while(tptr[0]!=0)
   {
    bool read, write, sysop;
    userflagst userflags;
    char empmask[TEXTLEN];
    userflagdbt olduserflags;

    if(stricmp(config.boxname, boxen[s->box].name)!=0)
     {
      olduserflags=config.userflagdb;
      config.userflagdb=boxen[s->box].config.userflagdb;
     }

    if(!getpermissions(tptr, empmask, &read, &write, &sysop, &userflags))
     {
      if(!checkuserflags(s, &userflags))
       {
        if(checkjoker(empmask, emp, TRUE))
         {
          if(stricmp(config.boxname, boxen[s->box].name)!=0)
            config.userflagdb=olduserflags;

          return !write;
         }
       }
     }

    if(stricmp(config.boxname, boxen[s->box].name)!=0)
      config.userflagdb=olduserflags;

    tptr=tptr+strlen(tptr)+1;
   }

  return TRUE;
 }

void spoolnews(zcmessaget *m, systemt *s, dword anzemps)
 {
  char tempstr[1000], systemlogfile[DIRLEN]="";
  dword i;

  for(i=0; i<anzsystems; i++)
    systems[i].got_the_article=FALSE;

  if(s!=NULL)
    snprintf(systemlogfile, DIRLEN, BOXUSERDIR "logfile", 
      boxen[s->box].config.boxdir, s->name);

   {
    bool zugriff_verweigert=FALSE;
    const char *dataptr, *inhalt;

    dataptr=m->data;
    while(!m->findheader(&dataptr, "EMP", &inhalt))
     {
      m->copyheadertext(tempstr, inhalt, 999);
      dnstring(tempstr);

      if(isnewsemp(tempstr))
       {
        if(verifyemp(s, tempstr))
         {
          upstring(tempstr);
          junkposting(m, JERR_ACCESS_DENIED, 
            "Schreibzugriff verweigert", s, tempstr);
          zugriff_verweigert=TRUE;
          return;
         }
       }
     }
   }

  #ifdef MSGIDCHECK
  m->copyheadertext(tempstr, m->mid, 999);

  if(msgidbase.add(tempstr))
   {
    char logfile[DIRLEN];

    snprintf(logfile, DIRLEN, BOXLOGDIR "news.log", 
      boxen[s->box].config.boxdir);

    logf.fileprintf(logfile, "spoolnews",
                   "Rekursion System %s ID <%s>", s->name, tempstr);

    logf.fileprintf(systemlogfile, "spoolnews", "Rekursion ID <%s>",
                   tempstr);

    printf("Rekursion.\r\n");
    return;
   }
  #endif

   {
    dword count=0, systemnr;
    bool verschickt=FALSE;
    const char *dataptr, *inhalt;

    dataptr=m->data;
    while(!m->findheader(&dataptr, "EMP", &inhalt) && count!=MAXEMPS)
     {
      m->copyheadertext(tempstr, inhalt, 999);
      dnstring(tempstr);

      if(isnewsemp(tempstr))
       {
        for(systemnr=0; systemnr<anzsystems; systemnr++)
          if(s!=(systems+systemnr))
           {
            if(checkemp(systems+systemnr, tempstr))
             {
              systems[systemnr].emps[count]=DOEMP;
              verschickt=TRUE;
              systems[systemnr].got_the_article=TRUE;
             }
            else if(systems[systemnr].allemps)
              systems[systemnr].emps[count]=ISNEWSEMP;
            else
              systems[systemnr].emps[count]=DOKOP;
           }
       }

      count++;
     }

    if(count!=anzemps)
      logf.internalerror("spoolnews", "count!=anzemps");

    if(verschickt)
     {
      for(systemnr=0; systemnr<anzsystems; systemnr++)
        if(systems[systemnr].allemps)
         {
          for(count=0; count<anzemps; count++)
            if(systems[systemnr].emps[count]==ISNEWSEMP)
              systems[systemnr].emps[count]=
                systems[systemnr].got_the_article?DOEMP:DOKOP;
         }
     }
    else
     {
      if(!s->point)
       {
       }
      else
        junkposting(m, JERR_UNKNOWN_EMP, 
          "Kein passendes Brett gefunden", s, "");
     }
   }
 }

void spoolposting(zcmessaget *m, systemt *s, dword pos)
 {
  bool news, mail;
  dword anzemps;
  char tempstr[1000];
  junkerrort error;

  display.startmail(boxen[s->box].name, s->name, m, pos);

  checkmailnews(m, &mail, &news, &anzemps);

  if(anzemps>MAXEMPS)
   {
    anzemps=MAXEMPS;
    logf.printf("spoolposting", "Warnung: Zu viele EMPs!");
   }

  if(!mail && !news) // Kein EMP!
   {
    junkposting(m, JERR_NO_EMP, "", s, "");
    return;
   }

  if(m->abs==NULL)
   {
    junkposting(m, JERR_NO_ABS, "", s, "");
    return;
   }

  if(m->bet==NULL)
   {
    junkposting(m, JERR_NO_BET, "", s, "");
    return;
   }

  if(m->mid==NULL)
   {
    junkposting(m, JERR_NO_MID, "", s, "");
    return;
   }

  if(m->eda==NULL)
   {
    junkposting(m, JERR_NO_EDA, "", s, "");
    return;
   }

  allocempflags(anzemps);

  m->copyheadertext(tempstr, m->mid, 999);
  if(checkmid(tempstr, s))
   {
    junkposting(m, JERR_INVALID_MID, "", s, "");
   }
  else 
   {
    error=checkabswab(s, m);

    if(error!=JERR_NO_ERROR)
     {
      junkposting(m, error, "", s, "");
     }
    else
     {
      if(mail)
        logf.printf("spoolposting", "FATAL: spoolzcn got mail!");

      if(news) spoolnews(m, s, anzemps);

      docopymail(s, m, anzemps);
     }
   }

  freeempflags();
 }

bool spoolfile(int handle, systemt *s)
 {
  zcmessaget m;
  dword laenge;
  char *posting=NULL;
  bool error=FALSE, mapped=FALSE;
  time_t t, lastcheck=0;

  laenge=filelength(handle);

  logf.printf("spoolfile", "Verteile %lu Bytes von System "
    "\"%s\" (%s)", laenge, s->name, boxen[s->box].name);

  if(laenge<=2) return FALSE;

  dowrite.open();

  if(laenge>60000)
   {
    posting=(char *)mmap(NULL, laenge, PROT_READ, MAP_FILE|MAP_PRIVATE, handle, 0);
    mapped=TRUE;
    if(posting==NULL)
     {
      logf.printf("spoolfile", "Fatal: mmap failed!");
      return TRUE;
     }
   }

  if(posting==NULL)
   {
    posting=(char *)malloc(laenge);
    if(posting==NULL) return TRUE;

    if(read(handle, posting, laenge)!=(signed long)laenge)
     {
      return TRUE;
     }
   }

  m.data=posting;
  m.endoffile=posting+laenge;

  while(!m.eof() && !error)
   {
    if(display.checkabort())
     {
      error=TRUE;
      logf.printf("spoolfile", "Einsortieren wurde manuell abgebrochen!");
     }
    else if((time(&t)-lastcheck)>10)
     {
      lastcheck=t;
      if(checkload(maxload))
       {
        if(display.waitload())
          maxload=0;
       }
     }
    else
     {
      if(m.parse())
        error=TRUE;
      else
       {
        spoolposting(&m, s, m.data-posting);
        m.data=m.endofmail;
       }
     }
   }

  if(mapped)
    munmap(posting, laenge);
  else
    free(posting);

  dowrite.close();

  return error;
 }

void spoolfile(const char *filename, byte box)
 {
  char system[TEXTLEN]="", *tptr, tempstr[DIRLEN];
  int handle;
  bool error;
  systemt *s;

  strmaxcpy(tempstr, filename, DIRLEN-1);
  tptr=strrchr(tempstr, '.');
  *tptr=0;
  strmaxcpy(system, tempstr, TEXTLEN-1);
  if((s=findsystem(system, box))==NULL)
   {
    logf.printf("spoolfile", "Fatal: Unbekanntes System %s (%s)!", 
      system, boxen[box].name);
    movetobad(FALSE, filename, boxen[box].config.boxdir);
    return;
   }

  snprintf(tempstr, DIRLEN, BOXSPOOLINZCNEWSDIR "%s", 
    boxen[box].config.boxdir, filename);

  handle=open(tempstr, O_RDONLY|O_BINARY);

  if(handle<0)
   {
    logf.printf("spoolfile", "Fatal: Failed to open \"%s\"!", tempstr);
    movetobad(FALSE, filename, boxen[box].config.boxdir);
    return;
   }

  error=spoolfile(handle, s);

  close(handle);

  if(error)
    movetobad(FALSE, filename, boxen[box].config.boxdir);
  else
   {
    logf.printf("spoolfile", "Daten verteilt");

    snprintf(tempstr, DIRLEN, BOXSPOOLINZCNEWSDIR "%s",
      boxen[box].config.boxdir, filename);

    if(remove(tempstr)!=0) 
      logf.printf("spoolfile", "remove(\"%s\") failed!", tempstr);
   }
 }

void getfilename(char *filename, byte box) 
 {
  DIR *dir;
  struct dirent *ent;
  char tempstr[DIRLEN];

  filename[0]=0;

  snprintf(tempstr, DIRLEN, BOXSPOOLINZCNEWSDIR ".", 
    boxen[box].config.boxdir);

  if((dir=opendir(tempstr))==NULL)
   {
    logf.printf("getfilename", "Fatal: Failed to open SPOOLINDIR!");
    return;
   }

  while((ent=readdir(dir))!=NULL)
    if(ent->d_name[0]!='.' && ent->d_name[0]!='_')
     {
      if(strchr(ent->d_name, '.')!=NULL)
       {
        strmaxcpy(filename, ent->d_name, DIRLEN-1);
        closedir(dir);
        return;
       }
     }

  closedir(dir);
 }

void processmapsfile(FILE *infile, systemt *s)
 {
  char tempstr[1000], *tptr;

  while(fgets(tempstr, 999, infile)!=NULL)
   {
    tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;
    tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;

    if(tempstr[0]=='/')
     {
      dnstring(tempstr);
      checkmaps.add(s, tempstr);
     }
   }
 }

bool getsystemdata(systemt *s, byte box)
 {
  char tempstr[DIRLEN];
  FILE *infile;
  userpt up(s->name);

  up.pconfig=&boxen[box].config;
  if(up.read()) return TRUE;

  if((up|"alias")[0]!=0) return TRUE;

  s->domains[0]=0;
  s->maxlen=0;
  s->emps=NULL;
  s->feed=NULL;
  s->alias=NULL;
  s->allemps=atoi(up|"allemps");
  s->box=box;
  s->user=0;
  s->isbox=FALSE;
  s->gruppe=boxen[box].config.groupdb.index(up|"gruppe");
  s->point=!atoi(up|"system");
  getuserflags(&up, &s->userflags);

  up.get("routing", tempstr);
  s->routing=atoi(tempstr);

  up.get("maxlen", tempstr);
  s->maxlen=atol(tempstr);

  up.get("domains", s->domains);

  #if 0
  for(i=0; i<anzboxen; i++)
    if(box!=i && stricmp(s->name, boxen[i].name)==0)
      s->isbox=TRUE;
  #endif

  snprintf(tempstr, DIRLEN, "%sfeed", up.getpath());
  s->feed=readtextconfigfile(tempstr);

  snprintf(tempstr, DIRLEN, "%salias", up.getpath());
  s->alias=readtextconfigfile(tempstr);

  snprintf(tempstr, DIRLEN, "%smaps", up.getpath());

  if((infile=fopen(tempstr, "rt"))!=NULL)
   {
    processmapsfile(infile, s);
    fclose(infile);
   }

  return FALSE;
 }

void getsystemlist()
 {
  DIR *dir;
  dword count=0;
  char tempstr[DIRLEN];
  byte box;
  struct dirent *ent;

  for(box=0; box<anzboxen; box++)
   {
    snprintf(tempstr, DIRLEN, BOXUSERDIR ".",
      boxen[box].config.boxdir, ".");

    if((dir=opendir(tempstr))!=NULL)
     {
      while((ent=readdir(dir))!=NULL)
        if(ent->d_name[0]!='.' && ent->d_name[0]!='_')
          count++;

      closedir(dir);
     }
   }

  anzsystems=count;
  systems=(systemt *)malloc(sizeof(systemt)*anzsystems);

  if(systems==NULL)
    logf.internalerror("getsystemlist", "Zu wenig RAM!");

  count=0;

  for(box=0; box<anzboxen; box++)
   {
    snprintf(tempstr, DIRLEN, BOXUSERDIR ".", 
      boxen[box].config.boxdir, ".");

    if((dir=opendir(tempstr))==NULL)
      logf.internalerror("getsystemlist", "opendir");
    else
     {
      while((ent=readdir(dir))!=NULL)
       {
        if(ent->d_name[0]!='.' && ent->d_name[0]!='_')
         {
          strmaxcpy(systems[count].name, ent->d_name, TEXTLEN-1);

          if(!getsystemdata(systems+count, box))
            count++;
         }
       }
      closedir(dir);
     }
   }

  if(anzsystems!=count)
   {
    systems=(systemt *)realloc(systems, count*sizeof(systemt));
    anzsystems=count;
   }
 }

void freesystemlist()
 {
  dword i;

  for(i=0; i<anzsystems; i++)
   {
    if(systems[i].feed!=NULL)
      free(systems[i].feed);

    if(systems[i].alias!=NULL)
      free(systems[i].alias);
   }

  free(systems);

  checkmaps.freeall();
 }

void getpermissions()
 {
  for(byte box=0; box<anzboxen; box++)
    for(int i=0; i<MAXGRUPPEN; i++)
     {
      boxen[box].gruppen[i].bretter=NULL;
       
      clear(&boxen[box].gruppen[i].userflags);

      if(boxen[box].config.groupdb[i]!=NULL)
       {
        char tempstr[DIRLEN];

        snprintf(tempstr, DIRLEN, BOXGRUPPENDIR "bretter",
          boxen[box].config.boxdir, boxen[box].config.groupdb[i]);
        boxen[box].gruppen[i].bretter=readtextconfigfile(tempstr);

        grouppt gp(boxen[box].config.groupdb[i]);
        gp.pconfig=&boxen[box].config;
        if(!gp.read())
          getuserflags(&gp, &boxen[box].gruppen[i].userflags);
       }
     }
 }

void freepermissions()
 {
  dword i;
  byte box;

  for(box=0; box<anzboxen; box++)
    for(i=0; i<MAXGRUPPEN; i++)
      if(boxen[box].gruppen[i].bretter!=NULL)
        free(boxen[box].gruppen[i].bretter);
 }

void getanzboxen()
 {
  DIR *dir;
  struct dirent *ent;
  char tempstr[DIRLEN], filename[DIRLEN];
  
  anzboxen=0;
  snprintf(tempstr, DIRLEN, "%s" DIRCHAR ".", config.dboxdir);
  if((dir=opendir(tempstr))==NULL)
   {
    logf.printf("getanzboxen", "Fatal: Failed to open %s!", config.dboxdir);
    return;
   }

  while((ent=readdir(dir))!=NULL)
    if(ent->d_name[0]!='.' && ent->d_name[0]!='_')
     {
      snprintf(filename, DIRLEN, 
               "%s" DIRCHAR "%s" DIRCHAR "system" DIRCHAR ".",
               config.dboxdir, ent->d_name);

      if(access(filename, 00)==0)
       {
        snprintf(filename, DIRLEN, 
                 "%s" DIRCHAR "%s" DIRCHAR "user" DIRCHAR ".",
                 config.dboxdir, ent->d_name);

        if(access(filename, 00)==0)
          anzboxen++;
       }
     }

  closedir(dir);
 }

void getboxen()
 {
  DIR *dir;
  struct dirent *ent;
  int box;
  char tempstr[DIRLEN], filename[DIRLEN];
  
  getanzboxen();

  if(anzboxen==0) return;  

  snprintf(tempstr, DIRLEN, "%s" DIRCHAR ".", config.dboxdir);
  if((dir=opendir(tempstr))==NULL)
   {
    logf.printf("getboxen", "Fatal: Failed to open %s!", config.dboxdir);
    return;
   }

  box=0;

  boxen=(mailboxt *)malloc(sizeof(mailboxt)*anzboxen);

  if(boxen==NULL)
    logf.internalerror("getboxen", "malloc failed");

  while((ent=readdir(dir))!=NULL)
    if(ent->d_name[0]!='.' && ent->d_name[0]!='_')
     {
      snprintf(filename, DIRLEN, 
               "%s" DIRCHAR "%s" DIRCHAR "system" DIRCHAR ".",
               config.dboxdir, ent->d_name);

      if(access(filename, 00)==0)
       {
        snprintf(filename, DIRLEN, 
                 "%s" DIRCHAR "%s" DIRCHAR "user" DIRCHAR ".",
                 config.dboxdir, ent->d_name);

        if(access(filename, 00)==0)
         {
          strmaxcpy(boxen[box].name, ent->d_name, TEXTLEN-1);
          boxen[box].config.initialize();
          boxen[box].config.read(boxen[box].name);
          box++;
         }
       }
     }

  closedir(dir);
 }

void freeboxen()
 {
  //byte box;

  //for(box=0; box<anzboxen; box++)
  //  boxen[anzboxen].config.~configt();
 }

void dospoolzcn()
 {
  char filename[DIRLEN];
  bool ende=FALSE, verteilt=FALSE;
  byte box;

  #ifdef MSGIDCHECK

  printf("Msgid-Datenbank\n");

  if(msgidbase.open())
   {
    logf.printf("dospool", "Fatal: msgidbase konnte nicht geffnet werden!");
    printf("Fatal: msgidbase konnte nicht geffnet werden!\n");
    return;
   }

  if(msgidbase.check())
   {
    logf.printf("dospool", "Fatal: msgidbase konnte nicht verifiziert werden!");
    printf("Fatal: msgidbase konnte nicht verifiziert werden!\n");
    msgidbase.close();
    return;
   }

  #endif

  printf("Boxen\n");

  getboxen();

  printf("Systemliste\n");

  getsystemlist();

  printf("Permissions\n");

  getpermissions();

  printf("Starte Einsortiervorgang\n");

  display.open();

  do
   {
    box=0;

   schleife:
    getfilename(filename, box);
    if(filename[0]==0)
     {
      box++;
      if(box==anzboxen)
        ende=TRUE;
      else
        goto schleife;
     }
    else
     {
      display.console.printf("\r\nVerteile Datei \"%s\" von Box %s...\r\n", 
        filename, boxen[box].name);
      spoolfile(filename, box);
      verteilt=TRUE;
     }
   }
  while(!ende);

  display.close();
  printf("\n");

  freepermissions();

  freesystemlist();

  freeboxen();

  if(!verteilt)
   {
    logf.timefprintf(stdout);
    printf("Nichts zu verteilen!\n");
   }

  #ifdef MSGIDCHECK
  if(msgidbase.close())
   {
    logf.printf("dospool", "Fatal: msgidbase konnte nicht geschlossen werden!");
    printf("Fatal: msgidbase konnte nicht geschlossen werden!\n");
   }
  #endif
 }

void doshowconfig()
 {
  dword i, j, k;

  printf("Konfiguration:\n\n");

  printf("Boxen:");
  getboxen();
  for(i=0; i<anzboxen; i++) printf(" %s", boxen[i].name);
  printf("\n");

  printf("\nGruppen:\n");
  for(i=0; i<anzboxen; i++)
   {
    printf("%s:", boxen[i].name);
    for(j=0; boxen[i].config.groupdb[j]!=NULL; j++)
      printf(" %s", boxen[i].config.groupdb[j]);
    printf("\n");
   }

  printf("\nLese Systemliste...\n");
  getsystemlist();
  printf(" Ok\n\n");

  printf("Hauptboxname:   %s.%s\n", config.boxname, config.domain);
  printf("Anzahl Systeme: %lu\n\n", anzsystems);

  printf("Systemliste:\n\n");

  for(i=0; i<anzsystems; i++)
   {
    printf("Systemname: %s (%s)\n", systems[i].name, boxen[systems[i].box].name);
    printf("alle EMPs:  %s\n", noyes1(systems[i].allemps));
    printf("Point:      %s\n", noyes1(systems[i].point));
    printf("Gruppe:     %s (%d)\n", 
      boxen[systems[i].box].config.groupdb[systems[i].gruppe],
      systems[i].gruppe);

    if(systems[i].feed!=NULL)
     {
      char *tptr=systems[i].feed;
      printf("Feed-File:\n");
      while(tptr[0]!=0)
       {
        printf("  >%s<\n", tptr);
        tptr+=strlen(tptr)+1;
       }
     }

    if(systems[i].alias!=NULL)
     {
      char *tptr=systems[i].alias;
      printf("Alias-File:\n");
      while(tptr[0]!=0)
       {
        printf("  >%s<\n", tptr);
        tptr+=strlen(tptr)+1;
       }
     }

    printf("\n");
   }

  printf("\nMaps-Liste:\n");

  checkmapst::hashentryt *h=checkmaps.list;

  while(h!=NULL)
   {
    printf("%s:", h->emp);
    for(i=0; i<anzsystems; i++)
      if(h->bestellt[i])
        printf(" %s (%s)", systems[i].name, boxen[systems[i].box].name);

    printf("\n");

    h=h->n;
   }

   {
    double mittel=1.0*checkmaps.anzentries/HASHSIZE;
    double abweichung=0;
    dword anz2=0;

    printf("\nHash-Verteilung:\n");

    k=0;

    for(i=0; i<HASHSIZE; i++)
     {
      h=checkmaps.hashtable[i];
      j=0;
      while(h!=NULL)
       {
        j++;
        h=h->nexthash;
       }
      printf("  %lu: %lu\n", i, j);
      if(j!=0) k++;

      anz2+=j;

      abweichung+=fabs(mittel-j);
     }

    printf("\n\nBelegte Eintrge: %lu (%f%%)\n", k, 1.0*k/HASHSIZE);
    printf("Idealbelegung pro Eintrag: %f\n", mittel);
    printf("durchschnittliche Abweichung: %f\n", abweichung/HASHSIZE);
    printf("Eintrge: %lu %lu\n", checkmaps.anzentries, anz2);
   }

  freesystemlist();

  getpermissions();

  printf("\nPermissions:\n");

  for(j=0; j<anzboxen; j++)
    for(i=0; i<MAXGRUPPEN; i++)
      if(boxen[j].gruppen[i].bretter!=NULL)
       {
        char *tptr=boxen[j].gruppen[i].bretter;

        printf("Gruppe: %s (%s)\n", 
          boxen[j].config.groupdb[i], boxen[j].name);

        while(*tptr!=0)
         {
          printf("  %s\n", tptr);
          tptr+=strlen(tptr)+1;
         }
       }

  freepermissions();
  freeboxen();
 }

void dobrettbestell()
 {
  checkmapst::hashentryt *p;

  fprintf(stderr, "Lese Konfiguration!\n");
  getboxen();
  getsystemlist();
  getpermissions();

  p=checkmaps.list;

  while(p!=NULL)
   {
    printf("%s", p->emp);

    #if 0
    for(i=0; i<anzsystems; i++)
      if(p->bestellt[i])
        printf(" %s", systems[i].name);
    #endif

    printf("\n");    
    p=p->n;
   }

  freesystemlist();
  freepermissions();
  freeboxen();
 }

void dotest()
 {
  printf("Lese Konfiguration!\n");
  getboxen();
  getsystemlist();
  getpermissions();

  while(TRUE)
   {
    char tempstr[1000];

    printf("\nTestmodus:\n");
    printf("1: verifyemp\n");
    printf("2: checkabswab\n");
    printf("Q: Quit\n");
    printf("\nEingabe: ");

    fgets(tempstr, 90, stdin);
    switch(upcase(tempstr[0]))
     {
     case '1':
       {
        char systemname[100], emp[100], boxname[100], *tptr;
        byte box=0;
        systemt *s;

        printf("\nBoxname: ");
        fgets(boxname, TEXTLEN-1, stdin);
        tptr=strchr(boxname, '\r'); if(tptr!=NULL) *tptr=0;
        tptr=strchr(boxname, '\n'); if(tptr!=NULL) *tptr=0;
        while(stricmp(boxname, boxen[box].name)!=0 && box<anzboxen) box++;
        if(box>=anzboxen) { printf("Gibts nicht!\n"); break; }

        printf("\nSystemname: ");
        fgets(systemname, TEXTLEN-1, stdin);
        tptr=strchr(systemname, '\r'); if(tptr!=NULL) *tptr=0;
        tptr=strchr(systemname, '\n'); if(tptr!=NULL) *tptr=0;

        s=findsystem(systemname, box);
        if(s==NULL)
          printf("Gibts nidd!\n");
        else
         {
          printf("Brettname: ");
          fgets(emp, TEXTLEN-1, stdin);
          tptr=strchr(emp, '\r'); if(tptr!=NULL) *tptr=0;
          tptr=strchr(emp, '\n'); if(tptr!=NULL) *tptr=0;

          printf("Verifyemp(\"%s\", \"%s\"): %u\n", systemname, emp,
            verifyemp(s, emp));
          printf("Gruppe: %d\n", s->gruppe);
          printf("Permissions:\n");
          char *tptr=boxen[box].gruppen[s->gruppe].bretter;

          if(tptr==NULL) 
            printf("NULL!\n");
          else
            while(*tptr!=0)
             {
              printf("%s\n", tptr);
              tptr+=strlen(tptr)+1;
             }
         }
       }
      break;

     case '2':
       {
        char abs[1000], wab[1000], sys[1000], boxname[1000], *tptr;
        //dword maxlen;
        systemt *s;
        byte box;

        printf("\nSystemname: ");
        fgets(sys, TEXTLEN-1, stdin);
        tptr=strchr(sys, '\r'); if(tptr!=NULL) *tptr=0;
        tptr=strchr(sys, '\n'); if(tptr!=NULL) *tptr=0;
        if(sys[0]==0) break;

        printf("ABS: ");
        fgets(abs, TEXTLEN-1, stdin);
        tptr=strchr(abs, '\r'); if(tptr!=NULL) *tptr=0;
        tptr=strchr(abs, '\n'); if(tptr!=NULL) *tptr=0;
        if(abs[0]==0) break;

        printf("WAB: ");
        fgets(wab, TEXTLEN-1, stdin);
        tptr=strchr(wab, '\r'); if(tptr!=NULL) *tptr=0;
        tptr=strchr(wab, '\n'); if(tptr!=NULL) *tptr=0;

        printf("Box: ");
        fgets(boxname, TEXTLEN-1, stdin);
        tptr=strchr(boxname, '\r'); if(tptr!=NULL) *tptr=0;
        tptr=strchr(boxname, '\n'); if(tptr!=NULL) *tptr=0;
        if(boxname[0]==0) break;

        box=0;
        while(stricmp(boxname, boxen[box].name)!=0 && box<anzboxen) box++;
        if(box>=anzboxen) { printf("Box gibts nicht!\n"); break; }
        s=findsystem(sys, box);
        if(s==NULL) { printf("System gibts nicht!\n"); break; }
        printf("Result: %d\n", checkabswab(s, abs, wab));
       }
      break;

     case 'Q':
      freesystemlist();
      freepermissions();
      freeboxen();
      return;
     }
   }
 }
