/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <time.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>

#include <fstream.h>
#include <string>

#include <dboxpath.h>
#include <flags.h>
#include <lock.h>
#include <stringtools.h>
#include <tools.h>
#include <logf.h>
#include "ostatus.h"
#include "webtools.h"
#include "htmlforms.h"
#include "ca.h"

#define OPENSSLDIR "/home/cert/openssl/"
#define OPENSSLCONF "/home/cert/emailCA/openssl.cnf"
#define OPENSSLLOG "/home/cert/emailCA/openssl.log"
#define OPENSSLINDEX "/home/cert/emailCA/index.txt"
#define OPENSSLKEY "/home/cert/emailCA/key.txt"

bool lockca()
 {
  return lock("ca", "ca", config.boxname);
 }

bool unlockca()
 {
  return unlock("ca", "ca", config.boxname);
 }

void serveca(httpt &http, const userpt &aliasp, const char *_filename)
 {
  htmlupformst htmlupforms;
  htmlupforms.setprofile(&status.userprofile);
  htmlupforms.set("alias", aliasp.name);
  string filename;
  getdboxhtmldir(filename, config.dboxdir);
  filename+="webonline" DIRCHAR "ca" DIRCHAR;
  filename+=_filename;
  htmlupforms.serve(filename, cout);
 }

void getcert(httpt &http, const userpt &aliasp)
 {
  char certfilename[DIRLEN];

  snprintf(certfilename, DIRLEN, "%smailcert.pem",
    aliasp.getpath());

  ifstream in(certfilename);
  dword len=filelength(certfilename);

  if(in==NULL || len==0)
   {
    serveca(http, aliasp, "nocert.html");
    return;
   }

  cout << "HTTP/1.1 200 Keep cool\r\n"
          "MIME-Version: 1.0\r\n"
          "Server: DBOX-WWW Interface\r\n"
          "Content-Type: application/x-x509-user-cert\r\n"
          "Content-Lenght: " << len << "\r\n"
          "\r\n";

  filecopy(in, cout, len);
 }

void makekey(httpt &http, userpt &aliasp, bool ie)
 {
  char tempoutfilename[DIRLEN];
  char certfilename[DIRLEN];

  snprintf(tempoutfilename, DIRLEN, DBOXTEMPDIR "webonline.%u.tmpca.out",
    config.dboxdir, getpid());

  snprintf(certfilename, DIRLEN, "%smailcert.pem",
    aliasp.getpath());

  if(access(certfilename, 00)==0)
   {
    FILE *outfile=fopen(OPENSSLINDEX, "w");
    if(outfile!=NULL)
      fclose(outfile);
    unlink(certfilename);
   }

   {
    string email, org, host, name, city, spkac;

     {
      const char *tptr=status.userprofile.get("ort");
      // skip zip code
      while(isdigit(*tptr) || isspace(*tptr)) tptr++;
      city=tptr;
      ibm2ascii(city);
     }

    if((status.userprofile|"abs_host")[0]!=0) 
      status.userprofile.get("abs_host", host);
    else if((status.gruppe|"abs_host")[0]!=0)
      status.gruppe.get("abs_host", host);
    else if(config.abs_host[0]!=0)
      host=config.abs_host;
    else
      host=string(config.boxname)+'.'+string(config.domain);

    email=string(aliasp.name)+'@'+host;

    name=string(status.userprofile|"vorname")+' '+
         string(status.userprofile|"nachname");
    ibm2ascii(name);

    org=status.userprofile|"firma";
    ibm2ascii(org);

    if(ie)
     {
      const char *tptr=http.getparameter("csr");
      if(tptr!=NULL)
       {
        for(; tptr[0]!=0; tptr++)
          if(tptr[0]!='\n' && tptr[0]!='\r')
            spkac+=tptr[0];
       }
     }
    else
     {
      const char *tptr=http.getparameter("spkac");
      if(tptr!=NULL)
       {
        for(; tptr[0]!=0; tptr++)
          if(tptr[0]!='\n')
            spkac+=tptr[0];
       }
     }

    if(spkac=="")
     {
      serveca(http, aliasp, "certfailed.html");
      return;
     }

    ofstream out(tempoutfilename);

    out << "C = DE\n"                  // country
         //"ST = \n"                   // state
           "L = " << city << "\n"      // city
           "O = " << org << "\n"       // organization
           "Email = " << email << "\n" // email
           "CN = " << name << "\n"     // name
           "SPKAC = " << spkac << "\n";
   }

   {     
    char command[1000];

    if(lockca())
     {
      unlink(tempoutfilename);
      startpage("DBOX WWW-Interface: CA");
      cout << "<h1>Fehler</h1><P>\n"
              "Diese Funktion ist vorbergehend nicht verfgbar<p>\n";
      showcopyright();
      endpage();
      return;
     }

     {
      ofstream out(OPENSSLLOG, ios::out|ios::app);
      out << "----------------------------\n";
     }

    logf.printf("ca", "creating certificate for %s/%s",
                aliasp.name, status.username);

    snprintf(command, 1000, "%sbin/openssl ca -config '%s' "
             "-spkac '%s' -out '%s' -key \"`cat %s`\" "
             ">> %s 2>&1",
             OPENSSLDIR, OPENSSLCONF, tempoutfilename,
             certfilename, OPENSSLKEY, OPENSSLLOG);

    system(command);

    unlockca();
   }

  if(!ie) // DK
    unlink(tempoutfilename);

   {
    dword len=filelength(certfilename);

    if(len==0)
     {
      serveca(http, aliasp, "certfailed.html");
      return;
     }
   }

  serveca(http, aliasp, "certok-netscape.html");
 }

void doca(httpt &http)
 {
  if(!getinternalflag(&status.userprofile, &status.gruppe, WEBONLINE_CA))
   {
    startpage("DBOX WWW-Interface: CA");
    cout << "<h1>Access denied!</h1><P>\n"
             "Sie haben keine Berechtigung fr diesen Bereich.<p>\n";
    showcopyright();
    endpage();
    return;
   }

  const char *alias=http.getparameter("alias");
  if(alias==NULL) return;
  userpt aliasp(alias);
  if(aliasp.read() ||
     (strcmp(aliasp.name, status.username)!=0 &&
      strcmp(aliasp.get("alias"), status.username)!=0))
   {
    startpage("DBOX WWW-Interface: CA");
    cout << "<h1>Access denied!</h1><P>\n";
    showcopyright();
    endpage();
    return;
   }
 
  const char *action=http.getparameter("action");

  if(action==NULL)
   {
   }
  else if(strcmp(action, "makekey-ns")==0)
    makekey(http, aliasp, FALSE);
  else if(strcmp(action, "makekey-ie")==0)
    makekey(http, aliasp, TRUE);
  else if(strcmp(action, "getcert")==0)
    getcert(http, aliasp);
 }

