/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: alias/account management

*/

#include <unistd.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include <logf.h>
#include <charsettools.h>
#include <account.h>
#include "ostatus.h"
#include "../online/gen_from.h"
#include "../online/userlog.h"
#include "../online/alias.h"
#include "../online/checkuser.h"
#include "challenge.h"
#include "webonline.h"
#include "homepage.h"
#include "www_prof.h"
#include "webtools.h"

dword countalias()
 {
  string filename=status.userprofile.getpath();
  filename+="alias";

  FILE *infile=fopen(filename.c_str(), "rt");
  if(infile==NULL) return 0;

  dword count=0;
  char tempstr[TEXTLEN*2], *tptr;

  while(fgets(tempstr, TEXTLEN-1, infile)!=NULL)
   {
    tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;
    tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;
    if(tempstr[0]!=0) count++;
   }

  fclose(infile);

  return count;
 }

dword get_maxalias()
 {
  char tempstr[TEXTLEN];

  getuser(&status.userprofile, &status.gruppe, "maxalias", tempstr);

  return atol(tempstr);
 }

bool get_set_account()
 {
  char tempstr[TEXTLEN];

  getuser(&status.userprofile, &status.gruppe, "account.set", tempstr);

  return atoi(tempstr);
 }

void getaddress(char *address, const char *name, bool homepage)
 {
  if(homepage)
   {
    wwwpt wwwprofile(name);

    wwwprofile.read();
    geturl(&wwwprofile, address);
   }
  else
   {
    char abs[TEXTLEN], org[TEXTLEN], *tptr;

    gen_abs(status, name, abs, org, TRUE);
    tptr=strchr(abs, ' '); if(tptr!=NULL) *tptr=0;
    strmaxcpy(address, abs, TEXTLEN-1);
    ibm2iso(address, strlen(address));
   }
 }

void doalias_show(httpt &http, ostream &out, bool homepage)
 {
  bool bg=FALSE;
  char filename[DIRLEN], tempstr[TEXTLEN], *tptr;
  char address[TEXTLEN];
  FILE *infile;

  out << "<center>\n"
         "<table cellpadding=2 cellspacing=0>\n"
         "<tr bgcolor=\"" << config.webonline_thbgcolor << "\">"
         "<th><font color=\"" << config.webonline_thfcolor 
      << "\">Name</font></th>"
         "<th><font color=\"" << config.webonline_thfcolor 
      << "\">" << (homepage?"WWW Adresse":"E-Mail Adresse")
      << "</font></th>";
         
  if(homepage)
    out << "<th colspan=2><font color=\""
        << config.webonline_thfcolor 
        << "\">Gre/Dateien</font></th>";

  out << "<th><font color=\"" << config.webonline_thfcolor
      << "\">Befehle</font></th>\n";

  out << "</tr>\n";

  getaddress(address, status.username, homepage);

  out << "<tr bgcolor=\""
      << (bg?config.webonline_tablebgcolor1:config.webonline_tablebgcolor2)
      << "\"><td>" << status.username << "</td>"
         "<td>" << address << "</td>";

  if(homepage)
   {
    show_size(status.username);

    out << "<td><a href=\"/homepage?alias=" << status.username 
        << "\">Homepage</a>";

    out << ", <a href=\"/homepage-stats/" << status.username 
        << "/index.html\">Statistik</a>";
   }
  else
    out << "<td>-</td>";

  out << "\n";

  bg=!bg;

  snprintf(filename, DIRLEN, "%salias", status.userprofile.getpath());
  infile=fopen(filename, "rt");

  string challenge;
  get_challenge(challenge, status.userprofile);

  if(infile!=NULL)
   {
    while(fgets(tempstr, TEXTLEN-1, infile)!=NULL)
     { 
      tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;

      if(tempstr[0]!=0)
       {
        getaddress(address, tempstr, homepage);

        out << "<tr bgcolor=\""
            << (bg?config.webonline_tablebgcolor1:config.webonline_tablebgcolor2)
            << "\"><td>" << tempstr << "</td>"
               "<td>" << address << "</td>";

        if(homepage)
          show_size(tempstr);

        out << "<td>";

        out << "<a href=\"/form/webonline/alias/edit-form.html?alias="
            << tempstr << "\">&auml;ndern</a>";

        out << ", <a href=\"/alias?command=del&alias="
            << tempstr << "&challenge=" << challenge
            << "\">l&ouml;schen</a>";

        if(homepage)
         {
          out << ", <a href=\"/homepage?alias="
              << tempstr << "\">Homepage</a>";

          out << ", <a href=\"/homepage-stats/"
              << tempstr << "/index.html\">Statistik</a>";
         }

        out << "</td>\n";

        bg=!bg;
       }
     }

    fclose(infile);
   }

  out << "<tr bgcolor=\""
      << (bg?config.webonline_tablebgcolor1:config.webonline_tablebgcolor2)
      << "\"><td></td><td></td>";

  if(homepage)
    out << "<td></td><td></td>";

  out << "<td><a href=\"/alias?command=new&challenge=" 
      << challenge << "\">neuen Alias anlegen</a></td>";

  out << "</tr>\n";

  out << "</table>\n"
         "</center>\n";
 }

void do_alias_password(httpt &http, ostream &out, const char *alias)
 {
  const char *newpass1, *newpass2;

  newpass1=http.getparameter("newpass1");
  newpass2=http.getparameter("newpass2");
 
  startpage("Passwort ndern");

  out << "<h1>Passwort ndern</h1>\n\n";

  if(newpass1==NULL || newpass2==NULL)
   {
    out << "Error: no form data\n";
    return;
   }

  if(checkusername(alias))
   {
    out << "Error: alias invalid\n"; 
    return;
   }

  userpt up(alias);
  dnstring(up.name);

  if(up.read())
   {
    out << "Fehler!\n";
    return;
   }

  if(stricmp(up|"alias", status.username)!=0)
   {
    out << "Error: permission denied\n";
    return;
   }

  if(verify_challenge(http.getparameter("challenge"), 
     up))
   {
    out << "Error: security protocol violated\n";
    return;
   }

  if(stricmp(newpass1, newpass2)!=0)
   {
    out << "Fehler: Die eingegebenen Passwrter sind verschieden!\n";
    return;
   }

  if(strlen(newpass1)<4)
   {
    out << "Fehler: Das eingegebene Passwort ist zu kurz!\n";
    return;
   }

  if(!atoi(up|"account"))
   { 
    out << "Fehler: Der Alias ist kein Account!\n";
    return;
   }

  const char *tptr=newpass1;
  while(*tptr!=0 && isascii(*tptr) && isalnum(*tptr)) tptr++;

  if(*tptr!=0)
   {
    out << "Fehler: Das eingegebene Passwort enthlt ungltige Zeichen!\n";
    return;
   }

  if(up.lock())
   {
    out << "Fehler: Vorgang fehlgeschlagen!\n";
    return;
   }

  cryptpassword(newpass1, up.get_ptr("cryptedpassword"));
  up.set("lastpasswordchange", time(NULL));
  up.write();
  up.unlock();

  userlogprintf2(status, up.name, "webonline.doalias", 
    "password of alias \"%s\" changed by user \"%s\"",
     up.name, status.username);

  out << "<p>Die nderung wurde gespeichert!</p>\n";
 }

void doalias(httpt &http, ostream &out)
 {
  const char *command, *alias;

  command=http.getparameter("command");
  alias=http.getparameter("alias");

  if(command==NULL)
   {
    startpage(out, "Aliase");

    doalias_show(http, out, FALSE);
   }
  else if(stricmp(command, "new")==0)
   {
    if(countalias()>=get_maxalias())
     {
      startpage(out, "Neuen Alias anlegen");

      out << "Die maximale Anzahl Aliase ist bereits erreicht.<p>\n"
             "Sie mssen einen Alias lschen, bevor ein neuer angelegt\n"
             "werden kann!<p>\n";
     }
    else if(alias!=NULL)
     {
      char errormsg[TEXTLEN];

      if(checkuser(alias, errormsg))
       {
        ibm2iso(errormsg, strlen(errormsg));

        startpage(out, "Neuen Alias anlegen");

        out << "Der Aliasname ist ungltig.<p>\n"
               "Grund: " << errormsg << "<p>\n"
               "Bitte whlen Sie einen gltigen Namen!<p>\n";
       }
      else
       {
        userpt up(alias);

        dnstring(up.name);

        if(up.read())
         {
          if(verify_challenge(http.getparameter("challenge"), 
             status.userprofile))
           {
            startpage(out, "Neuen Alias anlegen");
            out << "<p>Error: security protocol violated!</p>\n";
           }
          else if(add_alias(&status.userprofile, up.name))
           {
            startpage(out, "Neuen Alias anlegen");
            out << "<p>Eintragung fehlgeschlagen!</p>\n";
           }
          else
           {
            userlogprintf2(status, up.name, "webonline.doalias",
              "alias \"%s\" of user \"%s\" added by user \"%s\"",
              up.name, status.username, status.username);

            redirect(out, http.url);
           }
         }
        else if(stricmp(up|"alias", status.username)==0 ||
                stricmp(up.name, status.username)==0)
         {
          startpage(out, "Neuen Alias anlegen");
          out << "Sie haben diesen Alias bereits!<p>\n";
         }
        else
         {
          startpage(out, "Neuen Alias anlegen");
          out << "Der Aliasname ist ungltig, da er bereits von einem\n"
                 "anderen Benutzer verwendet wird.<p>\n"
                 "Bitte whlen Sie einen anderen Namen!<p>\n";
         }
       }
     }
    else
     {
      startpage(out, "Neuen Alias anlegen");

      string challenge;
      get_challenge(challenge, status.userprofile);

      out <<
        "<FORM ACTION=\"/alias\">\n"
        "Gewnschter Aliasname:<p>\n"
        "<input name=\"alias\" size=\"40\"><p>\n"
        "<INPUT TYPE=submit value=\"Alias anlegen\">\n"
        "<INPUT TYPE=reset value=\"Eingaben l&ouml;schen\">\n"
        "<INPUT TYPE=hidden name=\"command\" value=\"new\">\n"
        "<INPUT TYPE=hidden name=\"challenge\" value=\"" << 
        challenge << "\">\n"
        "</form>\n"
        "<p>\n";
     }
   }
  else if(stricmp(command, "del")==0)
   {
    if(alias!=NULL)
     {
      if(!checkusername(alias))
       {
        userpt up(alias);

        dnstring(up.name);

        if(up.read())
         {
          startpage(out, "Alias l&ouml;schen");
          out << "Fehler!\n";
         }
        else
         {
          if(verify_challenge(http.getparameter("challenge"), 
             status.userprofile))
           {
            startpage(out, "Alias l&ouml;schen");
            out << "<p>Error: security protocol violated!</p>\n";
           }
          else if(stricmp(up|"alias", status.username)==0)
           {
            userlogprintf2(status, up.name, "webonline.doalias", 
              "alias \"%s\" deleted by user \"%s\"",
               up.name, status.username);

            del_alias(&up); 

            redirect(out, http.url);
           }
         }
       }
     }
   }
  else if(stricmp(command, "edit")==0)
   {
    if(alias!=NULL)
     {
      if(!checkusername(alias))
       {
        userpt up(alias);

        dnstring(up.name);

        if(up.read())
         {
          startpage(out, "Alias &auml;ndern");
          out << "Fehler!\n";
         }
        else
         {
          if(stricmp(up|"alias", status.username)==0)
           {
            if(up.lock())
             {
              startpage(out, "Alias &auml;ndern");
              out << "Error: failed to lock profile!\n";
             }
            else
             {
              const char *data;

              data=http.getparameter("new.vorname");
              if(data!=NULL) up.set("vorname", data);

              data=http.getparameter("new.nachname");
              if(data!=NULL) up.set("nachname", data);

              data=http.getparameter("new.strasse");
              if(data!=NULL) up.set("strasse", data);

              data=http.getparameter("new.ort");
              if(data!=NULL) up.set("ort", data);

              data=http.getparameter("new.telefon");
              if(data!=NULL) up.set("telefon", data);

              if(atoi(status.userprofile|"account.set"))
               {
                data=http.getparameter("new.account");
                if(data!=NULL && atoi(data))
                  up.set("account", TRUE);
               }

              up.write();
              up.unlock();

              userlogprintf2(status, up.name, "webonline.doalias", 
                "alias \"%s\" edited by user \"%s\"",
                 up.name, status.username);

              redirect(out, http.url);
             }
           }
         }
       }
     }
   }
  else if(stricmp(command, "rename")==0)
   {

   }
  else if(stricmp(command, "password")==0)
    do_alias_password(http, out, alias);
  else
   {
    startpage(out, "Error");
    out << "Unknown command.\n";
   }

  showcopyright(out);
  endpage(out);
 }

