/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: email table management

*/

#include <stdio.h>
#include <string.h>

#include "../online/checkuser.h"
#include "challenge.h"
#include "ostatus.h"
#include "webtools.h"
#include "htmltools.h"
#include "emailtab.h"

string emailtab_filename()
 {
  return string(status.userprofile.getpath())+"emailtab";
 }

dword get_maxemailtab()
 {
  char tempstr[TEXTLEN];

  getuser(&status.userprofile, &status.gruppe, "maxemailtab", tempstr);

  return atol(tempstr);
 }

dword count_emailtab()
 {
  FILE *infile=fopen(emailtab_filename().c_str(), "rt");
  if(infile==NULL) return 0;

  dword count=0;
  char tempstr[TEXTLEN*2], *tptr;

  while(fgets(tempstr, TEXTLEN-1, infile)!=NULL)
   {
    tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;
    tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;
    if(tempstr[0]!=0) count++;
   }

  fclose(infile);

  return count;
 }

// true: addr is in table
bool check_emailtab(const char *addr)
 {
  FILE *infile=fopen(emailtab_filename().c_str(), "rt");
  if(infile==NULL) return 0;

  char t_addr[TEXTLEN], t_alias[TEXTLEN];
  while(readconfigline(infile, t_addr, t_alias, TEXTLEN-1)==0)
    if(stricmp(addr, t_addr)==0)
     {
      fclose(infile);
      return TRUE;
     }

  fclose(infile);

  return FALSE;
 }

void doemailtab_show(httpt &http, ostream &out)
 {
  bool bg=FALSE;

  out << "<center>\n"
         "<table cellpadding=2 cellspacing=0>\n"
         "<tr bgcolor=\"" << config.webonline_thbgcolor << "\">"
         "<th><font color=\"" << config.webonline_thfcolor 
      << "\">E-Mail Adresse</font></th>"
         "<th><font color=\"" << config.webonline_thfcolor 
      << "\">Ziel</font></th>"
      << "<th><font color=\"" << config.webonline_thfcolor
      << "\">Befehle</font></th>\n";

  out << "</tr>\n";

  string challenge;
  get_challenge(challenge, status.userprofile);

  FILE *infile=fopen(emailtab_filename().c_str(), "rt");
  if(infile!=NULL)
   {
    char addr[TEXTLEN], alias[TEXTLEN];

    while(readconfigline(infile, addr, alias, TEXTLEN-1)==0)
     { 
      out << "<tr bgcolor=\""
          << (bg?config.webonline_tablebgcolor1:config.webonline_tablebgcolor2)
          << "\"><td>" << htmlize_string(addr) << "</td>"
             "<td>" << htmlize_string(alias) << "</td>";

      out << "<td>";

      out << "<a href=\"/emailtab?command=edit&addr="
          << htmlize_string(addr) << "\">&auml;ndern</a>";

      out << ", <a href=\"/emailtab?command=del"
             "&addr=" << htmlize_string(addr) 
          << "&challenge=" << challenge 
          << "\">l&ouml;schen</a>";

      out << "</td>\n";

      bg=!bg;
     }

    fclose(infile);
   }

  out << "<tr bgcolor=\""
      << (bg?config.webonline_tablebgcolor1:config.webonline_tablebgcolor2)
      << "\"><td></td><td></td>";

  out << "<td><a href=\"/emailtab?command=new\">neuen Eintrag anlegen</a></td>";

  out << "</tr>\n";

  out << "</table>\n"
         "</center>\n";
 }

void doemailtab_select(ostream &out, const string &value)
 {
  out << "<SELECT NAME=\"alias\">\n";

  out << "<OPTION>" << htmlize_string(status.userprofile.name) 
      << "</OPTION>\n";

  string filename=status.userprofile.getpath();
  filename+="alias";

  FILE *infile=fopen(filename.c_str(), "rt");
  if(infile!=NULL)
   {
    char tempstr[TEXTLEN*2], *tptr;

    while(fgets(tempstr, TEXTLEN-1, infile)!=NULL)
     {
      tptr=strchr(tempstr, '\r'); if(tptr!=NULL) *tptr=0;
      tptr=strchr(tempstr, '\n'); if(tptr!=NULL) *tptr=0;
      if(tempstr[0]!=0) 
       {
        userpt aliasp(tempstr);
        if(!aliasp.read() && atoi(aliasp|"account"))
          out << "<OPTION>" << aliasp.name << "</OPTION>\n";
       }
     }

    fclose(infile);
   }

  out << "</SELECT>";
 }

bool doemailtab_check_alias(const char *alias)
 {
  userpt aliasp(alias);
  if(checkusername(alias) || aliasp.read())
    return TRUE;

  if(stricmp(alias, status.username)!=0)
   {
    if(!atoi(aliasp|"account"))
      return TRUE;

    if(stricmp(aliasp|"alias", status.username)!=0)
      return TRUE;
   }

  return FALSE;
 }

void doemailtab_new(httpt &http, ostream &out, const char *addr)
 {
  const char *alias=http.get_parameter("alias");

  if(count_emailtab()>=get_maxemailtab())
   {
    startpage(out, "Eintrag hinzuf&uuml;gen");

    out << "<p>Fehler: Die maximale Anzahl an Eintr&auml;gen ist "
           "bereits erreicht.</p>\n"
           "<p>Sie mssen einen Eintrag l&ouml;schen, bevor "
           "ein neuer angelegt werden kann!</p>\n";
   }
  else if(addr!=NULL && alias!=NULL)
   {
    if(verify_challenge(http.getparameter("challenge"), 
       status.userprofile))
     {
      startpage(out, "Eintrag hinzuf&uuml;gen");
      out << "<p>Error: security protocol violated!</p>\n";
      return;
     }

    const char *tptr=addr;
    while(*tptr!=0)
      if(!isalnum(*tptr) && *tptr!='@' && *tptr!='.' &&
         *tptr!='_' && *tptr!='-')
        break;
      else
        tptr++;

    if(*tptr!=0 || addr[0]==0 || strchr(addr, '@')==NULL)
     {
      startpage(out, "Eintrag hinzuf&uuml;gen");
      out << "<p>Fehler: Adresse ung&uuml;ltig!\n";
      return;
     }

    if(doemailtab_check_alias(alias))
     {
      startpage(out, "Eintrag hinzuf&uuml;gen");
      out << "<p>Fehler: \"" << htmlize_string(alias) 
          << "\" ung&uuml;ltig!</p>\n";
      return;
     }

    if(check_emailtab(addr))
     {
      startpage(out, "Eintrag hinzuf&uuml;gen");
      out << "<p>Fehler: Adresse ist bereits in der Tabelle!</p>\n";
      return;
     }

    FILE *outfile=fopen(emailtab_filename().c_str(), "at");
    if(outfile==NULL)
     {
      startpage(out, "Eintrag hinzuf&uuml;gen");
      out << "<p>Error: failed to open table!</p>\n";
      return;
     }

    fprintf(outfile, "%s=%s\n", addr, alias);
    fclose(outfile);
    redirect(out, http.url);
   }
  else
   {
    startpage(out, "Eintrag hinzuf&uuml;gen");

    out <<
      "<FORM ACTION=\"/emailtab\">\n"
      "<center>\n"
      "<table>\n"
      "<tr><th colspan=2>"
      "<font size=+1><u>Eintrag anlegen</u></font></th></tr>\n"
      "<tr><th align=right>E-Mail Adresse:</th>\n"
      "<td><input name=\"addr\" size=\"40\"></td></tr>\n"
      "<tr><th align=right>Ziel:</th>\n"
      "<td>";

    doemailtab_select(out, "");

    string challenge;
    get_challenge(challenge, status.userprofile);

    out << 
      "</td></tr>\n"
      "<tr><td colspan=2><hr size=1 noshade></td></tr>\n"
      "<tr><td></td><td>\n"
      "<INPUT TYPE=submit value=\"Eintrag anlegen\"></td></tr>\n"
      "<INPUT TYPE=hidden name=\"command\" value=\"new\">\n"
      "<INPUT TYPE=hidden name=\"challenge\" value=\"" << challenge << "\">\n"
      "</table>\n"
      "</center\n"
      "</form>\n"
      "<p>\n";
   }
 }

void doemailtab_del(httpt &http, ostream &out, const char *addr)
 {
  if(addr!=NULL)
   {
    if(verify_challenge(http.getparameter("challenge"), 
       status.userprofile))
     {
      startpage(out, "Eintrag l&ouml;schen");
      out << "<p>Error: security protocol violated!</p>\n";
      return;
     }

    string filename=emailtab_filename();
    FILE *infile=fopen(filename.c_str(), "rt");
    if(infile==NULL) return;

    string tmp_filename=filename+".tmp";
    FILE *outfile=fopen(tmp_filename.c_str(), "wt");
    if(outfile==NULL) { fclose(infile); return; }

    char t_addr[TEXTLEN], t_alias[TEXTLEN];
    while(readconfigline(infile, t_addr, t_alias, TEXTLEN-1)==0)
      if(stricmp(addr, t_addr)!=0)
        fprintf(outfile, "%s=%s\n", t_addr, t_alias);

    fclose(infile);
    fclose(outfile);

    rename(tmp_filename.c_str(), filename.c_str());
   }

  redirect(out, http.url);
 }

void doemailtab_edit(httpt &http, ostream &out, const char *addr)
 {
  const char *alias=http.get_parameter("alias");

  if(addr==NULL) return;

  if(alias!=NULL)
   {
    if(verify_challenge(http.getparameter("challenge"), 
       status.userprofile))
     {
      startpage(out, "Eintrag &auml;ndern");
      out << "<p>Error: security protocol violated!</p>\n";
      return;
     }

    if(doemailtab_check_alias(alias))
     {
      startpage(out, "Eintrag &auml;ndern");
      out << "<p>Fehler: \"" << htmlize_string(alias) 
          << "\" ung&uuml;ltig!</p>\n";
      return;
     }

    string filename=emailtab_filename();
    FILE *infile=fopen(filename.c_str(), "rt");
    if(infile==NULL) return;

    string tmp_filename=filename+".tmp";
    FILE *outfile=fopen(tmp_filename.c_str(), "wt");
    if(outfile==NULL) { fclose(infile); return; }

    char t_addr[TEXTLEN], t_alias[TEXTLEN];
    while(readconfigline(infile, t_addr, t_alias, TEXTLEN-1)==0)
      if(stricmp(addr, t_addr)!=0)
        fprintf(outfile, "%s=%s\n", t_addr, t_alias);
      else
        fprintf(outfile, "%s=%s\n", t_addr, alias);

    fclose(infile);
    fclose(outfile);

    rename(tmp_filename.c_str(), filename.c_str());

    redirect(out, http.url);
   }
  else
   {
    startpage(out, "Eintrag &auml;ndern");

    out <<
      "<FORM ACTION=\"/emailtab\">\n"
      "<center>\n"
      "<table>\n"
      "<tr><th colspan=2><font size=+1><u>Eintrag &auml;ndern\n"
      "</u></font></th></tr>\n"
      "<tr><th align=right>E-Mail Adresse:</th>\n"
      "<td><code>" << htmlize_string(addr) << "</code></td></tr>\n"
      "<tr><th align=right>Ziel:</th><td>\n";

    doemailtab_select(out, "");

    string challenge;
    get_challenge(challenge, status.userprofile);

    out << 
      "</td></tr>\n"
      "<tr><td colspan=2><hr size=1 noshade></td></tr>\n"
      "<tr><td></td><td>"
      "<INPUT TYPE=submit value=\"Eintrag &auml;ndern\"></td></tr>\n"
      "<INPUT TYPE=hidden name=\"command\" value=\"edit\">\n"
      "<INPUT TYPE=hidden name=\"addr\" value=\""
      << htmlize_string(addr) << "\">\n"
      "<INPUT TYPE=hidden name=\"challenge\" value=\""
      << challenge << "\">\n"
      "</table>\n"
      "</center\n"
      "</form>\n"
      "<p>\n";
   }
 }

void doemailtab(httpt &http, ostream &out)
 {
  const char *command, *addr;

  if((status.userprofile|"alias")[0]!=0)
   {
    startpage(out, "E-Mail Tabelle");
    out << "Zugriff verweigert!\n";
   }
  else
   {
    command=http.getparameter("command");
    addr=http.getparameter("addr");

    if(command==NULL)
     {
      startpage(out, "E-Mail Tabelle");
      doemailtab_show(http, out);
     }
    else if(stricmp(command, "new")==0)
      doemailtab_new(http, out, addr);
    else if(stricmp(command, "del")==0)
      doemailtab_del(http, out, addr);
    else if(stricmp(command, "edit")==0)
      doemailtab_edit(http, out, addr);
    else
     {
      startpage(out, "Error");
      out << "Unknown command.\n";
     }
   }

  showcopyright(out);
  endpage(out);
 }
