/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <dirent.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <dboxpath.h>
#include <config.h>
#include <logf.h>
#include "ostatus.h"
#include "../online/userlog.h"
#include "webonline.h"
#include "webtools.h"
#include "homepage.h"
#include "dbalias.h"
#include "www_prof.h"
#include "../webquota/www_quota.h"
#include "checkfilename.h"

void encode_name(const char *name, char *string)
 {
  while(name[0]!=0)
   {
    char data=*name;
    string[0]=(data&0x0f)+'a';
    string[1]=((data>>4)&0x0f)+'a';
    string+=2;
    name++;
   }
  string[0]=0;
 }

void decode_name(const char *string, char *name)
 {
  while(string[0]!=0 && string[1]!=0)
   {
    name[0]=((string[0]-'a') | ((string[1]-'a')<<4));
    name++;
    string+=2;
   }
  name[0]=0;
 }

void getfile(httpt &http, const char *directory)
 {
  char data[1000], *tptr, *tptr2, *tptr3, *file, filename[DIRLEN];
  FILE *outfile;
  dword len=0;

  const char *h_ptr=http.get_header("Content-Length");
  if(h_ptr!=NULL) len=atol(h_ptr);

  if(len>config.homepagemaxfilesize) 
   {
    startpage("Fehler");
    printf("Datei zu gro!\n");
    showcopyright();
    endpage();
    return;
   }

  file=(char *)malloc(len+1);
  if(file==NULL) abort();

  fread(file, len, 1, stdin);
  file[len]=0;

  snprintf(data, 999, "Homepage Manager f&uuml;r User %s", status.username);
  startpage(data);
  printf("<H1>%s</H1><p>\n", data);

  tptr=strstr(file, "filename=\"");
  if(tptr==NULL)
   {
   error:
    printf("Upload ung&uuml;ltig.\n");
    endpage();
    free(file);
    return;
   }

  tptr+=10;
  tptr2=strchr(tptr, '"');
  if(tptr2==NULL) goto error;
  *tptr2=0;
  strmaxcpy(data, tptr, 90);
  tptr=tptr2+1;

  tptr2=strrchr(data, '/');
  if(tptr2!=NULL)
   {
    strmaxcpy(filename, tptr2+1, DIRLEN-1); 
    strmaxcpy(data, filename, DIRLEN-1);
   }

  tptr2=strrchr(data, '\\');
  if(tptr2!=NULL)
   {
    strmaxcpy(filename, tptr2+1, DIRLEN-1); 
    strmaxcpy(data, filename, DIRLEN-1); 
   }

  strmaxcpy(filename, data, DIRLEN-1);

  if(checkfilename(filename))
   {
    logf.printf("getfile", "Dateiname \"%s\" ungueltig", filename);
    printf("Dateiname ung&uuml;ltig.\n");
    endpage();
    free(file);
    return;
   }

  tptr2=strstr(tptr, "\n\n");
  tptr3=strstr(tptr, "\n\r\n");
  if(tptr2!=NULL && tptr3!=NULL)
   {
    if(tptr3<tptr2) tptr2=tptr3;
   }
  else if(tptr2==NULL && tptr3!=NULL)
    tptr2=tptr3;
  else if(tptr2==NULL && tptr3==NULL)
    goto error;

  tptr2++;
  if(*tptr2=='\r') tptr2++;
  tptr2++;

  tptr=tptr2;

  tptr2=file+len-1;
  while(*tptr2!='\r')
   {
    if(tptr2<=tptr) goto error;
    tptr2--; 
   }
  tptr2--;
  while(*tptr2!='\r')
   {
    if(tptr2<=tptr) goto error;
    tptr2--; 
   }

  *tptr2=0;

  snprintf(data, 999, "%s%s", directory, filename);

  wwwpt wwwp(status.username);

  if(access(data, 00)==0)
    if(!wwwp.lock())
     {
      www_quota(wwwp, data, FALSE);
      wwwp.write();
      wwwp.unlock();
     }

  outfile=fopen(data, "wb");
  if(outfile==NULL) goto error;
  fwrite(tptr, tptr2-tptr, 1, outfile);
  fclose(outfile);
  chmod(data, 0644);

  logf.printf("getfile", "Datei \"%s\" gespeichert", filename);

  if(!wwwp.lock())
   {
    www_quota(wwwp, data, TRUE);
    wwwp.write();
    wwwp.unlock();
   }

  printf("<h2>Datei-Upload</h2>\n");
  printf("Dateiname: \"%s\"<p>\n"
         "Die Datei wurde in Ihrem Verzeichnis abgelegt.<p>\n",
         filename);
  endpage();

  free(file);
 }

void doupload(httpt &http)
 {
  char tempstr[1000], tempstr2[1000], directory[DIRLEN], *tptr,
       alias[TEXTLEN];

  if(config.homepagedir[0]==0)
    return;

  strmaxcpy(tempstr, http.url.c_str()+4, 999);
  tptr=strchr(tempstr, '/');
  if(tptr==NULL) return;

  *(tptr++)=0;
  strmaxcpy(alias, tempstr, TEXTLEN-1);

  if(tptr[0]!='c' || tptr[1]!='d') return;
  tptr+=2;

  if(tptr[0]==0)
    snprintf(directory, DIRLEN, "%s%s" DIRCHAR, config.homepagedir, alias);
  else
   {
    decode_name(tptr, tempstr2);

    if(tempstr2[0]!='/' || strstr(tempstr2, "..")!=NULL)
      return;

    snprintf(directory, DIRLEN, "%s%s%s" DIRCHAR, config.homepagedir, alias, tempstr2);
   }

  if(strcmp(alias, status.username)!=0)
   {
    userpt aliasp(alias);
    if(aliasp.read() || strcmp(aliasp|"alias", status.username)!=0)
      return;  
   }

  getfile(http, directory);
 }

class homepaget
 {
 public:
  char encoded_cd[DIRLEN];
  char directory[DIRLEN], cd[DIRLEN], url[TEXTLEN], alias[TEXTLEN];
  wwwpt wwwp;
  const char *p_cd;
  httpt &http;

  void docommand(const char *command);
  void docommand_list();
  void docommand_rename();
  void docommand_mkdir();
  void docommand_edit_profile();
  void docommand_anlegen();
  void docommand_upload();
  void docommand_delete();
  void selectalias();
  void dohomepage();

  homepaget(httpt &_http):http(_http) { }
 };

void homepaget::docommand_list()
 {
  char tempstr[DIRLEN];

  snprintf(tempstr, DIRLEN, "%s.", directory);
  if(access(tempstr, 00)!=0)
   {
    if(p_cd==NULL)
     {
      char link[TEXTLEN*2];

      printf("Sie haben noch keine Homepage fr den Alias %s.<p>\n"
             "M&ouml;chen Sie eine Homepage anlegen?<p>\n", alias);

      printf("<ul><li>");

      snprintf(link, TEXTLEN*2, "/homepage?alias=%s&command=anlegen", alias);

      makebutton("Ja, Homepage anlegen", link, NULL);

      printf("<li>");

      makebutton("Nein, zur&uuml;ck ins Hauptmen&uuml;", "/", "_top");

      printf("</ul>\n");
     }
    else
      printf("Verzeichnis nicht gefunden!\n");
   }
  else
   {
    DIR *dir;
    bool bg=FALSE;
    struct dirent *ent;

    printf("Verzeichnis: %s<p>\n", cd[0]==0?"/":cd);

    printf("<TABLE>\n"
      "<TR bgcolor=\"%s\">"
      "<TH><font color=\"%s\">Typ</font>"
      "<TH><font color=\"%s\">Datum</font>"
      "<TH><font color=\"%s\">Gr&ouml;&szlig;e</font>"
      "<TH><font color=\"%s\">Dateiname</font>"
      "<TH><font color=\"%s\">Kommandos</font>"
      "\n", config.webonline_thbgcolor,
      config.webonline_thfcolor, config.webonline_thfcolor,
      config.webonline_thfcolor, config.webonline_thfcolor,
      config.webonline_thfcolor);

    if(cd[0]!=0)
     {
      char tempstr2[DIRLEN], *tptr;
 
      strmaxcpy(tempstr, cd, DIRLEN-1);
      tptr=strrchr(tempstr, '/');
      if(tptr==NULL)
        tempstr2[0]=0;
      else
       {
        *tptr=0;
        encode_name(tempstr, tempstr2);
       }
    
      printf("<tr bgcolor=\"%s\"><td><img src=\"/html/icons/anthony-dirup.gif\"><td>"
             "<center>-</center><td><center>-</center>"
             "<td><a href=\"/homepage?alias=%s&cd=%s\">zurck</a><td>", 
             bg?config.webonline_tablebgcolor1:config.webonline_tablebgcolor2,
             alias, tempstr2);

      bg^=1;
     }

    snprintf(tempstr, DIRLEN, "%s.", directory);
    dir=opendir(tempstr);
    if(dir!=NULL)
     {
      while((ent=readdir(dir))!=NULL)
       {
        if(ent->d_name[0]!='.' && stricmp(ent->d_name, "profile")!=0)
         {
          struct stat buf;

          snprintf(tempstr, DIRLEN, "%s%s", directory, ent->d_name);
          if(stat(tempstr, &buf)==0)
           {
            char encoded_name[DIRLEN];

            encode_name(ent->d_name, encoded_name);

            printf("<tr bgcolor=\"%s\">",
              bg?config.webonline_tablebgcolor1:config.webonline_tablebgcolor2);
 
           bg^=1;

            if(buf.st_mode&S_IFDIR)
             {
              char tempstr2[DIRLEN];
 
              snprintf(tempstr, DIRLEN, "%s/%s", cd, ent->d_name);
              encode_name(tempstr, tempstr2);

              printf("<td><img src=\"/html/icons/anthony-dir.gif\"><td>"
                     "<center>-</center><td><center>-</center>"
                     "<td><a href=\"/homepage?alias=%s&cd=%s\">%s</a>", 
                     alias, tempstr2, ent->d_name);
             }
            else
             {
              struct tm *tm=localtime(&buf.st_mtime);
   
              if(buf.st_size<10000)
               {
                snprintf(tempstr, DIRLEN, "%4lu", buf.st_size);
               }
              else
               {
                dword l=buf.st_size/1024;
                if(l>999)
                 {
                  l/=1024;
                  if(l==0) l=1;
                  snprintf(tempstr, DIRLEN, "%3luM", l);
                 }
                else
                 {
                  if(l==0) l=1;
                  snprintf(tempstr, DIRLEN, "%3luK", l);
                 }
               }

              printf("<td><img src=\"/html/icons/anthony-unknown.gif\">"
                     "<td><center><code>%02d</code>.<code>%02d</code>.</center>"
                     "<td align=right><code>%s</code><td>"
                     "<a target=\"_top\" href=\"%s%s/%s\">%s</a>", 
                     tm->tm_mday, tm->tm_mon+1, tempstr,
                    url, cd, ent->d_name, ent->d_name);
             }

            printf("<td>"
                   "<a href=\"/homepage?alias=%s&command=rename&file=%s&cd=%s\">"
                   "umbenennen</a>, "
                   "<a href=\"/homepage?alias=%s&command=delete&file=%s&cd=%s\">"
                   "l&ouml;schen</a>",
                   alias, encoded_name, encoded_cd,
                   alias, encoded_name, encoded_cd);
           }
         }
       }
      closedir(dir);
     }

    printf("<tr bgcolor=\"%s\"><td><td><td><td><td>"
           "<a href=\"/homepage?alias=%s&command=upload&cd=%s\">neue Dateien "
           "holen</a>\n",
           bg?config.webonline_tablebgcolor1:config.webonline_tablebgcolor2,
           alias, encoded_cd);

    bg^=1;

    printf("<tr bgcolor=\"%s\"><td><td><td><td><td>"
           "<a href=\"/homepage?alias=%s&command=mkdir&cd=%s\">neues Verzeichnis "
           "anlegen</a>\n", 
           bg?config.webonline_tablebgcolor1:config.webonline_tablebgcolor2,
           alias, encoded_cd);

    printf("</TABLE>\n");

    if(cd[0]==0)
     {
      printf("<form action=\"/homepage\">\n");
      printf("Homepage-Beschreibung:\n");
      printf("<input type=text name=\"betreff\" value=\"%s\">\n", 
        wwwp|"betreff");
      printf("<input type=hidden name=\"command\" value=\"edit-profile\">\n");
      printf("<input type=hidden name=\"alias\" value=\"%s\">\n", alias);
    
      printf("<p>\n");
      printf("</form>\n");
     }
   }
 }

void homepaget::docommand_edit_profile()
 {
  const char *tptr, *tptr2;

  tptr=http.getparameter("betreff");
  if(tptr==NULL) return;

  tptr2=tptr;
  while(*tptr2!=0)
   {
    if(*tptr2=='<' || *tptr2=='&')
     {
      printf("Die Beschreibung enthlt ungltige Zeichen!");
      return;
     }
    tptr2++;
   }
   
  if(wwwp.lock())
   {
    printf("nderung fehlgeschlagen!");    
   }
  else
   {
    wwwp.set("betreff", tptr);
    wwwp.write();
    wwwp.unlock();

    userlogprintf1(status, "dohomepage", "WWW profile Betreff geaendert");
    printf("Ihre nderung wurde gespeichert!");    
   }
 }

void homepaget::docommand_rename()
 {
  char filename[DIRLEN]; 
  const char *tptr, *tptr2;

  tptr=http.getparameter("file");
  if(tptr==NULL) return;
  decode_name(tptr, filename);

  tptr2=http.getparameter("neu");
  if(tptr2==NULL)
   {
    printf("<h2>Datei umbenennen</h2>\n"
           "<form action=\"/homepage\">\n"
           "<table>\n"
           "<tr><th align=right>Alter Name:<td>%s/%s\n"
           "<tr><th align=right>Neuer Name:<td>"
           "<input name=\"neu\" size=\"40\">\n"
           "</table>\n"
           "<input type=submit value=\"Datei umbenennen\">\n"
           "<input type=hidden value=\"rename\" name=\"command\">\n"
           "<input type=hidden value=\"%s\" name=\"file\">\n"
           "<input type=hidden value=\"%s\" name=\"alias\">\n"
           "<input type=hidden value=\"%s\" name=\"cd\">\n",
           cd, filename, tptr, alias, encoded_cd);

    printf("</form>\n");
   }
  else
   {
    if(!checkfilename(tptr2) &&
       strchr(filename, '/')==NULL &&
       strchr(filename, '\\')==NULL &&
       stricmp(filename, "profile")!=0 &&
       filename[0]!='.')
     {
      char alt[DIRLEN], neu[DIRLEN];
      snprintf(alt, DIRLEN, "%s%s", directory, filename);
      snprintf(neu, DIRLEN, "%s%s", directory, tptr2);

      if(access(neu, 00)==0)
        printf("Der neue Dateiname ist bereits in Verwendung.\n");
      else
       {
        rename(alt, neu);
        printf("Die Datei wurde umbenannt.<p>\n");
        userlogprintf1(status, "dohomepage",
          "%s umbenannt in %s", alt, neu);
       }
     }
    else
      printf("Der neue Dateiname ist ung&uuml;ltig.\n");
   }
 }

void homepaget::docommand_delete()
 {
  char filename[DIRLEN]="";
  const char *tptr;

  tptr=http.getparameter("file");
  if(tptr==NULL) return;

  decode_name(tptr, filename);

  if(http.getparameter("ok")==NULL)
   {
    printf("<h2>Datei l&ouml;schen</h2>\n"
           "Soll die Datei wirklich gel&ouml;scht werden?<p>\n"
           "<form action=\"/homepage\">\n"
           "Name: %s/%s<p>\n"
           "<input type=submit value=\"Datei l&ouml;schen!\">\n"
           "<input type=hidden value=\"ja\" name=\"ok\">\n"
           "<input type=hidden value=\"delete\" name=\"command\">\n"
           "<input type=hidden value=\"%s\" name=\"file\">\n"
           "<input type=hidden value=\"%s\" name=\"alias\">\n"
           "<input type=hidden value=\"%s\" name=\"cd\">\n",
           cd, filename, tptr, alias, encoded_cd);

    printf("</form>\n");
   }
  else
   {
    if(strchr(filename, '/')==NULL &&
       strchr(filename, '\\')==NULL &&
       stricmp(filename, "profile")!=0 &&
       filename[0]!='.')
     {
      char name[DIRLEN];
      snprintf(name, DIRLEN, "%s%s", directory, filename);

      if(!wwwp.lock())
       {
        www_quota(wwwp, name, FALSE);
        wwwp.write();
        wwwp.unlock();
       }

      if(remove(name)>=0)
       {
        printf("Die Datei %s/%s wurde gel&ouml;scht.<p>\n", 
          cd, filename);
        userlogprintf1(status, "dohomepage", "%s geloescht", name);
       }
      else if(rmdir(name)>=0)
       {
        printf("Das Verzeichnis %s/%s wurde gel&ouml;scht.<p>\n", 
          cd, filename);
        userlogprintf1(status, "dohomepage", "%s geloescht", name);
       }
      else
       {
        printf("Lschung fehlgeschlagen!<p>\n");
        userlogprintf1(status, "dohomepage",
          "Failed to delete %s", name);
       }
     }
    else
      printf("Der Dateiname ist ung&uuml;ltig.\n");
   }
 }

void homepaget::docommand_mkdir()
 {
  char filename[DIRLEN]; 
  const char *tptr;

  tptr=http.getparameter("file");
  if(tptr==NULL)
   {
    printf("<h2>Verzeichnis anlegen</h2>\n"
           "<form action=\"/homepage\">\n"
           "<table>\n"
           "<tr><th align=right>Neues Verzeichnis:<td>"
           "<input name=\"file\" size=\"40\">\n"
           "</table>\n"
           "<input type=submit value=\"Verzeichnis anlegen\">\n"
           "<input type=hidden value=\"mkdir\" name=\"command\">\n"
           "<input type=hidden value=\"%s\" name=\"alias\">\n"
           "<input type=hidden value=\"%s\" name=\"cd\">\n",
           alias, encoded_cd);

    printf("</form>\n");
   }
  else
   {
    if(!checkfilename(tptr))
     {
      snprintf(filename, DIRLEN, "%s%s", directory, tptr);
      mkdir(filename, 0755);
      chmod(filename, 0755);
      printf("Das Verzeichnis wurde angelegt.<p>\n");
      userlogprintf1(status, "dohomepage", "%s angelegt", filename);

      if(!wwwp.lock())
       {
        www_quota(wwwp, filename, TRUE);
        wwwp.write();
        wwwp.unlock();
       }
     }
    else
      printf("Der Verzeichnisname ist ung&uuml;ltig.\n");
   }                     
 }

void homepaget::docommand_upload()
 {
  printf("<h2>Datei uploaden</h2>\n"
       "<FORM ENCTYPE=\"multipart/form-data\" "
       " ACTION=\"/hu/%s/cd%s\" METHOD=POST>\n"
       "Dateiname: <INPUT NAME=\"userfile\" TYPE=\"file\"><p>\n"
       "<INPUT TYPE=\"submit\" VALUE=\"Datei Absenden\">\n"
       "</FORM>\n", alias, encoded_cd);
 }

void homepaget::docommand_anlegen()
 {
  char tempstr[DIRLEN];

  snprintf(tempstr, DIRLEN, "%s.", directory);
  if(access(tempstr, 00)==0)
   {
    printf("Sie haben schon eine Homepage.<p>\n"
           "Bitte benutzen Sie den "
           "<a href=\"/homepage\">Homepage-Manager</a>.<p>\n");
   }
  else
   {
    strmaxcpy(tempstr, directory, DIRLEN-1);
    tempstr[strlen(tempstr)-1]=0;

    mkdir(tempstr, 0755);
    chmod(tempstr, 0755);
    printf("Ihre Homepage wurde angelegt.<p>\n"
           "Bitte benutzen Sie nun den "
           "<a href=\"/homepage\">Homepage-Manager</a>.<p>\n");
   }
 }

void homepaget::docommand(const char *command)
 {
  if(command==NULL)
    docommand_list();
  else if(stricmp(command, "edit-profile")==0)
    docommand_edit_profile();
  else if(stricmp(command, "rename")==0)
    docommand_rename();
  else if(stricmp(command, "delete")==0)
    docommand_delete();
  else if(stricmp(command, "mkdir")==0)
    docommand_mkdir();
  else if(stricmp(command, "upload")==0)
    docommand_upload();
  else if(stricmp(command, "anlegen")==0 && cd[0]==0)
    docommand_anlegen();
  else
   {
    printf("Ung&uuml;ltige URL.\n");
   }
 }

void homepaget::dohomepage()
 {
  const char *command;

  if(config.homepagedir[0]==0)
    return;

   {
    const char *tptr;

    tptr=http.getparameter("alias");
    if(tptr==NULL)
      alias[0]=0;
    else
     {
      strmaxcpy(alias, tptr, TEXTLEN-1);
      dnstring(alias);
      if(strcmp(alias, status.username)!=0)
       {
        userpt aliasp(alias);
        if(aliasp.read() || strcmp(aliasp|"alias", status.username)!=0)
         {
          startpage("Fehler");
          printf("Alias ungltig!\n");
          showcopyright();
          endpage();
          return;
         }
       }
     }
   }

  strmaxcpy(wwwp.name, alias, TEXTLEN-1);
  wwwp.read();

  geturl(&wwwp, url);
  if(url[0]!=0)
    if(url[strlen(url)-1]=='/')
      url[strlen(url)-1]=0;

  command=http.getparameter("command");
  p_cd=http.getparameter("cd");

  cd[0]=0;

  if(p_cd==NULL || p_cd[0]==0)
    snprintf(directory, DIRLEN, "%s%s" DIRCHAR, config.homepagedir, alias);
  else
   {
    decode_name(p_cd, cd);

    if(cd[0]!='/' || strstr(cd, "..")!=NULL)
     {
      startpage("Fehler");
      printf("Verzeichnis ungltig!\n");
      showcopyright();
      endpage();
      return;
     }

    snprintf(directory, DIRLEN, "%s%s%s" DIRCHAR, config.homepagedir, alias, cd);
   }

  encode_name(cd, encoded_cd);

   {
    char title[TEXTLEN*2];
    snprintf(title, TEXTLEN*2, "Homepage Manager f&uuml;r User %s", status.username);
    startpage(title);
    printf("<H1>%s</H1><p>\n", title);
   }
 
  if(alias[0]==0)
    doalias_show(http, cout, TRUE);
  else
    docommand(command);

  showcopyright();
  endpage();

  fflush(stdout);
 }

void dohomepage(httpt &http)
 {
  homepaget h(http);
  h.dohomepage();
 }

void show_size(const string &user)
 {
  wwwpt wwwprofile(user.c_str());
  wwwprofile.read();

  long size=atol(wwwprofile.get("size"));
  long files=atol(wwwprofile.get("files"));
  if(files<0) files=0;
  if(size<0) size=0;

  printf("<td align=right>");
  if(size<1024) 
    printf("%ld bytes", size);
  else if(size<(1024*1024))
    printf("%ld kb", size/1024+1);
  else 
    printf("%ld mb", size/(1024*1024)+1);

  printf("</td><td align=right>%ld</td>", files);
 }

