/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: show mailbox list

*/

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>

#include <algorithm>
#include <minmax.h>
#include <fstream.h>

#include <dboxpath.h>
#include <tools.h>
#include <folder.h>
#include <charsettools.h>
#include <stringtools.h>
#include <htmltools.h>
#include <nls.h>

#include "ostatus.h"
#include "webtools.h"
#include "webmail.h"

static void get_total_unseen(unsigned folder_nr, 
                             unsigned &total,
                             unsigned &unseen)
 {
  string folder_owner=status.username,
         subfolder=i2string(folder_nr);

  total=unseen=0;

  if(mailfolderalias(folder_owner, subfolder))
    return;

  string folder_name=subfolder+"@"+folder_owner;

  folderpt fp;
  fp.setname(folder_name);
  fp.setfilename();

  string indexfilename=fp.getpath();
  indexfilename+="index";

  int index=::open(indexfilename.c_str(), O_BINARY|O_RDONLY);
  folderindext fi;

  if(index>=0)
   {
    while(::read(index, &fi, sizeof fi)==sizeof fi)
     {
      total++;
      if(!fi.flags.seen) unseen++;
     }

    ::close(index);
   }
 }

struct folder_entryt
 {
  string name;
  int nr;
 };

static bool folder_entry_sort_pr
  (const folder_entryt &a, const folder_entryt &b)
 {
  return stringcasecmp(a.name, b.name)<0;
 }

void webmailt::show_mailbox_list()
 {
  vector<folder_entryt> folders;

  string filename=status.userprofile.getpath();
  filename+="folders";

  ifstream in(filename.c_str());

   {
    folder_entryt folder_entry;
    folder_entry.nr=0;

    if(!in)
     {
      folder_entry.name="INBOX";
      folders.push_back(folder_entry);
     }
    else
     {
      char line[TEXTLEN], *tptr, ch;

      while(in.get(line, TEXTLEN-1))
       {
        in.get(ch); // read \n
        tptr=strchr(line, '\n'); if(tptr!=NULL) *tptr=0;
        tptr=strchr(line, '\r'); if(tptr!=NULL) *tptr=0;

        folder_entry.name=line;
        folders.push_back(folder_entry);
        folder_entry.nr++;
       }
     }
   }

  if(!folders.empty())
    sort(folders.begin()+1, folders.end(), folder_entry_sort_pr);

  startpage(out, nls.get("webonline.folders.title"));

  out << "<center><table cellpadding=2 cellspacing=0>\n";

  out <<
    "<TR bgcolor=\"" << config.webonline_thbgcolor << "\">"
    "<TH><font color=\"" << config.webonline_thfcolor << "\">" << nls.get("webonline.folders.name") << "</font></th>"
    "<TH><font color=\"" << config.webonline_thfcolor << "\">" << nls.get("webonline.folders.total") << "</font></th>"
    "<TH><font color=\"" << config.webonline_thfcolor << "\">" << nls.get("webonline.folders.new") << "</font></th>";

  out << "</TR>\n";

  bool bg=FALSE;

  for(vector<folder_entryt>::const_iterator it=folders.begin();
      it!=folders.end();
      it++)
    if(it->name!="")
     {
      out << "<TR bgcolor=\""
          << (bg?config.webonline_tablebgcolor1:config.webonline_tablebgcolor2) 
          << "\">";

      out << "<td><a href=\"/email?mailbox="
          << urlize_string(it->name)
          << "\">";

      out << "<img src=\"/html/webonline/webmail/folder.gif\" border=0>";

      unsigned total, unseen;
      get_total_unseen(it->nr, total, unseen);

      if(unseen!=0) out << "<b>";

      if(it->name=="INBOX")
        out << nls.get("webonline.folders.inbox");
      else
        out << htmlize_string(it->name);

      if(unseen!=0) out << "</b>";

      out << "</a></td>";

      out << "<td align=center>" << total << "</td>";

      out << "<td align=center>";
      if(unseen!=0) out << "<b>";
      out << unseen;
      if(unseen!=0) out << "</b>";
      out << "</td>";

      out << "</tr>\n";

      bg^=1;
     }

  out << "</TABLE></center>\n";

  showcopyright();
  endpage();
 }

