/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>

#include <tools.h>
#include <dboxpath.h>
#include <portinfo.h>
#include <logf.h>
#include "../online/userlog.h"
#include "http.h"
#include "ostatus.h"
#include "nntpgroup.h"

void checkclient(internetaddresst peer,
                 const char *statusfile,
                 char *username,
                 char *boxname)
 {
  portinfot portinfo;
  byte port=0;

  strmaxcpy(portinfo.filename, statusfile, DIRLEN-1);
 
 schleife:
  if(port<MAXPORTS)
   {
    portinfo.read(port);

    if(portinfo.info.status==portinfot::infot::SLIP &&
       portinfo.info.slipaddr==peer)
     {
      strmaxcpy(username, portinfo.info.username, TEXTLEN-1);
      strmaxcpy(boxname, portinfo.info.boxname, TEXTLEN-1);
     }
    else
     {
      port++;
      goto schleife;
     }
   }
  
 }

void checkclients(internetaddresst peer,
                  char *username, char *boxname)
 {
  DIR *dir;
  struct dirent *ent;
  char path[DIRLEN], statusfile[DIRLEN];

  snprintf(path, DIRLEN, DBOXCLIENTDIR ".", config.dboxdir, ".");

  dir=opendir(path);
  username[0]=0;

  if(dir!=NULL)
   {
    while((ent=readdir(dir))!=NULL && username[0]==0)
      if(ent->d_name[0]!='.' && ent->d_name[0]!='_')
       {
        snprintf(statusfile, DIRLEN, DBOXCLIENTDIR "portinfo", config.dboxdir, ent->d_name);
        checkclient(peer, statusfile, username, boxname);
       }

    closedir(dir);
   }
 }

const char *prevline(const char *permissions, const char *ptr)
 {
  if(ptr<=permissions) return NULL;

  ptr--;
  if(ptr<=permissions) return NULL;

  do
    ptr--;
  while(ptr!=permissions && *(ptr-1)!=0);

  return ptr;
 }

void printpermissions(const char *permissions)
 {
  userflagst userflags;
  const char *ptr=permissions, *tptr;
  char empmask[TEXTLEN];
  bool read, write, sysop, first=TRUE;

  while(*ptr!=0)
    ptr+=strlen(ptr)+1;

  ptr--;

  while((ptr=prevline(permissions, ptr))!=NULL)
   {
    if(getpermissions(ptr, empmask, &read, &write, &sysop, &userflags))
      return;

    if(!checkuserflags(&status.userprofile, &status.gruppe, &userflags))
     {
      if(first)
        first=FALSE; 
      else
        printf(",");

      if(!read) printf("!");
      tptr=empmask;
      if(*tptr=='/') tptr++;
      while(*tptr!=0)
       {
        if(*tptr=='/')
          printf(".");
        else
          printf("%c", tolower(*tptr));
        tptr++;
       }
     }
   }
 }

void nntpgroup(httpt &http, const char *ip)
 {
  internetaddresst peer;
  char boxname[TEXTLEN], tempstr[DIRLEN];
  char *permissions;

  peer=getinternetaddress(ip);
  checkclients(peer, status.userprofile.name, boxname);

  if(status.userprofile.name[0]==0 || status.userprofile.read())
   {
    logf.printf("nntpgroup", "NNTP auth request for %s", ip);
   }
  else 
   {
    strmaxcpy(status.gruppe.name, status.userprofile|"gruppe", TEXTLEN-1);

    if(!status.gruppe.read())
     {
      strmaxcpy(status.username, status.userprofile.name, TEXTLEN-1);
      snprintf(status.userlogfile, DIRLEN, "%slogfile", status.userprofile.getpath());
      userlogprintf1(status, "nntpgroup", "Sending permissions");

      snprintf(tempstr, DIRLEN, "%sbretter", status.gruppe.getpath());
      permissions=(char *)readtextconfigfile(tempstr);
      if(permissions!=NULL)
       {
        printpermissions(permissions);
        free(permissions);
       }
     }
   }

  printf("\n");
 }
