/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <fcntl.h>
#include <dirent.h>

#include <dboxpath.h>
#include <charsettools.h>
#include <account.h>
#include <portinfo.h>
#include <logins.h>
#include "ostatus.h"
#include "webtools.h"
#include "webonline.h"
#include "showstate.h"

void doshowstatus3(const char *statusfile)
 {
  int i;
  bool loggedin=FALSE;
  char tempstr[2*TEXTLEN], *bgcolor;
  bool bg=FALSE;
  portinfot portinfo;

  strmaxcpy(portinfo.filename, statusfile, DIRLEN-1);

  cout.form(
         "<table>\n"
         "<tr bgcolor=\"%s\">"
         "<th><font color=\"%s\">Port</font>"
         "<th width=200><font color=\"%s\">Login</font>"
         "<th width=200><font color=\"%s\">Name</font>"
         "\n",
         config.webonline_thbgcolor,
         config.webonline_thfcolor,
         config.webonline_thfcolor,
         config.webonline_thfcolor);

  for(i=0; i<MAXPORTS; i++)
   {
    portinfo.read(i);

    bgcolor=bg?config.webonline_tablebgcolor1:config.webonline_tablebgcolor2;

    switch(portinfo.info.status)
     {
     case portinfot::infot::OFFLINE:
      break;
  
     case portinfot::infot::WORKING:
      printf("<tr bgcolor=\"%s\"><th>%3d<td>DBOX<td>-\n", bgcolor, i);
      bg^=1;
      loggedin=TRUE;
      break;
 
     case portinfot::infot::FREE:
      printf("<tr bgcolor=\"%s\"><th>%3d<td>frei<td>-\n", bgcolor, i);
      bg^=1;
      loggedin=TRUE;
      break;
 
     case portinfot::infot::RING:
      printf("<tr bgcolor=\"%s\"><th>%3d<td>Anruf<td>-\n", bgcolor, i);
      bg^=1;
      loggedin=TRUE;
      break;
 
     case portinfot::infot::LOGIN:
      printf("<tr bgcolor=\"%s\"><th>%3d<td>Login<td>-\n", bgcolor, i);
      bg^=1;
      loggedin=TRUE;
      break;

     case portinfot::infot::ERROR:
      printf("<tr bgcolor=\"%s\"><th>%3d<td><font color=red>FEHLER</font><td>-\n", 
        bgcolor, i);
      bg^=1;
      loggedin=TRUE;
      break;
 
     case portinfot::infot::ONLINE:
     case portinfot::infot::CHATTING:
     case portinfot::infot::SLIP:
      if(portinfo.info.status==portinfot::infot::SLIP)
        snprintf(tempstr, TEXTLEN, "SLIP %s", portinfo.info.username);
      else
        strmaxcpy(tempstr, portinfo.info.username, TEXTLEN-1);

      tempstr[20]=0;
      printf("<tr bgcolor=\"%s\"><th>%3d<td>User %s", bgcolor, i, tempstr);
      bg^=1;
      loggedin=TRUE;
      if(portinfo.info.username[0]!=0)
       {
        userpt up(portinfo.info.username);
        if(!up.read())
         {
          snprintf(tempstr, TEXTLEN, "%s %s", up|"vorname", up|"nachname");
          ibm2iso(tempstr, strlen(tempstr));
          tempstr[25]=0;
          printf("<td>%s", tempstr);
         }
        cout << "\n";
       }
      break;

     case portinfot::infot::NETCALL:
      printf("<tr bgcolor=\"%s\"><th>%3d<td>Netcall %s<td>-\n",
        bgcolor, i, portinfo.info.username);
      bg^=1;

      loggedin=TRUE;
      break;
     }
   }

  cout << "</table><p>\n";

  if(!loggedin)
    cout << "\nNiemand eingeloggt!\n";
 }

void doshowstatus2(const char *client)
 {
  char statusfile[DIRLEN];
  char tempstr[TEXTLEN], description[TEXTLEN];
  bool showclient;
  clientpt cp(client);

  dnstring(cp.name);

  showclient=FALSE;
  description[0]=0;

  if(cp.read()) 
    showclient=TRUE;
  else
   {
    cp.get("showstatus", tempstr);
    if(atoi(tempstr)==1 || tempstr[0]==0) showclient=TRUE;
    cp.get("description", description);
   }

  if(showclient)
   {
    snprintf(statusfile, DIRLEN, DBOXCLIENTDIR "portinfo", 
      config.dboxdir, cp.name);

    if(description[0]!=0) printf("<p><H2>%s</h2><p>\n\n", description);
    doshowstatus3(statusfile);
   }
 }

void doshowstatus(ostream &out)
 {
  DIR *dir;
  struct dirent *ent;
  char path[DIRLEN];

  out << "Derzeit nicht verfgbar.\n";
  return;

  snprintf(path, DIRLEN, DBOXCLIENTDIR ".", config.dboxdir, ".");

  dir=opendir(path);

  if(dir!=NULL)
   {
    while((ent=readdir(dir))!=NULL)
      if(ent->d_name[0]!='.' && ent->d_name[0]!='_')
        doshowstatus2(ent->d_name);

    closedir(dir);
   }
  
 }

void doboxstatus(ostream &out)
 {
  startpage(out, "Boxstatus");
  out << "<H1>Boxstatus</h1><p>\n\n";
  doshowstatus(out);
  showcopyright(out);
  endpage(out);
 }
