/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <unistd.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <dirent.h>
#include <getopt.h>

#include <dboxpath.h>
#include <logf.h>
#include <folder.h>
#include <flags.h>
#include <charsettools.h>
#include <dboxversion.h>
#include <libdboxversion.h>
#include <account.h>
#include <getpeer.h>
#include <portinfo.h>
#include <logins.h>
#include "ostatus.h"
#include "../online/gen_from.h"
#include "../online/userlog.h"
#include "../online/alias.h"
#include "../online/checkuser.h"
#include "challenge.h"
#include "webonline.h"
#include "homepage.h"
#include "nntpgroup.h"
#include "www_prof.h"
#include "konto.h"
#include "webmail.h"
#include "webnews.h"
#include "webtools.h"
#include "webserv.h"
#include "ca.h"
#include "custom.h"
#include "dbalias.h"
#include "showstate.h"
#include "password.h"
#include "emailtab.h"

void dodebug()
 {
  struct in_addr in;

  startpage("Debug");

  //in.s_addr=peer;
  //printf("Peer: %s<p><p>\n", inet_ntoa(in));

  for(byte port=0; port<MAXPORTS; port++)
   {
    portinfo.read(port);
    if(portinfo.info.status==portinfot::infot::SLIP)
     {
      in.s_addr=portinfo.info.slipaddr;
      printf("SLIP-User: %s<br>IP: %s<p>\n", portinfo.info.username,
        inet_ntoa(in));
     }
   }

  showcopyright();
  endpage();
 }

void dohomepage_stats(httpt &http, ostream &out)
 {
  char tempstr[1000], *tptr, filename[1000];

  strmaxcpy(tempstr, http.url.c_str()+strlen("/homepage-stats/"), 900);

  tptr=strchr(tempstr, '/');
  if(tptr==NULL) return;

  *(tptr++)=0;

  userpt aliasp(tempstr);

  dnstring(aliasp.name);
  
  if(aliasp.read()) return;

  if(stricmp(aliasp.name, status.username)==0 ||
     stricmp(aliasp|"alias", status.username)==0)
   {
    snprintf(filename, DIRLEN, "%swebstats" DIRCHAR "%s",
      aliasp.getpath(), tptr);
  
    if(strstr(filename, "..")!=NULL)
     {
      startpage("Error");
      out << "Invalid Request: \" << http.command << \"\n";
      endpage();
      logf.printf("webonline", "Don't break in: %s", http.command.c_str());
      return;
     }

    dohtml_file(http, out, filename);
   }
 }

class webonline_securityt
 {
 public:
  bool pre_authenticated;
  bool secure;
  bool allow_mail, allow_news;

  webonline_securityt()
   {
    pre_authenticated=FALSE;
    secure=FALSE;
    allow_mail=allow_news=TRUE;
   }
 };

void webonline_handle_request(httpt &http, ostream &out, 
                              webonline_securityt &security)
 {
  if(http.url=="/")
    dohtml(http, out, "/webonline/");
  else if(http.url=="/boxstatus")
    doboxstatus(out);
  else if(strncmp(http.url.c_str(), "/html/", 6)==0)
    dohtml(http, out);
  else if(strncmp(http.url.c_str(), "/nntp_permission_wildmat/", 25)==0)
    nntpgroup(http, http.url.c_str()+25);
  else if(!security.secure)
   {
    if(!security.pre_authenticated)
      if(req_auth(http, "webonline")) return;

    if(strnicmp(http.url.c_str(), "/hu/", 4)==0)
      doupload(http);
    else if(strnicmp(http.url.c_str(), "/form/", 6)==0)
      dohtmlform(http, out);
    else if(http.url=="/email")
      doemail(http, out);
    else if(strncmp(http.url.c_str(), "/email/", 7)==0)
      doemail(http, out);
    else if(http.url=="/news")
      donews(http, out);
    else if(http.url=="/handshake")
      dohandshake(http);
    else if(http.url=="/konto")
      dokonto(http);
    else if(http.url=="/password")
      dopassword(http, out);
    else if(http.url=="/alias")
      doalias(http, out);
    else if(http.url=="/emailtab")
      doemailtab(http, out);
    else if(http.url=="/homepage")
      dohomepage(http);
    else if(strnicmp(http.url.c_str(), "/homepage-stats/",
                               strlen("/homepage-stats/"))==0)
      dohomepage_stats(http, out);
    else if(http.url=="/hu")
      doupload(http);
    else if(http.url=="/ca")
      doca(http);
    else
     {
      startpage(out, "DBOX WWW-Interface: Error");
      out << "DBOX WWW-Interface: invalid URL \"" 
          << http.url << "\"<P>\n";
      showcopyright(out);
      endpage(out);
     }
   }
  else
   {
    startpage("Error");
    out << "Invalid Request: \"" << http.command << "\", "
           "\"" << http.url << "\"\n";
    endpage();
    logf.printf("webonline", "Invalid Request: %s", http.command.c_str());
   }
 }

void dowebonline(const string &url, ostream &out)
 {
  webonline_securityt security;
  get_url_httpt http;
  http.parse_get_url(url);
  security.pre_authenticated=TRUE;
  webonline_handle_request(http, out, security);
 }

void dowebonline(cmdlinet &cmdline)
 {
  handle_httpt http;
  webonline_securityt security;

  fcntl(fileno(stdout), F_SETFL, 0);

  http.read_request(fileno(stdin));

  string peer;
  getpeer(peer);
  strmaxcpy(status.from, peer.c_str(), TEXTLEN-1);

  if(cmdline.isset("preauth"))
   {
    status.userprofile.setname(cmdline.getval("preauth"));
    dnstring(status.userprofile.name);
    if(!status.userprofile.read())
      security.pre_authenticated=TRUE;
   }

  security.secure=cmdline.isset("secure");

  webonline_handle_request(http, cout, security);
 }

