/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <stdio.h>
#include <sys/stat.h>

#include <fstream.h>

#include <logf.h>
#include <dboxpath.h>
#include <config.h>
#include <tools.h>

#include "ostatus.h"
#include "htmlforms.h"
#include "webtools.h"
#include "webserv.h"

void dohtml_file(httpt &http, ostream &out, const char *filename)
 {
  struct stat statbuf;
  const char *mime_type=NULL;

  stat(filename, &statbuf);

  ifstream infile(filename, ios::in|ios::bin);
  if(!infile)
   {
    startpage(out, "Error");
    out << "File not found: \"" << http.url << "\"";
    endpage(out);
    logf.printf("webonline", "File not found: %s", filename);
    return;
   }

  mime_type=get_mime_type(filename);

  out << "HTTP/1.0 200 Keep cool\r\n"
         "MIME-Version: 1.0\r\n"
         "Server: Webonline by DK\r\n";

  if(mime_type!=NULL)
    out << "Content-Type: " << mime_type << "\r\n";

  out << "Content-Length: " << statbuf.st_size << "\r\n"
         "Content-Encoding: binary\r\n"
         "Last-Modified: " << http_time(statbuf.st_mtime) << "\r\n"
         "\r\n";

  if(filecopy(infile, out, statbuf.st_size))
   {
    /* Fehler */
   }
 }

void dohtml_file_form(httpt &http, ostream &out, const char *filename)
 {
  ifstream infile(filename);

  if(!infile)
   {
    startpage(out, "Error");
    out << "File not found: \""<< http.url << "\"";
    endpage();
    logf.printf("webonline", "File not found: %s", filename);
    return;
   }

  out << "HTTP/1.0 200 Keep cool\r\n"
         "MIME-Version: 1.0\r\n"
         "Server: WebOnline by DK\r\n"
         "Content-Type: text/html\r\n"
         "Content-Encoding: binary\r\n"
         "\r\n";
  
  htmlupformst htmlupforms;
  userpt aliasp;

  const char *alias=http.getparameter("alias");
  if(alias!=NULL && alias[0]!=0)
   {
    aliasp.setname(alias);
    if(aliasp.read()) return;
    if(strcmp(aliasp|"alias", status.username)!=0) return;
    htmlupforms.setprofile(&aliasp);
   }
  else
    htmlupforms.setprofile(&status.userprofile);

  htmlupforms.serve(infile, cout);
 }

void dohtml(httpt &http, ostream &out, const char *url, bool form)
 {
  char filename[DIRLEN], tempstr[DIRLEN];
  strmaxcpy(tempstr, url, TEXTLEN-1);

   {
    int len=strlen(tempstr);
    if(len==0) return;
    if(url[len-1]=='/') strmaxcpy(tempstr+len, "index.html", DIRLEN-len-1);
   }

  snprintf(filename, DIRLEN, DBOXHTMLDIR "%s", config.dboxdir, tempstr);

  if(strstr(filename, "..")!=NULL)
   {
    startpage("Error");
    out << "Invalid Request: \"" << http.command << "\"\n";
    endpage();
    logf.printf("webonline", "Don't break in: %s", http.command.c_str());
    return;
   }

  if(form)
    dohtml_file_form(http, out, filename);
  else
    dohtml_file(http, out, filename);
 }

void dohtml(httpt &http, ostream &out)
 {
  dohtml(http, out, http.url.c_str()+6, FALSE);
 }

void dohtmlform(httpt &http, ostream &out)
 {
  dohtml(http, out, http.url.c_str()+6, TRUE);
 }