/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <sys/stat.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>

#include <string>

#include <logf.h>
#include "../webonline/www_prof.h"
#include "check_quota.h"

void get_quota(long &files, long &size, const string &path)
 {
  DIR *dir=opendir(path.c_str());

  if(dir!=NULL)
   {
    dirent *ent;
    struct stat buf;

    while((ent=readdir(dir))!=NULL)
     {
      if(strcmp(ent->d_name, ".")!=0 &&
         strcmp(ent->d_name, "..")!=0 &&
         strcmp(ent->d_name, "profile")!=0)
       {
        string file=path+"/"+ent->d_name;
        if(lstat(file.c_str(), &buf)==0)
         {
          files++;

          if(S_ISDIR(buf.st_mode))
            get_quota(files, size, file);
          else
            size+=buf.st_blocks*512;
         }
        else
          logf.printf("get_quota", "failed to stat \"%s\"", file.c_str()); 
       }
     }

    closedir(dir);
   }
 }

void check_webquota(const string &name, bool verbose)
 {
  long files=0, size=0;
  string path;

  wwwpt p(name.c_str());
  p.read();

  path=p.getpath();
  path+=".";
  get_quota(files, size, path);

  if(verbose) 
   {
    cout << name << ": " << files << " files ";
    cout.form("%0.3lf MB\n", double(size)/1000000);
   }

  if(atol(p|"files")!=files ||
     atol(p|"size")!=size)
   {
    if(verbose) cout << "  saving profile\n";
    if(p.lock())
     {
      logf.printf("get_quota", "failed to lock \"%s\"", name.c_str()); 
      if(verbose) cerr << "  failed to lock \"" << name << "\"\n";
     }
    else
     {
      p.set("files", files);
      p.set("size", size);
      p.write();
      p.unlock();
     }
   }
 }

void check_webquota(bool verbose)
 {
  if(config.homepagedir[0]==0) return;

  string path=config.homepagedir;
  path+=".";
  DIR *dir=opendir(path.c_str());

  if(dir!=NULL)
   {
    dirent *ent;
    while((ent=readdir(dir))!=NULL)
     {
      if(ent->d_name[0]!='.' && ent->d_name[0]!='_')
        check_webquota(ent->d_name, verbose);
     }

    closedir(dir);
   }
 }

