/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <sys/stat.h>
#include <ctype.h>
#include <sys/types.h>
#include <fcntl.h>
#include <netdb.h>
#include <time.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dirent.h>
#include <getopt.h>

typedef unsigned char byte;
typedef unsigned long dword;
#define TRUE 1
#define FALSE 0

typedef long long unsigned int ulargeint;

#define IMAGEURL "/html/webstat/"
#define ROOTUSER "root"
#define PER_USER_STATISTIC 50 // display a top 50
#define PER_USER_STATISTIC_STRING "50" // display a top 50
#define MAXMONTHS 12*10 // 10 years of statistics

char *bgcolor="#ffffe0", 
     *thfcolor="#ffffff",
     *thbgcolor="#ff0000",
     *tablebgcolor="#efefef",
     *totalbgcolor="#c9c9c9";

const char *monthnames[]=
 {
  "January",
  "February",
  "March",
  "April",
  "May",
  "June",
  "July",
  "August",
  "September",
  "October",
  "November",
  "December",
  NULL
 };

char *logsubdir="",
     *prefix="",
     *filter="",
     *user="",
     *userdir="";
bool overwrite_cache=FALSE;

void help()
 {
  fprintf(stderr, "Aufruf:\n"
                  "  webstat [Optionen]\n"
                  "\n"
                  "Optionen:\n"
                  "  -p Prefix (z.B. /htdocs)\n"
                  "  -f Filter (z.B. /userseiten)\n"
                  "  -u Statistiken fr User berechnen (z.B. hmueller)\n"
                  "  -d Verzeichnis, das die Userverzeichnisse enthlt\n"
                  "  -l Unterverzeichnis fr Statistiken (z.B. stats)\n");
  exit(1);
 }

time_t parsetime(const char *cp)
 {
  struct tm *tp;
  struct tm tm_struct;
  const char *tm;

	tp=&tm_struct;

	if (strlen(cp) < 25 ||
	    cp[2] != '/' || cp[6] != '/' || cp[11] != ':' ||
	    cp[14] != ':' || cp[17] != ':') return 0;

	tm = cp;

	/*
	** Parse a date string in format DD/MMM/YYYY:HH:MM:SS
	** and fill in the elements of the LOGTIME structure.
	*/
	tp->tm_hour = (u_short)((tm[12]-'0') * 10  + (tm[13]-'0'));
	tp->tm_min  = (u_short)((tm[15]-'0') * 10  + (tm[16]-'0'));
	tp->tm_sec  = (u_short)((tm[18]-'0') * 10  + (tm[19]-'0'));
	tp->tm_mday = (u_short)((tm[0]-'0')  * 10   + (tm[1]-'0'));
	tp->tm_year = (u_short)((tm[7]-'0')  * 1000 + (tm[8]-'0')*100 +
			        (tm[9]-'0')  * 10   + (tm[10]-'0'))-1900;

	switch (tm[4]) {
	  case 'a':		/* jan, mar, may */
		switch (tm[5]) {
		  case 'n':	tp->tm_mon = 0;	break;
		  case 'r':	tp->tm_mon = 2;	break;
		  default:	tp->tm_mon = 4;	break;
		}
		break;

	  case 'u':		/* jun, jul, aug */
		switch (tm[5]) {
		  case 'n':	tp->tm_mon = 5;	break;
		  case 'l':	tp->tm_mon = 6;	break;
		  default:	tp->tm_mon = 7;	break;
		}
		break;

	  case 'e':		/* feb, sec, dec */
		switch (tm[3]) {
		  case 'F':	tp->tm_mon = 1;	break;
		  case 'S':	tp->tm_mon = 8;	break;
		  default:	tp->tm_mon = 11;break;
		}
		break;

	  default:		/* apr, oct, nov */
		switch (tm[3]) {
		  case 'A':	tp->tm_mon = 3;	break;
		  case 'O':	tp->tm_mon = 9;	break;
		  default:	tp->tm_mon = 10;break;
		}
		break;
	}

  return mktime(tp);
 }

char *copyentry(char *start, char *dest, int max, char separator)
 {
  if(separator!=' ' && *start==separator)
    start++;

  while(1)
   {
    if(*start==0) break;
    if(*start==separator) { start++; break; }

    if(max!=0)
     {
      *(dest++)=*(start++);
      max--;
     }
    else
      start++;
   }

  *dest=0;

  if(separator!=' ' && *start==' ')
    start++;

  return start;
 }

int match(char *a, char *b)
 {
  return (strncmp(a, b, strlen(b))==0);
 }

void gendir(const char *user, char *dir)
 {
  sprintf(dir, "%s/%s/%s", userdir, user, logsubdir);
 }

void logline(char *line, char *user, char *date)
 {
  FILE *outfile;
  char dir[10000], file[10000];
  time_t t;
  struct tm *tm;

  t=parsetime(date);
  tm=localtime(&t);

  if(user[0]==0)
    gendir(ROOTUSER, dir);
  else
    gendir(user, dir);

  if(access(dir, 0)!=0)
    if(mkdir(dir, 0700)!=0)
     {
      fprintf(stderr, "Failed to create %s\n", dir);
      return;
     }

  sprintf(file, "%s/%d-%d.log", dir, tm->tm_year+1900, tm->tm_mon+1);

  outfile=fopen(file, "at");
  if(outfile==NULL)
   {
    fprintf(stderr, "Failed to create %s\n", file);
    return;
   }

  fprintf(outfile, "%s\n", line);

  fclose(outfile);
 }

void collect()
 {
  char line[1000], *tptr, *tptr2;
  char host[100], file[1000], bytes[100], date[100], f[1000];

  sprintf(f, "%s%s", prefix, filter);

  while(fgets(line, 900, stdin)!=NULL)
   {
    tptr=strchr(line, '\r'); if(tptr!=NULL) *tptr=0;
    tptr=strchr(line, '\n'); if(tptr!=NULL) *tptr=0;

    tptr=line;
    tptr=copyentry(tptr, host, 99, ' ');

    if(*tptr!='[') continue;
    tptr++;
    tptr=copyentry(tptr, date, 99, ']');
    tptr=copyentry(tptr, file, 999, '"');
    tptr=copyentry(tptr, bytes, 99, ' ');

    if(match(file, f))
     {
      tptr=file+strlen(f);
      if(*tptr=='/') tptr++;
      tptr2=strchr(tptr, '/');
      if(tptr2==NULL) continue;
      *tptr2=0;

      logline(line, tptr, date);
     }
    else if(match(file, prefix))
     {
      logline(line, "", date);
     }
   }
 }

struct countt1
 {
  struct countt2
   {
    ulargeint by_weekday[7];
    ulargeint by_hour[24];
    ulargeint by_day[31];
    ulargeint total;

    countt2()
     {
      int i;
      for(i=0; i<7; i++) by_weekday[i]=0;
      for(i=0; i<24; i++) by_hour[i]=0;
      for(i=0; i<31; i++) by_day[i]=0;
      total=0;
     }
   } hits, pageviews, volume;
 };

struct montht
 {
  byte month;
  dword year;
 };

class makestatt
 {
 public:
  // directory

  char dir[10000];

  countt1 s_total;

  // functions

  bool processmonth(const char *filename, montht *current_month);
  void hour_averages(FILE *outfile);
  void makestat();

  void add(countt1::countt2 *counter, dword value, struct tm *tm);
  void add(countt1 *counter, countt1 *value);
  void print(FILE *outfile, countt1 *counter, byte what, int index, countt1 *max);
  void printvolume(FILE *outfile, ulargeint volume);
  void per_user_statistics(FILE *outfile);
  void printbar(FILE *outfile, byte percent);
 };

void makestatt::printbar(FILE *outfile, byte percent)
 {
  if(percent==0)
    fprintf(outfile, "&nbsp;");
  else
    fprintf(outfile, "<img alt=\"\" "
                     "src=\"%sdot_red.gif\" height=10 width=%d>",
      IMAGEURL, percent);
 }

void makestatt::add(countt1::countt2 *counter, dword value, struct tm *tm)
 {
  counter->total+=value;
  counter->by_day[tm->tm_mday-1]+=value;
  counter->by_hour[tm->tm_hour]+=value;
  counter->by_weekday[tm->tm_wday]+=value;
 }

void makestatt::add(countt1 *counter, countt1 *value)
 {
  int i;

  counter->hits.total+=value->hits.total;
  counter->pageviews.total+=value->pageviews.total;
  counter->volume.total+=value->volume.total;

  for(i=0; i<31; i++)
   {
    counter->hits.by_day[i]+=value->hits.by_day[i];
    counter->pageviews.by_day[i]+=value->pageviews.by_day[i];
    counter->volume.by_day[i]+=value->volume.by_day[i];
   }

  for(i=0; i<24; i++)
   {
    counter->hits.by_hour[i]+=value->hits.by_hour[i];
    counter->pageviews.by_hour[i]+=value->pageviews.by_hour[i];
    counter->volume.by_hour[i]+=value->volume.by_hour[i];
   }

  for(i=0; i<7; i++)
   {
    counter->hits.by_weekday[i]+=value->hits.by_weekday[i];
    counter->pageviews.by_weekday[i]+=value->pageviews.by_weekday[i];
    counter->volume.by_weekday[i]+=value->volume.by_weekday[i];
   }
 }

char *volume_units[]=
 {
  "Bytes",
  "KB",
  "MB",
  "GB",
  NULL
 };

void makestatt::printvolume(FILE *outfile, ulargeint volume)
 {
  int i;
  dword rest=0;

  for(i=0; TRUE; i++)
   {
    if(volume<1000 || volume_units[i+1]==NULL)
     {
      fprintf(outfile, "%llu", volume);
      if(volume<10 && i>0)
        fprintf(outfile, ",%lu", 10*rest/1024);
      fprintf(outfile, " %s", volume_units[i]);
      return;
     }

    rest=(volume&1023);
    volume>>=10;
   } 
 }

void makestatt::print(FILE *outfile, 
                      countt1 *counter,
                      byte what,
                      int index,
                      countt1 *max)
 {
  switch(what)
   {
   case 0: // total
    fprintf(outfile, "<td align=right>%llu<td>", counter->hits.total);
    if(max!=NULL && max->hits.total!=0)
      printbar(outfile, 100*counter->hits.total/max->hits.total);
    else
      fprintf(outfile, "-");

    fprintf(outfile, "<td align=right>%llu<td>", counter->pageviews.total);
    if(max!=NULL && max->pageviews.total!=0)
      printbar(outfile, 100*counter->pageviews.total/max->pageviews.total);
    else
      fprintf(outfile, "-");

    fprintf(outfile, "<td align=right>");
    printvolume(outfile, counter->volume.total);
    fprintf(outfile, "<td>");
    if(max!=NULL && max->volume.total!=0)
      printbar(outfile, 100*counter->volume.total/max->volume.total);
    else
      fprintf(outfile, "-");

    break;

   case 1: // by weekday
    fprintf(outfile, "<td align=right>%llu<td>", counter->hits.by_weekday[index]);
    if(max!=NULL && max->hits.total!=0)
      printbar(outfile, 100*counter->hits.by_weekday[index]/max->hits.total);
    else
      fprintf(outfile, "-");

    fprintf(outfile, "<td align=right>%llu<td>", counter->pageviews.by_weekday[index]);
    if(max!=NULL && max->pageviews.total!=0)
      printbar(outfile, 100*counter->pageviews.by_weekday[index]/max->pageviews.total);
    else
      fprintf(outfile, "-");

    fprintf(outfile, "<td align=right>");
    printvolume(outfile, counter->volume.by_weekday[index]);
    fprintf(outfile, "<td>");
    if(max!=NULL && max->volume.total!=0)
      printbar(outfile, 100*counter->volume.by_weekday[index]/max->volume.total);
    else
      fprintf(outfile, "-");

    break;
 
   case 2: // by day
    fprintf(outfile, "<td align=right>%llu<td>", counter->hits.by_day[index]);
    if(max!=NULL && max->hits.total!=0)
      printbar(outfile, 100*counter->hits.by_day[index]/max->hits.total);
    else
      fprintf(outfile, "-");

    fprintf(outfile, "<td align=right>%llu<td>", counter->pageviews.by_day[index]);
    if(max!=NULL && max->pageviews.total!=0)
      printbar(outfile, 100*counter->pageviews.by_day[index]/max->pageviews.total);
    else
      fprintf(outfile, "-");

    fprintf(outfile, "<td align=right>");
    printvolume(outfile, counter->volume.by_day[index]);
    fprintf(outfile, "<td>");
    if(max!=NULL && max->volume.total!=0)
      printbar(outfile, counter->volume.by_day[index]/max->volume.total);
    else
      fprintf(outfile, "-");

    break;
 
   case 3: // by hour
    fprintf(outfile, "<td align=right>%llu<td>", counter->hits.by_hour[index]);
    if(max!=NULL && max->hits.total!=0)
      printbar(outfile, 100*counter->hits.by_hour[index]/max->hits.total);
    else
      fprintf(outfile, "-");

    fprintf(outfile, "<td align=right>%llu<td>", counter->pageviews.by_hour[index]);
    if(max!=NULL && max->pageviews.total!=0)
      printbar(outfile, 100*counter->pageviews.by_hour[index]/max->pageviews.total);
    else
      fprintf(outfile, "-");

    fprintf(outfile, "<td align=right>");
    printvolume(outfile, counter->volume.by_hour[index]);
    fprintf(outfile, "<td>");
    if(max!=NULL && max->volume.total!=0)
      printbar(outfile, counter->volume.by_hour[index]/max->volume.total);
    else
      fprintf(outfile, "-");

    break;
   }
 }

bool makestatt::processmonth(const char *filename, montht *currentmonth)
 {
  char dir[1000], line[1000];
  char logfile[1000], htmlfile[1000], cachefile[1000], file[1000], *tptr;
  FILE *infile, *outfile;
  char host[100], bytes[100], date[100];
  int i;
  countt1 s_day, s_month, s_max;
  time_t t;
  struct tm *tm;
  struct stat statbuf1, statbuf2;
  byte month;
  dword year;
  time_t firstentry=0, lastentry=0;

  // generate directory

  gendir(user, dir);

  // parse filename for month
 
  year=atol(filename);
  tptr=strchr(filename, '-');
  if(tptr==NULL) return TRUE;
  month=atoi(tptr+1);
  currentmonth->month=month;
  currentmonth->year=year;

  // generate file names

  sprintf(logfile, "%s/%s", dir, filename);
  sprintf(htmlfile, "%s/%lu-%u.html", dir, year, month);
  sprintf(cachefile, "%s/%lu-%u.dat", dir, year, month);

  // check if cached

  if(stat(logfile, &statbuf1)==0 &&
     stat(cachefile, &statbuf2)==0 &&
     !overwrite_cache)
   {
    if(statbuf1.st_mtime<=statbuf2.st_mtime)
      return FALSE;
   }
 
  // open log file

  infile=fopen(logfile, "rt");
  if(infile==NULL) return TRUE;

  // open out file

  outfile=fopen(htmlfile, "wt");
  if(outfile==NULL) { fclose(infile); return TRUE; }

  // read file

  while(fgets(line, 900, infile)!=NULL)
   {
    tptr=strchr(line, '\r'); if(tptr!=NULL) *tptr=0;
    tptr=strchr(line, '\n'); if(tptr!=NULL) *tptr=0;

    tptr=line;
    tptr=copyentry(tptr, host, 99, ' ');

    if(*tptr!='[') continue;
    tptr++;
    tptr=copyentry(tptr, date, 99, ']');
    tptr=copyentry(tptr, file, 999, '"');
    tptr=copyentry(tptr, bytes, 99, ' ');

    t=parsetime(date);
    tm=localtime(&t);

    if(firstentry>t && t!=0) firstentry=t;
    if(lastentry<t) lastentry=t;

    tptr=strrchr(file, '.');
    if(tptr!=NULL && strncasecmp(tptr, ".htm", 4)==0)
     {
      add(&s_month.pageviews, 1, tm);
      add(&s_day.pageviews, 1, tm);
     }

    add(&s_month.hits, 1, tm);
    add(&s_day.hits, 1, tm);

    add(&s_month.volume, atol(bytes), tm);
    add(&s_day.volume, atol(bytes), tm);
   }

  // print day statistics

  fprintf(outfile, 
    "<html>\n"
    "<head>\n"
    "<title>\n"
    "Homepage Statistics for %s, %s %lu\n"
    "</title>\n"
    "</head>\n"
    "\n", user, monthnames[month-1], year);

  fprintf(outfile, 
    "<body bgcolor=\"%s\">\n"
    "\n"
    "<center>\n"
    "<h2>%s Access Summary for %s %lu</h2>\n"
    "<table>\n"
    "<tr bgcolor=\"%s\"><th><font color=\"%s\">Day</font>"
    "<th colspan=2><font color=\"%s\">Hits</font>"
    "<th colspan=2><font color=\"%s\">Pageviews</font>"
    "<th colspan=2><font color=\"%s\">Volume</font>"
    "\n", bgcolor, user, monthnames[month-1], year, thbgcolor,
    thfcolor, thfcolor, thfcolor, thfcolor);

  time(&t);
  tm=localtime(&t);

  for(i=0; i<31; i++)
   {
    if(s_day.hits.by_day[i]>s_max.hits.total)
      s_max.hits.total=s_day.hits.by_day[i];

    if(s_day.pageviews.by_day[i]>s_max.pageviews.total)
      s_max.pageviews.total=s_day.pageviews.by_day[i];

    if(s_day.volume.by_day[i]>s_max.volume.total)
      s_max.volume.total=s_day.volume.by_day[i];
   }

  for(i=0; i<31; i++)
   {
    tm->tm_mday=i+1;
    tm->tm_year=year-1900;
    tm->tm_mon=(month-1);
    tm->tm_hour=tm->tm_min=tm->tm_sec=0;
    tm->tm_sec=0;
    tm->tm_min=0;
    tm->tm_isdst=0;
    if(mktime(tm)>time(NULL)) break;
    if(tm->tm_mon!=(month-1)) break;

    fprintf(outfile, "<tr bgcolor=\"%s\"><td>%d.%d.", tablebgcolor, i+1, month);
    print(outfile, &s_day, 2, i, &s_max);
    fprintf(outfile, "\n");
   }

  fprintf(outfile, "<tr bgcolor=\"%s\"><td><b>TOTAL</b>\n", totalbgcolor);
  print(outfile, &s_day, 0, i, NULL);
  fprintf(outfile, "\n");

  fprintf(outfile, "</table>\n");

  fprintf(outfile, "\n</body>\n"
                   "</html>\n");

  // close log file and out file

  fclose(infile);
  fclose(outfile);

  // write cache

  outfile=fopen(cachefile, "wb");
  if(outfile!=NULL)
   {
    fwrite(&s_month, 1, sizeof(s_month), outfile);
    fwrite(&firstentry, 1, sizeof(firstentry), outfile);
    fwrite(&lastentry, 1, sizeof(lastentry), outfile);
    fclose(outfile);
   }

  return FALSE;
 }

void makestatt::hour_averages(FILE *outfile)
 {
  int i;
  dword days;
  ulargeint max_hits=0, max_pageviews=0;
  struct tm *tm;
  time_t t;
  char this_month_file[1000];
  countt1 this_month;  
  FILE *infile;
  time_t this_month_firstentry, this_month_lastentry;

  time(&t);
  tm=localtime(&t);
  sprintf(this_month_file, "%s/%d-%d.dat", dir, tm->tm_year+1900, tm->tm_mon+1);

  infile=fopen(this_month_file, "rb");
  if(infile==NULL) return;
  fread(&this_month, 1, sizeof(countt1), infile);
  fread(&this_month_firstentry, 1, sizeof(time_t), infile);
  fread(&this_month_lastentry, 1, sizeof(time_t), infile);
  fclose(infile);

  //if((this_month_lastentry-this_month_firstentry)==0) return;
  //days=((this_month_lastentry-this_month_firstentry)/60/60/24)+1;
  days=tm->tm_mday;
  //printf("%ld\n", days);

  fprintf(outfile, 
    "<h2>Access Summary by Hour</h2>\n"
    "<table>\n"
    "<tr bgcolor=\"%s\"><th><font color=\"%s\">Hour</font>"
    "<th colspan=2><font color=\"%s\">Hits total</font>"
    "<th><font color=\"%s\">average</font>"
    "<th colspan=2><font color=\"%s\">Pageviews total</font>"
    "<th><font color=\"%s\">average</font>\n",
    thbgcolor,
    thfcolor, thfcolor, thfcolor, thfcolor, thfcolor);

  for(i=0; i<24; i++)
   {
    if(this_month.hits.by_hour[i]>max_hits) max_hits=this_month.hits.by_hour[i];
    if(this_month.pageviews.by_hour[i]>max_pageviews) max_pageviews=this_month.pageviews.by_hour[i];
   }

  for(i=0; i<24; i++)
   {
    fprintf(outfile, "<tr bgcolor=\"%s\"><td align=center>%d", tablebgcolor, i);

    fprintf(outfile, "<td align=right>%llu", this_month.hits.by_hour[i]);
    fprintf(outfile, "<td>");
    if(max_hits>0) printbar(outfile, 100*this_month.hits.by_hour[i]/max_hits);
    fprintf(outfile, "<td align=right>%.2f", (double)this_month.hits.by_hour[i]/days);

    fprintf(outfile, "<td align=right>%llu", this_month.pageviews.by_hour[i]);
    fprintf(outfile, "<td>");
    if(max_pageviews>0) printbar(outfile, 100*this_month.pageviews.by_hour[i]/max_pageviews);
    fprintf(outfile, "<td align=right>%.2f", (double)this_month.pageviews.by_hour[i]/days);

    fprintf(outfile, "\n");
   }

  fprintf(outfile, "</table>\n");
 }

struct userinfot
 {
  char *name;
  countt1 total, this_month, last_month;
 } *userinfos;

int cmpuser_pageviews_total(const void *a, const void *b)
 {
  if(((userinfot *)a)->total.pageviews.total>
     ((userinfot *)b)->total.pageviews.total)
    return -1;

  if(((userinfot *)a)->total.pageviews.total<
     ((userinfot *)b)->total.pageviews.total)
    return 1;

  return 0;
 }

int cmpuser_pageviews_this_month(const void *a, const void *b)
 {
  if(((userinfot *)a)->this_month.pageviews.total>
     ((userinfot *)b)->this_month.pageviews.total)
    return -1;

  if(((userinfot *)a)->this_month.pageviews.total<
     ((userinfot *)b)->this_month.pageviews.total)
    return 1;

  return 0;
 }

int cmpuser_pageviews_last_month(const void *a, const void *b)
 {
  if(((userinfot *)a)->last_month.pageviews.total>
     ((userinfot *)b)->last_month.pageviews.total)
    return -1;

  if(((userinfot *)a)->last_month.pageviews.total<
     ((userinfot *)b)->last_month.pageviews.total)
    return 1;

  return 0;
 }

int cmpuser_hits_total(const void *a, const void *b)
 {
  if(((userinfot *)a)->total.hits.total>
     ((userinfot *)b)->total.hits.total)
    return -1;

  if(((userinfot *)a)->total.hits.total<
     ((userinfot *)b)->total.hits.total)
    return 1;

  return 0;
 }

int cmpuser_hits_this_month(const void *a, const void *b)
 {
  if(((userinfot *)a)->this_month.hits.total>
     ((userinfot *)b)->this_month.hits.total)
    return -1;

  if(((userinfot *)a)->this_month.hits.total<
     ((userinfot *)b)->this_month.hits.total)
    return 1;

  return 0;
 }

int cmpuser_hits_last_month(const void *a, const void *b)
 {
  if(((userinfot *)a)->last_month.hits.total>
     ((userinfot *)b)->last_month.hits.total)
    return -1;

  if(((userinfot *)a)->last_month.hits.total<
     ((userinfot *)b)->last_month.hits.total)
    return 1;

  return 0;
 }

int cmpuser_volume_total(const void *a, const void *b)
 {
  if(((userinfot *)a)->total.volume.total>
     ((userinfot *)b)->total.volume.total)
    return -1;

  if(((userinfot *)a)->total.volume.total<
     ((userinfot *)b)->total.volume.total)
    return 1;

  return 0;
 }

int cmpuser_volume_this_month(const void *a, const void *b)
 {
  if(((userinfot *)a)->this_month.volume.total>
     ((userinfot *)b)->this_month.volume.total)
    return -1;

  if(((userinfot *)a)->this_month.volume.total<
     ((userinfot *)b)->this_month.volume.total)
    return 1;

  return 0;
 }

int cmpuser_volume_last_month(const void *a, const void *b)
 {
  if(((userinfot *)a)->last_month.volume.total>
     ((userinfot *)b)->last_month.volume.total)
    return -1;

  if(((userinfot *)a)->last_month.volume.total<
     ((userinfot *)b)->last_month.volume.total)
    return 1;

  return 0;
 }

int cmpuser_name(const void *a, const void *b)
 {
  return strcasecmp(((userinfot *)a)->name, ((userinfot *)b)->name);
 }

char *user_sort_list[]=
 {
  "Username",
  "Hits Total",
  "Pageviews Total",
  "Volume Total",
  "Hits this Month",
  "Pageviews this Month",
  "Volume this Month",
  "Hits last Month",
  "Pageviews last Month",
  "Volume last Month",
  NULL
 };

char *user_sort_list_links[]=
 {
  "username",
  "hits-total",
  "pageviews-total",
  "volume-total",
  "hits-this-month",
  "pageviews-this-month",
  "volume-this-month",
  "hits-last-month",
  "pageviews-last-month",
  "volume-last-month",
  NULL
 };

char *user_view_list[]=
 {
  "Top " PER_USER_STATISTIC_STRING,
  "All",
  NULL
 };

char *user_view_list_links[]=
 {
  "top",
  "all",
  NULL
 };

void makestatt::per_user_statistics(FILE *outfile)
 {
  DIR *d;
  dirent *ent;
  char dir[1000], cachefile[1000], userfile[1000];
  int number_of_users=0, i, j, k;
  FILE *infile, *user_outfile;
  userinfot *userinfos;
  char this_month_file[1000], last_month_file[1000];
  struct tm *tm;
  time_t t;
  // countt1 s_max;

  time(&t);
  tm=localtime(&t);
  sprintf(this_month_file, "%d-%d.dat", tm->tm_year+1900, tm->tm_mon+1);
  tm->tm_mon--;
  mktime(tm);
  sprintf(last_month_file, "%d-%d.dat", tm->tm_year+1900, tm->tm_mon+1);
   
  d=opendir(userdir);
  if(d==NULL) return; 

  while((ent=readdir(d))!=NULL)
    if(ent->d_name[0]!='_' && ent->d_name[0]!='.')
     {
      gendir(ent->d_name, dir);
      sprintf(cachefile, "%s/summary.dat", dir);
      if(access(cachefile, 00)==0)
        number_of_users++;
     }

  closedir(d);

  if(number_of_users>0)
   {
    userinfos=(userinfot *)malloc(number_of_users*sizeof(userinfot));
    if(userinfos==NULL) abort();

    d=opendir(userdir);
    if(d==NULL) return; 

    i=0;
    while((ent=readdir(d))!=NULL && i<number_of_users)
      if(ent->d_name[0]!='_' && ent->d_name[0]!='.')
       {
        gendir(ent->d_name, dir);
        sprintf(cachefile, "%s/summary.dat", dir);
        infile=fopen(cachefile, "rb");
        if(infile!=NULL)
         {
          userinfos[i].name=strdup(ent->d_name);
          fread(&userinfos[i].total, 1, sizeof(countt1), infile);
          fclose(infile);

          sprintf(cachefile, "%s/%s", dir, this_month_file);
          infile=fopen(cachefile, "rb");
          if(infile!=NULL)
           {
            fread(&userinfos[i].this_month, 1, sizeof(countt1), infile);
            fclose(infile);
           }

          sprintf(cachefile, "%s/%s", dir, last_month_file);
          infile=fopen(cachefile, "rb");
          if(infile!=NULL)
           {
            fread(&userinfos[i].last_month, 1, sizeof(countt1), infile);
            fclose(infile);
           }

          i++;
         }
       }

    closedir(d);
    number_of_users=i;

    fprintf(outfile, 
      "<h2>Access Summary by User</h2>\n"
      "<table>\n"
      "<tr bgcolor=\"%s\">"
      "<th><font color=\"%s\">Sort by</font>",
      thbgcolor,
      thfcolor);

    for(j=0; user_view_list[j]!=NULL; j++)
      fprintf(outfile, 
        "<th><font color=\"%s\">%s</font>",
        thfcolor, user_view_list[j]);

    fprintf(outfile, "\n");

    for(i=0; user_sort_list[i]!=NULL; i++)
     {
      switch(i)
       {
       case 0: qsort(userinfos, number_of_users, sizeof(userinfot), cmpuser_name); break;
       case 1: qsort(userinfos, number_of_users, sizeof(userinfot), cmpuser_hits_total); break;
       case 2: qsort(userinfos, number_of_users, sizeof(userinfot), cmpuser_pageviews_total); break;
       case 3: qsort(userinfos, number_of_users, sizeof(userinfot), cmpuser_volume_total); break;
       case 4: qsort(userinfos, number_of_users, sizeof(userinfot), cmpuser_hits_this_month); break;
       case 5: qsort(userinfos, number_of_users, sizeof(userinfot), cmpuser_pageviews_this_month); break;
       case 6: qsort(userinfos, number_of_users, sizeof(userinfot), cmpuser_volume_this_month); break;
       case 7: qsort(userinfos, number_of_users, sizeof(userinfot), cmpuser_hits_last_month); break;
       case 8: qsort(userinfos, number_of_users, sizeof(userinfot), cmpuser_pageviews_last_month); break;
       case 9: qsort(userinfos, number_of_users, sizeof(userinfot), cmpuser_volume_last_month); break;
       }

      fprintf(outfile, "<tr bgcolor=\"%s\"><td>%s", 
        tablebgcolor, user_sort_list[i]);

      for(j=0; user_view_list[j]!=NULL; j++)
       {
        fprintf(outfile, "<td align=center><a href=\"%s-user-by-%s.html\">View</a>",
          user_view_list_links[j], user_sort_list_links[i]);

        gendir(ROOTUSER, dir);
        sprintf(userfile, "%s/%s-user-by-%s.html", dir,
          user_view_list_links[j], user_sort_list_links[i]);
        user_outfile=fopen(userfile, "wt");
        if(user_outfile!=NULL)
         {
          countt1 s_total;

          fprintf(user_outfile, 
            "<html>\n"
            "<head>\n"
            "<title>\n"
            "User Homepage Statistics Sorted by %s\n"
            "</title>\n"
            "</head>\n"
            "\n", user_sort_list[i]);

         fprintf(user_outfile, 
           "<body bgcolor=\"%s\">\n"
           "\n"
           "<center>\n"
           "<h2>User Homepage Access Summary Sorted by %s</h2>\n"
           "<table>\n"
           "<tr bgcolor=\"%s\"><th><font color=\"%s\">User</font>"
           "<th colspan=2><font color=\"%s\">Hits</font>"
           "<th colspan=2><font color=\"%s\">Pageviews</font>"
           "<th colspan=2><font color=\"%s\">Volume</font>"
           "\n", bgcolor, user_sort_list[i], thbgcolor,
           thfcolor, thfcolor, thfcolor, thfcolor);

          for(k=0; k<number_of_users && (j==1 || k<PER_USER_STATISTIC); k++)
           {
            fprintf(user_outfile, "<tr bgcolor=\"%s\"><td>%s", 
              tablebgcolor, userinfos[k].name);

            switch(i)
             {
             case 0:
             case 1:
             case 2:
             case 3: print(user_outfile, &userinfos[k].total, 0, 0, NULL);
                     add(&s_total, &userinfos[k].total);
                     break;
             case 4:
             case 5:
             case 6: print(user_outfile, &userinfos[k].this_month, 0, 0, NULL);
                     add(&s_total, &userinfos[k].this_month);
                     break;
             case 7:
             case 8:
             case 9: print(user_outfile, &userinfos[k].last_month, 0, 0, NULL);
                     add(&s_total, &userinfos[k].last_month);
                     break;
             }

            fprintf(user_outfile, "\n");
           }

          fprintf(user_outfile, "<tr bgcolor=\"%s\"><td><b>TOTAL</b>\n", totalbgcolor);
          print(user_outfile, &s_total, 0, i, NULL);
          fprintf(user_outfile, "\n");

          fprintf(user_outfile, "</table>\n");

          fprintf(user_outfile, "\n</body>\n"
                           "</html>\n");
        
          fclose(user_outfile);
         }
       }

      fprintf(outfile, "\n");
     }

    fprintf(outfile, "</table>\n");

    for(i=0; i<number_of_users; i++)
      free(userinfos[i].name);

    free(userinfos);
   }
 }

static int cmpmontht(const void *a, const void *b)
 {
  if(((montht *)a)->year>((montht *)b)->year)
    return 1;

  if(((montht *)a)->year<((montht *)b)->year)
    return -1;

  if(((montht *)a)->month>((montht *)b)->month)
    return 1;

  if(((montht *)a)->month<((montht *)b)->month)
    return -1;

  return 0;
 }

void makestatt::makestat()
 {
  char *tptr;
  DIR *d;
  dirent *ent;
  char htmlfile[1000], cachefile[1000], logfile[1000];
  FILE *outfile, *infile;
  bool showlogfiles=TRUE;
  int number_of_months=0, i;
  montht months[MAXMONTHS];
  countt1 s_max;

  gendir(user, dir);

  // read logfiles

  d=opendir(dir);

  if(d==NULL) return;

  while((ent=readdir(d))!=NULL)
   {
    tptr=strrchr(ent->d_name, '.');
    if(tptr!=NULL)
     {
      if((strcasecmp(tptr, ".log")==0 ||
         strcasecmp(tptr, ".html")==0) && 
         isdigit(ent->d_name[0]))
       {
        montht currentmonth;

        if(!processmonth(ent->d_name, &currentmonth))
         {
          for(i=0; i<number_of_months; i++) 
            if(months[i].month==currentmonth.month &&
               months[i].year==currentmonth.year)
              break;

          if(i==number_of_months)
           {
            months[i]=currentmonth;
            number_of_months++;
           }
         }
       }
     }
   }

  closedir(d);

  // sort by time

  qsort(months, number_of_months, sizeof(montht), &cmpmontht);

  // generate statistics

  sprintf(htmlfile, "%s/index.html", dir);
  outfile=fopen(htmlfile, "wt");
  if(outfile==NULL) return;
  fprintf(outfile, 
    "<html>\n"
    "<head>\n"
    "<title>\n"
    "Homepage Statistics for %s\n"
    "</title>\n"
    "</head>\n"
    "\n", user);

  fprintf(outfile, 
    "<body bgcolor=\"%s\">\n"
    "\n"
    "<center>\n"
    "<h2>%s Access Summary by Month</h2>\n"
    "<table>\n"
    "<tr bgcolor=\"%s\"><th><font color=\"%s\">Month</font>"
    "<th colspan=2><font color=\"%s\">Hits</font>"
    "<th colspan=2><font color=\"%s\">Pageviews</font>"
    "<th colspan=2><font color=\"%s\">Volume</font>"
    "<th><font color=\"%s\">Detail</font>\n", bgcolor, user,
    thbgcolor,
    thfcolor, thfcolor, thfcolor, thfcolor, thfcolor);

  if(showlogfiles)
    fprintf(outfile, "<th><font color=\"%s\">Logfile</font>\n", thfcolor);

  for(i=0; i<number_of_months; i++)
   {
    sprintf(cachefile, "%s/%lu-%u.dat", dir, months[i].year, months[i].month);
    infile=fopen(cachefile, "rb");
    if(infile!=NULL)
     {
      countt1 s_month;
      fread(&s_month, 1, sizeof(s_month), infile);
      fclose(infile);

      add(&s_total, &s_month);

      if(s_month.hits.total>s_max.hits.total)
        s_max.hits.total=s_month.hits.total;
      if(s_month.pageviews.total>s_max.pageviews.total)
        s_max.pageviews.total=s_month.pageviews.total;
      if(s_month.volume.total>s_max.volume.total)
        s_max.volume.total=s_month.volume.total;
     }
   }

  for(i=0; i<number_of_months; i++)
   {
    sprintf(cachefile, "%s/%lu-%u.dat", dir, months[i].year, months[i].month);
    infile=fopen(cachefile, "rb");
    if(infile!=NULL)
     {
      countt1 s_month;
      fread(&s_month, 1, sizeof(s_month), infile);
      fclose(infile);

      fprintf(outfile, "<tr bgcolor=\"%s\"><td>%s %lu", 
        tablebgcolor, monthnames[months[i].month-1], months[i].year);

      print(outfile, &s_month, 0, 0, &s_total);

      fprintf(outfile, "<td><a href=\"%lu-%u.html\">View</a>",
        months[i].year, months[i].month);

      if(showlogfiles)
       {
        sprintf(logfile, "%s/%lu-%u.log", dir, months[i].year, months[i].month);
        if(access(logfile, 00)==0)
          fprintf(outfile, "<td><a href=\"%lu-%u.log\">View</a>",
            months[i].year, months[i].month);
        else
          fprintf(outfile, "<td>-");
       }

      fprintf(outfile, "\n");
     }
   }

  fprintf(outfile, "<tr bgcolor=\"%s\"><td><b>TOTAL</b>\n", totalbgcolor);
  print(outfile, &s_total, 0, 0, NULL);
  fprintf(outfile, "<td align=center>-");
  fprintf(outfile, "<td align=center>-");
  fprintf(outfile, "\n");

  fprintf(outfile, "</table>\n");

  // print hour averages

  hour_averages(outfile);

  if(strcasecmp(user, ROOTUSER)==0)
    per_user_statistics(outfile);

  fprintf(outfile, "\n</body>\n"
                   "</html>\n");

  fclose(outfile);

  // write summary

  sprintf(cachefile, "%s/summary.dat", dir);
  outfile=fopen(cachefile, "wb");
  if(outfile!=NULL)
   {
    fwrite(&s_total, 1, sizeof(s_total), outfile);
    //fwrite(&firstentry, 1, sizeof(time_t), outfile);
    //fwrite(&lastentry, 1, sizeof(time_t), outfile);
    fclose(outfile);
   }
 }

void makestat()
 {
  makestatt makestat;

  makestat.makestat();
 }

void doit()
 {
  if(user[0]==0)
    collect();
  else
    makestat();
 }

int main(int argc, char **argv)
 {
  while(1)
    switch(getopt(argc, argv, "p:f:u:l:d:o"))
     {
     case 'p':
      prefix=optarg;
      break;

     case 'd':
      userdir=optarg;
      break;

     case 'f':
      filter=optarg;
      break;

     case 'u':
      user=optarg;
      break;

     case 'l':
      logsubdir=optarg;
      break;

     case 'o':
      overwrite_cache=TRUE;
      break;

     case -1:
      doit();
      return 0;

     default:
      help();
     }
 }
